module forcing
  
  use solver
  use parallel
  
  implicit none

  real(WP) :: kf,ka,kb,c
  real(WP) :: A
  

end module forcing


subroutine forcing_init

  use forcing
  use parser

  implicit none

  integer :: i,j,k

  real(WP) :: kk
  real(WP) :: force_sum,force_spectrum
  real(WP) :: P
  real(WP) :: keta
  real(WP) :: eta
  real(WP) :: pi
  real(WP) :: Rlm,tke
  real(WP) :: Pf
  real(WP) :: B,nnodes
  real(WP), dimension(:), allocatable :: s,stotal
  integer :: nsize
  integer :: ierr,ik


  pi = acos(-1.0_WP)
  
  call parser_read('Kmax eta', keta)
  call parser_read('Number of points',nsize)
  call parser_read('R Lambda',Rlm)
  eta = (2.0_WP*keta)/real(nsize,WP)
  P = mu_solver**3.0_WP/eta**4.0_WP

  kf = (2.0_WP*pi)/((3.0_WP/20.0_WP*Rlm*Rlm)**0.75_WP*eta)
!  kf = (2.0_WP*pi)/((3.0_WP/20.0_WP*Rlm*Rlm)**0.75_WP*eta)-1.
  
  call parser_read('Power factor',Pf,1.0_WP)
  kf = kf/Pf

  if (rank.eq.0) print *,' Forcing wavenumber ', kf
  c = 0.5_WP
  ka = max(1e-10_WP,kf-2._WP) ! max(1e-10_WP,kf-0.5_WP)
  kb = kf+1._WP

  force_sum = 0.0_WP
  nnodes = 0.0_WP
  
  allocate(s(ns1+1),stotal(ns1+1))
  s = 0.0_WP
  stotal = 0.0_WP
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           kk = sqrt(kx(i)**2.0_WP + ky(j)**2.0_WP + kz(k)**2.0_WP)
           ik = 1+idint(kk + 0.5_WP)
           if (kk.ge.ka.and.kk.le.kb) then
              force_sum = force_sum + force_spectrum(kk,kf,c)/(2.0_WP*pi*kk**2.0_WP)
              s(ik) = s(ik) + force_spectrum(kk,kf,c)/(2.0_WP*pi*kk**2.0_WP)
              nnodes = nnodes + 1.0_WP
           endif
        enddo
     enddo
  enddo

  call MPI_ALLREDUCE(force_sum,A,1,MPI_REAL_WP,MPI_SUM,&
       &MPI_COMM_WORLD,ierr)
  call MPI_ALLREDUCE(nnodes,B,1,MPI_REAL_WP,MPI_SUM,&
       &MPI_COMM_WORLD,ierr)
  call MPI_ALLREDUCE(s,stotal,ns1+1,MPI_REAL_WP,MPI_SUM,&
       &MPI_COMM_WORLD,ierr)
  

  A = P/A

  if (rank.eq.0) print *,' Pre-multiplier for forcing term + nnodes', A,B

  
end subroutine forcing_init
  

subroutine force_compute

  use forcing

  implicit none
  
  integer :: i,j,k

  real(WP) :: rand,pi,kk,force_spectrum
  real(WP) :: phi,psi,theta1,theta2
  
  real(WP) :: e1x,e1y,e1z
  real(WP) :: e2x,e2y,e2z
  real(WP) :: kk2,ga,gb
  complex(WP) :: xi1,xi2,af,bf
  real(WP) :: rxi1,ixi1,rxi2,ixi2
  real(WP) :: num,den,f



  pi = acos(-1.0_WP)


  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           
           kk = sqrt(kx(i)**2.0_WP + ky(j)**2.0_WP + kz(k)**2.0_WP)
           kk2 = sqrt(kx(i)**2.0_WP + ky(j)**2.0_WP)
           if (kk2.gt.1e-10_WP.and.kk.gt.1e-10_WP) then
              if (kk.ge.ka.and.kk.le.kb) then
                 call random_number(rand)
                 phi = pi*rand
                 call random_number(rand)
                 psi = 2.0_WP*pi*rand
                 
                 e1x = ky(j)/kk2
                 e1y = -kx(i)/kk2
                 e1z = 0.0_WP
                 e2x = kx(i)*kz(k)/(kk*kk2)
                 e2y = ky(j)*kz(k)/(kk*kk2)
                 e2z = -(kk2)/kk
                 xi1 = Uk(i,j,k)*e1x + Vk(i,j,k)*e1y + Wk(i,j,k)*e1z
                 xi2 = Uk(i,j,k)*e2x + Vk(i,j,k)*e2y + Wk(i,j,k)*e2z
                 rxi1 = real(0.5_WP*(xi1 + conjg(xi1)),WP)
                 rxi2 = real(0.5_WP*(xi2 + conjg(xi2)),WP)
                 ixi1 = real(0.5_WP*(-ii)*(xi1 - conjg(xi1)),WP)
                 ixi2 = real(0.5_WP*(-ii)*(xi2 - conjg(xi2)),WP)
                 ga = sin(2.0_WP*phi)
                 gb = cos(2.0_WP*phi)
                 num =  ga*rxi1 + gb*(sin(psi)*ixi2 + cos(psi)*rxi2)
                 den = -ga*ixi1 + gb*(sin(psi)*rxi2 - cos(psi)*ixi2)
                 theta1 = atan(num/den)
                 theta2 = psi + theta1
                 f = A/dt*force_spectrum(kk,kf,c)/(2.0_WP*pi*kk**2.0_WP)
                 af = f**0.5_WP*exp(ii*theta1)*ga
                 bf = f**0.5_WP*exp(ii*theta2)*gb
                 nlx(i,j,k) = nlx(i,j,k) + (af*e1x+bf*e2x)
                 nly(i,j,k) = nly(i,j,k) + (af*e1y+bf*e2y)
                 nlz(i,j,k) = nlz(i,j,k) + (bf*e2z)
              endif
           endif
        enddo
     enddo
  enddo

  
end subroutine force_compute

function force_spectrum(kk,kf,c)

  use precision
  implicit none
  
  real(WP) :: force_spectrum
  real(WP) :: kk,kf,c

  
  force_spectrum = exp(-(kk-kf)**2.0_WP/c)

end function force_spectrum
