C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
	SUBROUTINE  interho(npart,g1,g2,g3,xp,yp,zp)

C
C   Interpolation routine with M'4 
C
! geometry=unit box, periodic in x and y
! last ponits in z direction assume extension by continuity
! that is gg(i,j,0)=gg(i,j,1) gg(i,j,m+1)=gg(i,j,m)


c----------------------------------------------------------------

	include 'param.i'
	include 'param.h'

	dimension g1(*),g2(*),g3(*),xp(*),yp(*),zp(*)

        COMMON/GRID/     omg1(npgx,npgy,npgz),
     &  omg2(npgx,npgy,npgz),omg3(npgx,npgy,npgz),
     &  vxg(npgx,npgy,npgz),vyg(npgx,npgy,npgz),vzg(npgx,npgy,npgz),
     &  gg1(npgx,npgy,npgz),gg2(npgx,npgy,npgz),gg3(npgx,npgy,npgz),
     &  strg1(npgx,npgy,npgz),strg2(npgx,npgy,npgz),
     &  strg3(npgx,npgy,npgz),rhog(npg,npg,npg)


	do 10 i=1,npart
	g1(i)=0.
	g2(i)=0.
	g3(i)=0.
10	continue

	ny3=nx3
	nz3=nx3
	dx3=xmax/float(nx3)
        dy3=dx3
        dz3=dx3

        dxinv=1./dx3
        dyinv=1./dy3
        dzinv=1./dz3
        dh3=dx3*dy3*dz3
        dhinv3=1./dh3  


c-------------------------------------------------------------------- 
c- PART II : Determination of  the  circulation  of each particle
c-------------------------------------------------------------------- 

	x0=xmin
	y0=ymin
	z0=zmin


	 DO 20 i = 1,npart

            x = XP(i)
            y = YP(i)
            z = ZP(i)

            ip1 = int((x-x0)*dxinv) 
            jp1 = int((y-y0)*dyinv) 
            kp1 = int((z-z0)*dzinv) 

            ip2 = ip1 + 1
	    jp2 = jp1 + 1
	    kp2 = kp1 + 1

            ip3 = ip1 + 2
            jp3 = jp1 + 2
            kp3 = kp1 + 2

C get the circulations from the nine neighboring cells

	   xx1 = (x - float(ip1)*dx3-x0)*dxinv
	   yy1 = (y - float(jp1)*dy3-y0)*dyinv
	   zz1 = (z - float(kp1)*dz3-z0)*dzinv

	   xx2=1-xx1
	   yy2=1-yy1
	   zz2=1-zz1

           xx3=2-xx1
           yy3=2-yy1
           zz3=2-zz1
            
C
C 	on repositionne les points de grille par periodicite
! entre 0 et m-1, puis on numerote de 1 a m
C
	ip1=mod(ip1+nx3,nx3) +1
	ip2=mod(ip2+nx3,nx3) +1
	ip3=mod(ip3+nx3,nx3) +1


	jp1=mod(jp1+ny3,ny3) +1
	jp2=mod(jp2+ny3,ny3) +1
	jp3=mod(jp3+ny3,ny3) +1

	kp1=mod(kp1+nz3,nz3) +1
	kp2=mod(kp2+nz3,nz3) +1
	kp3=mod(kp3+nz3,nz3) +1

C
C The M'4 scheme
C

           a0 = .5*((2.-xx0)**2)*(1.-xx0)
           b0 = .5*((2.-yy0)**2)*(1.-yy0)
           c0 = .5*((2.-zz0)**2)*(1.-zz0)

           a1 = 1.-2.5*xx1*xx1 +  1.5*xx1*xx1*xx1
           b1 = 1.-2.5*yy1*yy1 +  1.5*yy1*yy1*yy1
           c1 = 1.-2.5*zz1*zz1 +  1.5*zz1*zz1*zz1

           a2 = 1.-2.5*xx2*xx2 +  1.5*xx2*xx2*xx2
           b2 = 1.-2.5*yy2*yy2 +  1.5*yy2*yy2*yy2
           c2 = 1.-2.5*zz2*zz2 +  1.5*zz2*zz2*zz2

           a3 = .5*((2.-xx3)**2)*(1.-xx3)
           b3 = .5*((2.-yy3)**2)*(1.-yy3)
           c3 = .5*((2.-zz3)**2)*(1.-zz3)

            g1(i)= g1(i) +  GG1(ip0,jp0,kp0)*a0*b0*c0
            g1(i)= g1(i) +  GG1(ip0,jp1,kp0)*a0*b1*c0
            g1(i)= g1(i) +  GG1(ip0,jp2,kp0)*a0*b2*c0
            g1(i)= g1(i) +  GG1(ip0,jp3,kp0)*a0*b3*c0

            g2(i)= g2(i) + GG2(ip0,jp0,kp0)*a0*b0*c0
            g2(i)= g2(i) + GG2(ip0,jp1,kp0)*a0*b1*c0
            g2(i)= g2(i) + GG2(ip0,jp2,kp0)*a0*b2*c0
            g2(i)= g2(i) + GG2(ip0,jp3,kp0)*a0*b3*c0

            g3(i)= g3(i) + GG3(ip0,jp0,kp0)*a0*b0*c0
            g3(i)= g3(i) + GG3(ip0,jp1,kp0)*a0*b1*c0
            g3(i)= g3(i) + GG3(ip0,jp2,kp0)*a0*b2*c0
            g3(i)= g3(i) + GG3(ip0,jp3,kp0)*a0*b3*c0
c
           g1(i)= g1(i) +  GG1(ip1,jp0,kp0)*a1*b0*c0
           g1(i)= g1(i) +  GG1(ip1,jp1,kp0)*a1*b1*c0
           g1(i)= g1(i) +  GG1(ip1,jp2,kp0)*a1*b2*c0
           g1(i)= g1(i) +  GG1(ip1,jp3,kp0)*a1*b3*c0

           g2(i)= g2(i) + GG2(ip1,jp0,kp0)*a1*b0*c0
           g2(i)= g2(i) + GG2(ip1,jp1,kp0)*a1*b1*c0
           g2(i)= g2(i) + GG2(ip1,jp2,kp0)*a1*b2*c0
           g2(i)= g2(i) + GG2(ip1,jp3,kp0)*a1*b3*c0

           g3(i)= g3(i) + GG3(ip1,jp0,kp0)*a1*b0*c0
           g3(i)= g3(i) + GG3(ip1,jp1,kp0)*a1*b1*c0
           g3(i)= g3(i) + GG3(ip1,jp2,kp0)*a1*b2*c0
           g3(i)= g3(i) + GG3(ip1,jp3,kp0)*a1*b3*c0
c
           g1(i)= g1(i) +  GG1(ip2,jp0,kp0)*a2*b0*c0
           g1(i)= g1(i) +  GG1(ip2,jp1,kp0)*a2*b1*c0
           g1(i)= g1(i) +  GG1(ip2,jp2,kp0)*a2*b2*c0
           g1(i)= g1(i) +  GG1(ip2,jp3,kp0)*a2*b3*c0
:wq

20      CONTINUE



	RETURN
	END
