subroutine fftw3d(r,c,nx,ny,nz,nxs2,nys2,nzs2,wk,irc)

  ! Ce programme calcul la transformee de fourier rapide
  !---------------------------------------------------------------------- 
  ! TRANSFORMATION DE FOURIER REELLE<--->COMPLEXE TRI-DIMENSIONNELLE       
  !    X  : VECTEUR REEL DE DIMENSION (nx,ny,nz)             
  !    C  : VECTEUR COMPLEXE DE DIMENSION (0:nx/2,-ny/2+1:ny/2,-nz/2+1:nz/2)   
  !    WK   : Vecteur de travail pour les fftw de dimension (0:nx/2,0:ny-1,0:nz-1)
  !    IRC  :  =0: REELLE   ---> COMPLEXE  (X-->C)  Directe               
  !           <>0: COMPLEXE ---> REELLE    (C-->X)  inverse               
  !                                                                       
  ! REMARQUE:                                                             
  !    CALCULE LA TRANSFORMATION SUIVANTE:                                
  !                                                                       
  !  X(J,K,L)  J(1..nx),K(1..ny),L(1..Nz) <--->  C(J,K,L)  J=0..nx/2 K=-ny/2+1,...,ny/2, 
  !                                           L=-nz/2+1,...,nz/2
  !                                                                       
  !---------------------------------------------------------------------- 
  ! 
  !    Declaration pour utiliser les routines fftw 
  !
  !     This file contains PARAMETER statements for various constants
  !     that can be passed to FFTW routines.  You should include
  !     this file in any FORTRAN program that calls the fftw_f77
  !     routines (either directly or with an #include statement
  !     if you use the C preprocessor).
  !

  include '/opt/Softs/fftw/fftw-3.2.2-intel/include/fftw3.f'

  !      INTEGER FFTW_FORWARD,FFTW_BACKWARD
  !      PARAMETER (FFTW_FORWARD=-1,FFTW_BACKWARD=1)
  !
  !      INTEGER FFTW_REAL_TO_COMPLEX,FFTW_COMPLEX_TO_REAL
  !      PARAMETER (FFTW_REAL_TO_COMPLEX=-1,FFTW_COMPLEX_TO_REAL=1)
  !
  !      INTEGER FFTW_ESTIMATE,FFTW_MEASURE
  !      PARAMETER (FFTW_ESTIMATE=0,FFTW_MEASURE=1)
  !      INTEGER FFTW_IN_PLACE,FFTW_USE_WISDOM
  !      PARAMETER (FFTW_IN_PLACE=8,FFTW_USE_WISDOM=16)   
  
  integer plan,nx,ny,nz,nxs2,nys2,nzs2,irc,i,j,k
  
  complex c,wk
  dimension c(0:nxs2,1-nys2:nys2,1-nzs2:nzs2),wk(nxs2+1,ny,nz)
  
  real r
  dimension r(nx,ny,nz)
  
  do k=1,nz
     do j=1,ny
        do i=1,nxs2+1
           wk(i,j,k)=cmplx(0.0,0.0)
        enddo
     enddo
  enddo
  
  plan=0
  if (irc.eq.0) then

     call sfftw_plan_dft_r2c_3d(plan,nx,ny,nz,r,wk,FFTW_ESTIMATE)
     call sfftw_execute(plan)
     call sfftw_destroy_plan(plan)

     do k=1,nz
        do j=1,ny
           do i=1,nxs2+1
              wk(i,j,k)=wk(i,j,k)/nx/ny/nz
           enddo
        enddo
     enddo

     do k=1,nzs2+1
        do j=1,nys2+1
           do i=1,nxs2+1
              c(i-1,j-1,k-1)=wk(i,j,k)
           enddo
        enddo
     enddo

     do k=1,nzs2+1
        do j=1,nys2-1
           do i=1,nxs2+1
              c(i-1,j-nys2,k-1)=wk(i,j+nys2+1,k)
           enddo
        enddo
     enddo

     do k=1,nzs2-1
        do j=1,nys2+1
           do i=1,nxs2+1
              c(i-1,j-1,k-nzs2)=wk(i,j,k+nzs2+1)
           enddo
        enddo
     enddo

     do k=1,nzs2-1
        do j=1,nys2-1
           do i=1,nxs2+1
              c(i-1,j-nys2,k-nzs2)=wk(i,j+nys2+1,k+nzs2+1)
           enddo
        enddo
     enddo
  else
     do k=1,nzs2+1
        do j=1,nys2+1
           do i=1,nxs2+1
              wk(i,j,k)=c(i-1,j-1,k-1)
           enddo
        enddo
     enddo
     do k=1,nzs2+1
        do j=1,nys2-1
           do i=1,nxs2+1
              wk(i,j+nys2+1,k)=c(i-1,j-nys2,k-1)
           enddo
        enddo
     enddo

     do k=1,nzs2-1
        do j=1,nys2+1
           do i=1,nxs2+1
              wk(i,j,k+nzs2+1)=c(i-1,j-1,k-nzs2)
           enddo
        enddo
     enddo

     do k=1,nzs2-1
        do j=1,nys2-1
           do i=1,nxs2+1
              wk(i,j+nys2+1,k+nzs2+1)=c(i-1,j-nys2,k-nzs2)
           enddo
        enddo
     enddo

     call sfftw_plan_dft_c2r_3d(plan,nx,ny,nz,wk,r,FFTW_ESTIMATE)
     call sfftw_execute(plan)
     call sfftw_destroy_plan(plan)

  endif
  return
end subroutine fftw3d
