!================================================================
SUBROUTINE  remesh_om(npart,om1,om2,om3,xp1,yp1,zp1,dv1)
  !   This subroutine asssigns vorticity on a grid
  !----------------------------------------------------------------

  include 'param.i'
  include 'param.h'
  include 'arrays.h'



  dimension xp1(*),yp1(*),zp1(*),dv1(*)
  dimension om1(*),om2(*),om3(*)

  nx1=nx
  ny1=nx
  nz1=nx
  dx1=dx


  do i=1,nx1
     do j=1,ny1
        do k=1,nz1
           omg1(i,j,k)=0.
           omg2(i,j,k)=0.
           omg3(i,j,k)=0.
        end do
     end do
  end do

  dy1=dx1
  dz1=dx1

  dxinv=1./(dx1)
  dyinv=dxinv
  dzinv=dxinv



  !-------------------------------------------------------------------- 
  !- PART II : Determination of  the  circulation  of each particle
  !-------------------------------------------------------------------- 

  x0=0.
  y0=0.
  z0=0.

  vol=dx1*dx1*dx1

  DO n = 1,npart

     g2 = om1(n)/vol
     g3 = om2(n)/vol
     g4 = om3(n)/vol

     x = xp1(n)
     y = yp1(n)
     z = zp1(n)

     ip1 = int((x-x0)*dxinv) 
     jp1 = int((y-y0)*dyinv) 
     kp1 = int((z-z0)*dzinv)

     ip0 = ip1 - 1
     jp0 = jp1 - 1
     kp0 = kp1 - 1

     ip2 = ip1 + 1
     jp2 = jp1 + 1
     kp2 = kp1 + 1

     ip3 = ip1 + 2
     jp3 = jp1 + 2
     kp3 = kp1 + 2

     ! Assign the circulations to the nine neighboring cells

     xx1 = (x - float(ip1)*dx1-x0)*dxinv
     yy1 = (y - float(jp1)*dy1-y0)*dyinv
     zz1 = (z - float(kp1)*dz1-z0)*dzinv

     xx0=xx1+1.
     yy0=yy1+1.
     zz0=zz1+1.

     xx2=1.-xx1
     yy2=1.-yy1
     zz2=1.-zz1

     xx3=2.-xx1
     yy3=2.-yy1
     zz3=2.-zz1


     !
     ! 	on repositionne les points de grille par periodicite
     !	entre 0 et npx-1, puis on numerote de 1 a npx
     !

     ip1=mod(ip1+nx1,nx1) +1 
     ip0=mod(ip0+nx1,nx1) +1
     ip2=mod(ip2+nx1,nx1) +1
     ip3=mod(ip3+nx1,nx1) +1

     jp1=mod(jp1+ny1,ny1) +1 
     jp0=mod(jp0+ny1,ny1) +1
     jp2=mod(jp2+ny1,ny1) +1
     jp3=mod(jp3+ny1,ny1) +1

     kp1=mod(kp1+nz1,nz1) +1 
     kp0=mod(kp0+nz1,nz1) +1
     kp2=mod(kp2+nz1,nz1) +1
     kp3=mod(kp3+nz1,nz1) +1

     ! The M'4 scheme
     !
     a0 = .5*((2.-xx0)**2)*(1.-xx0)
     b0 = .5*((2.-yy0)**2)*(1.-yy0)
     c0 = .5*((2.-zz0)**2)*(1.-zz0)

     a1 = 1.-2.5*xx1*xx1 +  1.5*xx1*xx1*xx1
     b1 = 1.-2.5*yy1*yy1 +  1.5*yy1*yy1*yy1
     c1 = 1.-2.5*zz1*zz1 +  1.5*zz1*zz1*zz1

     a2 = 1.-2.5*xx2*xx2 +  1.5*xx2*xx2*xx2
     b2 = 1.-2.5*yy2*yy2 +  1.5*yy2*yy2*yy2
     c2 = 1.-2.5*zz2*zz2 +  1.5*zz2*zz2*zz2

     a3 = .5*((2.-xx3)**2)*(1.-xx3)
     b3 = .5*((2.-yy3)**2)*(1.-yy3)
     c3 = .5*((2.-zz3)**2)*(1.-zz3)

     !       b1=1.
     !       b0=0.
     !       b2=0.
     !       b3=0.
     !       a1=1.
     !       a0=0.
     !       a2=0.
     !       a3=0.
     !       c1=1.
     !       c0=0.
     !       c2=0.
     !       c3=0.


     coef=a0*b0*c0
     omg1(ip0,jp0,kp0) = omg1(ip0,jp0,kp0) + g2*coef
     omg2(ip0,jp0,kp0) = omg2(ip0,jp0,kp0) + g3*coef
     omg3(ip0,jp0,kp0) = omg3(ip0,jp0,kp0) + g4*coef

     coef=a0*b0*c1
     omg1(ip0,jp0,kp1) = omg1(ip0,jp0,kp1) + g2*coef
     omg2(ip0,jp0,kp1) = omg2(ip0,jp0,kp1) + g3*coef
     omg3(ip0,jp0,kp1) = omg3(ip0,jp0,kp1) + g4*coef

     coef=a0*b0*c2
     omg1(ip0,jp0,kp2) = omg1(ip0,jp0,kp2) + g2*coef
     omg2(ip0,jp0,kp2) = omg2(ip0,jp0,kp2) + g3*coef
     omg3(ip0,jp0,kp2) = omg3(ip0,jp0,kp2) + g4*coef

     coef=a0*b0*c3
     omg1(ip0,jp0,kp3) = omg1(ip0,jp0,kp3) + g2*coef
     omg2(ip0,jp0,kp3) = omg2(ip0,jp0,kp3) + g3*coef
     omg3(ip0,jp0,kp3) = omg3(ip0,jp0,kp3) + g4*coef

     coef=a0*b1*c0
     omg1(ip0,jp1,kp0) = omg1(ip0,jp1,kp0) + g2*coef
     omg2(ip0,jp1,kp0) = omg2(ip0,jp1,kp0) + g3*coef
     omg3(ip0,jp1,kp0) = omg3(ip0,jp1,kp0) + g4*coef

     coef=a0*b1*c1
     omg1(ip0,jp1,kp1) = omg1(ip0,jp1,kp1) + g2*coef
     omg2(ip0,jp1,kp1) = omg2(ip0,jp1,kp1) + g3*coef
     omg3(ip0,jp1,kp1) = omg3(ip0,jp1,kp1) + g4*coef

     coef=a0*b1*c2
     omg1(ip0,jp1,kp2) = omg1(ip0,jp1,kp2) + g2*coef
     omg2(ip0,jp1,kp2) = omg2(ip0,jp1,kp2) + g3*coef
     omg3(ip0,jp1,kp2) = omg3(ip0,jp1,kp2) + g4*coef

     coef=a0*b1*c3
     omg1(ip0,jp1,kp3) = omg1(ip0,jp1,kp3) + g2*coef
     omg2(ip0,jp1,kp3) = omg2(ip0,jp1,kp3) + g3*coef
     omg3(ip0,jp1,kp3) = omg3(ip0,jp1,kp3) + g4*coef

     coef=a0*b2*c0
     omg1(ip0,jp2,kp0) = omg1(ip0,jp2,kp0) + g2*coef
     omg2(ip0,jp2,kp0) = omg2(ip0,jp2,kp0) + g3*coef
     omg3(ip0,jp2,kp0) = omg3(ip0,jp2,kp0) + g4*coef

     coef=a0*b2*c1
     omg1(ip0,jp2,kp1) = omg1(ip0,jp2,kp1) + g2*coef
     omg2(ip0,jp2,kp1) = omg2(ip0,jp2,kp1) + g3*coef
     omg3(ip0,jp2,kp1) = omg3(ip0,jp2,kp1) + g4*coef

     coef=a0*b2*c2
     omg1(ip0,jp2,kp2) = omg1(ip0,jp2,kp2) + g2*coef
     omg2(ip0,jp2,kp2) = omg2(ip0,jp2,kp2) + g3*coef
     omg3(ip0,jp2,kp2) = omg3(ip0,jp2,kp2) + g4*coef


     coef=a0*b2*c3
     omg1(ip0,jp2,kp3) = omg1(ip0,jp2,kp3) + g2*coef
     omg2(ip0,jp2,kp3) = omg2(ip0,jp2,kp3) + g3*coef
     omg3(ip0,jp2,kp3) = omg3(ip0,jp2,kp3) + g4*coef

     coef=a0*b3*c0
     omg1(ip0,jp3,kp0) = omg1(ip0,jp3,kp0) + g2*coef
     omg2(ip0,jp3,kp0) = omg2(ip0,jp3,kp0) + g3*coef
     omg3(ip0,jp3,kp0) = omg3(ip0,jp3,kp0) + g4*coef

     coef=a0*b3*c1
     omg1(ip0,jp3,kp1) = omg1(ip0,jp3,kp1) + g2*coef
     omg2(ip0,jp3,kp1) = omg2(ip0,jp3,kp1) + g3*coef
     omg3(ip0,jp3,kp1) = omg3(ip0,jp3,kp1) + g4*coef

     coef=a0*b3*c2
     omg1(ip0,jp3,kp2) = omg1(ip0,jp3,kp2) + g2*coef
     omg2(ip0,jp3,kp2) = omg2(ip0,jp3,kp2) + g3*coef
     omg3(ip0,jp3,kp2) = omg3(ip0,jp3,kp2) + g4*coef

     coef=a0*b3*c3
     omg1(ip0,jp3,kp3) = omg1(ip0,jp3,kp3) + g2*coef
     omg2(ip0,jp3,kp3) = omg2(ip0,jp3,kp3) + g3*coef
     omg3(ip0,jp3,kp3) = omg3(ip0,jp3,kp3) + g4*coef

     coef=a1*b0*c0
     omg1(ip1,jp0,kp0) = omg1(ip1,jp0,kp0) + g2*coef
     omg2(ip1,jp0,kp0) = omg2(ip1,jp0,kp0) + g3*coef
     omg3(ip1,jp0,kp0) = omg3(ip1,jp0,kp0) + g4*coef

     coef=a1*b0*c1
     omg1(ip1,jp0,kp1) = omg1(ip1,jp0,kp1) + g2*coef
     omg2(ip1,jp0,kp1) = omg2(ip1,jp0,kp1) + g3*coef
     omg3(ip1,jp0,kp1) = omg3(ip1,jp0,kp1) + g4*coef

     coef=a1*b0*c2
     omg1(ip1,jp0,kp2) = omg1(ip1,jp0,kp2) + g2*coef
     omg2(ip1,jp0,kp2) = omg2(ip1,jp0,kp2) + g3*coef
     omg3(ip1,jp0,kp2) = omg3(ip1,jp0,kp2) + g4*coef

     coef=a1*b0*c3
     omg1(ip1,jp0,kp3) = omg1(ip1,jp0,kp3) + g2*coef
     omg2(ip1,jp0,kp3) = omg2(ip1,jp0,kp3) + g3*coef
     omg3(ip1,jp0,kp3) = omg3(ip1,jp0,kp3) + g4*coef

     coef=a1*b1*c0
     omg1(ip1,jp1,kp0) = omg1(ip1,jp1,kp0) + g2*coef
     omg2(ip1,jp1,kp0) = omg2(ip1,jp1,kp0) + g3*coef
     omg3(ip1,jp1,kp0) = omg3(ip1,jp1,kp0) + g4*coef

     coef=a1*b1*c1
     omg1(ip1,jp1,kp1) = omg1(ip1,jp1,kp1) + g2*coef
     omg2(ip1,jp1,kp1) = omg2(ip1,jp1,kp1) + g3*coef
     omg3(ip1,jp1,kp1) = omg3(ip1,jp1,kp1) + g4*coef

     coef=a1*b1*c2
     omg1(ip1,jp1,kp2) = omg1(ip1,jp1,kp2) + g2*coef
     omg2(ip1,jp1,kp2) = omg2(ip1,jp1,kp2) + g3*coef
     omg3(ip1,jp1,kp2) = omg3(ip1,jp1,kp2) + g4*coef

     coef=a1*b1*c3
     omg1(ip1,jp1,kp3) = omg1(ip1,jp1,kp3) + g2*coef
     omg2(ip1,jp1,kp3) = omg2(ip1,jp1,kp3) + g3*coef
     omg3(ip1,jp1,kp3) = omg3(ip1,jp1,kp3) + g4*coef

     coef=a1*b2*c0
     omg1(ip1,jp2,kp0) = omg1(ip1,jp2,kp0) + g2*coef
     omg2(ip1,jp2,kp0) = omg2(ip1,jp2,kp0) + g3*coef
     omg3(ip1,jp2,kp0) = omg3(ip1,jp2,kp0) + g4*coef

     coef=a1*b2*c1
     omg1(ip1,jp2,kp1) = omg1(ip1,jp2,kp1) + g2*coef
     omg2(ip1,jp2,kp1) = omg2(ip1,jp2,kp1) + g3*coef
     omg3(ip1,jp2,kp1) = omg3(ip1,jp2,kp1) + g4*coef

     coef=a1*b2*c2
     omg1(ip1,jp2,kp2) = omg1(ip1,jp2,kp2) + g2*coef
     omg2(ip1,jp2,kp2) = omg2(ip1,jp2,kp2) + g3*coef
     omg3(ip1,jp2,kp2) = omg3(ip1,jp2,kp2) + g4*coef

     coef=a1*b2*c3
     omg1(ip1,jp2,kp3) = omg1(ip1,jp2,kp3) + g2*coef
     omg2(ip1,jp2,kp3) = omg2(ip1,jp2,kp3) + g3*coef
     omg3(ip1,jp2,kp3) = omg3(ip1,jp2,kp3) + g4*coef

     coef=a1*b3*c0
     omg1(ip1,jp3,kp0) = omg1(ip1,jp3,kp0) + g2*coef
     omg2(ip1,jp3,kp0) = omg2(ip1,jp3,kp0) + g3*coef
     omg3(ip1,jp3,kp0) = omg3(ip1,jp3,kp0) + g4*coef

     coef=a1*b3*c1
     omg1(ip1,jp3,kp1) = omg1(ip1,jp3,kp1) + g2*coef
     omg2(ip1,jp3,kp1) = omg2(ip1,jp3,kp1) + g3*coef
     omg3(ip1,jp3,kp1) = omg3(ip1,jp3,kp1) + g4*coef

     coef=a1*b3*c2
     omg1(ip1,jp3,kp2) = omg1(ip1,jp3,kp2) + g2*coef
     omg2(ip1,jp3,kp2) = omg2(ip1,jp3,kp2) + g3*coef
     omg3(ip1,jp3,kp2) = omg3(ip1,jp3,kp2) + g4*coef

     coef=a1*b3*c3
     omg1(ip1,jp3,kp3) = omg1(ip1,jp3,kp3) + g2*coef
     omg2(ip1,jp3,kp3) = omg2(ip1,jp3,kp3) + g3*coef
     omg3(ip1,jp3,kp3) = omg3(ip1,jp3,kp3) + g4*coef

     coef=a2*b0*c0
     omg1(ip2,jp0,kp0) = omg1(ip2,jp0,kp0) + g2*coef
     omg2(ip2,jp0,kp0) = omg2(ip2,jp0,kp0) + g3*coef
     omg3(ip2,jp0,kp0) = omg3(ip2,jp0,kp0) + g4*coef

     coef=a2*b0*c1
     omg1(ip2,jp0,kp1) = omg1(ip2,jp0,kp1) + g2*coef
     omg2(ip2,jp0,kp1) = omg2(ip2,jp0,kp1) + g3*coef
     omg3(ip2,jp0,kp1) = omg3(ip2,jp0,kp1) + g4*coef

     coef=a2*b0*c2
     omg1(ip2,jp0,kp2) = omg1(ip2,jp0,kp2) + g2*coef
     omg2(ip2,jp0,kp2) = omg2(ip2,jp0,kp2) + g3*coef
     omg3(ip2,jp0,kp2) = omg3(ip2,jp0,kp2) + g4*coef

     coef=a2*b0*c3
     omg1(ip2,jp0,kp3) = omg1(ip2,jp0,kp3) + g2*coef
     omg2(ip2,jp0,kp3) = omg2(ip2,jp0,kp3) + g3*coef
     omg3(ip2,jp0,kp3) = omg3(ip2,jp0,kp3) + g4*coef

     coef=a2*b1*c0
     omg1(ip2,jp1,kp0) = omg1(ip2,jp1,kp0) + g2*coef
     omg2(ip2,jp1,kp0) = omg2(ip2,jp1,kp0) + g3*coef
     omg3(ip2,jp1,kp0) = omg3(ip2,jp1,kp0) + g4*coef

     coef=a2*b1*c1
     omg1(ip2,jp1,kp1) = omg1(ip2,jp1,kp1) + g2*coef
     omg2(ip2,jp1,kp1) = omg2(ip2,jp1,kp1) + g3*coef
     omg3(ip2,jp1,kp1) = omg3(ip2,jp1,kp1) + g4*coef

     coef=a2*b1*c2
     omg1(ip2,jp1,kp2) = omg1(ip2,jp1,kp2) + g2*coef
     omg2(ip2,jp1,kp2) = omg2(ip2,jp1,kp2) + g3*coef
     omg3(ip2,jp1,kp2) = omg3(ip2,jp1,kp2) + g4*coef

     coef=a2*b1*c3
     omg1(ip2,jp1,kp3) = omg1(ip2,jp1,kp3) + g2*coef
     omg2(ip2,jp1,kp3) = omg2(ip2,jp1,kp3) + g3*coef
     omg3(ip2,jp1,kp3) = omg3(ip2,jp1,kp3) + g4*coef

     coef=a2*b2*c0
     omg1(ip2,jp2,kp0) = omg1(ip2,jp2,kp0) + g2*coef
     omg2(ip2,jp2,kp0) = omg2(ip2,jp2,kp0) + g3*coef
     omg3(ip2,jp2,kp0) = omg3(ip2,jp2,kp0) + g4*coef

     coef=a2*b2*c1
     omg1(ip2,jp2,kp1) = omg1(ip2,jp2,kp1) + g2*coef
     omg2(ip2,jp2,kp1) = omg2(ip2,jp2,kp1) + g3*coef
     omg3(ip2,jp2,kp1) = omg3(ip2,jp2,kp1) + g4*coef

     coef=a2*b2*c2
     omg1(ip2,jp2,kp2) = omg1(ip2,jp2,kp2) + g2*coef
     omg2(ip2,jp2,kp2) = omg2(ip2,jp2,kp2) + g3*coef
     omg3(ip2,jp2,kp2) = omg3(ip2,jp2,kp2) + g4*coef

     coef=a2*b2*c3
     omg1(ip2,jp2,kp3) = omg1(ip2,jp2,kp3) + g2*coef
     omg2(ip2,jp2,kp3) = omg2(ip2,jp2,kp3) + g3*coef
     omg3(ip2,jp2,kp3) = omg3(ip2,jp2,kp3) + g4*coef

     coef=a2*b3*c0
     omg1(ip2,jp3,kp0) = omg1(ip2,jp3,kp0) + g2*coef
     omg2(ip2,jp3,kp0) = omg2(ip2,jp3,kp0) + g3*coef
     omg3(ip2,jp3,kp0) = omg3(ip2,jp3,kp0) + g4*coef

     coef=a2*b3*c1
     omg1(ip2,jp3,kp1) = omg1(ip2,jp3,kp1) + g2*coef
     omg2(ip2,jp3,kp1) = omg2(ip2,jp3,kp1) + g3*coef
     omg3(ip2,jp3,kp1) = omg3(ip2,jp3,kp1) + g4*coef

     coef=a2*b3*c2
     omg1(ip2,jp3,kp2) = omg1(ip2,jp3,kp2) + g2*coef
     omg2(ip2,jp3,kp2) = omg2(ip2,jp3,kp2) + g3*coef
     omg3(ip2,jp3,kp2) = omg3(ip2,jp3,kp2) + g4*coef

     coef=a2*b3*c3
     omg1(ip2,jp3,kp3) = omg1(ip2,jp3,kp3) + g2*coef
     omg2(ip2,jp3,kp3) = omg2(ip2,jp3,kp3) + g3*coef
     omg3(ip2,jp3,kp3) = omg3(ip2,jp3,kp3) + g4*coef

     coef=a3*b0*c0
     omg1(ip3,jp0,kp0) = omg1(ip3,jp0,kp0) + g2*coef
     omg2(ip3,jp0,kp0) = omg2(ip3,jp0,kp0) + g3*coef
     omg3(ip3,jp0,kp0) = omg3(ip3,jp0,kp0) + g4*coef

     coef=a3*b0*c1
     omg1(ip3,jp0,kp1) = omg1(ip3,jp0,kp1) + g2*coef
     omg2(ip3,jp0,kp1) = omg2(ip3,jp0,kp1) + g3*coef
     omg3(ip3,jp0,kp1) = omg3(ip3,jp0,kp1) + g4*coef

     coef=a3*b0*c2
     omg1(ip3,jp0,kp2) = omg1(ip3,jp0,kp2) + g2*coef
     omg2(ip3,jp0,kp2) = omg2(ip3,jp0,kp2) + g3*coef
     omg3(ip3,jp0,kp2) = omg3(ip3,jp0,kp2) + g4*coef

     coef=a3*b0*c3
     omg1(ip3,jp0,kp3) = omg1(ip3,jp0,kp3) + g2*coef
     omg2(ip3,jp0,kp3) = omg2(ip3,jp0,kp3) + g3*coef
     omg3(ip3,jp0,kp3) = omg3(ip3,jp0,kp3) + g4*coef

     coef=a3*b1*c0
     omg1(ip3,jp1,kp0) = omg1(ip3,jp1,kp0) + g2*coef
     omg2(ip3,jp1,kp0) = omg2(ip3,jp1,kp0) + g3*coef
     omg3(ip3,jp1,kp0) = omg3(ip3,jp1,kp0) + g4*coef

     coef=a3*b1*c1
     omg1(ip3,jp1,kp1) = omg1(ip3,jp1,kp1) + g2*coef
     omg2(ip3,jp1,kp1) = omg2(ip3,jp1,kp1) + g3*coef
     omg3(ip3,jp1,kp1) = omg3(ip3,jp1,kp1) + g4*coef

     coef=a3*b1*c2
     omg1(ip3,jp1,kp2) = omg1(ip3,jp1,kp2) + g2*coef
     omg2(ip3,jp1,kp2) = omg2(ip3,jp1,kp2) + g3*coef
     omg3(ip3,jp1,kp2) = omg3(ip3,jp1,kp2) + g4*coef

     coef=a3*b1*c3
     omg1(ip3,jp1,kp3) = omg1(ip3,jp1,kp3) + g2*coef
     omg2(ip3,jp1,kp3) = omg2(ip3,jp1,kp3) + g3*coef
     omg3(ip3,jp1,kp3) = omg3(ip3,jp1,kp3) + g4*coef

     coef=a3*b2*c0
     omg1(ip3,jp2,kp0) = omg1(ip3,jp2,kp0) + g2*coef
     omg2(ip3,jp2,kp0) = omg2(ip3,jp2,kp0) + g3*coef
     omg3(ip3,jp2,kp0) = omg3(ip3,jp2,kp0) + g4*coef

     coef=a3*b2*c1
     omg1(ip3,jp2,kp1) = omg1(ip3,jp2,kp1) + g2*coef
     omg2(ip3,jp2,kp1) = omg2(ip3,jp2,kp1) + g3*coef
     omg3(ip3,jp2,kp1) = omg3(ip3,jp2,kp1) + g4*coef

     coef=a3*b2*c2
     omg1(ip3,jp2,kp2) = omg1(ip3,jp2,kp2) + g2*coef
     omg2(ip3,jp2,kp2) = omg2(ip3,jp2,kp2) + g3*coef
     omg3(ip3,jp2,kp2) = omg3(ip3,jp2,kp2) + g4*coef

     coef=a3*b2*c3
     omg1(ip3,jp2,kp3) = omg1(ip3,jp2,kp3) + g2*coef
     omg2(ip3,jp2,kp3) = omg2(ip3,jp2,kp3) + g3*coef
     omg3(ip3,jp2,kp3) = omg3(ip3,jp2,kp3) + g4*coef

     coef=a3*b3*c0
     omg1(ip3,jp3,kp0) = omg1(ip3,jp3,kp0) + g2*coef
     omg2(ip3,jp3,kp0) = omg2(ip3,jp3,kp0) + g3*coef
     omg3(ip3,jp3,kp0) = omg3(ip3,jp3,kp0) + g4*coef

     coef=a3*b3*c1
     omg1(ip3,jp3,kp1) = omg1(ip3,jp3,kp1) + g2*coef
     omg2(ip3,jp3,kp1) = omg2(ip3,jp3,kp1) + g3*coef
     omg3(ip3,jp3,kp1) = omg3(ip3,jp3,kp1) + g4*coef

     coef=a3*b3*c2
     omg1(ip3,jp3,kp2) = omg1(ip3,jp3,kp2) + g2*coef
     omg2(ip3,jp3,kp2) = omg2(ip3,jp3,kp2) + g3*coef
     omg3(ip3,jp3,kp2) = omg3(ip3,jp3,kp2) + g4*coef

     coef=a3*b3*c3
     omg1(ip3,jp3,kp3) = omg1(ip3,jp3,kp3) + g2*coef
     omg2(ip3,jp3,kp3) = omg2(ip3,jp3,kp3) + g3*coef
     omg3(ip3,jp3,kp3) = omg3(ip3,jp3,kp3) + g4*coef

  end DO

end SUBROUTINE remesh_om
