module advection_mod
  use donnees_mod
  use tab_mod 
  use remaillage_mod
  use interpolation_mod
contains
  
  subroutine def_v_advec (vitx,vity,tps)    
    implicit none
    real(kind=8),dimension(1:nx_gro,1:ny_gro),intent(out) :: vitx,vity
    real(kind=8),intent(in)                               :: tps
    integer                                               :: i,j
    real(kind=8)                                          :: x,y,r2,tmp,cte,r,alpha,beta,rint,rext,w1,w2,w3,t1,t4,t5


    do j=1,ny_gro
       do i=1,nx_gro
          
          x=xg+(i-1)*dx_gro
          y=yb+(j-1)*dy_gro


!!$          !rotation avec chgmt signe
!!$          !-------------------------
!!$          r2=x**2+y**2         
!!$          !tmp=1      
!!$          tmp=cos(3.*0.5*pi*sqrt(r2)) 
!!$          !tmp=cos(10.*sqrt(r2))          
!!$          vitx(i,j)=-tmp*y
!!$          vity(i,j)=tmp*x

!!$          !test Staniforth sur [0,100]**2
!!$          !----
!!$          vitx(i,j)=8.*pi*sin(pi*x/25.)*sin(pi*y/25.)/25.
!!$          vity(i,j)=8.*pi*cos(pi*x/25.)*cos(pi*y/25.)/25.

          !Staniforth sur [0,1] avec retour pour t=0.5
          !--------------------------------------------
!!$          vitx(i,j)=0.08*pi*sin(4.*pi*x)*sin(4.*pi*y)/(cos(2.*pi*tps)/4.)
!!$          vity(i,j)=0.08*pi*cos(4.*pi*x)*cos(4.*pi*y)/(cos(2.*pi*tps)/4.)
!!$

          !filaments
          !----------
!!$          t1=10.
!!$
!!$          vitx(i,j)=-2.*sin(pi*x)**2*sin(pi*y)*cos(pi*y)  !*cos(pi*tps/t1)
!!$          vity(i,j)=2.*sin(pi*y)**2*sin(pi*x)*cos(pi*x) !*cos(pi*tps/t1)


       
          !test instationaire
          !--------------------

!!$          t1=1.
!!$          t4=0.3
!!$          t5=1.
!!$
!!$          vitx(i,j)=t5*sin(2.*pi*x/t1)*sin(2.*pi*y/t1)*cos(pi*tps/t4)
!!$          vity(i,j)=t5*cos(2.*pi*x/t1)*cos(2.*pi*y/t1)*cos(pi*tps/t4)


          !vitesse radiale
          !----------------
          r=sqrt(x**2+y**2)
          vitx(i,j)=x/r
          vity(i,j)=y/r

          if (r<0.4) then
             vitx(i,j)=x/0.4
             vity(i,j)=y/0.4
          end if



       end do
    end do

end subroutine def_v_advec


subroutine evalv (vitx,vity,tps,x,y)    
  implicit none
  real(kind=8),intent(in)     :: tps,x,y
  real(kind=8),intent(out)    :: vitx,vity
  integer                           :: i,j
  real(kind=8)                      :: r2,tmp,cte,r,alpha,beta,rint,rext,w1,w2,w3,t1,t4,t5


  t1=1.
  t4=0.3
  t5=1.

  vitx=t5*sin(2.*pi*x/t1)*sin(2.*pi*y/t1)*cos(pi*tps/t4)
  vity=t5*cos(2.*pi*x/t1)*cos(2.*pi*y/t1)*cos(pi*tps/t4)

end subroutine evalv








    subroutine crea_part_x
      implicit none
      integer                  :: i,j
      real(kind=8)             :: x,y,m     
     
      m=cutoff*maxval(abs(qg))
      npart=0  
      numpg=0
      do j=1,ny         
         do i=1,nx
            x=xtab(i,j)
            y=ytab(i,j)                      

            if (abs(qg(i,j))>=m) then
               npart=npart+1         
               numpg(i,j)=npart     
               xp(npart)=x
               yp(npart)=y
               vx(npart)=vxg(i,j)
               vy(npart)=vyg(i,j)
               !vx(npart)=interpol_v_l3(vxg,x,y)
               !vy(npart)=interpol_v_l3(vyg,x,y)           
               qp(npart)=qg(i,j)   
            end if
         end do         
      end do
    
    end subroutine crea_part_x

    subroutine crea_part_y
      implicit none
      integer                  :: i,j
      real(kind=8)             :: x,y,m     

      m=cutoff*maxval(abs(qg))
      npart=0       
      numpg=0
      do i=1,nx
         do j=1,ny   
            x=xtab(i,j)
            y=ytab(i,j)  

            if (abs(qg(i,j))>=m) then                    
               npart=npart+1  
               numpg(i,j)=npart
               xp(npart)=x
               yp(npart)=y
               vx(npart)=vxg(i,j)
               vy(npart)=vyg(i,j)
               !vx(npart)=interpol_v_l3(vxg,x,y)
               !vy(npart)=interpol_v_l3(vyg,x,y)   
               qp(npart)=qg(i,j)                           
            end if
         end do         
      end do
    
    end subroutine crea_part_y

  
    !==================
    !TENSORIEL
    !==================

    subroutine ad_tenso_euler     
      implicit none

      integer                  :: i,j
      real(kind=8)             :: x,y

      do i=1,npart         
         xp(i)=xp(i)+dt*vx(i)                              
         yp(i)=yp(i)+dt*vy(i)   
      end do

    end subroutine ad_tenso_euler

    subroutine ad_tenso_rk2
       implicit none
       integer                                :: ib      

       allocate (xp1(1:npart),yp1(1:npart)) 

       do ib=1,npart         
          xp1(ib)=xp(ib)+0.5*dt*vx(ib)                                                           
          yp1(ib)=yp(ib)+0.5*dt*vy(ib)                                                           
       end do
       
       call interpo_l2_2d(vxg,xp1,yp1,vx)
       call interpo_l2_2d(vyg,xp1,yp1,vy)       

       !call interpo_l3_2d(vxg1,xp1,yp1,vx)
       !call interpo_l3_2d(vyg1,xp1,yp1,vy)       
      
       do ib=1,npart
          xp(ib)=xp(ib)+dt*vx(ib)
          yp(ib)=yp(ib)+dt*vy(ib)
       end do

       deallocate(xp1,yp1)

     end subroutine ad_tenso_rk2

    




     subroutine ad_tenso_rk3
       implicit none
       integer              :: ib


       allocate (xp1(1:npart),xp2(1:npart),yp1(1:npart),yp2(1:npart)) 
       allocate (vx1(1:npart),vx2(1:npart),vy1(1:npart),vy2(1:npart))           

       do ib=1,npart
          xp1(ib)=xp(ib)+0.5*dt*vx(ib)
          yp1(ib)=yp(ib)+0.5*dt*vy(ib)
       end do

       call interpo_l3_2d(vxg1,xp1,yp1,vx1)
       call interpo_l3_2d(vyg1,xp1,yp1,vy1)      

       do ib=1,npart         
          xp2(ib)=xp(ib)-dt*vx(ib)+2.*dt*vx1(ib)                                          
          yp2(ib)=yp(ib)-dt*vy(ib)+2.*dt*vy1(ib)                                          
       end do
      
       call interpo_l3_2d(vxg2,xp2,yp2,vx2)
       call interpo_l3_2d(vyg2,xp2,yp2,vy2)

       do ib=1,npart                  
          xp(ib)=xp(ib)+dt*(vx(ib)/6.+2.*vx1(ib)/3.+vx2(ib)/6.)
          yp(ib)=yp(ib)+dt*(vy(ib)/6.+2.*vy1(ib)/3.+vy2(ib)/6.)
       end do

       deallocate (xp1,xp2,yp1,yp2)  
       deallocate (vx1,vx2,vy1,vy2)


     end subroutine ad_tenso_rk3


     subroutine ad_tenso_rk4
       implicit none
       integer              :: ib


       allocate (xp1(1:npart),xp2(1:npart),xp3(1:npart),yp1(1:npart),yp2(1:npart),yp3(1:npart)) 
       allocate (vx1(1:npart),vx2(1:npart),vx3(1:npart),vy1(1:npart),vy2(1:npart),vy3(1:npart))           

       do ib=1,npart
          xp1(ib)=xp(ib)+0.5*dt*vx(ib)
          yp1(ib)=yp(ib)+0.5*dt*vy(ib)
       end do

       call interpo_l4_2d(vxg1,xp1,yp1,vx1)
       call interpo_l4_2d(vyg1,xp1,yp1,vy1)      

       do ib=1,npart         
          xp2(ib)=xp(ib)+0.5*dt*vx1(ib)                                          
          yp2(ib)=yp(ib)+0.5*dt*vy1(ib)                                          
       end do
      
       call interpo_l4_2d(vxg1,xp2,yp2,vx2)
       call interpo_l4_2d(vyg1,xp2,yp2,vy2)

       do ib=1,npart         
          xp3(ib)=xp(ib)+dt*vx2(ib)                                          
          yp3(ib)=yp(ib)+dt*vy2(ib)                                          
       end do
      
       call interpo_l4_2d(vxg2,xp3,yp3,vx3)
       call interpo_l4_2d(vyg2,xp3,yp3,vy3)

       do ib=1,npart                  
          xp(ib)=xp(ib)+dt*(vx(ib)+2.*vx1(ib)+2.*vx2(ib)+vx3(ib))/6.
          yp(ib)=yp(ib)+dt*(vy(ib)+2.*vy1(ib)+2.*vy2(ib)+vy3(ib))/6.
       end do

       deallocate (xp1,xp2,xp3,yp1,yp2,yp3)  
       deallocate (vx1,vx2,vx3,vy1,vy2,vy3)


     end subroutine ad_tenso_rk4


     !===============================
     !ADVECTION non tensoriel
     !===============================

     subroutine ad_euler_x           
      implicit none

      integer                  :: i,j
      real(kind=8)             :: x,y

      do i=1,npart         
         xp(i)=xp(i)+dt*vx(i)                               
      end do

    end subroutine ad_euler_x

    subroutine ad_euler_y           
      implicit none
     
      integer                  :: i,j
      real(kind=8)             :: x,y

      do i=1,npart         
         yp(i)=yp(i)+dt*vy(i)   
      end do

    end subroutine ad_euler_y


     !===============================
     !SPLITTING
     !calcul de la vitesse pour une advection d'euler à l'ordre 2 ou 3
     !===============================   

     subroutine update_vx_2
       implicit none
       integer                  :: ib

       allocate (xp1(1:npart),yp1(1:npart))           
       do ib=1,npart         
          xp1(ib)=xp(ib)+0.5*dt*vx(ib)                                                           
          yp1(ib)=yp(ib)+0.5*dt*vy(ib)                                                           
       end do
       !call interpo_l2_2d(vxg1,xp1,yp1,vx)
       call interpo_l2_2d(vxg,xp1,yp1,vx)             
       deallocate(xp1,yp1)

     end subroutine update_vx_2

     subroutine update_vy_2
       implicit none
       integer                  :: ib

       allocate (xp1(1:npart),yp1(1:npart))           
       do ib=1,npart         
          xp1(ib)=xp(ib)-0.5*dt*vx(ib)                                                           
          yp1(ib)=yp(ib)+0.5*dt*vy(ib)                                                           
       end do
       !call interpo_l2_2d(vyg1,xp1,yp1,vy)   
       call interpo_l2_2d(vyg,xp1,yp1,vy)       
       deallocate(xp1,yp1)

     end subroutine update_vy_2

   

     subroutine update_vx_3
       implicit none    
       integer                  :: ib
      

       allocate (xp1(1:npart),xp2(1:npart),yp1(1:npart),yp2(1:npart)) 
       allocate (vx1(1:npart),vx2(1:npart),vy1(1:npart),vy2(1:npart))           

       do ib=1,npart
          xp1(ib)=xp(ib)+2.*dt*vx(ib)/3.
          yp1(ib)=yp(ib)+2.*dt*vy(ib)/3.
       end do

       !call interpo_l3_2d(vxg2,xp1,yp1,vx1)
       !call interpo_l3_2d(vyg2,xp1,yp1,vy1)  
       call interpo_l3_2d(vxg,xp1,yp1,vx1)
       call interpo_l3_2d(vyg,xp1,yp1,vy1)     

       do ib=1,npart         
          xp2(ib)=xp(ib)+dt*(-vx(ib)/4.+vx1(ib)/4.)                                          
          yp2(ib)=yp(ib)+dt*(-vy(ib)+vy1(ib))                                          
       end do   

       !call interpo_l3_2d(vxg,xp2,yp,vx2)
       !call interpo_l3_2d(vxg,xp,yp2,vy2)
       call interpo_l3_2d(vxg,xp2,yp,vx2)
       call interpo_l3_2d(vxg,xp,yp2,vy2)     

       do ib=1,npart                  
          vx(ib)=-vx(ib)+3.*vx1(ib)/4.+vx2(ib)+vy2(ib)/4.
       end do

       deallocate (xp1,xp2,yp1,yp2)  
       deallocate (vx1,vx2,vy1,vy2)


     end subroutine update_vx_3

     subroutine update_vy_3
       implicit none      
       integer                          :: ib,i,j
      
       allocate (xp1(1:npart),xp2(1:npart),yp1(1:npart),yp2(1:npart)) 
       allocate (xp0(1:npart)) 
       allocate (vx1(1:npart),vy1(1:npart),vy2(1:npart),vy3(1:npart))
       
       do ib=1,npart
          xp1(ib)=xp(ib)-2.*dt*vx(ib)/3.
          yp1(ib)=yp(ib)+dt*vy(ib)/3.
          xp0(ib)=xp(ib)-dt*vx(ib)/6.
       end do

       !call interpo_l3_2d(vxg1,xp0,yp1,vx1)
       !call interpo_l3_2d(vyg1,xp1,yp1,vy1)
       call interpo_l3_2d(vxg,xp0,yp1,vx1)
       call interpo_l3_2d(vyg,xp1,yp1,vy1)
     
       do ib=1,npart         
          xp2(ib)=xp(ib)+dt*(vx(ib)-vx1(ib))                                          
          yp2(ib)=yp(ib)+dt*(-vy(ib)+2.*vy1(ib))                                          
       end do
       
       !call interpo_l3_2d(vyg,xp2,yp,vy2)
       !call interpo_l3_2d(vyg3,xp,yp2,vy3)
       call interpo_l3_2d(vyg,xp2,yp,vy2)
       call interpo_l3_2d(vyg,xp,yp2,vy3)

       do ib=1,npart                  
          vy(ib)=-vy(ib)+3.*vy1(ib)/4.+vy2(ib)+vy3(ib)/4.
       end do
     

       deallocate (xp1,xp2,yp1,yp2)
       deallocate (xp0)
       deallocate (vx1,vy1,vy2,vy3)

     end subroutine update_vy_3





     !==================================
     !STRANG
     !==================================

     subroutine update_vx_strang
       implicit none
       integer                  :: ib

       allocate (xp1(1:npart))           
       do ib=1,npart         
          xp1(ib)=xp(ib)+dt*vx(ib)/2.
       end do
       !call interpo_l3_2d(vxg1,xp1,yp,vx) 
       call interpo_l3_2d(vxg,xp1,yp,vx)              
       deallocate(xp1)

     end subroutine update_vx_strang

     subroutine update_vy_strang
       implicit none
       integer                  :: ib

       allocate (yp1(1:npart))           
       do ib=1,npart         
          yp1(ib)=yp(ib)+0.5*dt*vy(ib)
       end do
       !call interpo_l3_2d(vyg1,xp,yp1,vy) 
       call interpo_l3_2d(vyg,xp,yp1,vy)              
       deallocate(yp1)

     end subroutine update_vy_strang


     !==================================
     !ordre 4 
     !==================================
          
     subroutine update_vx_4
       implicit none
       integer              :: ib


       allocate (xp1(1:npart),xp2(1:npart),xp3(1:npart)) 
       allocate (vx1(1:npart),vx2(1:npart),vx3(1:npart))           

       do ib=1,npart
          xp1(ib)=xp(ib)+0.5*dt*vx(ib)         
       end do

       call interpo_l5_2d(vxg,xp1,yp,vx1)      

       do ib=1,npart         
          xp2(ib)=xp(ib)+0.5*dt*vx1(ib)                                                                                 
       end do
      
       call interpo_l5_2d(vxg,xp2,yp,vx2)

       do ib=1,npart         
          xp3(ib)=xp(ib)+dt*vx2(ib)                                          
       end do
      
       call interpo_l5_2d(vxg,xp3,yp,vx3)

       do ib=1,npart                  
          vx(ib)=(vx(ib)+2.*vx1(ib)+2.*vx2(ib)+vx3(ib))/6.
       end do

       deallocate (xp1,xp2,xp3)  
       deallocate (vx1,vx2,vx3)

     end subroutine update_vx_4


     subroutine update_vy_4
       implicit none
       integer              :: ib


       allocate (yp1(1:npart),yp2(1:npart),yp3(1:npart)) 
       allocate (vy1(1:npart),vy2(1:npart),vy3(1:npart))           

       do ib=1,npart         
          yp1(ib)=yp(ib)+0.5*dt*vy(ib)
       end do
      
       call interpo_l5_2d(vyg,xp,yp1,vy1)      

       do ib=1,npart                                                        
          yp2(ib)=yp(ib)+0.5*dt*vy1(ib)                                          
       end do
      
       call interpo_l5_2d(vyg,xp,yp2,vy2)

       do ib=1,npart                                                       
          yp3(ib)=yp(ib)+dt*vy2(ib)                                          
       end do
      
       call interpo_l5_2d(vyg,xp,yp3,vy3)

       do ib=1,npart                           
          vy(ib)=(vy(ib)+2.*vy1(ib)+2.*vy2(ib)+vy3(ib))/6.
       end do

       deallocate (yp1,yp2,yp3)  
       deallocate (vy1,vy2,vy3)


     end subroutine update_vy_4




end module advection_mod


