module interpolation_mod
  use donnees_mod
  contains


    subroutine  interpo_l4_2d(tab_grille,posx,posy,tab_part)      
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_grille
      real(kind=8),dimension(:),intent(in)          :: posx,posy
      real(kind=8),dimension(1:npart),intent(out)   :: tab_part
      integer                                       :: i,c,d,ib,jb
      integer,dimension(0:5)                        :: ip,jp
      real(kind=8),dimension(0:5)                   :: poidx,poidy
      real(kind=8)                                  :: xx1,yy1,x2,x3,x4,x5,y1,y2,y3,y4,y5

    tab_part(1:npart)=0.  

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx_gro)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy_gro)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx_gro-xg)/dx_gro   !relatif
       yy1 = (posy(i) - real(jp(2),kind=8)*dy_gro-yb)/dy_gro   !relatif



       !conditions au bord
       !------------------
       !periodique:    
       do c=0,5               
          ip(c)=mod(ip(c)+nx_gro,nx_gro)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny_gro,ny_gro)                
       end do


       !calcul des poids
       !----------------   
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4

       if (xx1<=0.5) then
          poidx(0)=(2.*xx1-x2-2*x3+x4)/24.
          poidx(1)=(-4.*xx1+4.*x2+x3-x4)/6.
          poidx(2)=1.+(-5.*x2+x4)/4.
          poidx(3)=(4.*xx1+4.*x2-x3-x4)/6.
          poidx(4)=(-2.*xx1-x2+2*x3+x4)/24.
          poidx(5)=0.
       else
          poidx(0)=0.
          poidx(1)=(-6.*xx1+11.*x2-6.*x3+x4)/24.
          poidx(2)=1.+(-5.*xx1-5.*x2+5.*x3-x4)/6.
          poidx(3)=(6.*xx1+x2-4.*x3+x4)/4.
          poidx(4)=(-3.*xx1+x2+3.*x3-x4)/6.
          poidx(5)=(2.*xx1-x2-2.*x3+x4)/24.
       end if

       if (yy1<=0.5) then
          poidy(0)=(2.*yy1-y2-2*y3+y4)/24.
          poidy(1)=(-4.*yy1+4.*y2+y3-y4)/6.
          poidy(2)=1.+(-5.*y2+y4)/4.
          poidy(3)=(4.*yy1+4.*y2-y3-y4)/6.
          poidy(4)=(-2.*yy1-y2+2*y3+y4)/24.
          poidy(5)=0.
       else
          poidy(0)=0.
          poidy(1)=(-6.*yy1+11.*y2-6.*y3+y4)/24.
          poidy(2)=1.+(-5.*yy1-5.*y2+5.*y3-y4)/6.
          poidy(3)=(6.*yy1+y2-4.*y3+y4)/4.
          poidy(4)=(-3.*yy1+y2+3.*y3-y4)/6.
          poidy(5)=(2.*yy1-y2-2.*y3+y4)/24.
       end if

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          do d=0,5              
             tab_part(i)=tab_part(i)+tab_grille(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
          end do
       end do
    end do
  end subroutine interpo_l4_2d

   subroutine  interpo_l5_2d(tab_grille,posx,posy,tab_part)      
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_grille
      real(kind=8),dimension(:),intent(in)          :: posx,posy
      real(kind=8),dimension(1:npart),intent(out)   :: tab_part
      integer                                       :: i,c,d,ib,jb
      integer,dimension(0:5)                        :: ip,jp
      real(kind=8),dimension(0:5)                   :: poidx,poidy
      real(kind=8)                                  :: xx1,yy1,x2,x3,x4,x5,y1,y2,y3,y4,y5
      
      tab_part(1:npart)=0.  

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx_gro)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy_gro)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx_gro-xg)/dx_gro   !relatif
       yy1 = (posy(i) - real(jp(2),kind=8)*dy_gro-yb)/dy_gro   !relatif



       !conditions au bord
       !------------------
       !periodique:          
       do c=0,5               
          ip(c)=mod(ip(c)+nx_gro,nx_gro)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny_gro,ny_gro)                
       end do


       !calcul des poids
       !----------------  
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4
       y5=yy1**5

       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.

       poidy(0)=yy1/20.-y2/24.-y3/24.+y4/24.-y5/120.
       poidy(1)=-yy1/2.+2.*y2/3.-y3/24.-y4/6.+y5/24.
       poidy(2)=1.-yy1/3.-5*y2/4.+5.*y3/12.+y4/4.-y5/12.
       poidy(3)=yy1+2.*y2/3.-7.*y3/12.-y4/6.+y5/12.
       poidy(4)=-yy1/4.-y2/24.+7.*y3/24.+y4/24.-y5/24.
       poidy(5)=yy1/30.-y3/24.+y5/120.



       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          do d=0,5              
             tab_part(i)=tab_part(i)+tab_grille(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
          end do
       end do
    end do

  end subroutine interpo_l5_2d

   

 subroutine  interpo_l3_2d(tab_grille,posx,posy,tab_part)      
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_grille
      real(kind=8),dimension(:),intent(in)          :: posx,posy
      real(kind=8),dimension(1:npart),intent(out)   :: tab_part
      integer                                       :: i,c,d,ib,jb
      integer,dimension(0:3)                        :: ip,jp
      real(kind=8),dimension(0:3)                   :: poidx,poidy
      real(kind=8)                                  :: xx1,yy1
      
      tab_part(1:npart)=0.      

      !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
      !---------------------------------------------------------------------------------------------------------

      do i=1,npart

         !numero des points sur le maillage
         !--------------------------------
         ip(1) = floor((posx(i)-xg)/dx_gro)         
         ip(0) = ip(1) - 1        
         ip(2) = ip(1) + 1         
         ip(3) = ip(1) + 2

         jp(1) = floor((posy(i)-yb)/dy_gro)         
         jp(0) = jp(1) - 1        
         jp(2) = jp(1) + 1         
         jp(3) = jp(1) + 2
                  

         !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
         !----------------------------------------------------         
          xx1 = (posx(i) - real(ip(1),kind=8)*dx_gro-xg)/dx_gro  
          yy1 = (posy(i) - real(jp(1),kind=8)*dy_gro-yb)/dy_gro  

          !conditions au bord
          !------------------
          !periodique:       
          do c=0,3               
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=0,3               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do          
         
         !calcul des poids
         !----------------
         poidx(0)=-1./6.*xx1*(xx1-1.)*(xx1-2.)
         poidx(1)=0.5*(1.-xx1)*(1.+xx1)*(2.-xx1)
         poidx(2)=-0.5*xx1*(xx1+1.)*(xx1-2.)
         poidx(3)=1/6.*xx1*(1.+xx1)*(xx1-1.)

         poidy(0)=-1./6.*yy1*(yy1-1.)*(yy1-2.)
         poidy(1)=0.5*(1.-yy1)*(1.+yy1)*(2.-yy1)
         poidy(2)=-0.5*yy1*(yy1+1.)*(yy1-2.)
         poidy(3)=1/6.*yy1*(1.+yy1)*(yy1-1.)

         !remaillage à l' interrieur domaine
         !---------------------------------         
         do c=0,3
            do d=0,3               
               tab_part(i)=tab_part(i)+tab_grille(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
            end do
         end do
      end do
    end subroutine interpo_l3_2d


 subroutine  interpo_l2_2d(tab_grille,posx,posy,tab_part)      
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_grille
      real(kind=8),dimension(:),intent(in)          :: posx,posy
      real(kind=8),dimension(1:npart),intent(out)   :: tab_part
      integer                                       :: i,c,d,ib,jb
      integer,dimension(0:2)                        :: ip,jp
      real(kind=8),dimension(0:2)                   :: poidx,poidy
      real(kind=8)                                  :: xx1,yy1
      
      tab_part(1:npart)=0.      

      !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
      !---------------------------------------------------------------------------------------------------------

      do i=1,npart

         !numero des points sur le maillage
         !--------------------------------
         ip(1) = floor((posx(i)-xg)/dx_gro)         
         ip(0) = ip(1) - 1        
         ip(2) = ip(1) + 1         
         
         jp(1) = floor((posy(i)-yb)/dy_gro)         
         jp(0) = jp(1) - 1        
         jp(2) = jp(1) + 1         
                  

         !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
         !----------------------------------------------------         
          xx1 = (posx(i) - real(ip(1),kind=8)*dx_gro-xg)/dx_gro   !relatif
          yy1 = (posy(i) - real(jp(1),kind=8)*dy_gro-yb)/dy_gro   !relatif

          !conditions au bord
          !------------------
          !periodique:       
          do c=0,2               
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=0,2               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do          
         
         !calcul des poids
         !----------------        
         poidx(0)=0.5*xx1*(xx1-1.) 
         poidx(1)=1.-xx1**2        
         poidx(2)=0.5*xx1*(1.+xx1) 
        
         poidy(0)=0.5*yy1*(yy1-1.) 
         poidy(1)=1.-yy1**2        
         poidy(2)=0.5*yy1*(1.+yy1)
         

         !remaillage à l' interrieur domaine
         !---------------------------------         
         do c=0,2
            do d=0,2               
               tab_part(i)=tab_part(i)+tab_grille(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
            end do
         end do
      end do
    end subroutine interpo_l2_2d

    subroutine  interpo_l1_2d(tab_grille,posx,posy,tab_part)      
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_grille
      real(kind=8),dimension(:),intent(in)          :: posx,posy
      real(kind=8),dimension(1:npart),intent(out)   :: tab_part
      integer                                       :: i,c,d,ib,jb
      integer,dimension(1:2)                        :: ip,jp
      real(kind=8),dimension(1:2)                   :: poidx,poidy
      real(kind=8)                                  :: xx1,yy1
      
      tab_part=0.      

      !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
      !---------------------------------------------------------------------------------------------------------

      do i=1,npart

         !numero des points sur le maillage
         !--------------------------------
         ip(1) = floor((posx(i)-xg)/dx_gro)         
         ip(2) = ip(1) + 1        
               
         
         jp(1) = floor((posy(i)-yb)/dy_gro)         
         jp(2) = jp(1) + 1        
              
                  

         !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
         !----------------------------------------------------         
          xx1 = (posx(i) - real(ip(1),kind=8)*dx_gro-xg)/dx_gro   !relatif
          yy1 = (posy(i) - real(jp(1),kind=8)*dy_gro-yb)/dy_gro   !relatif

          !conditions au bord
          !------------------
          !periodique:       
          do c=1,2              
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=1,2               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do          
         
         !calcul des poids
         !----------------        
         poidx(1)=1.-xx1
         poidx(2)=xx1               
        
         poidy(1)=1.-yy1
         poidy(2)=yy1        
        
         

         !remaillage à l' interrieur domaine
         !---------------------------------         
         do c=1,2
            do d=1,2               
               tab_part(i)=tab_part(i)+tab_grille(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
            end do
         end do
      end do
    end subroutine interpo_l1_2d





    !===========================================================================
    !pour interpoler le champ de vitesse de la grille grossiere à la grille fine
    !===========================================================================


    function interpol_v_l1(tab_gro,posx,posy) result (v)
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_gro
      real(kind=8),intent(in)                       :: posx,posy
      real(kind=8)                                  :: v
      integer,dimension(0:1)                        :: ip,jp
      real(kind=8),dimension(0:1)                   :: poidx,poidy
      real(kind=8)                                  :: xx,yy
      integer                                       :: c,d

      v=0.

         !numero des points sur le maillage
         !--------------------------------
         ip(0) = floor((posx-xg)/dx_gro)       !de 0 à nx_gro -1
         ip(1) = ip(0)+1        
               
         jp(0) = floor((posy-yb)/dy_gro)       
         jp(1) = jp(0)+1 
                 
   
         !distance de la particule à remailler au premier point gauche
         !----------------------------------------------------------- 
          xx = (posx - real(ip(0),kind=8)*dx_gro-xg)/dx_gro   
          yy = (posy - real(jp(0),kind=8)*dy_gro-yb)/dy_gro   

          !conditions au bord
          !------------------
          !periodique:       
          do c=0,1            
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=0,1               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do
         
         !calcul des poids
         !----------------        
         poidx(0)=1.-xx
         poidx(1)=xx               
        
         poidy(0)=1.-yy
         poidy(1)=yy        
         
         !remaillage à l' interrieur domaine
         !---------------------------------         
         do c=0,1
            do d=0,1               
               v=v+tab_gro(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
            end do
         end do           

    end function interpol_v_l1
   
    function interpol_v_l3(tab_gro,posx,posy) result (v)
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_gro
      real(kind=8),intent(in)                       :: posx,posy
      real(kind=8)                                  :: v
      integer,dimension(-1:2)                       :: ip,jp
      real(kind=8),dimension(-1:2)                  :: poidx,poidy
      real(kind=8)                                  :: xx,yy
      integer                                       :: c,d

      v=0.

         !numero des points sur le maillage
         !--------------------------------
         ip(0) = floor((posx-xg)/dx_gro)       !de 0 à nx_gro -1
         ip(-1) = ip(0)-1
         ip(1) = ip(0)+1
         ip(2) = ip(0)+2        
               
         jp(0) = floor((posy-yb)/dy_gro)       
         jp(-1) = jp(0)-1 
         jp(1) = jp(0)+1 
         jp(2) = jp(0)+2 
                 
                  
         !distance de la particule à remailler au premier point gauche
         !----------------------------------------------------------- 
          xx = (posx - real(ip(0),kind=8)*dx_gro-xg)/dx_gro   
          yy = (posy - real(jp(0),kind=8)*dy_gro-yb)/dy_gro   

          !conditions au bord
          !------------------
          !periodique:       
          do c=-1,2            
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=-1,2               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do
 
         !calcul des poids
         !----------------        
          poidx(-1)=-1./6.*xx*(xx-1.)*(xx-2.)
          poidx(0)=0.5*(1.-xx)*(1.+xx)*(2.-xx)
          poidx(1)=-0.5*xx*(xx+1.)*(xx-2.)
          poidx(2)=1/6.*xx*(1.+xx)*(xx-1.)
          
          poidy(-1)=-1./6.*yy*(yy-1.)*(yy-2.)
          poidy(0)=0.5*(1.-yy)*(1.+yy)*(2.-yy)
          poidy(1)=-0.5*yy*(yy+1.)*(yy-2.)
          poidy(2)=1/6.*yy*(1.+yy)*(yy-1.) 
        
         
         !remaillage à l' interrieur domaine
         !---------------------------------         
         do c=-1,2
            do d=-1,2               
               v=v+tab_gro(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
            end do
         end do           

    end function interpol_v_l3

    function interpol_v_l5(tab_gro,posx,posy) result (v)
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:),intent(in)        :: tab_gro
      real(kind=8),intent(in)                       :: posx,posy
      real(kind=8)                                  :: v
      integer,dimension(-2:3)                       :: ip,jp
      real(kind=8),dimension(-2:3)                  :: poidx,poidy
      real(kind=8)                                  :: xx,yy
      integer                                       :: c,d
      real(kind=8)                                  :: x2,x3,x4,x5,y2,y3,y4,y5

      v=0.

         !numero des points sur le maillage
         !--------------------------------
         ip(0) = floor((posx-xg)/dx_gro)      !de 0 à nx_gro -1
         ip(-2) = ip(0)-2
         ip(-1) = ip(0)-1
         ip(1) = ip(0)+1
         ip(2) = ip(0)+2  
         ip(3) = ip(0)+3       
               
         jp(0) = floor((posy-yb)/dy_gro)       
         jp(-2) = jp(0)-2
         jp(-1) = jp(0)-1 
         jp(1) = jp(0)+1 
         jp(2) = jp(0)+2 
         jp(3) = jp(0)+3 
                 
                  
         !distance de la particule à remailler au premier point gauche
         !----------------------------------------------------------- 
          xx = (posx - real(ip(0),kind=8)*dx_gro-xg)/dx_gro   
          yy = (posy - real(jp(0),kind=8)*dy_gro-yb)/dy_gro   

          !conditions au bord
          !------------------
          !periodique:       
          do c=-2,3            
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=-2,3               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do
         
         !calcul des poids
         !----------------        
          x2=xx**2
          x3=xx**3
          x4=xx**4
          x5=xx**5
          y2=yy**2
          y3=yy**3
          y4=yy**4
          y5=yy**5

          poidx(-2)=xx/20.-x2/24.-x3/24.+x4/24.-x5/120.
          poidx(-1)=-xx/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
          poidx(0)=1.-xx/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
          poidx(1)=xx+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
          poidx(2)=-xx/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
          poidx(3)=xx/30.-x3/24.+x5/120.

          poidy(-2)=yy/20.-y2/24.-y3/24.+y4/24.-y5/120.
          poidy(-1)=-yy/2.+2.*y2/3.-y3/24.-y4/6.+y5/24.
          poidy(0)=1.-yy/3.-5*y2/4.+5.*y3/12.+y4/4.-y5/12.
          poidy(1)=yy+2.*y2/3.-7.*y3/12.-y4/6.+y5/12.
          poidy(2)=-yy/4.-y2/24.+7.*y3/24.+y4/24.-y5/24.
          poidy(3)=yy/30.-y3/24.+y5/120.
        
         
         !remaillage à l' interrieur domaine
         !---------------------------------         
         do c=-2,3
            do d=-2,3               
               v=v+tab_gro(ip(c)+1,jp(d)+1)*poidx(c)*poidy(d)
            end do
         end do           

    end function interpol_v_l5
   





  end module interpolation_mod
