module utile_mod
  use donnees_mod
  use tab_mod
  use interpolation_mod
  use advection_mod
contains


  subroutine make_grille
    implicit none
    integer            :: i,j  

    do j=1,ny
       do i=1,nx
          xtab(i,j)=xg+(i-1)*dx
          ytab(i,j)=yb+(j-1)*dy
       end do
    end do

  end subroutine make_grille


  function tsource(t,x,y)
    implicit none
    real(kind=8),intent(in)  :: t,x,y
    real(kind=8)             :: tsource,t1,t2,t3,t4,t5

    t1=1.
    t2=1.
    t3=0.2
    t4=0.3
    t5=1.


    !u=u(t,x,y)
    !----------
    tsource=-( pi*sin(pi*t/t3)*cos(2.*pi*x/t1)*cos(2.*pi*y/t2) )/t3  - (2.*pi*t5*sin(2.*pi*x/t1)**2*sin(2.*pi*y/t1)*cos(pi*t/t4)*cos(pi*t/t3)*cos(2.*pi*y/t2) )/t1  -(2.*pi*t5*cos(2.*pi*x/t1)**2*cos(2.*pi*y/t1)*cos(pi*t/t4)*cos(pi*t/t3)*sin(2.*pi*y/t2))/t2

    !tsource=0.


  end function tsource



  subroutine source_rk4
    implicit none
    integer                                  :: i
    real(kind=8)                             :: rk1,rk2,rk3,rk4,t1,t2,t3
    real(kind=8)                             :: x1,x2,x3,y1,y2,y3,v1x,v2x,v3x,v1y,v2y,v3y



    do i=1,npart

       !euler
       !------
       ! qp(i)=qp(i)+dt*tsource(time,xp(i),yp(i))

       !rk2
       !----
       !qp(i)=qp(i)+dt*tsource(time+0.5*dt,xp(i)+0.5*dt*vx(i),yp(i)+0.5*dt*vy(i))        

       !rk3
       !---
!!$           rk1=tsource(time,xp(i),yp(i))
!!$           x1=xp(i)+0.5*dt*vx(i)
!!$           y1=yp(i)+0.5*dt*vy(i)
!!$           rk2=tsource(time+0.5*dt,x1,y1)
!!$           call evalv (v1x,v1y,time+0.5*dt,x1,y1)
!!$           x2=xp(i)-dt*vx(i)+2.*dt*v1x
!!$           y2=yp(i)-dt*vy(i)+2.*dt*v1y
!!$           rk3=tsource(time+dt,x2,y2)
!!$           
!!$           qp(i)=qp(i)+dt*(rk1/6.+2.*rk2/3.+rk3/6.)

       !rk4
       !---
       rk1=tsource(time,xp(i),yp(i))
       x1=xp(i)+0.5*dt*vx(i)
       y1=yp(i)+0.5*dt*vy(i)
       rk2=tsource(time+0.5*dt,x1,y1)
       call evalv (v1x,v1y,time+0.5*dt,x1,y1)
       x2=xp(i)+0.5*dt*v1x
       y2=yp(i)+0.5*dt*v1y
       rk3=tsource(time+0.5*dt,x2,y2)
       call evalv (v2x,v2y,time+0.5*dt,x2,y2)
       x3=xp(i)+dt*v2x
       y3=yp(i)+dt*v2y
       rk4=tsource(time+dt,x3,y3)

       qp(i)=qp(i)+dt*(rk1+2.*rk2+2.*rk3+rk4)/6.

    end do


  end subroutine source_rk4


  subroutine source_split_1
    implicit none
    integer                                  :: i,j
    real(kind=8)                             :: rk1,rk2,rk3,rk4,t1,t2,t3
    real(kind=8)                             :: x1,x2,x3,y1,y2,y3,v1x,v2x,v3x,v1y,v2y,v3y
    real(kind=8)                             :: x,y


    !ordre2
    !------
    do i=1,npart           
       qp(i)=qp(i)+0.5*dt*tsource(time,xp(i),yp(i))
    end do


  end subroutine source_split_1

  subroutine source_split_2
    implicit none
    integer                                  :: i,j
    real(kind=8)                             :: rk1,rk2,rk3,rk4,t1,t2,t3
    real(kind=8)                             :: x0,x1,x2,x3,y1,y2,y3,v1x,v2x,v3x,v1y,v2y,v3y
    real(kind=8)                             :: x,y


    !ordre2
    !------
    do  j=1,ny
       do i=1,nx
          x=xtab(i,j)
          y=ytab(i,j)

          qg(i,j)=qg(i,j)+0.5*dt*tsource(time+dt,x,y)

       end do
    end do


  end subroutine source_split_2


  subroutine make_bloc(esp)
    use donnees_mod
    implicit none
    integer,intent(in)       :: esp
    integer                  :: fin,ib,i,j,k,im,ip,dim1,dim2
    logical                  :: zero
    real(kind=8)             :: pas

    select case(esp)
    case(1)
       dim1=nx
       dim2=ny
       pas=dx
    case(2)
       dim1=ny
       dim2=nx
       pas=dy
    end select


    !-----------------------------------------------------------------------------------------
    !Determine la nature des blocs
    !Donne un flag correspondant au type de remaillage necessaire (centre,decentre ou modifie)
    !------------------------------------------------------------------------------------------

    blocg(1:npart)=1
    blocd(1:npart)=1    
         

    ordonne: do j=1,dim2

       !peut calculer la longeur -> gain si longeur eleve

       !parcours la grille, bloc par bloc:determine le type des bloc
       !---------------------------------

       blocg(0)=100
       blocd(0)=100
       Nbloc(0)=100

       type_bloc:do i=1,dim1,long_bloc+1


          if ( (i+long_bloc)<(dim1-1) ) then
             fin=i+long_bloc
             call sub_typeb
          else
             !parcours le bloc plus premier point du bloc suivant
             fin=dim1-1
             call sub_typeb            
             blocg(pg(dim1,j))=blocg(pg(dim1-1,j))
             blocd(pg(dim1,j))=blocd(pg(dim1-1,j))               
             Nbloc(pg(dim1,j))=Nbloc(pg(dim1-1,j))            
          end if         

       end do type_bloc


       !parcours la grille, bloc par bloc:determine si modif de remaillage necessaire a l'intersection des blocs
       !---------------------------------
       type_remaill:do i=1,dim1,long_bloc+1

          if (((i+long_bloc)<dim1).and.(i>1+long_bloc)) then            
             fin=i+long_bloc
             call sub_remb
          else             
             if (i<=1+long_bloc)then
                !premier bloc
                !-------------
                fin=i+long_bloc
                call sub_remb              

             else
                !dernier bloc 
                !-------------
                fin=dim1
                call sub_remb

             end if

          end if

       end do type_remaill
    end do ordonne

  contains

    subroutine sub_typeb

      integer            :: ni,N
      real(kind=8)       :: li
      logical            :: pasfait


      !parcours une premiere fois le bloc pour savoir si c'est un bloc en 0 ou 1/2
      !----------------------------------     
      pasfait=.true.
      zero=.true.
      do_zero:do ib=i,fin+1  !eviter cas change de bloc quand maille vide
         if (pg(ib,j)/=0) then
            if (pasfait) then  !le N doit etre le meme pour tout le bloc !
               li=dt/pas*vit(pg(ib,j))
               ni=floor(li+0.5)
               pasfait=.false.
            end if
            li=dt/pas*vit(pg(ib,j))
            if ( ((li)<(ni-0.5)).or.((li)>(ni+0.5)) ) then
               zero=.false.
               exit do_zero 
            end if
         end if
      end do do_zero


      !parcours a nouveau le bloc 
      !-------------------------
      !affecte le type de bloc a toute les part du bloc

      pasfait=.true.
      bloc2:do ib=i,fin

         if (pg(ib,j)/=0) then

            !calcul de N
            !------------   
            if (pasfait) then
               if (zero) then                 
                  N=floor(dt/pas*vit(pg(ib,j))+0.5)
                  pasfait=.false.
               else
                  N=floor(dt/pas*vit(pg(ib,j)))
                  pasfait=.false.
               end if
            end if

            if (zero) then
               !bloc pour lambda2 centre
               blocg(pg(ib,j))=0
               blocd(pg(ib,j))=0                
            end if

            !affecte la constante N               
            Nbloc(pg(ib,j))=N

         end if
      end do bloc2

    end subroutine sub_typeb


    subroutine sub_remb
      implicit none

      integer,dimension(-2:2)  :: iper
      integer                  :: k


      !parcours  a nvx le bloc     
      !-----------------------
      !detecte s'il y aura une correction de remaillage a apporter à l'intersection de ces blocs

      bloc3:do ib=i,fin,fin-i

         if (pg(ib,j)/=0) then

            !si fin de bloc en 0. :
            !----------------------
            do k=-2,2              
               iper(k)=pg(mod(ib+k+dim1-1,dim1)+1,j)          
            end do


            if ((ib==fin).and.(blocd(pg(ib,j))==0).and.((blocg(iper(1))==1).or.(blocg(iper(2))==1)).and.((Nbloc(iper(1))==Nbloc(pg(ib,j))-1) .or.(Nbloc(iper(2))==Nbloc(pg(ib,j))-1) ) ) then

               if (type_b==2) then

                  if ( dt/pas*vit(pg(ib,j))<=Nbloc(pg(ib,j))  ) then
                     blocg(iper(1))=2
                     blocd(pg(ib,j))=3
                  else
                     blocg(iper(1))=2
                     blocd(pg(ib,j))=4
                     blocg(pg(ib,j))=5
                  end if

               else
                  blocg(iper(1))=2
                  blocg(iper(2))=5

                  if ( dt/pas*vit(pg(ib,j))<=Nbloc(pg(ib,j))  ) then                 
                     blocd(pg(ib,j))=3
                  else                 
                     blocd(pg(ib,j))=7
                     blocg(pg(ib,j))=8
                  end if

                  if ( dt/pas*vit(iper(-1))<=Nbloc(pg(ib,j))  ) then
                     blocd(iper(-1))=6
                  else
                     blocd(iper(-1))=5
                  end if
               end if

            end if


            !si debut de bloc en 0 : detecte quelle correction il faudra apporter
            !----------------------
            if ((ib==i) .and.(blocg(pg(ib,j))==0).and.((blocd(iper(-1))==1).or.(blocd(iper(-2))==1)).and.((Nbloc(iper(-1))==Nbloc(pg(ib,j))-1) .or. (Nbloc(iper(-2))==Nbloc(pg(ib,j))-1) ) ) then

               if (type_b==2) then

                  if ( dt/pas*vit(pg(ib,j))>Nbloc(pg(ib,j)) ) then
                     blocd(iper(-1))=2
                     blocg(pg(ib,j))=4
                  else
                     blocd(iper(-1))=2
                     blocg(pg(ib,j))=3
                  end if

               else

                  blocd(iper(-1))=2
                  blocd(iper(-2))=4

                  if ( dt/pas*vit(pg(ib,j))>Nbloc(pg(ib,j)) ) then                 
                     blocg(pg(ib,j))=4
                  else                 
                     blocg(pg(ib,j))=3
                  end if

                  if ( dt/pas*vit(iper(1))>Nbloc(pg(ib,j)) ) then
                     blocg(iper(1))=7
                  else
                     blocg(iper(1))=6
                  end if

               end if
            end if



         end if
      end do bloc3

    end subroutine sub_remb

    function pg(a,b) 
      implicit none
      integer,intent(in)  :: a,b
      integer             :: pg

      select case (esp)
      case(1)
         pg=numpg(a,b)
      case(2)
         pg=numpg(b,a)
      end select

    end function pg

    function vit(ind) 
      implicit none
      integer,intent(in)  :: ind
      real(kind=8)        :: vit

      select case (esp)
      case(1)
         vit=vx(ind)
      case(2)
         vit=vy(ind)
      end select

    end function vit


  end subroutine make_bloc



  subroutine make_bloc_v2(esp)
    use donnees_mod
    implicit none
    integer,intent(in)       :: esp
    integer                  :: fin,ib,i,j,k,im,ip,dim1,dim2,n,np,part,partp,partm,partpp
    logical                  :: zero
    real(kind=8)             :: pas

    select case(esp)
    case(1)
       dim1=nx
       dim2=ny
       pas=dx
    case(2)
       dim1=ny
       dim2=nx
       pas=dy
    end select


    !-----------------------------------------------------------------------------------------
    !Determine la nature des blocs
    !Donne un flag correspondant au type de remaillage necessaire (centre,decentre ou modifie)
    !------------------------------------------------------------------------------------------

    blocg(1:npart)=0 
    blocd(1:npart)=0    

    blocg(0)=100 
    blocd(0)=100    
    Nbloc(0)=100

    ordonne: do j=1,dim2

       !peut calculer la longeur -> gain si longeur eleve

       !parcours la grille, bloc par bloc:determine le type des bloc
       !---------------------------------

      

       type_bloc:do i=1,dim1,long_bloc+1


          if ( (i+long_bloc).le.dim1 ) then
             fin=i+long_bloc
             call sub_typeb
          else
             !met les dernieres particules du meme type que bloc precedent
             do k=i,dim1
                blocg(pg(k,j))=blocg(pg(i-1,j))
                blocd(pg(k,j))=blocd(pg(i-1,j))                          
                Nbloc(pg(k,j))=Nbloc(pg(i-1,j)) 
             end do           
          end if

       end do type_bloc


       !parcours la grille, bloc par bloc:determine si modif de remaillage necessaire a l'intersection des blocs
       !---------------------------------
       type_remaill:do i=1,dim1,long_bloc+1

          if ((i+long_bloc)<dim1) then            
             fin=i+long_bloc
             !call sub_remb
            
             k=fin
             do while ((pg(k,j)==0).and.(k>i))
                k=k-1
             end do
             n=Nbloc(pg(k,j)) ! s'il n'y a pas de part dans le bloc N=100 -> ne fera pas de correction
             
             k=fin+1
             do while ((pg(k,j)==0).and.(k<(fin+long_bloc+2)).and.(k<nx))
                k=k+1
             end do
             np=Nbloc(pg(k,j))

             part=pg(fin,j)
             partp=pg(fin+1,j)
             partm=pg(fin-1,j)
             partpp=pg(fin+2,j)

             if (type_b==2) call sub_remb_2(n,np,part,partp)
             if (type_b==4) call sub_remb_4(n,np,part,partp,partm,partpp)
             
          else                          
             !dernier bloc 
             !-------------
             fin=dim1
             !call sub_remb

             k=fin
             do while ((pg(k,j)==0).and.(k>i))
                k=k-1
             end do
             n=Nbloc(pg(k,j)) 
             
             k=1
             do while ((pg(k,j)==0).and.(k<(1+long_bloc+1)))
                k=k+1
             end do
             np=Nbloc(pg(k,j))
     
             part=pg(fin,j)
             partp=pg(1,j)
             partm=pg(fin-1,j)
             partpp=pg(2,j)
             
             if (type_b==2) call sub_remb_2(n,np,part,partp)
             if (type_b==4) call sub_remb_4(n,np,part,partp,partm,partpp)


          end if

       end do type_remaill



    end do ordonne

  contains

    subroutine sub_typeb

      integer            :: N,intl
      real(kind=8)       :: li,lmin


      !calcul de lambda_min
      !--------------------
      lmin=99999999999999999999999.
      do_lmin:do ib=i,fin
         if (pg(ib,j)/=0) then
            li=dt/pas*vit(pg(ib,j))
            lmin=min(lmin,li)
         end if
      end do do_lmin

      !calcul du type de bloc et de N
      !-------------------------------
      intl=floor(lmin)
      if ((lmin-intl).le.0.5) then
         !type gauche
         !-----------
         N=intl
         do_typeg:do ib=i,fin
            if (pg(ib,j)/=0) then
               blocg(pg(ib,j))=1 
               blocd(pg(ib,j))=1  
               Nbloc(pg(ib,j))=N              
            end if
         end do do_typeg
      else
         !type centre
         !-----------
         N=intl+1
         do_typec:do ib=i,fin
            if (pg(ib,j)/=0) then
               blocg(pg(ib,j))=0
               blocd(pg(ib,j))=0
               Nbloc(pg(ib,j))=N
            end if
         end do do_typec
      end if


    end subroutine sub_typeb



    subroutine sub_remb_2(n,np,partf,partfp)
      !attention les bords ne sont pas bien traités
      implicit none

      integer,intent(in) ::n,np,partf,partfp
      real(kind=8)       :: li


      !parcours  a nvx le bloc     
      !-----------------------
      !detecte s'il y aura une correction de remaillage a apporter à l'intersection des blocs
      if ((np) == (n-1)) then        
         li=dt/pas*vit(partf)
         if ( (li<n).and.(blocg(partf)==0) ) then !seconde condition=bloc centre
            blocg(partf)=2
         else
            blocg(partf)=3
         end if
         blocg(partfp)=4
      end if
      if ((np) == (n+1)) then
         li=dt/pas*vit(partfp)
         blocg(partf)=5               
         if ((li<np).and.(blocg(partfp)==0)) then
            blocg(partfp)=6
         else
            blocg(partfp)=7
         end if
      end if



    end subroutine sub_remb_2


    subroutine sub_remb_4(n,np,partf,partfp,partfm,partfpp)
      !attention les bords ne sont pas bien traités
      implicit none

      integer,intent(in) ::n,np,partf,partfp,partfm,partfpp
      real(kind=8)       :: li


      !parcours  a nvx le bloc     
      !-----------------------
      !detecte s'il y aura une correction de remaillage a apporter à l'intersection des blocs

      !N_{m+1}=N_m -1
      !--------------
      if ((np) == (n-1)) then    
         !si bloc centre
         if (blocd(partfm)==0) then
            li=dt/pas*vit(partfm)           
            if ( (li<n) ) then 
               blocd(partfm)=2
            else
               blocd(partfm)=3
            end if
         else
            blocd(partfm)=3
         end if
         if (blocg(partf)==0) then
            li=dt/pas*vit(partf)
            if ( (li<n) ) then 
               blocg(partf)=2
               blocd(partf)=4
            else
               blocg(partf)=3
               blocd(partf)=5
            end if                      
         else
            blocg(partf)=3
            blocd(partf)=5           
         end if
         blocg(partfp)=4
         blocd(partfp)=1
         li=dt/pas*vit(partfpp)
         if ( (blocg(partfpp)==0).and.(li<np) ) then
            blocg(partfpp)=5
         else
            blocg(partfpp)=6
         end if
      end if
      !N_{m+1}=N_m +1
      !--------------
      if ((np) == (n+1)) then 
         li=dt/pas*vit(partfm)
         if ( (blocd(partfm)==0).and.(li<n) ) then
            blocd(partfm)=6
         else
            blocd(partfm)=7
         end if
         blocg(partf)=1
         blocd(partf)=8   
         if (blocg(partfp)==0) then
            li=dt/pas*vit(partfp)
            if ( (li<np) ) then 
               blocg(partfp)=7
               blocd(partfp)=0
            else
               blocg(partfp)=8
               blocd(partfp)=1
            end if
         else
            blocg(partfp)=8
            blocd(partfp)=1
         end if
         if (blocg(partfpp)==0) then
            li=dt/pas*vit(partfpp)
            if ( (li<np) ) then 
               blocg(partfpp)=9
            else
               blocg(partfpp)=10
            end if
         else
            blocg(partfpp)=10
         end if
                           
      end if           



    end subroutine sub_remb_4



    function pg(a,b) 
      implicit none
      integer,intent(in)  :: a,b
      integer             :: pg

      select case (esp)
      case(1)
         pg=numpg(a,b)
      case(2)
         pg=numpg(b,a)
      end select

    end function pg

    function vit(ind) 
      implicit none
      integer,intent(in)  :: ind
      real(kind=8)        :: vit

      select case (esp)
      case(1)
         vit=vx(ind)
      case(2)
         vit=vy(ind)
      end select

    end function vit


  end subroutine make_bloc_v2







end module utile_mod


