module remaillage_mod
  use donnees_mod
  use tab_mod
contains

  subroutine remaill_l2 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(1:4)                              :: ip,jp,kp
    real(kind=8),dimension(1:4)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1,x2,x3,x4,x5,y2,y3,y4,y5,z2,z3,z4,z5

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)           
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2

       jp(2) = floor((posy(i)-yb)/dy)                 
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2

       kp(2) = floor((posz(i)-zd)/dz)                 
       kp(1) = kp(2) - 1         
       kp(3) = kp(2) + 1
       kp(4) = kp(2) + 2



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(2),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=1,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=1,4               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=1,4               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
       if (xx1<=0.5) then   

          poidx(1)=-0.5*xx1*(1.-xx1)
          poidx(2)=1.-xx1**2
          poidx(3)=0.5*xx1*(1.+xx1)
          poidx(4)=0.

       else

          poidx(1)=0.
          poidx(2)=0.5*(xx1-1.)*(xx1-2.)
          poidx(3)=xx1*(2.-xx1)
          poidx(4)=0.5*xx1*(xx1-1.)

       end if

       if (yy1<=0.5) then

          poidy(1)=-0.5*yy1*(1.-yy1)
          poidy(2)=1.-yy1**2
          poidy(3)=0.5*yy1*(1.+yy1)
          poidy(4)=0.

       else

          poidy(1)=0.
          poidy(2)=0.5*(yy1-1.)*(yy1-2.)
          poidy(3)=yy1*(2.-yy1)
          poidy(4)=0.5*yy1*(yy1-1.)        

       end if

       if (zz1<=0.5) then

          poidz(1)=-0.5*zz1*(1.-zz1)
          poidz(2)=1.-zz1**2
          poidz(3)=0.5*zz1*(1.+zz1)
          poidz(4)=0.

       else

          poidz(1)=0.
          poidz(2)=0.5*(zz1-1.)*(zz1-2.)
          poidz(3)=zz1*(2.-zz1)
          poidz(4)=0.5*zz1*(zz1-1.)        

       end if



       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=1,4
          do d=1,4
             do c=1,4              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_l2

  subroutine remaill_l3 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(0:3)                              :: ip,jp,kp
    real(kind=8),dimension(0:3)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2
      
       jp(1) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(1) - 1        
       jp(2) = jp(1) + 1         
       jp(3) = jp(1) + 2
     

       kp(1) = floor((posz(i)-zd)/dz)         
       kp(0) = kp(1) - 1        
       kp(2) = kp(1) + 1         
       kp(3) = kp(1) + 2
     



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(1),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(1),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=0,3               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,3               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=0,3               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
       poidx(0)=-1./6.*xx1*(xx1-1.)*(xx1-2.)
       poidx(1)=0.5*(1.-xx1)*(1.+xx1)*(2.-xx1)
       poidx(2)=-0.5*xx1*(xx1+1.)*(xx1-2.)
       poidx(3)=1/6.*xx1*(1.+xx1)*(xx1-1.)

       poidy(0)=-1./6.*yy1*(yy1-1.)*(yy1-2.)
       poidy(1)=0.5*(1.-yy1)*(1.+yy1)*(2.-yy1)
       poidy(2)=-0.5*yy1*(yy1+1.)*(yy1-2.)
       poidy(3)=1/6.*yy1*(1.+yy1)*(yy1-1.)

       poidz(0)=-1./6.*zz1*(zz1-1.)*(zz1-2.)
       poidz(1)=0.5*(1.-zz1)*(1.+zz1)*(2.-zz1)
       poidz(2)=-0.5*zz1*(zz1+1.)*(zz1-2.)
       poidz(3)=1/6.*zz1*(1.+zz1)*(zz1-1.)


       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=0,3
          do d=0,3
             do c=0,3              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_l3

 subroutine remaill_mp4 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(0:3)                              :: ip,jp,kp
    real(kind=8),dimension(0:3)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1,x2,x3,x4,x5,y2,y3,y4,y5,z2,z3,z4,z5

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2
      
       jp(1) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(1) - 1        
       jp(2) = jp(1) + 1         
       jp(3) = jp(1) + 2
     

       kp(1) = floor((posz(i)-zd)/dz)         
       kp(0) = kp(1) - 1        
       kp(2) = kp(1) + 1         
       kp(3) = kp(1) + 2
     



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(1),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(1),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=0,3               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,3               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=0,3               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
      
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       poidx(0)=-0.5*xx1+x2-0.5*x3
       poidx(1)=1-5./2.*x2+3.*0.5*x3
       poidx(2)=0.5*xx1+2.*x2-3.*0.5*x3
       poidx(3)=-0.5*x2+0.5*x3

       y2=yy1**2
       y3=yy1**3
       y4=yy1**4
       y5=yy1**5
       poidy(0)=-0.5*yy1+y2-0.5*y3
       poidy(1)=1-5./2.*y2+3.*0.5*y3
       poidy(2)=0.5*yy1+2.*y2-3.*0.5*y3
       poidy(3)=-0.5*y2+0.5*y3

       z2=zz1**2
       z3=zz1**3
       z4=zz1**4
       z5=zz1**5
       poidz(0)=-0.5*zz1+z2-0.5*z3
       poidz(1)=1-5./2.*z2+3.*0.5*z3
       poidz(2)=0.5*zz1+2.*z2-3.*0.5*z3
       poidz(3)=-0.5*z2+0.5*z3


       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=0,3
          do d=0,3
             do c=0,3              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_mp4

  subroutine remaill_l4 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(0:5)                              :: ip,jp,kp
    real(kind=8),dimension(0:5)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1,x2,x3,x4,x5,y2,y3,y4,y5,z2,z3,z4,z5

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3

       kp(2) = floor((posz(i)-zd)/dz)         
       kp(0) = kp(2) - 2        
       kp(1) = kp(2) - 1         
       kp(3) = kp(2) + 1
       kp(4) = kp(2) + 2
       kp(5) = kp(2) + 3



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(2),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=0,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=0,5               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4            
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4            
       z2=zz1**2
       z3=zz1**3
       z4=zz1**4            

       if (xx1<=0.5) then
          poidx(0)=(2.*xx1-x2-2*x3+x4)/24.
          poidx(1)=(-4.*xx1+4.*x2+x3-x4)/6.
          poidx(2)=1.+(-5.*x2+x4)/4.
          poidx(3)=(4.*xx1+4.*x2-x3-x4)/6.
          poidx(4)=(-2.*xx1-x2+2*x3+x4)/24.
          poidx(5)=0.
       else
          poidx(0)=0.
          poidx(1)=(-6.*xx1+11.*x2-6.*x3+x4)/24.
          poidx(2)=1.+(-5.*xx1-5.*x2+5.*x3-x4)/6.
          poidx(3)=(6.*xx1+x2-4.*x3+x4)/4.
          poidx(4)=(-3.*xx1+x2+3.*x3-x4)/6.
          poidx(5)=(2.*xx1-x2-2.*x3+x4)/24.
       end if

       if (yy1<=0.5) then
          poidy(0)=(2.*yy1-y2-2*y3+y4)/24.
          poidy(1)=(-4.*yy1+4.*y2+y3-y4)/6.
          poidy(2)=1.+(-5.*y2+y4)/4.
          poidy(3)=(4.*yy1+4.*y2-y3-y4)/6.
          poidy(4)=(-2.*yy1-y2+2*y3+y4)/24.
          poidy(5)=0.
       else
          poidy(0)=0.
          poidy(1)=(-6.*yy1+11.*y2-6.*y3+y4)/24.
          poidy(2)=1.+(-5.*yy1-5.*y2+5.*y3-y4)/6.
          poidy(3)=(6.*yy1+y2-4.*y3+y4)/4.
          poidy(4)=(-3.*yy1+y2+3.*y3-y4)/6.
          poidy(5)=(2.*yy1-y2-2.*y3+y4)/24.
       end if
       if (zz1<=0.5) then
          poidz(0)=(2.*zz1-z2-2*z3+z4)/24.
          poidz(1)=(-4.*zz1+4.*z2+z3-z4)/6.
          poidz(2)=1.+(-5.*z2+z4)/4.
          poidz(3)=(4.*zz1+4.*z2-z3-z4)/6.
          poidz(4)=(-2.*zz1-z2+2*z3+z4)/24.
          poidz(5)=0.
       else
          poidz(0)=0.
          poidz(1)=(-6.*zz1+11.*z2-6.*z3+z4)/24.
          poidz(2)=1.+(-5.*zz1-5.*z2+5.*z3-z4)/6.
          poidz(3)=(6.*zz1+z2-4.*z3+z4)/4.
          poidz(4)=(-3.*zz1+z2+3.*z3-z4)/6.
          poidz(5)=(2.*zz1-z2-2.*z3+z4)/24.
       end if



       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=0,5
          do d=0,5
             do c=0,5              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_l4

  subroutine remaill_l5 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(0:5)                              :: ip,jp,kp
    real(kind=8),dimension(0:5)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1,x2,x3,x4,x5,y2,y3,y4,y5,z2,z3,z4,z5

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3

       kp(2) = floor((posz(i)-zd)/dz)         
       kp(0) = kp(2) - 2        
       kp(1) = kp(2) - 1         
       kp(3) = kp(2) + 1
       kp(4) = kp(2) + 2
       kp(5) = kp(2) + 3



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(2),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=0,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=0,5               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4
       y5=yy1**5
       z2=zz1**2
       z3=zz1**3
       z4=zz1**4
       z5=zz1**5

       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.

       poidy(0)=yy1/20.-y2/24.-y3/24.+y4/24.-y5/120.
       poidy(1)=-yy1/2.+2.*y2/3.-y3/24.-y4/6.+y5/24.
       poidy(2)=1.-yy1/3.-5*y2/4.+5.*y3/12.+y4/4.-y5/12.
       poidy(3)=yy1+2.*y2/3.-7.*y3/12.-y4/6.+y5/12.
       poidy(4)=-yy1/4.-y2/24.+7.*y3/24.+y4/24.-y5/24.
       poidy(5)=yy1/30.-y3/24.+y5/120.

       poidz(0)=zz1/20.-z2/24.-z3/24.+z4/24.-z5/120.
       poidz(1)=-zz1/2.+2.*z2/3.-z3/24.-z4/6.+z5/24.
       poidz(2)=1.-zz1/3.-5*z2/4.+5.*z3/12.+z4/4.-z5/12.
       poidz(3)=zz1+2.*z2/3.-7.*z3/12.-z4/6.+z5/12.
       poidz(4)=-zz1/4.-z2/24.+7.*z3/24.+z4/24.-z5/24.
       poidz(5)=zz1/30.-z3/24.+z5/120.

       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=0,5
          do d=0,5
             do c=0,5              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_l5


  subroutine remaill_l5_x(donne,posx,posy,posz,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k,kvar,jvar
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,x2,x3,x4,x5

    remaille=0.      

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------   
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx                         

       !conditions au bord
       !------------------         

       !periodique: 
       do c=0,5          
          ip(c)=mod(ip(c)+nx,nx)            
       end do



       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5

       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ip(c)+1,j,kvar)=remaille(ip(c)+1,j,kvar)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l5_x


  subroutine remaill_l5_y(donne,posx,posy,posz,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k,kvar,jvar
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,x2,x3,x4,x5

    remaille=0.      

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posy(i)-yb)/dy)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------   
       xx1 = (posy(i) - real(ip(2),kind=8)*dy-yb)/dy                         

       !conditions au bord
       !------------------         

       !periodique: 
       do c=0,5          
          ip(c)=mod(ip(c)+ny,ny)            
       end do



       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5

       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l5_y


  subroutine remaill_l5_z(donne,posx,posy,posz,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k,kvar,jvar
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,x2,x3,x4,x5

    remaille=0.      

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1
       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posz(i)-zd)/dz)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------   
       xx1 = (posz(i) - real(ip(2),kind=8)*dz-zd)/dz                         

       !conditions au bord
       !------------------         

       !periodique: 
       do c=0,5          
          ip(c)=mod(ip(c)+nz,nz)            
       end do



       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5

       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.


       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l5_z







  subroutine remaill_l6 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(-3:4)                             :: ip,jp,kp
    real(kind=8),dimension(-3:4)                        :: poidx,poidy,poidz
    real(kind=8)                                        :: xx,yy,zz
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6

    remaille=0.      


    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4

       jp(0) = floor((posy(i)-yb)/dy)         
       jp(-1) = jp(0) - 1        
       jp(-2) = jp(0) - 2         
       jp(-3) = jp(0) - 3
       jp(1) = jp(0) + 1
       jp(2) = jp(0) + 2
       jp(3) = jp(0) + 3
       jp(4) = jp(0) + 4

       kp(0) = floor((posz(i)-zd)/dz)         
       kp(-1) = kp(0) - 1        
       kp(-2) = kp(0) - 2         
       kp(-3) = kp(0) - 3
       kp(1) = kp(0) + 1
       kp(2) = kp(0) + 2
       kp(3) = kp(0) + 3
       kp(4) = kp(0) + 4




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif
       yy = (posy(i) - real(jp(0),kind=8)*dy-yb)/dy   !relatif
       zz = (posz(i) - real(kp(0),kind=8)*dz-zd)/dz   !relatif



       !conditions au bord
       !------------------
       !periodique:          
       do c=-3,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=-3,4              
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=-3,4              
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------        
       if (xx<=0.5) then         
          t3=(xx+1.)*(xx+2.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+3.)
          t6=(xx-1.)*xx

          t1=t4*(xx-1.)
          t2=t3*(xx+3.)
        
          poidx(-3)=t1*xx*t3/720.
          poidx(-2)=-t1*t5*(xx+1.)/120.
          poidx(-1)=t1*t5*(xx+2.)/48.
          poidx(0)=-t1*t2/36.
          poidx(1)=t4*xx*t2/48.
          poidx(2)=-(xx-3.)*t6*t2/120.
          poidx(3)=(xx-2.)*t6*t2/720.
          poidx(4)=0.
       else
         
          t3=xx*(xx+1.)
          t4=(xx-3.)*(xx-2.)

          t1=(xx-4.)*t4*(xx-1.)
          t2=(xx-1.)*t3*(xx+2.)

          poidx(-3)=0.
          poidx(-2)=t1*t3/720.
          poidx(-1)=-t1*xx*(xx+2.)/120.
          poidx(0)=t1*(xx+1.)*(xx+2.)/48.
          poidx(1)=-(xx-4.)*t4*t3*(xx+2.)/36.
          poidx(2)=(xx-4.)*(xx-3.)*t2/48.
          poidx(3)=-(xx-4.)*(xx-2.)*t2/120.
          poidx(4)=t4*t2/720.
       end if

       if (yy<=0.5) then         
          t3=(yy+1.)*(yy+2.)
          t4=(yy-3.)*(yy-2.)
          t5=yy*(yy+3.)
          t6=(yy-1.)*yy

          t1=t4*(yy-1.)
          t2=t3*(yy+3.)
        
          poidy(-3)=t1*yy*t3/720.
          poidy(-2)=-t1*t5*(yy+1.)/120.
          poidy(-1)=t1*t5*(yy+2.)/48.
          poidy(0)=-t1*t2/36.
          poidy(1)=t4*yy*t2/48.
          poidy(2)=-(yy-3.)*t6*t2/120.
          poidy(3)=(yy-2.)*t6*t2/720.
          poidy(4)=0.
       else
         
          t3=yy*(yy+1.)
          t4=(yy-3.)*(yy-2.)

          t1=(yy-4.)*t4*(yy-1.)
          t2=(yy-1.)*t3*(yy+2.)

          poidy(-3)=0.
          poidy(-2)=t1*t3/720.
          poidy(-1)=-t1*yy*(yy+2.)/120.
          poidy(0)=t1*(yy+1.)*(yy+2.)/48.
          poidy(1)=-(yy-4.)*t4*t3*(yy+2.)/36.
          poidy(2)=(yy-4.)*(yy-3.)*t2/48.
          poidy(3)=-(yy-4.)*(yy-2.)*t2/120.
          poidy(4)=t4*t2/720.
       end if

       if (zz<=0.5) then         
          t3=(zz+1.)*(zz+2.)
          t4=(zz-3.)*(zz-2.)
          t5=zz*(zz+3.)
          t6=(zz-1.)*zz

          t1=t4*(zz-1.)
          t2=t3*(zz+3.)
        
          poidz(-3)=t1*zz*t3/720.
          poidz(-2)=-t1*t5*(zz+1.)/120.
          poidz(-1)=t1*t5*(zz+2.)/48.
          poidz(0)=-t1*t2/36.
          poidz(1)=t4*zz*t2/48.
          poidz(2)=-(zz-3.)*t6*t2/120.
          poidz(3)=(zz-2.)*t6*t2/720.
          poidz(4)=0.
       else
         
          t3=zz*(zz+1.)
          t4=(zz-3.)*(zz-2.)

          t1=(zz-4.)*t4*(zz-1.)
          t2=(zz-1.)*t3*(zz+2.)

          poidz(-3)=0.
          poidz(-2)=t1*t3/720.
          poidz(-1)=-t1*zz*(zz+2.)/120.
          poidz(0)=t1*(zz+1.)*(zz+2.)/48.
          poidz(1)=-(zz-4.)*t4*t3*(zz+2.)/36.
          poidz(2)=(zz-4.)*(zz-3.)*t2/48.
          poidz(3)=-(zz-4.)*(zz-2.)*t2/120.
          poidz(4)=t4*t2/720.
       end if
      

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do e=-3,4
          do d=-3,4
             do c=-3,4              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do

  end subroutine remaill_l6



  subroutine remaill_l8 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(-4:5)                             :: ip,jp,kp
    real(kind=8),dimension(-4:5)                        :: poidx,poidy,poidz
    real(kind=8)                                        :: xx,yy,zz
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6,t7,t8

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(-4) = ip(0) - 4
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4
       ip(5) = ip(0) + 5

       jp(0) = floor((posy(i)-yb)/dy)         
       jp(-1) = jp(0) - 1        
       jp(-2) = jp(0) - 2         
       jp(-3) = jp(0) - 3
       jp(-4) = jp(0) - 4
       jp(1) = jp(0) + 1
       jp(2) = jp(0) + 2
       jp(3) = jp(0) + 3
       jp(4) = jp(0) + 4
       jp(5) = jp(0) + 5

       kp(0) = floor((posz(i)-zd)/dz)         
       kp(-1) = kp(0) - 1        
       kp(-2) = kp(0) - 2         
       kp(-3) = kp(0) - 3
       kp(-4) = kp(0) - 4
       kp(1) = kp(0) + 1
       kp(2) = kp(0) + 2
       kp(3) = kp(0) + 3
       kp(4) = kp(0) + 4
       kp(5) = kp(0) + 5




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif
       yy = (posy(i) - real(jp(0),kind=8)*dy-yb)/dy   !relatif
       zz = (posz(i) - real(kp(0),kind=8)*dz-zd)/dz   !relatif



       !conditions au bord
       !------------------
       !periodique:          
       do c=-4,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=-4,5              
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=-4,5              
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------             
       if (xx<=0.5) then
         
          t3=(xx+1.)*(xx+2.)
          t4=(xx+3.)*(xx+4.)
          t5=(xx-4.)*(xx-3.)
          t6=(xx-2.)*(xx-1.)*xx

          t1=t5*t6
          t2=t3*t4

          poidx(-4)=t1*t3*(xx+3.)/(40320.)
          poidx(-3)=-t1*t3*(xx+4.)/(5040.)
          poidx(-2)=t1*(xx+1.)*t4/(1440.)
          poidx(-1)=-t1*(xx+2.)*t4/(720.)
          poidx(0)=t5*(xx-2.)*(xx-1.)*t2/(576.)
          poidx(1)=-t5*(xx-2.)*xx*t2/(720.)
          poidx(2)=t5*(xx-1.)*xx*t2/(1440.)
          poidx(3)=-(xx-4.)*t6*t2/(5040.)
          poidx(4)=(xx-3.)*t6*t2/(40320.)
          poidx(5)=0.
       else
         
          t3=(xx-5.)*(xx-4.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+1.)
          t6=(xx+2.)*(xx+3.)

          t1=t3*t4
          t2=t5*t6

          t7=t1*(xx-1.)
          t8=(xx-1.)*t2

          poidx(-4)=0.
          poidx(-3)=t7*t5*(xx+2.)/(40320.)
          poidx(-2)=-t7*t5*(xx+3.)/(5040.)
          poidx(-1)=t7*xx*t6/(1440.)
          poidx(0)=-t7*(xx+1.)*t6/(720.)
          poidx(1)=t1*t2/(576.)
          poidx(2)=-t3*(xx-3.)*t8/(720.)
          poidx(3)=t3*(xx-2.)*t8/(1440.)
          poidx(4)=-(xx-5.)*t4*t8/(5040.)
          poidx(5)=(xx-4.)*t4*t8/(40320.)
       end if
 if (yy<=0.5) then
         
          t3=(yy+1.)*(yy+2.)
          t4=(yy+3.)*(yy+4.)
          t5=(yy-4.)*(yy-3.)
          t6=(yy-2.)*(yy-1.)*yy

          t1=t5*t6
          t2=t3*t4

          poidy(-4)=t1*t3*(yy+3.)/(40320.)
          poidy(-3)=-t1*t3*(yy+4.)/(5040.)
          poidy(-2)=t1*(yy+1.)*t4/(1440.)
          poidy(-1)=-t1*(yy+2.)*t4/(720.)
          poidy(0)=t5*(yy-2.)*(yy-1.)*t2/(576.)
          poidy(1)=-t5*(yy-2.)*yy*t2/(720.)
          poidy(2)=t5*(yy-1.)*yy*t2/(1440.)
          poidy(3)=-(yy-4.)*t6*t2/(5040.)
          poidy(4)=(yy-3.)*t6*t2/(40320.)
          poidy(5)=0.
       else
         
          t3=(yy-5.)*(yy-4.)
          t4=(yy-3.)*(yy-2.)
          t5=yy*(yy+1.)
          t6=(yy+2.)*(yy+3.)

          t1=t3*t4
          t2=t5*t6

          t7=t1*(yy-1.)
          t8=(yy-1.)*t2

          poidy(-4)=0.
          poidy(-3)=t7*t5*(yy+2.)/(40320.)
          poidy(-2)=-t7*t5*(yy+3.)/(5040.)
          poidy(-1)=t7*yy*t6/(1440.)
          poidy(0)=-t7*(yy+1.)*t6/(720.)
          poidy(1)=t1*t2/(576.)
          poidy(2)=-t3*(yy-3.)*t8/(720.)
          poidy(3)=t3*(yy-2.)*t8/(1440.)
          poidy(4)=-(yy-5.)*t4*t8/(5040.)
          poidy(5)=(yy-4.)*t4*t8/(40320.)
       end if
 if (zz<=0.5) then
         
          t3=(zz+1.)*(zz+2.)
          t4=(zz+3.)*(zz+4.)
          t5=(zz-4.)*(zz-3.)
          t6=(zz-2.)*(zz-1.)*zz

          t1=t5*t6
          t2=t3*t4

          poidz(-4)=t1*t3*(zz+3.)/(40320.)
          poidz(-3)=-t1*t3*(zz+4.)/(5040.)
          poidz(-2)=t1*(zz+1.)*t4/(1440.)
          poidz(-1)=-t1*(zz+2.)*t4/(720.)
          poidz(0)=t5*(zz-2.)*(zz-1.)*t2/(576.)
          poidz(1)=-t5*(zz-2.)*zz*t2/(720.)
          poidz(2)=t5*(zz-1.)*zz*t2/(1440.)
          poidz(3)=-(zz-4.)*t6*t2/(5040.)
          poidz(4)=(zz-3.)*t6*t2/(40320.)
          poidz(5)=0.
       else
         
          t3=(zz-5.)*(zz-4.)
          t4=(zz-3.)*(zz-2.)
          t5=zz*(zz+1.)
          t6=(zz+2.)*(zz+3.)

          t1=t3*t4
          t2=t5*t6

          t7=t1*(zz-1.)
          t8=(zz-1.)*t2

          poidz(-4)=0.
          poidz(-3)=t7*t5*(zz+2.)/(40320.)
          poidz(-2)=-t7*t5*(zz+3.)/(5040.)
          poidz(-1)=t7*zz*t6/(1440.)
          poidz(0)=-t7*(zz+1.)*t6/(720.)
          poidz(1)=t1*t2/(576.)
          poidz(2)=-t3*(zz-3.)*t8/(720.)
          poidz(3)=t3*(zz-2.)*t8/(1440.)
          poidz(4)=-(zz-5.)*t4*t8/(5040.)
          poidz(5)=(zz-4.)*t4*t8/(40320.)
       end if
      

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do e=-4,5
          do d=-4,5
             do c=-4,5              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do

  end subroutine remaill_l8


  subroutine remaill_l4_x (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-2:3)                             :: ip
    real(kind=8),dimension(-2:3)                        :: poidx
    real(kind=8)                                        :: xx,x2,x3,x4

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
      
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
      




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-2,3              
          ip(c)=mod(ip(c)+nx,nx)                
       end do



       !calcul des poids
       !---------------- 
!!$       x2=xx**2
!!$       x3=xx**3
!!$       x4=xx**4            
!!$             
!!$       if (xx<=0.5) then
!!$          poidx(-2)=(2.*xx-x2-2*x3+x4)/24.
!!$          poidx(-1)=(-4.*xx+4.*x2+x3-x4)/6.
!!$          poidx(0)=1.+(-5.*x2+x4)/4.
!!$          poidx(1)=(4.*xx+4.*x2-x3-x4)/6.
!!$          poidx(2)=(-2.*xx-x2+2*x3+x4)/24.
!!$          poidx(3)=0.
!!$       else
!!$          poidx(-2)=0.
!!$          poidx(-1)=(-6.*xx+11.*x2-6.*x3+x4)/24.
!!$          poidx(0)=1.+(-5.*xx-5.*x2+5.*x3-x4)/6.
!!$          poidx(1)=(6.*xx+x2-4.*x3+x4)/4.
!!$          poidx(2)=(-3.*xx+x2+3.*x3-x4)/6.
!!$          poidx(3)=(2.*xx-x2-2.*x3+x4)/24.
!!$       end if

       if (xx<=0.5) then                
          poidx(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
          poidx(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
          poidx(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          poidx(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
          poidx(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
          poidx(3)=0.
       else           
          poidx(-2)=0. 
          poidx(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
          poidx(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
          poidx(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
          poidx(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
          poidx(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
       end if


       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-2,3           
          remaille(ip(c)+1,jvar,kvar)=remaille(ip(c)+1,jvar,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l4_x



  subroutine remaill_l4_y (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ivar,jvar,kvar
    integer,dimension(-2:3)                             :: ip
    real(kind=8),dimension(-2:3)                        :: poidx
    real(kind=8)                                        :: xx,x2,x3,x4

    remaille=0.      


    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posy(i)-yb)/dy)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
      
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
  





       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         

       xx = (posy(i) - real(ip(0),kind=8)*dy-yb)/dy   !relatif




       !conditions au bord
       !------------------


       do c=-2,3             
          ip(c)=mod(ip(c)+ny,ny)                
       end do




       !calcul des poids
       !---------------- 
!!$       x2=xx**2
!!$       x3=xx**3
!!$       x4=xx**4            
!!$             
!!$       if (xx<=0.5) then
!!$          poidx(-2)=(2.*xx-x2-2*x3+x4)/24.
!!$          poidx(-1)=(-4.*xx+4.*x2+x3-x4)/6.
!!$          poidx(0)=1.+(-5.*x2+x4)/4.
!!$          poidx(1)=(4.*xx+4.*x2-x3-x4)/6.
!!$          poidx(2)=(-2.*xx-x2+2*x3+x4)/24.
!!$          poidx(3)=0.
!!$       else
!!$          poidx(-2)=0.
!!$          poidx(-1)=(-6.*xx+11.*x2-6.*x3+x4)/24.
!!$          poidx(0)=1.+(-5.*xx-5.*x2+5.*x3-x4)/6.
!!$          poidx(1)=(6.*xx+x2-4.*x3+x4)/4.
!!$          poidx(2)=(-3.*xx+x2+3.*x3-x4)/6.
!!$          poidx(3)=(2.*xx-x2-2.*x3+x4)/24.
!!$       end if

       if (xx<=0.5) then                
          poidx(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
          poidx(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
          poidx(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          poidx(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
          poidx(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
          poidx(3)=0.
       else           
          poidx(-2)=0. 
          poidx(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
          poidx(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
          poidx(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
          poidx(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
          poidx(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
       end if




       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-2,3             
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l4_y

  subroutine remaill_l4_z (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-2:3)                             :: ip
    real(kind=8),dimension(-2:3)                        :: poidx
    real(kind=8)                                        :: xx,x2,x3,x4

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posz(i)-zd)/dz) 
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
      
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
    





       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------              
       xx = (posz(i) - real(ip(0),kind=8)*dz-zd)/dz   !relatif



       !conditions au bord
       !------------------
       !periodique:          

       do c=-2,3           
          ip(c)=mod(ip(c)+nz,nz)                
       end do


       !calcul des poids
       !---------------- 
!!$        x2=xx**2
!!$       x3=xx**3
!!$       x4=xx**4            
!!$             
!!$       if (xx<=0.5) then
!!$          poidx(-2)=(2.*xx-x2-2*x3+x4)/24.
!!$          poidx(-1)=(-4.*xx+4.*x2+x3-x4)/6.
!!$          poidx(0)=1.+(-5.*x2+x4)/4.
!!$          poidx(1)=(4.*xx+4.*x2-x3-x4)/6.
!!$          poidx(2)=(-2.*xx-x2+2*x3+x4)/24.
!!$          poidx(3)=0.
!!$       else
!!$          poidx(-2)=0.
!!$          poidx(-1)=(-6.*xx+11.*x2-6.*x3+x4)/24.
!!$          poidx(0)=1.+(-5.*xx-5.*x2+5.*x3-x4)/6.
!!$          poidx(1)=(6.*xx+x2-4.*x3+x4)/4.
!!$          poidx(2)=(-3.*xx+x2+3.*x3-x4)/6.
!!$          poidx(3)=(2.*xx-x2-2.*x3+x4)/24.
!!$       end if

       if (xx<=0.5) then                
          poidx(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
          poidx(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
          poidx(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          poidx(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
          poidx(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
          poidx(3)=0.
       else           
          poidx(-2)=0. 
          poidx(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
          poidx(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
          poidx(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
          poidx(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
          poidx(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
       end if




       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-2,3             
          remaille(ivar,jvar,ip(c)+1)=remaille(ivar,jvar,ip(c)+1)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l4_z

  subroutine remaill_l6_x (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-3:4)                             :: ip
    real(kind=8),dimension(-3:4)                        :: poidx
    real(kind=8)                                        :: xx
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6,t7,t8

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-3,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do



       !calcul des poids
       !----------------        
      if (xx<=0.5) then
         
          t3=(xx+1.)*(xx+2.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+3.)
          t6=(xx-1.)*xx

          t1=t4*(xx-1.)
          t2=t3*(xx+3.)
        
          poidx(-3)=t1*xx*t3/720.
          poidx(-2)=-t1*t5*(xx+1.)/120.
          poidx(-1)=t1*t5*(xx+2.)/48.
          poidx(0)=-t1*t2/36.
          poidx(1)=t4*xx*t2/48.
          poidx(2)=-(xx-3.)*t6*t2/120.
          poidx(3)=(xx-2.)*t6*t2/720.
          poidx(4)=0.
       else
         
          t3=xx*(xx+1.)
          t4=(xx-3.)*(xx-2.)

          t1=(xx-4.)*t4*(xx-1.)
          t2=(xx-1.)*t3*(xx+2.)

          poidx(-3)=0.
          poidx(-2)=t1*t3/720.
          poidx(-1)=-t1*xx*(xx+2.)/120.
          poidx(0)=t1*(xx+1.)*(xx+2.)/48.
          poidx(1)=-(xx-4.)*t4*t3*(xx+2.)/36.
          poidx(2)=(xx-4.)*(xx-3.)*t2/48.
          poidx(3)=-(xx-4.)*(xx-2.)*t2/120.
          poidx(4)=t4*t2/720.
       end if





       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-3,4              
          remaille(ip(c)+1,jvar,kvar)=remaille(ip(c)+1,jvar,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l6_x



  subroutine remaill_l6_y (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ivar,jvar,kvar
    integer,dimension(-3:4)                             :: ip
    real(kind=8),dimension(-3:4)                        :: poidx
    real(kind=8)                                        :: xx
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6,t7,t8

    remaille=0.      


    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posy(i)-yb)/dy)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4






       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         

       xx = (posy(i) - real(ip(0),kind=8)*dy-yb)/dy   !relatif




       !conditions au bord
       !------------------


       do c=-3,4              
          ip(c)=mod(ip(c)+ny,ny)                
       end do




       !calcul des poids
       !----------------     
       if (xx<=0.5) then
         
          t3=(xx+1.)*(xx+2.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+3.)
          t6=(xx-1.)*xx

          t1=t4*(xx-1.)
          t2=t3*(xx+3.)
        
          poidx(-3)=t1*xx*t3/720.
          poidx(-2)=-t1*t5*(xx+1.)/120.
          poidx(-1)=t1*t5*(xx+2.)/48.
          poidx(0)=-t1*t2/36.
          poidx(1)=t4*xx*t2/48.
          poidx(2)=-(xx-3.)*t6*t2/120.
          poidx(3)=(xx-2.)*t6*t2/720.
          poidx(4)=0.
       else
         
          t3=xx*(xx+1.)
          t4=(xx-3.)*(xx-2.)

          t1=(xx-4.)*t4*(xx-1.)
          t2=(xx-1.)*t3*(xx+2.)

          poidx(-3)=0.
          poidx(-2)=t1*t3/720.
          poidx(-1)=-t1*xx*(xx+2.)/120.
          poidx(0)=t1*(xx+1.)*(xx+2.)/48.
          poidx(1)=-(xx-4.)*t4*t3*(xx+2.)/36.
          poidx(2)=(xx-4.)*(xx-3.)*t2/48.
          poidx(3)=-(xx-4.)*(xx-2.)*t2/120.
          poidx(4)=t4*t2/720.
       end if



       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-3,4              
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l6_y

  subroutine remaill_l6_z (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-3:4)                             :: ip
    real(kind=8),dimension(-3:4)                        :: poidx
    real(kind=8)                                        :: xx
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6,t7,t8

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posz(i)-zd)/dz) 
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4






       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------              
       xx = (posz(i) - real(ip(0),kind=8)*dz-zd)/dz   !relatif



       !conditions au bord
       !------------------
       !periodique:          

       do c=-3,4              
          ip(c)=mod(ip(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------      
       if (xx<=0.5) then
         
          t3=(xx+1.)*(xx+2.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+3.)
          t6=(xx-1.)*xx

          t1=t4*(xx-1.)
          t2=t3*(xx+3.)
        
          poidx(-3)=t1*xx*t3/720.
          poidx(-2)=-t1*t5*(xx+1.)/120.
          poidx(-1)=t1*t5*(xx+2.)/48.
          poidx(0)=-t1*t2/36.
          poidx(1)=t4*xx*t2/48.
          poidx(2)=-(xx-3.)*t6*t2/120.
          poidx(3)=(xx-2.)*t6*t2/720.
          poidx(4)=0.
       else
         
          t3=xx*(xx+1.)
          t4=(xx-3.)*(xx-2.)

          t1=(xx-4.)*t4*(xx-1.)
          t2=(xx-1.)*t3*(xx+2.)

          poidx(-3)=0.
          poidx(-2)=t1*t3/720.
          poidx(-1)=-t1*xx*(xx+2.)/120.
          poidx(0)=t1*(xx+1.)*(xx+2.)/48.
          poidx(1)=-(xx-4.)*t4*t3*(xx+2.)/36.
          poidx(2)=(xx-4.)*(xx-3.)*t2/48.
          poidx(3)=-(xx-4.)*(xx-2.)*t2/120.
          poidx(4)=t4*t2/720.
       end if




       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-3,4              
          remaille(ivar,jvar,ip(c)+1)=remaille(ivar,jvar,ip(c)+1)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l6_z

 subroutine remaill_l8_x (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-4:5)                             :: ip
    real(kind=8),dimension(-4:5)                        :: poidx
    real(kind=8)                                        :: xx
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6,t7,t8

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(-4) = ip(0) - 4
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4
       ip(5) = ip(0) + 5




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-4,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do



       !calcul des poids
       !---------------- 
       if (xx<=0.5) then
         
          t3=(xx+1.)*(xx+2.)
          t4=(xx+3.)*(xx+4.)
          t5=(xx-4.)*(xx-3.)
          t6=(xx-2.)*(xx-1.)*xx

          t1=t5*t6
          t2=t3*t4

          poidx(-4)=t1*t3*(xx+3.)/(40320.)
          poidx(-3)=-t1*t3*(xx+4.)/(5040.)
          poidx(-2)=t1*(xx+1.)*t4/(1440.)
          poidx(-1)=-t1*(xx+2.)*t4/(720.)
          poidx(0)=t5*(xx-2.)*(xx-1.)*t2/(576.)
          poidx(1)=-t5*(xx-2.)*xx*t2/(720.)
          poidx(2)=t5*(xx-1.)*xx*t2/(1440.)
          poidx(3)=-(xx-4.)*t6*t2/(5040.)
          poidx(4)=(xx-3.)*t6*t2/(40320.)
          poidx(5)=0.
       else
         
          t3=(xx-5.)*(xx-4.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+1.)
          t6=(xx+2.)*(xx+3.)

          t1=t3*t4
          t2=t5*t6

          t7=t1*(xx-1.)
          t8=(xx-1.)*t2

          poidx(-4)=0.
          poidx(-3)=t7*t5*(xx+2.)/(40320.)
          poidx(-2)=-t7*t5*(xx+3.)/(5040.)
          poidx(-1)=t7*xx*t6/(1440.)
          poidx(0)=-t7*(xx+1.)*t6/(720.)
          poidx(1)=t1*t2/(576.)
          poidx(2)=-t3*(xx-3.)*t8/(720.)
          poidx(3)=t3*(xx-2.)*t8/(1440.)
          poidx(4)=-(xx-5.)*t4*t8/(5040.)
          poidx(5)=(xx-4.)*t4*t8/(40320.)
       end if



       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-4,5              
          remaille(ip(c)+1,jvar,kvar)=remaille(ip(c)+1,jvar,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l8_x

subroutine remaill_l8_y (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-4:5)                             :: ip
    real(kind=8),dimension(-4:5)                        :: poidx
    real(kind=8)                                        :: xx
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6,t7,t8

    remaille=0.      


    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posy(i)-yb)/dy)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(-4) = ip(0) - 4
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4
       ip(5) = ip(0) + 5




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posy(i) - real(ip(0),kind=8)*dy-yb)/dy   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-4,5               
          ip(c)=mod(ip(c)+ny,ny)                
       end do



       !calcul des poids
       !---------------- 
      if (xx<=0.5) then
         
          t3=(xx+1.)*(xx+2.)
          t4=(xx+3.)*(xx+4.)
          t5=(xx-4.)*(xx-3.)
          t6=(xx-2.)*(xx-1.)*xx

          t1=t5*t6
          t2=t3*t4

          poidx(-4)=t1*t3*(xx+3.)/(40320.)
          poidx(-3)=-t1*t3*(xx+4.)/(5040.)
          poidx(-2)=t1*(xx+1.)*t4/(1440.)
          poidx(-1)=-t1*(xx+2.)*t4/(720.)
          poidx(0)=t5*(xx-2.)*(xx-1.)*t2/(576.)
          poidx(1)=-t5*(xx-2.)*xx*t2/(720.)
          poidx(2)=t5*(xx-1.)*xx*t2/(1440.)
          poidx(3)=-(xx-4.)*t6*t2/(5040.)
          poidx(4)=(xx-3.)*t6*t2/(40320.)
          poidx(5)=0.
       else
         
          t3=(xx-5.)*(xx-4.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+1.)
          t6=(xx+2.)*(xx+3.)

          t1=t3*t4
          t2=t5*t6

          t7=t1*(xx-1.)
          t8=(xx-1.)*t2

          poidx(-4)=0.
          poidx(-3)=t7*t5*(xx+2.)/(40320.)
          poidx(-2)=-t7*t5*(xx+3.)/(5040.)
          poidx(-1)=t7*xx*t6/(1440.)
          poidx(0)=-t7*(xx+1.)*t6/(720.)
          poidx(1)=t1*t2/(576.)
          poidx(2)=-t3*(xx-3.)*t8/(720.)
          poidx(3)=t3*(xx-2.)*t8/(1440.)
          poidx(4)=-(xx-5.)*t4*t8/(5040.)
          poidx(5)=(xx-4.)*t4*t8/(40320.)
       end if



       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-4,5              
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l8_y

subroutine remaill_l8_z (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-4:5)                             :: ip
    real(kind=8),dimension(-4:5)                        :: poidx
    real(kind=8)                                        :: xx
    real(kind=8)                                        :: t1,t2,t3,t4,t5,t6,t7,t8

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posz(i)-zd)/dz)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(-4) = ip(0) - 4
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4
       ip(5) = ip(0) + 5




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posz(i) - real(ip(0),kind=8)*dz-zd)/dz   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-4,5               
          ip(c)=mod(ip(c)+nz,nz)                
       end do



       !calcul des poids
       !---------------- 
      if (xx<=0.5) then
         
          t3=(xx+1.)*(xx+2.)
          t4=(xx+3.)*(xx+4.)
          t5=(xx-4.)*(xx-3.)
          t6=(xx-2.)*(xx-1.)*xx

          t1=t5*t6
          t2=t3*t4

          poidx(-4)=t1*t3*(xx+3.)/(40320.)
          poidx(-3)=-t1*t3*(xx+4.)/(5040.)
          poidx(-2)=t1*(xx+1.)*t4/(1440.)
          poidx(-1)=-t1*(xx+2.)*t4/(720.)
          poidx(0)=t5*(xx-2.)*(xx-1.)*t2/(576.)
          poidx(1)=-t5*(xx-2.)*xx*t2/(720.)
          poidx(2)=t5*(xx-1.)*xx*t2/(1440.)
          poidx(3)=-(xx-4.)*t6*t2/(5040.)
          poidx(4)=(xx-3.)*t6*t2/(40320.)
          poidx(5)=0.
       else
         
          t3=(xx-5.)*(xx-4.)
          t4=(xx-3.)*(xx-2.)
          t5=xx*(xx+1.)
          t6=(xx+2.)*(xx+3.)

          t1=t3*t4
          t2=t5*t6

          t7=t1*(xx-1.)
          t8=(xx-1.)*t2

          poidx(-4)=0.
          poidx(-3)=t7*t5*(xx+2.)/(40320.)
          poidx(-2)=-t7*t5*(xx+3.)/(5040.)
          poidx(-1)=t7*xx*t6/(1440.)
          poidx(0)=-t7*(xx+1.)*t6/(720.)
          poidx(1)=t1*t2/(576.)
          poidx(2)=-t3*(xx-3.)*t8/(720.)
          poidx(3)=t3*(xx-2.)*t8/(1440.)
          poidx(4)=-(xx-5.)*t4*t8/(5040.)
          poidx(5)=(xx-4.)*t4*t8/(40320.)
       end if


       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-4,5              
          remaille(ivar,jvar,ip(c)+1)=remaille(ivar,jvar,ip(c)+1)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l8_z


  subroutine remaill_l2_x (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-1:2)                             :: ip
    real(kind=8),dimension(-1:2)                        :: poidx
    real(kind=8)                                        :: xx

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
      
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
      




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-1,2              
          ip(c)=mod(ip(c)+nx,nx)                
       end do



       !calcul des poids
       !---------------- 
       if (xx<=0.5) then
          poidx(-1)=0.5*xx*(xx-1.)
          poidx(0)=1.-xx**2
          poidx(1)=0.5*xx*(1.+xx)
          poidx(2)=0.
       else
          poidx(-1)=0.
          poidx(0)=0.5*(1.-xx)*(2.-xx)
          poidx(1)=2.*xx-xx**2
          poidx(2)=0.5*(xx-1.)*xx
       end if





       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-1,2            
          remaille(ip(c)+1,jvar,kvar)=remaille(ip(c)+1,jvar,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l2_x



  subroutine remaill_l2_y (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ivar,jvar,kvar
    integer,dimension(-1:2)                             :: ip
    real(kind=8),dimension(-1:2)                        :: poidx
    real(kind=8)                                        :: xx

    remaille=0.      


    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posy(i)-yb)/dy)         
       ip(-1) = ip(0) - 1        
     
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
     





       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         

       xx = (posy(i) - real(ip(0),kind=8)*dy-yb)/dy   !relatif




       !conditions au bord
       !------------------


       do c=-1,2            
          ip(c)=mod(ip(c)+ny,ny)                
       end do




       !calcul des poids
       !----------------       
       if (xx<=0.5) then
          poidx(-1)=0.5*xx*(xx-1.)
          poidx(0)=1.-xx**2
          poidx(1)=0.5*xx*(1.+xx)
          poidx(2)=0.
       else
          poidx(-1)=0.
          poidx(0)=0.5*(1.-xx)*(2.-xx)
          poidx(1)=2.*xx-xx**2
          poidx(2)=0.5*(xx-1.)*xx
       end if





       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-1,2             
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l2_y

  subroutine remaill_l2_z (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-1:2)                             :: ip
    real(kind=8),dimension(-1:2)                        :: poidx
    real(kind=8)                                        :: xx

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posz(i)-zd)/dz) 
       ip(-1) = ip(0) - 1        
      
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
     





       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------              
       xx = (posz(i) - real(ip(0),kind=8)*dz-zd)/dz   !relatif



       !conditions au bord
       !------------------
       !periodique:          

       do c=-1,2     
          ip(c)=mod(ip(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------        
       if (xx<=0.5) then
          poidx(-1)=0.5*xx*(xx-1.)
          poidx(0)=1.-xx**2
          poidx(1)=0.5*xx*(1.+xx)
          poidx(2)=0.
       else
          poidx(-1)=0.
          poidx(0)=0.5*(1.-xx)*(2.-xx)
          poidx(1)=2.*xx-xx**2
          poidx(2)=0.5*(xx-1.)*xx
       end if





       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-1,2           
          remaille(ivar,jvar,ip(c)+1)=remaille(ivar,jvar,ip(c)+1)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l2_z




 subroutine remaill_l2_bloc_x (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,j,k,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1
          k=nint((posz(i)-zd)/dz)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx


          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)
                poids(0)=1.-xx**2
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx)
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
          case(2)
             poids(0)=1.-0.5*xx*(1.+xx)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
          case(4)
             poids(-2)=0.5*xx*(1.+xx)
             poids(-1)=-xx
             poids(0)=1.-xx**2
          case(5)
             poids(-1)=-0.5*xx+0.5*xx**2
             poids(0)=1.-poids(-1)

          end select


          select case (blocd(i))

          case(0)
             if (xx<=0.5) then
                poids(1)=0.5*xx*(1.+xx)               
             else
                poids(1)=2.*xx-xx**2
                poids(2)=0.5*(xx-1.)*xx
             end if
          case(1)
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1)
          case(3)
             poids(1)=3.*0.5*xx-0.5*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ip(c)+1,j,k)=remaille(ip(c)+1,j,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_x


     subroutine remaill_l2_bloc_y (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,k
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          k=nint((posz(i)-zd)/dz)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)
                poids(0)=1.-xx**2
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx)
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
          case(2)
             poids(0)=1.-0.5*xx*(1.+xx)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
          case(4)
             poids(-2)=0.5*xx*(1.+xx)
             poids(-1)=-xx
             poids(0)=1.-xx**2
          case(5)
             poids(-1)=-0.5*xx+0.5*xx**2
             poids(0)=1.-poids(-1)

          end select

          select case (blocd(i))

          case(0)
             if (xx<=0.5) then
                poids(1)=0.5*xx*(1.+xx)               
             else
                poids(1)=2.*xx-xx**2
                poids(2)=0.5*(xx-1.)*xx
             end if
          case(1)
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1)
          case(3)
             poids(1)=3.*0.5*xx-0.5*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,ip(c)+1,k)=remaille(ivar,ip(c)+1,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_y

  subroutine remaill_l2_bloc_z (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,j
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          j=nint((posy(i)-yb)/dy)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posz(i)-zd)/dz)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posz(i) - real(ip(0) ,kind=8)*dz-zd)/dz       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nz,nz)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)
                poids(0)=1.-xx**2
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx)
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
          case(2)
             poids(0)=1.-0.5*xx*(1.+xx)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
          case(4)
             poids(-2)=0.5*xx*(1.+xx)
             poids(-1)=-xx
             poids(0)=1.-xx**2
          case(5)
             poids(-1)=-0.5*xx+0.5*xx**2
             poids(0)=1.-poids(-1)

          end select

          select case (blocd(i))

          case(0)
             if (xx<=0.5) then
                poids(1)=0.5*xx*(1.+xx)               
             else
                poids(1)=2.*xx-xx**2
                poids(2)=0.5*(xx-1.)*xx
             end if
          case(1)
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1)
          case(3)
             poids(1)=3.*0.5*xx-0.5*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_z



     subroutine remaill_l2_bloc_x_v2 (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,j,k,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1
          k=nint((posz(i)-zd)/dz)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx


          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

         case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)     !alpha
                poids(0)=1.-xx**2            !beta
                poids(1)=0.5*xx*(1.+xx)      !gamma
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx) !alpha'
                poids(1)=2.*xx-xx**2         !beta'
                poids(2)=0.5*(xx-1.)*xx      !gamma'
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(0)=0.5*(1.-xx)*(2.-xx)
             poids(1)=1.-poids(0)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-poids(-1)
          case(4)
             poids(0)=-0.5*xx*(xx+1.)+1.
             poids(1)=1.-poids(0)
          case(5)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2 
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1.)
          case(6)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
             poids(1)=2.*xx-xx**2 
             poids(2)=0.5*(xx-1.)*xx
          case(7)
             poids(-2)=0.5*xx*(xx+1)
             poids(-1)=-xx
             poids(0)=1.-xx**2 
             poids(1)=0.5*xx*(1.+xx)  
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ip(c)+1,j,k)=remaille(ip(c)+1,j,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_x_v2


     subroutine remaill_l2_bloc_y_v2 (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,k
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          k=nint((posz(i)-zd)/dz)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poids
          !----------------
 select case (blocg(i))

         case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)     !alpha
                poids(0)=1.-xx**2            !beta
                poids(1)=0.5*xx*(1.+xx)      !gamma
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx) !alpha'
                poids(1)=2.*xx-xx**2         !beta'
                poids(2)=0.5*(xx-1.)*xx      !gamma'
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(0)=0.5*(1.-xx)*(2.-xx)
             poids(1)=1.-poids(0)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-poids(-1)
          case(4)
             poids(0)=-0.5*xx*(xx+1.)+1.
             poids(1)=1.-poids(0)
          case(5)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2 
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1.)
          case(6)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
             poids(1)=2.*xx-xx**2 
             poids(2)=0.5*(xx-1.)*xx
          case(7)
             poids(-2)=0.5*xx*(xx+1)
             poids(-1)=-xx
             poids(0)=1.-xx**2 
             poids(1)=0.5*xx*(1.+xx)  
          end select
        

          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,ip(c)+1,k)=remaille(ivar,ip(c)+1,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_y_v2

  subroutine remaill_l2_bloc_z_v2 (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,j
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          j=nint((posy(i)-yb)/dy)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posz(i)-zd)/dz)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posz(i) - real(ip(0) ,kind=8)*dz-zd)/dz       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nz,nz)     
          end do

          !calcul des poids
          !----------------

           select case (blocg(i))

         case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)     !alpha
                poids(0)=1.-xx**2            !beta
                poids(1)=0.5*xx*(1.+xx)      !gamma
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx) !alpha'
                poids(1)=2.*xx-xx**2         !beta'
                poids(2)=0.5*(xx-1.)*xx      !gamma'
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(0)=0.5*(1.-xx)*(2.-xx)
             poids(1)=1.-poids(0)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-poids(-1)
          case(4)
             poids(0)=-0.5*xx*(xx+1.)+1.
             poids(1)=1.-poids(0)
          case(5)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2 
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1.)
          case(6)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
             poids(1)=2.*xx-xx**2 
             poids(2)=0.5*(xx-1.)*xx
          case(7)
             poids(-2)=0.5*xx*(xx+1)
             poids(-1)=-xx
             poids(0)=1.-xx**2 
             poids(1)=0.5*xx*(1.+xx)  
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_z_v2















    subroutine remaill_l4_bloc_x (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,j,per,k
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   


       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1 
          k=nint((posz(i)-zd)/dz)+1 
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx 


          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx-1.) 
                t2=xx**2-4.
                poids(-2)=t1/24.*(xx-2.)*(xx+1.)
                poids(-1)=-t1/6.*t2
                poids(0)=t2/4.*(xx**2-1.)
             else  
                t1=(xx-1.)*(xx-2.)*(xx-3.)
                poids(-1)=t1*xx/24.
                poids(0)=-t1*(xx+1.)/6.
             end if
          case(1)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=t2/4.*(xx**2-1.)

          case(2)
             t1=(xx-1.)*(xx-2.)/12.
             poids(-1)=t1*xx*(xx-3)/2.
             poids(0)=t1*(xx+6.)*(xx+1.)
          case(3)
             t1=0.5*(xx-1.)*(xx-2.)
             poids(-2)=t1*xx/12.*(xx+1.)
             poids(-1)=-t1*xx/3.*(xx+2.)
             poids(0)=t1*(xx+1.)
          case(4)
             t2=(xx**2-1.)
             t1=xx*(xx+2.)*0.5
             poids(-3)=t2*t1/12.
             poids(-2)=-t2*xx/6.*(xx+3.)
             poids(-1)=t1*(xx-1.)
             poids(0)=t2*(xx**2-4.)/4.
          case(5)
             t1=(xx-2.)*(xx-1.)/4.
             poids(-1)=-t1*xx/6.*(3*xx+7.)
             poids(0)=t1*(xx+2.)*(xx+1.)
          case(6)
             t1= (xx-1.)*(xx-2.)/6.
             t2=t1*(xx+1.)
             poids(-2)=t2*xx/4.
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.)
          case(7)
             t1=(xx-1.)*(xx+2.)
             t2=xx*(xx+1)/6.
             poids(-3)=t1*t2/4.
             poids(-2)=-t2*(xx-1.)
             poids(-1)=-xx*t1/6.*(xx-2.)
             poids(0)=t1*(xx-2.)/4.*(xx+1.)
          case(8)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=(xx-6.)*(xx+2.)*(xx**2-1.)/12.
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx+1.)*(xx+2.)
                poids(1)=-t1*(xx-2.)/6.
                poids(2)=t1*(xx-1.)/24.
             else  
                t1=xx*(xx+1.)*(xx-3.)
                poids(1)=t1*(xx-2.)/4.
                poids(2)=-t1*(xx-1.)/6.
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             end if
          case(1)
             t1=xx*(xx+1.)*(xx+2.)
             poids(1)=-t1*(xx-2.)/6.
             poids(2)=t1*(xx-1.)/24.           
          case(2)
             t1=xx*(xx+1.)*0.5
             t2=t1*(xx-1.)
             poids(1)=-t1*(xx-2.)
             poids(2)=-t2/3.*(xx-3.)
             poids(3)=t2*(xx-2.)/12.
          case(3)
             t1=xx*(xx+1.)/12.
             poids(1)=t1*(xx-2.)*(xx-7.)
             poids(2)=t1*(xx+2.)*(xx-1.)/2.
          case(4)
             t1=xx*(xx+1.)
             t2=t1*(xx-1.)/6.
             poids(1)=-t1*(xx+2.)*(xx-2.)/6.
             poids(2)=t2
             poids(3)=t2*(xx-2.)/4.
          case(5)
             poids(1)=-xx/24.*(3.*xx-7.)*(xx+2.)*(xx+1.)
          case(6)
             t1=xx*(xx+1.)/4.
             poids(1)=t1*(xx-3.)*(xx-2.)
             poids(2)=-t1*(3.*xx-10.)*(xx-1.)/6.
          case(7)
             poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ip(c)+1,j,k)=remaille(ip(c)+1,j,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_x


     subroutine remaill_l4_bloc_y (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,k
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1
          k=nint((posz(i)-zd)/dz)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy

          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poidss
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx-1.) 
                t2=xx**2-4.
                poids(-2)=t1/24.*(xx-2.)*(xx+1.)
                poids(-1)=-t1/6.*t2
                poids(0)=t2/4.*(xx**2-1.)
             else  
                t1=(xx-1.)*(xx-2.)*(xx-3.)
                poids(-1)=t1*xx/24.
                poids(0)=-t1*(xx+1.)/6.
             end if
          case(1)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=t2/4.*(xx**2-1.)

          case(2)
             t1=(xx-1.)*(xx-2.)/12.
             poids(-1)=t1*xx*(xx-3)/2.
             poids(0)=t1*(xx+6.)*(xx+1.)
          case(3)
             t1=0.5*(xx-1.)*(xx-2.)
             poids(-2)=t1*xx/12.*(xx+1.)
             poids(-1)=-t1*xx/3.*(xx+2.)
             poids(0)=t1*(xx+1.)
          case(4)
             t2=(xx**2-1.)
             t1=xx*(xx+2.)*0.5
             poids(-3)=t2*t1/12.
             poids(-2)=-t2*xx/6.*(xx+3.)
             poids(-1)=t1*(xx-1.)
             poids(0)=t2*(xx**2-4.)/4.
          case(5)
             t1=(xx-2.)*(xx-1.)/4.
             poids(-1)=-t1*xx/6.*(3*xx+7.)
             poids(0)=t1*(xx+2.)*(xx+1.)
          case(6)
             t1= (xx-1.)*(xx-2.)/6.
             t2=t1*(xx+1.)
             poids(-2)=t2*xx/4.
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.)
          case(7)
             t1=(xx-1.)*(xx+2.)
             t2=xx*(xx+1)/6.
             poids(-3)=t1*t2/4.
             poids(-2)=-t2*(xx-1.)
             poids(-1)=-xx*t1/6.*(xx-2.)
             poids(0)=t1*(xx-2.)/4.*(xx+1.)
          case(8)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=(xx-6.)*(xx+2.)*(xx**2-1.)/12.
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx+1.)*(xx+2.)
                poids(1)=-t1*(xx-2.)/6.
                poids(2)=t1*(xx-1.)/24.
             else  
                t1=xx*(xx+1.)*(xx-3.)
                poids(1)=t1*(xx-2.)/4.
                poids(2)=-t1*(xx-1.)/6.
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             end if
          case(1)
             t1=xx*(xx+1.)*(xx+2.)
             poids(1)=-t1*(xx-2.)/6.
             poids(2)=t1*(xx-1.)/24.           
          case(2)
             t1=xx*(xx+1.)*0.5
             t2=t1*(xx-1.)
             poids(1)=-t1*(xx-2.)
             poids(2)=-t2/3.*(xx-3.)
             poids(3)=t2*(xx-2.)/12.
          case(3)
             t1=xx*(xx+1.)/12.
             poids(1)=t1*(xx-2.)*(xx-7.)
             poids(2)=t1*(xx+2.)*(xx-1.)/2.
          case(4)
             t1=xx*(xx+1.)
             t2=t1*(xx-1.)/6.
             poids(1)=-t1*(xx+2.)*(xx-2.)/6.
             poids(2)=t2
             poids(3)=t2*(xx-2.)/4.
          case(5)
             poids(1)=-xx/24.*(3.*xx-7.)*(xx+2.)*(xx+1.)
          case(6)
             t1=xx*(xx+1.)/4.
             poids(1)=t1*(xx-3.)*(xx-2.)
             poids(2)=-t1*(3.*xx-10.)*(xx-1.)/6.
          case(7)
             poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ivar,ip(c)+1,k)=remaille(ivar,ip(c)+1,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_y


     subroutine remaill_l4_bloc_z (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,j
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1
          j=nint((posy(i)-yb)/dy)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posz(i)-zd)/dz)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posz(i) - real(ip(0) ,kind=8)*dz-zd)/dz

          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+nz,nz)     
          end do

          !calcul des poidss
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx-1.) 
                t2=xx**2-4.
                poids(-2)=t1/24.*(xx-2.)*(xx+1.)
                poids(-1)=-t1/6.*t2
                poids(0)=t2/4.*(xx**2-1.)
             else  
                t1=(xx-1.)*(xx-2.)*(xx-3.)
                poids(-1)=t1*xx/24.
                poids(0)=-t1*(xx+1.)/6.
             end if
          case(1)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=t2/4.*(xx**2-1.)

          case(2)
             t1=(xx-1.)*(xx-2.)/12.
             poids(-1)=t1*xx*(xx-3)/2.
             poids(0)=t1*(xx+6.)*(xx+1.)
          case(3)
             t1=0.5*(xx-1.)*(xx-2.)
             poids(-2)=t1*xx/12.*(xx+1.)
             poids(-1)=-t1*xx/3.*(xx+2.)
             poids(0)=t1*(xx+1.)
          case(4)
             t2=(xx**2-1.)
             t1=xx*(xx+2.)*0.5
             poids(-3)=t2*t1/12.
             poids(-2)=-t2*xx/6.*(xx+3.)
             poids(-1)=t1*(xx-1.)
             poids(0)=t2*(xx**2-4.)/4.
          case(5)
             t1=(xx-2.)*(xx-1.)/4.
             poids(-1)=-t1*xx/6.*(3*xx+7.)
             poids(0)=t1*(xx+2.)*(xx+1.)
          case(6)
             t1= (xx-1.)*(xx-2.)/6.
             t2=t1*(xx+1.)
             poids(-2)=t2*xx/4.
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.)
          case(7)
             t1=(xx-1.)*(xx+2.)
             t2=xx*(xx+1)/6.
             poids(-3)=t1*t2/4.
             poids(-2)=-t2*(xx-1.)
             poids(-1)=-xx*t1/6.*(xx-2.)
             poids(0)=t1*(xx-2.)/4.*(xx+1.)
          case(8)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=(xx-6.)*(xx+2.)*(xx**2-1.)/12.
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx+1.)*(xx+2.)
                poids(1)=-t1*(xx-2.)/6.
                poids(2)=t1*(xx-1.)/24.
             else  
                t1=xx*(xx+1.)*(xx-3.)
                poids(1)=t1*(xx-2.)/4.
                poids(2)=-t1*(xx-1.)/6.
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             end if
          case(1)
             t1=xx*(xx+1.)*(xx+2.)
             poids(1)=-t1*(xx-2.)/6.
             poids(2)=t1*(xx-1.)/24.           
          case(2)
             t1=xx*(xx+1.)*0.5
             t2=t1*(xx-1.)
             poids(1)=-t1*(xx-2.)
             poids(2)=-t2/3.*(xx-3.)
             poids(3)=t2*(xx-2.)/12.
          case(3)
             t1=xx*(xx+1.)/12.
             poids(1)=t1*(xx-2.)*(xx-7.)
             poids(2)=t1*(xx+2.)*(xx-1.)/2.
          case(4)
             t1=xx*(xx+1.)
             t2=t1*(xx-1.)/6.
             poids(1)=-t1*(xx+2.)*(xx-2.)/6.
             poids(2)=t2
             poids(3)=t2*(xx-2.)/4.
          case(5)
             poids(1)=-xx/24.*(3.*xx-7.)*(xx+2.)*(xx+1.)
          case(6)
             t1=xx*(xx+1.)/4.
             poids(1)=t1*(xx-3.)*(xx-2.)
             poids(2)=-t1*(3.*xx-10.)*(xx-1.)/6.
          case(7)
             poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_z


 subroutine remaill_l4_bloc_x_v2 (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,j,per,k
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   


       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1 
          k=nint((posz(i)-zd)/dz)+1 
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx 


          !conditions au bord
          !------------------       
          do c=-3,4             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------
 select case (blocg(i))
          case(0)
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
                poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
                poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
                poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
                poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
             end if
          case(1)
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          case(2)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx**4/12.-2.*xx**3/3.+5.*xx**2/12.+7.*xx/6.
          case(3)             
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6. 
             poids(0)=1.+xx**4/12.-13.*xx**2/12.-xx**3/3.+xx/3.   
          case(4)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=xx*(xx-1.)*(xx-2.)*(xx-3.)/24.
             poids(0)=xx**4/12.+xx**3/3.-13.*xx**2/12.-xx/3.+1.
          case(5)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=0.
             poids(0)=1.-xx**4/8.+7.*xx**3/12.-3.*xx**2/8.-13.*xx/12.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.
          case(6)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=-xx**4/8.+xx**3/12.+5.*xx**2/8.-7.*xx/12.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.
          case(7)
             poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=0.5*xx**3-xx**2-0.5*xx+1.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.  
          case(8)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=-xx*(xx+3.)*(xx**2-1.)/6.
             poids(-1)=0.5*xx**3+0.5*xx**2-xx
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.  
          case(9)
            poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx**3/6.+0.5*xx**2-xx/3.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4. 
          case(10)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=xx*(1.-xx**2)/6.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.   
          end select

          select case (blocd(i))        
          case(0)
             if (xx<=0.5) then                
                poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
                poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
                poids(3)=0.
                poids(4)=0.
             else                 
                poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
                poids(4)=0.
             end if
          case(1)
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
             poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
             poids(3)=0.
             poids(4)=0.
          case(2)
             poids(2)=-xx**4/8.+5.*xx**3/12.+xx**2/8.-5.*xx/12.
             poids(3)=0.
             poids(4)=0.
          case(3)
              poids(1)=-xx**4/8.-xx**3/12.+5.*xx**2/8.+7.*xx/12.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(4)
             poids(2)=xx*(xx+2.)*(xx**2-1.)/24.
             poids(3)=0.
             poids(4)=0.
          case(5)
            poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(6)            
             poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.
             poids(3)=xx**3/6.-0.5*xx*2+xx/3.
             poids(4)=xx*(xx-1)*(xx-2.)*(xx-3.)/24.
          case(7)            
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.
             poids(2)=xx*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          case(8)
               poids(1)=-0.5*xx**3+0.5*xx**2+xx
             poids(2)=-xx*(xx-3)*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          end select
 
         


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ip(c)+1,j,k)=remaille(ip(c)+1,j,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_x_v2


     subroutine remaill_l4_bloc_y_v2 (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,k
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1
          k=nint((posz(i)-zd)/dz)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy

          !conditions au bord
          !------------------       
          do c=-3,4            
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poidss
          !----------------
 select case (blocg(i))
          case(0)
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
                poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
                poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
                poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
                poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
             end if
          case(1)
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          case(2)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx**4/12.-2.*xx**3/3.+5.*xx**2/12.+7.*xx/6.
          case(3)             
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6. 
             poids(0)=1.+xx**4/12.-13.*xx**2/12.-xx**3/3.+xx/3.   
          case(4)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=xx*(xx-1.)*(xx-2.)*(xx-3.)/24.
             poids(0)=xx**4/12.+xx**3/3.-13.*xx**2/12.-xx/3.+1.
          case(5)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=0.
             poids(0)=1.-xx**4/8.+7.*xx**3/12.-3.*xx**2/8.-13.*xx/12.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.
          case(6)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=-xx**4/8.+xx**3/12.+5.*xx**2/8.-7.*xx/12.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.
          case(7)
             poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=0.5*xx**3-xx**2-0.5*xx+1.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.  
          case(8)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=-xx*(xx+3.)*(xx**2-1.)/6.
             poids(-1)=0.5*xx**3+0.5*xx**2-xx
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.  
          case(9)
            poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx**3/6.+0.5*xx**2-xx/3.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4. 
          case(10)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=xx*(1.-xx**2)/6.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.   
          end select

          select case (blocd(i))        
          case(0)
             if (xx<=0.5) then                
                poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
                poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
                poids(3)=0.
                poids(4)=0.
             else                 
                poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
                poids(4)=0.
             end if
          case(1)
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
             poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
             poids(3)=0.
             poids(4)=0.
          case(2)
             poids(2)=-xx**4/8.+5.*xx**3/12.+xx**2/8.-5.*xx/12.
             poids(3)=0.
             poids(4)=0.
          case(3)
              poids(1)=-xx**4/8.-xx**3/12.+5.*xx**2/8.+7.*xx/12.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(4)
             poids(2)=xx*(xx+2.)*(xx**2-1.)/24.
             poids(3)=0.
             poids(4)=0.
          case(5)
            poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(6)            
             poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.
             poids(3)=xx**3/6.-0.5*xx*2+xx/3.
             poids(4)=xx*(xx-1)*(xx-2.)*(xx-3.)/24.
          case(7)            
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.
             poids(2)=xx*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          case(8)
               poids(1)=-0.5*xx**3+0.5*xx**2+xx
             poids(2)=-xx*(xx-3)*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ivar,ip(c)+1,k)=remaille(ivar,ip(c)+1,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_y_v2


     subroutine remaill_l4_bloc_z_v2 (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,j
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1
          j=nint((posy(i)-yb)/dy)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posz(i)-zd)/dz)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posz(i) - real(ip(0) ,kind=8)*dz-zd)/dz

          !conditions au bord
          !------------------       
          do c=-3,4             
             ip(c)=mod(ip(c)+nz,nz)     
          end do

          !calcul des poidss
          !----------------
 select case (blocg(i))
          case(0)
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
                poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
                poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
                poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
                poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
             end if
          case(1)
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          case(2)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx**4/12.-2.*xx**3/3.+5.*xx**2/12.+7.*xx/6.
          case(3)             
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6. 
             poids(0)=1.+xx**4/12.-13.*xx**2/12.-xx**3/3.+xx/3.   
          case(4)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=xx*(xx-1.)*(xx-2.)*(xx-3.)/24.
             poids(0)=xx**4/12.+xx**3/3.-13.*xx**2/12.-xx/3.+1.
          case(5)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=0.
             poids(0)=1.-xx**4/8.+7.*xx**3/12.-3.*xx**2/8.-13.*xx/12.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.
          case(6)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=-xx**4/8.+xx**3/12.+5.*xx**2/8.-7.*xx/12.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.
          case(7)
             poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=0.5*xx**3-xx**2-0.5*xx+1.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.  
          case(8)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=-xx*(xx+3.)*(xx**2-1.)/6.
             poids(-1)=0.5*xx**3+0.5*xx**2-xx
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.  
          case(9)
            poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx**3/6.+0.5*xx**2-xx/3.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4. 
          case(10)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=xx*(1.-xx**2)/6.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.   
          end select

          select case (blocd(i))        
          case(0)
             if (xx<=0.5) then                
                poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
                poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
                poids(3)=0.
                poids(4)=0.
             else                 
                poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
                poids(4)=0.
             end if
          case(1)
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
             poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
             poids(3)=0.
             poids(4)=0.
          case(2)
             poids(2)=-xx**4/8.+5.*xx**3/12.+xx**2/8.-5.*xx/12.
             poids(3)=0.
             poids(4)=0.
          case(3)
              poids(1)=-xx**4/8.-xx**3/12.+5.*xx**2/8.+7.*xx/12.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(4)
             poids(2)=xx*(xx+2.)*(xx**2-1.)/24.
             poids(3)=0.
             poids(4)=0.
          case(5)
            poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(6)            
             poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.
             poids(3)=xx**3/6.-0.5*xx*2+xx/3.
             poids(4)=xx*(xx-1)*(xx-2.)*(xx-3.)/24.
          case(7)            
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.
             poids(2)=xx*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          case(8)
               poids(1)=-0.5*xx**3+0.5*xx**2+xx
             poids(2)=-xx*(xx-3)*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          end select

          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_z_v2



 subroutine limit_l2_bloc_x (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar,j,per
    integer,dimension(-2:2)                             :: ip
    real(kind=8),dimension(-2:2)                        :: poidx
    real(kind=8)                                        :: xx,tmp_pos  
    real(kind=8),dimension(0:2)                         :: phi_mdemi,phi_pdemi
    real(kind=8),dimension(0:1)                         :: phi
        

    remaille=0.      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       poidx=0.

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2    
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
      




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-2,2              
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       !calcul du limiteur pour cas de bloc centre avec xx>0.5
       !--------------------------------------------------------
       call calcul_phi_tvd(phi,1)         
       phi_mdemi(1)=phi(0)
       phi_pdemi(1)=phi(1)           
       
       !les autres cas ...
       !---------------------
       call calcul_phi_tvd(phi,0)         
       phi_mdemi(0)=phi(0)
       phi_pdemi(0)=phi(1) 
       
       phi_mdemi(2)=0.
       phi_pdemi(2)=0.     

      select case (blocg(i))

          case(0)
             if (xx<=0.5) then

                poidx(-1)=ca(xx,0)
                if (test (i,1) ) then
                   poidx(0)=cb(xx,0,1)
                   poidx(1)=cc(xx,1)
                else
                   poidx(0)=cb(xx,0,0)
                   poidx(1)=cc(xx,0)
                end if

             else     
               
                poidx(0)=ca(xx-1.,1)
                if (test (i,1) ) then
                   poidx(1)=cb(xx-1.,1,1)
                   poidx(2)=cc(xx-1.,1)
                else                  
                   poidx(1)=cb(xx-1.,1,0)
                   poidx(2)=cc(xx-1.,0)
                end if                        
          
             end if

          case(1)
               
             poidx(-1)=ca(xx,0)
             if (test (i,1) ) then
                poidx(0)=cb(xx,0,1)
                poidx(1)=cc(xx,1)
             else
                poidx(0)=cb(xx,0,0)
                poidx(1)=cc(xx,0)
             end if
               

          case(2)
                poidx(0)=ca(xx-1.,1)
                poidx(1)=cb(xx-1.,1,0)+cc(xx-1.,0)
          case(3)
                poidx(-1)=ca(xx,0)
                poidx(0)=cb(xx,0)+cc(xx,0)
          case(4)
                poidx(0)=ca(xx,1)+cb(xx,1,0)
                poidx(1)=cc(xx,0)


                     
          case(5)
             poidx(-1)=ca(xx,0)
             poidx(0)=cb(xx,0)             
             poidx(1)=cc(xx,0)-cc(xx-1.,1)
             poidx(2)=cc(xx-1.,1)
            
          case(6)
             poidx(-1)=ca(xx,0)
             poidx(0)=ca(xx-1.,1)-ca(xx,0)
             poidx(1)=cb(xx-1.,1)
             poidx(2)=cc(xx-1.,1)


          case(7)
             poidx(-2)=ca(xx+1.,0)
             poidx(-1)=ca(xx,0)-ca(xx+1.,0)
             poidx(0)=cb(xx,0)
             poidx(1)=cc(xx,0)
          end select

       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-2,2            
          remaille(ip(c)+1,jvar,kvar)=remaille(ip(c)+1,jvar,kvar)+donne(i)*poidx(c)
       end do

    end do

  contains
   
       function coeff_l2g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)
        case(0)
           c=1.-x**2
        case(1)
           c=0.5*x*(1.+x)
        end select
        
      end function coeff_l2g

      function coeff_tscg (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)  +1./8. ! +1./6.
        case(0)
           c=-x**2+1.    -2./8. !-2./6. 
        case(1)
           c=0.5*x*(1.+x)   +1./8. ! +1./6.
        end select
        
      end function coeff_tscg

      function ca(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: ca
        
        ca=coeff_tscg(-1,x)+phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))
        
      end function ca

      function cb(x,num,num2)
        integer      :: num
        real(kind=8) :: x
        integer,optional :: num2
        real(kind=8) :: cb
       
        if (present (num2)) then
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num2)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        else
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        end if

      end function cb

      function cc(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: cc
      
        cc=coeff_tscg(1,x)+phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))
        

      end function cc
            
      function test (ipart,vois)
        implicit none
        integer       :: ipart,vois
        logical       :: test
        real(kind=8)  :: x
        
        test=.false.       
        x=(posx(mod(ipart-1+vois+npart,npart)+1) - real( floor((posx(mod(ipart-1+vois+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx
        if ( ((blocg(mod(ipart-1+vois+npart,npart)+1)==0).or.(blocg(mod(ipart-1+vois+npart,npart)+1)==2)).and.(x>0.5)) test=.true.
        
      end function test



 subroutine calcul_phi_tvd(phi_out,num) 
        implicit none
        integer                                :: num
        real(kind=8),dimension(0:1),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u
        real(kind=8)                           :: rp,rm,diff1,diff2,diff3,diff0,rmm,y,r 
        integer                                :: ib,jb
        real(kind=8),dimension(-5:5)           :: xx_vois,x,tp1,tp2,tp3

        !particules voisines
        !-------------------       
        do ib=-2,2
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        do ib=-1,2
           x(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
        end do


        !cas bloc centre xx>0.5 (psi)
        !-----------------------
        if (num==1) then
          
           do ib=0,1                          
              x(ib)=x(ib)-1.              
              tp1(ib)=6.-8.*x(ib)*x(ib)             
              tp3(ib)=4.*(x(ib)-0.5)**2          
           end do
           
           do ib=0,1
              r=(u(ib+1)-u(ib))/(u(ib)-u(ib-1))               

              !anti diffusif
              !phi_out(ib)=max(0.,min(tp3(ib),tp1(ib)*r))                            

              !du type Minmod (attention non justifié pour y>=0.8)
              phi_out(ib)=max(0.,min(1.,4.*r))

              !bornes dépendent de la condition cfl 
              !phi_out(ib)=max(0.,min(1.,tp3(ib),tp1(ib)*r))              
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.
           end do
 



        else  
           !sinon (phi)
           !-----------------------       


        do ib=-1,0
           tp1(ib)=6.-8.*x(ib)*x(ib)
           tp2(ib)=4.*(x(ib)+0.5)**2                
        end do


        do ib=0,1
              r=(u(ib-1)-u(ib-2))/(u(ib)-u(ib-1))                                                 
              !phi_out(ib)=max(0.,min(tp2(ib-1),tp1(ib-1)*r))
              phi_out(ib)=max(0.,min(1.,4.*r))
              !phi_out(ib)=max(0.,min(1.,tp2(ib-1),tp1(ib-1)*r))
             
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.          
        end do

     end if
    

      end subroutine calcul_phi_tvd
                          

  end subroutine limit_l2_bloc_x




 subroutine limit_l2_bloc_y (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar,j,per
    integer,dimension(-2:2)                             :: ip
    real(kind=8),dimension(-2:2)                        :: poidx
    real(kind=8)                                        :: xx,tmp_pos  
    real(kind=8),dimension(0:2)                         :: phi_mdemi,phi_pdemi
    real(kind=8),dimension(0:1)                         :: phi
        

    remaille=0.      


    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       poidx=0.

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posy(i)-yb)/dy)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2    
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
      




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posy(i) - real(ip(0),kind=8)*dy-yb)/dy   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-2,2              
          ip(c)=mod(ip(c)+ny,ny)                
       end do

       !calcul du limiteur pour cas de bloc centre avec xx>0.5
       !--------------------------------------------------------
       call calcul_phi_tvd(phi,1)         
       phi_mdemi(1)=phi(0)
       phi_pdemi(1)=phi(1)           
       
       !les autres cas ...
       !---------------------
       call calcul_phi_tvd(phi,0)         
       phi_mdemi(0)=phi(0)
       phi_pdemi(0)=phi(1) 
       
       phi_mdemi(2)=0.
       phi_pdemi(2)=0.     

      select case (blocg(i))

          case(0)
             if (xx<=0.5) then

                poidx(-1)=ca(xx,0)
                if (test (i,1) ) then
                   poidx(0)=cb(xx,0,1)
                   poidx(1)=cc(xx,1)
                else
                   poidx(0)=cb(xx,0,0)
                   poidx(1)=cc(xx,0)
                end if

             else     
               
                poidx(0)=ca(xx-1.,1)
                if (test (i,1) ) then
                   poidx(1)=cb(xx-1.,1,1)
                   poidx(2)=cc(xx-1.,1)
                else                  
                   poidx(1)=cb(xx-1.,1,0)
                   poidx(2)=cc(xx-1.,0)
                end if                        
          
             end if

          case(1)
               
             poidx(-1)=ca(xx,0)
             if (test (i,1) ) then
                poidx(0)=cb(xx,0,1)
                poidx(1)=cc(xx,1)
             else
                poidx(0)=cb(xx,0,0)
                poidx(1)=cc(xx,0)
             end if
               

          case(2)
                poidx(0)=ca(xx-1.,1)
                poidx(1)=cb(xx-1.,1,0)+cc(xx-1.,0)
          case(3)
                poidx(-1)=ca(xx,0)
                poidx(0)=cb(xx,0)+cc(xx,0)
          case(4)
                poidx(0)=ca(xx,1)+cb(xx,1,0)
                poidx(1)=cc(xx,0)


                     
          case(5)
             poidx(-1)=ca(xx,0)
             poidx(0)=cb(xx,0)             
             poidx(1)=cc(xx,0)-cc(xx-1.,1)
             poidx(2)=cc(xx-1.,1)
            
          case(6)
             poidx(-1)=ca(xx,0)
             poidx(0)=ca(xx-1.,1)-ca(xx,0)
             poidx(1)=cb(xx-1.,1)
             poidx(2)=cc(xx-1.,1)


          case(7)
             poidx(-2)=ca(xx+1.,0)
             poidx(-1)=ca(xx,0)-ca(xx+1.,0)
             poidx(0)=cb(xx,0)
             poidx(1)=cc(xx,0)
          end select

       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-2,2            
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do

    end do

  contains
   
       function coeff_l2g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)
        case(0)
           c=1.-x**2
        case(1)
           c=0.5*x*(1.+x)
        end select
        
      end function coeff_l2g

      function coeff_tscg (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)  +1./8. 
        case(0)
           c=-x**2+1.    -2./8. 
        case(1)
           c=0.5*x*(1.+x)   +1./8. 
        end select
        
      end function coeff_tscg

      function ca(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: ca
        
        ca=coeff_tscg(-1,x)+phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))
        
      end function ca

      function cb(x,num,num2)
        integer      :: num
        real(kind=8) :: x
        integer,optional :: num2
        real(kind=8) :: cb
       
        if (present (num2)) then
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num2)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        else
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        end if

      end function cb

      function cc(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: cc
      
        cc=coeff_tscg(1,x)+phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))
        

      end function cc
            
      function test (ipart,vois)
        implicit none
        integer       :: ipart,vois
        logical       :: test
        real(kind=8)  :: x
        
        test=.false.       
        x=(posy(mod(ipart-1+vois+npart,npart)+1) - real( floor((posy(mod(ipart-1+vois+npart,npart)+1)-yb)/dy) ,kind=8)*dy-yb)/dy
        if ( ((blocg(mod(ipart-1+vois+npart,npart)+1)==0).or.(blocg(mod(ipart-1+vois+npart,npart)+1)==2)).and.(x>0.5)) test=.true.
        
      end function test



 subroutine calcul_phi_tvd(phi_out,num) 
        implicit none
        integer                                :: num
        real(kind=8),dimension(0:1),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u
        real(kind=8)                           :: rp,rm,diff1,diff2,diff3,diff0,rmm,y,r 
        integer                                :: ib,jb
        real(kind=8),dimension(-5:5)           :: xx_vois,x,tp1,tp2,tp3

        !particules voisines
        !-------------------       
        do ib=-2,2
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        do ib=-1,2
           x(ib)=(posy(mod(i-1+ib+npart,npart)+1) - real( floor((posy(mod(i-1+ib+npart,npart)+1)-yb)/dy) ,kind=8)*dy-yb)/dy 
        end do


        !cas bloc centre xx>0.5 (psi)
        !-----------------------
        if (num==1) then
          
           do ib=0,1                          
              x(ib)=x(ib)-1.              
              tp1(ib)=6.-8.*x(ib)*x(ib)             
              tp3(ib)=4.*(x(ib)-0.5)**2          
           end do
           
           do ib=0,1
              r=(u(ib+1)-u(ib))/(u(ib)-u(ib-1))               

              !anti diffusif
              !phi_out(ib)=max(0.,min(tp3(ib),tp1(ib)*r))                            

              !du type Minmod (attention non justifié pour y>=0.8)
              phi_out(ib)=max(0.,min(1.,4.*r))

              !bornes dépendent de la condition cfl 
              !phi_out(ib)=max(0.,min(1.,tp3(ib),tp1(ib)*r))              
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.
           end do
 



        else  
           !sinon (phi)
           !-----------------------       


        do ib=-1,0
           tp1(ib)=6.-8.*x(ib)*x(ib)
           tp2(ib)=4.*(x(ib)+0.5)**2                
        end do


        do ib=0,1
              r=(u(ib-1)-u(ib-2))/(u(ib)-u(ib-1))                                                 
              !phi_out(ib)=max(0.,min(tp2(ib-1),tp1(ib-1)*r))
              phi_out(ib)=max(0.,min(1.,4.*r))
              !phi_out(ib)=max(0.,min(1.,tp2(ib-1),tp1(ib-1)*r))

              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.          
        end do

     end if
    

      end subroutine calcul_phi_tvd
                          

  end subroutine limit_l2_bloc_y




 subroutine limit_l2_bloc_z (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar,j,per
    integer,dimension(-2:2)                             :: ip
    real(kind=8),dimension(-2:2)                        :: poidx
    real(kind=8)                                        :: xx,tmp_pos  
    real(kind=8),dimension(0:2)                         :: phi_mdemi,phi_pdemi
    real(kind=8),dimension(0:1)                         :: phi
        

    remaille=0.      


    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       jvar=nint((posy(i)-yb)/dy)+1

       poidx=0.

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posz(i)-zd)/dz)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2    
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
      




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posz(i) - real(ip(0),kind=8)*dz-zd)/dz   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-2,2              
          ip(c)=mod(ip(c)+nz,nz)                
       end do

       !calcul du limiteur pour cas de bloc centre avec xx>0.5
       !--------------------------------------------------------
       call calcul_phi_tvd(phi,1)         
       phi_mdemi(1)=phi(0)
       phi_pdemi(1)=phi(1)           
       
       !les autres cas ...
       !---------------------
       call calcul_phi_tvd(phi,0)         
       phi_mdemi(0)=phi(0)
       phi_pdemi(0)=phi(1) 
       
       phi_mdemi(2)=0.
       phi_pdemi(2)=0.     

      select case (blocg(i))

          case(0)
             if (xx<=0.5) then

                poidx(-1)=ca(xx,0)
                if (test (i,1) ) then
                   poidx(0)=cb(xx,0,1)
                   poidx(1)=cc(xx,1)
                else
                   poidx(0)=cb(xx,0,0)
                   poidx(1)=cc(xx,0)
                end if

             else     
               
                poidx(0)=ca(xx-1.,1)
                if (test (i,1) ) then
                   poidx(1)=cb(xx-1.,1,1)
                   poidx(2)=cc(xx-1.,1)
                else                  
                   poidx(1)=cb(xx-1.,1,0)
                   poidx(2)=cc(xx-1.,0)
                end if                        
          
             end if

          case(1)
               
             poidx(-1)=ca(xx,0)
             if (test (i,1) ) then
                poidx(0)=cb(xx,0,1)
                poidx(1)=cc(xx,1)
             else
                poidx(0)=cb(xx,0,0)
                poidx(1)=cc(xx,0)
             end if
               

          case(2)
                poidx(0)=ca(xx-1.,1)
                poidx(1)=cb(xx-1.,1,0)+cc(xx-1.,0)
          case(3)
                poidx(-1)=ca(xx,0)
                poidx(0)=cb(xx,0)+cc(xx,0)
          case(4)
                poidx(0)=ca(xx,1)+cb(xx,1,0)
                poidx(1)=cc(xx,0)


                     
          case(5)
             poidx(-1)=ca(xx,0)
             poidx(0)=cb(xx,0)             
             poidx(1)=cc(xx,0)-cc(xx-1.,1)
             poidx(2)=cc(xx-1.,1)
            
          case(6)
             poidx(-1)=ca(xx,0)
             poidx(0)=ca(xx-1.,1)-ca(xx,0)
             poidx(1)=cb(xx-1.,1)
             poidx(2)=cc(xx-1.,1)


          case(7)
             poidx(-2)=ca(xx+1.,0)
             poidx(-1)=ca(xx,0)-ca(xx+1.,0)
             poidx(0)=cb(xx,0)
             poidx(1)=cc(xx,0)
          end select

       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-2,2            
          remaille(ivar,jvar,ip(c)+1)=remaille(ivar,jvar,ip(c)+1)+donne(i)*poidx(c)
       end do

    end do

  contains
   
       function coeff_l2g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)
        case(0)
           c=1.-x**2
        case(1)
           c=0.5*x*(1.+x)
        end select
        
      end function coeff_l2g

      function coeff_tscg (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)  +1./8. 
        case(0)
           c=-x**2+1.    -2./8. 
        case(1)
           c=0.5*x*(1.+x)   +1./8. 
        end select
        
      end function coeff_tscg

      function ca(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: ca
        
        ca=coeff_tscg(-1,x)+phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))
        
      end function ca

      function cb(x,num,num2)
        integer      :: num
        real(kind=8) :: x
        integer,optional :: num2
        real(kind=8) :: cb
       
        if (present (num2)) then
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num2)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        else
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        end if

      end function cb

      function cc(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: cc
      
        cc=coeff_tscg(1,x)+phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))
        

      end function cc
            
      function test (ipart,vois)
        implicit none
        integer       :: ipart,vois
        logical       :: test
        real(kind=8)  :: x
        
        test=.false.       
        x=(posz(mod(ipart-1+vois+npart,npart)+1) - real( floor((posz(mod(ipart-1+vois+npart,npart)+1)-zd)/dz) ,kind=8)*dz-zd)/dz
        if ( ((blocg(mod(ipart-1+vois+npart,npart)+1)==0).or.(blocg(mod(ipart-1+vois+npart,npart)+1)==2)).and.(x>0.5)) test=.true.
        
      end function test



 subroutine calcul_phi_tvd(phi_out,num) 
        implicit none
        integer                                :: num
        real(kind=8),dimension(0:1),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u
        real(kind=8)                           :: rp,rm,diff1,diff2,diff3,diff0,rmm,y,r 
        integer                                :: ib,jb
        real(kind=8),dimension(-5:5)           :: xx_vois,x,tp1,tp2,tp3

        !particules voisines
        !-------------------       
        do ib=-2,2
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        do ib=-1,2
           x(ib)=(posz(mod(i-1+ib+npart,npart)+1) - real( floor((posz(mod(i-1+ib+npart,npart)+1)-zd)/dz) ,kind=8)*dz-zd)/dz 
        end do


        !cas bloc centre xx>0.5 (psi)
        !-----------------------
        if (num==1) then
          
           do ib=0,1                          
              x(ib)=x(ib)-1.              
              tp1(ib)=6.-8.*x(ib)*x(ib)             
              tp3(ib)=4.*(x(ib)-0.5)**2          
           end do
           
           do ib=0,1
              r=(u(ib+1)-u(ib))/(u(ib)-u(ib-1))               

              !anti diffusif
              !phi_out(ib)=max(0.,min(tp3(ib),tp1(ib)*r))                            

              !du type Minmod (attention non justifié pour y>=0.8)
              phi_out(ib)=max(0.,min(1.,4.*r))

              !bornes dépendent de la condition cfl 
              !phi_out(ib)=max(0.,min(1.,tp3(ib),tp1(ib)*r))              
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.
           end do
 



        else  
           !sinon (phi)
           !-----------------------       


        do ib=-1,0
           tp1(ib)=6.-8.*x(ib)*x(ib)
           tp2(ib)=4.*(x(ib)+0.5)**2                
        end do


        do ib=0,1
              r=(u(ib-1)-u(ib-2))/(u(ib)-u(ib-1))                                                 
              !phi_out(ib)=max(0.,min(tp2(ib-1),tp1(ib-1)*r))
              phi_out(ib)=max(0.,min(1.,4.*r))
              !phi_out(ib)=max(0.,min(1.,tp2(ib-1),tp1(ib-1)*r))
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.          
        end do

     end if
    

      end subroutine calcul_phi_tvd
                          

  end subroutine limit_l2_bloc_z






end module remaillage_mod


