module utile_mod
  use donnees_mod
  use tab_mod
contains

  subroutine alloc_sub
    allocate (qg(1:nx,1:ny,1:nz),vxg(1:nx_gro,1:ny_gro,1:nz_gro),vyg(1:nx_gro,1:ny_gro,1:nz_gro),vzg(1:nx_gro,1:ny_gro,1:nz_gro))
    allocate (xp(1:npart),qp(1:npart),vx(1:npart),yp(1:npart),vy(1:npart),zp(1:npart),vz(1:npart))       
    
     allocate (xtab(1:nx),ytab(1:ny),ztab(1:nz))
     !allocate (exa(1:nx,1:ny,1:nz))

     allocate (xp1(1:npart),yp1(1:npart),zp1(1:npart))


     !rk3
     allocate (xp2(1:npart),yp2(1:npart),zp2(1:npart)) 
     allocate (vx1(1:npart),vx2(1:npart),vx3(1:npart),vx4(1:npart),vy1(1:npart),vz1(1:npart))
     allocate (xp0(1:npart))       
     allocate (vy2(1:npart),vy3(1:npart),vy4(1:npart),vy5(1:npart))
     allocate (zp0(1:npart))       
     allocate (vz2(1:npart),vz3(1:npart),vz4(1:npart))
      
     !rk4
     allocate (xp3(1:npart),yp3(1:npart),zp3(1:npart)) 

     !pour la correction de consistance
     allocate (numpg(1:nx,1:ny,1:nz)) 
     allocate (blocg(0:npart),blocd(0:npart),Nbloc(0:npart)) 
   end subroutine alloc_sub



   subroutine make_grille
     implicit none
     integer            :: i,j,k

     do k=1,nz
        ztab(k)=zd+(k-1)*dz
     end do
     do j=1,ny
        ytab(j)=yb+(j-1)*dy
     end do
     do i=1,nx
        xtab(i)=xg+(i-1)*dx
     end do
        
   end subroutine make_grille


   function tsource(t,x,y,z)
    implicit none
    real(kind=8),intent(in)  :: t,x,y,z
    real(kind=8)             :: tsource,t1,t2,t3,c1,c2,c3

    t2=0.2
    t3=0.3

    !u=u(t,x,y,z)
    !------------
    c1=cos(2.*pi*x)*cos(2.*pi*y)*cos(2.*pi*z)
    c2=cos(pi*t/t2)*cos(pi*t/t3)
    c3=sin(2.*pi*x)*sin(2.*pi*y)*sin(2.*pi*z)

    tsource=-pi*(sin(pi*t/t2)*c1)/t2 &
      +2.*pi*c3*c2*c1&
      -4.*pi*sin(pi*x)**2*sin(2.*pi*y)*sin(2.*pi*z)*c2*sin(2.*pi*x)*cos(2.*pi*y)*cos(2.*pi*z)&
      -pi*c3*c2*c1&
      +2.*pi*sin(2.*pi*x)*sin(pi*y)**2*sin(2.*pi*z)*c2*cos(2.*pi*x)*sin(2.*pi*y)*cos(2.*pi*z)&
      -pi*c3*c2*c1&
      +2.*pi*sin(2.*pi*x)*sin(2.*pi*y)*sin(pi*z)**2*c2*cos(2.*pi*x)*cos(2.*pi*y)*sin(2.*pi*z)
!!$
!!$    tsource=0.

  end function tsource


subroutine source_rk4
  use advection_mod
    implicit none
    integer                                  :: i
    real(kind=8)                             :: rk1,rk2,rk3,rk4,t1,t2,t3
    real(kind=8)                             :: x1,x2,x3,y1,y2,y3,z1,z2,z3
    real(kind=8)                             :: v1x,v2x,v3x,v1y,v2y,v3y,v1z,v2z,v3z



    do i=1,npart

       !euler
       !------
       !qp(i)=qp(i)+dt*tsource(time,xp(i),yp(i),zp(i))

       !rk2
       !----
       !qp(i)=qp(i)+dt*tsource(time+0.5*dt,xp(i)+0.5*dt*vx(i),yp(i)+0.5*dt*vy(i),zp(i)+0.5*dt*vz(i))        


       !rk4
       !---
       rk1=tsource(time,xp(i),yp(i),zp(i))
       x1=xp(i)+0.5*dt*vx(i)
       y1=yp(i)+0.5*dt*vy(i)
       z1=zp(i)+0.5*dt*vz(i)
       rk2=tsource(time+0.5*dt,x1,y1,z1)
       call evalv (v1x,v1y,v1z,time+0.5*dt,x1,y1,z1)
       x2=xp(i)+0.5*dt*v1x
       y2=yp(i)+0.5*dt*v1y
       z2=zp(i)+0.5*dt*v1z
       rk3=tsource(time+0.5*dt,x2,y2,z2)
       call evalv (v2x,v2y,v2z,time+0.5*dt,x2,y2,z2)
       x3=xp(i)+dt*v2x
       y3=yp(i)+dt*v2y
       z3=zp(i)+dt*v2z
       rk4=tsource(time+dt,x3,y3,z3)

       qp(i)=qp(i)+dt*(rk1+2.*rk2+2.*rk3+rk4)/6.

    end do


  end subroutine source_rk4


  subroutine source_split_1
    implicit none
    integer                                  :: i
      
    !ordre2
    !------
    do i=1,npart           
       qp(i)=qp(i)+0.5*dt*tsource(time,xp(i),yp(i),zp(i))
    end do

         
  end subroutine source_split_1

  subroutine source_split_2
    implicit none
    integer                                  :: i,j,k
    real(kind=8)                             :: x,y,z


    !ordre2
    !------
    do k=1,nz
       z=ztab(k)
       do j=1,ny
          y=ytab(j)
          do i=1,nx
             x=xtab(i)
          
             qg(i,j,k)=qg(i,j,k)+0.5*dt*tsource(time+dt,x,y,z)
             !qg(i,j,k)=qg(i,j,k)+0.5*dt*tsource(time,x,y,z)

          end do
       end do
    end do

  end subroutine source_split_2



 subroutine make_bloc(esp)
    use donnees_mod
    implicit none
    integer,intent(in)       :: esp
    integer                  :: fin,ib,i,j,k,l,im,ip,dim1,dim2,dim3
    logical                  :: zero
    real(kind=8)             :: pas

    select case(esp)
    case(1)
       dim1=nx
       dim2=ny
       dim3=nz
       pas=dx
    case(2)
       dim1=ny
       dim2=nz
       dim3=nx
       pas=dy
    case(3)
       dim1=nz
       dim2=nx
       dim3=ny
       pas=dz
    end select


    !-----------------------------------------------------------------------------------------
    !Determine la nature des blocs
    !Donne un flag correspondant au type de remaillage necessaire (centre,decentre ou modifie)
    !------------------------------------------------------------------------------------------

    blocg(1:npart)=1
    blocd(1:npart)=1    
         
    direction1: do l=1,dim3

    direction2: do j=1,dim2

       !peut calculer la longeur -> gain si longeur eleve

       !parcours la grille, bloc par bloc:determine le type des bloc
       !---------------------------------

       blocg(0)=100
       blocd(0)=100
       Nbloc(0)=100

       type_bloc:do i=1,dim1,long_bloc+1


          if ( (i+long_bloc)<(dim1-1) ) then
             fin=i+long_bloc
             call sub_typeb
          else
             !parcours le bloc plus premier point du bloc suivant
             fin=dim1-1
             call sub_typeb            
             blocg(pg(dim1,j,l))=blocg(pg(dim1-1,j,l))
             blocd(pg(dim1,j,l))=blocd(pg(dim1-1,j,l))               
             Nbloc(pg(dim1,j,l))=Nbloc(pg(dim1-1,j,l))            
          end if         

       end do type_bloc

       !parcours la grille, bloc par bloc:determine si modif de remaillage necessaire a l'intersection des blocs
       !---------------------------------
       type_remaill:do i=1,dim1,long_bloc+1

          if (((i+long_bloc)<dim1).and.(i>1+long_bloc)) then            
             fin=i+long_bloc
             call sub_remb
          else             
             if (i<=1+long_bloc)then
                !premier bloc
                !-------------
                fin=i+long_bloc
                call sub_remb              

             else
                !dernier bloc 
                !-------------
                fin=dim1
                call sub_remb

             end if

          end if

       end do type_remaill
    end do direction2
 end do direction1

  contains

    subroutine sub_typeb

      integer            :: ni,N
      real(kind=8)       :: li
      logical            :: pasfait


      !parcours une premiere fois le bloc pour savoir si c'est un bloc en 0 ou 1/2
      !----------------------------------     
      pasfait=.true.
      zero=.true.
      do_zero:do ib=i,fin+1  !eviter cas change de bloc quand maille vide
         if (pg(ib,j,l)/=0) then
            if (pasfait) then  !le N doit etre le meme pour tout le bloc !
               li=dt/pas*vit(pg(ib,j,l))
               ni=floor(li+0.5)
               pasfait=.false.
            end if
            li=dt/pas*vit(pg(ib,j,l))
            if ( ((li)<(ni-0.5)).or.((li)>(ni+0.5)) ) then
               zero=.false.
               exit do_zero 
            end if
         end if
      end do do_zero


      !parcours a nouveau le bloc 
      !-------------------------
      !affecte le type de bloc a toute les part du bloc

      pasfait=.true.
      bloc2:do ib=i,fin

         if (pg(ib,j,l)/=0) then

            !calcul de N
            !------------   
            if (pasfait) then
               if (zero) then                 
                  N=floor(dt/pas*vit(pg(ib,j,l))+0.5)
                  pasfait=.false.
               else
                  N=floor(dt/pas*vit(pg(ib,j,l)))
                  pasfait=.false.
               end if
            end if

            if (zero) then
               !bloc pour lambda2 centre
               blocg(pg(ib,j,l))=0
               blocd(pg(ib,j,l))=0                
            end if

            !affecte la constante N               
            Nbloc(pg(ib,j,l))=N

         end if
      end do bloc2

    end subroutine sub_typeb


    subroutine sub_remb
      implicit none

      integer,dimension(-2:2)  :: iper
      integer                  :: k


      !parcours  a nvx le bloc     
      !-----------------------
      !detecte s'il y aura une correction de remaillage a apporter à l'intersection de ces blocs

      if (i==fin) then
         blocg(i)=blocg(i-1)
         blocd(i)=blocd(i-1)
      else
      bloc3:do ib=i,fin,fin-i

         if (pg(ib,j,l)/=0) then

            !si fin de bloc en 0. :
            !----------------------
            do k=-2,2              
               iper(k)=pg(mod(ib+k+dim1-1,dim1)+1,j,l)          
            end do


            if ((ib==fin).and.(blocd(pg(ib,j,l))==0).and.((blocg(iper(1))==1).or.(blocg(iper(2))==1)).and.((Nbloc(iper(1))==Nbloc(pg(ib,j,l))-1) .or.(Nbloc(iper(2))==Nbloc(pg(ib,j,l))-1) ) ) then

               if (type_b==2) then

                  if ( dt/pas*vit(pg(ib,j,l))<=Nbloc(pg(ib,j,l))  ) then
                     blocg(iper(1))=2
                     blocd(pg(ib,j,l))=3
                  else
                     blocg(iper(1))=2
                     blocd(pg(ib,j,l))=4
                     blocg(pg(ib,j,l))=5
                  end if

               else
                  blocg(iper(1))=2
                  blocg(iper(2))=5

                  if ( dt/pas*vit(pg(ib,j,l))<=Nbloc(pg(ib,j,l))  ) then                 
                     blocd(pg(ib,j,l))=3
                  else                 
                     blocd(pg(ib,j,l))=7
                     blocg(pg(ib,j,l))=8
                  end if

                  if ( dt/pas*vit(iper(-1))<=Nbloc(pg(ib,j,l))  ) then
                     blocd(iper(-1))=6
                  else
                     blocd(iper(-1))=5
                  end if
               end if

            end if


            !si debut de bloc en 0 : detecte quelle correction il faudra apporter
            !----------------------
            if ((ib==i) .and.(blocg(pg(ib,j,l))==0).and.((blocd(iper(-1))==1).or.(blocd(iper(-2))==1)).and.((Nbloc(iper(-1))==Nbloc(pg(ib,j,l))-1) .or. (Nbloc(iper(-2))==Nbloc(pg(ib,j,l))-1) ) ) then

               if (type_b==2) then

                  if ( dt/pas*vit(pg(ib,j,l))>Nbloc(pg(ib,j,l)) ) then
                     blocd(iper(-1))=2
                     blocg(pg(ib,j,l))=4
                  else
                     blocd(iper(-1))=2
                     blocg(pg(ib,j,l))=3
                  end if

               else

                  blocd(iper(-1))=2
                  blocd(iper(-2))=4

                  if ( dt/pas*vit(pg(ib,j,l))>Nbloc(pg(ib,j,l)) ) then                 
                     blocg(pg(ib,j,l))=4
                  else                 
                     blocg(pg(ib,j,l))=3
                  end if

                  if ( dt/pas*vit(iper(1))>Nbloc(pg(ib,j,l)) ) then
                     blocg(iper(1))=7
                  else
                     blocg(iper(1))=6
                  end if

               end if
            end if



         end if
      end do bloc3
   end if

    end subroutine sub_remb

    function pg(a,b,c) 
      implicit none
      integer,intent(in)  :: a,b,c
      integer             :: pg

      select case (esp)
      case(1)
         pg=numpg(a,b,c)
      case(2)
         pg=numpg(c,a,b)
      case(3)
         pg=numpg(b,c,a)
      end select

    end function pg

    function vit(ind) 
      implicit none
      integer,intent(in)  :: ind
      real(kind=8)        :: vit

      select case (esp)
      case(1)
         vit=vx(ind)
      case(2)
         vit=vy(ind)
      case(3)
         vit=vz(ind)
      end select

    end function vit


  end subroutine make_bloc



subroutine make_bloc_v2(esp)
    use donnees_mod
    implicit none
    integer,intent(in)       :: esp
    integer                  :: fin,ib,i,j,k,l,im,ip,dim1,dim2,dim3
    integer                  :: n,np,part,partp,partm,partpp
    real(kind=8)             :: pas

    select case(esp)
    case(1)
       dim1=nx
       dim2=ny
       dim3=nz
       pas=dx
    case(2)
       dim1=ny
       dim2=nz
       dim3=nx
       pas=dy
    case(3)
       dim1=nz
       dim2=nx
       dim3=ny
       pas=dz
    end select

    !-----------------------------------------------------------------------------------------
    !Determine la nature des blocs
    !Donne un flag correspondant au type de remaillage necessaire (centre,decentre ou modifie)
    !------------------------------------------------------------------------------------------

    blocg(1:npart)=1
    blocd(1:npart)=1   

    blocg(0)=100
    blocd(0)=100
    Nbloc(0)=100 
         


    direction1: do l=1,dim3

    direction2: do j=1,dim2

       !peut calculer la longeur -> gain si longeur eleve

       !parcours la grille, bloc par bloc:determine le type des bloc
       !---------------------------------      

       type_bloc:do i=1,dim1,long_bloc+1


          if ( (i+long_bloc)<(dim1-1) ) then
             fin=i+long_bloc
             call sub_typeb
          else
             !parcours le bloc plus premier point du bloc suivant
             do k=i,dim1
                blocg(pg(k,j,l))=blocg(pg(i-1,j,l))
                blocd(pg(k,j,l))=blocd(pg(i-1,j,l))               
                Nbloc(pg(k,j,l))=Nbloc(pg(i-1,j,l))            
             end do
          end if         

       end do type_bloc

       !parcours la grille, bloc par bloc:determine si modif de remaillage necessaire a l'intersection des blocs
       !---------------------------------
       type_remaill:do i=1,dim1,long_bloc+1

          if ((i+long_bloc)<dim1) then            

             fin=i+long_bloc

             k=fin
             do while ((pg(k,j,l)==0).and.(k>i))
                k=k-1
             end do
             n=Nbloc(pg(k,j,l)) ! s'il n'y a pas de part dans le bloc N=100 -> ne fera pas de correction

             k=fin+1
             do while ((pg(k,j,l)==0).and.(k<(fin+long_bloc+2)).and.(k<nx))
                k=k+1
             end do
             np=Nbloc(pg(k,j,l))

             part=pg(fin,j,l)
             partp=pg(fin+1,j,l)
             partm=pg(fin-1,j,l)
             partpp=pg(fin+2,j,l)

             if (type_b==2) call sub_remb_2(n,np,part,partp)
             if (type_b==4) call sub_remb_4(n,np,part,partp,partm,partpp)
            

          else             

             !dernier bloc 
             !-------------
             fin=dim1             

             k=fin
             do while ((pg(k,j,l)==0).and.(k>i))
                k=k-1
             end do
             n=Nbloc(pg(k,j,l)) 
             
             k=1
             do while ((pg(k,j,l)==0).and.(k<(1+long_bloc+1)))
                k=k+1
             end do
             np=Nbloc(pg(k,j,l))
     
             part=pg(fin,j,l)
             partp=pg(1,j,l)
             partm=pg(fin-1,j,l)
             partpp=pg(2,j,l)
             
             if (type_b==2) call sub_remb_2(n,np,part,partp)
             if (type_b==4) call sub_remb_4(n,np,part,partp,partm,partpp)


          end if

       end do type_remaill
    end do direction2
 end do direction1

  contains

    subroutine sub_typeb
    
      integer            :: N,intl
      real(kind=8)       :: li,lmin


      !parcours une premiere fois le bloc pour savoir si c'est un bloc en 0 ou 1/2
      !----------------------------------     
      lmin=99999999999999999999999.
      do_lmin:do ib=i,fin
         if (pg(ib,j,l)/=0) then
            li=dt/pas*vit(pg(ib,j,l))
            lmin=min(lmin,li)
         end if
      end do do_lmin

      !calcul du type de bloc et de N
      !-------------------------------
      intl=floor(lmin)
      if ((lmin-intl).le.0.5) then
         !type gauche
         !-----------
         N=intl
         do_typeg:do ib=i,fin
            if (pg(ib,j,l)/=0) then
               blocg(pg(ib,j,l))=1 
               blocd(pg(ib,j,l))=1  
               Nbloc(pg(ib,j,l))=N              
            end if
         end do do_typeg
      else
         !type centre
         !-----------
         N=intl+1
         do_typec:do ib=i,fin
            if (pg(ib,j,l)/=0) then
               blocg(pg(ib,j,l))=0
               blocd(pg(ib,j,l))=0
               Nbloc(pg(ib,j,l))=N
            end if
         end do do_typec
      end if

    end subroutine sub_typeb

    subroutine sub_remb_2(n,np,partf,partfp)
      !attention les bords ne sont pas bien traités
      implicit none

      integer,intent(in) ::n,np,partf,partfp
      real(kind=8)       :: li


      !parcours  a nvx le bloc     
      !-----------------------
      !detecte s'il y aura une correction de remaillage a apporter à l'intersection des blocs
      if ((np) == (n-1)) then        
         li=dt/pas*vit(partf)
         if ( (li<n).and.(blocg(partf)==0) ) then !seconde condition=bloc centre
            blocg(partf)=2
         else
            blocg(partf)=3
         end if
         blocg(partfp)=4
      end if
      if ((np) == (n+1)) then
         li=dt/pas*vit(partfp)
         blocg(partf)=5               
         if ((li<np).and.(blocg(partfp)==0)) then
            blocg(partfp)=6
         else
            blocg(partfp)=7
         end if
      end if



    end subroutine sub_remb_2


    subroutine sub_remb_4(n,np,partf,partfp,partfm,partfpp)
      !attention les bords ne sont pas bien traités
      implicit none

      integer,intent(in) ::n,np,partf,partfp,partfm,partfpp
      real(kind=8)       :: li


      !parcours  a nvx le bloc     
      !-----------------------
      !detecte s'il y aura une correction de remaillage a apporter à l'intersection des blocs

      !N_{m+1}=N_m -1
      !--------------
      if ((np) == (n-1)) then    
         !si bloc centre
         if (blocd(partfm)==0) then
            li=dt/pas*vit(partfm)           
            if ( (li<n) ) then 
               blocd(partfm)=2
            else
               blocd(partfm)=3
            end if
         else
            blocd(partfm)=3
         end if
         if (blocg(partf)==0) then
            li=dt/pas*vit(partf)
            if ( (li<n) ) then 
               blocg(partf)=2
               blocd(partf)=4
            else
               blocg(partf)=3
               blocd(partf)=5
            end if                      
         else
            blocg(partf)=3
            blocd(partf)=5           
         end if
         blocg(partfp)=4
         blocd(partfp)=1
         li=dt/pas*vit(partfpp)
         if ( (blocg(partfpp)==0).and.(li<np) ) then
            blocg(partfpp)=5
         else
            blocg(partfpp)=6
         end if
      end if
      !N_{m+1}=N_m +1
      !--------------
      if ((np) == (n+1)) then 
         li=dt/pas*vit(partfm)
         if ( (blocd(partfm)==0).and.(li<n) ) then
            blocd(partfm)=6
         else
            blocd(partfm)=7
         end if
         blocg(partf)=1
         blocd(partf)=8   
         if (blocg(partfp)==0) then
            li=dt/pas*vit(partfp)
            if ( (li<np) ) then 
               blocg(partfp)=7
               blocd(partfp)=0
            else
               blocg(partfp)=8
               blocd(partfp)=1
            end if
         else
            blocg(partfp)=8
            blocd(partfp)=1
         end if
         if (blocg(partfpp)==0) then
            li=dt/pas*vit(partfpp)
            if ( (li<np) ) then 
               blocg(partfpp)=9
            else
               blocg(partfpp)=10
            end if
         else
            blocg(partfpp)=10
         end if
                           
      end if           



    end subroutine sub_remb_4
  

    function pg(a,b,c) 
      implicit none
      integer,intent(in)  :: a,b,c
      integer             :: pg

      select case (esp)
      case(1)
         pg=numpg(a,b,c)
      case(2)
         pg=numpg(c,a,b)
      case(3)
         pg=numpg(b,c,a)
      end select

    end function pg

    function vit(ind) 
      implicit none
      integer,intent(in)  :: ind
      real(kind=8)        :: vit

      select case (esp)
      case(1)
         vit=vx(ind)
      case(2)
         vit=vy(ind)
      case(3)
         vit=vz(ind)
      end select

    end function vit


  end subroutine make_bloc_v2



  end module utile_mod


