module advection_mod
  use donnees_mod
  use tab_mod
  use remaillage_mod
  use interpolation_mod
  use nrtype 
  USE nrutil
  USE nr 
contains

 subroutine def_v_advec  (vitx,vity,vitz,tps)  
    implicit none

    real(kind=8),intent(in)                                        :: tps
    real(kind=8),dimension(1:nx_gro,1:ny_gro,1:nz_gro),intent(out) :: vitx,vity,vitz 
    integer                                                        :: i,j,k
    real(kind=8)                                                   :: x,y,z


    !-------------------
    !deformation sphere: 
    !--------------------
    do k=1,nz_gro
       z=zd+(k-1)*dz_gro
       do j=1,ny_gro
          y=yb+(j-1)*dy_gro
          do i=1,nx_gro
             x=xg+(i-1)*dx_gro

             vitx(i,j,k)=2.*sin(pi*x)**2*sin(2.*pi*y)*sin(2.*pi*z)
             vity(i,j,k)=-sin(2.*pi*x)*sin(pi*y)**2*sin(2.*pi*z)
             vitz(i,j,k)=-sin(2.*pi*x)*sin(2.*pi*y)*sin(pi*z)**2

          end do
       end do
    end do

    !----------------------------------
    !champ turbulent 
    !----------------------------------
   
!!$    allocate (vxgr(1:nx,1:ny,1:nz),vygr(1:nx,1:ny,1:nz),vzgr(1:nx,1:ny,1:nz))
!!$    open(20,file='data_gh/datavelx',form='unformatted',convert='big_endian',status='unknown')             
!!$    read(20)  (((vxgr(i,j,k),i=1,nx_gro),j=1,nx_gro),k=1,nx_gro)             
!!$    close(20)
!!$    
!!$    open(20,file='data_gh/datavely',form='unformatted',convert='big_endian',status='unknown')             
!!$    read(20)  (((vygr(i,j,k),i=1,nx_gro),j=1,nx_gro),k=1,nx_gro)             
!!$    close(20)
!!$    
!!$    open(20,file='data_gh/datavelz',form='unformatted',convert='big_endian',status='unknown')             
!!$    read(20)  (((vzgr(i,j,k),i=1,nx_gro),j=1,nx_gro),k=1,nx_gro)             
!!$    close(20)
!!$
!!$    vitx=vxgr
!!$    vity=vygr
!!$    vitz=vzgr
!!$    deallocate (vxgr,vygr,vzgr)


  end subroutine def_v_advec


  subroutine vitesse_fft
    implicit none

    complex(kind=8),dimension(:,:,:),pointer   :: sf1x, sf1y, sf1z
    complex(kind=8),dimension(:,:,:),pointer   :: su1x, su1y, su1z
    complex(kind=8),dimension(:,:),pointer     :: sf2x, sf2y, sf2z
    complex(kind=8),dimension(:,:),pointer     :: su2x, su2y, su2z 

    integer                                     :: i,j,k,m,n,p
    real(kind=8)                                :: fac,x,y,z,l1,l2,l3,kx,ky,kz,r2,coeff
    complex(kind=8)                             :: coef

  
    l1=(xd-xg)
    l2=(yh-yb)
    l3=(zf-zd)

    allocate (sf1x(1:nx/2,1:ny,1:nz),su1x(1:nx/2,1:ny,1:nz) )
    allocate (sf1y(1:nx/2,1:ny,1:nz),su1y(1:nx/2,1:ny,1:nz) )
    allocate (sf1z(1:nx/2,1:ny,1:nz),su1z(1:nx/2,1:ny,1:nz) )
    allocate (sf2x(1:ny,1:nz),su2x(1:ny,1:nz) )
    allocate (sf2y(1:ny,1:nz),su2y(1:ny,1:nz) )
    allocate (sf2z(1:ny,1:nz),su2z(1:ny,1:nz) )

   
    !fft forward
    call rlft3(omgx,sf1x,sf2x,1)
    call rlft3(omgy,sf1y,sf2y,1)
    call rlft3(omgz,sf1z,sf2z,1)

    !frequence=0 : n=0
    !0<f<fc : 1<=n<=N/2-1
    !f=fc=-fc : n=N/2
    !-fc<f<0 : N/2+1<=n<=N-1 

    fac=2./(nx*ny*nz) ! pour normalisation de la fft inverse

    !calcul de la solution dans l'espace des frequences 
 do p=1,nz
     if ( (p-1)<=(nz/2) ) then
        kz=(p-1)/l3
     else
        kz=(p-1-nz)/l3
     end if
     do n=1,ny
        if ( (n-1)<=(ny/2) ) then
           ky=(n-1)/l2
        else
           ky=(n-1-ny)/l2
        end if
        do m=1,nx/2           

           if ( (m-1)<=(nx/2) ) then
              kx=(m-1)/l1
           else
              kx=(m-1-nx)/l1
           end if



           r2=kx**2+ky**2+kz**2
           coef=-fac*cmplx(0.,1.)/(2.*pi*r2)


           su1x(m,n,p)=coef*(ky*sf1z(m,n,p)-kz*sf1y(m,n,p))
           if ((r2==0)) su1x(m,n,p)=0.

           su1y(m,n,p)=coef*(kz*sf1x(m,n,p)-kx*sf1z(m,n,p))              
           if ((r2==0)) su1y(m,n,p)=0.

           su1z(m,n,p)=coef*(kx*sf1y(m,n,p)-ky*sf1x(m,n,p))
           if ((r2==0)) su1z(m,n,p)=0.



        end do
     end do
  end do

  m=nx/2+1
  kx=(m-1)/l1
  do p=1,nz
     if ( (p-1)<=(nz/2) ) then
        kz=(p-1)/l3
     else
        kz=(p-1-nz)/l3
     end if
     do n=1,ny

        if ( (n-1)<=(ny/2) ) then
           ky=(n-1)/l2
        else
           ky=(n-1-ny)/l2
        end if


        r2=kx**2+ky**2+kz**2
        coef=-fac*cmplx(0.,1.)/(2.*pi*r2)

        !poisson+vitess
        su2x(n,p)=coef*(ky*sf2z(n,p)-kz*sf2y(n,p))                 
        su2y(n,p)=coef*(kz*sf2x(n,p)-kx*sf2z(n,p))               
        su2z(n,p)=coef*(kx*sf2y(n,p)-ky*sf2x(n,p))
       

        !pour -omega et sans normalisation:
        !---------------------------------
        !su2z(n,p)=-2.*pi*cmplx(0.,1.)*(-kx*sf2y(n,p)+ky*sf2x(n,p))/(4.*pi**2*r2)
        !if ((r2==0)) su2z(n,p)=0.



     end do
  end do

    !fft backward   

    call rlft3(vxg,su1x,su2x,-1)
    call rlft3(vyg,su1y,su2y,-1)
    call rlft3(vzg,su1z,su2z,-1)

    deallocate(sf1x,su1x)
    deallocate(sf1y,su1y)
    deallocate(sf1z,su1z)
    deallocate(sf2x,su2x)
    deallocate(sf2y,su2y)
    deallocate(sf2z,su2z)

   
  end subroutine vitesse_fft






  subroutine vitesse_fft_uinf
    implicit none

    complex(kind=8),dimension(:,:,:),pointer   :: sf1x, sf1y, sf1z
    complex(kind=8),dimension(:,:,:),pointer   :: su1x, su1y, su1z
    complex(kind=8),dimension(:,:),pointer     :: sf2x, sf2y, sf2z
    complex(kind=8),dimension(:,:),pointer     :: su2x, su2y, su2z 

    integer                                     :: i,j,k,m,n,p
    real(kind=8)                                :: fac,x,y,z,l1,l2,l3,kx,ky,kz,r2,coeff,uinfx,uinfy
    complex(kind=8)                             :: coef

  
    l1=(xd-xg)
    l2=(yh-yb)
    l3=(zf-zd)

    allocate (sf1x(1:nx/2,1:ny,1:nz),su1x(1:nx/2,1:ny,1:nz) )
    allocate (sf1y(1:nx/2,1:ny,1:nz),su1y(1:nx/2,1:ny,1:nz) )
    allocate (sf1z(1:nx/2,1:ny,1:nz),su1z(1:nx/2,1:ny,1:nz) )
    allocate (sf2x(1:ny,1:nz),su2x(1:ny,1:nz) )
    allocate (sf2y(1:ny,1:nz),su2y(1:ny,1:nz) )
    allocate (sf2z(1:ny,1:nz),su2z(1:ny,1:nz) )

   
    !fft forward
    call rlft3(omgx,sf1x,sf2x,1)
    call rlft3(omgy,sf1y,sf2y,1)
    call rlft3(omgz,sf1z,sf2z,1)

    !frequence=0 : n=0
    !0<f<fc : 1<=n<=N/2-1
    !f=fc=-fc : n=N/2
    !-fc<f<0 : N/2+1<=n<=N-1 

    fac=2./(nx*ny*nz) ! pour normalisation de la fft inverse

    !calcul de la solution dans l'espace des frequences 
 do p=1,nz
     if ( (p-1)<=(nz/2) ) then
        kz=(p-1)/l3
     else
        kz=(p-1-nz)/l3
     end if
     do n=1,ny
        if ( (n-1)<=(ny/2) ) then
           ky=(n-1)/l2
        else
           ky=(n-1-ny)/l2
        end if
        do m=1,nx/2           

           if ( (m-1)<=(nx/2) ) then
              kx=(m-1)/l1
           else
              kx=(m-1-nx)/l1
           end if



           r2=kx**2+ky**2+kz**2
           coef=-fac*cmplx(0.,1.)/(2.*pi*r2)
           
         
           uinfx=(4.*(1./0.4844)**2)/((yh-yb)*(zf-zd))!debit*surface domaine physique/surface du domaine numérique 

           !if ((time>=3.).and.(time<=4.)) then
           !   uinfy=sin(pi*(time-3.))
           !else
              uinfy=0.
           !end if


           su1x(m,n,p)=coef*(ky*sf1z(m,n,p)-kz*sf1y(m,n,p))
           if ((r2==0)) su1x(m,n,p)=2.*uinfx  !atttention mettre 2*uinfx

           su1y(m,n,p)=coef*(kz*sf1x(m,n,p)-kx*sf1z(m,n,p))              
           if ((r2==0)) su1y(m,n,p)=2.*uinfy

           su1z(m,n,p)=coef*(kx*sf1y(m,n,p)-ky*sf1x(m,n,p))
           if ((r2==0)) su1z(m,n,p)=0.



        end do
     end do
  end do

  m=nx/2+1
  kx=(m-1)/l1
  do p=1,nz
     if ( (p-1)<=(nz/2) ) then
        kz=(p-1)/l3
     else
        kz=(p-1-nz)/l3
     end if
     do n=1,ny

        if ( (n-1)<=(ny/2) ) then
           ky=(n-1)/l2
        else
           ky=(n-1-ny)/l2
        end if


        r2=kx**2+ky**2+kz**2
        coef=-fac*cmplx(0.,1.)/(2.*pi*r2)

        !poisson+vitess
        su2x(n,p)=coef*(ky*sf2z(n,p)-kz*sf2y(n,p))                 
        su2y(n,p)=coef*(kz*sf2x(n,p)-kx*sf2z(n,p))               
        su2z(n,p)=coef*(kx*sf2y(n,p)-ky*sf2x(n,p))
       

        !pour -omega et sans normalisation:
        !---------------------------------
        !su2z(n,p)=-2.*pi*cmplx(0.,1.)*(-kx*sf2y(n,p)+ky*sf2x(n,p))/(4.*pi**2*r2)
        !if ((r2==0)) su2z(n,p)=0.



     end do
  end do

    !fft backward   

    call rlft3(vxg,su1x,su2x,-1)
    call rlft3(vyg,su1y,su2y,-1)
    call rlft3(vzg,su1z,su2z,-1)

    deallocate(sf1x,su1x)
    deallocate(sf1y,su1y)
    deallocate(sf1z,su1z)
    deallocate(sf2x,su2x)
    deallocate(sf2y,su2y)
    deallocate(sf2z,su2z)

   
  end subroutine vitesse_fft_uinf








  subroutine crea_part_x
    implicit none
    integer                  :: i,j,k
    real(kind=8)             :: x,y,z,m     

    m=cutoff*maxval(sqrt(omgx**2+omgy**2+omgz**2))
    npart=0  
    numpg=0
    do k=1,nz
       z=ztab(k)
       do j=1,ny
          y=ytab(j)
          do i=1,nx
             x=xtab(i) 

             if (sqrt(omgx(i,j,k)**2+omgy(i,j,k)**2+omgz(i,j,k)**2)>=m) then
                npart=npart+1 
                numpg(i,j,k)=npart
                xp(npart)=x
                yp(npart)=y
                zp(npart)=z
                vx(npart)=vxg(i,j,k)
                vy(npart)=vyg(i,j,k)
                vz(npart)=vzg(i,j,k)
                !vx(npart)=interpol_v_l3(vxg,x,y,z)
                !vy(npart)=interpol_v_l3(vyg,x,y,z) 
                !vz(npart)=interpol_v_l3(vzg,x,y,z)
                omx(npart)=omgx(i,j,k)
                omy(npart)=omgy(i,j,k)
                omz(npart)=omgz(i,j,k)
             end if
          end do
       end do
    end do

  end subroutine crea_part_x

  subroutine crea_part_y
    implicit none
    integer                  :: i,j,k
    real(kind=8)             :: x,y,z,m     

    m=cutoff*maxval(sqrt(omgx**2+omgy**2+omgz**2))
    npart=0  
    numpg=0
    do k=1,nz
       z=ztab(k)        
       do i=1,nx
          x=xtab(i) 
          do j=1,ny
             y=ytab(j)

             if (sqrt(omgx(i,j,k)**2+omgy(i,j,k)**2+omgz(i,j,k)**2)>=m) then
                npart=npart+1   
                numpg(i,j,k)=npart           
                xp(npart)=x
                yp(npart)=y
                zp(npart)=z
                vx(npart)=vxg(i,j,k)
                vy(npart)=vyg(i,j,k)
                vz(npart)=vzg(i,j,k)
                !vx(npart)=interpol_v_l3(vxg,x,y,z)
                !vy(npart)=interpol_v_l3(vyg,x,y,z) 
                !vz(npart)=interpol_v_l3(vzg,x,y,z) 
                omx(npart)=omgx(i,j,k)
                omy(npart)=omgy(i,j,k)
                omz(npart)=omgz(i,j,k)  
             end if
          end do
       end do
    end do

  end subroutine crea_part_y

  subroutine crea_part_z
    implicit none
    integer                  :: i,j,k
    real(kind=8)             :: x,y,z,m     

    m=cutoff*maxval(sqrt(omgx**2+omgy**2+omgz**2))
    npart=0    
    numpg=0

    do j=1,ny
       y=ytab(j)
       do i=1,nx
          x=xtab(i) 
          do k=1,nz
             z=ztab(k)

             if (sqrt(omgx(i,j,k)**2+omgy(i,j,k)**2+omgz(i,j,k)**2)>=m) then
                npart=npart+1 
                numpg(i,j,k)=npart             
                xp(npart)=x
                yp(npart)=y
                zp(npart)=z
                vx(npart)=vxg(i,j,k)
                vy(npart)=vyg(i,j,k)
                vz(npart)=vzg(i,j,k)
                !vx(npart)=interpol_v_l3(vxg,x,y,z)
                !vy(npart)=interpol_v_l3(vyg,x,y,z) 
                !vz(npart)=interpol_v_l3(vzg,x,y,z) 
                omx(npart)=omgx(i,j,k)
                omy(npart)=omgy(i,j,k)
                omz(npart)=omgz(i,j,k)   
             end if
          end do
       end do
    end do

  end subroutine crea_part_z

 subroutine ad_euler_x           
    implicit none

    integer                  :: i,j
    real(kind=8)             :: x,y

    do i=1,npart         
       xp(i)=xp(i)+dt*vx(i)                               
    end do

  end subroutine ad_euler_x

  subroutine ad_euler_y           
    implicit none

    integer                  :: i,j
    real(kind=8)             :: x,y

    do i=1,npart         
       yp(i)=yp(i)+dt*vy(i)   
    end do

  end subroutine ad_euler_y

  subroutine ad_euler_z           
    implicit none

    integer                  :: i

    do i=1,npart         
       zp(i)=zp(i)+dt*vz(i)   
    end do

  end subroutine ad_euler_z






  !===============================
  !SPLITTING
  !calcul de la vitesse pour une advection d'euler à l'ordre 2 ou 3
  !===============================

  subroutine update_vx_2
    implicit none
    integer                  :: ib

    allocate (xp1(1:npart),yp1(1:npart),zp1(1:npart))           
    do ib=1,npart         
       xp1(ib)=xp(ib)+0.5D0*dt*vx(ib)                                                           
       yp1(ib)=yp(ib)+0.5D0*dt*vy(ib)                                                           
       zp1(ib)=zp(ib)+0.5D0*dt*vz(ib)                                                           
    end do
    call interpo_l3_3d(vxg1,xp1,yp1,zp1,vx)       
    deallocate(xp1,yp1,zp1)
  end subroutine update_vx_2

  subroutine update_vy_2
    implicit none
    integer                  :: ib

    allocate (xp1(1:npart),yp1(1:npart),zp1(1:npart))           
    do ib=1,npart         
       xp1(ib)=xp(ib)-0.5D0*dt*vx(ib)                                                           
       yp1(ib)=yp(ib)+0.5D0*dt*vy(ib)                                                           
       zp1(ib)=zp(ib)+0.5D0*dt*vz(ib)                                                           
    end do
    call interpo_l3_3d(vyg1,xp1,yp1,zp1,vy)       
    deallocate(xp1,yp1,zp1)
  end subroutine update_vy_2

  subroutine update_vz_2
    implicit none
    integer                  :: ib

    allocate (xp1(1:npart),yp1(1:npart),zp1(1:npart))           
    do ib=1,npart         
       xp1(ib)=xp(ib)-0.5D0*dt*vx(ib)                                                           
       yp1(ib)=yp(ib)-0.5D0*dt*vy(ib)                                                           
       zp1(ib)=zp(ib)+0.5D0*dt*vz(ib)                                                           
    end do
    call interpo_l3_3d(vzg1,xp1,yp1,zp1,vz)       
    deallocate(xp1,yp1,zp1)
  end subroutine update_vz_2



  subroutine update_vx_3
    implicit none    
    integer                  :: ib


    allocate (xp1(1:npart),xp2(1:npart),yp1(1:npart),yp2(1:npart),zp1(1:npart),zp2(1:npart)) 
    allocate (vx1(1:npart),vx2(1:npart),vx3(1:npart),vx4(1:npart),vy1(1:npart),vz1(1:npart))           

    do ib=1,npart
       xp1(ib)=xp(ib)+2.D0*dt*vx(ib)/3.D0
       yp1(ib)=yp(ib)+2.D0*dt*vy(ib)/3.D0
       zp1(ib)=zp(ib)+2.D0*dt*vz(ib)/3.D0
    end do

    call interpo_l3_3d(vxg1,xp1,yp1,zp1,vx1)
    call interpo_l3_3d(vyg1,xp1,yp1,zp1,vy1)
    call interpo_l3_3d(vzg1,xp1,yp1,zp1,vz1)     

    do ib=1,npart         
       xp2(ib)=xp(ib)+dt*(-vx(ib)+vx1(ib))                                          
       yp2(ib)=yp(ib)+dt*(-vy(ib)+vy1(ib)) 
       zp2(ib)=zp(ib)+dt*(-vz(ib)+vz1(ib))                                          
    end do

    call interpo_l3_3d(vxg,xp2,yp,zp,vx2)
    call interpo_l3_3d(vxg,xp,yp2,zp,vx3) 
    call interpo_l3_3d(vxg,xp,yp,zp2,vx4)     

    do ib=1,npart                  
       vx(ib)=-0.5D0*vx(ib)+3.D0*vx1(ib)/4.D0+vx2(ib)/4.D0+vx3(ib)/4.D0+vx4(ib)/4.D0
    end do

    deallocate (xp1,xp2,yp1,yp2,zp1,zp2)  
    deallocate (vx1,vx2,vx3,vx4,vy1,vz1)


  end subroutine update_vx_3

  subroutine update_vy_3
    implicit none      
    integer                          :: ib,i,j

    allocate (xp0(1:npart),xp1(1:npart),xp2(1:npart),yp1(1:npart),yp2(1:npart),zp1(1:npart),zp2(1:npart))       
    allocate (vx1(1:npart),vz1(1:npart),vy1(1:npart),vy2(1:npart),vy3(1:npart),vy4(1:npart),vy5(1:npart))

    do ib=1,npart
       xp1(ib)=xp(ib)-dt*vx(ib)/3.D0
       yp1(ib)=yp(ib)+2.D0*dt*vy(ib)/3.D0
       zp1(ib)=zp(ib)+2.D0*dt*vz(ib)/3.D0
       xp0(ib)=xp(ib)-4.D0*dt*vx(ib)/3.D0
    end do

    call interpo_l3_3d(vxg1,xp1,yp1,zp1,vx1)
    call interpo_l3_3d(vyg1,xp1,yp1,zp1,vy1)
    call interpo_l3_3d(vyg1,xp0,yp1,zp1,vy2)
    call interpo_l3_3d(vzg1,xp0,yp1,zp1,vz1)

    do ib=1,npart         
       xp2(ib)=xp(ib)+dt*(vx(ib)-2.D0*vx1(ib))                                          
       yp2(ib)=yp(ib)+dt*(-vy(ib)+vy2(ib))  
       zp2(ib)=zp(ib)+dt*(-vz(ib)/4.D0+vz1(ib)/4.D0)                                          
    end do

    call interpo_l3_3d(vyg,xp2,yp,zp,vy3)
    call interpo_l3_3d(vyg,xp,yp2,zp,vy4)
    call interpo_l3_3d(vyg,xp,yp,zp2,vy5)

    do ib=1,npart                  
       vy(ib)=-5.D0*vy(ib)/4.D0+3.D0*vy1(ib)/4.D0+vy3(ib)/4.D0+vy4(ib)/4.D0+vy5(ib)
    end do


    deallocate (xp0,xp1,xp2,yp1,yp2,zp1,zp2)       
    deallocate (vx1,vz1,vy1,vy2,vy3,vy4,vy5)

  end subroutine update_vy_3

  subroutine update_vz_3
    implicit none      
    integer                          :: ib,i,j,k

    allocate (xp1(1:npart),xp2(1:npart),yp1(1:npart),yp2(1:npart),zp0(1:npart),zp1(1:npart),zp2(1:npart))       
    allocate (vx1(1:npart),vy1(1:npart),vz1(1:npart),vz2(1:npart),vz3(1:npart),vz4(1:npart))

    do ib=1,npart
       xp1(ib)=xp(ib)-2.D0*dt*vx(ib)/3.D0
       yp1(ib)=yp(ib)-2.D0*dt*vy(ib)/3.D0
       zp1(ib)=zp(ib)+dt*vz(ib)/3.D0
       zp0(ib)=zp(ib)+4.D0*dt*vz(ib)/3.D0
    end do

    call interpo_l3_3d(vzg1,xp1,yp1,zp1,vz1)
    call interpo_l3_3d(vxg2,xp1,yp1,zp0,vx1)
    call interpo_l3_3d(vyg2,xp1,yp1,zp0,vy1)

    do ib=1,npart         
       xp2(ib)=xp(ib)+dt*(-vx(ib)+vx1(ib))                                          
       yp2(ib)=yp(ib)+dt*(vy(ib)-vy1(ib))  
       zp2(ib)=zp(ib)+dt*(-vz(ib)+2.D0*vz1(ib))                                          
    end do

    call interpo_l3_3d(vzg,xp2,yp,zp,vz2)
    call interpo_l3_3d(vzg,xp,yp2,zp,vz3)
    call interpo_l3_3d(vzg3,xp,yp,zp2,vz4)

    do ib=1,npart                  
       vz(ib)=3.D0*vz1(ib)/4.D0-vz2(ib)/4.D0+vz3(ib)/4.D0+vz4(ib)/4.D0
    end do


    deallocate (xp1,xp2,yp1,yp2,zp0,zp1,zp2)       
    deallocate (vx1,vy1,vz1,vz2,vz3,vz4)

  end subroutine update_vz_3


     !==================================
     !STRANG
     !==================================

      subroutine update_vx_strang
       implicit none
       integer                  :: ib

       allocate (xp1(1:npart))           
       do ib=1,npart         
          xp1(ib)=xp(ib)+0.5D0*dt*vx(ib)
       end do
       call interpo_l3_3d(vxg1,xp1,yp,zp,vx)       
       deallocate(xp1)
     end subroutine update_vx_strang

     subroutine update_vy_strang
       implicit none
       integer                  :: ib

       allocate (yp1(1:npart))           
       do ib=1,npart         
          yp1(ib)=yp(ib)+0.5D0*dt*vy(ib)
       end do
       call interpo_l3_3d(vyg1,xp,yp1,zp,vy)       
       deallocate(yp1)
     end subroutine update_vy_strang

     subroutine update_vz_strang
       implicit none
       integer                  :: ib

       allocate (zp1(1:npart))           
       do ib=1,npart         
          zp1(ib)=zp(ib)+0.5D0*dt*vz(ib)
       end do
       call interpo_l3_3d(vzg1,xp,yp,zp1,vz)       
       deallocate(zp1)
     end subroutine update_vz_strang






end module advection_mod


