module drag_mod
  use donnees_mod
  use tab_mod
contains

  function cd(f,u,d)     
    implicit none      
    real(kind=8),intent(in)   :: f,u,d
    real(kind=8)              :: cd
    !calcul du coeff de traine
    !f=force scalaire x
    !d=diametre (surface projette)
    !u vitesse a l'infini
    cd=2.*f/(u*u*d)    
  end function cd

  function cl(f,u,d)    
    implicit none      
    real(kind=8),intent(in)   :: f,u,d
    real(kind=8)              :: cl
    !calcul du coeff de portance
    !f=force scalaire x
    !d=diametre (surface projette)
    !u vitesse a l'infini
    cl=2.*f/(u*u*d)
  end function cl













!====================================
!methode de l'impulsion
!=====================================

  function ix()       
    implicit none
    real(kind=8)     :: ix
    integer          :: i,j,k
    real(kind=8)     :: x,y,z,dvx,dvy,dvz
    ix=0.
    do i=1,nx       
       if ((i==1).or.(i==nx)) then
          dvx=0.5*dx
       else
          dvx=dx
       end if
       do j=1,ny 
          if ((j==1).or.(j==ny)) then
             dvy=0.5*dy
          else
             dvy=dy
          end if
          y=yb+(j-1)*dy
          do k=1,nz
             if ((k==1).or.(k==nz)) then
                dvz=0.5*dz
             else
                dvz=dz
             end if
             z=zd+(k-1)*dz           

             ix =ix-0.5*(y*omgz(i,j,k)-z*omgy(i,j,k))*dvx*dvy*dvz   
          end do
       end do
    end do

  end function ix

  function drag_tps()
    implicit none

    real(kind=8)             :: drag_tps
    real(kind=8)             :: f,ix_new
    real(kind=8)             :: rayon,uinf

    rayon=0.5
    uinf=1.

    !calcul du drag coeff pour t= time-dt
    if(ind_drag==1)then
       ix_2old=ix()
       drag_tps=0.
    else
       if (ind_drag==2) then         
          ix_old=ix()
          drag_tps=0.
       else
          ix_new=ix()
          f=(ix_new-ix_2old)/(2.*dt)
          ix_2old=ix_old
          ix_old=ix_new
          drag_tps=cd(f,uinf,2.*rayon)          

       end if
    end if

  end function drag_tps


  function ix_lift()result(ix)       
    implicit none
    real(kind=8)     :: ix
    integer          :: i,j,k
    real(kind=8)     :: x,y,z,dvx,dvy,dvz
    ix=0.
    do i=1,nx
       x=xg+(i-1)*dx
       if ((i==1).or.(i==nx)) then
          dvx=0.5*dx
       else
          dvx=dx
       end if
       do j=1,ny 
          if ((j==1).or.(j==ny)) then
             dvy=0.5*dy
          else
             dvy=dy
          end if
          do k=1,nz
             if ((k==1).or.(k==nz)) then
                dvz=0.5*dz
             else
                dvz=dz
             end if
             z=zd+(k-1)*dz

             ix =ix-0.5*(z*omgx(i,j,k)-x*omgz(i,j,k))*dvx*dvy*dvz   
          end do
       end do
    end do

  end function ix_lift

  function lift_tps()     
    implicit none

    real(kind=8)             :: lift_tps
    real(kind=8)             :: f,ix_lift_new
    real(kind=8)             :: rayon,uinf

    rayon=0.5
    uinf=1.

    !calcul du lift coeff pour t= time-dt
    if(ind_drag==1)then
       ix_lift_2old=ix_lift()
       lift_tps=0.
    else
       if (ind_drag==2) then         
          ix_lift_old=ix_lift()
          lift_tps=0.
       else
          ix_lift_new=ix_lift()
          f=(ix_lift_new-ix_lift_2old)/(2.*dt)
          ix_lift_2old=ix_lift_old
          ix_lift_old=ix_lift_new
          lift_tps=cl(f,uinf,2.*rayon)          

       end if
    end if

  end function lift_tps





  
  
  !========================================
  !methode de l'écoulement poreux
  !===========================================
  
  function drag_poreux()  result(drag_po)
    implicit none
    real(kind=8)     :: drag_po
    integer          :: i,j,k
    real(kind=8)     :: dvx,dvy,dvz
    real(kind=8)     :: rayon,uinf

    rayon=0.5
    uinf=1.


    drag_po=0.


    do i=1,nx
       if ((i==1).or.(i==nx)) then
          dvx=0.5*dx
       else
          dvx=dx
       end if
       do j=1,ny 
          if ((j==1).or.(j==ny)) then
             dvy=0.5*dy
          else
             dvy=dy
          end if
          do k=1,nz
             if ((k==1).or.(k==nz)) then
                dvz=0.5*dz
             else
                dvz=dz
             end if
             drag_po =drag_po+lambda(i,j,k)*chi_sphere(i,j,k)*vxg(i,j,k)*dvx*dvy*dvz   
          end do
       end do
    end do


    drag_po=cd(drag_po,uinf,2.*rayon)

  end function drag_poreux

 function lift_poreux()  result(lift_po)
    implicit none
    real(kind=8)     :: lift_po
    integer          :: i,j,k
    real(kind=8)     :: dvx,dvy,dvz
    real(kind=8)     :: rayon,uinf

    rayon=0.5
    uinf=1.


    lift_po=0.


    do i=1,nx
       if ((i==1).or.(i==nx)) then
          dvx=0.5*dx
       else
          dvx=dx
       end if
       do j=1,ny 
          if ((j==1).or.(j==ny)) then
             dvy=0.5*dy
          else
             dvy=dy
          end if
          do k=1,nz
             if ((k==1).or.(k==nz)) then
                dvz=0.5*dz
             else
                dvz=dz
             end if
             lift_po =lift_po+lambda(i,j,k)*chi_sphere(i,j,k)*vyg(i,j,k)*dvx*dvy*dvz   
          end do
       end do
    end do


    lift_po=cd(lift_po,uinf,2.*rayon)

  end function lift_poreux











end module drag_mod


