module interpolation_mod
  use donnees_mod
  contains

    subroutine  interpo_l3_3d(tab_grille,posx,posy,posz,tab_part)       
      implicit none

      real(kind=8),dimension(:,:,:),intent(in)      :: tab_grille
      real(kind=8),dimension(:),intent(in)          :: posx,posy,posz
      real(kind=8),dimension(1:npart),intent(out)   :: tab_part
      integer                                       :: i,c,d,e,ib,jb
      integer,dimension(0:3)                        :: ip,jp,kp
      real(kind=8),dimension(0:3)                   :: poidx,poidy,poidz
      real(kind=8)                                  :: xx1,yy1,zz1
      
      tab_part(1:npart)=0.      

      !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
      !---------------------------------------------------------------------------------------------------------

      do i=1,npart

         !numero des points sur le maillage
         !--------------------------------
         ip(1) = floor((posx(i)-xg)/dx_gro)         
         ip(0) = ip(1) - 1        
         ip(2) = ip(1) + 1         
         ip(3) = ip(1) + 2

         jp(1) = floor((posy(i)-yb)/dy_gro)         
         jp(0) = jp(1) - 1        
         jp(2) = jp(1) + 1         
         jp(3) = jp(1) + 2

         kp(1) = floor((posz(i)-zd)/dz_gro)         
         kp(0) = kp(1) - 1        
         kp(2) = kp(1) + 1         
         kp(3) = kp(1) + 2
                   

         !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
         !----------------------------------------------------         
          xx1 = (posx(i) - real(ip(1),kind=8)*dx_gro-xg)/dx_gro   
          yy1 = (posy(i) - real(jp(1),kind=8)*dy_gro-yb)/dy_gro   
          zz1 = (posz(i) - real(kp(1),kind=8)*dz_gro-zd)/dz_gro   

          !conditions au bord
          !------------------
          !periodique: 
         
             do c=0,3               
                ip(c)=mod(ip(c)+nx_gro,nx_gro)                
             end do
         
             do c=0,3               
                jp(c)=mod(jp(c)+ny_gro,ny_gro)                
             end do
         
             do c=0,3               
                kp(c)=mod(kp(c)+nz_gro,nz_gro)                
             end do
        
         
         !calcul des poids
         !----------------
         poidx(0)=-1./6.*xx1*(xx1-1.)*(xx1-2.)
         poidx(1)=0.5*(1.-xx1)*(1.+xx1)*(2.-xx1)
         poidx(2)=-0.5*xx1*(xx1+1.)*(xx1-2.)
         poidx(3)=1/6.*xx1*(1.+xx1)*(xx1-1.)

         poidy(0)=-1./6.*yy1*(yy1-1.)*(yy1-2.)
         poidy(1)=0.5*(1.-yy1)*(1.+yy1)*(2.-yy1)
         poidy(2)=-0.5*yy1*(yy1+1.)*(yy1-2.)
         poidy(3)=1/6.*yy1*(1.+yy1)*(yy1-1.)

         poidz(0)=-1./6.*zz1*(zz1-1.)*(zz1-2.)
         poidz(1)=0.5*(1.-zz1)*(1.+zz1)*(2.-zz1)
         poidz(2)=-0.5*zz1*(zz1+1.)*(zz1-2.)
         poidz(3)=1/6.*zz1*(1.+zz1)*(zz1-1.)

         !remaillage à l' interrieur domaine
         !---------------------------------         
         do e=0,3
            do d=0,3
               do c=0,3                
                  tab_part(i)=tab_part(i)+tab_grille(ip(c)+1,jp(d)+1,kp(e)+1)*poidx(c)*poidy(d)*poidz(e)    
               end do
            end do
         end do
      end do
    end subroutine interpo_l3_3d







    !===========================================================================
    !pour interpoler le champ de vitesse de la grille grossiere à la grille fine
    !===========================================================================


    function interpol_v_l1(tab_gro,posx,posy,posz) result (v)
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:,:),intent(in)      :: tab_gro
      real(kind=8),intent(in)                       :: posx,posy,posz
      real(kind=8)                                  :: v
      integer,dimension(0:1)                        :: ip,jp,kp
      real(kind=8),dimension(0:1)                   :: poidx,poidy,poidz
      real(kind=8)                                  :: xx,yy,zz
      integer                                       :: c,d,e

      v=0.

         !numero des points sur le maillage
         !--------------------------------
         ip(0) = floor((posx-xg)/dx_gro)       !de 0 à nx_gro -1
         ip(1) = ip(0)+1        
               
         jp(0) = floor((posy-yb)/dy_gro)       
         jp(1) = jp(0)+1 

         kp(0) = floor((posz-zd)/dz_gro)       
         kp(1) = kp(0)+1 
                 
   
         !distance de la particule à remailler au premier point gauche
         !----------------------------------------------------------- 
          xx = (posx - real(ip(0),kind=8)*dx_gro-xg)/dx_gro   
          yy = (posy - real(jp(0),kind=8)*dy_gro-yb)/dy_gro 
          zz = (posz - real(kp(0),kind=8)*dz_gro-zd)/dz_gro   

          !conditions au bord
          !------------------
          !periodique:       
          do c=0,1            
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=0,1               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do

          do c=0,1               
             kp(c)=mod(kp(c)+nz_gro,nz_gro)                
          end do
         
         !calcul des poids
         !----------------        
         poidx(0)=1.-xx
         poidx(1)=xx               
        
         poidy(0)=1.-yy
         poidy(1)=yy        
         
         poidz(0)=1.-zz
         poidz(1)=zz        
         
         !remaillage à l' interrieur domaine
         !---------------------------------         
         do e=0,1
            do c=0,1
               do d=0,1               
                  v=v+tab_gro(ip(c)+1,jp(d)+1,kp(e)+1)*poidx(c)*poidy(d)*poidz(e)
               end do
            end do
         end do

    end function interpol_v_l1
   
    function interpol_v_l3(tab_gro,posx,posy,posz) result (v)
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:,:),intent(in)      :: tab_gro
      real(kind=8),intent(in)                       :: posx,posy,posz
      real(kind=8)                                  :: v
      integer,dimension(-1:2)                       :: ip,jp,kp
      real(kind=8),dimension(-1:2)                  :: poidx,poidy,poidz
      real(kind=8)                                  :: xx,yy,zz
      integer                                       :: c,d,e

      v=0.

         !numero des points sur le maillage
         !--------------------------------
         ip(0) = floor((posx-xg)/dx_gro)       !de 0 à nx_gro -1
         ip(-1) = ip(0)-1
         ip(1) = ip(0)+1
         ip(2) = ip(0)+2        
               
         jp(0) = floor((posy-yb)/dy_gro)       
         jp(-1) = jp(0)-1 
         jp(1) = jp(0)+1 
         jp(2) = jp(0)+2 

         kp(0) = floor((posz-zd)/dz_gro)       
         kp(-1) = kp(0)-1 
         kp(1) = kp(0)+1 
         kp(2) = kp(0)+2 
                 
                  
         !distance de la particule à remailler au premier point gauche
         !----------------------------------------------------------- 
          xx = (posx - real(ip(0),kind=8)*dx_gro-xg)/dx_gro   
          yy = (posy - real(jp(0),kind=8)*dy_gro-yb)/dy_gro 
          zz = (posz - real(kp(0),kind=8)*dz_gro-zd)/dz_gro   

          !conditions au bord
          !------------------
          !periodique:       
          do c=-1,2            
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=-1,2               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do

          do c=-1,2               
             kp(c)=mod(kp(c)+nz_gro,nz_gro)                
          end do
 
         !calcul des poids
         !----------------        
          poidx(-1)=-1./6.*xx*(xx-1.)*(xx-2.)
          poidx(0)=0.5*(1.-xx)*(1.+xx)*(2.-xx)
          poidx(1)=-0.5*xx*(xx+1.)*(xx-2.)
          poidx(2)=1/6.*xx*(1.+xx)*(xx-1.)
          
          poidy(-1)=-1./6.*yy*(yy-1.)*(yy-2.)
          poidy(0)=0.5*(1.-yy)*(1.+yy)*(2.-yy)
          poidy(1)=-0.5*yy*(yy+1.)*(yy-2.)
          poidy(2)=1/6.*yy*(1.+yy)*(yy-1.) 

          poidz(-1)=-1./6.*zz*(zz-1.)*(zz-2.)
          poidz(0)=0.5*(1.-zz)*(1.+zz)*(2.-zz)
          poidz(1)=-0.5*zz*(zz+1.)*(zz-2.)
          poidz(2)=1/6.*zz*(1.+zz)*(zz-1.) 
        
         
         !remaillage à l' interrieur domaine
         !---------------------------------         
          do e=-1,2
             do c=-1,2
                do d=-1,2               
                   v=v+tab_gro(ip(c)+1,jp(d)+1,kp(e)+1)*poidx(c)*poidy(d)*poidy(e)
                end do
             end do
          end do

    end function interpol_v_l3

    function interpol_v_l5(tab_gro,posx,posy,posz) result (v)
      use tab_mod
      implicit none

      real(kind=8),dimension(:,:,:),intent(in)      :: tab_gro
      real(kind=8),intent(in)                       :: posx,posy,posz
      real(kind=8)                                  :: v
      integer,dimension(-2:3)                       :: ip,jp,kp
      real(kind=8),dimension(-2:3)                  :: poidx,poidy,poidz
      real(kind=8)                                  :: xx,yy,zz
      integer                                       :: c,d,e
      real(kind=8)                                  :: x2,x3,x4,x5,y2,y3,y4,y5

      v=0.

         !numero des points sur le maillage
         !--------------------------------
         ip(0) = floor((posx-xg)/dx_gro)      !de 0 à nx_gro -1
         ip(-2) = ip(0)-2
         ip(-1) = ip(0)-1
         ip(1) = ip(0)+1
         ip(2) = ip(0)+2  
         ip(3) = ip(0)+3       
               
         jp(0) = floor((posy-yb)/dy_gro)       
         jp(-2) = jp(0)-2
         jp(-1) = jp(0)-1 
         jp(1) = jp(0)+1 
         jp(2) = jp(0)+2 
         jp(3) = jp(0)+3 

         kp(0) = floor((posz-zd)/dz_gro)       
         kp(-2) = kp(0)-2
         kp(-1) = kp(0)-1 
         kp(1) = kp(0)+1 
         kp(2) = kp(0)+2 
         kp(3) = kp(0)+3 
                 
                  
         !distance de la particule à remailler au premier point gauche
         !----------------------------------------------------------- 
          xx = (posx - real(ip(0),kind=8)*dx_gro-xg)/dx_gro   
          yy = (posy - real(jp(0),kind=8)*dy_gro-yb)/dy_gro 
          zz = (posz - real(kp(0),kind=8)*dz_gro-zd)/dz_gro   

          !conditions au bord
          !------------------
          !periodique:       
          do c=-2,3            
             ip(c)=mod(ip(c)+nx_gro,nx_gro)                
          end do

          do c=-2,3               
             jp(c)=mod(jp(c)+ny_gro,ny_gro)                
          end do

          do c=-2,3               
             kp(c)=mod(kp(c)+nz_gro,nz_gro)                
          end do
         
         !calcul des poids
         !----------------        
          x2=xx**2
          x3=xx**3
          x4=xx**4
          x5=xx**5
          y2=yy**2
          y3=yy**3
          y4=yy**4
          y5=yy**5

          poidx(-2)=xx/20.-x2/24.-x3/24.+x4/24.-x5/120.
          poidx(-1)=-xx/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
          poidx(0)=1.-xx/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
          poidx(1)=xx+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
          poidx(2)=-xx/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
          poidx(3)=xx/30.-x3/24.+x5/120.

          poidy(-2)=yy/20.-y2/24.-y3/24.+y4/24.-y5/120.
          poidy(-1)=-yy/2.+2.*y2/3.-y3/24.-y4/6.+y5/24.
          poidy(0)=1.-yy/3.-5*y2/4.+5.*y3/12.+y4/4.-y5/12.
          poidy(1)=yy+2.*y2/3.-7.*y3/12.-y4/6.+y5/12.
          poidy(2)=-yy/4.-y2/24.+7.*y3/24.+y4/24.-y5/24.
          poidy(3)=yy/30.-y3/24.+y5/120.

          poidz(-2)=zz/20.-y2/24.-y3/24.+y4/24.-y5/120.
          poidz(-1)=-zz/2.+2.*y2/3.-y3/24.-y4/6.+y5/24.
          poidz(0)=1.-zz/3.-5*y2/4.+5.*y3/12.+y4/4.-y5/12.
          poidz(1)=zz+2.*y2/3.-7.*y3/12.-y4/6.+y5/12.
          poidz(2)=-zz/4.-y2/24.+7.*y3/24.+y4/24.-y5/24.
          poidz(3)=zz/30.-y3/24.+y5/120.
        
         
         !remaillage à l' interrieur domaine
         !---------------------------------         
          do e=-2,3
             do c=-2,3
                do d=-2,3               
                   v=v+tab_gro(ip(c)+1,jp(d)+1,kp(e)+1)*poidx(c)*poidy(d)*poidz(e)
                end do
             end do
          end do

    end function interpol_v_l5
   


   
  end module interpolation_mod
