program NS
  !--------------------------------------------------------------------------
  !resolution Navier Stokes + penalisation en periodoque
  !
  !--------------------------------------------------------------------------
  use donnees_mod         ! donnees 
  use tab_mod             ! donnees dans tableaux 
  use init_mod            ! init_grille
  use advection_mod       ! crea_part,advection
  use remaillage_mod      ! remaill_4m_centre (formules de remaillage)
  use interpolation_mod   ! formules d'interpolation (pour range kutta)
  use resultats_mod       ! res_grille_tps,res_grille_freq
  use utile_mod  
  use penal_mod
  use source_mod          ! diffusion + strech (+ penal)
  use drag_mod            !calcul drag/lift
  use old_mod             ! ancien algo

  real(kind=8)        :: x,y,z 
  integer             :: n ,indi1,indi2,indj,indk
  real(kind=8)        :: time1,time2,time3,time4,maxv,dragi,dragp,lifti,liftp

  !pour test
  integer             :: c,i,j,ib,jb,k,cpt,nx_boucle
  real(kind=8)        :: t1,t2,t3,t4,t5,t6,t7,t8,t9,tmp,dt_boucle,r,grad,div,vexa,omexa
  real(kind=8),dimension(1:3)   :: pen
 
  !lecture données
  !---------------
  open(unit=10,file="parameter",form="formatted")
  read(10,*) xg,xd,yb,yh,zd,zf 
  read(10,*) nx,ny,nz 
  read(10,*) tfin 
  read(10,*) cutoff
  read(10,*) type_b
  read(10,*) long_bloc      
  read(10,*) nom_fich_vtk
  read(10,*) lambda_flu,lambda_sol,lambda_por
  read(10,*) nu
  read(10,*) nom_fich_omg_nite,nom_fich_vit_nite,nom_fich_chi,nom_fich_lambda,nite_vtk
  read(10,*) nom_fich_coupev,nom_fich_drag
  read(10,*) num_suite,sauv_cpt_ite,name_relance
  close(10)


  nx_gro=nx
  ny_gro=nx_gro
  nz_gro=nx_gro

  dx=(xd-xg)/(nx-1.)
  dy=(yh-yb)/(ny-1.)
  dz=(zf-zd)/(nz-1.)

  dx_gro=(xd-xg)/(nx_gro-1.)
  dy_gro=(yh-yb)/(ny_gro-1.)
  dz_gro=(zf-zd)/(nz_gro-1.)



  !maxv=1.D0
  !maxv=2.D0
  !cfl=0.4D0


  !dt=cfl*dx/maxv
  !dt=0.0125
  !dt=0.001
  !dt=min(cfl*dx/maxv,cfl pour diffusion)
  dt=0.005
  !dt=0.01
  !dt=3./8.*dx**2/nu

  print*,"dt",dt

  !
  !calcul init
  !-----------
  cpt_ite=0
  time=0.D0
  ind_drag=0
  if (num_suite==1) then
     cpt_ite=sauv_cpt_ite
     time=cpt_ite*dt
  end if

  npart=nx*ny*nz

  nx=nx-1
  ny=ny-1
  nz=nz-1

  nx_gro=nx_gro-1
  ny_gro=ny_gro-1
  nz_gro=nz_gro-1


  PI=3.141592653589793238462643383279502884197D0


  !
  !alloc tableaux
  !--------------
  allocate (vxg(1:nx_gro,1:ny_gro,1:nz_gro),vyg(1:nx_gro,1:ny_gro,1:nz_gro),vzg(1:nx_gro,1:ny_gro,1:nz_gro))
  allocate (omgx(1:nx,1:ny,1:nz),omgy(1:nx,1:ny,1:nz),omgz(1:nx,1:ny,1:nz))
  allocate (xp(1:npart),qp(1:npart),vx(1:npart),yp(1:npart),vy(1:npart),zp(1:npart),vz(1:npart))       
  allocate (omx(1:npart),omy(1:npart),omz(1:npart))
  allocate (xtab(1:nx),ytab(1:ny),ztab(1:nz))
  allocate (chi(1:nx,1:ny,1:nz),chi_sphere(1:nx,1:ny,1:nz),lambda(1:nx,1:ny,1:nz))

  !pour la correction de consistance
  allocate (numpg(1:nx,1:ny,1:nz)) 
  allocate (blocg(0:npart),blocd(0:npart),Nbloc(0:npart)) 


  allocate (deb1(1:nx,1:ny,1:nz),deb2(1:nx,1:ny,1:nz),deb3(1:nx,1:ny,1:nz))

  allocate(stxg(1:nx,1:ny,1:nz),styg(1:nx,1:ny,1:nz),stzg(1:nx,1:ny,1:nz))
  allocate(stx(1:npart),sty(1:npart),stz(1:npart))

  open(unit=12,file=trim(nom_fich_drag),form="formatted")
  open(unit=13,file=trim(nom_fich_coupev),form="formatted")
  open(unit=14,file="RES/test/fich1.deb",form="formatted")
  open(unit=15,file="RES/test/fich2.deb",form="formatted")
  open(unit=16,file="RES/test/fich3.deb",form="formatted")
  open(unit=17,file="RES/test/fich4.deb",form="formatted")
  open(unit=18,file="RES/test/fich5.deb",form="formatted")
  open(unit=19,file="RES/test/fich6.deb",form="formatted")
  open(unit=20,file="RES/test/fich7.deb",form="formatted")




  !=================
  !initialisations
  !=================
  call make_grille
  call make_chi
  call init_grille()

  call res_vtk_chi(nom_fich_chi)
  call res_vtk_lambda(nom_fich_lambda)
  !============================================================
  !test: v->w retrouve par fft
  !
  !si fonction en sin avec div=0 => erreur de 10-8
  ! si "        "  "    "  div/=0 => erreur elevée
  ! doit regulariser et avoir u1diff(chi,x1)+ u2diff(chi,x2)+ u3diff(chi,x3)=0
  ! la vitesse apres penalisatin n'est pas à div nulle
  ! mais celle calcule apres (a partir de la vorticite) est à div nulle
  !=============================================================
!!$   do k=1,nz
!!$      z=ztab(k)
!!$        do j=1,ny
!!$           y=ytab(j)
!!$           do i=1,nx
!!$              x=xtab(i)
!!$              !vxg(i,j,k)=(1.-(x**2+y**2+z**2))**6
!!$              !if ( (x**2+y**2+z**2)>1. ) vxg(i,j,k)=0.
!!$              !vxg(i,j,k)=cos(2.*Pi*x)!divu/=0
!!$              vxg(i,j,k)=(1.-chi(i,j,k))*y
!!$              !if ( (x**2+y**2+z**2)<0.5 ) vxg(i,j,k)=0.
!!$              vyg(i,j,k)=(1.-chi(i,j,k))*(-x)
!!$              vzg(i,j,k)=0.
!!$
!!$
!!$
!!$              !vxg(i,j,k)=2.*sin(pi*x)**2*sin(2.*pi*y)*sin(2.*pi*z) 
!!$              !vyg(i,j,k)=-sin(2.*pi*x)*sin(pi*y)**2*sin(2.*pi*z) 
!!$              !vzg(i,j,k)=-sin(2.*pi*x)*sin(2.*pi*y)*sin(pi*z)**2 
!!$           
!!$           end do
!!$        end do
!!$      end do
!!$      call res_vtk_omega(trim("RES/vtk/test_omg1.vtk"))
!!$      call res_vtk_vit(trim("RES/vtk/test_vg1.vtk"))
!!$     deb1=vxg
!!$     deb2=vyg
!!$     deb3=vzg
!!$
!!$     call penal_fft  ! calcul w=rot(v) par fft
!!$    
!!$
!!$
!!$  call res_vtk_omega(trim("RES/vtk/test_omg2.vtk"))
!!$
!!$  call vitesse_fft_uinf
!!$  !call vitesse_fft_uavt
!!$  call res_vtk_vit(trim("RES/vtk/test_vg2.vtk"))
!!$
!!$  print*,"erreur"
!!$  print*,maxval(abs(deb1-vxg)),maxval(abs(deb2-vyg)),maxval(abs(deb3-vzg))
!!$  print*,"dx",dx
!!$  stop

 

  !===================
  !iterations en temps
  !===================

  temps: do while(time<tfin)

     if ((time+dt)>tfin) dt=tfin-time

     !=======================
     !calcul champ de vitesse
     !========================
     !call def_v_advec (vxg,vyg,vzg,time)    
     !call vitesse_fft
     call cpu_time(t1)
     call vitesse_fft_uinf
     call cpu_time(t2)

 
     !call imposer_debit(vxg,4.d0*2.06441*2.06441,nx,ny,nz,dx,dy,dz)    

     !call imposer_debit(vxg,1.d0,nx,ny,nz,dx,dy,dz)    
     !vxg=vxg+1./(4.*1.*1.5) !debit=debit voulu +c*surface du domaine num, c etant la constante à ajouter au champ de vitesse.
    
     !test la valeur du débit
     !-----------------------
!!$     t4=0.
!!$     do k=1,nz
!!$        do j=1,ny
!!$              t4=t4+vxg(1,j,k)
!!$        end do
!!$     end do
!!$     print*, "debit i=1 ",t4*dy*dz
!!$     print*,"bulk velocity",t4*dy*dz/(4.*(1./0.4854)**2)
    

!-------------------------test calcul vitesse avec omg exact------------------------------------------------------
!!$     indi1=1
!!$     indi2=nx/2
!!$     indk1=1
!!$     indk2=nz/2
!!$    
!!$        do indj=1,ny
!!$           if ( ((yb+(indj-1)*dy).ge.-(1./0.4844)).and.((yb+(indj-1)*dy).le.(1./0.4844))) then             
!!$              vexa=1.5*(1.-(yb+(indj-1)*dy)**2/(1./0.4844)**2)
!!$              omexa=3.*(yb+(indj-1)*dy)/((1./0.4844)**2)
!!$           else
!!$              vexa=0.
!!$              omexa=0.
!!$           end if    
!!$           write(14,'(8(e25.15,2x))') time,yb+(indj-1)*dy,vxg(indi1,indj,indk1),omgz(indi1,indj,indk1),vexa,omexa,vxg(indi2,indj,indk2),omgz(indi2,indj,indk2)
!!$        end do    
!!$!stop
!------------------------------------------------------------------------------------------------------

   

     !penalisation: nouveau calcul de v
     !---------------------------------
     do k=1,nz
        do j=1,ny
           do i=1,nx
             
              !tmp=exp(-lambda*dt*chi(i,j,k))
              tmp=1./(1.+lambda(i,j,k)*dt*chi(i,j,k))
              vxg(i,j,k)=vxg(i,j,k)*tmp
              vyg(i,j,k)=vyg(i,j,k)*tmp
              vzg(i,j,k)=vzg(i,j,k)*tmp
              
           end do
        end do
     end do

     call cpu_time(t3)

     !resultats
     !---------

     !-----------------(poiseuil en y)---------------------------------------
!!$     call res_vtk_nite(nom_fich_omg_nite,nom_fich_vit_nite,nite_vtk,time)
!!$     indi1=-xg/dx+1
!!$     indi2=(0.7-xg)/dx+1
!!$     indj=-yb/dy+1
!!$     if (mod(cpt_ite,5)==0)  then
!!$        do indk=1,nz
!!$           if ( ((zd+(indk-1)*dz).ge.-1.).and.((zd+(indk-1)*dz).le.1.)) then
!!$              vexa=1.5*(1.-(zd+(indk-1)*dz)**2)/(4.*0.5*abs(yh-yb))
!!$              omexa=-3.*(zd+(indk-1)*dz)/(4.*0.5*abs(yh-yb))
!!$           else
!!$              vexa=0.
!!$              omexa=0.
!!$           end if
!!$           write(13,'(8(e25.15,2x))') time,zd+(indk-1)*dz,vxg(indi1,indj,indk),omgy(indi1,indj,indk),vexa,omexa,vxg(indi2,indj,indk),omgy(indi2,indj,indk)
!!$        end do
!!$     end if
     !---------------------------------------------------------------------------


     !-----------------(poiseuil en z)---------------------------------------   
!     call res_vtk_nite(nom_fich_omg_nite,nom_fich_vit_nite,nite_vtk,time)
     indi1=1
     indi2=nx/2
     indk1=1
     indk2=nz/2
     if (mod(cpt_ite,20)==0)  then
        do indj=1,ny
           if ( ((yb+(indj-1)*dy).ge.-(1./0.4844)).and.((yb+(indj-1)*dy).le.(1./0.4844))) then             
              vexa=1.5*(1.-(yb+(indj-1)*dy)**2/(1./0.4844)**2)
              omexa=3.*(yb+(indj-1)*dy)/((1./0.4844)**2)
           else
              vexa=0.
              omexa=0.
           end if    
           write(13,'(8(e25.15,2x))') time,yb+(indj-1)*dy,vxg(indi1,indj,indk1), &
	omgz(indi1,indj,indk1),vexa,omexa,vxg(indi2,indj,indk2),omgz(indi2,indj,indk2)
        end do  
     end if
     !---------------------------------------------------------------------------


     !================================================================================================
     !algo penalisation "marche":
     !
     !penalisation:
     !-------------
     !si explicite: lambda<2/dt, mais drag poreux marche car vitesse dans obstacle de l'ordre de lambda
     !si implicite: 1/peut pas mettre en terme source et 2/grandes vitesses dans l obstacle
     !              (la formule developpe n'a pas l'air de marcher)
     !==================================================================================================    
    
     !call penal_exacte()        
     !call strech_old
     !call diffusion_old()   


     !===================================================================
     !calcul du drag :      !vitesse penalisee faible dans l'obstacle !
     !====================================================================
     dragi=drag_tps()
     dragp=drag_poreux()
     lifti=lift_tps()
     liftp=lift_poreux()       
     if (time>dt) then
        print*,"time",time
	write(12,'(6(e25.15,2x))') time,time-dt,dragi,dragp,lifti,liftp
     end if
     if (ind_drag<5) ind_drag=ind_drag+1

     !================================
     !algo sans splitting
     !================================

     !call penal_fft !calcul omg=rot(v)
     call rotv_df4
     !call rotv_df2

     call strech_diff_penal()  ! penal commentee

     call crea_part_old()   

     call advection_old() !RK2 

     call cpu_time(t4)

     call remaill_l4(omx,xp,yp,zp,omgx)
     call remaill_l4(omy,xp,yp,zp,omgy)
     call remaill_l4(omz,xp,yp,zp,omgz)

     call cpu_time(t5)

 
!!$     !======================================
!!$     !splitting en espace : ordre2 en temps
!!$     !======================================  
!!$     call penal_exacte()
!!$
!!$
!!$     call crea_part_x()   
!!$  
!!$     allocate(vxg1(1:nx_gro,1:ny_gro,1:nz_gro),vyg1(1:nx_gro,1:ny_gro,1:nz_gro),vzg1(1:nx_gro,1:ny_gro,1:nz_gro))
!!$     !call def_v_advec (vxg1,vyg1,vzg1,time+0.5*dt)
!!$     vxg1=vxg
!!$     vyg1=vyg
!!$     vzg1=vzg
!!$
!!$     !t source
!!$     !---------
!!$     call source_split_1       
!!$
!!$     !update v
!!$     !--------
!!$     call update_vx_2
!!$
!!$     !test cfl
!!$     !--------
!!$     !grad=0.   
!!$     !do k=1,nz
!!$     !   do j=1,ny
!!$     !      do i=1,nx-1
!!$     !         grad=max(grad,abs(vx(numpg(i+1,j,k))-vx(numpg(i,j,k))))    
!!$     !         !grad=max(grad,abs(vxg(i+1,j,k)-vxg(i,j,k)))        
!!$     !      end do
!!$     !   end do
!!$     !end do          
!!$     !print*,dt,dx/(4.*grad),dx/(6.*grad)
!!$
!!$
!!$     !bloc x
!!$     !-------
!!$     !call make_bloc(1)
!!$     blocg=0
!!$     blocd=0
!!$
!!$     !advection
!!$     !----------
!!$     call ad_euler_x 
!!$
!!$                
!!$
!!$     !remaillage
!!$     !----------       
!!$     !call remaill_l6_x(omx,xp,yp,zp,omgx)
!!$     !call remaill_l6_x(omy,xp,yp,zp,omgy)
!!$     !call remaill_l6_x(omz,xp,yp,zp,omgz)
!!$     if (type_b==2) call remaill_l2_bloc_x(omx,xp,yp,zp,omgx) 
!!$     if (type_b==2) call remaill_l2_bloc_x(omy,xp,yp,zp,omgy) 
!!$     if (type_b==2) call remaill_l2_bloc_x(omz,xp,yp,zp,omgz) 
!!$     if (type_b==4) call remaill_l4_bloc_x(omx,xp,yp,zp,omgx) 
!!$     if (type_b==4) call remaill_l4_bloc_x(omy,xp,yp,zp,omgy) 
!!$     if (type_b==4) call remaill_l4_bloc_x(omz,xp,yp,zp,omgz) 
!!$    
!!$
!!$     !y
!!$     !--
!!$     call crea_part_y()
!!$
!!$     !update v
!!$     !--------
!!$     call update_vy_2
!!$
!!$     !bloc y
!!$     !-------              
!!$     !call make_bloc(2)
!!$     blocg=0
!!$     blocd=0
!!$
!!$     !advection
!!$     !----------
!!$     call ad_euler_y          
!!$
!!$     !remaillage
!!$     !----------
!!$     !call remaill_l6_y(omx,xp,yp,zp,omgx) 
!!$     !call remaill_l6_y(omy,xp,yp,zp,omgy) 
!!$     !call remaill_l6_y(omz,xp,yp,zp,omgz) 
!!$     if (type_b==2)call remaill_l2_bloc_y(omx,xp,yp,zp,omgx)        
!!$     if (type_b==2)call remaill_l2_bloc_y(omy,xp,yp,zp,omgy)        
!!$     if (type_b==2)call remaill_l2_bloc_y(omz,xp,yp,zp,omgz)        
!!$     if (type_b==4)call remaill_l4_bloc_y(omx,xp,yp,zp,omgx)        
!!$     if (type_b==4)call remaill_l4_bloc_y(omy,xp,yp,zp,omgy)        
!!$     if (type_b==4)call remaill_l4_bloc_y(omz,xp,yp,zp,omgz)        
!!$
!!$
!!$     !z
!!$     !--
!!$     call crea_part_z()
!!$
!!$     !update v
!!$     !--------
!!$     call update_vz_2
!!$
!!$
!!$     !bloc z
!!$     !-------      
!!$     !call make_bloc(3)
!!$     blocg=0
!!$     blocd=0
!!$
!!$     !advection
!!$     !----------
!!$     call ad_euler_z         
!!$
!!$     !remaillage
!!$     !----------   
!!$     !call remaill_l6_z(omx,xp,yp,zp,omgx) 
!!$     !call remaill_l6_z(omy,xp,yp,zp,omgy) 
!!$     !call remaill_l6_z(omz,xp,yp,zp,omgz) 
!!$     if (type_b==2)call remaill_l2_bloc_z(omx,xp,yp,zp,omgx)        
!!$     if (type_b==2)call remaill_l2_bloc_z(omy,xp,yp,zp,omgy)        
!!$     if (type_b==2)call remaill_l2_bloc_z(omz,xp,yp,zp,omgz)        
!!$     if (type_b==4)call remaill_l4_bloc_z(omx,xp,yp,zp,omgx)        
!!$     if (type_b==4)call remaill_l4_bloc_z(omy,xp,yp,zp,omgy)        
!!$     if (type_b==4)call remaill_l4_bloc_z(omz,xp,yp,zp,omgz)         
!!$
!!$
!!$
!!$     !t source
!!$     !---------
!!$     call source_split_2
!!$
!!$
!!$     deallocate(vxg1,vyg1,vzg1)




     time=time+dt
     cpt_ite=cpt_ite+1

     !resultats
     !-----------

     !call res_omg_nite(nom_fich_omg_nite,nom_fich_vit_nite,nite_vtk,time)


   
!!$  print*,"tps"
!!$  print*,""
!!$  print*,"u fourier",t2-t1
!!$  print*,"penal",t3-t2
!!$  print*,"remaillage",t5-t4


  end do temps


 



  !resultat final
  !--------------
  print*,"tps final",time
  print*,"nbre d'ite",cpt_ite
  print*,"nart final",npart
  print*,"dt,dx,dy,dz",dt,dx,dy,dz



  !call res_vtk("RES/vtk/"//nom_fich_vtk)
  !call res_vtk_omgx("RES/vtk/"//nom_fich_vtk) 

  

  !
  !dealloc fermeture
  !-----------------

  deallocate (xp,qp,vx,vy,yp,zp,vz)
  deallocate (omgx,omgy,omgz)
  deallocate (omx,omy,omz)
  deallocate (vxg,vyg,vzg)
  deallocate (xtab,ytab,ztab)
  deallocate (chi,chi_sphere,lambda)
  deallocate (deb1,deb2,deb3)

  deallocate(stxg,styg,stzg,stx,sty,stz)

  close(12)
  close(13)
  close(14)
  close(15)
  close(16)
  close(17)
  close(18)
  close(19)
  close(20)

end program NS




