MODULE nrtype
	INTEGER, PARAMETER :: I4B = SELECTED_INT_KIND(9)
	INTEGER, PARAMETER :: I2B = SELECTED_INT_KIND(4)
	INTEGER, PARAMETER :: I1B = SELECTED_INT_KIND(2)
	INTEGER, PARAMETER :: SP = KIND(1.0D0)
	INTEGER, PARAMETER :: DP = KIND(1.0D0)
	INTEGER, PARAMETER :: SPC = KIND((1.0D0,1.0D0))
	INTEGER, PARAMETER :: DPC = KIND((1.0D0,1.0D0))
	INTEGER, PARAMETER :: LGT = KIND(.true.)
!	REAL(SP), PARAMETER :: PI=3.141592653589793238462643383279502884197_sp
	REAL(SP), PARAMETER :: PIO2=1.57079632679489661923132169163975144209858_sp
	REAL(SP), PARAMETER :: TWOPI=6.283185307179586476925286766559005768394_sp
	REAL(SP), PARAMETER :: SQRT2=1.41421356237309504880168872420969807856967_sp
	REAL(SP), PARAMETER :: EULER=0.5772156649015328606065120900824024310422_sp
	REAL(DP), PARAMETER :: PI_D=3.141592653589793238462643383279502884197_dp
	REAL(DP), PARAMETER :: PIO2_D=1.57079632679489661923132169163975144209858_dp
	REAL(DP), PARAMETER :: TWOPI_D=6.283185307179586476925286766559005768394_dp
	TYPE sprs2_sp
		INTEGER(I4B) :: n,len
		REAL(SP), DIMENSION(:), POINTER :: val
		INTEGER(I4B), DIMENSION(:), POINTER :: irow
		INTEGER(I4B), DIMENSION(:), POINTER :: jcol
	END TYPE sprs2_sp
	TYPE sprs2_dp
		INTEGER(I4B) :: n,len
		REAL(DP), DIMENSION(:), POINTER :: val
		INTEGER(I4B), DIMENSION(:), POINTER :: irow
		INTEGER(I4B), DIMENSION(:), POINTER :: jcol
	END TYPE sprs2_dp
END MODULE nrtype

!About Converting to Higher Precision
!=====================================

!!$You might hope that changing all the Numerical Recipes routines from single
!!$precision to double precision would be as simple as redefining the values of SP and
!!$DP in nrtype. Well . . . not quite.
!!$Converting algorithms to a higher precision is not a purely mechanical task
!!$because of the distinction between âroundoff errorâ and âtruncation error.â (Please
!!$see Volume 1, Â§1.2, if you are not familiar with these concepts.) While increasing the
!!$precision implied by the kind values SP and DP will indeed reduce a routineâs roundoff
!!$error, it will not reduce any truncation error that may be intrinsic to the algorithm.
!!$Sometimes, a routine contains âaccuracy parametersâ that can be adjusted to reduce
!!$the truncation error to the new, desired level. In other cases, however, the truncation
!!$error cannot be so easily reduced; then, a whole new algorithm is needed. Clearly
!!$such new algorithms are beyond the scope of a simple mechanical âconversion.â
!!$If, despite these cautionary words, you want to proceed with converting some
!!$routines to a higher precision, here are some hints:
!!$If your machine has a kind type that is distinct from, and has equal or greater
!!$precision than, the kind type that we use for DP, then, in nrtype, you can simply
!!$redefine DP to this new highest precision and redefine SP to what was previously
!!$DP. For example, DEC machines usually have a âquadruple precisionâ real type
!!$available, which can be used in this way. You should not need to make any further
!!$edits of nrtype or nrutil.
!!$If, on the other hand, the kind type that we already use for DP is the highest
!!$precision available, then you must leave DP defined as it is, and redefine SP in nrtype
!!$to be this same kind type. Now, however, you will also have to edit nrutil, because
!!$some overloaded routines that were previously distinguishable (by the different kind
!!$types) will now be seen by the compiler as indistinguishable â and it will object
!!$strenuously. Simply delete all the â dpâ function names from the list of overloaded
!!$procedures (i.e., from the MODULE PROCEDURE statements). Note that it is not
!!$necessary to delete the routines from the MODULE itself. Similarly, in the interface
!!$file nr.f90 you must delete the â dpâ interfaces, except for the sprs... routines.
!!$(Since they have TYPE(sprs2 dp) or TYPE(sprs2 sp), they are treated as distinct
!!$even though they have functionally equivalent kind types.)
!!$Finally, the following table gives some suggestions for changing the accuracy
!!$parameters, or constants, in some of the routines. Please note that this table is not
!!$necessarily complete, and that higher-precision performance is not guaranteed for all
!!$the routines, even if you make all the changes indicated. The above edits, and these
!!$suggestions, do, however, work in the majority of cases.
!!$
!!$
!!$
!!$
!!$
!!$In routine... change... to...
!!$beschb NUSE1=5,NUSE2=5 NUSE1=7,NUSE2=8
!!$bessi IACC=40 IACC=200
!!$bessik EPS=1.0e-10 dp EPS=epsilon(x)
!!$bessj IACC=40 IACC=160
!!$bessjy EPS=1.0e-10 dp EPS=epsilon(x)
!!$broydn TOLF=1.0e-4 sp TOLF=1.0e-8 sp
!!$TOLMIN=1.0e-6 sp TOLMIN=1.0e-12 sp
!!$fdjac EPS=1.0e-4 sp EPS=1.0e-8 sp
!!$frprmn EPS=1.0e-10 sp EPS=1.0e-18 sp
!!$gauher EPS=3.0e-13 dp EPS=1.0e-14 dp
!!$gaujac EPS=3.0e-14 dp EPS=1.0e-14 dp
!!$gaulag EPS=3.0e-13 dp EPS=1.0e-14 dp
!!$gauleg EPS=3.0e-14 dp EPS=1.0e-14 dp
!!$hypgeo EPS=1.0e-6 sp EPS=1.0e-14 sp
!!$linmin TOL=1.0e-4 sp TOL=1.0e-8 sp
!!$newt TOLF=1.0e-4 sp TOLF=1.0e-8 sp
!!$TOLMIN=1.0e-6 sp TOLMIN=1.0e-12 sp
!!$probks EPS1=0.001 sp EPS1=1.0e-6 sp
!!$EPS2=1.0e-8 sp EPS2=1.0e-16 sp
!!$qromb EPS=1.0e-6 sp EPS=1.0e-10 sp
!!$qromo EPS=1.0e-6 sp EPS=1.0e-10 sp
!!$qroot TINY=1.0e-6 sp TINY=1.0e-14 sp
!!$qsimp EPS=1.0e-6 sp EPS=1.0e-10 sp
!!$qtrap EPS=1.0e-6 sp EPS=1.0e-10 sp
!!$rc ERRTOL=0.04 sp ERRTOL=0.0012 sp
!!$rd ERRTOL=0.05 sp ERRTOL=0.0015 sp
!!$rf ERRTOL=0.08 sp ERRTOL=0.0025 sp
!!$rj ERRTOL=0.05 sp ERRTOL=0.0015 sp
!!$sfroid conv=5.0e-6 sp conv=1.0e-14 sp
!!$shoot EPS=1.0e-6 sp EPS=1.0e-14 sp
!!$shootf EPS=1.0e-6 sp EPS=1.0e-14 sp
!!$simplx EPS=1.0e-6 sp EPS=1.0e-14 sp
!!$sncndn CA=0.0003 sp CA=1.0e-8 sp
!!$sor EPS=1.0e-5 dp EPS=1.0e-13 dp
!!$sphfpt DXX=1.0e-4 sp DXX=1.0e-8 sp
!!$sphoot dx=1.0e-4 sp dx=1.0e-8 sp
!!$svdfit TOL=1.0e-5 sp TOL=1.0e-13 sp
!!$zroots EPS=1.0e-6 sp EPS=1.0e-14 sp
