module old_mod
  use donnees_mod
  use tab_mod
  use remaillage_mod
  use interpolation_mod
contains



  subroutine crea_part_old
    implicit none
    integer                  :: i,j,k
    real(kind=8)             :: x,y,z,m     

    m=cutoff*maxval(sqrt(omgx**2+omgy**2+omgz**2))
    npart=0  
    numpg=0
    do k=1,nz
       z=ztab(k)
       do j=1,ny
          y=ytab(j)
          do i=1,nx
             x=xtab(i) 

             if (sqrt(omgx(i,j,k)**2+omgy(i,j,k)**2+omgz(i,j,k)**2)>=m) then
                npart=npart+1 
                numpg(i,j,k)=npart
                xp(npart)=x
                yp(npart)=y
                zp(npart)=z
                vx(npart)=vxg(i,j,k)
                vy(npart)=vyg(i,j,k)
                vz(npart)=vzg(i,j,k)               
                omx(npart)=omgx(i,j,k)
                omy(npart)=omgy(i,j,k)
                omz(npart)=omgz(i,j,k)
                stx(npart)=stxg(i,j,k)
                sty(npart)=styg(i,j,k)
                stz(npart)=stzg(i,j,k)
             end if
          end do
       end do
    end do

  end subroutine crea_part_old




  subroutine advection_old()  
    implicit none

    real(kind=8),dimension(:),pointer      :: xp0,yp0,zp0,omx0,omy0,omz0
    integer                                :: i,ib

    allocate (xp0(1:npart),yp0(1:npart),zp0(1:npart))

    !mise en memoire de la position initiale
    !----------------------------------------
    do i=1,npart
       xp0(i)=xp(i)
       yp0(i)=yp(i)
       zp0(i)=zp(i)

       !advection de la moitie d'un pas de temps : sous ite RK
       !------------------------------------------------------

       xp(i)=xp0(i)+0.5*dt*vx(i)
       yp(i)=yp0(i)+0.5*dt*vy(i)
       zp(i)=zp0(i)+0.5*dt*vz(i)

    end do


    call interpo_l3_3d(vxg,xp,yp,zp,vx) 
    call interpo_l3_3d(vyg,xp,yp,zp,vy) 
    call interpo_l3_3d(vzg,xp,yp,zp,vz) 

    call interpo_l3_3d(stxg,xp,yp,zp,stx) 
    call interpo_l3_3d(styg,xp,yp,zp,sty) 
    call interpo_l3_3d(stzg,xp,yp,zp,stz) 


    !advection depuis position initiale avec cette nouvelle vitesse : seconde ite de RK
    !----------------------------------------------------------------------------------
    do i=1,npart
       xp(i)=xp0(i)+dt*vx(i)
       yp(i)=yp0(i)+dt*vy(i)
       zp(i)=zp0(i)+dt*vz(i)

       omx(i)=omx(i)+dt*stx(i)
       omy(i)=omy(i)+dt*sty(i)
       omz(i)=omz(i)+dt*stz(i)
    end do


    deallocate(xp0,yp0,zp0)

  end subroutine advection_old


  subroutine penal_exacte     
    implicit none
    real(kind=8)                             :: dyvecez,dzvecey,dzvecex,dxvecez,dxvecey,dyvecex
    integer                                  :: i,j,k
    integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2
    real(kind=8),dimension(1:3)   :: tx,ty,tz
    real(kind=8)                  :: facx,facy,facz


    !PENALISATION
    !--------------
    ! si (on est dans l'obstacle  ou  derive(chi)/=0)) alors
    !    omega = rot (vitesse*exp(-lambda*chi*dt) )





    do k=1,nz       ! ordre 4
       do j=1,ny
          do i=1,nx

             !calcul penalisation uniquement dans l'obstacle et environs
             if (.not.((chi(i,j,k)==0.).and.(derivx(chi,i,j,k)==0.) &
	.and.(derivy(chi,i,j,k)==0.).and.(derivz(chi,i,j,k)==0.))) then



                if((i/=2).and.(i/=1).and.(i/=nx-1).and.(i/=nx).and.(j/=2) &
		.and.(j/=1).and.(j/=ny-1).and.(j/=ny).and.(k/=2).and.(k/=1) &
		.and.(k/=nz-1).and.(k/=nz))then 
                   !schema centre au milieu du domaine

                   omgx(i,j,k)=1./(12.*dy)*(1.*penz(i,j-2,k)-8.*penz(i,j-1,k) &
	+8.*penz(i,j+1,k)-1.*penz(i,j+2,k))-(1./(12.*dz)*(1.*peny(i,j,k-2)-8.*peny(i,j,k-1) &
	+8.*peny(i,j,k+1)-1.*peny(i,j,k+2)))
                   omgy(i,j,k)=1./(12.*dz)*(1.*penx(i,j,k-2)-8.*penx(i,j,k-1) &
	+8.*penx(i,j,k+1)-1.*penx(i,j,k+2))-(1./(12.*dx)*(1.*penz(i-2,j,k)-8.*penz(i-1,j,k) &
	+8.*penz(i+1,j,k)-1.*penz(i+2,j,k)))
                   omgz(i,j,k)=1./(12.*dx)*(1.*peny(i-2,j,k)-8.*peny(i-1,j,k) &
	+8.*peny(i+1,j,k)-1.*peny(i+2,j,k))-(1./(12.*dy)*(1.*penx(i,j-2,k)-8.*penx(i,j-1,k) &
	+8.*penx(i,j+1,k)-1.*penx(i,j+2,k)))

                else

                   dxvecey=1./(12.*dx)*(1.*peny(i-2,j,k)-8.*peny(i-1,j,k)+8.*peny(i+1,j,k)-1.*peny(i+2,j,k))
                   dxvecez=1./(12.*dx)*(1.*penz(i-2,j,k)-8.*penz(i-1,j,k)+8.*penz(i+1,j,k)-1.*penz(i+2,j,k))
                   dyvecex=1./(12.*dy)*(1.*penx(i,j-2,k)-8.*penx(i,j-1,k)+8.*penx(i,j+1,k)-1.*penx(i,j+2,k))
                   dyvecez=1./(12.*dy)*(1.*penz(i,j-2,k)-8.*penz(i,j-1,k)+8.*penz(i,j+1,k)-1.*penz(i,j+2,k))
                   dzvecex=1./(12.*dz)*(1.*penx(i,j,k-2)-8.*penx(i,j,k-1)+8.*penx(i,j,k+1)-1.*penx(i,j,k+2))
                   dzvecey=1./(12.*dz)*(1.*peny(i,j,k-2)-8.*peny(i,j,k-1)+8.*peny(i,j,k+1)-1.*peny(i,j,k+2))



                   if((i==1).or.(i==2))then
                      dxvecey=1./(12.*dx)*(-25.*peny(i,j,k)+48.*peny(i+1,j,k)-36.*peny(i+2,j,k) &
	+16.*peny(i+3,j,k)-3.*peny(i+4,j,k))
                      dxvecez=1./(12.*dx)*(-25.*penz(i,j,k)+48.*penz(i+1,j,k)-36.*penz(i+2,j,k) &
	+16.*penz(i+3,j,k)-3.*penz(i+4,j,k))
                   end if

                   if((i==nx).or.(i==nx-1))then
                      dxvecey=1./(12.*dx)*(25.*peny(i,j,k)-48.*peny(i-1,j,k)+36.*peny(i-2,j,k) &
	-16.*peny(i-3,j,k)+3.*peny(i-4,j,k))
                      dxvecez=1./(12.*dx)*(25.*penz(i,j,k)-48.*penz(i-1,j,k)+36.*penz(i-2,j,k) &
	-16.*penz(i-3,j,k)+3.*penz(i-4,j,k))
                   endif

                   if((j==1).or.(j==2))then
                      dyvecez=1./(12.*dy)*(-25.*penz(i,j,k)+48.*penz(i,j+1,k)-36.*penz(i,j+2,k) &
	+16.*penz(i,j+3,k)-3.*penz(i,j+4,k))
                      dyvecex=1./(12.*dy)*(-25.*penx(i,j,k)+48.*penx(i,j+1,k)-36.*penx(i,j+2,k) &
	+16.*penx(i,j+3,k)-3.*penx(i,j+4,k))
                   end if

                   if((j==ny).or.(j==ny-1))then
                      dyvecez=1./(12.*dy)*(+25.*penz(i,j,k)-48.*penz(i,j-1,k)+36.*penz(i,j-2,k)	&
	-16.*penz(i,j-3,k)+3.*penz(i,j-4,k))         
                      dyvecex=1./(12.*dy)*(+25.*penx(i,j,k)-48.*penx(i,j-1,k)+36.*penx(i,j-2,k) &
	-16.*penx(i,j-3,k)+3.*penx(i,j-4,k))            
                   endif

                   if((k==1).or.(k==2))then
                      dzvecex=1./(12.*dz)*(-25.*penx(i,j,k)+48.*penx(i,j,k+1)-36.*penx(i,j,k+2) &
	+16.*penx(i,j,k+3)-3.*penx(i,j,k+4))
                      dzvecey=1./(12.*dz)*(-25.*peny(i,j,k)+48.*peny(i,j,k+1)-36.*peny(i,j,k+2) &
	+16.*peny(i,j,k+3)-3.*peny(i,j,k+4))
                   end if

                   if((k==nz).or.(k==nz-1))then
                      dzvecex=1./(12.*dz)*(25.*penx(i,j,k)-48.*penx(i,j,k-1)+36.*penx(i,j,k-2) &
	-16.*penx(i,j,k-3)+3.*penx(i,j,k-4))
                      dzvecey=1./(12.*dz)*(25.*peny(i,j,k)-48.*peny(i,j,k-1)+36.*peny(i,j,k-2) &
	-16.*peny(i,j,k-3)+3.*peny(i,j,k-4))
                   endif



                   omgx(i,j,k)=dyvecez-dzvecey
                   omgy(i,j,k)=dzvecex-dxvecez
                   omgz(i,j,k)=dxvecey-dyvecex

                end if

             end if
          end do
       end do
    end do

  contains

    function penx (l,m,n)     
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: penx


      !penx=vxg(l,m,n)*exp(-lambda*chi(l,m,n)*dt)
      penx=vxg(l,m,n)/(1.+lambda(l,m,n)*chi(l,m,n)*dt)

    end function penx

    function peny (l,m,n)     
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: peny


      !peny=vyg(l,m,n)*exp(-lambda*chi(l,m,n)*dt)
      peny=vyg(l,m,n)/(1.+lambda(l,m,n)*chi(l,m,n)*dt)

    end function peny

    function penz (l,m,n)      
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: penz


      !penz=vzg(l,m,n)*exp(-lambda*chi(l,m,n)*dt)
      penz=vzg(l,m,n)/(1.+lambda(l,m,n)*chi(l,m,n)*dt)

    end function penz

  end subroutine penal_exacte




  function derivx (var,i,j,k) result (vars) ! calcul derive en x de var (decentre au bord, centre milieu) !ordre4  
    implicit none
    real(kind=8),dimension(:,:,:)  :: var
    integer                        :: i,j,k
    real(kind=8)                   :: vars

    if((i/=2).and.(i/=1).and.(i/=nx-1).and.(i/=nx))then 
       !schema centre au milieu du domaine
       vars=1./(12.*dx)*(1.*var(i-2,j,k)-8.*var(i-1,j,k)+8.*var(i+1,j,k)-1.*var(i+2,j,k))
    else             
       if((i==1).or.(i==2))then
          vars=1./(12.*dx)*(-25.*var(i,j,k)+48.*var(i+1,j,k)-36.*var(i+2,j,k)+16.*var(i+3,j,k)-3.*var(i+4,j,k))
       end if
       if((i==nx).or.(i==nx-1))then
          vars=1./(12.*dx)*(+25.*var(i,j,k)-48.*var(i-1,j,k)+36.*var(i-2,j,k)-16.*var(i-3,j,k)+3.*var(i-4,j,k))
       endif
    end if
  end function derivx

  function derivy (var,i,j,k)  result(vars) ! calcul derive en y de var (decentre au bord, centre milieu) !ordre4   
    implicit none
    real(kind=8),dimension(:,:,:)  :: var
    integer                        :: i,j,k      
    real(kind=8)                   :: vars

    if((j/=2).and.(j/=1).and.(j/=ny-1).and.(j/=ny))then 
       !schema centre au milieu du domaine
       vars=1./(12.*dy)*(1.*var(i,j-2,k)-8.*var(i,j-1,k)+8.*var(i,j+1,k)-1.*var(i,j+2,k))
    else             
       if((j==1).or.(j==2))then
          vars=1./(12.*dy)*(-25.*var(i,j,k)+48.*var(i,j+1,k)-36.*var(i,j+2,k)+16.*var(i,j+3,k)-3.*var(i,j+4,k))
       end if
       if((j==ny).or.(j==ny-1))then
          vars=1./(12.*dy)*(+25.*var(i,j,k)-48.*var(i,j-1,k)+36.*var(i,j-2,k)-16.*var(i,j-3,k)+3.*var(i,j-4,k))
       endif
    end if
  end function derivy

  function derivz (var,i,j,k)  result(vars) ! calcul derive en z de var (decentre au bord, centre milieu) !ordre4   
    implicit none
    real(kind=8),dimension(:,:,:)  :: var
    integer                        :: i,j,k      
    real(kind=8)                   :: vars

    if((k/=2).and.(k/=1).and.(k/=nz-1).and.(k/=nz))then 
       !schema centre au milieu du domaine
       vars=1./(12.*dz)*(1.*var(i,j,k-2)-8.*var(i,j,k-1)+8.*var(i,j,k+1)-1.*var(i,j,k+2))
    else             
       if((k==1).or.(k==2))then
          vars=1./(12.*dz)*(-25.*var(i,j,k)+48.*var(i,j,k+1)-36.*var(i,j,k+2)+16.*var(i,j,k+3)-3.*var(i,j,k+4))
       end if
       if((k==ny).or.(k==ny-1))then
          vars=1./(12.*dz)*(+25.*var(i,j,k)-48.*var(i,j,k-1)+36.*var(i,j,k-2)-16.*var(i,j,k-3)+3.*var(i,j,k-4))
       endif
    end if
  end function derivz





  subroutine strech_old()     
    implicit none
    integer                       :: i,j,k
    real(kind=8),dimension(1:3)   :: tx,ty,tz
    real(kind=8)                  :: x,y,z

    !forme conservative : div(w:u)

    do k=1,nz
       do j=1,ny
          do i=1,nx

!!$               tx(1)=dx_c_4(vxg*omgx,i,j,k,dx)
!!$               tx(2)=dx_c_4(vyg*omgx,i,j,k,dx)
!!$               tx(3)=dx_c_4(vzg*omgx,i,j,k,dx)
!!$               
!!$               ty(1)=dy_c_4(vxg*omgy,i,j,k,dy)
!!$               ty(2)=dy_c_4(vyg*omgy,i,j,k,dy)
!!$               ty(3)=dy_c_4(vzg*omgy,i,j,k,dy)
!!$
!!$               tz(1)=dz_c_4(vxg*omgz,i,j,k,dz)
!!$               tz(2)=dz_c_4(vyg*omgz,i,j,k,dz)
!!$               tz(3)=dz_c_4(vzg*omgz,i,j,k,dz)

             tx(1)=1./(12.*dx)*(1.*omgx(i-2,j,k)*vxg(i-2,j,k)-8.*omgx(i-1,j,k)*vxg(i-1,j,k) &
	+8.*omgx(i+1,j,k)*vxg(i+1,j,k)-1.*omgx(i+2,j,k)*vxg(i+2,j,k))
             tx(2)=1./(12.*dx)*(1.*omgx(i-2,j,k)*vyg(i-2,j,k)-8.*omgx(i-1,j,k)*vyg(i-1,j,k) &
	+8.*omgx(i+1,j,k)*vyg(i+1,j,k)-1.*omgx(i+2,j,k)*vyg(i+2,j,k))
             tx(3)=1./(12.*dx)*(1.*omgx(i-2,j,k)*vzg(i-2,j,k)-8.*omgx(i-1,j,k)*vzg(i-1,j,k) &
	+8.*omgx(i+1,j,k)*vzg(i+1,j,k)-1.*omgx(i+2,j,k)*vzg(i+2,j,k))

             ty(1)=1./(12.*dy)*(1.*omgy(i,j-2,k)*vxg(i,j-2,k)-8.*omgy(i,j-1,k)*vxg(i,j-1,k) &
	+8.*omgy(i,j+1,k)*vxg(i,j+1,k)-1.*omgy(i,j+2,k)*vxg(i,j+2,k))
             ty(2)=1./(12.*dy)*(1.*omgy(i,j-2,k)*vyg(i,j-2,k)-8.*omgy(i,j-1,k)*vyg(i,j-1,k) &
	+8.*omgy(i,j+1,k)*vyg(i,j+1,k)-1.*omgy(i,j+2,k)*vyg(i,j+2,k))
             ty(3)=1./(12.*dy)*(1.*omgy(i,j-2,k)*vzg(i,j-2,k)-8.*omgy(i,j-1,k)*vzg(i,j-1,k) &
	+8.*omgy(i,j+1,k)*vzg(i,j+1,k)-1.*omgy(i,j+2,k)*vzg(i,j+2,k))

             tz(1)=1./(12.*dz)*(1.*omgz(i,j,k-2)*vxg(i,j,k-2)-8.*omgz(i,j,k-1)*vxg(i,j,k-1) &
	+8.*omgz(i,j,k+1)*vxg(i,j,k+1)-1.*omgz(i,j,k+2)*vxg(i,j,k+2))
             tz(2)=1./(12.*dz)*(1.*omgz(i,j,k-2)*vyg(i,j,k-2)-8.*omgz(i,j,k-1)*vyg(i,j,k-1) &
	+8.*omgz(i,j,k+1)*vyg(i,j,k+1)-1.*omgz(i,j,k+2)*vyg(i,j,k+2))
             tz(3)=1./(12.*dz)*(1.*omgz(i,j,k-2)*vzg(i,j,k-2)-8.*omgz(i,j,k-1)*vzg(i,j,k-1) &
	+8.*omgz(i,j,k+1)*vzg(i,j,k+1)-1.*omgz(i,j,k+2)*vzg(i,j,k+2))

!!$               !schema decentre au bord
!!$               if((i==1).or.(i==2))then
!!$                  tx(1)=dx_dr_5(vxg*omgx,i,j,k,dx)
!!$                  tx(2)=dx_dr_5(vyg*omgx,i,j,k,dx)
!!$                  tx(3)=dx_dr_5(vzg*omgx,i,j,k,dx)
!!$               end if
!!$               
!!$               if((i==nx).or.(i==nx-1))then
!!$                  tx(1)=dx_ga_5(vxg*omgx,i,j,k,dx)
!!$                  tx(2)=dx_ga_5(vyg*omgx,i,j,k,dx)
!!$                  tx(3)=dx_ga_5(vzg*omgx,i,j,k,dx)
!!$               endif
!!$               
!!$               if((j==1).or.(j==2))then
!!$                  ty(1)=dy_dr_5(vxg*omgy,i,j,k,dy)
!!$                  ty(2)=dy_dr_5(vyg*omgy,i,j,k,dy)
!!$                  ty(3)=dy_dr_5(vzg*omgy,i,j,k,dy)
!!$               end if
!!$               
!!$               if((j==ny).or.(j==ny-1))then
!!$                  ty(1)=dy_ga_5(vxg*omgy,i,j,k,dy)
!!$                  ty(2)=dy_ga_5(vyg*omgy,i,j,k,dy)
!!$                  ty(3)=dy_ga_5(vzg*omgy,i,j,k,dy)
!!$               endif
!!$               
!!$               if((k==1).or.(k==2))then
!!$                  tz(1)=dz_ha_5(vxg*omgz,i,j,k,dz)
!!$                  tz(2)=dz_ha_5(vyg*omgz,i,j,k,dz)
!!$                  tz(3)=dz_ha_5(vzg*omgz,i,j,k,dz)
!!$               end if
!!$               
!!$               if((k==nz).or.(k==nz-1))then
!!$                  tz(1)=dz_ba_5(vxg*omgz,i,j,k,dz)
!!$                  tz(2)=dz_ba_5(vyg*omgz,i,j,k,dz)
!!$                  tz(3)=dz_ba_5(vzg*omgz,i,j,k,dz)
!!$               endif

             !schema decentre au bord
             if((i==1).or.(i==2))then
                tx(1)=1./(12.*dx)*(-25.*omgx(i,j,k)*vxg(i,j,k)+48.*omgx(i+1,j,k)*vxg(i+1,j,k) &
	-36.*omgx(i+2,j,k)*vxg(i+2,j,k)+16.*omgx(i+3,j,k)*vxg(i+3,j,k)-3.*omgx(i+4,j,k)*vxg(i+4,j,k))
                tx(2)=1./(12.*dx)*(-25.*omgx(i,j,k)*vyg(i,j,k)+48.*omgx(i+1,j,k)*vyg(i+1,j,k) &
	-36.*omgx(i+2,j,k)*vyg(i+2,j,k)+16.*omgx(i+3,j,k)*vyg(i+3,j,k)-3.*omgx(i+4,j,k)*vyg(i+4,j,k))
                tx(3)=1./(12.*dx)*(-25.*omgx(i,j,k)*vzg(i,j,k)+48.*omgx(i+1,j,k)*vzg(i+1,j,k) &
	-36.*omgx(i+2,j,k)*vzg(i+2,j,k)+16.*omgx(i+3,j,k)*vzg(i+3,j,k)-3.*omgx(i+4,j,k)*vzg(i+4,j,k))
             end if

             if((i==nx).or.(i==nx-1))then
                tx(1)=1./(12.*dx)*(-25.*omgx(i,j,k)*vxg(i,j,k)+48.*omgx(i-1,j,k)*vxg(i-1,j,k) &
	-36.*omgx(i-2,j,k)*vxg(i-2,j,k)+16.*omgx(i-3,j,k)*vxg(i-3,j,k)-3.*omgx(i-4,j,k)*vxg(i-4,j,k))
                tx(2)=1./(12.*dx)*(-25.*omgx(i,j,k)*vyg(i,j,k)+48.*omgx(i-1,j,k)*vyg(i-1,j,k) &
	-36.*omgx(i-2,j,k)*vyg(i-2,j,k)+16.*omgx(i-3,j,k)*vyg(i-3,j,k)-3.*omgx(i-4,j,k)*vyg(i-4,j,k))
                tx(3)=1./(12.*dx)*(-25.*omgx(i,j,k)*vzg(i,j,k)+48.*omgx(i-1,j,k)*vzg(i-1,j,k) &
	-36.*omgx(i-2,j,k)*vzg(i-2,j,k)+16.*omgx(i-3,j,k)*vzg(i-3,j,k)-3.*omgx(i-4,j,k)*vzg(i-4,j,k))
             endif

             if((j==1).or.(j==2))then
                ty(1)=1./(12.*dy)*(-25.*omgy(i,j,k)*vxg(i,j,k)+48.*omgy(i,j+1,k)*vxg(i,j+1,k) &
	-36.*omgy(i,j+2,k)*vxg(i,j+2,k)+16.*omgy(i,j+3,k)*vxg(i,j+3,k)-3.*omgy(i,j+4,k)*vxg(i,j+4,k))
                ty(2)=1./(12.*dy)*(-25.*omgy(i,j,k)*vyg(i,j,k)+48.*omgy(i,j+1,k)*vyg(i,j+1,k) &
	-36.*omgy(i,j+2,k)*vyg(i,j+2,k)+16.*omgy(i,j+3,k)*vyg(i,j+3,k)-3.*omgy(i,j+4,k)*vyg(i,j+4,k))
                ty(3)=1./(12.*dy)*(-25.*omgy(i,j,k)*vzg(i,j,k)+48.*omgy(i,j+1,k)*vzg(i,j+1,k) &
	-36.*omgy(i,j+2,k)*vzg(i,j+2,k)+16.*omgy(i,j+3,k)*vzg(i,j+3,k)-3.*omgy(i,j+4,k)*vzg(i,j+4,k))
             end if

             if((j==ny).or.(j==ny-1))then
                ty(1)=1./(12.*dy)*(-25.*omgy(i,j,k)*vxg(i,j,k)+48.*omgy(i,j-1,k)*vxg(i,j-1,k) &
	-36.*omgy(i,j-2,k)*vxg(i,j-2,k)+16.*omgy(i,j-3,k)*vxg(i,j-3,k)-3.*omgy(i,j-4,k)*vxg(i,j-4,k))
                ty(2)=1./(12.*dy)*(-25.*omgy(i,j,k)*vyg(i,j,k)+48.*omgy(i,j-1,k)*vyg(i,j-1,k) &
	-36.*omgy(i,j-2,k)*vyg(i,j-2,k)+16.*omgy(i,j-3,k)*vyg(i,j-3,k)-3.*omgy(i,j-4,k)*vyg(i,j-4,k))
                ty(3)=1./(12.*dy)*(-25.*omgy(i,j,k)*vzg(i,j,k)+48.*omgy(i,j-1,k)*vzg(i,j-1,k) &
	-36.*omgy(i,j-2,k)*vzg(i,j-2,k)+16.*omgy(i,j-3,k)*vzg(i,j-3,k)-3.*omgy(i,j-4,k)*vzg(i,j-4,k))
             endif

             if((k==1).or.(k==2))then
                tz(1)=1./(12.*dz)*(-25.*omgz(i,j,k)*vxg(i,j,k)+48.*omgz(i,j,k+1)*vxg(i,j,k+1) &
	-36.*omgz(i,j,k+2)*vxg(i,j,k+2)+16.*omgz(i,j,k+3)*vxg(i,j,k+3)-3.*omgz(i,j,k+4)*vxg(i,j,k+4))
                tz(2)=1./(12.*dz)*(-25.*omgz(i,j,k)*vyg(i,j,k)+48.*omgz(i,j,k+1)*vyg(i,j,k+1) &
	-36.*omgz(i,j,k+2)*vyg(i,j,k+2)+16.*omgz(i,j,k+3)*vyg(i,j,k+3)-3.*omgz(i,j,k+4)*vyg(i,j,k+4))
                tz(3)=1./(12.*dz)*(-25.*omgz(i,j,k)*vzg(i,j,k)+48.*omgz(i,j,k+1)*vzg(i,j,k+1) &
	-36.*omgz(i,j,k+2)*vzg(i,j,k+2)+16.*omgz(i,j,k+3)*vzg(i,j,k+3)-3.*omgz(i,j,k+4)*vzg(i,j,k+4))
             end if

             if((k==nz).or.(k==nz-1))then
                tz(1)=1./(12.*dz)*(-25.*omgz(i,j,k)*vxg(i,j,k)+48.*omgz(i,j,k-1)*vxg(i,j,k-1) &
	-36.*omgz(i,j,k-2)*vxg(i,j,k-2)+16.*omgz(i,j,k-3)*vxg(i,j,k-3)-3.*omgz(i,j,k-4)*vxg(i,j,k-4))
                tz(2)=1./(12.*dz)*(-25.*omgz(i,j,k)*vyg(i,j,k)+48.*omgz(i,j,k-1)*vyg(i,j,k-1) &
	-36.*omgz(i,j,k-2)*vyg(i,j,k-2)+16.*omgz(i,j,k-3)*vyg(i,j,k-3)-3.*omgz(i,j,k-4)*vyg(i,j,k-4))
                tz(3)=1./(12.*dz)*(-25.*omgz(i,j,k)*vzg(i,j,k)+48.*omgz(i,j,k-1)*vzg(i,j,k-1) &
	-36.*omgz(i,j,k-2)*vzg(i,j,k-2)+16.*omgz(i,j,k-3)*vzg(i,j,k-3)-3.*omgz(i,j,k-4)*vzg(i,j,k-4))
             endif

             x=xg+(i-1)*dx
             y=yb+(j-1)*dy
             z=zd+(k-1)*dz



             stxg(i,j,k)=tx(1)+ty(1)+tz(1)
             styg(i,j,k)=tx(2)+ty(2)+tz(2)
             stzg(i,j,k)=tx(3)+ty(3)+tz(3)
          end do
       end do
    end do

  end subroutine strech_old


  subroutine diffusion_old    
    implicit none
    real(kind=8),dimension(1:3,1:nx,1:ny,1:nz)   :: diffusion
    integer                       :: i,j,k
    real(kind=8)                  :: facx,facy,facz
    real(kind=8),dimension(1:3)   :: tx,ty,tz
    integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2

    facx=1.D0/(12.D0*dx**2)
    facy=1.D0/(12.D0*dy**2)
    facz=1.D0/(12.D0*dz**2)

    do k=1,nz
       do j=1,ny
          do i=1,nx

             ip1=mod(i+1+nx-1,nx)+1
             im1=mod(i-1+nx-1,nx)+1
             ip2=mod(i+2+nx-1,nx)+1
             im2=mod(i-2+nx-1,nx)+1

             jp1=mod(j+1+ny-1,ny)+1
             jm1=mod(j-1+ny-1,ny)+1
             jp2=mod(j+2+ny-1,ny)+1
             jm2=mod(j-2+ny-1,ny)+1

             kp1=mod(k+1+nz-1,nz)+1
             km1=mod(k-1+nz-1,nz)+1
             kp2=mod(k+2+nz-1,nz)+1
             km2=mod(k-2+nz-1,nz)+1

             !ordre4
             tx(1)=facx*(-omgx(ip2,j,k)+16.D0*omgx(ip1,j,k)-30.D0*omgx(i,j,k)+16.D0*omgx(im1,j,k)-omgx(im2,j,k))
             tx(2)=facx*(-omgy(ip2,j,k)+16.D0*omgy(ip1,j,k)-30.D0*omgy(i,j,k)+16.D0*omgy(im1,j,k)-omgy(im2,j,k))
             tx(3)=facx*(-omgz(ip2,j,k)+16.D0*omgz(ip1,j,k)-30.D0*omgz(i,j,k)+16.D0*omgz(im1,j,k)-omgz(im2,j,k))

             ty(1)=facy*(-omgx(i,jp2,k)+16.D0*omgx(i,jp1,k)-30.D0*omgx(i,j,k)+16.D0*omgx(i,jm1,k)-omgx(i,jm2,k))
             ty(2)=facy*(-omgy(i,jp2,k)+16.D0*omgy(i,jp1,k)-30.D0*omgy(i,j,k)+16.D0*omgy(i,jm1,k)-omgy(i,jm2,k))
             ty(3)=facy*(-omgz(i,jp2,k)+16.D0*omgz(i,jp1,k)-30.D0*omgz(i,j,k)+16.D0*omgz(i,jm1,k)-omgz(i,jm2,k))

             tz(1)=facz*(-omgx(i,j,kp2)+16.D0*omgx(i,j,kp1)-30.D0*omgx(i,j,k)+16.D0*omgx(i,j,km1)-omgx(i,j,km2))
             tz(2)=facz*(-omgy(i,j,kp2)+16.D0*omgy(i,j,kp1)-30.D0*omgy(i,j,k)+16.D0*omgy(i,j,km1)-omgy(i,j,km2))
             tz(3)=facz*(-omgz(i,j,kp2)+16.D0*omgz(i,j,kp1)-30.D0*omgz(i,j,k)+16.D0*omgz(i,j,km1)-omgz(i,j,km2))



             diffusion(1,i,j,k)=tx(1)+ty(1)+tz(1)
             diffusion(2,i,j,k)=tx(2)+ty(2)+tz(2)
             diffusion(3,i,j,k)=tx(3)+ty(3)+tz(3)
          end do
       end do
    end do

    do k=1,nz
       do j=1,ny
          do i=1,nx
             omgx(i,j,k)=omgx(i,j,k)+ nu*dt*diffusion(1,i,j,k)
             omgy(i,j,k)=omgy(i,j,k)+ nu*dt*diffusion(2,i,j,k)
             omgz(i,j,k)=omgz(i,j,k)+ nu*dt*diffusion(3,i,j,k)
          end do
       end do
    end do

  end subroutine diffusion_old


  subroutine strech_diff_penal
    implicit none
    integer                       :: i,j,k
    real(kind=8),dimension(1:3)   :: tx,ty,tz
    real(kind=8)                  :: x,y,z
    real(kind=8)                  :: facx,facy,facz
    integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2




    do k=1,nz
       do j=1,ny
          do i=1,nx


             facx=1.D0/(12.D0*dx)
             facy=1.D0/(12.D0*dy)
             facz=1.D0/(12.D0*dz)

             ip1=mod(i+1+nx-1,nx)+1
             im1=mod(i-1+nx-1,nx)+1
             ip2=mod(i+2+nx-1,nx)+1
             im2=mod(i-2+nx-1,nx)+1

             jp1=mod(j+1+ny-1,ny)+1
             jm1=mod(j-1+ny-1,ny)+1
             jp2=mod(j+2+ny-1,ny)+1
             jm2=mod(j-2+ny-1,ny)+1

             kp1=mod(k+1+nz-1,nz)+1
             km1=mod(k-1+nz-1,nz)+1
             kp2=mod(k+2+nz-1,nz)+1
             km2=mod(k-2+nz-1,nz)+1

             !strech
             !------
             tx(1)=facx*(omgx(im2,j,k)*vxg(im2,j,k)-8.D0*omgx(im1,j,k)*vxg(im1,j,k) &
	+8.D0*omgx(ip1,j,k)*vxg(ip1,j,k)-omgx(ip2,j,k)*vxg(ip2,j,k))
             tx(2)=facx*(omgx(im2,j,k)*vyg(im2,j,k)-8.D0*omgx(im1,j,k)*vyg(im1,j,k) &
	+8.D0*omgx(ip1,j,k)*vyg(ip1,j,k)-omgx(ip2,j,k)*vyg(ip2,j,k))
             tx(3)=facx*(omgx(im2,j,k)*vzg(im2,j,k)-8.D0*omgx(im1,j,k)*vzg(im1,j,k) &
	+8.D0*omgx(ip1,j,k)*vzg(ip1,j,k)-omgx(ip2,j,k)*vzg(ip2,j,k))

             ty(1)=facy*(omgy(i,jm2,k)*vxg(i,jm2,k)-8.D0*omgy(i,jm1,k)*vxg(i,jm1,k) &
	+8.D0*omgy(i,jp1,k)*vxg(i,jp1,k)-omgy(i,jp2,k)*vxg(i,jp2,k))
             ty(2)=facy*(omgy(i,jm2,k)*vyg(i,jm2,k)-8.D0*omgy(i,jm1,k)*vyg(i,jm1,k) &
	+8.D0*omgy(i,jp1,k)*vyg(i,jp1,k)-omgy(i,jp2,k)*vyg(i,jp2,k))
             ty(3)=facy*(omgy(i,jm2,k)*vzg(i,jm2,k)-8.D0*omgy(i,jm1,k)*vzg(i,jm1,k) &
	+8.D0*omgy(i,jp1,k)*vzg(i,jp1,k)-omgy(i,jp2,k)*vzg(i,jp2,k))

             tz(1)=facz*(omgz(i,j,km2)*vxg(i,j,km2)-8.D0*omgz(i,j,km1)*vxg(i,j,km1) &
	+8.D0*omgz(i,j,kp1)*vxg(i,j,kp1)-omgz(i,j,kp2)*vxg(i,j,kp2))
             tz(2)=facz*(omgz(i,j,km2)*vyg(i,j,km2)-8.D0*omgz(i,j,km1)*vyg(i,j,km1) &
	+8.D0*omgz(i,j,kp1)*vyg(i,j,kp1)-omgz(i,j,kp2)*vyg(i,j,kp2))
             tz(3)=facz*(omgz(i,j,km2)*vzg(i,j,km2)-8.D0*omgz(i,j,km1)*vzg(i,j,km1) &
	+8.D0*omgz(i,j,kp1)*vzg(i,j,kp1)-omgz(i,j,kp2)*vzg(i,j,kp2))



             stxg(i,j,k)=tx(1)+ty(1)+tz(1)
             styg(i,j,k)=tx(2)+ty(2)+tz(2)
             stzg(i,j,k)=tx(3)+ty(3)+tz(3)


             !diffusion
             !----------
             facx=1.D0/(12.D0*dx**2)
             facy=1.D0/(12.D0*dy**2)
             facz=1.D0/(12.D0*dz**2)

             tx(1)=facx*(-omgx(ip2,j,k)+16.D0*omgx(ip1,j,k)-30.D0*omgx(i,j,k)+16.D0*omgx(im1,j,k)-omgx(im2,j,k))
             tx(2)=facx*(-omgy(ip2,j,k)+16.D0*omgy(ip1,j,k)-30.D0*omgy(i,j,k)+16.D0*omgy(im1,j,k)-omgy(im2,j,k))
             tx(3)=facx*(-omgz(ip2,j,k)+16.D0*omgz(ip1,j,k)-30.D0*omgz(i,j,k)+16.D0*omgz(im1,j,k)-omgz(im2,j,k))

             ty(1)=facy*(-omgx(i,jp2,k)+16.D0*omgx(i,jp1,k)-30.D0*omgx(i,j,k)+16.D0*omgx(i,jm1,k)-omgx(i,jm2,k))
             ty(2)=facy*(-omgy(i,jp2,k)+16.D0*omgy(i,jp1,k)-30.D0*omgy(i,j,k)+16.D0*omgy(i,jm1,k)-omgy(i,jm2,k))
             ty(3)=facy*(-omgz(i,jp2,k)+16.D0*omgz(i,jp1,k)-30.D0*omgz(i,j,k)+16.D0*omgz(i,jm1,k)-omgz(i,jm2,k))

             tz(1)=facz*(-omgx(i,j,kp2)+16.D0*omgx(i,j,kp1)-30.D0*omgx(i,j,k)+16.D0*omgx(i,j,km1)-omgx(i,j,km2))
             tz(2)=facz*(-omgy(i,j,kp2)+16.D0*omgy(i,j,kp1)-30.D0*omgy(i,j,k)+16.D0*omgy(i,j,km1)-omgy(i,j,km2))
             tz(3)=facz*(-omgz(i,j,kp2)+16.D0*omgz(i,j,kp1)-30.D0*omgz(i,j,k)+16.D0*omgz(i,j,km1)-omgz(i,j,km2))



             stxg(i,j,k)=stxg(i,j,k)+nu*(tx(1)+ty(1)+tz(1))
             styg(i,j,k)=styg(i,j,k)+nu*(tx(2)+ty(2)+tz(2))
             stzg(i,j,k)=stzg(i,j,k)+nu*(tx(3)+ty(3)+tz(3))

!!$             if (.not.((chi(i,j,k)==0.).and.(derivx(chi,i,j,k)==0.).and.(derivy(chi,i,j,k)==0.).and.(derivz(chi,i,j,k)==0.))) then
                !penal
                !-----
!!$                facx=1.D0/(12.D0*dx)
!!$                facy=1.D0/(12.D0*dy)
!!$                facz=1.D0/(12.D0*dz)
!!$
!!$                tx(2)=facx*(peny(im2,j,k)-8.D0*peny(im1,j,k)+8.D0*peny(ip1,j,k)-peny(ip2,j,k))
!!$                tx(3)=facx*(penz(im2,j,k)-8.D0*penz(im1,j,k)+8.D0*penz(ip1,j,k)-penz(ip2,j,k))
!!$
!!$                ty(1)=facy*(penx(i,jm2,k)-8.D0*penx(i,jm1,k)+8.D0*penx(i,jp1,k)-penx(i,jp2,k))
!!$                ty(3)=facy*(penz(i,jm2,k)-8.D0*penz(i,jm1,k)+8.D0*penz(i,jp1,k)-penz(i,jp2,k))
!!$
!!$
!!$                tz(1)=facz*(penx(i,j,km2)-8.D0*penx(i,j,km1)+8.D0*penx(i,j,kp1)-penx(i,j,kp2))
!!$                tz(2)=facz*(peny(i,j,km2)-8.D0*peny(i,j,km1)+8.D0*peny(i,j,kp1)-peny(i,j,kp2))


!!$
!!$                !stxg(i,j,k)=stxg(i,j,k)+ty(3)-tz(2)
!!$                !styg(i,j,k)=styg(i,j,k)+tz(1)-tx(3)
!!$                !stzg(i,j,k)=stzg(i,j,k)+tx(2)-ty(1)
!!$
!!$                omgx(i,j,k)=ty(3)-tz(2)
!!$                omgy(i,j,k)=tz(1)-tx(3)
!!$                omgz(i,j,k)=tx(2)-ty(1)
!!$
!!$                !explicite
!!$                !omgx(i,j,k)=omgx(i,j,k)+dt*(ty(3)-tz(2))
!!$                !omgy(i,j,k)=omgy(i,j,k)+dt*(tz(1)-tx(3))
!!$                !omgz(i,j,k)=omgz(i,j,k)+dt*(tx(2)-ty(1))

!!$                tx=rot_vec_gradsca(vxg,vyg,vzg,exp(-lambda*chi*dt),i,j,k)
!!$                
!!$                omgx(i,j,k)=omgx(i,j,k)*exp(-lambda*chi(i,j,k)*dt)-tx(1)
!!$                omgy(i,j,k)=omgy(i,j,k)*exp(-lambda*chi(i,j,k)*dt)-tx(2)
!!$                omgz(i,j,k)=omgz(i,j,k)*exp(-lambda*chi(i,j,k)*dt)-tx(3)

!!$                tx=rot_vec_gradsca(vxg,vyg,vzg,1./(1.+lambda*dt*chi),i,j,k)
!!$                
!!$                omgx(i,j,k)=omgx(i,j,k)/(1.+lambda*chi(i,j,k)*dt)-tx(1)
!!$                omgy(i,j,k)=omgy(i,j,k)/(1.+lambda*chi(i,j,k)*dt)-tx(2)
!!$                omgz(i,j,k)=omgz(i,j,k)/(1.+lambda*chi(i,j,k)*dt)-tx(3)

!!$                tx=rot_vec_gradsca(vxg,vyg,vzg,1./(1.+lambda*dt*chi),i,j,k)
!!$                
!!$                omgx(i,j,k)=omgx(i,j,k)*(1.-lambda*chi(i,j,k)*dt)-lambda*dt*tx(1)
!!$                omgy(i,j,k)=omgy(i,j,k)*(1.-lambda*chi(i,j,k)*dt)-lambda*dt*tx(2)
!!$                omgz(i,j,k)=omgz(i,j,k)*(1.-lambda*chi(i,j,k)*dt)-lambda*dt*tx(3)
              

!!$                tx=rot_vec_gradsca(vxg,vyg,vzg,chi,i,j,k)
!!$
!!$                omgx(i,j,k)=omgx(i,j,k)+dt*lambda*tx(1)
!!$                omgy(i,j,k)=omgy(i,j,k)+dt*lambda*tx(2)
!!$                omgz(i,j,k)=omgz(i,j,k)+dt*lambda*tx(3)
!!$
!!$
!!$             end if
          end do
       end do
    end do

  contains
    function penx (l,m,n)     
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: penx


      penx=vxg(l,m,n)*exp(-lambda(l,m,n)*chi(l,m,n)*dt)
      !penx=vxg(l,m,n)/(1.+lambda*chi(l,m,n)*dt)
      !penx=-lambda*chi(l,m,n)*vxg(l,m,n)
      !penx=vxg(l,m,n)

    end function penx

    function peny (l,m,n)     
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: peny


      peny=vyg(l,m,n)*exp(-lambda(l,m,n)*chi(l,m,n)*dt)
      !peny=vyg(l,m,n)/(1.+lambda*chi(l,m,n)*dt)
      !peny=-lambda*chi(l,m,n)*vyg(l,m,n)
      !peny=vyg(l,m,n)

    end function peny

    function penz (l,m,n)      
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: penz


      penz=vzg(l,m,n)*exp(-lambda(l,m,n)*chi(l,m,n)*dt)
      !penz=vzg(l,m,n)/(1.+lambda*chi(l,m,n)*dt)
      !penz=-lambda*chi(l,m,n)*vzg(l,m,n)
      !penz=vzg(l,m,n)

    end function penz
   

    function rot_vec_gradsca(vecx,vecy,vecz,sca,l,m,n) result(r)
      implicit none
      real(kind=8),dimension(:,:,:)       :: vecx,vecy,vecz,sca
      integer                             :: l,m,n
      real(kind=8),dimension(1:3)         :: r

      integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2
      real(kind=8)                  :: facx,facy,facz,dxsca,dysca,dzsca
 
      facx=1.D0/(12.D0*dx)
      facy=1.D0/(12.D0*dy)
      facz=1.D0/(12.D0*dz)

      ip1=mod(l+1+nx-1,nx)+1
      im1=mod(l-1+nx-1,nx)+1
      ip2=mod(l+2+nx-1,nx)+1
      im2=mod(l-2+nx-1,nx)+1
      
      jp1=mod(m+1+ny-1,ny)+1
      jm1=mod(m-1+ny-1,ny)+1
      jp2=mod(m+2+ny-1,ny)+1
      jm2=mod(m-2+ny-1,ny)+1
      
      kp1=mod(n+1+nz-1,nz)+1
      km1=mod(n-1+nz-1,nz)+1
      kp2=mod(n+2+nz-1,nz)+1
      km2=mod(n-2+nz-1,nz)+1
      
      dxsca=facx*(sca(im2,j,k)-8.*sca(im1,j,k)+8.*sca(ip1,j,k)-sca(ip2,j,k))
      dysca=facy*(sca(i,jm2,k)-8.*sca(i,jm1,k)+8.*sca(i,jp1,k)-sca(i,jp2,k))
      dzsca=facz*(sca(i,j,km2)-8.*sca(i,j,km1)+8.*sca(i,j,kp1)-sca(i,j,kp2))

      r(1)=vecy(l,m,n)*dzsca-vecz(l,m,n)*dysca
      r(2)=vecz(l,m,n)*dxsca-vecx(l,m,n)*dzsca
      r(3)=vecx(l,m,n)*dysca-vecy(l,m,n)*dxsca

    end function rot_vec_gradsca
      

  end subroutine strech_diff_penal




  subroutine rotv_df4
      implicit none
      integer                             :: l,m,n,i,j,k

      integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2
      real(kind=8)                  :: facx,facy,facz,dxsca,dysca,dzsca
 
      !omgx = dy vz - dz vy
      !omgy = dz vx - dx vz
      !omgz = dx vy - dy vx

      facx=1.D0/(12.D0*dx)
      facy=1.D0/(12.D0*dy)
      facz=1.D0/(12.D0*dz)

      do k=1,nz
         do j=1,ny
            do i=1,nx

               ip1=mod(i+1+nx-1,nx)+1
               im1=mod(i-1+nx-1,nx)+1
               ip2=mod(i+2+nx-1,nx)+1
               im2=mod(i-2+nx-1,nx)+1
               
               jp1=mod(j+1+ny-1,ny)+1
               jm1=mod(j-1+ny-1,ny)+1
               jp2=mod(j+2+ny-1,ny)+1
               jm2=mod(j-2+ny-1,ny)+1
               
               kp1=mod(k+1+nz-1,nz)+1
               km1=mod(k-1+nz-1,nz)+1
               kp2=mod(k+2+nz-1,nz)+1
               km2=mod(k-2+nz-1,nz)+1

               omgx(i,j,k)=facy*(vzg(i,jm2,k)-8.*vzg(i,jm1,k)+8.*vzg(i,jp1,k)-vzg(i,jp2,k)) &
                          -facz*(vyg(i,j,km2)-8.*vyg(i,j,km1)+8.*vyg(i,j,kp1)-vyg(i,j,kp2))
               omgy(i,j,k)=facz*(vxg(i,j,km2)-8.*vxg(i,j,km1)+8.*vxg(i,j,kp1)-vxg(i,j,kp2)) &
                          -facx*(vzg(im2,j,k)-8.*vzg(im1,j,k)+8.*vzg(ip1,j,k)-vzg(ip2,j,k))
               omgz(i,j,k)=facx*(vyg(im2,j,k)-8.*vyg(im1,j,k)+8.*vyg(ip1,j,k)-vyg(ip2,j,k)) &
                          -facy*(vxg(i,jm2,k)-8.*vxg(i,jm1,k)+8.*vxg(i,jp1,k)-vxg(i,jp2,k))
            end do
         end do
      end do

    end subroutine rotv_df4

    subroutine rotv_df2
      implicit none
      integer                             :: l,m,n,i,j,k

      integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2
      real(kind=8)                  :: facx,facy,facz,dxsca,dysca,dzsca
 
      !omgx = dy vz - dz vy
      !omgy = dz vx - dx vz
      !omgz = dx vy - dy vx

      facx=1.D0/(2.D0*dx)
      facy=1.D0/(2.D0*dy)
      facz=1.D0/(2.D0*dz)

      do k=1,nz
         do j=1,ny
            do i=1,nx

               ip1=mod(i+1+nx-1,nx)+1
               im1=mod(i-1+nx-1,nx)+1
               
               jp1=mod(j+1+ny-1,ny)+1
               jm1=mod(j-1+ny-1,ny)+1
               
               kp1=mod(k+1+nz-1,nz)+1
               km1=mod(k-1+nz-1,nz)+1

               omgx(i,j,k)=facy*(-vzg(i,jm1,k)+vzg(i,jp1,k)) &
                          -facz*(-vyg(i,j,km1)+vyg(i,j,kp1))
               omgy(i,j,k)=facz*(-vxg(i,j,km1)+vxg(i,j,kp1)) &
                          -facx*(-vzg(im1,j,k)+vzg(ip1,j,k))
               omgz(i,j,k)=facx*(-vyg(im1,j,k)+vyg(ip1,j,k)) &
                          -facy*(-vxg(i,jm1,k)+vxg(i,jp1,k))
            end do
         end do
      end do

    end subroutine rotv_df2







end module old_mod
