module penal_mod 
  use donnees_mod
  use tab_mod 
  use nrtype 
  USE nrutil
  USE nr 
contains

  subroutine make_chi
    implicit none

    integer       :: i,j,k
    real(kind=8)  :: x,y,z,d,rayon,centrex,centrey,centrez,epaisseur !,dist

    do i=1,nx
       x=xtab(i)
       do j=1,ny
          y=ytab(j)
          do k=1,nz
             z=ztab(k)

             centrex=0.
             centrey=0.
             centrez=0.

             !========================================
             !       sphere
             !========================================
!!$             rayon=0.5
!!$             d=dist(centrex,centrey,centrez,rayon,x,y,z)
!!$             chi(i,j,k)=carac(d,0.*dx) 

             !========================================
             !       poiseuil
             !========================================
!!$             chi(i,j,k)=0.
!!$             if ( (z>1.).or.(z<-1.) ) then
!!$                chi(i,j,k)=1.
!!$             end if

             !========================================
             !       poiseuil/Sphere
             !========================================
             chi(i,j,k)=0.
!             if ( (y<(1./0.4844)).and.(y>-(1./0.4844)) ) then
 	     if ( (y<=0.).and.(y>-(1./0.4844)) ) then
                !chi(i,j,k)=0.
                rayon=0.5
                d=dist(centrex,centrey,centrez,rayon,x,y,z)
                chi(i,j,k)=carac(d,0.*dx) 
             end if

             chi_sphere(i,j,k)=0.
!             if ( (y<(1./0.4844)).and.(y>-(1./0.4844)) ) then
             if ( (y<=0.).and.(y>-(1./0.4844)) ) then
                rayon=0.5
                d=dist(centrex,centrey,centrez,rayon,x,y,z)
                chi_sphere(i,j,k)=carac(d,0.*dx) 
             end if

             lambda(i,j,k)=lambda_flu
	     rayon=0.5
	     epaisseur=2*rayon*0.1
	     !dist=sqrt((x-centrex)**2+(y-centrey)**2+(z-centrez)**2)
             if ( (y<=0.).and.(sqrt((x-centrex)**2+(y-centrey)**2+(z-centrez)**2)<=(rayon-epaisseur)) ) then
                lambda(i,j,k)=lambda_sol
	     else 
		if ( (y<=0.).and.(sqrt((x-centrex)**2+(y-centrey)**2+(z-centrez)**2)>(rayon-epaisseur)) &
		.and.(sqrt((x-centrex)**2+(y-centrey)**2+(z-centrez)**2)<=rayon) ) then
	             lambda(i,j,k)=lambda_por
	        end if
             end if

!!$             !========================================
!!$             !       rien
!!$             !========================================
!!$             chi(i,j,k)=0.

             !========================================
             !       sphere en dirichlet
             !========================================
!!$             chi(i,j,k)=0.
!!$             if ( (z<2.0645).or.(z>-2.0645) ) then
!!$                rayon=0.5
!!$                d=dist(centrex,centrey,centrez,rayon,x,y,z)
!!$                chi(i,j,k)=carac(d,0.*dx) 
!!$             end if
            
          end do
       end do
    end do

  end subroutine make_chi


  function carac(d,eps_c)    
    implicit none
    real(kind=8),intent(in)        :: d,eps_c
    real(kind=8)                   :: carac
    pi=4.*atan(1.0)

    if (d<=-eps_c) carac= 1.
    if (d>eps_c) carac= 0.
    if((d<=eps_c).and.(d>-eps_c)) carac=0.5-0.5*d/eps_c-sin(pi*d/eps_c)/(2.*pi)

    !if((d<=eps_c).and.(d>-eps_c)) carac=sin(pi*(eps_c-d)/(4.*eps_c))

  end function carac

  function dist (cx,cy,cz,r,px,py,pz)     
    implicit none

    real(kind=8),intent(in)        :: cx,cy,cz,r,px,py,pz
    real(kind=8)                   :: dist

    dist=sqrt((cx-px)**2+(py-cy)**2+(pz-cz)**2)-r

  end function dist




  subroutine penal_fft
    implicit none

    complex(kind=8),dimension(:,:,:),pointer   :: sf1x, sf1y, sf1z
    complex(kind=8),dimension(:,:,:),pointer   :: su1x, su1y, su1z
    complex(kind=8),dimension(:,:),pointer     :: sf2x, sf2y, sf2z
    complex(kind=8),dimension(:,:),pointer     :: su2x, su2y, su2z 

    integer                                     :: i,j,k,m,n,p
    real(kind=8)                                :: fac,x,y,z,l1,l2,l3,kx,ky,kz,r2,coeff,uinfx,uinfy,tmp
    complex(kind=8)                             :: coef


    l1=(xd-xg)
    l2=(yh-yb)
    l3=(zf-zd)

    allocate (sf1x(1:nx/2,1:ny,1:nz),su1x(1:nx/2,1:ny,1:nz) )
    allocate (sf1y(1:nx/2,1:ny,1:nz),su1y(1:nx/2,1:ny,1:nz) )
    allocate (sf1z(1:nx/2,1:ny,1:nz),su1z(1:nx/2,1:ny,1:nz) )
    allocate (sf2x(1:ny,1:nz),su2x(1:ny,1:nz) )
    allocate (sf2y(1:ny,1:nz),su2y(1:ny,1:nz) )
    allocate (sf2z(1:ny,1:nz),su2z(1:ny,1:nz) )


    !fft forward
    call rlft3(vxg,sf1x,sf2x,1)
    call rlft3(vyg,sf1y,sf2y,1)
    call rlft3(vzg,sf1z,sf2z,1)

    !frequence=0 : n=0
    !0<f<fc : 1<=n<=N/2-1
    !f=fc=-fc : n=N/2
    !-fc<f<0 : N/2+1<=n<=N-1 

    fac=2./(nx*ny*nz) ! pour normalisation de la fft inverse
    coef=-fac*cmplx(0.,1.)*2.*pi

    !calcul de la solution dans l'espace des frequences 
    do p=1,nz
       if ( (p-1)<=(nz/2) ) then
          kz=(p-1)/l3
       else
          kz=(p-1-nz)/l3
       end if
       do n=1,ny
          if ( (n-1)<=(ny/2) ) then
             ky=(n-1)/l2
          else
             ky=(n-1-ny)/l2
          end if
          do m=1,nx/2           

             if ( (m-1)<=(nx/2) ) then
                kx=(m-1)/l1
             else
                kx=(m-1-nx)/l1
             end if



             su1x(m,n,p)=coef*(ky*sf1z(m,n,p)-kz*sf1y(m,n,p))
             su1y(m,n,p)=coef*(kz*sf1x(m,n,p)-kx*sf1z(m,n,p))              
             su1z(m,n,p)=coef*(kx*sf1y(m,n,p)-ky*sf1x(m,n,p))

          end do
       end do
    end do

    m=nx/2+1
    kx=(m-1)/l1
    do p=1,nz
       if ( (p-1)<=(nz/2) ) then
          kz=(p-1)/l3
       else
          kz=(p-1-nz)/l3
       end if
       do n=1,ny

          if ( (n-1)<=(ny/2) ) then
             ky=(n-1)/l2
          else
             ky=(n-1-ny)/l2
          end if
       
        
          su2x(n,p)=coef*(ky*sf2z(n,p)-kz*sf2y(n,p))                 
          su2y(n,p)=coef*(kz*sf2x(n,p)-kx*sf2z(n,p))               
          su2z(n,p)=coef*(kx*sf2y(n,p)-ky*sf2x(n,p))


        
       end do
    end do

    !fft backward   

    call rlft3(omgx,su1x,su2x,-1)
    call rlft3(omgy,su1y,su2y,-1)
    call rlft3(omgz,su1z,su2z,-1)

    deallocate(sf1x,su1x)
    deallocate(sf1y,su1y)
    deallocate(sf1z,su1z)
    deallocate(sf2x,su2x)
    deallocate(sf2y,su2y)
    deallocate(sf2z,su2z)


  end subroutine penal_fft


end module penal_mod
