module remaillage_mod
  use donnees_mod
  use tab_mod
contains

  subroutine remaill_l2 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(1:4)                              :: ip,jp,kp
    real(kind=8),dimension(1:4)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1,x2,x3,x4,x5,y2,y3,y4,y5,z2,z3,z4,z5

    remaille=0.D0      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)           
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2

       jp(2) = floor((posy(i)-yb)/dy)                 
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2

       kp(2) = floor((posz(i)-zd)/dz)                 
       kp(1) = kp(2) - 1         
       kp(3) = kp(2) + 1
       kp(4) = kp(2) + 2



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(2),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=1,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=1,4               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=1,4               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
       if (xx1<=0.5D0) then   

          poidx(1)=-0.5D0*xx1*(1.D0-xx1)
          poidx(2)=1.D0-xx1**2
          poidx(3)=0.5D0*xx1*(1.D0+xx1)
          poidx(4)=0.D0

       else

          poidx(1)=0.D0
          poidx(2)=0.5D0*(xx1-1.D0)*(xx1-2.D0)
          poidx(3)=xx1*(2.D0-xx1)
          poidx(4)=0.5D0*xx1*(xx1-1.D0)

       end if

       if (yy1<=0.5D0) then

          poidy(1)=-0.5D0*yy1*(1.D0-yy1)
          poidy(2)=1.D0-yy1**2
          poidy(3)=0.5D0*yy1*(1.D0+yy1)
          poidy(4)=0.D0

       else

          poidy(1)=0.D0
          poidy(2)=0.5D0*(yy1-1.D0)*(yy1-2.D0)
          poidy(3)=yy1*(2.D0-yy1)
          poidy(4)=0.5D0*yy1*(yy1-1.D0)        

       end if

       if (zz1<=0.5D0) then

          poidz(1)=-0.5D0*zz1*(1.D0-zz1)
          poidz(2)=1.D0-zz1**2
          poidz(3)=0.5D0*zz1*(1.D0+zz1)
          poidz(4)=0.D0

       else

          poidz(1)=0.D0
          poidz(2)=0.5D0*(zz1-1.D0)*(zz1-2.D0)
          poidz(3)=zz1*(2.D0-zz1)
          poidz(4)=0.5D0*zz1*(zz1-1.D0)        

       end if



       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=1,4
          do d=1,4
             do c=1,4              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_l2

  subroutine remaill_l4 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(0:5)                              :: ip,jp,kp
    real(kind=8),dimension(0:5)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1,x2,x3,x4,x5,y2,y3,y4,y5,z2,z3,z4,z5

    remaille=0.D0      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3

       kp(2) = floor((posz(i)-zd)/dz)         
       kp(0) = kp(2) - 2        
       kp(1) = kp(2) - 1         
       kp(3) = kp(2) + 1
       kp(4) = kp(2) + 2
       kp(5) = kp(2) + 3



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(2),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=0,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=0,5               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4            
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4            
       z2=zz1**2
       z3=zz1**3
       z4=zz1**4            

       if (xx1<=0.5D0) then
          poidx(0)=(2.D0*xx1-x2-2.D0*x3+x4)/24.D0
          poidx(1)=(-4.D0*xx1+4.D0*x2+x3-x4)/6.D0
          poidx(2)=1.D0+(-5.D0*x2+x4)/4.D0
          poidx(3)=(4.D0*xx1+4.D0*x2-x3-x4)/6.D0
          poidx(4)=(-2.D0*xx1-x2+2.D0*x3+x4)/24.D0
          poidx(5)=0.D0
       else
          poidx(0)=0.D0
          poidx(1)=(-6.D0*xx1+11.D0*x2-6.D0*x3+x4)/24.D0
          poidx(2)=1.D0+(-5.D0*xx1-5.D0*x2+5.D0*x3-x4)/6.D0
          poidx(3)=(6.D0*xx1+x2-4.D0*x3+x4)/4.D0
          poidx(4)=(-3.D0*xx1+x2+3.D0*x3-x4)/6.D0
          poidx(5)=(2.D0*xx1-x2-2.D0*x3+x4)/24.D0
       end if

       if (yy1<=0.5D0) then
          poidy(0)=(2.D0*yy1-y2-2.D0*y3+y4)/24.D0
          poidy(1)=(-4.D0*yy1+4.D0*y2+y3-y4)/6.D0
          poidy(2)=1.D0+(-5.D0*y2+y4)/4.D0
          poidy(3)=(4.D0*yy1+4.D0*y2-y3-y4)/6.D0
          poidy(4)=(-2.D0*yy1-y2+2.D0*y3+y4)/24.D0
          poidy(5)=0.D0
       else
          poidy(0)=0.D0
          poidy(1)=(-6.D0*yy1+11.D0*y2-6.D0*y3+y4)/24.D0
          poidy(2)=1.D0+(-5.D0*yy1-5.D0*y2+5.D0*y3-y4)/6.D0
          poidy(3)=(6.D0*yy1+y2-4.D0*y3+y4)/4.D0
          poidy(4)=(-3.D0*yy1+y2+3.D0*y3-y4)/6.D0
          poidy(5)=(2.D0*yy1-y2-2.D0*y3+y4)/24.D0
       end if
       if (zz1<=0.5D0) then
          poidz(0)=(2.D0*zz1-z2-2.D0*z3+z4)/24.D0
          poidz(1)=(-4.D0*zz1+4.D0*z2+z3-z4)/6.D0
          poidz(2)=1.D0+(-5.D0*z2+z4)/4.D0
          poidz(3)=(4.D0*zz1+4.D0*z2-z3-z4)/6.D0
          poidz(4)=(-2.D0*zz1-z2+2.D0*z3+z4)/24.D0
          poidz(5)=0.D0
       else
          poidz(0)=0.D0
          poidz(1)=(-6.D0*zz1+11.D0*z2-6.D0*z3+z4)/24.D0
          poidz(2)=1.D0+(-5.D0*zz1-5.D0*z2+5.D0*z3-z4)/6.D0
          poidz(3)=(6.D0*zz1+z2-4.D0*z3+z4)/4.D0
          poidz(4)=(-3.D0*zz1+z2+3.D0*z3-z4)/6.D0
          poidz(5)=(2.D0*zz1-z2-2.D0*z3+z4)/24.D0
       end if



       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=0,5
          do d=0,5
             do c=0,5              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_l4

  subroutine remaill_l5 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(0:5)                              :: ip,jp,kp
    real(kind=8),dimension(0:5)                         :: poidx,poidy,poidz
    real(kind=8)                                        :: xx1,yy1,zz1,x2,x3,x4,x5,y2,y3,y4,y5,z2,z3,z4,z5

    remaille=0.D0      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3

       kp(2) = floor((posz(i)-zd)/dz)         
       kp(0) = kp(2) - 2        
       kp(1) = kp(2) - 1         
       kp(3) = kp(2) + 1
       kp(4) = kp(2) + 2
       kp(5) = kp(2) + 3



       !distance de la particule à remailler au second point 
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy 
       zz1 = (posz(i) - real(kp(2),kind=8)*dz-zd)/dz  


       !conditions au bord
       !------------------
       !periodique: 

       do c=0,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=0,5               
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !----------------   
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4
       y5=yy1**5
       z2=zz1**2
       z3=zz1**3
       z4=zz1**4
       z5=zz1**5

       poidx(0)=xx1/20.D0-x2/24.D0-x3/24.D0+x4/24.D0-x5/120.D0
       poidx(1)=-xx1/2.D0+2.D0*x2/3.D0-x3/24.D0-x4/6.D0+x5/24.D0
       poidx(2)=1.D0-xx1/3.D0-5*x2/4.D0+5.D0*x3/12.D0+x4/4.D0-x5/12.D0
       poidx(3)=xx1+2.D0*x2/3.D0-7.D0*x3/12.D0-x4/6.D0+x5/12.D0
       poidx(4)=-xx1/4.D0-x2/24.D0+7.D0*x3/24.D0+x4/24.D0-x5/24.D0
       poidx(5)=xx1/30.D0-x3/24.D0+x5/120.D0

       poidy(0)=yy1/20.D0-y2/24.D0-y3/24.D0+y4/24.D0-y5/120.D0
       poidy(1)=-yy1/2.D0+2.D0*y2/3.D0-y3/24.D0-y4/6.D0+y5/24.D0
       poidy(2)=1.D0-yy1/3.D0-5*y2/4.D0+5.D0*y3/12.D0+y4/4.D0-y5/12.D0
       poidy(3)=yy1+2.D0*y2/3.D0-7.D0*y3/12.D0-y4/6.D0+y5/12.D0
       poidy(4)=-yy1/4.D0-y2/24.D0+7.D0*y3/24.D0+y4/24.D0-y5/24.D0
       poidy(5)=yy1/30.D0-y3/24.D0+y5/120.D0

       poidz(0)=zz1/20.D0-z2/24.D0-z3/24.D0+z4/24.D0-z5/120.D0
       poidz(1)=-zz1/2.D0+2.D0*z2/3.D0-z3/24.D0-z4/6.D0+z5/24.D0
       poidz(2)=1.D0-zz1/3.D0-5*z2/4.D0+5.D0*z3/12.D0+z4/4.D0-z5/12.D0
       poidz(3)=zz1+2.D0*z2/3.D0-7.D0*z3/12.D0-z4/6.D0+z5/12.D0
       poidz(4)=-zz1/4.D0-z2/24.D0+7.D0*z3/24.D0+z4/24.D0-z5/24.D0
       poidz(5)=zz1/30.D0-z3/24.D0+z5/120.D0

       !remaillage à l' interrieur domaine
       !---------------------------------   
       do e=0,5
          do d=0,5
             do c=0,5              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do
  end subroutine remaill_l5


  subroutine remaill_l5_x(donne,posx,posy,posz,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k,kvar,jvar
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,x2,x3,x4,x5

    remaille=0.D0      

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------   
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx                         

       !conditions au bord
       !------------------         

       !periodique: 
       do c=0,5          
          ip(c)=mod(ip(c)+nx,nx)            
       end do



       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5

       poidx(0)=xx1/20.D0-x2/24.D0-x3/24.D0+x4/24.D0-x5/120.D0
       poidx(1)=-xx1/2.D0+2.D0*x2/3.D0-x3/24.D0-x4/6.D0+x5/24.D0
       poidx(2)=1.D0-xx1/3.D0-5*x2/4.D0+5.D0*x3/12.D0+x4/4.D0-x5/12.D0
       poidx(3)=xx1+2.D0*x2/3.D0-7.D0*x3/12.D0-x4/6.D0+x5/12.D0
       poidx(4)=-xx1/4.D0-x2/24.D0+7.D0*x3/24.D0+x4/24.D0-x5/24.D0
       poidx(5)=xx1/30.D0-x3/24.D0+x5/120.D0

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ip(c)+1,j,kvar)=remaille(ip(c)+1,j,kvar)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l5_x


  subroutine remaill_l5_y(donne,posx,posy,posz,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k,kvar,jvar
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,x2,x3,x4,x5

    remaille=0.D0      

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posy(i)-yb)/dy)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------   
       xx1 = (posy(i) - real(ip(2),kind=8)*dy-yb)/dy                         

       !conditions au bord
       !------------------         

       !periodique: 
       do c=0,5          
          ip(c)=mod(ip(c)+ny,ny)            
       end do



       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5

       poidx(0)=xx1/20.D0-x2/24.D0-x3/24.D0+x4/24.D0-x5/120.D0
       poidx(1)=-xx1/2.D0+2.D0*x2/3.D0-x3/24.D0-x4/6.D0+x5/24.D0
       poidx(2)=1.D0-xx1/3.D0-5*x2/4.D0+5.D0*x3/12.D0+x4/4.D0-x5/12.D0
       poidx(3)=xx1+2.D0*x2/3.D0-7.D0*x3/12.D0-x4/6.D0+x5/12.D0
       poidx(4)=-xx1/4.D0-x2/24.D0+7.D0*x3/24.D0+x4/24.D0-x5/24.D0
       poidx(5)=xx1/30.D0-x3/24.D0+x5/120.D0

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l5_y


  subroutine remaill_l5_z(donne,posx,posy,posz,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k,kvar,jvar
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,x2,x3,x4,x5

    remaille=0.D0      

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1
       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posz(i)-zd)/dz)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------   
       xx1 = (posz(i) - real(ip(2),kind=8)*dz-zd)/dz                         

       !conditions au bord
       !------------------         

       !periodique: 
       do c=0,5          
          ip(c)=mod(ip(c)+nz,nz)            
       end do



       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5

       poidx(0)=xx1/20.D0-x2/24.D0-x3/24.D0+x4/24.D0-x5/120.D0
       poidx(1)=-xx1/2.D0+2.D0*x2/3.D0-x3/24.D0-x4/6.D0+x5/24.D0
       poidx(2)=1.D0-xx1/3.D0-5*x2/4.D0+5.D0*x3/12.D0+x4/4.D0-x5/12.D0
       poidx(3)=xx1+2.D0*x2/3.D0-7.D0*x3/12.D0-x4/6.D0+x5/12.D0
       poidx(4)=-xx1/4.D0-x2/24.D0+7.D0*x3/24.D0+x4/24.D0-x5/24.D0
       poidx(5)=xx1/30.D0-x3/24.D0+x5/120.D0


       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l5_z














  subroutine remaill_l6 (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb
    integer,dimension(-3:4)                             :: ip,jp,kp
    real(kind=8),dimension(-3:4)                        :: poidx,poidy,poidz
    real(kind=8)                                        :: xx,yy,zz

    remaille=0.D0      


    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4

       jp(0) = floor((posy(i)-yb)/dy)         
       jp(-1) = jp(0) - 1        
       jp(-2) = jp(0) - 2         
       jp(-3) = jp(0) - 3
       jp(1) = jp(0) + 1
       jp(2) = jp(0) + 2
       jp(3) = jp(0) + 3
       jp(4) = jp(0) + 4

       kp(0) = floor((posz(i)-zd)/dz)         
       kp(-1) = kp(0) - 1        
       kp(-2) = kp(0) - 2         
       kp(-3) = kp(0) - 3
       kp(1) = kp(0) + 1
       kp(2) = kp(0) + 2
       kp(3) = kp(0) + 3
       kp(4) = kp(0) + 4




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif
       yy = (posy(i) - real(jp(0),kind=8)*dy-yb)/dy   !relatif
       zz = (posz(i) - real(kp(0),kind=8)*dz-zd)/dz   !relatif



       !conditions au bord
       !------------------
       !periodique:          
       do c=-3,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=-3,4              
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       do c=-3,4              
          kp(c)=mod(kp(c)+nz,nz)                
       end do


       !calcul des poids
       !---------------- 
       if (xx<=0.5D0) then
          poidx(-3)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
          poidx(-2)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+3.D0)/120.D0
          poidx(-1)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(0)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/36.D0
          poidx(1)=(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(2)=-(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/120.D0
          poidx(3)=(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/720.D0
          poidx(4)=0.D0
       else
          poidx(-3)=0.D0
          poidx(-2)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)/720.D0
          poidx(-1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)/120.D0
          poidx(0)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)/36.D0
          poidx(2)=(xx-4.D0)*(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(3)=-(xx-4.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/120.D0
          poidx(4)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
       end if

       if (yy<=0.5D0) then
          poidy(-3)=(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*yy*(yy+1.D0)*(yy+2.D0)/720.D0
          poidy(-2)=-(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*yy*(yy+1.D0)*(yy+3.D0)/120.D0
          poidy(-1)=(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*yy*(yy+2.D0)*(yy+3.D0)/48.D0
          poidy(0)=-(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*(yy+1.D0)*(yy+2.D0)*(yy+3.D0)/36.D0
          poidy(1)=(yy-3.D0)*(yy-2.D0)*yy*(yy+1.D0)*(yy+2.D0)*(yy+3.D0)/48.D0
          poidy(2)=-(yy-3.D0)*(yy-1.D0)*yy*(yy+1.D0)*(yy+2.D0)*(yy+3.D0)/120.D0
          poidy(3)=(yy-2.D0)*(yy-1.D0)*yy*(yy+1.D0)*(yy+2.D0)*(yy+3.D0)/720.D0
          poidy(4)=0.D0
       else
          poidy(-3)=0.D0
          poidy(-2)=(yy-4.D0)*(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*yy*(yy+1.D0)/720.D0
          poidy(-1)=-(yy-4.D0)*(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*yy*(yy+2.D0)/120.D0
          poidy(0)=(yy-4.D0)*(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*(yy+1.D0)*(yy+2.D0)/48.D0
          poidy(1)=-(yy-4.D0)*(yy-3.D0)*(yy-2.D0)*yy*(yy+1.D0)*(yy+2.D0)/36.D0
          poidy(2)=(yy-4.D0)*(yy-3.D0)*(yy-1.D0)*yy*(yy+1.D0)*(yy+2.D0)/48.D0
          poidy(3)=-(yy-4.D0)*(yy-2.D0)*(yy-1.D0)*yy*(yy+1.D0)*(yy+2.D0)/120.D0
          poidy(4)=(yy-3.D0)*(yy-2.D0)*(yy-1.D0)*yy*(yy+1.D0)*(yy+2.D0)/720.D0
       end if

       if (zz<=0.5D0) then
          poidz(-3)=(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*zz*(zz+1.D0)*(zz+2.D0)/720.D0
          poidz(-2)=-(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*zz*(zz+1.D0)*(zz+3.D0)/120.D0
          poidz(-1)=(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*zz*(zz+2.D0)*(zz+3.D0)/48.D0
          poidz(0)=-(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*(zz+1.D0)*(zz+2.D0)*(zz+3.D0)/36.D0
          poidz(1)=(zz-3.D0)*(zz-2.D0)*zz*(zz+1.D0)*(zz+2.D0)*(zz+3.D0)/48.D0
          poidz(2)=-(zz-3.D0)*(zz-1.D0)*zz*(zz+1.D0)*(zz+2.D0)*(zz+3.D0)/120.D0
          poidz(3)=(zz-2.D0)*(zz-1.D0)*zz*(zz+1.D0)*(zz+2.D0)*(zz+3.D0)/720.D0
          poidz(4)=0.D0
       else
          poidz(-3)=0.D0
          poidz(-2)=(zz-4.D0)*(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*zz*(zz+1.D0)/720.D0
          poidz(-1)=-(zz-4.D0)*(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*zz*(zz+2.D0)/120.D0
          poidz(0)=(zz-4.D0)*(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*(zz+1.D0)*(zz+2.D0)/48.D0
          poidz(1)=-(zz-4.D0)*(zz-3.D0)*(zz-2.D0)*zz*(zz+1.D0)*(zz+2.D0)/36.D0
          poidz(2)=(zz-4.D0)*(zz-3.D0)*(zz-1.D0)*zz*(zz+1.D0)*(zz+2.D0)/48.D0
          poidz(3)=-(zz-4.D0)*(zz-2.D0)*(zz-1.D0)*zz*(zz+1.D0)*(zz+2.D0)/120.D0
          poidz(4)=(zz-3.D0)*(zz-2.D0)*(zz-1.D0)*zz*(zz+1.D0)*(zz+2.D0)/720.D0
       end if




       !remaillage à l' interrieur domaine
       !---------------------------------         
       do e=-3,4
          do d=-3,4
             do c=-3,4              
                remaille(ip(c)+1,jp(d)+1,kp(e)+1)=remaille(ip(c)+1,jp(d)+1,kp(e)+1)+donne(i)*poidx(c)*poidy(d)*poidz(e)
             end do
          end do
       end do
    end do

  end subroutine remaill_l6

  subroutine remaill_l6_x (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-3:4)                             :: ip
    real(kind=8),dimension(-3:4)                        :: poidx
    real(kind=8)                                        :: xx

    remaille=0.D0      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4




       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif




       !conditions au bord
       !------------------
       !periodique:          
       do c=-3,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do



       !calcul des poids
       !---------------- 
       if (xx<=0.5D0) then
          poidx(-3)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
          poidx(-2)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+3.D0)/120.D0
          poidx(-1)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(0)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/36.D0
          poidx(1)=(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(2)=-(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/120.D0
          poidx(3)=(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/720.D0
          poidx(4)=0.D0
       else
          poidx(-3)=0.D0
          poidx(-2)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)/720.D0
          poidx(-1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)/120.D0
          poidx(0)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)/36.D0
          poidx(2)=(xx-4.D0)*(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(3)=-(xx-4.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/120.D0
          poidx(4)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
       end if





       !remaillage à l' interrieur domaine
       !---------------------------------               
       do c=-3,4              
          remaille(ip(c)+1,jvar,kvar)=remaille(ip(c)+1,jvar,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l6_x



  subroutine remaill_l6_y (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ivar,jvar,kvar
    integer,dimension(-3:4)                             :: ip
    real(kind=8),dimension(-3:4)                        :: poidx
    real(kind=8)                                        :: xx

    remaille=0.D0      


    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1
       kvar=nint((posz(i)-zd)/dz)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posy(i)-yb)/dy)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4






       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         

       xx = (posy(i) - real(ip(0),kind=8)*dy-yb)/dy   !relatif




       !conditions au bord
       !------------------


       do c=-3,4              
          ip(c)=mod(ip(c)+ny,ny)                
       end do




       !calcul des poids
       !---------------- 
       if (xx<=0.5D0) then
          poidx(-3)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
          poidx(-2)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+3.D0)/120.D0
          poidx(-1)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(0)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/36.D0
          poidx(1)=(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(2)=-(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/120.D0
          poidx(3)=(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/720.D0
          poidx(4)=0.D0
       else
          poidx(-3)=0.D0
          poidx(-2)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)/720.D0
          poidx(-1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)/120.D0
          poidx(0)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)/36.D0
          poidx(2)=(xx-4.D0)*(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(3)=-(xx-4.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/120.D0
          poidx(4)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
       end if





       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-3,4              
          remaille(ivar,ip(c)+1,kvar)=remaille(ivar,ip(c)+1,kvar)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l6_y

  subroutine remaill_l6_z (donne,posx,posy,posz,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)          :: donne
    real(kind=8),dimension(1:npart),intent(in)          :: posx,posy,posz
    real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)  :: remaille
    integer                                             :: i,c,d,e,ib,jb,kb,ivar,jvar,kvar
    integer,dimension(-3:4)                             :: ip
    real(kind=8),dimension(-3:4)                        :: poidx
    real(kind=8)                                        :: xx

    remaille=0.D0      


    do i=1,npart

       jvar=nint((posy(i)-yb)/dy)+1
       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posz(i)-zd)/dz) 
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4






       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------              
       xx = (posz(i) - real(ip(0),kind=8)*dz-zd)/dz   !relatif



       !conditions au bord
       !------------------
       !periodique:          

       do c=-3,4              
          ip(c)=mod(ip(c)+nz,nz)                
       end do


       !calcul des poids
       !---------------- 
       if (xx<=0.5D0) then
          poidx(-3)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
          poidx(-2)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+3.D0)/120.D0
          poidx(-1)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(0)=-(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/36.D0
          poidx(1)=(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/48.D0
          poidx(2)=-(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/120.D0
          poidx(3)=(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/720.D0
          poidx(4)=0.D0
       else
          poidx(-3)=0.D0
          poidx(-2)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)/720.D0
          poidx(-1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+2.D0)/120.D0
          poidx(0)=(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(1)=-(xx-4.D0)*(xx-3.D0)*(xx-2.D0)*xx*(xx+1.D0)*(xx+2.D0)/36.D0
          poidx(2)=(xx-4.D0)*(xx-3.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/48.D0
          poidx(3)=-(xx-4.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/120.D0
          poidx(4)=(xx-3.D0)*(xx-2.D0)*(xx-1.D0)*xx*(xx+1.D0)*(xx+2.D0)/720.D0
       end if





       !remaillage à l' interrieur domaine
       !---------------------------------         

       do c=-3,4              
          remaille(ivar,jvar,ip(c)+1)=remaille(ivar,jvar,ip(c)+1)+donne(i)*poidx(c)
       end do

    end do

  end subroutine remaill_l6_z




 subroutine remaill_l2_bloc_x (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,j,k,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.D0      

       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1
          k=nint((posz(i)-zd)/dz)+1
          poids=0.D0

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx


          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5D0) then
                poids(-1)=0.5D0*xx*(xx-1.D0)
                poids(0)=1.D0-xx**2
             else               
                poids(0)=0.5D0*(1.D0-xx)*(2.D0-xx)
             end if
          case(1)
             poids(-1)=0.5D0*xx*(xx-1.D0)
             poids(0)=1.D0-xx**2
          case(2)
             poids(0)=1.D0-0.5D0*xx*(1.D0+xx)
          case(3)
             poids(-1)=0.5D0*xx*(xx-1.D0)
             poids(0)=1.D0-xx
          case(4)
             poids(-2)=0.5D0*xx*(1.D0+xx)
             poids(-1)=-xx
             poids(0)=1.D0-xx**2
          case(5)
             poids(-1)=-0.5D0*xx+0.5D0*xx**2
             poids(0)=1.D0-poids(-1)

          end select


          select case (blocd(i))

          case(0)
             if (xx<=0.5D0) then
                poids(1)=0.5D0*xx*(1.D0+xx)               
             else
                poids(1)=2.D0*xx-xx**2
                poids(2)=0.5D0*(xx-1.D0)*xx
             end if
          case(1)
             poids(1)=0.5D0*xx*(1.D0+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5D0*xx*(xx-1.D0)
          case(3)
             poids(1)=3.D0*0.5D0*xx-0.5D0*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ip(c)+1,j,k)=remaille(ip(c)+1,j,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_x


     subroutine remaill_l2_bloc_y (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,k
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.D0      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          k=nint((posz(i)-zd)/dz)+1
          poids=0.D0

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5D0) then
                poids(-1)=0.5D0*xx*(xx-1.D0)
                poids(0)=1.D0-xx**2
             else               
                poids(0)=0.5D0*(1.D0-xx)*(2.D0-xx)
             end if
          case(1)
             poids(-1)=0.5D0*xx*(xx-1.D0)
             poids(0)=1.D0-xx**2
          case(2)
             poids(0)=1.D0-0.5D0*xx*(1.D0+xx)
          case(3)
             poids(-1)=0.5D0*xx*(xx-1.D0)
             poids(0)=1.D0-xx
          case(4)
             poids(-2)=0.5D0*xx*(1.D0+xx)
             poids(-1)=-xx
             poids(0)=1.D0-xx**2
          case(5)
             poids(-1)=-0.5D0*xx+0.5D0*xx**2
             poids(0)=1.D0-poids(-1)

          end select

          select case (blocd(i))

          case(0)
             if (xx<=0.5D0) then
                poids(1)=0.5D0*xx*(1.D0+xx)               
             else
                poids(1)=2.D0*xx-xx**2
                poids(2)=0.5D0*(xx-1.D0)*xx
             end if
          case(1)
             poids(1)=0.5D0*xx*(1.D0+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5D0*xx*(xx-1.D0)
          case(3)
             poids(1)=3.D0*0.5D0*xx-0.5D0*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,ip(c)+1,k)=remaille(ivar,ip(c)+1,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_y

  subroutine remaill_l2_bloc_z (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,j
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.D0      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          j=nint((posy(i)-yb)/dy)+1
          poids=0.D0

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posz(i)-zd)/dz)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posz(i) - real(ip(0) ,kind=8)*dz-zd)/dz       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nz,nz)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5D0) then
                poids(-1)=0.5D0*xx*(xx-1.D0)
                poids(0)=1.D0-xx**2
             else               
                poids(0)=0.5D0*(1.D0-xx)*(2.D0-xx)
             end if
          case(1)
             poids(-1)=0.5D0*xx*(xx-1.D0)
             poids(0)=1.D0-xx**2
          case(2)
             poids(0)=1.D0-0.5D0*xx*(1.D0+xx)
          case(3)
             poids(-1)=0.5D0*xx*(xx-1.D0)
             poids(0)=1.D0-xx
          case(4)
             poids(-2)=0.5D0*xx*(1.D0+xx)
             poids(-1)=-xx
             poids(0)=1.D0-xx**2
          case(5)
             poids(-1)=-0.5D0*xx+0.5D0*xx**2
             poids(0)=1.D0-poids(-1)

          end select

          select case (blocd(i))

          case(0)
             if (xx<=0.5D0) then
                poids(1)=0.5D0*xx*(1.D0+xx)               
             else
                poids(1)=2.D0*xx-xx**2
                poids(2)=0.5D0*(xx-1.D0)*xx
             end if
          case(1)
             poids(1)=0.5D0*xx*(1.D0+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5D0*xx*(xx-1.D0)
          case(3)
             poids(1)=3.D0*0.5D0*xx-0.5D0*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_z


    subroutine remaill_l4_bloc_x (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,j,per,k
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.D0   


       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1 
          k=nint((posz(i)-zd)/dz)+1 
          poids=0.D0

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx 


          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5D0) then 
                t1=xx*(xx-1.D0) 
                t2=xx**2-4.D0
                poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
                poids(-1)=-t1/6.D0*t2
                poids(0)=t2/4.D0*(xx**2-1.D0)
             else  
                t1=(xx-1.D0)*(xx-2.D0)*(xx-3.D0)
                poids(-1)=t1*xx/24.D0
                poids(0)=-t1*(xx+1.D0)/6.D0
             end if
          case(1)
             t1=xx*(xx-1.D0) 
             t2=xx**2-4.D0
             poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
             poids(-1)=-t1/6.D0*t2
             poids(0)=t2/4.D0*(xx**2-1.D0)

          case(2)
             t1=(xx-1.D0)*(xx-2.D0)/12.D0
             poids(-1)=t1*xx*(xx-3)/2.D0
             poids(0)=t1*(xx+6.D0)*(xx+1.D0)
          case(3)
             t1=0.5D0*(xx-1.D0)*(xx-2.D0)
             poids(-2)=t1*xx/12.D0*(xx+1.D0)
             poids(-1)=-t1*xx/3.D0*(xx+2.D0)
             poids(0)=t1*(xx+1.D0)
          case(4)
             t2=(xx**2-1.D0)
             t1=xx*(xx+2.D0)*0.5D0
             poids(-3)=t2*t1/12.D0
             poids(-2)=-t2*xx/6.D0*(xx+3.D0)
             poids(-1)=t1*(xx-1.D0)
             poids(0)=t2*(xx**2-4.D0)/4.D0
          case(5)
             t1=(xx-2.D0)*(xx-1.D0)/4.D0
             poids(-1)=-t1*xx/6.D0*(3*xx+7.D0)
             poids(0)=t1*(xx+2.D0)*(xx+1.D0)
          case(6)
             t1= (xx-1.D0)*(xx-2.D0)/6.D0
             t2=t1*(xx+1.D0)
             poids(-2)=t2*xx/4.D0
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.D0)
          case(7)
             t1=(xx-1.D0)*(xx+2.D0)
             t2=xx*(xx+1)/6.D0
             poids(-3)=t1*t2/4.D0
             poids(-2)=-t2*(xx-1.D0)
             poids(-1)=-xx*t1/6.D0*(xx-2.D0)
             poids(0)=t1*(xx-2.D0)/4.D0*(xx+1.D0)
          case(8)
             t1=xx*(xx-1.D0) 
             t2=xx**2-4.D0
             poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
             poids(-1)=-t1/6.D0*t2
             poids(0)=(xx-6.D0)*(xx+2.D0)*(xx**2-1.D0)/12.D0
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5D0) then 
                t1=xx*(xx+1.D0)*(xx+2.D0)
                poids(1)=-t1*(xx-2.D0)/6.D0
                poids(2)=t1*(xx-1.D0)/24.D0
             else  
                t1=xx*(xx+1.D0)*(xx-3.D0)
                poids(1)=t1*(xx-2.D0)/4.D0
                poids(2)=-t1*(xx-1.D0)/6.D0
                poids(3)=xx*(xx-2.D0)*(xx**2-1.D0)/24.D0
             end if
          case(1)
             t1=xx*(xx+1.D0)*(xx+2.D0)
             poids(1)=-t1*(xx-2.D0)/6.D0
             poids(2)=t1*(xx-1.D0)/24.D0           
          case(2)
             t1=xx*(xx+1.D0)*0.5D0
             t2=t1*(xx-1.D0)
             poids(1)=-t1*(xx-2.D0)
             poids(2)=-t2/3.D0*(xx-3.D0)
             poids(3)=t2*(xx-2.D0)/12.D0
          case(3)
             t1=xx*(xx+1.D0)/12.D0
             poids(1)=t1*(xx-2.D0)*(xx-7.D0)
             poids(2)=t1*(xx+2.D0)*(xx-1.D0)/2.D0
          case(4)
             t1=xx*(xx+1.D0)
             t2=t1*(xx-1.D0)/6.D0
             poids(1)=-t1*(xx+2.D0)*(xx-2.D0)/6.D0
             poids(2)=t2
             poids(3)=t2*(xx-2.D0)/4.D0
          case(5)
             poids(1)=-xx/24.D0*(3.D0*xx-7.D0)*(xx+2.D0)*(xx+1.D0)
          case(6)
             t1=xx*(xx+1.D0)/4.D0
             poids(1)=t1*(xx-3.D0)*(xx-2.D0)
             poids(2)=-t1*(3.D0*xx-10.D0)*(xx-1.D0)/6.D0
          case(7)
             poids(1)=xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/24.D0
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ip(c)+1,j,k)=remaille(ip(c)+1,j,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_x


     subroutine remaill_l4_bloc_y (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,k
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.D0   

       do i=1,npart

          poids=0.D0
          ivar=nint((posx(i)-xg)/dx)+1
          k=nint((posz(i)-zd)/dz)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy

          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poidss
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5D0) then 
                t1=xx*(xx-1.D0) 
                t2=xx**2-4.D0
                poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
                poids(-1)=-t1/6.D0*t2
                poids(0)=t2/4.D0*(xx**2-1.D0)
             else  
                t1=(xx-1.D0)*(xx-2.D0)*(xx-3.D0)
                poids(-1)=t1*xx/24.D0
                poids(0)=-t1*(xx+1.D0)/6.D0
             end if
          case(1)
             t1=xx*(xx-1.D0) 
             t2=xx**2-4.D0
             poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
             poids(-1)=-t1/6.D0*t2
             poids(0)=t2/4.D0*(xx**2-1.D0)

          case(2)
             t1=(xx-1.D0)*(xx-2.D0)/12.D0
             poids(-1)=t1*xx*(xx-3)/2.D0
             poids(0)=t1*(xx+6.D0)*(xx+1.D0)
          case(3)
             t1=0.5D0*(xx-1.D0)*(xx-2.D0)
             poids(-2)=t1*xx/12.D0*(xx+1.D0)
             poids(-1)=-t1*xx/3.D0*(xx+2.D0)
             poids(0)=t1*(xx+1.D0)
          case(4)
             t2=(xx**2-1.D0)
             t1=xx*(xx+2.D0)*0.5D0
             poids(-3)=t2*t1/12.D0
             poids(-2)=-t2*xx/6.D0*(xx+3.D0)
             poids(-1)=t1*(xx-1.D0)
             poids(0)=t2*(xx**2-4.D0)/4.D0
          case(5)
             t1=(xx-2.D0)*(xx-1.D0)/4.D0
             poids(-1)=-t1*xx/6.D0*(3*xx+7.D0)
             poids(0)=t1*(xx+2.D0)*(xx+1.D0)
          case(6)
             t1= (xx-1.D0)*(xx-2.D0)/6.D0
             t2=t1*(xx+1.D0)
             poids(-2)=t2*xx/4.D0
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.D0)
          case(7)
             t1=(xx-1.D0)*(xx+2.D0)
             t2=xx*(xx+1)/6.D0
             poids(-3)=t1*t2/4.D0
             poids(-2)=-t2*(xx-1.D0)
             poids(-1)=-xx*t1/6.D0*(xx-2.D0)
             poids(0)=t1*(xx-2.D0)/4.D0*(xx+1.D0)
          case(8)
             t1=xx*(xx-1.D0) 
             t2=xx**2-4.D0
             poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
             poids(-1)=-t1/6.D0*t2
             poids(0)=(xx-6.D0)*(xx+2.D0)*(xx**2-1.D0)/12.D0
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5D0) then 
                t1=xx*(xx+1.D0)*(xx+2.D0)
                poids(1)=-t1*(xx-2.D0)/6.D0
                poids(2)=t1*(xx-1.D0)/24.D0
             else  
                t1=xx*(xx+1.D0)*(xx-3.D0)
                poids(1)=t1*(xx-2.D0)/4.D0
                poids(2)=-t1*(xx-1.D0)/6.D0
                poids(3)=xx*(xx-2.D0)*(xx**2-1.D0)/24.D0
             end if
          case(1)
             t1=xx*(xx+1.D0)*(xx+2.D0)
             poids(1)=-t1*(xx-2.D0)/6.D0
             poids(2)=t1*(xx-1.D0)/24.D0           
          case(2)
             t1=xx*(xx+1.D0)*0.5D0
             t2=t1*(xx-1.D0)
             poids(1)=-t1*(xx-2.D0)
             poids(2)=-t2/3.D0*(xx-3.D0)
             poids(3)=t2*(xx-2.D0)/12.D0
          case(3)
             t1=xx*(xx+1.D0)/12.D0
             poids(1)=t1*(xx-2.D0)*(xx-7.D0)
             poids(2)=t1*(xx+2.D0)*(xx-1.D0)/2.D0
          case(4)
             t1=xx*(xx+1.D0)
             t2=t1*(xx-1.D0)/6.D0
             poids(1)=-t1*(xx+2.D0)*(xx-2.D0)/6.D0
             poids(2)=t2
             poids(3)=t2*(xx-2.D0)/4.D0
          case(5)
             poids(1)=-xx/24.D0*(3.D0*xx-7.D0)*(xx+2.D0)*(xx+1.D0)
          case(6)
             t1=xx*(xx+1.D0)/4.D0
             poids(1)=t1*(xx-3.D0)*(xx-2.D0)
             poids(2)=-t1*(3.D0*xx-10.D0)*(xx-1.D0)/6.D0
          case(7)
             poids(1)=xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/24.D0
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ivar,ip(c)+1,k)=remaille(ivar,ip(c)+1,k)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_y


     subroutine remaill_l4_bloc_z (donne,posx,posy,posz,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy,posz
       real(kind=8),dimension(1:nx,1:ny,1:nz),intent(out)           :: remaille
       integer                                                      :: i,c,d,ib,ivar,per,j
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.D0   

       do i=1,npart

          poids=0.D0
          ivar=nint((posx(i)-xg)/dx)+1
          j=nint((posy(i)-yb)/dy)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posz(i)-zd)/dz)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posz(i) - real(ip(0) ,kind=8)*dz-zd)/dz

          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+nz,nz)     
          end do

          !calcul des poidss
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5D0) then 
                t1=xx*(xx-1.D0) 
                t2=xx**2-4.D0
                poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
                poids(-1)=-t1/6.D0*t2
                poids(0)=t2/4.D0*(xx**2-1.D0)
             else  
                t1=(xx-1.D0)*(xx-2.D0)*(xx-3.D0)
                poids(-1)=t1*xx/24.D0
                poids(0)=-t1*(xx+1.D0)/6.D0
             end if
          case(1)
             t1=xx*(xx-1.D0) 
             t2=xx**2-4.D0
             poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
             poids(-1)=-t1/6.D0*t2
             poids(0)=t2/4.D0*(xx**2-1.D0)

          case(2)
             t1=(xx-1.D0)*(xx-2.D0)/12.D0
             poids(-1)=t1*xx*(xx-3)/2.D0
             poids(0)=t1*(xx+6.D0)*(xx+1.D0)
          case(3)
             t1=0.5D0*(xx-1.D0)*(xx-2.D0)
             poids(-2)=t1*xx/12.D0*(xx+1.D0)
             poids(-1)=-t1*xx/3.D0*(xx+2.D0)
             poids(0)=t1*(xx+1.D0)
          case(4)
             t2=(xx**2-1.D0)
             t1=xx*(xx+2.D0)*0.5D0
             poids(-3)=t2*t1/12.D0
             poids(-2)=-t2*xx/6.D0*(xx+3.D0)
             poids(-1)=t1*(xx-1.D0)
             poids(0)=t2*(xx**2-4.D0)/4.D0
          case(5)
             t1=(xx-2.D0)*(xx-1.D0)/4.D0
             poids(-1)=-t1*xx/6.D0*(3*xx+7.D0)
             poids(0)=t1*(xx+2.D0)*(xx+1.D0)
          case(6)
             t1= (xx-1.D0)*(xx-2.D0)/6.D0
             t2=t1*(xx+1.D0)
             poids(-2)=t2*xx/4.D0
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.D0)
          case(7)
             t1=(xx-1.D0)*(xx+2.D0)
             t2=xx*(xx+1)/6.D0
             poids(-3)=t1*t2/4.D0
             poids(-2)=-t2*(xx-1.D0)
             poids(-1)=-xx*t1/6.D0*(xx-2.D0)
             poids(0)=t1*(xx-2.D0)/4.D0*(xx+1.D0)
          case(8)
             t1=xx*(xx-1.D0) 
             t2=xx**2-4.D0
             poids(-2)=t1/24.D0*(xx-2.D0)*(xx+1.D0)
             poids(-1)=-t1/6.D0*t2
             poids(0)=(xx-6.D0)*(xx+2.D0)*(xx**2-1.D0)/12.D0
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5D0) then 
                t1=xx*(xx+1.D0)*(xx+2.D0)
                poids(1)=-t1*(xx-2.D0)/6.D0
                poids(2)=t1*(xx-1.D0)/24.D0
             else  
                t1=xx*(xx+1.D0)*(xx-3.D0)
                poids(1)=t1*(xx-2.D0)/4.D0
                poids(2)=-t1*(xx-1.D0)/6.D0
                poids(3)=xx*(xx-2.D0)*(xx**2-1.D0)/24.D0
             end if
          case(1)
             t1=xx*(xx+1.D0)*(xx+2.D0)
             poids(1)=-t1*(xx-2.D0)/6.D0
             poids(2)=t1*(xx-1.D0)/24.D0           
          case(2)
             t1=xx*(xx+1.D0)*0.5D0
             t2=t1*(xx-1.D0)
             poids(1)=-t1*(xx-2.D0)
             poids(2)=-t2/3.D0*(xx-3.D0)
             poids(3)=t2*(xx-2.D0)/12.D0
          case(3)
             t1=xx*(xx+1.D0)/12.D0
             poids(1)=t1*(xx-2.D0)*(xx-7.D0)
             poids(2)=t1*(xx+2.D0)*(xx-1.D0)/2.D0
          case(4)
             t1=xx*(xx+1.D0)
             t2=t1*(xx-1.D0)/6.D0
             poids(1)=-t1*(xx+2.D0)*(xx-2.D0)/6.D0
             poids(2)=t2
             poids(3)=t2*(xx-2.D0)/4.D0
          case(5)
             poids(1)=-xx/24.D0*(3.D0*xx-7.D0)*(xx+2.D0)*(xx+1.D0)
          case(6)
             t1=xx*(xx+1.D0)/4.D0
             poids(1)=t1*(xx-3.D0)*(xx-2.D0)
             poids(2)=-t1*(3.D0*xx-10.D0)*(xx-1.D0)/6.D0
          case(7)
             poids(1)=xx*(xx+1.D0)*(xx+2.D0)*(xx+3.D0)/24.D0
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ivar,j,ip(c)+1)=remaille(ivar,j,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_z








end module remaillage_mod


