module source_mod
  use donnees_mod
  use tab_mod

contains

  !resolution du terme source :
  !div(u:omega)+Delta(omega)/Re+rot(v*exp(-lambda*chi*dt))
  !par DF d'ordre4 decentre sur les bords

  subroutine source_split_1
    implicit none
    integer                                  :: i
    real(kind=8),dimension(1:3)              :: ts,tsa

    !ordre2
    !------
    do i=1,npart           
       !qp(i)=qp(i)+0.5D0*dt*tsource(time,xp(i),yp(i),zp(i))

       ts=strech( nint((xp(i)-xg)/dx+1),nint((yp(i)-yb)/dy+1),nint((zp(i)-zd)/dz+1) )&
         +nu*diffusion( nint((xp(i)-xg)/dx+1),nint((yp(i)-yb)/dy+1),nint((zp(i)-zd)/dz+1) )&
         +penal( nint((xp(i)-xg)/dx+1),nint((yp(i)-yb)/dy+1),nint((zp(i)-zd)/dz+1) )

       !call source_anal(tsa, xp(i),yp(i),zp(i) )

       !omx(i)=omx(i)+0.5D0*dt*(ts(1)+tsa(1))
       !omy(i)=omy(i)+0.5D0*dt*(ts(2)+tsa(2))
       !omz(i)=omz(i)+0.5D0*dt*(ts(3)+tsa(3))

       omx(i)=omx(i)+0.5*dt*ts(1)
       omy(i)=omy(i)+0.5*dt*ts(2)
       omz(i)=omz(i)+0.5*dt*ts(3)
  
    end do

  end subroutine source_split_1

  subroutine source_split_2
    implicit none
    integer                                  :: i,j,k
    real(kind=8)                             :: x,y,z
    real(kind=8),dimension(1:3)              :: ts,tsa
    real(kind=8),dimension(:,:,:),pointer    :: omgx_tmp,omgy_tmp,omgz_tmp

    real(kind=8)                             :: cx,c2x,cy,c2y,cz,c2z,sx,s2x,sy,s2y,sz,s2z,ct,st,c2t,s2t

    allocate(omgx_tmp(1:nx,1:ny,1:nz),omgy_tmp(1:nx,1:ny,1:nz),omgz_tmp(1:nx,1:ny,1:nz))
    omgx_tmp=0.
    omgy_tmp=0.
    omgz_tmp=0.

    !ordre2
    !------
    do k=1,nz
       z=ztab(k)
       do j=1,ny
          y=ytab(j)
          do i=1,nx
             x=xtab(i)

             !qg(i,j,k)=qg(i,j,k)+0.5D0*dt*tsource(time+dt,x,y,z)

             ts=strech(i,j,k)+nu*diffusion(i,j,k)+penal(i,j,k)

             !call source_anal(tsa,x,y,z)


             !pour l'instant en t
             !omgx_tmp(i,j,k)=omgx_tmp(i,j,k)+0.5D0*dt*(ts(1)+tsa(1))
             !omgy_tmp(i,j,k)=omgy_tmp(i,j,k)+0.5D0*dt*(ts(2)+tsa(2))
             !omgz_tmp(i,j,k)=omgz_tmp(i,j,k)+0.5D0*dt*(ts(3)+tsa(3))

             omgx_tmp(i,j,k)=omgx_tmp(i,j,k)+0.5D0*dt*ts(1)
             omgy_tmp(i,j,k)=omgy_tmp(i,j,k)+0.5D0*dt*ts(2)
             omgz_tmp(i,j,k)=omgz_tmp(i,j,k)+0.5D0*dt*ts(3)



          end do
       end do
    end do

    omgx=omgx+omgx_tmp
    omgy=omgy+omgy_tmp
    omgz=omgz+omgz_tmp

    deallocate(omgx_tmp,omgy_tmp,omgz_tmp)


  end subroutine source_split_2



  function strech (i,j,k)     
    implicit none
    real(kind=8),dimension(1:3)   :: strech
    integer,intent(in)            :: i,j,k
    real(kind=8),dimension(1:3)   :: tx,ty,tz
    real(kind=8)                  :: facx,facy,facz
    integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2

    !forme conservative : div(w:u)

    facx=1.D0/(12.D0*dx)
    facy=1.D0/(12.D0*dy)
    facz=1.D0/(12.D0*dz)

    ip1=mod(i+1+nx-1,nx)+1
    im1=mod(i-1+nx-1,nx)+1
    ip2=mod(i+2+nx-1,nx)+1
    im2=mod(i-2+nx-1,nx)+1

    jp1=mod(j+1+ny-1,ny)+1
    jm1=mod(j-1+ny-1,ny)+1
    jp2=mod(j+2+ny-1,ny)+1
    jm2=mod(j-2+ny-1,ny)+1

    kp1=mod(k+1+nz-1,nz)+1
    km1=mod(k-1+nz-1,nz)+1
    kp2=mod(k+2+nz-1,nz)+1
    km2=mod(k-2+nz-1,nz)+1

    !conditions periodiques
    !-----------------------
    tx(1)=facx*(omgx(im2,j,k)*vxg(im2,j,k)-8.D0*omgx(im1,j,k)*vxg(im1,j,k) &
	+8.D0*omgx(ip1,j,k)*vxg(ip1,j,k)-omgx(ip2,j,k)*vxg(ip2,j,k))
    tx(2)=facx*(omgx(im2,j,k)*vyg(im2,j,k)-8.D0*omgx(im1,j,k)*vyg(im1,j,k) &
	+8.D0*omgx(ip1,j,k)*vyg(ip1,j,k)-omgx(ip2,j,k)*vyg(ip2,j,k))
    tx(3)=facx*(omgx(im2,j,k)*vzg(im2,j,k)-8.D0*omgx(im1,j,k)*vzg(im1,j,k) &
	+8.D0*omgx(ip1,j,k)*vzg(ip1,j,k)-omgx(ip2,j,k)*vzg(ip2,j,k))

    ty(1)=facy*(omgy(i,jm2,k)*vxg(i,jm2,k)-8.D0*omgy(i,jm1,k)*vxg(i,jm1,k) &
	+8.D0*omgy(i,jp1,k)*vxg(i,jp1,k)-omgy(i,jp2,k)*vxg(i,jp2,k))
    ty(2)=facy*(omgy(i,jm2,k)*vyg(i,jm2,k)-8.D0*omgy(i,jm1,k)*vyg(i,jm1,k) &
	+8.D0*omgy(i,jp1,k)*vyg(i,jp1,k)-omgy(i,jp2,k)*vyg(i,jp2,k))
    ty(3)=facy*(omgy(i,jm2,k)*vzg(i,jm2,k)-8.D0*omgy(i,jm1,k)*vzg(i,jm1,k) &
	+8.D0*omgy(i,jp1,k)*vzg(i,jp1,k)-omgy(i,jp2,k)*vzg(i,jp2,k))

    tz(1)=facz*(omgz(i,j,km2)*vxg(i,j,km2)-8.D0*omgz(i,j,km1)*vxg(i,j,km1) &
	+8.D0*omgz(i,j,kp1)*vxg(i,j,kp1)-omgz(i,j,kp2)*vxg(i,j,kp2))
    tz(2)=facz*(omgz(i,j,km2)*vyg(i,j,km2)-8.D0*omgz(i,j,km1)*vyg(i,j,km1) &
	+8.D0*omgz(i,j,kp1)*vyg(i,j,kp1)-omgz(i,j,kp2)*vyg(i,j,kp2))
    tz(3)=facz*(omgz(i,j,km2)*vzg(i,j,km2)-8.D0*omgz(i,j,km1)*vzg(i,j,km1) &
	+8.D0*omgz(i,j,kp1)*vzg(i,j,kp1)-omgz(i,j,kp2)*vzg(i,j,kp2))

    !conditions non periodiques
    !--------------------------
!!$    if((i/=1).and.(i/=2).and.(i/=nx).and.(i/=nx-1).and.(j/=1).and.(j/=2).and.(j/=ny).and.(j/=ny-1).and.(k/=1).and.(k/=2).and.(k/=nz).and.(k/=nz-1))then
!!$       !schema centre d'ordre4
!!$
!!$       tx(1)=facx*(omgx(i-2,j,k)*vxg(i-2,j,k)-8.D0*omgx(i-1,j,k)*vxg(i-1,j,k)+8.D0*omgx(i+1,j,k)*vxg(i+1,j,k)-omgx(i+2,j,k)*vxg(i+2,j,k))
!!$       tx(2)=facx*(omgx(i-2,j,k)*vyg(i-2,j,k)-8.D0*omgx(i-1,j,k)*vyg(i-1,j,k)+8.D0*omgx(i+1,j,k)*vyg(i+1,j,k)-omgx(i+2,j,k)*vyg(i+2,j,k))
!!$       tx(3)=facx*(omgx(i-2,j,k)*vzg(i-2,j,k)-8.D0*omgx(i-1,j,k)*vzg(i-1,j,k)+8.D0*omgx(i+1,j,k)*vzg(i+1,j,k)-omgx(i+2,j,k)*vzg(i+2,j,k))
!!$
!!$       ty(1)=facy*(omgy(i,j-2,k)*vxg(i,j-2,k)-8.D0*omgy(i,j-1,k)*vxg(i,j-1,k)+8.D0*omgy(i,j+1,k)*vxg(i,j+1,k)-omgy(i,j+2,k)*vxg(i,j+2,k))
!!$       ty(2)=facy*(omgy(i,j-2,k)*vyg(i,j-2,k)-8.D0*omgy(i,j-1,k)*vyg(i,j-1,k)+8.D0*omgy(i,j+1,k)*vyg(i,j+1,k)-omgy(i,j+2,k)*vyg(i,j+2,k))
!!$       ty(3)=facy*(omgy(i,j-2,k)*vzg(i,j-2,k)-8.D0*omgy(i,j-1,k)*vzg(i,j-1,k)+8.D0*omgy(i,j+1,k)*vzg(i,j+1,k)-omgy(i,j+2,k)*vzg(i,j+2,k))
!!$
!!$       tz(1)=facz*(omgz(i,j,k-2)*vxg(i,j,k-2)-8.D0*omgz(i,j,k-1)*vxg(i,j,k-1)+8.D0*omgz(i,j,k+1)*vxg(i,j,k+1)-omgz(i,j,k+2)*vxg(i,j,k+2))
!!$       tz(2)=facz*(omgz(i,j,k-2)*vyg(i,j,k-2)-8.D0*omgz(i,j,k-1)*vyg(i,j,k-1)+8.D0*omgz(i,j,k+1)*vyg(i,j,k+1)-omgz(i,j,k+2)*vyg(i,j,k+2))
!!$       tz(3)=facz*(omgz(i,j,k-2)*vzg(i,j,k-2)-8.D0*omgz(i,j,k-1)*vzg(i,j,k-1)+8.D0*omgz(i,j,k+1)*vzg(i,j,k+1)-omgz(i,j,k+2)*vzg(i,j,k+2))
!!$
!!$    else
!!$
!!$       !schema decentre au bord (ordre4)
!!$       if((i==1).or.(i==2))then
!!$          tx(1)=facx*(-25.D0*omgx(i,j,k)*vxg(i,j,k)+48.D0*omgx(i+1,j,k)*vxg(i+1,j,k)-36.D0*omgx(i+2,j,k)*vxg(i+2,j,k)+16.D0*omgx(i+3,j,k)*vxg(i+3,j,k)-3.D0*omgx(i+4,j,k)*vxg(i+4,j,k))
!!$          tx(2)=facx*(-25.D0*omgx(i,j,k)*vyg(i,j,k)+48.D0*omgx(i+1,j,k)*vyg(i+1,j,k)-36.D0*omgx(i+2,j,k)*vyg(i+2,j,k)+16.D0*omgx(i+3,j,k)*vyg(i+3,j,k)-3.D0*omgx(i+4,j,k)*vyg(i+4,j,k))
!!$          tx(3)=facx*(-25.D0*omgx(i,j,k)*vzg(i,j,k)+48.D0*omgx(i+1,j,k)*vzg(i+1,j,k)-36.D0*omgx(i+2,j,k)*vzg(i+2,j,k)+16.D0*omgx(i+3,j,k)*vzg(i+3,j,k)-3.D0*omgx(i+4,j,k)*vzg(i+4,j,k))
!!$       end if
!!$
!!$       if((i==nx).or.(i==nx-1))then
!!$          tx(1)=facx*(-25.D0*omgx(i,j,k)*vxg(i,j,k)+48.D0*omgx(i-1,j,k)*vxg(i-1,j,k)-36.D0*omgx(i-2,j,k)*vxg(i-2,j,k)+16.D0*omgx(i-3,j,k)*vxg(i-3,j,k)-3.D0*omgx(i-4,j,k)*vxg(i-4,j,k))
!!$          tx(2)=facx*(-25.D0*omgx(i,j,k)*vyg(i,j,k)+48.D0*omgx(i-1,j,k)*vyg(i-1,j,k)-36.D0*omgx(i-2,j,k)*vyg(i-2,j,k)+16.D0*omgx(i-3,j,k)*vyg(i-3,j,k)-3.D0*omgx(i-4,j,k)*vyg(i-4,j,k))
!!$          tx(3)=facx*(-25.D0*omgx(i,j,k)*vzg(i,j,k)+48.D0*omgx(i-1,j,k)*vzg(i-1,j,k)-36.D0*omgx(i-2,j,k)*vzg(i-2,j,k)+16.D0*omgx(i-3,j,k)*vzg(i-3,j,k)-3.D0*omgx(i-4,j,k)*vzg(i-4,j,k))
!!$       endif
!!$
!!$       if((j==1).or.(j==2))then
!!$          ty(1)=facy*(-25.D0*omgy(i,j,k)*vxg(i,j,k)+48.D0*omgy(i,j+1,k)*vxg(i,j+1,k)-36.D0*omgy(i,j+2,k)*vxg(i,j+2,k)+16.D0*omgy(i,j+3,k)*vxg(i,j+3,k)-3.D0*omgy(i,j+4,k)*vxg(i,j+4,k))
!!$          ty(2)=facy*(-25.D0*omgy(i,j,k)*vyg(i,j,k)+48.D0*omgy(i,j+1,k)*vyg(i,j+1,k)-36.D0*omgy(i,j+2,k)*vyg(i,j+2,k)+16.D0*omgy(i,j+3,k)*vyg(i,j+3,k)-3.D0*omgy(i,j+4,k)*vyg(i,j+4,k))
!!$          ty(3)=facy*(-25.D0*omgy(i,j,k)*vzg(i,j,k)+48.D0*omgy(i,j+1,k)*vzg(i,j+1,k)-36.D0*omgy(i,j+2,k)*vzg(i,j+2,k)+16.D0*omgy(i,j+3,k)*vzg(i,j+3,k)-3.D0*omgy(i,j+4,k)*vzg(i,j+4,k))
!!$       end if
!!$
!!$       if((j==ny).or.(j==ny-1))then
!!$          ty(1)=facy*(-25.D0*omgy(i,j,k)*vxg(i,j,k)+48.D0*omgy(i,j-1,k)*vxg(i,j-1,k)-36.D0*omgy(i,j-2,k)*vxg(i,j-2,k)+16.D0*omgy(i,j-3,k)*vxg(i,j-3,k)-3.D0*omgy(i,j-4,k)*vxg(i,j-4,k))
!!$          ty(2)=facy*(-25.D0*omgy(i,j,k)*vyg(i,j,k)+48.D0*omgy(i,j-1,k)*vyg(i,j-1,k)-36.D0*omgy(i,j-2,k)*vyg(i,j-2,k)+16.D0*omgy(i,j-3,k)*vyg(i,j-3,k)-3.D0*omgy(i,j-4,k)*vyg(i,j-4,k))
!!$          ty(3)=facy*(-25.D0*omgy(i,j,k)*vzg(i,j,k)+48.D0*omgy(i,j-1,k)*vzg(i,j-1,k)-36.D0*omgy(i,j-2,k)*vzg(i,j-2,k)+16.D0*omgy(i,j-3,k)*vzg(i,j-3,k)-3.D0*omgy(i,j-4,k)*vzg(i,j-4,k))
!!$       endif
!!$
!!$       if((k==1).or.(k==2))then
!!$          tz(1)=facz*(-25.D0*omgz(i,j,k)*vxg(i,j,k)+48.D0*omgz(i,j,k+1)*vxg(i,j,k+1)-36.D0*omgz(i,j,k+2)*vxg(i,j,k+2)+16.D0*omgz(i,j,k+3)*vxg(i,j,k+3)-3.D0*omgz(i,j,k+4)*vxg(i,j,k+4))
!!$          tz(2)=facz*(-25.D0*omgz(i,j,k)*vyg(i,j,k)+48.D0*omgz(i,j,k+1)*vyg(i,j,k+1)-36.D0*omgz(i,j,k+2)*vyg(i,j,k+2)+16.D0*omgz(i,j,k+3)*vyg(i,j,k+3)-3.D0*omgz(i,j,k+4)*vyg(i,j,k+4))
!!$          tz(3)=facz*(-25.D0*omgz(i,j,k)*vzg(i,j,k)+48.D0*omgz(i,j,k+1)*vzg(i,j,k+1)-36.D0*omgz(i,j,k+2)*vzg(i,j,k+2)+16.D0*omgz(i,j,k+3)*vzg(i,j,k+3)-3.D0*omgz(i,j,k+4)*vzg(i,j,k+4))
!!$       end if
!!$
!!$       if((k==nz).or.(k==nz-1))then
!!$          tz(1)=facz*(-25.D0*omgz(i,j,k)*vxg(i,j,k)+48.D0*omgz(i,j,k-1)*vxg(i,j,k-1)-36.D0*omgz(i,j,k-2)*vxg(i,j,k-2)+16.D0*omgz(i,j,k-3)*vxg(i,j,k-3)-3.D0*omgz(i,j,k-4)*vxg(i,j,k-4))
!!$          tz(2)=facz*(-25.D0*omgz(i,j,k)*vyg(i,j,k)+48.D0*omgz(i,j,k-1)*vyg(i,j,k-1)-36.D0*omgz(i,j,k-2)*vyg(i,j,k-2)+16.D0*omgz(i,j,k-3)*vyg(i,j,k-3)-3.D0*omgz(i,j,k-4)*vyg(i,j,k-4))
!!$          tz(3)=facz*(-25.D0*omgz(i,j,k)*vzg(i,j,k)+48.D0*omgz(i,j,k-1)*vzg(i,j,k-1)-36.D0*omgz(i,j,k-2)*vzg(i,j,k-2)+16.D0*omgz(i,j,k-3)*vzg(i,j,k-3)-3.D0*omgz(i,j,k-4)*vzg(i,j,k-4))
!!$       endif
!!$
!!$    end if

    strech(1)=tx(1)+ty(1)+tz(1)
    strech(2)=tx(2)+ty(2)+tz(2)
    strech(3)=tx(3)+ty(3)+tz(3)

  end function strech


  function diffusion (i,j,k)     
    implicit none
    real(kind=8),dimension(1:3)   :: diffusion
    integer,intent(in)            :: i,j,k
    real(kind=8)                  :: facx,facy,facz
    real(kind=8),dimension(1:3)   :: tx,ty,tz
    integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2

    facx=1.D0/(12.D0*dx**2)
    facy=1.D0/(12.D0*dy**2)
    facz=1.D0/(12.D0*dz**2)

    ip1=mod(i+1+nx-1,nx)+1
    im1=mod(i-1+nx-1,nx)+1
    ip2=mod(i+2+nx-1,nx)+1
    im2=mod(i-2+nx-1,nx)+1

    jp1=mod(j+1+ny-1,ny)+1
    jm1=mod(j-1+ny-1,ny)+1
    jp2=mod(j+2+ny-1,ny)+1
    jm2=mod(j-2+ny-1,ny)+1

    kp1=mod(k+1+nz-1,nz)+1
    km1=mod(k-1+nz-1,nz)+1
    kp2=mod(k+2+nz-1,nz)+1
    km2=mod(k-2+nz-1,nz)+1

    !ordre4
    tx(1)=facx*(-omgx(ip2,j,k)+16.D0*omgx(ip1,j,k)-30.D0*omgx(i,j,k)+16.D0*omgx(im1,j,k)-omgx(im2,j,k))
    tx(2)=facx*(-omgy(ip2,j,k)+16.D0*omgy(ip1,j,k)-30.D0*omgy(i,j,k)+16.D0*omgy(im1,j,k)-omgy(im2,j,k))
    tx(3)=facx*(-omgz(ip2,j,k)+16.D0*omgz(ip1,j,k)-30.D0*omgz(i,j,k)+16.D0*omgz(im1,j,k)-omgz(im2,j,k))

    ty(1)=facy*(-omgx(i,jp2,k)+16.D0*omgx(i,jp1,k)-30.D0*omgx(i,j,k)+16.D0*omgx(i,jm1,k)-omgx(i,jm2,k))
    ty(2)=facy*(-omgy(i,jp2,k)+16.D0*omgy(i,jp1,k)-30.D0*omgy(i,j,k)+16.D0*omgy(i,jm1,k)-omgy(i,jm2,k))
    ty(3)=facy*(-omgz(i,jp2,k)+16.D0*omgz(i,jp1,k)-30.D0*omgz(i,j,k)+16.D0*omgz(i,jm1,k)-omgz(i,jm2,k))

    tz(1)=facz*(-omgx(i,j,kp2)+16.D0*omgx(i,j,kp1)-30.D0*omgx(i,j,k)+16.D0*omgx(i,j,km1)-omgx(i,j,km2))
    tz(2)=facz*(-omgy(i,j,kp2)+16.D0*omgy(i,j,kp1)-30.D0*omgy(i,j,k)+16.D0*omgy(i,j,km1)-omgy(i,j,km2))
    tz(3)=facz*(-omgz(i,j,kp2)+16.D0*omgz(i,j,kp1)-30.D0*omgz(i,j,k)+16.D0*omgz(i,j,km1)-omgz(i,j,km2))
!!$
!!$    !ordre2
!!$    facx=1.D0/(dx**2)
!!$    facy=1.D0/(dy**2)
!!$    facz=1.D0/(dz**2)
!!$
!!$    tx(1)=facx*(omgx(ip1,j,k)-2.*omgx(i,j,k)+omgx(im1,j,k))
!!$    tx(2)=facx*(omgy(ip1,j,k)-2.*omgy(i,j,k)+omgy(im1,j,k))
!!$    tx(3)=facx*(omgz(ip1,j,k)-2.*omgz(i,j,k)+omgz(im1,j,k))
!!$    
!!$    ty(1)=facy*(omgx(i,jp1,k)-2.*omgx(i,j,k)+omgx(i,jm1,k))
!!$    ty(2)=facy*(omgy(i,jp1,k)-2.*omgy(i,j,k)+omgy(i,jm1,k))
!!$    ty(3)=facy*(omgz(i,jp1,k)-2.*omgz(i,j,k)+omgz(i,jm1,k))
!!$    
!!$    tz(1)=facz*(omgx(i,j,kp1)-2.*omgx(i,j,k)+omgx(i,j,km1))
!!$    tz(2)=facz*(omgy(i,j,kp1)-2.*omgy(i,j,k)+omgy(i,j,km1))
!!$    tz(3)=facz*(omgz(i,j,kp1)-2.*omgz(i,j,k)+omgz(i,j,km1))



    diffusion(1)=tx(1)+ty(1)+tz(1)
    diffusion(2)=tx(2)+ty(2)+tz(2)
    diffusion(3)=tx(3)+ty(3)+tz(3)


  end function diffusion








  
  function penal(i,j,k)     
    implicit none
    real(kind=8),dimension(1:3)   :: penal
    integer,intent(in)            :: i,j,k
    real(kind=8),dimension(1:3)   :: tx,ty,tz
    real(kind=8)                  :: facx,facy,facz
    integer                       :: ip1,ip2,im1,im2,jp1,jp2,jm1,jm2,kp1,kp2,km1,km2

    !forme conservative : div(w:u)

    facx=1.D0/(12.D0*dx)
    facy=1.D0/(12.D0*dy)
    facz=1.D0/(12.D0*dz)

    ip1=mod(i+1+nx-1,nx)+1
    im1=mod(i-1+nx-1,nx)+1
    ip2=mod(i+2+nx-1,nx)+1
    im2=mod(i-2+nx-1,nx)+1

    jp1=mod(j+1+ny-1,ny)+1
    jm1=mod(j-1+ny-1,ny)+1
    jp2=mod(j+2+ny-1,ny)+1
    jm2=mod(j-2+ny-1,ny)+1

    kp1=mod(k+1+nz-1,nz)+1
    km1=mod(k-1+nz-1,nz)+1
    kp2=mod(k+2+nz-1,nz)+1
    km2=mod(k-2+nz-1,nz)+1

    !conditions periodiques
    !-----------------------

    tx(2)=facx*(peny(im2,j,k)-8.D0*peny(im1,j,k)+8.D0*peny(ip1,j,k)-peny(ip2,j,k))
    tx(3)=facx*(penz(im2,j,k)-8.D0*penz(im1,j,k)+8.D0*penz(ip1,j,k)-penz(ip2,j,k))

    ty(1)=facy*(penx(i,jm2,k)-8.D0*penx(i,jm1,k)+8.D0*penx(i,jp1,k)-penx(i,jp2,k))
    ty(3)=facy*(penz(i,jm2,k)-8.D0*penz(i,jm1,k)+8.D0*penz(i,jp1,k)-penz(i,jp2,k))


    tz(1)=facz*(penx(i,j,km2)-8.D0*penx(i,j,km1)+8.D0*penx(i,j,kp1)-penx(i,j,kp2))
    tz(2)=facz*(peny(i,j,km2)-8.D0*peny(i,j,km1)+8.D0*peny(i,j,kp1)-peny(i,j,kp2))


    penal(1)=ty(3)-tz(2)
    penal(2)=tz(1)-tx(3)
    penal(3)=tx(2)-ty(1)

  contains
    function penx (l,m,n)     
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: penx


      !penx=vxg(l,m,n)*exp(-lambda*chi(l,m,n)*dt)
      !penx=vxg(l,m,n)/(1.+lambda*chi(l,m,n)*dt)

      !explicite
      !penx=-lambda*chi(l,m,n)*vxg(l,m,n)
      penx=vxg(l,m,n)

    end function penx

    function peny (l,m,n)     
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: peny


      !peny=vyg(l,m,n)*exp(-lambda*chi(l,m,n)*dt)
      !peny=vyg(l,m,n)/(1.+lambda*chi(l,m,n)*dt)

      !explicite
      !peny=-lambda*chi(l,m,n)*vyg(l,m,n)
      peny=vyg(l,m,n)

    end function peny

    function penz (l,m,n)      
      implicit none
      integer, intent(in)          :: l,m,n
      real(kind=8)                 :: penz


      !penz=vzg(l,m,n)*exp(-lambda*chi(l,m,n)*dt)
      !penz=vzg(l,m,n)/(1.+lambda*chi(l,m,n)*dt)

      !explicite
      !penz=-lambda*chi(l,m,n)*vzg(l,m,n)
      penz=vzg(l,m,n)

    end function penz

  end function penal






  subroutine source_anal (s,x,y,z)
    !pour test terme source pour sol de NS
    implicit none
    real(kind=8),dimension(1:3),intent(out)  :: s
    real(kind=8),intent(in)                  :: x,y,z
    real(kind=8)                             :: cx,c2x,cy,c2y,cz,c2z,sx,s2x,sy,s2y,sz,s2z,ct,st,c2t,s2t

    real(kind=8)             :: t1,t2,t3,c1,c2,c3


    !=================
    !sur [0,1]**3
    !=================

    cx=cos(pi*x)
    c2x=cos(2.*pi*x)
    sx=sin(pi*x)
    s2x=sin(2.*pi*x)

    cy=cos(pi*y)
    c2y=cos(2.*pi*y)
    sy=sin(pi*y)
    s2y=sin(2.*pi*y)

    cz=cos(pi*z)
    c2z=cos(2.*pi*z)
    sz=sin(pi*z)
    s2z=sin(2.*pi*z)

    ct=cos(pi*time)
    c2t=cos(2.*pi*time)
    st=sin(pi*time)
    s2t=sin(2.*pi*time)

    !sol analytique instationnaire
    !-----------------------------
!!$    s(1)=2.*pi**2*(s2x*c2y*sz**2*st&
!!$         -s2x*sy**2*c2z*st&
!!$         -8.*sx**2*s2y*s2z*ct**2*c2x*c2y*sz**2&
!!$         +8.*sx**2*s2y*s2z*ct**2*c2x*sy**2*c2z&
!!$         -2.*s2x**2*sy**3*s2z*ct**2*c2z*cy&
!!$         +2.*s2x**2*s2y*sz**3*ct**2*c2y*cz&
!!$         +4.*sx*s2y*s2z*ct**2*cx*s2x*c2y*sz**2&
!!$         -4.*sx*s2y*s2z*ct**2*cx*s2x*sy**2*c2z&
 !!$         -10.*s2x*c2y*pi*sz**2*ct&
!!$         +10.*s2x*sy**2*c2z*pi*ct&
!!$         -2.*s2x*cy**2*pi*c2z*ct&
!!$         +2.*s2x*c2y*pi*cz**2*ct)
!!$
!!$    s(2)=2.*pi**2*(-2.*sx**2*s2y*c2z*st - c2x*s2y*sz**2*st &
!!$         +8.*sx**3*s2y**2*s2z*ct**2*c2z*cx - 8.*s2x*sy**2*s2z*ct**2*sx**2*c2y*c2z &
!!$         -4.*s2x*sy**2*s2z*ct**2*c2x*c2y*sz**2 - 2.*s2x*s2y**2*sz**3*ct**2*c2x*cz &
!!$         +4.*s2x*sy*s2z*ct**2*cy*sx**2*s2y*c2z &
!!$         +2.*s2x*sy*s2z*ct**2*cy*c2x*s2y*sz**2 - 4.*cx**2*pi*s2y*c2z*ct &
!!$         +20.*sx**2*s2y*c2z*pi*ct + 10.*c2x*pi*s2y*sz**2*ct - 2.*cx*pi*s2y*cz**2*ct)
!!$
!!$    s(3)=-2.*pi**2*(-c2x*sy**2*s2z*st - 2.*sx**2*c2y*s2z*st &
!!$         +8.*sx**3*s2y*s2z**2*ct**2*c2y*cx - 2.*s2x*sy**3*s2z**2*ct**2*c2x*cy &
!!$         -4.* s2x*s2y*sz**2*ct**2*c2x*sy**2*c2z - 8.*s2x*s2y*sz**2*ct**2*sx**2*c2y*c2z &
!!$         +2.*s2x*s2y*sz*ct**2*cz*c2x*sy**2*s2z &
!!$         +4.*s2x*s2y*sz*ct**2*cz*sx**2*c2y*s2z&
!!$         +10.*c2x*pi*sy**2*s2z*ct - 4.*cx**2*pi*c2y*s2z*ct&
!!$         +20.*sx**2*c2y*pi*s2z*ct - 2.*c2x*pi*cy**2*s2z*ct)


    !sol analytique stationnaire
    !---------------------------
    s(1)=4.*pi**2*(-4.*sx**2*s2y*s2z*c2x*c2y*sz**2 &
         +4.*sx**2*s2y*s2z*c2x*sy**2*c2z - s2x**2*sy**3*s2z*c2z*cy&
         +s2x**2*s2y*sz**3*c2y*cz&
         +2.*sx*s2y*s2z*cx*s2x*c2y*sz**2 - 2.*sx*s2y*s2z*cx*s2x*sy**2*c2z&
         -5.*nu*s2x*c2y*pi*sz**2 + 5.*nu*s2x*sy**2*c2z*pi - nu*s2x*cy**2*pi*c2z + nu*s2x*c2y*pi*cz**2)

    s(2)=4.*pi**2*(4.*sx**3*s2y**2*s2z*c2z*cx - 4.*s2x*sy**2*s2z*sx**2*c2y*c2z &
         -2.*s2x*sy**2*s2z*c2x*c2y*sz**2 - s2x*s2y**2*sz**3*c2x*cz &
         +2.*s2x*sy*s2z*cy*sx**2*s2y*c2z&
         +s2x*sy*s2z*cy*c2x*s2y*sz**2 - 2.*nu*cx**2*pi*s2y*c2z + 10.*nu*sx**2*s2y*c2z*pi&
         +5.*nu*c2x*pi*s2y*sz**2 - nu*c2x*pi*s2y*cz**2)

    s(3)=-4.*pi**2*(4.*sx**3*s2y*s2z**2*c2y*cx - s2x*sy**3*s2z**2*c2x*cy&
         -2.*s2x*s2y*sz**2*c2x*sy**2*c2z - 4.*s2x*s2y*sz**2*sx**2*c2y*c2z&
         +s2x*s2y*sz*cz*c2x*sy**2*s2z + 2.*s2x*s2y*sz*cz*sx**2*c2y*s2z&
         +5.*nu*c2x*pi*sy**2*s2z - 2.*nu*cx**2*pi*c2y*s2z + 10.*nu*sx**2*c2y*pi*s2z - nu*c2x*pi*cy**2*s2z) 

    !sol analytique stationnaire: sans strech ni diffusion
    !-------------------------------------------------------
!!$    s(1)=4.*pi**2*(-2.*sx**2*s2y*s2z*c2x*c2y*sz**2&
!!$         +2.*sx**2*s2y*s2z*c2x*sy**2*c2z - s2x**2*sy**3*s2z*c2z*cy&
!!$         +s2x**2*s2y*sz**3*c2y*cz)
!!$
!!$    s(2)=4.*pi**2*(4.*sx**3*s2y**2*s2z*c2z*cx - 2.*s2x*sy**2*s2z*sx**2*c2y*c2z &
!!$         -s2x*sy**2*s2z*c2x*c2y*sz**2 - s2x*s2y**2*sz**3*c2x*cz)
!!$
!!$    s(3)=-4.*pi**2*(4.*sx**3*s2y*s2z**2*c2y*cx - s2x*sy**3*s2z**2*c2x*cy&
!!$             -s2x*s2y*sz**2*c2x*sy**2*c2z - 2.*s2x*s2y*sz**2*sx**2*c2y*c2z)


    !sol analytique stationnaire: que diffusion
    !---------------------------------------------
!!$    s(1)=4.*pi**2*(-2.*sx**2*s2y*s2z*c2x*c2y*sz**2 &
!!$         +2.*sx**2*s2y*s2z*c2x*sy**2*c2z - s2x**2*sy**3*s2z*c2z*cy&
!!$         +s2x**2*s2y*sz**3*c2y*cz - 5.*nu*s2x*c2y*pi*sz**2&
!!$         + 5.*nu*s2x*sy**2*c2z*pi - nu*s2x*cy**2*pi*c2z + nu*s2x*c2y*pi*cz**2)
!!$          
!!$
!!$    s(2)=4.*pi**2*(4.*sx**3*s2y**2*s2z*c2z*cx - 2.*s2x*sy**2*s2z*sx**2*c2y*c2z &
!!$         -s2x*sy**2*s2z*c2x*c2y*sz**2 - s2x*s2y**2*sz**3*c2x*cz &         
!!$         - 2.*nu*cx**2*pi*s2y*c2z + 10.*nu*sx**2*s2y*c2z*pi&
!!$         +5.*nu*c2x*pi*s2y*sz**2 - nu*c2x*pi*s2y*cz**2)
!!$
!!$    s(3)=-4.*pi**2*(4.*sx**3*s2y*s2z**2*c2y*cx - s2x*sy**3*s2z**2*c2x*cy&
!!$         -s2x*s2y*sz**2*c2x*sy**2*c2z - 2.*s2x*s2y*sz**2*sx**2*c2y*c2z&        
!!$         +5.*nu*c2x*pi*sy**2*s2z - 2.*nu*cx**2*pi*c2y*s2z + 10.*nu*sx**2*c2y*pi*s2z - nu*c2x*pi*cy**2*s2z) 

    !sol analytique stationnaire: que strech
    !----------------------------------------
!!$    s(1)=4.*pi**2*(-4.*sx**2*s2y*s2z*c2x*c2y*sz**2 &
!!$         +4.*sx**2*s2y*s2z*c2x*sy**2*c2z - s2x**2*sy**3*s2z*c2z*cy&
!!$         +s2x**2*s2y*sz**3*c2y*cz&
!!$         +2.*sx*s2y*s2z*cx*s2x*c2y*sz**2 - 2.*sx*s2y*s2z*cx*s2x*sy**2*c2z)
!!$
!!$    s(2)=4.*pi**2*(4.*sx**3*s2y**2*s2z*c2z*cx - 4.*s2x*sy**2*s2z*sx**2*c2y*c2z &
!!$         -2.*s2x*sy**2*s2z*c2x*c2y*sz**2 - s2x*s2y**2*sz**3*c2x*cz &
!!$         +2.*s2x*sy*s2z*cy*sx**2*s2y*c2z&
!!$         +s2x*sy*s2z*cy*c2x*s2y*sz**2)
!!$
!!$    s(3)=-4.*pi**2*(4.*sx**3*s2y*s2z**2*c2y*cx - s2x*sy**3*s2z**2*c2x*cy&
!!$         -2.*s2x*s2y*sz**2*c2x*sy**2*c2z - 4.*s2x*s2y*sz**2*sx**2*c2y*c2z&
!!$         +s2x*s2y*sz*cz*c2x*sy**2*s2z + 2.*s2x*s2y*sz*cz*sx**2*c2y*s2z)


  end subroutine source_anal



end module source_mod
