module utile_mod
  use donnees_mod
  use tab_mod
contains


   subroutine make_grille
     implicit none
     integer            :: i,j,k

     do k=1,nz
        ztab(k)=zd+(k-1)*dz
     end do
     do j=1,ny
        ytab(j)=yb+(j-1)*dy
     end do
     do i=1,nx
        xtab(i)=xg+(i-1)*dx
     end do
        
   end subroutine make_grille



 subroutine make_bloc(esp)
    use donnees_mod
    implicit none
    integer,intent(in)       :: esp
    integer                  :: fin,ib,i,j,k,l,im,ip,dim1,dim2,dim3
    logical                  :: zero
    real(kind=8)             :: pas

    select case(esp)
    case(1)
       dim1=nx
       dim2=ny
       dim3=nz
       pas=dx
    case(2)
       dim1=ny
       dim2=nz
       dim3=nx
       pas=dy
    case(3)
       dim1=nz
       dim2=nx
       dim3=ny
       pas=dz
    end select


    !-----------------------------------------------------------------------------------------
    !Determine la nature des blocs
    !Donne un flag correspondant au type de remaillage necessaire (centre,decentre ou modifie)
    !------------------------------------------------------------------------------------------

    blocg(1:npart)=1
    blocd(1:npart)=1    
         
    direction1: do l=1,dim3

    direction2: do j=1,dim2

       !peut calculer la longeur -> gain si longeur eleve

       !parcours la grille, bloc par bloc:determine le type des bloc
       !---------------------------------

       blocg(0)=100
       blocd(0)=100
       Nbloc(0)=100

       type_bloc:do i=1,dim1,long_bloc+1


          if ( (i+long_bloc)<(dim1-1) ) then
             fin=i+long_bloc
             call sub_typeb
          else
             !parcours le bloc plus premier point du bloc suivant
             fin=dim1-1
             call sub_typeb            
             blocg(pg(dim1,j,l))=blocg(pg(dim1-1,j,l))
             blocd(pg(dim1,j,l))=blocd(pg(dim1-1,j,l))               
             Nbloc(pg(dim1,j,l))=Nbloc(pg(dim1-1,j,l))            
          end if         

       end do type_bloc


       !parcours la grille, bloc par bloc:determine si modif de remaillage necessaire a l'intersection des blocs
       !---------------------------------
       type_remaill:do i=1,dim1,long_bloc+1

          if (((i+long_bloc)<dim1).and.(i>1+long_bloc)) then            
             fin=i+long_bloc
             call sub_remb
          else             
             if (i<=1+long_bloc)then
                !premier bloc
                !-------------
                fin=i+long_bloc
                call sub_remb              

             else
                !dernier bloc 
                !-------------
                fin=dim1
                call sub_remb

             end if

          end if

       end do type_remaill
    end do direction2
 end do direction1

  contains

    subroutine sub_typeb

      integer            :: ni,N
      real(kind=8)       :: li
      logical            :: pasfait


      !parcours une premiere fois le bloc pour savoir si c'est un bloc en 0 ou 1/2
      !----------------------------------     
      pasfait=.true.
      zero=.true.
      do_zero:do ib=i,fin+1  !eviter cas change de bloc quand maille vide
         if (pg(ib,j,l)/=0) then
            if (pasfait) then  !le N doit etre le meme pour tout le bloc !
               li=dt/pas*vit(pg(ib,j,l))
               ni=floor(li+0.5D0)
               pasfait=.false.
            end if
            li=dt/pas*vit(pg(ib,j,l))
            if ( ((li)<(ni-0.5D0)).or.((li)>(ni+0.5D0)) ) then
               zero=.false.
               exit do_zero 
            end if
         end if
      end do do_zero


      !parcours a nouveau le bloc 
      !-------------------------
      !affecte le type de bloc a toute les part du bloc

      pasfait=.true.
      bloc2:do ib=i,fin

         if (pg(ib,j,l)/=0) then

            !calcul de N
            !------------   
            if (pasfait) then
               if (zero) then                 
                  N=floor(dt/pas*vit(pg(ib,j,l))+0.5D0)
                  pasfait=.false.
               else
                  N=floor(dt/pas*vit(pg(ib,j,l)))
                  pasfait=.false.
               end if
            end if

            if (zero) then
               !bloc pour lambda2 centre
               blocg(pg(ib,j,l))=0
               blocd(pg(ib,j,l))=0                
            end if

            !affecte la constante N               
            Nbloc(pg(ib,j,l))=N

         end if
      end do bloc2

    end subroutine sub_typeb


    subroutine sub_remb
      implicit none

      integer,dimension(-2:2)  :: iper
      integer                  :: k


      !parcours  a nvx le bloc     
      !-----------------------
      !detecte s'il y aura une correction de remaillage a apporter à l'intersection de ces blocs

      bloc3:do ib=i,fin,fin-i

         if (pg(ib,j,l)/=0) then

            !si fin de bloc en 0. :
            !----------------------
            do k=-2,2              
               iper(k)=pg(mod(ib+k+dim1-1,dim1)+1,j,l)          
            end do


            if ((ib==fin).and.(blocd(pg(ib,j,l))==0).and.((blocg(iper(1))==1) &
		.or.(blocg(iper(2))==1)).and.((Nbloc(iper(1))==Nbloc(pg(ib,j,l))-1) &
		.or.(Nbloc(iper(2))==Nbloc(pg(ib,j,l))-1) ) ) then

               if (type_b==2) then

                  if ( dt/pas*vit(pg(ib,j,l))<=Nbloc(pg(ib,j,l))  ) then
                     blocg(iper(1))=2
                     blocd(pg(ib,j,l))=3
                  else
                     blocg(iper(1))=2
                     blocd(pg(ib,j,l))=4
                     blocg(pg(ib,j,l))=5
                  end if

               else
                  blocg(iper(1))=2
                  blocg(iper(2))=5

                  if ( dt/pas*vit(pg(ib,j,l))<=Nbloc(pg(ib,j,l))  ) then                 
                     blocd(pg(ib,j,l))=3
                  else                 
                     blocd(pg(ib,j,l))=7
                     blocg(pg(ib,j,l))=8
                  end if

                  if ( dt/pas*vit(iper(-1))<=Nbloc(pg(ib,j,l))  ) then
                     blocd(iper(-1))=6
                  else
                     blocd(iper(-1))=5
                  end if
               end if

            end if


            !si debut de bloc en 0 : detecte quelle correction il faudra apporter
            !----------------------
            if ((ib==i) .and.(blocg(pg(ib,j,l))==0).and.((blocd(iper(-1))==1) &
		.or.(blocd(iper(-2))==1)).and.((Nbloc(iper(-1))==Nbloc(pg(ib,j,l))-1) &
		.or. (Nbloc(iper(-2))==Nbloc(pg(ib,j,l))-1) ) ) then

               if (type_b==2) then

                  if ( dt/pas*vit(pg(ib,j,l))>Nbloc(pg(ib,j,l)) ) then
                     blocd(iper(-1))=2
                     blocg(pg(ib,j,l))=4
                  else
                     blocd(iper(-1))=2
                     blocg(pg(ib,j,l))=3
                  end if

               else

                  blocd(iper(-1))=2
                  blocd(iper(-2))=4

                  if ( dt/pas*vit(pg(ib,j,l))>Nbloc(pg(ib,j,l)) ) then                 
                     blocg(pg(ib,j,l))=4
                  else                 
                     blocg(pg(ib,j,l))=3
                  end if

                  if ( dt/pas*vit(iper(1))>Nbloc(pg(ib,j,l)) ) then
                     blocg(iper(1))=7
                  else
                     blocg(iper(1))=6
                  end if

               end if
            end if



         end if
      end do bloc3

    end subroutine sub_remb

    function pg(a,b,c) 
      implicit none
      integer,intent(in)  :: a,b,c
      integer             :: pg

      select case (esp)
      case(1)
         pg=numpg(a,b,c)
      case(2)
         pg=numpg(c,a,b)
      case(3)
         pg=numpg(b,c,a)
      end select

    end function pg

    function vit(ind) 
      implicit none
      integer,intent(in)  :: ind
      real(kind=8)        :: vit

      select case (esp)
      case(1)
         vit=vx(ind)
      case(2)
         vit=vy(ind)
      case(3)
         vit=vz(ind)
      end select

    end function vit


  end subroutine make_bloc


  subroutine imposer_debit(u,debit,dimx,dimy,dimz,deltax,deltay,deltaz)
    implicit none
    real(kind=8),dimension(:,:,:),intent(inout)  :: u
    real(kind=8),intent(in)     :: debit,deltax,deltay,deltaz
    integer,intent(in)          :: dimx,dimy,dimz
    real(kind=8)                :: debit0
    integer                     :: i,j,k

    !calcul du debit volumique actuel: int_x int_y int_z u dx dy dz
    !---------------------------------
    debit0=0.

    do k=1,dimz
       do j=1,dimy
          debit0=debit0+u(1,j,k)
       end do
    end do    
    debit0=debit0*deltay*deltaz
       
    !on impose le débit voulu
    !------------------------
    do i=1,dimx
       do k=1,dimz
          do j=1,dimy
             u(i,j,k)=u(i,j,k)-debit0/(dimy*dimz*deltay*deltaz)!soustraction du debit actuel
             u(i,j,k)=u(i,j,k)+debit/(dimy*dimz*deltay*deltaz)!affectation de la valeur du débit désirée
          end do
       end do
    end do

    !ON a ainsi:
    !------------
    !
    !int u(nouveau) dx dy dz = dx dy dz sum u (nouveau)
    !                        = dx dy dz sum u (old) - debit0 + debit = debit
    !
    !


  end subroutine imposer_debit






  end module utile_mod


