module data
  use parallel
  use precision
  use parser
  use fileio
  use string
  implicit none


  integer :: nvar

  real(WP) :: L
  real(WP) :: Ut, le, ld, epsilon, mu,diff
  character(len=str_short) :: spectrum
  character(len=str_medium) :: spfilename,spfilename2

  integer :: file_counter,file_fourier

  real(WP), dimension(:,:,:,:), pointer :: data_store
  real(WP), dimension(:,:,:), pointer :: data_storesc
  real(WP), dimension(:,:,:), pointer :: U
  real(WP), dimension(:,:,:), pointer :: V
  real(WP), dimension(:,:,:), pointer :: W
  real(WP), dimension(:,:,:), pointer :: SC

  type MPI_IO_VAR
     real(WP), dimension(:,:,:), pointer :: var
     integer :: view
  end type MPI_IO_VAR

  type(MPI_IO_VAR), dimension(:), pointer :: MPI_IO_DATA

  integer, dimension(4) :: dims_w  

end module data

subroutine data_init

  use data

  implicit none

  integer :: gsizes(4),lsizes(4),var,start(4),ierr,nv_fourier
  integer :: gsizessc(3),lsizessc(3),startsc(3)
  real(WP) :: sch

!  call parser_read('Number of points',nx)
!  call parser_read('Number of points for scalar',nxsc)
  print*,'nx,nxsc',nx,nxsc
  call parser_read('Length of domain', L)
  call parser_read('Spectrum form',spectrum)
  call parser_read('Viscosity',mu,1e-02_WP)
  call parser_read('Schmidt number',sch,0.7_WP)
  diff = mu/sch

  ny = nx
  nz = nx
  nvar = 3
  nv_fourier = 3
  nysc = nxsc
  nzsc = nxsc

  allocate(data_store(ns1,ns2,ns3,nvar))
  allocate(data_storesc(ns1sc,ns2sc,ns3sc))

  data_store = 0.0_WP
  data_storesc = 0.0_WP
  U    => data_store(:,:,:,1); ! names(1)  = 'U'
  V    => data_store(:,:,:,2); ! names(2)  = 'V'
  W    => data_store(:,:,:,3); ! names(3)  = 'W'
  SC => data_storesc(:,:,:); ! names(4)  = 'P'


  dims_w(1) = nx
  dims_w(2) = ny
  dims_w(3) = nz
  dims_w(4) = 3

  allocate(MPI_IO_DATA(dims_w(4)+1))

  do var = 1,dims_w(4)
     MPI_IO_DATA(var)%var => data_store(:,:,:,var)
  enddo
  MPI_IO_DATA(4)%var => data_storesc(:,:,:)
   
  gsizes(1) = nx
  gsizes(2) = ny
  gsizes(3) = nz
  gsizes(4) = dims_w(4)

  lsizes(1) = ns1
  lsizes(2) = ns2
  lsizes(3) = ns3
  lsizes(4) = 1

  start(1) = nxs-1
  start(2) = nys-1
  start(3) = nzs-1


  do var = 1, dims_w(4)
     start(4) = var -1
     call MPI_TYPE_CREATE_SUBARRAY(4,gsizes,lsizes,start,&
          &MPI_ORDER_FORTRAN,MPI_REAL_WP,MPI_IO_DATA(var)%view,ierr)
     call MPI_TYPE_COMMIT(MPI_IO_DATA(var)%view,ierr)
  end do

  gsizessc(1) = nxsc
  gsizessc(2) = nysc
  gsizessc(3) = nzsc

  lsizessc(1) = ns1sc
  lsizessc(2) = ns2sc
  lsizessc(3) = ns3sc

  startsc(1) = nxssc-1
  startsc(2) = nyssc-1
  startsc(3) = nzssc-1


  call MPI_TYPE_CREATE_SUBARRAY(3,gsizessc,lsizessc,startsc,&
       &MPI_ORDER_FORTRAN,MPI_REAL_WP,MPI_IO_DATA(4)%view,ierr)
  call MPI_TYPE_COMMIT(MPI_IO_DATA(4)%view,ierr)


  file_counter = 0
  file_fourier = 0 
end subroutine data_init

subroutine data_write

  use parallel
  use data
  implicit none


  integer :: ifile, ierr, var, data_size
  integer, dimension(MPI_STATUS_SIZE) :: status
  integer(KIND=MPI_OFFSET_KIND) :: disp
  character(len=str_medium) :: filename,buffer
  
  logical :: file_is_there

  file_counter = file_counter+1
 
  call parser_read('Data file to write',filename)
  write(buffer,'(I3)') file_counter
  spfilename = trim(adjustl(filename))//'_sp_'//trim(adjustl(buffer))
  spfilename2 = trim(adjustl(filename))//'_sz_'//trim(adjustl(buffer))
  call get_dns_numbers(0)

  filename = trim(filename)
  write(buffer,'(I3)') file_counter
  filename = trim(adjustl(filename))//'-'//trim(adjustl(buffer))

    
  inquire(file=filename, exist=file_is_there)
  if (file_is_there) call MPI_FILE_DELETE(filename,MPI_INFO_NULL,ierr)
  call MPI_FILE_OPEN(MPI_COMM_WORLD,filename,MPI_MODE_WRONLY+MPI_MODE_CREATE, &
       MPI_INFO_NULL,ifile,ierr)
  
  ! Write header
  if (rank.eq.0) then
     ! Write dimensions
     call MPI_FILE_WRITE(ifile,dims_w,4,MPI_INTEGER,status,ierr)
  end if
!!$  print*,dims_w

!!$  if (rank.eq.0) then
!!$     print *,' Computed values before writing ',MPI_IO_DATA(2)%var(1:5,1,3)
!!$     print *,' data values from datastore ', data_store(nxs:nxs+5,nys,nzs,2)
!!$  endif

  disp = 4*4
  data_size = ns1*ns2*ns3
  do var = 1,dims_w(4)
     call MPI_FILE_SET_VIEW(ifile,disp,MPI_REAL_WP,MPI_IO_DATA(var)%view,&
          &"native",MPI_INFO_NULL,ierr)
     call MPI_FILE_WRITE_ALL(ifile,MPI_IO_DATA(var)%var,data_size,&
          &MPI_REAL_WP,status,ierr)
  enddo

  disp = 4_MPI_OFFSET_KIND*4 + 8_MPI_OFFSET_KIND*ns1*ns1*ns1*3
  data_size = ns1sc*ns2sc*ns3sc
  call MPI_FILE_SET_VIEW(ifile,disp,MPI_REAL_WP,MPI_IO_DATA(4)%view,&
       &"native",MPI_INFO_NULL,ierr)
  call MPI_FILE_WRITE_ALL(ifile,MPI_IO_DATA(4)%var,data_size,&
       &MPI_REAL_WP,status,ierr)
!  if(rank.eq.0) print*,' size data write 1 : ',4*4 + 8*ns1*ns1*ns1*3 + 8*nproc*ns1sc*ns2sc*ns3sc
!  if(rank.eq.0) print*,' size data write 2 : ',ns1,nproc,ns1sc,ns2sc,ns3sc
!  if(rank.eq.0) print*,' size data write 3 : ',disp

!!$  if (rank.eq.0) then
!!$     print *,' test writing ',MPI_IO_DATA(1)%var(1,1,1)
!!$     print *,' test writing ',MPI_IO_DATA(2)%var(1,1,1)
!!$     print *,' test writing ',MPI_IO_DATA(3)%var(1,1,1)
!!$     print *,' test writing ',MPI_IO_DATA(4)%var(1,1,1)
!!$  endif

  call MPI_FILE_CLOSE(ifile,ierr)

end subroutine data_write


subroutine tab_write(tab,filename)

  use parallel
  use data
  implicit none

  real(WP) :: tab(ns1,ns2,ns3)

  integer :: ifile, ierr, var, data_size, view1
  integer :: gsize1(4),lsize1(4),start1(4)
  integer, dimension(MPI_STATUS_SIZE) :: status
  integer(KIND=MPI_OFFSET_KIND) :: disp
  character*13 :: filename,buffer
  
  logical :: file_is_there

  inquire(file=filename, exist=file_is_there)
  if (file_is_there) call MPI_FILE_DELETE(filename,MPI_INFO_NULL,ierr)

  call MPI_FILE_OPEN(MPI_COMM_WORLD,filename,MPI_MODE_WRONLY+MPI_MODE_CREATE, &
       MPI_INFO_NULL,ifile,ierr)
  disp = 4*4
  data_size = ns1*ns2*ns3

  gsize1(1) = nx
  gsize1(2) = ny
  gsize1(3) = nz
  gsize1(4) = 1

  lsize1(1) = ns1
  lsize1(2) = ns2
  lsize1(3) = ns3
  lsize1(4) = 1 

  start1(1) = nxs-1
  start1(2) = nys-1
  start1(3) = nzs-1
  start1(4) = 0

  call MPI_TYPE_CREATE_SUBARRAY(4,gsize1,lsize1,start1,&
          &MPI_ORDER_FORTRAN,MPI_REAL_WP,view1,ierr)
  call MPI_TYPE_COMMIT(view1,ierr)

  call MPI_FILE_SET_VIEW(ifile,disp,MPI_REAL_WP,view1,&
          &"native",MPI_INFO_NULL,ierr)

  call MPI_FILE_WRITE_ALL(ifile,tab,data_size,&
          &MPI_REAL_WP,status,ierr)


  !if(rank.eq.0) print*,'1',' 1',' 1',tab(1,1,1)
  !if(rank.eq.0) print*,'1',' 1',' 32',tab(1,1,32)
  !if(rank.eq.7) print*,'1',' 1',' 256',tab(1,1,32)
  !if(rank.eq.7) print*,'1',' 128',' 256',tab(1,128,32)

  call MPI_FILE_CLOSE(ifile,ierr)

end subroutine tab_write

subroutine tab_write_ne(tab,filename,ns1ne,ns2ne,ns3ne)

  use parallel
  use data
  implicit none


  integer :: ns1ne,ns2ne,ns3ne
  real(WP) :: tab(ns1ne,ns2ne,ns3ne)

  integer :: ifile, ierr, var, data_size, view1
  integer :: gsize1(4),lsize1(4),start1(4)
  integer, dimension(MPI_STATUS_SIZE) :: status
  integer(KIND=MPI_OFFSET_KIND) :: disp
  character*13 :: filename,buffer
  
  logical :: file_is_there

  inquire(file=filename, exist=file_is_there)
  if (file_is_there) call MPI_FILE_DELETE(filename,MPI_INFO_NULL,ierr)

  call MPI_FILE_OPEN(MPI_COMM_WORLD,filename,MPI_MODE_WRONLY+MPI_MODE_CREATE, &
       MPI_INFO_NULL,ifile,ierr)
  disp = 4*4
  data_size = ns1ne*ns2ne*ns3ne

  gsize1(1) = ns1ne
  gsize1(2) = ns1ne
  gsize1(3) = ns1ne
  gsize1(4) = 1

  lsize1(1) = ns1ne
  lsize1(2) = ns2ne
  lsize1(3) = ns3ne
  lsize1(4) = 1 

  start1(1) = 0
  start1(2) = 0
  start1(3) = (rank)*ns3ne
  start1(4) = 0

  call MPI_TYPE_CREATE_SUBARRAY(4,gsize1,lsize1,start1,&
          &MPI_ORDER_FORTRAN,MPI_REAL_WP,view1,ierr)
  call MPI_TYPE_COMMIT(view1,ierr)

  call MPI_FILE_SET_VIEW(ifile,disp,MPI_REAL_WP,view1,&
          &"native",MPI_INFO_NULL,ierr)

  call MPI_FILE_WRITE_ALL(ifile,tab,data_size,&
          &MPI_REAL_WP,status,ierr)


  !if(rank.eq.0) print*,'1',' 1',' 1',tab(1,1,1)
  !if(rank.eq.0) print*,'1',' 1',' 32',tab(1,1,32)
  !if(rank.eq.7) print*,'1',' 1',' 256',tab(1,1,32)
  !if(rank.eq.7) print*,'1',' 128',' 256',tab(1,128,32)

  call MPI_FILE_CLOSE(ifile,ierr)

end subroutine tab_write_ne
  
  

subroutine data_read

  use parallel
  use data
  implicit none


  integer :: ifile, ierr, var, data_size
  integer, dimension(MPI_STATUS_SIZE) :: status
  integer(KIND=MPI_OFFSET_KIND) :: disp
  character(len=str_medium) :: filename,buffer
  
  logical :: file_is_there


 
  call parser_read('Data file to read',filename)
  filename = trim(filename)
  call MPI_FILE_OPEN(MPI_COMM_WORLD,filename,MPI_MODE_RDONLY, &
       MPI_INFO_NULL,ifile,ierr)
  
  ! Write header
  if (rank.eq.0) then
     ! Write dimensions
     call MPI_FILE_READ(ifile,dims_w,4,MPI_INTEGER,status,ierr)
  end if

!!$  if (rank.eq.0) then
!!$     print *,' Computed values before writing ',MPI_IO_DATA(2)%var(1:5,1,3)
!!$     print *,' data values from datastore ', data_store(nxs:nxs+5,nys,nzs,2)
!!$  endif

  

  disp = 4*4
  data_size = ns1*ns2*ns3
  do var = 1,dims_w(4)
     call MPI_FILE_SET_VIEW(ifile,disp,MPI_REAL_WP,MPI_IO_DATA(var)%view,&
          &"native",MPI_INFO_NULL,ierr)
     call MPI_FILE_READ_ALL(ifile,MPI_IO_DATA(var)%var,data_size,&
          &MPI_REAL_WP,status,ierr)
  enddo


  disp = 4_MPI_OFFSET_KIND*4 + 8_MPI_OFFSET_KIND*ns1*ns1*ns1*3
  data_size = ns1sc*ns2sc*ns3sc


  call MPI_FILE_SET_VIEW(ifile,disp,MPI_REAL_WP,MPI_IO_DATA(4)%view,&
          &"native",MPI_INFO_NULL,ierr)


  call MPI_FILE_READ_ALL(ifile,MPI_IO_DATA(4)%var,data_size,&
          &MPI_REAL_WP,status,ierr)


  call MPI_FILE_CLOSE(ifile,ierr)

  
end subroutine data_read


subroutine data_filter_sc

  use parallel
  use data

  implicit none

  integer :: nx2,ny2,nz2
  integer :: ns1f,ns2f,ns3f
  integer :: i,j,k,jl
  integer :: nk2, fnys2,fnye2,nkk

  real(WP) :: dk2, pi,kk,grms,buf, xx

  real(WP), dimension(:,:,:), pointer :: tab 
  real(WP), dimension(:,:,:), pointer :: SCF, UF, VF, WF

  integer :: gsizes(4),lsizes(4),var,start(4),ierr,nv_fourier,ifilter
  integer :: gsizessc(3),lsizessc(3),startsc(3)

  call parser_read('final number of points',nx2)
  call parser_read('filter',ifilter,0)

  ny2 = nx2
  nz2 = nx2

  ns1f = nx2
  ns2f = nx2
  ns3f = nx2/nproc

  ! -- filtering operation >> UF, VF, WF, SCF
  allocate(SCF(ns1f,ns2f,ns3f))
  allocate(UF(ns1,ns2,ns3))
  allocate(VF(ns1,ns2,ns3))
  allocate(WF(ns1,ns2,ns3))

  !-- -- -- compute the fourier coefficient
  pi = acos(-1.0_WP)
  dk2 = 2.0_WP*pi/L

  nk2 = nx2/2+1
  nkk = nxsc/2+1

  !-- -- -- SPECTRAL SPACE : SC_nx -> SCF_nx2 

  allocate(tab(ns1sc,ns2sc,ns3sc))
  tab=SC**2
  buf=0.
  do k=1,ns3sc
     do j=1,ns2sc
        do i=1,ns1sc
           buf=buf+tab(i,j,k)
        enddo
     enddo
  enddo
  call MPI_ALLREDUCE(buf,grms,1,MPI_REAL_WP,MPI_SUM,              &
                      MPI_COMM_WORLD,ierr)
  if (rank.eq.0) print*," U rms =",grms/(2.0*ns1sc**3)
  deallocate(tab)

if (nk2.le.nkk) then

!   do i = 1, ns1sc
!      xx = (i-1)*L/(ns1sc-1)
!      SC(i,:,:) = sin(xx)
!      print*,xx,SC(i,1,1)
!   end do

   call discretisation3(SC,SCF,ns1sc,ns2sc,ns3sc,ns1f,ns2f,ns3f)

   UF = U
   VF = V
   WF = W
   call barrier
   if (rank.eq.0) print*,'SC done'

!   do i = 1, ns1f
!      xx = (i-1)*L/(ns1f-1)
!      print*,xx,SCF(i,2,2)
!   end do
  
else

   call discretisation2(SC,SCF,ns1sc,ns2sc,ns3sc,ns1f,ns2f,ns3f)
   UF = U
   VF = V
   WF = W

end if
 
  ns1sc = ns1f
  ns2sc = ns2f
  ns3sc = ns3f

  allocate(tab(ns1sc,ns2sc,ns3sc))
  tab=SCF**2
  buf=0.
  do k=1,ns3sc
     do j=1,ns2sc
        do i=1,ns1sc
           buf=buf+tab(i,j,k)
        enddo
     enddo
  enddo
  call MPI_ALLREDUCE(buf,grms,1,MPI_REAL_WP,MPI_SUM,              &
                      MPI_COMM_WORLD,ierr)
  print*," U rms =",grms/(2.0*ns1sc**3.),ns1sc,ns1f
  deallocate(tab)

!!  do i = 1, ns1sc
!!     do j = 1, ns2sc
!!        do k = 1, ns3sc
!!           SCF(i,j,k) = 100*i + 10*j + k
!!           print*,'SCF',i,j,k,SCF(i,j,k)
!!        end do
!!     end do
!!  end do


  ! -- end filtering operation

  deallocate(data_storesc)
  deallocate(data_store)

  nxsc = nx2
  nysc = ny2
  nzsc = nz2

  nxssc = 1
  nxesc = nxsc
  nyssc = 1
  nyesc = nysc
  nzssc = 1 +(rank)*ns3sc
  nzesc = (rank+1)*ns3sc
  
  allocate(data_store(ns1,ns2,ns3,nvar))
  allocate(data_storesc(ns1sc,ns2sc,ns3sc))

  data_store = 0.0_WP
  data_storesc = 0.0_WP
  U    => data_store(:,:,:,1); ! names(1)  = 'U'
  V    => data_store(:,:,:,2); ! names(2)  = 'V'
  W    => data_store(:,:,:,3); ! names(3)  = 'W'
  SC => data_storesc(:,:,:); ! names(4)  = 'P'


  dims_w(1) = nx
  dims_w(2) = ny
  dims_w(3) = nz
  dims_w(4) = 3

  deallocate(MPI_IO_DATA)
  allocate(MPI_IO_DATA(dims_w(4)+1))

  do var = 1,dims_w(4)
     MPI_IO_DATA(var)%var => data_store(:,:,:,var)
  enddo
  MPI_IO_DATA(4)%var => data_storesc(:,:,:)
   
  gsizes(1) = nx
  gsizes(2) = ny
  gsizes(3) = nz
  gsizes(4) = dims_w(4)

  lsizes(1) = ns1
  lsizes(2) = ns2
  lsizes(3) = ns3
  lsizes(4) = 1

  start(1) = nxs-1
  start(2) = nys-1
  start(3) = nzs-1


  do var = 1, dims_w(4)
     start(4) = var -1
     call MPI_TYPE_CREATE_SUBARRAY(4,gsizes,lsizes,start,&
          &MPI_ORDER_FORTRAN,MPI_REAL_WP,MPI_IO_DATA(var)%view,ierr)
     call MPI_TYPE_COMMIT(MPI_IO_DATA(var)%view,ierr)
  end do

  gsizessc(1) = nxsc
  gsizessc(2) = nysc
  gsizessc(3) = nzsc

  lsizessc(1) = ns1sc
  lsizessc(2) = ns2sc
  lsizessc(3) = ns3sc

  startsc(1) = nxssc-1
  startsc(2) = nyssc-1
  startsc(3) = nzssc-1


  call MPI_TYPE_CREATE_SUBARRAY(3,gsizessc,lsizessc,startsc,&
       &MPI_ORDER_FORTRAN,MPI_REAL_WP,MPI_IO_DATA(4)%view,ierr)
  call MPI_TYPE_COMMIT(MPI_IO_DATA(4)%view,ierr)

  SC = SCF
  U = UF
  V = VF
  W = WF
  deallocate(SCF)
  deallocate(UF)
  deallocate(VF)
  deallocate(WF)
!!  do i = 1, ns1sc
!!     do j = 1, ns2sc
!!        do k = 1, ns3sc
!!           print*,'SC',i,j,k,SC(i,j,k)
!!        end do
!!     end do
!!  end do

  file_counter = 0
end subroutine data_filter_sc

