subroutine init_scalar

  use parallel
  use random
  use transform
  use data
  implicit none
!  include 'fftw3.f'
  
  ! Spectrum computation
  real(WP) :: psr,ps1,ps2
  real(WP) :: ke,kd,ks,dk,ksk0ratio,kc,kcksratio,kk,kx,ky,kz,kk2,kcut
  real(WP) :: alpha,spec_amp,eps,amp_disc,f_phi,sch
  integer  :: kl,jl,il,impose
  real(WP) :: e_total,energy_spec,diss_total,ndense
  real(WP), dimension(:,:), pointer :: spect,sg
  real(WP), dimension(:), pointer :: s1,s2
  complex(WP), dimension(:,:,:), pointer :: ak,bk
  real(WP) :: scmean,scmean_global
  ! Other
  integer :: i,j,k,ik,iunit,dim
  real(WP), dimension(:), pointer :: nnodes
  complex(WP) :: ii=(0.0_WP,1.0_WP)
  real(WP) :: rand,pi,spec,kfix

  real(WP) :: keta, Rlambda, L11, tke,L_large, dissipation,eta
  real(WP) :: e_spec,d_spec, e_local, d_local, spec_factor
  ! Fourier coefficients
  real(WP) :: umax,vmax,wmax,cos,sin
  complex(WP), dimension(:,:,:), pointer :: Uk,Vk,Wk,scalar
  complex(WP), dimension(:,:,:), pointer :: Cbuf
  real(WP), dimension(:,:,:), pointer :: Rbuf

  logical :: flg_inplace

  integer :: ierr,gg(3),iforce
  integer,dimension(:,:,:),pointer :: node_index 
  integer(KIND=8) :: plan_c2r

!!$  real(WP), dimension(:,:,:), pointer :: A
!!$  complex(WP), dimension(:,:,:), pointer :: B


  ! Create pi
  pi = acos(-1.0_WP)
  dk = 2.0_WP*pi/L
  
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Input for scalar field
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  
  call parser_read('ks/ko', ksk0ratio)
  ks = ksk0ratio*dk
!!! Assume kc/ks to be 2 as in Eswaran and Pope
  kc = 2.0_WP*ks


  allocate(scalar(fns1,fns2,fns3))

  scalar = (0.0_WP,0.0_WP)

  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           il = i + fnxs - 1
           jl = j + fnys - 1
           kl = k + fnzs - 1
           
           kx=real(il-1,WP)*dk
           ky=real(jl-1,WP)*dk
           if (jl.gt.nk) ky=-real(nx-jl+1,WP)*dk
           kz=real(kl-1,WP)*dk
           if (kl.gt.nk) kz=-real(nx-kl+1,WP)*dk
           kk =sqrt(kx**2+ky**2+kz**2)
           kk2=sqrt(kx**2+ky**2)
           if ((ks-dk/2.0_WP.le.kk).and.(kk.le.ks+dk/2.0_WP)) then
              f_phi = 1.0_WP
           else
              f_phi = 0.0_WP
           endif
           call random_number(rand)
           if (kk.lt.1e-10) then
              scalar(i,j,k) = 0.0_WP
           else
              scalar(i,j,k) = sqrt(f_phi/(4.0_WP*pi*kk**2.0_WP))*exp(ii*2.0_WP*pi*rand)
           endif
        enddo
     enddo
  enddo

  call btran_wrap(scalar,SC,ns1,ns2,ns3,fns1,fns2,fns3)
  
  do k = 1,ns3
     do j = 1,ns2
        do i = 1,ns1
           if (SC(i,j,k).le.0.0_WP) then
              SC(i,j,k) = 0.0_WP
           else
              SC(i,j,k) = 1.0_WP
           endif
        enddo
     enddo
  enddo

  call ftran_wrap(SC,scalar,ns1,ns2,ns3,fns1,fns2,fns3)
  
  kcut = sqrt(2.0_WP)*real(nx,WP)/3.0_WP
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           il = i + fnxs - 1
           jl = j + fnys - 1
           kl = k + fnzs - 1
           
           kx=real(il-1,WP)*dk
           ky=real(jl-1,WP)*dk
           if (jl.gt.nk) ky=-real(nx-jl+1,WP)*dk
           kz=real(kl-1,WP)*dk
           if (kl.gt.nk) kz=-real(nx-kl+1,WP)*dk
           kk =sqrt(kx**2+ky**2+kz**2)
           kk2=sqrt(kx**2+ky**2)

           if (kk.gt.kcut) scalar(i,j,k) = 0.0_WP
           
           if (kk.le.kc) then
              scalar(i,j,k) = scalar(i,j,k)
           else
              scalar(i,j,k) = scalar(i,j,k)*(kc/kk)**2.0_WP
           endif
        enddo
     enddo
  enddo

  call btran_wrap(scalar,SC,ns1,ns2,ns3,fns1,fns2,fns3)
  SC = SC/real(ns1**3.0,WP)


  call parser_read('Imposing mean gradient',impose,0)
  

  if (impose.eq.1) then
     scmean = 0.0_WP
     do k = 1,ns3
        do j = 1,ns2
           do i = 1,ns1
              scmean = scmean + SC(i,j,k)
           enddo
        enddo
     enddo
  
     call MPI_ALLREDUCE(scmean,scmean_global,1,MPI_REAL_WP,MPI_SUM,&
          &MPI_COMM_WORLD,ierr)
     
     scmean = scmean_global/(real(nx,WP)**3.0_WP)
     
!!!!!!!!!!!! Setting scalar to zero - see if it lies within bounds
     do k = 1,ns3
        do j = 1,ns2
           do i = 1,ns1
              SC(i,j,k) = 0.0_WP
           enddo
        enddo
     enddo
     
  endif



  
  ! Inverse Fourier transform

  ! Clean up
  deallocate(scalar)


  
end subroutine init_scalar



