subroutine dynsmagmodel

  use data
  use parallel
  use solver

  integer :: i,j,k
  real(WP) :: deltx,deltt, cst1, cst2
  real(WP) :: Ekf,diss_sgs,diss_mu,tau12,mean_uv, nt

  complex(WP), dimension(:,:,:), pointer :: ddxk, ddyk, ddzk
  real(WP), dimension(:,:,:), pointer :: ddx, ddy, ddz

  real(WP), dimension(:,:,:), pointer :: S11, S12, S13, S22, S23, S33
  real(WP), dimension(:,:,:), pointer :: L11, L12, L13, L22, L23, L33
  real(WP), dimension(:,:,:), pointer :: M11, M12, M13, M22, M23, M33
  real(WP), dimension(:,:,:), pointer :: T11, T12, T13, T22, T23, T33
  real(WP), dimension(:,:,:), pointer :: R11, R12, R13, R22, R23, R33
  real(WP), dimension(:,:,:), pointer :: V11, V12, V13, V22, V23, V33
  real(WP), dimension(:,:,:), pointer :: uft, vft, wft
  real(WP), dimension(:,:,:), pointer :: dudx, dudy, dudz
  real(WP), dimension(:,:,:), pointer :: dvdx, dvdy, dvdz
  real(WP), dimension(:,:,:), pointer :: dwdx, dwdy, dwdz
  real(WP), dimension(:,:,:), pointer :: normS, normR
  real(WP), dimension(:,:), pointer :: SPEC1, SPEC11
  real(WP), dimension(:), pointer :: coefffy, cst1y, cst2y

  deltx = L/(real(nx))
  deltt = 2*deltx
  nt = 2.0

  call u_compute
  
  allocate(dudx(ns1,ns2,ns3))
  allocate(dudy(ns1,ns2,ns3))
  allocate(dudz(ns1,ns2,ns3))
  allocate(dvdx(ns1,ns2,ns3))
  allocate(dvdy(ns1,ns2,ns3))
  allocate(dvdz(ns1,ns2,ns3))
  allocate(dwdx(ns1,ns2,ns3))
  allocate(dwdy(ns1,ns2,ns3))
  allocate(dwdz(ns1,ns2,ns3))
  call gradient(U,0,dudx,dudy,dudz)
  call gradient(V,0,dvdx,dvdy,dvdz)
  call gradient(W,0,dwdx,dwdy,dwdz)
  allocate(S11(ns1,ns2,ns3))
  allocate(S12(ns1,ns2,ns3))
  allocate(S13(ns1,ns2,ns3))
  allocate(S22(ns1,ns2,ns3))
  allocate(S23(ns1,ns2,ns3))
  allocate(S33(ns1,ns2,ns3))
  allocate(normS(ns1,ns2,ns3))
  S11 = dudx
  S22 = dvdy
  S33 = dwdz
  S12 = 0.5*(dudy+dvdx)
  S13 = 0.5*(dudz+dwdx)
  S23 = 0.5*(dvdz+dwdy)
  normS = sqrt(2.0*(S11*S11 + S22*S22 + S33*S33 + 2.0*S12*S12 + 2.0*S13*S13 + 2.0*S23*S23))

  ! - dyn procedure
  allocate(R11(ns1,ns2,ns3))
  allocate(R12(ns1,ns2,ns3))
  allocate(R13(ns1,ns2,ns3))
  allocate(R22(ns1,ns2,ns3))
  allocate(R23(ns1,ns2,ns3))
  allocate(R33(ns1,ns2,ns3))
  R11 = normS*S11
  R12 = normS*S12
  R13 = normS*S13
  R22 = normS*S22
  R23 = normS*S23
  R33 = normS*S33
  allocate(V11(ns1,ns2,ns3))
  allocate(V12(ns1,ns2,ns3))
  allocate(V13(ns1,ns2,ns3))
  allocate(V22(ns1,ns2,ns3))
  allocate(V23(ns1,ns2,ns3))
  allocate(V33(ns1,ns2,ns3))
  call specboxfilter(R11,V11,nt)
  call specboxfilter(R12,V12,nt)
  call specboxfilter(R13,V13,nt)
  call specboxfilter(R22,V22,nt)
  call specboxfilter(R23,V23,nt)
  call specboxfilter(R33,V33,nt)
  allocate(uft(ns1,ns2,ns3))
  allocate(vft(ns1,ns2,ns3))
  allocate(wft(ns1,ns2,ns3))
  call specboxfilter(U,uft,nt)
  call specboxfilter(V,vft,nt)
  call specboxfilter(W,wft,nt)
  call gradient(uft,0,dudx,dudy,dudz)
  call gradient(vft,0,dvdx,dvdy,dvdz)
  call gradient(wft,0,dwdx,dwdy,dwdz)
  R11 = dudx
  R22 = dvdy
  R33 = dwdz
  R12 = 0.5*(dudy+dvdx)
  R13 = 0.5*(dudz+dwdx)
  R23 = 0.5*(dvdz+dwdy)
  allocate(normR(ns1,ns2,ns3))
  normR = 2*sqrt(R11*R11 + R22*R22 + R33*R33 + 2.0*R12*R12 + 2.0*R13*R13 + 2.0*R23*R23)
  deallocate(dudx)
  deallocate(dudy)
  deallocate(dudz)
  deallocate(dvdx)
  deallocate(dvdy)
  deallocate(dvdz)
  deallocate(dwdx)
  deallocate(dwdy)
  deallocate(dwdz)
  R11 = normR*R11
  R12 = normR*R12
  R13 = normR*R13
  R22 = normR*R22
  R23 = normR*R23
  R33 = normR*R33
  deallocate(normR)
  allocate(M11(ns1,ns2,ns3))
  allocate(M12(ns1,ns2,ns3))
  allocate(M13(ns1,ns2,ns3))
  allocate(M22(ns1,ns2,ns3))
  allocate(M23(ns1,ns2,ns3))
  allocate(M33(ns1,ns2,ns3))
  M11 = deltt**2.*R11 - deltx**2.*V11
  M22 = deltt**2.*R22 - deltx**2.*V22
  M33 = deltt**2.*R33 - deltx**2.*V33
  M12 = deltt**2.*R12 - deltx**2.*V12
  M13 = deltt**2.*R13 - deltx**2.*V13
  M23 = deltt**2.*R23 - deltx**2.*V23
  deallocate(V11)
  deallocate(V22)
  deallocate(V33)
  deallocate(V12)
  deallocate(V13)
  deallocate(V23)
  allocate(L11(ns1,ns2,ns3))
  allocate(L12(ns1,ns2,ns3))
  allocate(L13(ns1,ns2,ns3))
  allocate(L22(ns1,ns2,ns3))
  allocate(L23(ns1,ns2,ns3))
  allocate(L33(ns1,ns2,ns3))
  R11 = U*U
  R22 = V*V
  R33 = W*W
  R12 = U*V
  R13 = U*W
  R23 = V*W
  call specboxfilter(R11,L11,nt)
  call specboxfilter(R12,L12,nt)
  call specboxfilter(R13,L13,nt)
  call specboxfilter(R22,L22,nt)
  call specboxfilter(R23,L23,nt)
  call specboxfilter(R33,L33,nt)
  L11 = L11 - uft*uft
  L22 = L22 - vft*vft
  L33 = L33 - wft*wft
  L12 = L12 - uft*vft
  L13 = L13 - uft*wft
  L23 = L23 - vft*wft
  deallocate(R11)
  deallocate(R22)
  deallocate(R33)
  deallocate(R12)
  deallocate(R13)
  deallocate(R23)
  deallocate(uft)
  deallocate(vft)
  deallocate(wft)
  L11 = L11*M11 + 2.*L12*M12 + 2.*L13*M13 + L22*M22 + 2.*L23*M23 + L33*M33
  M11 = M11*M11 + 2.*M12*M12 + 2.*M13*M13 + M22*M22 + 2.*M23*M23 + M33*M33
  allocate(cst1y(ns2))
  allocate(cst2y(ns2))
  allocate(coefffy(ns2))
  call moyen_y(L11,cst1y)
  call moyen_y(M11,cst2y)
  coefffy = cst1y / cst2y
  deallocate(cst1y)
  deallocate(cst2y)
  deallocate(L11)
  deallocate(L22)
  deallocate(L33)
  deallocate(L12)
  deallocate(L13)
  deallocate(L23)
  deallocate(M11)
  deallocate(M22)
  deallocate(M33)
  deallocate(M12)
  deallocate(M13)
  deallocate(M23)
  allocate(T11(ns1,ns2,ns3))
  allocate(T12(ns1,ns2,ns3))
  allocate(T13(ns1,ns2,ns3))
  allocate(T22(ns1,ns2,ns3))
  allocate(T23(ns1,ns2,ns3))
  allocate(T33(ns1,ns2,ns3))
  do j=1,ns2
     T11(:,j,:) = coefffy(j) * deltx**2.0 * normS(:,j,:) * S11(:,j,:)
     T12(:,j,:) = coefffy(j) * deltx**2.0 * normS(:,j,:) * S12(:,j,:)
     T13(:,j,:) = coefffy(j) * deltx**2.0 * normS(:,j,:) * S13(:,j,:)
     T22(:,j,:) = coefffy(j) * deltx**2.0 * normS(:,j,:) * S22(:,j,:)
     T23(:,j,:) = coefffy(j) * deltx**2.0 * normS(:,j,:) * S23(:,j,:)
     T33(:,j,:) = coefffy(j) * deltx**2.0 * normS(:,j,:) * S33(:,j,:)
  end do
  deallocate(coefffy)

  ! postprocessing
  ! Ekf
  call moyen(U**2.+V**2.+W**2.,cst1)
  Ekf = cst1/2
  ! SGS dissipation
  normS =  - ( T11*S11 + T22*S22 + T33*S33 + 2.*T12*S12 + 2.*T13*S13 + 2.*T23*S23 )
  call moyen(normS,diss_sgs)
  
  allocate(SPEC1(2,nx+1))
  allocate(SPEC11(2,nx+1))
  SPEC1 = 0.0
  SPEC11 = 0.0
!  call get_spectrum(normS,ns1,ns2,ns3,L,SPEC1,SPEC11)
!  if (rank.eq.0) then
!     open(14,file='spectrum_sgs_diss', form = 'formatted')
!     do i = 1, nx+1
!        if ((SPEC1(1,i).ne.0.0).and.(SPEC1(2,i).ne.0.0)) then
!           write(14,*) SPEC11(1,i), SPEC1(2,i)
!        end if
!     end do
!   end if
   deallocate(SPEC1)
   deallocate(SPEC11)
  ! mu dissipation
  call moyen( ( S11*S11 + S22*S22 + S33*S33 + 2.*S12*S12 + 2.*S13*S13 + 2.*S23*S23 ),diss_mu)
  diss_mu = mu_solver * diss_mu
  ! tau12 
  call moyen((T12)**2.,tau12)
  ! <uv>
  call moyen(U*V,mean_uv)
  if (rank.eq.0) then
     write(*,'(A10,10(E12.5,3X))') '',sim_time,Ekf,diss_sgs,diss_mu,tau12,mean_uv
  end if

  deallocate(S11)
  deallocate(S22)
  deallocate(S33)
  deallocate(S12)
  deallocate(S13)
  deallocate(S23)
  deallocate(normS)

  allocate(ddx(ns1,ns2,ns3))
  allocate(ddy(ns1,ns2,ns3))
  allocate(ddz(ns1,ns2,ns3))
  allocate(ddxk(fns1,fns2,fns3))
  allocate(ddyk(fns1,fns2,fns3))
  allocate(ddzk(fns1,fns2,fns3))
  ! -- x
  ddx = - T11
  ddy = - T12
  ddz = - T13
  call ftran_wrap(ddx,ddxk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddy,ddyk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddz,ddzk,ns1,ns2,ns3,fns1,fns2,fns3)
  ddxk = ddxk/real((ns1**3.0),WP)
  ddyk = ddyk/real((ns1**3.0),WP)
  ddzk = ddzk/real((ns1**3.0),WP)
  do k = 1, fns3
     do j = 1, fns2
        do i = 1, fns1
           nlx(i,j,k) = nlx(i,j,k) - ii*kx(i)*ddxk(i,j,k) - ii*ky(j)*ddyk(i,j,k) - ii*kz(k)*ddzk(i,j,k)
        end do
     end do
  end do
  ! -- y
  ddx = - T12
  ddy = - T22
  ddz = - T23
  call ftran_wrap(ddx,ddxk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddy,ddyk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddz,ddzk,ns1,ns2,ns3,fns1,fns2,fns3)
  ddxk = ddxk/real((ns1**3.0),WP)
  ddyk = ddyk/real((ns1**3.0),WP)
  ddzk = ddzk/real((ns1**3.0),WP)
  do k = 1, fns3
     do j = 1, fns2
        do i = 1, fns1
           nly(i,j,k) = nly(i,j,k) - ii*kx(i)*ddxk(i,j,k) - ii*ky(j)*ddyk(i,j,k) - ii*kz(k)*ddzk(i,j,k)
        end do
     end do
  end do
  ! -- z
  ddx = - T13
  ddy = - T23
  ddz = - T33
  call ftran_wrap(ddx,ddxk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddy,ddyk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddz,ddzk,ns1,ns2,ns3,fns1,fns2,fns3)
  ddxk = ddxk/real((ns1**3.0),WP)
  ddyk = ddyk/real((ns1**3.0),WP)
  ddzk = ddzk/real((ns1**3.0),WP)
  do k = 1, fns3
     do j = 1, fns2
        do i = 1, fns1
           nlz(i,j,k) = nlz(i,j,k) - ii*kx(i)*ddxk(i,j,k) - ii*ky(j)*ddyk(i,j,k) - ii*kz(k)*ddzk(i,j,k)
        end do
     end do
  end do

  deallocate(T11)
  deallocate(T22)
  deallocate(T33)
  deallocate(T12)
  deallocate(T13)
  deallocate(T23)
  deallocate(ddxk)
  deallocate(ddyk)
  deallocate(ddzk)
  deallocate(ddx)
  deallocate(ddy)
  deallocate(ddz)
  
end subroutine dynsmagmodel
