module parallel

  implicit none
  include 'mpif.h'

  integer :: nproc, rank

  integer :: nx,ny,nz
  integer :: nxsc,nysc,nzsc

  integer :: ns1,ns2,ns3,nk
  integer :: fns1, fns2, fns3

  integer :: ns1sc,ns2sc,ns3sc,nksc
  integer :: fns1sc, fns2sc, fns3sc

  integer :: nxs,nxe,nys,nye,nzs,nze
  integer :: fnxs,fnxe,fnys,fnye,fnzs,fnze

  integer :: nxssc,nxesc,nyssc,nyesc,nzssc,nzesc
  integer :: fnxssc,fnxesc,fnyssc,fnyesc,fnzssc,fnzesc


  integer :: ndim

  integer :: MPI_REAL_WP
  integer :: MPI_COMPLEX_WP

end module parallel


subroutine mpi_initialize

  use precision
  use parallel
  use parser
  implicit none
  
  integer :: ierr
  integer :: size_dp


  call MPI_INIT(ierr)
  call MPI_COMM_SIZE(MPI_COMM_WORLD,nproc,ierr)
  call MPI_COMM_RANK(MPI_COMM_WORLD,rank,ierr)



  call parser_read('Number of points',nx)
  ny = nx
  nz = nx

  if (mod(nz,nproc).ne.0) then
     print *,' NX should be exactly divided by the number of processors'
     stop
  endif

  nk = nx/2+1

  ns1 = nx
  ns2 = ny
  ns3 = nz/nproc

  nxs = 1
  nxe = nx

  nys = 1
  nye = ny

  nzs = 1 +(rank)*ns3
  nze = (rank+1)*ns3


  fns1 = nk
  fns2 = ny/nproc
  fns3 = nz

  fnxs = 1
  fnxe = nk

  fnys = 1 + (rank)*fns2
  fnye = (rank+1)*fns2

  fnzs = 1
  fnze = fns3

!!$
!!$   print *,' After getdims 11', rank, nxs,nxe,ns1
!!$   print *,' After getdims 12', rank, nys,nye,ns2
!!$   print *,' After getdims 13', rank, nzs,nze,ns3
!!$  
!!$
!!$ 
!!$  print *,' dimensions 31', rank, fnxs, fnxe, fns1
!!$  print *,' dimensions 32', rank, fnys, fnye, fns2
!!$  print *,' dimensions 33', rank, fnzs, fnze, fns3
!!$



  call parser_read('Number of points for scalar',nxsc)
  nysc = nxsc
  nzsc = nxsc

  if (mod(nzsc,nproc).ne.0) then
     print *,' NXSC should be exactly divided by the number of processors'
     stop
  endif

  nksc = nxsc/2+1

  ns1sc = nxsc
  ns2sc = nysc
  ns3sc = nzsc/nproc

  nxssc = 1
  nxesc = nxsc

  nyssc = 1
  nyesc = nysc

  nzssc = 1 +(rank)*ns3sc
  nzesc = (rank+1)*ns3sc

  fns1sc = nksc
  fns2sc = nysc/nproc
  fns3sc = nzsc

  fnxssc = 1
  fnxesc = nksc

  fnyssc = 1 + (rank)*fns2sc
  fnyesc = (rank+1)*fns2sc

  fnzssc = 1
  fnzesc = fns3sc



  call MPI_TYPE_SIZE(MPI_DOUBLE_PRECISION,size_dp,ierr)
  if (WP.eq.size_dp) then
     MPI_REAL_WP = MPI_DOUBLE_PRECISION
  else
     MPI_REAL_WP = MPI_REAL
  endif

  call MPI_TYPE_SIZE(MPI_DOUBLE_COMPLEX,size_dp,ierr)

  if ((2*WP).eq.size_dp) then
     MPI_COMPLEX_WP = MPI_DOUBLE_COMPLEX
  else
     MPI_COMPLEX_WP = MPI_COMPLEX
  endif

  
end subroutine mpi_initialize


subroutine parallel_max(gmax,mdummy)

  use parallel
  use precision
  implicit none

  real(WP) :: gmax, mdummy
  integer :: ierr

  call MPI_ALLREDUCE(gmax,mdummy,1,MPI_REAL_WP,MPI_MAX,&
       &MPI_COMM_WORLD,ierr)

  gmax = mdummy

end subroutine parallel_max

subroutine barrier

  use parallel

  implicit none

  integer :: ierr

  call MPI_BARRIER(MPI_COMM_WORLD,ierr)

end subroutine barrier
