subroutine postprocess7
  
  use solver
  use parallel
  use data

  implicit none
  integer :: i,j,k,pas,ns1ne,ns2ne,ns3ne
  character(len=30) :: Filename
  integer :: ierr,ik,il,jl,kl,iunit

  real(WP), dimension(:),     pointer :: xx,pdf
  real(WP), dimension(:,:,:), pointer :: convU, convV, convW
  real(WP), dimension(:,:,:), pointer :: Us1, Vs1, Ws1 , norme
  real(WP), dimension(:,:,:), pointer :: dudx, dudy, dudz
  real(WP), dimension(:,:,:), pointer :: dvdx, dvdy, dvdz
  real(WP), dimension(:,:,:), pointer :: dwdx, dwdy, dwdz
  integer :: dime

  dime = 200

  !! STEP 1 
  call solver_step
  call dns_stats
  if (rank.eq.0) print*,'STEP 1' 
  allocate(Us1(ns1,ns2,ns3))
  allocate(Vs1(ns1,ns2,ns3))
  allocate(Ws1(ns1,ns2,ns3))
  Us1 = U
  Vs1 = V
  Ws1 = W

  ! STEP 2
  if (rank.eq.0) print*,'STEP 2' 
  call solver_step
  call dns_stats
  allocate(dudx(ns1,ns2,ns3))
  allocate(dudy(ns1,ns2,ns3))
  allocate(dudz(ns1,ns2,ns3))
  allocate(dvdx(ns1,ns2,ns3))
  allocate(dvdy(ns1,ns2,ns3))
  allocate(dvdz(ns1,ns2,ns3))
  allocate(dwdx(ns1,ns2,ns3))
  allocate(dwdy(ns1,ns2,ns3))
  allocate(dwdz(ns1,ns2,ns3))

  if (rank.eq.0) print*,'comput tab' 
  call gradient(U,0.,dudx,dudy,dudz)
  call gradient(V,0.,dvdx,dvdy,dvdz)
  call gradient(W,0.,dwdx,dwdy,dwdz)

  if (rank.eq.0) print*,'comput conv term' 
  allocate(convU(ns1,ns2,ns3))
  allocate(convV(ns1,ns2,ns3))
  allocate(convW(ns1,ns2,ns3))
  convU = U*dudx + V*dudy + W*dudz
  convV = U*dvdx + V*dvdy + W*dvdz
  convW = U*dwdx + V*dwdy + W*dwdz
  deallocate(dudx)
  deallocate(dudy)
  deallocate(dudz)
  deallocate(dvdx)
  deallocate(dvdy)
  deallocate(dvdz)
  deallocate(dwdx)
  deallocate(dwdy)
  deallocate(dwdz)
 
  !! STEP 3
  if (rank.eq.0) print*,'STEP 3'
  call solver_step
  call dns_stats


  if (rank.eq.0) print*,'acceleration term, dt =', dt
  
  Us1 = (U - Us1) / (2*dt) + convU
  Vs1 = (V - Vs1) / (2*dt) + convV
  Ws1 = (W - Ws1) / (2*dt) + convW

  if (rank.eq.0) print*,'norme'
  allocate(norme(ns1,ns2,ns3))
  norme = (Us1*Us1 + Vs1*Vs1 + Ws1*Ws1)**(0.5)

  deallocate(convU)
  deallocate(convV)
  deallocate(convW)

  if (rank.eq.0) print*,'PDF'
  allocate(xx(dime))
  allocate(pdf(dime))
  write(FileName,'(A,I2.2,A)') 'pdf_U'
  call c_pdf(xx,pdf,Us1,dime)
  if(rank.eq.0) then
         open(14,file=Filename,form='formatted')
          do i=1,dime-1
           write(14,*) xx(i), pdf(i)
          enddo
         close(14)
  endif
  write(FileName,'(A,I2.2,A)') 'pdf_V'
  call c_pdf(xx,pdf,Vs1,dime)
  if(rank.eq.0) then
         open(14,file=Filename,form='formatted')
          do i=1,dime-1
           write(14,*) xx(i), pdf(i)
          enddo
         close(14)
  endif
  write(FileName,'(A,I2.2,A)') 'pdf_W'
  call c_pdf(xx,pdf,Ws1,dime)
  if(rank.eq.0) then
         open(14,file=Filename,form='formatted')
          do i=1,dime-1
           write(14,*) xx(i), pdf(i)
          enddo
         close(14)
  endif
  write(FileName,'(A,I2.2,A)') 'pdf_nor'
  call c_pdf(xx,pdf,norme,dime)
  if(rank.eq.0) then
         open(14,file=Filename,form='formatted')
          do i=1,dime-1
           write(14,*) xx(i), pdf(i)
          enddo
         close(14)
  endif
  deallocate(xx)
  deallocate(pdf)

  if (rank.eq.0) print*,'PARAVIEW'
  call dump_data(Us1,'paravie_acc_1')
  call dump_data(Vs1,'paravie_acc_2')
  call dump_data(Ws1,'paravie_acc_3')
  call dump_data(norme,'paravie_acc_n')

  if( rank.eq.0 ) print*,Us1(1,1,1)
  if( rank.eq.nproc-1 ) print*,Us1(ns1,ns2,ns3)
  call tab_write_ne(Us1,'tabaccU.out',ns1,ns2,ns3)
  if( rank.eq.0 ) print*,Vs1(1,1,1)
  if( rank.eq.nproc-1 ) print*,Vs1(ns1,ns2,ns3)
  call tab_write_ne(Vs1,'tabaccV.out',ns1,ns2,ns3)
  if( rank.eq.0 ) print*,Ws1(1,1,1)
  if( rank.eq.nproc-1 ) print*,Ws1(ns1,ns2,ns3)
  call tab_write_ne(Ws1,'tabaccW.out',ns1,ns2,ns3)
  if( rank.eq.0 ) print*,norme(1,1,1)
  if( rank.eq.nproc-1 ) print*,norme(ns1,ns2,ns3)
  call tab_write_ne(norme,'tabaccnorme.out',ns1,ns2,ns3)
  

  deallocate(norme)
  deallocate(Us1)
  deallocate(Vs1)
  deallocate(Ws1)

end subroutine postprocess7
