module solver

  use precision

  implicit none
  
  real(WP), dimension(:), allocatable :: kx
  real(WP), dimension(:), allocatable :: ky
  real(WP), dimension(:), allocatable :: kz

  real(WP), dimension(:), allocatable :: kxsc
  real(WP), dimension(:), allocatable :: kysc
  real(WP), dimension(:), allocatable :: kzsc

  integer :: deal
  real(WP) :: dk,kmax,kcut
  real(WP) :: kmaxsc,kcutsc

  real(WP) :: mu_solver, diff_solver
  real(WP) :: courant

  integer :: iforce,iscalar,iles,ilessc,ydir,ipost


  complex(WP), dimension(:,:,:), allocatable :: Uk
  complex(WP), dimension(:,:,:), allocatable :: Vk
  complex(WP), dimension(:,:,:), allocatable :: Wk
  complex(WP), dimension(:,:,:), allocatable :: Zk

  complex(WP), dimension(:,:,:),pointer :: Udummy
  complex(WP), dimension(:,:,:),pointer :: Vdummy
  complex(WP), dimension(:,:,:),pointer :: Wdummy
  complex(WP), dimension(:,:,:),pointer :: Zdummy

  complex(WP), dimension(:,:,:), allocatable :: nlx
  complex(WP), dimension(:,:,:), allocatable :: nly
  complex(WP), dimension(:,:,:), allocatable :: nlz
  complex(WP), dimension(:,:,:), allocatable :: nlmf

  complex(WP), dimension(:,:,:), allocatable :: nlu2x
  complex(WP), dimension(:,:,:), allocatable :: nlu2y
  complex(WP), dimension(:,:,:), allocatable :: nlu2z
  


  logical, parameter :: flg_inplace = .false.
  complex(WP), parameter :: ii = (0.0_WP,1.0_WP)
  real(WP), parameter :: kmin = 1e-06_WP

  real(WP) :: dt,sim_time

  integer :: impose
  real(WP) :: cx,cy,cz

end module solver


subroutine solver_init

  use parallel
  use data
  use solver
  use transform

  implicit none

  integer ::  i,j,k,il,jl,kl,gg(3),itype
  real(WP) :: pi 
  real(WP) :: umax,vmax,wmax


  allocate(kx(fns1))
  allocate(ky(fns2))
  allocate(kz(fns3))

  allocate(kxsc(fns1sc))
  allocate(kysc(fns2sc))
  allocate(kzsc(fns3sc))

  pi = acos(-1.0_WP)
  dk = 2.0_WP*pi/L

  do i = fnxs,fnxe
     il = i -fnxs+1
     kx(il) = real((i-1),WP)*dk
  enddo
  do i = fnxssc,fnxesc
     il = i -fnxssc+1
     kxsc(il) = real((i-1),WP)*dk
  enddo

  do j = fnys,fnye
     jl = j - fnys +1
     ky(jl) = real((j-1),WP)*dk
     if (j.gt.nk) ky(jl) = -real(nx+1-j,WP)*dk
  enddo
  do j = fnyssc,fnyesc
     jl = j - fnyssc +1
     kysc(jl) = real((j-1),WP)*dk
     if (j.gt.nksc) kysc(jl) = -real(nxsc+1-j,WP)*dk
  enddo

  do k = fnzs,fnze
     kl = k -fnzs+1
     kz(kl) = real((k-1),WP)*dk
     if (k.gt.nk) kz(kl) = -real(nx+1-k,WP)*dk
  enddo
  do k = fnzssc,fnzesc
     kl = k -fnzssc+1
     kzsc(kl) = real((k-1),WP)*dk
     if (k.gt.nksc) kzsc(kl) = -real(nxsc+1-k,WP)*dk
  enddo


!  print *,'First checkpoint',rank


  allocate(Uk(fns1,fns2,fns3))
  allocate(Vk(fns1,fns2,fns3))
  allocate(Wk(fns1,fns2,fns3))
  allocate(Zk(fns1sc,fns2sc,fns3sc))

  allocate(Udummy(fns1,fns2,fns3))
  allocate(Vdummy(fns1,fns2,fns3))
  allocate(Wdummy(fns1,fns2,fns3))
!!PAR!!  allocate(Zdummy(fns1sc,fns2sc,fns3sc))



  call ftran_wrap(U,Uk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(V,Vk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(W,Wk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(SC,Zk,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)


  Uk = Uk/real(nx**3.0,WP)
  Vk = Vk/real(nx**3.0,WP)
  Wk = Wk/real(nx**3.0,WP)
  Zk = Zk/real(nxsc**3.0,WP)

  call parallel_max(maxval(abs(Uk)),umax)
  call parallel_max(maxval(abs(Vk)),vmax)
  call parallel_max(maxval(abs(Wk)),wmax)

  if (rank.eq.0) then
     print *,' Maximum of U,V,W after trans',umax,vmax,wmax
  endif



!!$  call data_write_fourier(Uk,Vk,Wk,fns1,fns2,fns3)

  allocate(nlx(fns1,fns2,fns3))
  allocate(nly(fns1,fns2,fns3))
  allocate(nlz(fns1,fns2,fns3))
!!PAR!!  allocate(nlmf(fns1sc,fns2sc,fns3sc))
  allocate(nlu2x(fns1,fns2,fns3))
  allocate(nlu2y(fns1,fns2,fns3))
  allocate(nlu2z(fns1,fns2,fns3))

!!$
!!$  call parallel_max(maxval(abs(Uk)),umax)
!!$  call parallel_max(maxval(abs(Vk)),vmax)
!!$  call parallel_max(maxval(abs(Wk)),wmax)

!!$  if (rank.eq.0) then
!!$     print *,' Maximum U,V,W in solver',umax,vmax,wmax
!!$  endif
!!$
!!$  if (umax.eq.maxval(abs(Uk))) then
!!$     print *,' Maximum in rank', rank
!!$     print *,' location ', maxloc(abs(Uk))
!!$     gg = maxloc(abs(Uk))
!!$     print *,' max value for conformation', Uk(gg(1),gg(2),gg(3))
!!$  endif
!!$
!!$  if (wmax.eq.maxval(abs(Wk))) then
!!$     print *,' Maximum in rank W', rank
!!$     print *,' location W', maxloc(abs(Wk))
!!$     gg = maxloc(abs(Wk))
!!$     print *,' max value for W conformation', Wk(gg(1),gg(2),gg(3))
!!$  endif

  sim_time = 0.0_WP

  call parser_read('Dealiasing',deal,0) 

  kmax = sqrt(3.0_WP)*kx(nx/2+1) !sqrt(2.0_WP)*real(nx,WP)/3.0_WP !sqrt(3.0_WP)*real((nx/2+1),WP)
!!PAR!!  kmaxsc = sqrt(3.0_WP)*kxsc(nxsc/2+1) !sqrt(2.0_WP)*real(nx,WP)/3.0_WP !sqrt(3.0_WP)*real((nx/2+1),WP)
  if (deal.eq.1) then
     kcut = 2.0_WP/3.0_WP*kmax
!!PAR!!     kcutsc = 2.0_WP/3.0_WP*kmaxsc
  else
     kcut = 10000.0_WP
!!PAR!!     kcutsc = 10000.0_WP
  endif

  mu_solver = mu
  diff_solver = diff

  call parser_read('LES model',iles,0)
  call parser_read('LES scalar model',ilessc,0)
  call parser_read('y inhomogeneity',ydir,0)

  call parser_read('Postprocessing',ipost,0)

  call parser_read('Forcing',iforce,0)
  if (iforce.eq.1) call forcing_init

  call parser_read('Courant number', courant,0.5_WP)

!!!! Determine if it is required to solver the scalar equation
!!!! If running from scratch with forcing, dont solver scalar equation

  call parser_read('Simulation type', itype)
  call parser_read('Compute scalar', iscalar)
  if (iscalar.eq.0) then
     if (rank.eq.0) print *,'Not solving scalar transport equation .....'
  endif
  
  call parser_read('Imposing mean gradient',impose,0)
  if (impose.eq.1) then
     call parser_read('X-gradient',cx,0.0_WP)
     call parser_read('Y-gradient',cy,0.0_WP)
     call parser_read('Z-gradient',cz,0.0_WP)
  else
     cx = 0.0_WP
     cy = 0.0_WP
     cz = 0.0_WP
  endif

  spfilename = 'spectrum_solver.init'
  spfilename2 = 'spectrum_solver.init.z'
  call get_dns_numbers(0)

  print*,'test U init solver', U(1,1,1), V(1,1,1),W(1,1,1)

end subroutine solver_init



subroutine solver_step
  
  use solver
  use parallel
  use data

  implicit none

  complex(WP), dimension(:,:,:), allocatable :: Pressk
  real(WP), dimension(:,:,:), allocatable :: tab
  character(len=str_long) :: sim_name
  real(WP) :: Ufz, Vfz, Wfz, xx

  integer :: i,j,k,ierr

  real(WP) :: kk,h, divumax


  allocate(Pressk(fns1,fns2,fns3))
  allocate(tab(ns1,ns2,ns3))


!!$  if (rank.eq.0) print *,' Viscosity', mu_solver
!!!! GEL DU CHAMP DE VITESSE POUR LES TEST !!!! 
call parser_read('Simulation name', sim_name)
if (trim(sim_name).eq.'Test case') then
  call parser_read('Frozen U', Ufz)
  call parser_read('Frozen V', Vfz)
  call parser_read('Frozen W', Wfz)
  do i = 1, nx
     xx = float(i-1)*L/nx
     U(i,:,:) = Ufz + 0.5*sin(xx)
  end do
  V = Vfz
  W = Wfz
  print*,'1. U,V,W',U(1,1,1),V(1,1,1),W(1,1,1)    !!!! GEL DU CHAMP DE VITESSE POUR LES TESTS !!!! 
endif

  call get_timestep
  sim_time = sim_time +dt

!!!!!!!!!!!!
!!! Second-order RK scheme
!!!!!!!!!!!
  h = dt/2.0_WP

  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           Udummy(i,j,k) = Uk(i,j,k)
           Vdummy(i,j,k) = Vk(i,j,k)
           Wdummy(i,j,k) = Wk(i,j,k)
        enddo
     enddo
  enddo

  call non_linear

!  if (iforce.eq.1) call force_compute
!!!!!! Step 1 for RK scheme  
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           kk = kx(i)**2.0 + ky(j)**2.0 + kz(k)**2.0
           Uk(i,j,k) = (Uk(i,j,k) + h*nlx(i,j,k))/exp(mu_solver*kk*h)
           Vk(i,j,k) = (Vk(i,j,k) + h*nly(i,j,k))/exp(mu_solver*kk*h)
           Wk(i,j,k) = (Wk(i,j,k) + h*nlz(i,j,k))/exp(mu_solver*kk*h)
        enddo
     enddo
  enddo


!!$
!!$!!!!!!! Step 2 for RK scheme
!!$
  call non_linear
  if (iforce.eq.1) call force_compute

  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           kk = kx(i)**2.0 + ky(j)**2.0 + kz(k)**2.0
           Uk(i,j,k) = (Udummy(i,j,k) + dt*nlx(i,j,k)*exp(mu_solver*kk*h))/exp(mu_solver*kk*dt)
           Vk(i,j,k) = (Vdummy(i,j,k) + dt*nly(i,j,k)*exp(mu_solver*kk*h))/exp(mu_solver*kk*dt)
           Wk(i,j,k) = (Wdummy(i,j,k) + dt*nlz(i,j,k)*exp(mu_solver*kk*h))/exp(mu_solver*kk*dt)
        enddo
     enddo
  enddo

  ! Projection
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           kk = kx(i)**2.0 + ky(j)**2.0 + kz(k)**2.0
           if(kk.gt.0.0_WP) then
              Pressk(i,j,k) = - ( ii*kx(i)*Uk(i,j,k) + ii*ky(j)*Vk(i,j,k) + ii*kz(k)*Wk(i,j,k) )/kk
           end if
           Uk(i,j,k) = Uk(i,j,k) - ii*kx(i)*Pressk(i,j,k)
           Vk(i,j,k) = Vk(i,j,k) - ii*ky(j)*Pressk(i,j,k)
           Wk(i,j,k) = Wk(i,j,k) - ii*kz(k)*Pressk(i,j,k)
           Pressk(i,j,k) = - ( ii*kx(i)*Uk(i,j,k) + ii*ky(j)*Vk(i,j,k) + ii*kz(k)*Wk(i,j,k) )
         end do
      end do
  end do
!  call btran_wrap(Pressk,tab,ns1,ns2,ns3,fns1,fns2,fns3)
!  call parallel_max(maxval(abs(tab)),divumax)
!  if (rank.eq.0) print*,'divergence max = ', divumax
!  call parallel_max(maxval(abs(Uk)),divumax)
!  if (rank.eq.0) print*,'abs(Uk) max = ', divumax

!!PAR!! - INTERFACAGE AVEC LA METHODE PARTICULAIRE

  call ftran_wrap(SC,Zk,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
  Zk = Zk/real(nxsc**3.0,WP)
  if (iscalar.ne.0) then
   do k = 1,fns3sc
     do j = 1,fns2sc
        do i = 1,fns1sc
           kk = kxsc(i)**2.0 + kysc(j)**2.0 + kzsc(k)**2.0
           Zk(i,j,k) = Zk(i,j,k)/exp(diff_solver*kk*dt)
        enddo
     enddo
   enddo
  end if
  call btran_wrap(Zk,SC,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
  if (nproc.ne.1) then
     if (rank.eq.0) print*,'NOT AVAILABLE IN PARALLEL'
     STOP
  else 
    
!!PAR!! 1. on remet les vitesses dans l'espace physique
  if (trim(sim_name).eq.'Test case') then
    call parser_read('Frozen U', Ufz)
    call parser_read('Frozen V', Vfz)
    call parser_read('Frozen W', Wfz)
    do i = 1, nx
       xx = float(i-1)*L/nx  
       U(i,:,:) = Ufz+0.5*sin(xx)
    end do
    V = Vfz
    W = Wfz
    print*,'1b. U,V,W',U(1,1,1),V(1,1,1),W(1,1,1)    !!!! GEL DU CHAMP DE VITESSE POUR LES TESTS !!!! 
  endif
  if (trim(sim_name).ne.'Test case') call u_compute    !!!! GEL DU CHAMP DE VITESSE POUR LES TESTS !!!! 
  if (trim(sim_name).eq.'Test case') print*,'2. U,V,W',U(1,1,1),V(1,1,1),W(1,1,1)    !!!! GEL DU CHAMP DE VITESSE POUR LES TESTS !!!! 

!!PAR!! 2. appelle du transport particulaire
  call x_advec
  call y_advec
  call z_advec

  end if
!!PAR!! - FIN - INTERFACAGE AVEC LA METHODE PARTICULAIRE

  

  deallocate(Pressk)
  deallocate(tab)


end subroutine solver_step
  
subroutine non_linear

  use solver
  use parallel
  use data
  
  implicit none
  
  real(WP), dimension(:,:,:),allocatable :: ddx,ddy,ddz,nis
  real(WP), dimension(:,:,:),allocatable :: us,vs,ws
  real(WP), dimension(:,:,:),allocatable :: uss,vss,wss
  complex(WP), dimension(:,:,:),allocatable :: ddxk,ddyk,ddzk
  ! LES

  complex(WP) :: knl
  real(WP) :: kk

  integer :: i,j,k
  real(WP) :: umax,vmax,wmax

  integer ::   sizemess, tag, ierr, status(MPI_STATUS_SIZE)
  integer ::   yeloc, ysloc, yelocsc, yslocsc, rank2send1, rank2send2, fs1, fe1, fs2, fe2, fnysrank, fnyerank, cptrank

  

  allocate(ddx(ns1,ns2,ns3))
  allocate(ddy(ns1,ns2,ns3))
  allocate(ddz(ns1,ns2,ns3))
  allocate(nis(ns1,ns2,ns3))

  allocate(ddxk(fns1,fns2,fns3))
  allocate(ddyk(fns1,fns2,fns3))
  allocate(ddzk(fns1,fns2,fns3))

 
  allocate(us(ns1,ns2,ns3))
  allocate(vs(ns1,ns2,ns3))
  allocate(ws(ns1,ns2,ns3))

!!!!!!!!!!!
!! Convert velocity to spatial coordinates
!!!!!!!!!!

  call btran_wrap(Uk,us,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(Vk,vs,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(Wk,ws,ns1,ns2,ns3,fns1,fns2,fns3)

!!$  call parallel_max(maxval(abs(us)),umax)
!!$  call parallel_max(maxval(abs(vs)),vmax)
!!$  call parallel_max(maxval(abs(ws)),wmax)
!!$  
!!$!  if (rank.eq.0) write(*,'(A10,3(E12.5,3X))') ' MAX VAL', umax,vmax,wmax


!!$   if (rank.eq.0) print *,' After velocity conversion'


!!!!!!!!!!
!! Form the first term in the skew-symmetric form
!!!!!!!!!


!!!! U component

  do k = 1,ns3
     do j = 1,ns2
        do i = 1,ns1
           ddx(i,j,k) = us(i,j,k)*us(i,j,k)
           ddy(i,j,k) = vs(i,j,k)*us(i,j,k)
           ddz(i,j,k) = ws(i,j,k)*us(i,j,k)
        enddo
     enddo
  enddo

  call ftran_wrap(ddx,ddxk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddy,ddyk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddz,ddzk,ns1,ns2,ns3,fns1,fns2,fns3)

  ddxk = ddxk/real(ns1**3.0,WP)
  ddyk = ddyk/real(ns1**3.0,WP)
  ddzk = ddzk/real(ns1**3.0,WP)
  
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           nlu2x(i,j,k) = ii*(kx(i)*ddxk(i,j,k) + &
                &ky(j)*ddyk(i,j,k) + kz(k)*ddzk(i,j,k))
        enddo
     enddo
  enddo

!!!! V component

  do k = 1,ns3
     do j = 1,ns2
        do i = 1,ns1
           ddx(i,j,k) = us(i,j,k)*vs(i,j,k)
           ddy(i,j,k) = vs(i,j,k)*vs(i,j,k)
           ddz(i,j,k) = ws(i,j,k)*vs(i,j,k)
        enddo
     enddo
  enddo

  call ftran_wrap(ddx,ddxk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddy,ddyk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddz,ddzk,ns1,ns2,ns3,fns1,fns2,fns3)

  ddxk = ddxk/real(ns1**3.0,WP)
  ddyk = ddyk/real(ns1**3.0,WP)
  ddzk = ddzk/real(ns1**3.0,WP)

  
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           nlu2y(i,j,k) = ii*(kx(i)*ddxk(i,j,k) + &
                &ky(j)*ddyk(i,j,k) + kz(k)*ddzk(i,j,k))
        enddo
     enddo
  enddo

!!!! W component

  do k = 1,ns3
     do j = 1,ns2
        do i = 1,ns1
           ddx(i,j,k) = us(i,j,k)*ws(i,j,k)
           ddy(i,j,k) = vs(i,j,k)*ws(i,j,k)
           ddz(i,j,k) = ws(i,j,k)*ws(i,j,k)
        enddo
     enddo
  enddo

  call ftran_wrap(ddx,ddxk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddy,ddyk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(ddz,ddzk,ns1,ns2,ns3,fns1,fns2,fns3)

  ddxk = ddxk/real(ns1**3.0,WP)
  ddyk = ddyk/real(ns1**3.0,WP)
  ddzk = ddzk/real(ns1**3.0,WP)

  
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           nlu2z(i,j,k) = ii*(kx(i)*ddxk(i,j,k) + &
                &ky(j)*ddyk(i,j,k) + kz(k)*ddzk(i,j,k))
        enddo
     enddo
  enddo


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!! Compute the second term in the skew symmetric form
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!   

!!!!!!!!!!!!!!
!! Compute U products
!!!!!!!!!!!!!!  
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           ddxk(i,j,k) = ii*kx(i)*Uk(i,j,k)
           ddyk(i,j,k) = ii*ky(j)*Uk(i,j,k)
           ddzk(i,j,k) = ii*kz(k)*Uk(i,j,k)
        enddo
     enddo
  enddo

  ddx = 0.0_WP
  ddy = 0.0_WP
  ddz = 0.0_WP


  call btran_wrap(ddxk,ddx,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(ddyk,ddy,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(ddzk,ddz,ns1,ns2,ns3,fns1,fns2,fns3)

!!$  if (rank.eq.0) print *,' After btran in U computations'

  do k = 1,ns3
     do j = 1,ns2
        do i = 1,ns1
           nis(i,j,k) = us(i,j,k)*ddx(i,j,k) + &
                & vs(i,j,k)*ddy(i,j,k) +&
                & ws(i,j,k)*ddz(i,j,k)
        enddo
     enddo
  enddo

!!$  if (rank.eq.0) print *,' After product evaluations'

  call ftran_wrap(nis,nlx,ns1,ns2,ns3,fns1,fns2,fns3)
  
!!$  if (rank.eq.0) print *,' After forward transforms'

  nlx = nlx/real((ns1**3.0),WP)

!!$  if (rank.eq.0) print *,' After nlx computation'

!!!!!!!!!!!!!!
!! Compute U products
!!!!!!!!!!!!!!  
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           ddxk(i,j,k) = ii*kx(i)*Vk(i,j,k)
           ddyk(i,j,k) = ii*ky(j)*Vk(i,j,k)
           ddzk(i,j,k) = ii*kz(k)*Vk(i,j,k)
        enddo
     enddo
  enddo

  ddx = 0.0_WP
  ddy = 0.0_WP
  ddz = 0.0_WP


  call btran_wrap(ddxk,ddx,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(ddyk,ddy,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(ddzk,ddz,ns1,ns2,ns3,fns1,fns2,fns3)

!!$  if (rank.eq.0) print *,' After b trans in V compute'

  do k = 1,ns3
     do j = 1,ns2
        do i = 1,ns1
           nis(i,j,k) = us(i,j,k)*ddx(i,j,k) + &
                & vs(i,j,k)*ddy(i,j,k) +&
                & ws(i,j,k)*ddz(i,j,k)
        enddo
     enddo
  enddo

!!$  if (rank.eq.0) print *,' After product eval. in V'

  call ftran_wrap(nis,nly,ns1,ns2,ns3,fns1,fns2,fns3)
  
!!$  if (rank.eq.0) print *,' After forward trans in V'
  
  nly = nly/real((ns1**3.0),WP)
  
!!$  if (rank.eq.0) print *,' After nly computation'

!!!!!!!!!!!!!!
!! Compute W products
!!!!!!!!!!!!!!  
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           ddxk(i,j,k) = ii*kx(i)*Wk(i,j,k)
           ddyk(i,j,k) = ii*ky(j)*Wk(i,j,k)
           ddzk(i,j,k) = ii*kz(k)*Wk(i,j,k)
        enddo
     enddo
  enddo

  ddx = 0.0_WP
  ddy = 0.0_WP
  ddz = 0.0_WP

  call btran_wrap(ddxk,ddx,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(ddyk,ddy,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(ddzk,ddz,ns1,ns2,ns3,fns1,fns2,fns3)

  do k = 1,ns3
     do j = 1,ns2
        do i = 1,ns1
           nis(i,j,k) = us(i,j,k)*ddx(i,j,k) + &
                & vs(i,j,k)*ddy(i,j,k) +&
                & ws(i,j,k)*ddz(i,j,k)
        enddo
     enddo
  enddo

  call ftran_wrap(nis,nlz,ns1,ns2,ns3,fns1,fns2,fns3)
  
  nlz = nlz/real((ns1**3.0),WP)

!!$  if (rank.eq.0) print *,' After nlz computation'

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!! Form the complex skew-symmetric form
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!$
  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           nlx(i,j,k) = 0.5*(nlx(i,j,k) + nlu2x(i,j,k))
           nly(i,j,k) = 0.5*(nly(i,j,k) + nlu2y(i,j,k))
           nlz(i,j,k) = 0.5*(nlz(i,j,k) + nlu2z(i,j,k))
        enddo
     enddo
  enddo




!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!! Form non-linear term for the scalar
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
  deallocate(ddx)
  deallocate(ddy)
  deallocate(ddz)
  deallocate(ddxk)
  deallocate(ddyk)
  deallocate(ddzk)
  deallocate(nis)


!!PAR!! if (iscalar.eq.0) then

     deallocate(us)
     deallocate(vs)
     deallocate(ws)
!!PAR!!     nlmf = 0.0_WP

!!PAR!! else

!!PAR!!  allocate(ddx(ns1sc,ns2sc,ns3sc))
!!PAR!!  allocate(ddy(ns1sc,ns2sc,ns3sc))
!!PAR!!  allocate(ddz(ns1sc,ns2sc,ns3sc))
!!PAR!!  allocate(nis(ns1sc,ns2sc,ns3sc))

!!PAR!!  allocate(ddxk(fns1sc,fns2sc,fns3sc))
!!PAR!!  allocate(ddyk(fns1sc,fns2sc,fns3sc))
!!PAR!!  allocate(ddzk(fns1sc,fns2sc,fns3sc))

!!PAR!!     do k = 1,fns3sc
!!PAR!!        do j = 1,fns2sc
!!PAR!!           do i = 1,fns1sc
!!PAR!!              ddxk(i,j,k) = ii*kxsc(i)*Zk(i,j,k)
!!PAR!!              ddyk(i,j,k) = ii*kysc(j)*Zk(i,j,k)
!!PAR!!              ddzk(i,j,k) = ii*kzsc(k)*Zk(i,j,k)
!!PAR!!           enddo
!!PAR!!        enddo
!!PAR!!     enddo
     
!!PAR!!     ddx = 0.0_WP
!!PAR!!     ddy = 0.0_WP
!!PAR!!     ddz = 0.0_WP
     
!!PAR!!     call btran_wrap(ddxk,ddx,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
!!PAR!!     call btran_wrap(ddyk,ddy,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
!!PAR!!     call btran_wrap(ddzk,ddz,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
    
!!PAR!!     deallocate(ddxk)
!!PAR!!     deallocate(ddyk)
!!PAR!!     deallocate(ddzk)

!!PAR!!     allocate(uss(ns1sc,ns2sc,ns3sc))
!!PAR!!     allocate(vss(ns1sc,ns2sc,ns3sc))
!!PAR!!     allocate(wss(ns1sc,ns2sc,ns3sc))
!!PAR!!     uss = 0.0
!!PAR!!     vss = 0.0
!!PAR!!     wss = 0.0

!!PAR!!     if (ns1sc.gt.ns1) then
!!PAR!!        call discretisation2(us,uss,ns1,ns2,ns3,ns1sc,ns2sc,ns3sc)
!!PAR!!        call discretisation2(vs,vss,ns1,ns2,ns3,ns1sc,ns2sc,ns3sc)
!!PAR!!        call discretisation2(ws,wss,ns1,ns2,ns3,ns1sc,ns2sc,ns3sc)
!!PAR!!     else if (ns1sc.lt.ns1) then
!!PAR!!        call discretisation3(us,uss,ns1,ns2,ns3,ns1sc,ns2sc,ns3sc)
!!PAR!!        call discretisation3(vs,vss,ns1,ns2,ns3,ns1sc,ns2sc,ns3sc)
!!PAR!!        call discretisation3(ws,wss,ns1,ns2,ns3,ns1sc,ns2sc,ns3sc)
!!PAR!!     else
!!PAR!!        uss = us
!!PAR!!        vss = vs
!!PAR!!        wss = ws
!!PAR!!     end if

!!PAR!!     deallocate(us)
!!PAR!!     deallocate(vs)
!!PAR!!     deallocate(ws)
  
!!PAR!!     do k = 1,ns3sc
!!PAR!!        do j = 1,ns2sc
!!PAR!!           do i = 1,ns1sc
!!PAR!!              nis(i,j,k) = uss(i,j,k)*(ddx(i,j,k)+cx) + &
!!PAR!!                   & vss(i,j,k)*(ddy(i,j,k)+cy) +&
!!PAR!!                   & wss(i,j,k)*(ddz(i,j,k)+cz)
!!PAR!!           enddo
!!PAR!!        enddo
!!PAR!!     enddo
!!PAR!!    deallocate(uss)
!!PAR!!    deallocate(vss)
!!PAR!!    deallocate(wss)
!!PAR!!    deallocate(ddx)
!!PAR!!    deallocate(ddy)
!!PAR!!    deallocate(ddz)

!!PAR!!     call ftran_wrap(nis,nlmf,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
!!PAR!!     nlmf = nlmf/real((ns1sc**3.0),WP)

!!PAR!!     deallocate(nis)

     
!!PAR!!  endif




!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!! Dealias  the non-linear term
     
     call dealias

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!! LES MODEL 

if (iles.ne.0) then
!    if (iles.eq.1) call smagmodel
    if (iles.eq.2) call dynsmagmodel
!    if (iles.eq.8) call SISM
!    if (iles.eq.12) call dynsmagmodel2
!    if (iles.eq.13) call dynsmagmodel3
endif
if (ilessc.ne.0) then
!    if (ilessc.eq.1) call smagmodelsca
!    if (ilessc.eq.2) call dynsmagmodelsca
!    if (ilessc.eq.3) call dynsmagmodelscanew
!    if (ilessc.eq.4) call dynsmagmodelscacut
!    if (ilessc.eq.5) call dynsmagmodelscanewcut
!    if (ilessc.eq.6) call dynsfmodelscacut
!    if (ilessc.eq.7) call dynsfmodelscanewcut
!    if (ilessc.eq.8) call dynsigmamodelscacut
!    if (ilessc.eq.9) call dynsigmamodelscanewcut
!    if (ilessc.eq.12) call dyndd2modelscacut
!    if (ilessc.eq.13) call dyndd4modelscacut
!    if (ilessc.eq.14) call dynddsmagmodelscanewcut
!    if (ilessc.eq.15) call dynsflpmodelscacut
!    if (ilessc.eq.16) call ssmodelscacut
!    if (ilessc.eq.17) call dynsmagtestmodelscacut 
endif

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


!!!!!!!!!! 
!! Form the pressure term and add it to the nonlinear term
!!!!!!!!!!

!! SEMBLE NE PAS DONNER DIV(U)=0. ???
!!
!!  do k = 1,fns3
!!     do j = 1,fns2
!!        do i = 1,fns1
!!           
!!           kk = kx(i)**2.0 + ky(j)**2.0 + kz(k)**2.0
!!           if (sqrt(kk).gt.kmin) then
!!              knl = kx(i)*nlx(i,j,k) + ky(j)*nly(i,j,k) +&
!!                   & kz(k)*nlz(i,j,k)
!!              nlx(i,j,k) = kx(i)*knl/kk - nlx(i,j,k)
!!              nly(i,j,k) = ky(j)*knl/kk - nly(i,j,k)
!!              nlz(i,j,k) = kz(k)*knl/kk - nlz(i,j,k)
!!              nlmf(i,j,k) = -nlmf(i,j,k)
!!           else
!!              nlx(i,j,k) = 0.0_WP 
!!              nly(i,j,k) = 0.0_WP
!!              nlz(i,j,k) = 0.0_WP
!!              nlmf(i,j,k) = 0.0_WP
!!!!!!!!!!!!!!!INSTEAD
    nlx = -nlx
    nly = -nly
    nlz = -nlz
!!PAR!!    nlmf = -nlmf
!!           endif
!!        enddo
!!     enddo
!!  enddo


end subroutine non_linear


subroutine get_timestep

  use solver
  use parallel
  use data

  real(WP) :: max_s, max_sg
  character(len=str_long) :: sim_name

  integer :: ierr,nn

  max_s = 0.0_WP
  max_sg = 0.0_WP

  call parser_read('Simulation name', sim_name)
  if (trim(sim_name).ne.'Test case') call u_compute    !!!! GEL DU CHAMP DE VITESSE POUR LES TESTS !!!!

  max_s = maxval(abs(U) +abs(V)+abs(W))
 
 
  call MPI_ALLREDUCE(max_s,max_sg,1,MPI_REAL_WP,MPI_MAX,&
       &MPI_COMM_WORLD,ierr)

  call parser_read('fixed time step',dt)

  if (dt.eq.0) then 
     if (max_sg.gt.0.0_WP) then
        nn = nx
!!PAR!!        if(iscalar.ne.0) nn = max(nx,nxsc)
        dt = courant/max_sg*(L/nn)
     else
        print *,' SOMETHING WRONG IN TIME_STEP', rank, max_sg,max_s,nx
        stop
     endif
  endif


!!$  print *,' MAX_TIME ', dt

end subroutine get_timestep


subroutine dealias

  use solver
  use parallel
  use precision

  implicit none

  real(WP) :: kk
  integer :: i,j,k

  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           
           kk = sqrt(kx(i)**2.0_WP + ky(j)**2.0_WP + kz(k)**2.0_WP)
 !          kk = sqrt(kk)
!           if ((abs(kz(k)).gt.kmax).or.(abs(ky(j)).gt.kmax).or.(abs(kx(i)).gt.kmax)) then
!           if (kk.gt.kmax) then
           if (kk.gt.kcut) then
              nlx(i,j,k)  = 0.0_WP
              nly(i,j,k)  = 0.0_WP
              nlz(i,j,k)  = 0.0_WP
           endif
        enddo
     enddo
  enddo
!!PAR!!  do k = 1,fns3sc
!!PAR!!     do j = 1,fns2sc
!!PAR!!        do i = 1,fns1sc
!!PAR!!           kk = sqrt(kxsc(i)**2.0_WP + kysc(j)**2.0_WP + kzsc(k)**2.0_WP)
!!PAR!!           if (kk.gt.kcutsc) then
!!PAR!!              nlmf(i,j,k) = 0.0_WP
!!PAR!!           endif
!!PAR!!        enddo
!!PAR!!     enddo
!!PAR!!  enddo

end subroutine dealias

subroutine fourier_write
  use solver

!  call data_write_fourier(Uk,Vk,Wk,fns1,fns2,fns3)

end subroutine fourier_write


