subroutine z_advec

  use x_advec_m

  implicit none

  real(WP) :: cfl, dx, yy, zz
  integer :: i,j,k,np,n, ir, jr

 
  dx = L/nxsc 
  cfl = dt/dx 

   do j=1,nxsc
      zz=xmin+float(k-1)*dx
      do i=1,nxsc
         np=0
	 yy=xmin+float(i-1)*dx
         do k=1,nxsc
            if (abs(SC(i,j,k)).gt.circlim) then
               np=np+1
!               up(np)=SC(i,j,k)
               xp(np)=xmin+float(k-1)*dx
               up_aux(np)=SC(i,j,k)
               xp_aux(np)=xmin+float(k-1)*dx
            endif
         enddo
         if (np.ne.0) then
            call velox_z(np,i,j)   
            do n=1,np
               xp0(n)=xp(n)
               xp(n)=xp(n)+0.5*dt*vx(n)
               if (xp(n).gt.xmax) xp(n)=xp(n)-xmax+xmin
               if (xp(n).lt.xmin) xp(n)=xp(n)+xmax-xmin
            enddo
            call velox_z(np,i,j)
            do n=1,np
               xp_aux(n)=xp0(n)
               vx_aux(n)=vx(n)
               itype(n)=1
            enddo
            do n=1,np
               xp_aux(n)=xp_aux(n)+dt*vx_aux(n)
	       if (xp_aux(n).gt.xmax) xp_aux(n)=xp_aux(n)-xmax+xmin
	       if (xp_aux(n).lt.xmin) xp_aux(n)=xp_aux(n)+xmax-xmin
            enddo
            ir=i
            jr=j
            call remeshz(np,ir,jr)   
         endif
      enddo
   enddo

end subroutine z_advec



subroutine velox_z(np,j,k)

  use x_advec_m

  implicit none

  integer :: np, j, k, i, nx3, ny3, nz3, jp1, jp2, kp1, kp2, ip1, ip2
  real(WP) :: dx3, dy3, dz3, dxinv, yy, zz, x0, y0, z0, dx2
  real(WP) :: yy1, yy2, zz1, zz2, b1, b2, c1, c2, a1, a2
  real(WP) :: x, xx1, xx2

  do  i=1,np
        vx(i)=0.
  end do     

  nx3=nx
  ny3=nx3   
  nz3=nx3
  dx3=xmax/float(nx3)
  dy3=dx3
  dz3=dx3

  dxinv=1./dx3

  x0=xmin
  y0=ymin
  z0=zmin

  dx2 = L/nxsc

  yy=+float(j-1)*dx2 
  zz=+float(k-1)*dx2

  jp1 = int((yy)/dx3)
  jp2 = jp1 + 1
  kp1 = int((zz)/dx3)
  kp2 = kp1 + 1
  yy1 = (yy-float(jp1)*dx3)/dx3
  yy2 = 1.0 - yy1  
  zz1 = (zz-float(kp1)*dx3)/dx3
  zz2 = 1.0 - zz1
  b1=yy2
  b2=yy1
  c1=zz2
  c2=zz1

  jp1=mod(jp1+nx3,nx3) +1
  jp2=mod(jp2+nx3,nx3) +1
  kp1=mod(kp1+nx3,nx3) +1
  kp2=mod(kp2+nx3,nx3) +1

  do i = 1,np

     x = xp(i)

     ip1 = int((x-x0)*dxinv)
     ip2 = ip1 + 1

     xx1 = (x - float(ip1)*dx3-x0)*dxinv
     xx2=1-xx1

     ip1=mod(ip1+nx3,nx3) +1
     ip2=mod(ip2+nx3,nx3) +1

     a1 = xx2
     a2 = xx1
    
     vx(i)= vx(i) +  W(jp1,kp1,ip1)*a1*b1*c1
     vx(i)= vx(i) +  W(jp1,kp1,ip2)*a2*b1*c1
     vx(i)= vx(i) +  W(jp2,kp1,ip1)*a1*b2*c1
     vx(i)= vx(i) +  W(jp2,kp1,ip2)*a2*b2*c1
     vx(i)= vx(i) +  W(jp1,kp2,ip1)*a1*b1*c2
     vx(i)= vx(i) +  W(jp1,kp2,ip2)*a2*b1*c2
     vx(i)= vx(i) +  W(jp2,kp2,ip1)*a1*b2*c2
     vx(i)= vx(i) +  W(jp2,kp2,ip2)*a2*b2*c2

  end do

end subroutine velox_z

subroutine remeshz(np1,jj,kk)

  use x_advec_m

  implicit none

  integer :: np1, jj, kk
  integer :: i, nx2, n, ip0, ip1, ip2, k, j
  real(WP) :: dxinv, dx2, x0, g1, x, xx0, xx1, xx2, a0, a1, a2

  nx2 = nxsc
  dx2 = L/nx2

  dxinv=1./dx2

  do i=1,nx2
     SC(jj,kk,i)=0.
  enddo

  x0=xmin

  do n = 1,np1
     g1 = up_aux(n)
     x = xp_aux(n)

     if (itype(n).eq.0) then
        ip1 = int((x-x0)*dxinv)
        ip0 = ip1 - 1
        ip2 = ip1 + 1

        xx1 = (x - float(ip1)*dx2-x0)*dxinv
        xx0=xx1+1
        xx2=1-xx1

        ip1=mod(ip1+nx2,nx2) +1
        ip0=mod(ip0+nx2,nx2) +1
        ip2=mod(ip2+nx2,nx2) +1

        a0=-0.5*xx1*xx2
        a1=1.-xx1**2
        a2=0.5*xx0*xx1

        SC(jj,kk,ip0) = SC(jj,kk,ip0) + g1*a0
        SC(jj,kk,ip1) = SC(jj,kk,ip1) + g1*a1
        SC(jj,kk,ip2) = SC(jj,kk,ip2) + g1*a2

    else

        ip1 = nint((x-x0)*dxinv)
        ip0 = ip1 - 1
        ip2 = ip1 + 1

        xx1 = (x - float(ip1)*dx2-x0)*dxinv
        xx0=xx1+1
        xx2=1-xx1

        ip1=mod(ip1+nx2,nx2) +1
        ip0=mod(ip0+nx2,nx2) +1
        ip2=mod(ip2+nx2,nx2) +1

        a0=-0.5*xx1*xx2
        a1=1.-xx1**2
        a2=0.5*xx0*xx1

        SC(jj,kk,ip0) = SC(jj,kk,ip0) + g1*a0
        SC(jj,kk,ip1) = SC(jj,kk,ip1) + g1*a1
        SC(jj,kk,ip2) = SC(jj,kk,ip2) + g1*a2

    end if
 end do

!!! go to 222 !! -> ca veut dire quoi goto ?
!!! do k=3,nx2-2
!!!    do j=1,nx2
!!!       do i=1,nx2
!!!          if (SC(i,j,k).gt.1.2) then
!!!              print*,'BOUM', i,j,k,SC(i,j,k)
!!!              goto 222
!!!          end if
!!!       end do
!!!    end do
!!! end do

end subroutine remeshz
