#!/usr/bin/python
import time
import parmepy as pp
import parmepy.f2py
import numpy as np
import mpi4py.MPI as MPI
import math as m
from parmepy.constants import PARMES_REAL, ORDER
from parmepy.domain.obstacle.obstacle_3D import Obstacle3D
from parmepy.operator.advection import Advection
from parmepy.operator.stretching import Stretching
from parmepy.operator.poisson import Poisson
from parmepy.operator.diffusion import Diffusion
from parmepy.operator.penalization import Penalization
from parmepy.operator.redistribute import Redistribute
from parmepy.problem.navier_stokes import NSProblem
from parmepy.operator.monitors.energy_enstrophy import Energy_enstrophy
from parmepy.operator.monitors.printer import Printer
from parmepy.operator.monitors.compute_forces import Compute_forces

#from numpy import linalg as LA

pi = m.pi

rank = MPI.COMM_WORLD.Get_rank()
print "Mpi process number ", rank

## ----------- A 3d problem -----------
print " ========= Start test for penalized Navier-Stokes 3D ========="

## Opening/reading input data file
inputData = open("input.dat", "r")

Ox = np.float64(inputData.readline().rstrip('\n\r'))
Oy = np.float64(inputData.readline().rstrip('\n\r'))
Oz = np.float64(inputData.readline().rstrip('\n\r'))
Lx = np.float64(inputData.readline().rstrip('\n\r'))
Ly = np.float64(inputData.readline().rstrip('\n\r'))
Lz = np.float64(inputData.readline().rstrip('\n\r'))
resX = np.uint32(inputData.readline().rstrip('\n\r'))
resY = np.uint32(inputData.readline().rstrip('\n\r'))
resZ = np.uint32(inputData.readline().rstrip('\n\r'))
timeStep = np.float64(inputData.readline().rstrip('\n\r'))
finalTime = np.float64(inputData.readline().rstrip('\n\r'))
outModuloPrinter = np.uint32(inputData.readline().rstrip('\n\r'))
outputPrinter = inputData.readline().rstrip('\n\r')
outModuloForces = np.uint32(inputData.readline().rstrip('\n\r'))
outputForces = inputData.readline().rstrip('\n\r')
outModuloEnergies = np.uint32(inputData.readline().rstrip('\n\r'))
outputEnergies = inputData.readline().rstrip('\n\r')
visco = np.float64(inputData.readline().rstrip('\n\r'))
vortProj = np.uint32(inputData.readline().rstrip('\n\r'))
methodAdv = inputData.readline().rstrip('\n\r')
methodStretch = inputData.readline().rstrip('\n\r')
methodPenal = inputData.readline().rstrip('\n\r')
methodForces = inputData.readline().rstrip('\n\r')
uinf = np.float64(inputData.readline().rstrip('\n\r'))
obstName = inputData.readline().rstrip('\n\r')
lambdFluid = np.float64(inputData.readline().rstrip('\n\r'))
lambdPorous = np.float64(inputData.readline().rstrip('\n\r'))
lambdSolid = np.float64(inputData.readline().rstrip('\n\r'))
obstRadius = np.float64(inputData.readline().rstrip('\n\r'))
obstOx = np.float64(inputData.readline().rstrip('\n\r'))
obstOy = np.float64(inputData.readline().rstrip('\n\r'))
obstOz = np.float64(inputData.readline().rstrip('\n\r'))
obstOrientation = inputData.readline().rstrip('\n\r')
thicknPorous = np.float64(inputData.readline().rstrip('\n\r'))
thicknZbound = np.float64(inputData.readline().rstrip('\n\r'))

inputData.close()

# Parameters
dim = 3
nbElem = [resX, resY, resZ]

t0 = time.time()

## Domain
box = pp.Box(dim, length=[1., 1., 1.], origin=[0., 0., 0.])

## Obstacle
lambd = np.array([lambdSolid, lambdPorous], 
                 dtype=PARMES_REAL, order=ORDER)
sphere = Obstacle3D(box,
                    center=[obstOx, obstOy, obstOz],
                    zlayer=thicknZbound,
                    porousLayerThickn=thicknPorous,
                    porousLayerConfig='',
                    obstacleName=obstName,
                    radius=obstRadius)

## Fields declaration
def computeVel(x, y, z):
#    vx = 2. / np.sqrt(3) * np.sin(2. * pi / 3.) * np.sin(x) \
#        * np.cos(y) * np.cos(z)
#    vy = 2. / np.sqrt(3) * np.sin(-2. * pi / 3.) * np.cos(x) \
#        * np.sin(y) * np.cos(z)
#    vz = 0.
#----------- flow past sphere ----------
    vx = 0.
    module = (x - obstOx) ** 2 + (y - obstOy) ** 2 + (z - obstOz) ** 2
    if (module >= obstRadius ** 2):
        vx = uinf * (1 - (obstRadius ** 2 / module))
    vy = 0.
    vz = 0.
#    # --------Taylor Green------------
#    vx = np.sin(x) * np.cos(y) * np.cos(z)
#    vy = - np.cos(x) * np.sin(y) * np.cos(z)
#    vz = 0.
#-----------------
#    vx = 0.
#    vy = 0.
#    vz = 0.
    return vx, vy, vz


def computeVort(x, y, z):
#    wx = - np.cos(x) * np.sin(y) * np.sin(z)
#    wy = - np.sin(x) * np.cos(y) * np.sin(z)
#    wz = 2. * np.sin(x) * np.sin(y) * np.cos(z)
#----------- flow past sphere ----------
    wx = 0.
    wy = 0.
    wz = 0.
#    module = (x - obstOx) ** 2 + (y - obstOy) ** 2 + (z - obstOz) ** 2
#    if (module >= obstRadius ** 2):
#        wy = (2.0 * uinf * obstRadius ** 2 * (z - obstOz)) / module ** 2
#        wz = -(2.0 * uinf * obstRadius ** 2 * (y - obstOy)) / module ** 2
#     # --------Taylor Green------------
#    wx = - np.cos(x) * np.sin(y) * np.sin(z)
#    wy = - np.sin(x) * np.cos(y) * np.sin(z)
#    wz = 2. * np.sin(x) * np.sin(y) * np.cos(z)
#-----------------
#    xc = 6./2.
#    yc = 6./2.
#    zc = 6./6.
#    R = 1.5
#    sigma = R / 3.
#    Gamma = 0.0075
#    dist = m.sqrt((x-xc)**2 + (y-yc)**2)
#    s2 = (z - zc)**2 + (dist - R)**2
#    wx = 0.
#    wy = 0.
#    wz = 0.
#    if (dist != 0.):
#        cosTheta = (x-xc) / dist
#        sinTheta = (y-yc) / dist
#        wTheta = Gamma / (pi * sigma**2) * m.exp(-(s2 / sigma**2))
#        wx = - wTheta * sinTheta
#        wy = wTheta * cosTheta
#        wz = 0.
    return wx, wy, wz

def computeVortNorm(x, y, z):
    norm = 0.
#----------------
#    xc = 6./2.
#    yc = 6./2.
#    zc = 6./6.
#    R = 1.5
#    sigma = R / 3.
#    Gamma = 0.0075
#    dist = m.sqrt((x-xc)**2 + (y-yc)**2)
#    s2 = (z - zc)**2 + (dist - R)**2
#    norm = 0.
#    if (dist != 0.):
#        cosTheta = (x-xc) / dist
#        sinTheta = (y-yc) / dist
#        wTheta = Gamma / (pi * sigma**2) * m.exp(-(s2 / sigma**2))
#        wx = - wTheta * sinTheta
#        wy = wTheta * cosTheta
#        wz = 0.
#        norm = np.sqrt(wx ** 2 + wy ** 2 + wz ** 2)
    return norm

#def computeDensity(x, y, z):
#    if (x>=0):
#        rho = 1000.
#    else : 
#        rho = 1.
#    return rho


## Fields
velo = pp.Field(domain=box, formula=computeVel, 
                       name='Velocity', isVector=True)
vorti = pp.Field(domain=box, formula=computeVort, 
                        name='Vorticity', isVector=True)
#vortNorm = pp.Field(domain=box, formula=computeVortNorm,
#                           name='VortNorm', isVector=False)
#density = pp.Field(domain=box, formula=computeDensity,
#                             name='Density', isVector=False)

## Operators
advec = Advection(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  method=methodAdv
                 )

stretch = Stretching(velo, vorti,
                     resolutions={velo: nbElem,
                                  vorti: nbElem},
                     method=methodStretch,
                     propertyOp='divConservation'
                    )

diffusion = Diffusion(vorti,
                      resolution=nbElem,
                      method='fftw',
                      viscosity=visco
                     )

poisson = Poisson(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  method='fftw',
                  projection=False
                 )

penal = Penalization(velo, vorti, sphere, lambd,
                     resolutions={velo: nbElem,
                               vorti: nbElem},
                     method=methodPenal,
                     with_curl=True
                    )

## Diagnostics related to the problem
forces = Compute_forces(velo, vorti,
                        resolutions={velo: nbElem,
                                     vorti: nbElem},
                        penal=penal,
                        boxMin=[Ox + Lx / 10., Oy + Ly / 10., Oz + Lz / 10.],
                        boxMax=[Lx - Lx / 10., Ly - Ly / 10., Lz - Lz / 10.],
                        Reynolds=uinf * obstRadius/ visco,
                        method=methodForces,
                        frequency=outModuloForces,
                        outputPrefix=outputForces)

energy = Energy_enstrophy(velo, vorti,
                          resolutions={velo: nbElem,
                                       vorti: nbElem},
                          viscosity=visco, 
                          frequency=outModuloEnergies,
                          outputPrefix=outputEnergies)

printer = Printer(fields=[vorti, velo],
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  frequency=outModuloPrinter,
                  outputPrefix=outputPrinter)

distrAdvStr = Redistribute([vorti, velo], advec, stretch)
distrStrPoiss = Redistribute([vorti, velo], stretch, poisson)
distrPoissPen = Redistribute([vorti, velo], poisson, penal)
distrPenAdv = Redistribute([vorti, velo], penal, advec)

## Define the problem to solve
#pb = NSProblem([advec, distrAdvPen, penal, distrPenAdv],
#                monitors=[energy, printer])
pb = NSProblem([advec, distrAdvStr, stretch, distrStrPoiss, 
               diffusion, poisson, distrPoissPen, penal, distrPenAdv],
               monitors=[energy, printer])

## Setting solver to Problem 
pb.setUp(finalTime, timeStep)

t1 = time.time()

## Solve problem
penal.apply(0., timeStep, 0)
distrPenAdv.apply(0., timeStep, 0)
timings = pb.solve()

tf = time.time()

print "\n"
print "Total time : ", tf - t0, "sec (CPU)"
print "Init time : ", t1 - t0, "sec (CPU)"
print "Solving time : ", tf - t1, "sec (CPU)"

## end of time loop ##

# Clean memory buffers
fftw2py.clean_fftw_solver(box.dimension)
