#!/usr/bin/python
#import sys
#sys.path.insert(0,'/scratch/mimeau/install-parmes3/')
import time
import parmepy as pp
from parmepy.f2py import fftw2py
import numpy as np
import mpi4py.MPI as MPI
import math as m
from parmepy.constants import PARMES_REAL, ORDER
from parmepy.operator.advection import Advection
from parmepy.operator.stretching import Stretching
from parmepy.operator.poisson import Poisson
from parmepy.operator.diffusion import Diffusion
from parmepy.operator.penalization import Penalization
from parmepy.operator.redistribute import Redistribute
from parmepy.problem.navier_stokes import NSProblem
from parmepy.problem.simulation import Simulation
from parmepy.operator.monitors.energy_enstrophy import Energy_enstrophy
from parmepy.operator.monitors.reprojection_criterion import Reprojection_criterion
from parmepy.operator.monitors.printer import Printer
from parmepy.dataloader import DataLoader


#from numpy import linalg as LA

pi = m.pi

rank = MPI.COMM_WORLD.Get_rank()
print "Mpi process number ", rank

## ----------- A 3d problem -----------
print " ========= Start test for Navier-Stokes 3D (Taylor Green benchmark)========="

## Opening/reading input data file
data = DataLoader('inputData_TG.dat')

#inputData = open("input.dat", "r")

#Ox = np.float64(inputData.readline().rstrip('\n\r'))
#Oy = np.float64(inputData.readline().rstrip('\n\r'))
#Oz = np.float64(inputData.readline().rstrip('\n\r'))
#Lx = np.float64(inputData.readline().rstrip('\n\r'))
#Ly = np.float64(inputData.readline().rstrip('\n\r'))
#Lz = np.float64(inputData.readline().rstrip('\n\r'))
#resX = np.uint32(inputData.readline().rstrip('\n\r'))
#resY = np.uint32(inputData.readline().rstrip('\n\r'))
#resZ = np.uint32(inputData.readline().rstrip('\n\r'))
#dt = np.float64(inputData.readline().rstrip('\n\r'))
#finalTime = np.float64(inputData.readline().rstrip('\n\r'))
#restart = np.uint32(inputData.readline().rstrip('\n\r'))
#dumpfreq = np.uint32(inputData.readline().rstrip('\n\r'))
#dumpfile = inputData.readline().rstrip('\n\r')
#outModuloPrinter = np.uint32(inputData.readline().rstrip('\n\r'))
#outputPrinter = inputData.readline().rstrip('\n\r')
#outModuloForces = np.uint32(inputData.readline().rstrip('\n\r'))
#outputForces = inputData.readline().rstrip('\n\r')
#outModuloEnergies = np.uint32(inputData.readline().rstrip('\n\r'))
#outputEnergies = inputData.readline().rstrip('\n\r')
#visco = np.float64(inputData.readline().rstrip('\n\r'))
#vortProj = np.uint32(inputData.readline().rstrip('\n\r'))
#methodAdv = inputData.readline().rstrip('\n\r')
#methodStretch = inputData.readline().rstrip('\n\r')
#multires = np.uint32(inputData.readline().rstrip('\n\r'))
#resFilterX = np.uint32(inputData.readline().rstrip('\n\r'))
#resFilterY = np.uint32(inputData.readline().rstrip('\n\r'))
#resFilterZ = np.uint32(inputData.readline().rstrip('\n\r'))

#inputData.close()

# Parameters
dim = 3
nbElem = [data.resX, data.resY, data.resZ]
nbElemFilter = [data.resFilterX, data.resFilterY, data.resFilterZ]

t0 = time.time()

## Domain
box = pp.Box(dim, length=[2.0 * pi, 2.0 * pi, 2.0 * pi], 
             origin=[data.Ox, data.Oy, data.Oz])

## Fields declaration
def computeVel(x, y, z):
#     # --------Vorticity ring----------
#    vx = 0.
#    vy = 0.
#    vz = 0.
#    # --------Taylor Green------------
    vx = np.sin(x) * np.cos(y) * np.cos(z)
    vy = - np.cos(x) * np.sin(y) * np.cos(z)
    vz = 0.
    return vx, vy, vz

def computeVort(x, y, z):
     # --------Vorticity ring----------
#    xc = 6. / 2.
#    yc = 6. / 2.
#    zc = 6. / 6.
#    R = 1.5
#    sigma = R / 3.
#    Gamma = 0.75
#    dist = m.sqrt((x - xc) ** 2 + (y - yc) ** 2)
#    s2 = (z - zc) ** 2 + (dist - R) ** 2
#    wx = 0.
#    wy = 0.
#    wz = 0.
#    if (dist != 0.):
#        cosTheta = (x - xc) / dist
#        sinTheta = (y - yc) / dist
#        wTheta = Gamma / (pi * sigma ** 2) * m.exp(-(s2 / sigma ** 2))
#        wx = - wTheta * sinTheta
#        wy = wTheta * cosTheta
#        wz = 0.
#     # --------Taylor Green------------
    wx = - np.cos(x) * np.sin(y) * np.sin(z)
    wy = - np.sin(x) * np.cos(y) * np.sin(z)
    wz = 2. * np.sin(x) * np.sin(y) * np.cos(z)
    return wx, wy, wz

def computeVortNorm(x, y, z):
    xc = 6. / 2.
    yc = 6. / 2.
    zc = 6. / 6.
    R = 1.5
    sigma = R / 3.
    Gamma = 7.5
    dist = m.sqrt((x - xc) ** 2 + (y - yc) ** 2)
    s2 = (z - zc) ** 2 + (dist - R) ** 2
    norm = 0.
    if (dist != 0.):
        cosTheta = (x - xc) / dist
        sinTheta = (y - yc) / dist
        wTheta = Gamma / (pi * sigma ** 2) * m.exp(-(s2 / sigma ** 2))
        wx = - wTheta * sinTheta
        wy = wTheta * cosTheta
        wz = 0.
        norm = np.sqrt(wx ** 2 + wy ** 2 + wz ** 2)
    return norm

## Fields
velo = pp.Field(domain=box, formula=computeVel, 
                       name='Velocity', isVector=True)
vorti = pp.Field(domain=box, formula=computeVort, 
                        name='Vorticity', isVector=True)
#vortNorm = pp.Field(domain=box, formula=computeVortNorm,
#                           name='VortNorm', isVector=False)

## Operators
advec = Advection(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  method=data.methodAdv
                 )

stretch = Stretching(velo, vorti,
                     resolutions={velo: nbElem,
                                  vorti: nbElem},
                     method=data.methodStretch,
                     propertyOp=data.propStretch,
                    )

diffusion = Diffusion(vorti,
                      resolution=nbElem,
                      method='fftw',
                      viscosity=data.visco
                     )

poisson = Poisson(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  method='fftw',
                  projection=data.vortProj,
                  multires=data.multires,
                  filterSize=box.length / \
                             (np.asarray(nbElemFilter, 
                                         dtype=PARMES_REAL) - 1)
                 )

## Diagnostics related to the problem
energy = Energy_enstrophy(velo, vorti,
                          resolutions={velo: nbElem,
                                       vorti: nbElem},
                          viscosity=data.visco, 
                          frequency=data.outModuloEnergies,
                          prefix=data.outputEnergies)

reproj = Reprojection_criterion(velo, vorti,
                                resolutions={velo: nbElem,
                                             vorti: nbElem},
                                method='FD_order4',
                                frequency=1,
                                prefix='./Reproj.dat')

printer = Printer(fields=[vorti, velo],
                  frequency=data.outModuloPrinter,
                  prefix=data.outputPrinter,
                  ext='.vtk')

distrAdvStr = Redistribute([vorti, velo], advec, stretch)
distrStrPoiss = Redistribute([vorti, velo], stretch, poisson)

## Definition of simulation parameters
simu = Simulation(tinit=0.0, tend=data.finalTime, timeStep=data.timeStep, iterMax=1000000)

## Define the problem to solve
#pb = NSProblem([diffusion, poisson],
pb = NSProblem(operators=[advec, distrAdvStr, stretch, distrStrPoiss, diffusion, poisson],
               simulation=simu, monitors=[printer, energy, reproj], 
               dumpFreq=int(data.dumpfreq), name=data.dumpfile)
## Setting solver to Problem
pb.setUp()

## Restarting Problem from dumped field values 
if data.restart :
    pb.restart(data.dumpfile)

t1 = time.time()

## Solve problem
#poisson.apply(simu)
timings = pb.solve()

tf = time.time()

print "\n"
print "Total time : ", tf - t0, "sec (CPU)"
print "Init time : ", t1 - t0, "sec (CPU)"
print "Solving time : ", tf - t1, "sec (CPU)"

## end of time loop ##

# Clean memory buffers
fftw2py.clean_fftw_solver(box.dimension)
