# -*- coding: utf-8 -*-
import time
import parmepy as pp
import math


def vitesse(x):
    r = math.sqrt(x[0] * x[0] + x[1] * x[1])
    c = math.cos(3 * math.pi * r / 2.)
    return [-c * x[1], c * x[0]]


def scalaire(x):
    r = math.sqrt(x[0] * x[0] + x[1] * x[1])
    if r < 1:
        return (1. - r * r) ** 6
    else:
        return 0.


def run():
    dim = 2
    nb = 256
    boxLength = [2., 2.]
    boxMin = [-1., -1.]
    nbElem = [nb, nb]

    timeStep = 0.02
    finalTime = 2*timeStep
    outputFilePrefix = './res/gaussianSheared_results_'
    outputModulo = 1

    t0 = time.time()

    t0 = time.time()

    ## Domain
    box = pp.Box(dim, length=boxLength, origin=boxMin)

    ## Fields
    scal = pp.ContinuousField(domain=box, name='Scalar')
    velo = pp.ContinuousField(domain=box, name='Velocity', vector=True)
    #scal = pp.AnalyticalField(domain=box, formula=scalaire, name='Scalar')
    #velo = pp.AnalyticalField(domain=box, formula=vitesse, name='Velocity', vector=True)

    ## Operators
    advec = pp.Transport(velo, scal)

    ## Solver creation (discretisation of objects is done in solver initialisation)
    topo3D = pp.CartesianTopology(domain=box, resolution=nbElem, dim=dim)

    ##Problem
    pb = pp.Problem(topo3D, [advec])

    ## Setting solver to Problem
    pb.setSolver(finalTime, timeStep, 'gpu',
                 src=['./gaussianSheared.cl'],
                 io=pp.Printer(fields=[scal, velo], frequency=outputModulo, outputPrefix=outputFilePrefix))
    pb.initSolver()

    t1 = time.time()

    ## Solve problem
    pb.solve()

    tf = time.time()

    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    run()
