
__kernel void initScalar(__global float* values,
			 __private const float t,
			 __private const float4 min,
			 __private const uint4 nb,
			 __private const float4 size)
{
  __private uint ind,ix,iy;
  __private float px,py,s;

  ix = get_global_id(0);
  iy = get_global_id(1);

  px = min.x + (float)(ix)*size.x;
  py = min.y + (float)(iy)*size.y;

  if (sqrt((px-0.5f)*(px-0.5f) + (py-0.75f)*(py-0.75f)) < 0.15f)
    s = 1.0f;
  else
    s = 0.0f;
  // Write
  ind = iy + ix*nb.y;
  values[ind] = s;
}

// velocity field
__kernel void velocity(__global float* gvelo,
			__private const float t,
		        __private const float period,
			__private const float4 min,
			__private const uint4 nb,
			__private const float4 size
			)
{
  __private uint ix,iy, ind;
  __private float px,py,vx,vy;

  ix = get_global_id(0);
  iy = get_global_id(1);

  px = min.x + (float)(ix)*size.x; //2flop
  py = min.y + (float)(iy)*size.y; //2flop

  vx = -sin(px*M_PI_F)*sin(px*M_PI_F)*sin(2.0f*py*M_PI_F)*cos(t*M_PI_F/period); // 14flop
  vy = sin(py*M_PI_F)*sin(py*M_PI_F)*sin(2.0f*px*M_PI_F)*cos(t*M_PI_F/period); // 14flop

  ind = iy + ix*nb.y;
  gvelo[ind] = vx;
  ind = nb.x*nb.y + ix + iy*nb.x;
  gvelo[ind] = vy;
}
