# -*- coding: utf-8 -*-
import time
import new_ParMePy as pp
import math


def run():
    dim = 3
    nb = 160
    boxLength = [1., 1., 1.]
    boxMin = [0., 0., 0.]
    nbElem = [nb, nb, nb]

    timeStep = 0.07
    period = 3.
    FinalTime = period
    outputFilePrefix = './res/RK2_'
    outputModulo = 1

    t0 = time.time()

    box = pp.Domain.Box(dimension=dim,
              length=boxLength,
              minPosition=boxMin)
    velo = pp.Variable.AnalyticalVariable(domain=box,
                              dimension=dim, name="Velocity"
        )
    scal = pp.Variable.AnalyticalVariable(domain=box,
                              dimension=1,
                              scalar=True, name="Scalar"
        )
    advec = pp.Operator.Advection(velo, scal)
    cTspPb = pp.Problem.ContinuousTransportProblem(advection=advec)
    cTspPb.addOperator(pp.Operator.VelocityOp(velo, period=period), 0)

    cTspPb.discretize(dSpec=[nbElem])

    InterpolationMethod = pp.Utils.Linear(grid=box.discreteDomain, gvalues=velo.discreteVariable)
    RemeshingMethod = pp.Utils.M6Prime(grid=box.discreteDomain, gvalues=scal.discreteVariable)
    ODESolver = pp.Utils.RK2(InterpolationMethod, box.discreteDomain.applyConditions)
    ## cpu
    #solver = pp.Utils.ParticularSolver(cTspPb, ODESolver, InterpolationMethod, RemeshingMethod)
    ## gpu
    #solver = pp.GPUParticularSolver(cTspPb, ODESolver, InterpolationMethod, RemeshingMethod, device_type='gpu')
    solver = pp.GPUParticularSolver_GLRender(cTspPb,
                                             ODESolver, InterpolationMethod, RemeshingMethod,
                                             device_type='gpu', dt=timeStep,
                                             src='./examples/main_Rotating_3D_GH_kernels.cl'
        )
    cTspPb.setSolver(solver)
    cTspPb.setPrinter(pp.Utils.Printer(frequency=outputModulo, outputPrefix=outputFilePrefix, problem=cTspPb))

    t1 = time.time()

    cTspPb.solve(T=FinalTime, dt=timeStep)

    tf = time.time()

    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    run()
