
__kernel void initScalar(__global float* values,
			 __private const float t,
			 __private const float4 min,
			 __private const uint4 nb,
			 __private const float4 size)
{
  __private uint ind,ix,iy,iz;
  __private float px,py,pz,s;

  ix = get_global_id(0);
  iy = get_global_id(1);
  iz = get_global_id(2);

  px = min.x + (float)(ix)*size.x;
  py = min.y + (float)(iy)*size.y;
  pz = min.z + (float)(iz)*size.z;

  if (sqrt((px-0.35f)*(px-0.35f) + (py-0.35f)*(py-0.35f) + (pz-0.35f)*(pz-0.35f)) < 0.15f)
    s = 1.0f;
  else
    s = 0.0f;
  // Write
  ind = iz + iy*nb.z + ix*nb.y*nb.z;
  values[ind] = s;
}

// velocity field
__kernel void velocity(__global float* gvelo,
			__private const float t,
		        __private const float period,
			__private const float4 min,
			__private const uint4 nb,
			__private const float4 size
			)
{
  __private uint ix,iy, iz, ind;
  __private float px,py, pz,vx,vy,vz;

  ix = get_global_id(0);
  iy = get_global_id(1);
  iz = get_global_id(2);

  px = min.x + (float)(ix)*size.x;
  py = min.y + (float)(iy)*size.y;
  pz = min.z + (float)(iz)*size.z;

  vx = 2.0f * sin(px*M_PI_F)*sin(px*M_PI_F)*sin(2.0f*py*M_PI_F)*sin(2.0f*pz*M_PI_F)*cos(t*M_PI_F/period);
  vy = -sin(2.0f*px*M_PI_F)*sin(py*M_PI_F)*sin(py*M_PI_F)*sin(2.0f*pz*M_PI_F)*cos(t*M_PI_F/period);
  vz = -sin(2.0f*px*M_PI_F)*sin(pz*M_PI_F)*sin(pz*M_PI_F)*sin(2.0f*py*M_PI_F)*cos(t*M_PI_F/period);

  ind = iz + iy*nb.z + ix*nb.y*nb.z;
  gvelo[ind] = vx;
  ind = (nb.x*nb.y*nb.z) + iz + ix*nb.z + iy*nb.x*nb.z;
  gvelo[ind] = vy;
  ind = (nb.x*nb.y*nb.z)*2 + iy + ix*nb.y + iz*nb.x*nb.y;
  gvelo[ind] = vz;
}
