# -*- coding: utf-8 -*-
import time
import parmepy as pp
import math


def vitesse(x):
    r = math.sqrt(x[0] * x[0] + x[1] * x[1])
    c = math.cos(3 * math.pi * r / 2.)
    return [-c * x[1], c * x[0]]


def scalaire(x):
    r = math.sqrt(x[0] * x[0] + x[1] * x[1])
    if r < 1:
        return (1. - r * r) ** 6
    else:
        return 0.


def run():
    dim = 2
    nb = 128
    boxLength = [2., 2.]
    boxMin = [-1., -1.]
    nbElem = [nb, nb]

    timeStep = 0.02
    FinalTime = 1.
    outputFilePrefix = './res/RK2_'
    outputModulo = 1

    t0 = time.time()

    box = pp.Domain.Box(dimension=dim,
              length=boxLength,
              minPosition=boxMin)
    velo = pp.Variable.AnalyticalVariable(domain=box,
                              dimension=dim,
                              formula=vitesse)
    scal = pp.Variable.AnalyticalVariable(domain=box,
                              dimension=1,
                              formula=scalaire, scalar=True)
    advec = pp.Operator.Advection(velo, scal)
    cTspPb = pp.Problem.ContinuousTransportProblem(advection=advec)

    cTspPb.discretize(dSpec=[nbElem])

    InterpolationMethod = pp.Utils.Linear(grid=box.discreteDomain, gvalues=velo.discreteVariable)
    RemeshingMethod = pp.Utils.M6Prime(grid=box.discreteDomain, gvalues=scal.discreteVariable)
    ODESolver = pp.Utils.RK2(InterpolationMethod, box.discreteDomain.applyConditions)
    ## cpu
    #solver = pp.Utils.ParticularSolver(cTspPb, ODESolver, InterpolationMethod, RemeshingMethod)
    ## gpu
    #solver = pp.GPUParticularSolver(cTspPb, ODESolver, InterpolationMethod, RemeshingMethod, device_type='gpu')
    solver = pp.GPUParticularSolver_GLRender(cTspPb, ODESolver, InterpolationMethod, RemeshingMethod, device_type='gpu', dt=timeStep)
    cTspPb.setSolver(solver)
    cTspPb.setPrinter(pp.Utils.Printer(frequency=outputModulo, outputPrefix=outputFilePrefix, problem=cTspPb))

    t1 = time.time()

    cTspPb.solve(T=FinalTime, dt=timeStep)

    tf = time.time()

    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    run()
