#!/usr/bin/python
#import sys
#sys.path.insert(0,'/scratch/mimeau/install-parmes3/')
import parmepy as pp
from parmepy.f2py import fftw2py
import numpy as np
from parmepy.fields.continuous import Field
from parmepy.variables.variables import Variables
from parmepy.mpi.topology import Cartesian
from parmepy.domain.obstacle.sphere import Sphere
from parmepy.operator.advection import Advection
from parmepy.operator.stretching import Stretching
from parmepy.operator.poisson import Poisson
from parmepy.operator.velocity_correction import VelocityCorrection
from parmepy.operator.diffusion import Diffusion
from parmepy.operator.differential import Curl
from parmepy.operator.redistribute import Redistribute
from parmepy.operator.adapt_timestep import AdaptTimeStep
from parmepy.operator.baroclinic import Baroclinic
from parmepy.operator.density import DensityColor
from parmepy.domain.obstacle.planes import PlaneBoundaries
from parmepy.operator.penalization import Penalization
from parmepy.problem.navier_stokes import NSProblem
from parmepy.operator.monitors.printer import Printer
from parmepy.operator.monitors.energy_enstrophy import Energy_enstrophy
from parmepy.operator.monitors.compute_forces import Forces
from parmepy.problem.simulation import Simulation
from parmepy.constants import VTK, HDF5
import math as m
from dataNS_RMI import dim, nb, NBGHOSTS, ADVECTION_METHOD, \
    BAROCLINIC_METHOD, VISCOSITY, \
    OUTPUT_FREQ, FILENAME_ENER, PROJ, LCFL, CFL, CURL_METHOD,\
    TIMESTEP_METHOD, RHO_BOTT, RHO_TOP

## ----------- A 3d problem -----------
print " ========= Start test for Navier-Stokes 3D (Rayleigh-Taylor instability)========="

## constants
pi = np.pi
cos = np.cos
sin = np.sin
exp = np.exp
abs = np.abs
tanh = np.tanh

## Parameters
dim = 3
densityVal = [RHO_BOTT, RHO_TOP]

## Domain
#box = pp.Box(dim, length=[0.5, 2.0, 0.125], 
#             origin=[-0.25, -1.0, -0.0625])
box = pp.Box(dim, length=[3.0, 3.0, 0.125], 
             origin=[-1.5, -1.5, -0.0625])
#box = pp.Box(dim, length=[3.0, 3.0, 0.0625], 
#             origin=[0., 0., 0.])

## Global resolution
#nbElem = [nb] * dim
nbElem = [289, 289, 13] 
#nbElem = [97, 289, 97] 
#nbElem = [101, 401, 25] 


uinf = 1.0

## Fields declaration
def computeVel(res, x, y, z, t):
#    res[0][...] = 0.
#    res[1][...] = 0.
#    res[2][...] = 0.

# -------------- Cond papier Arepo --------------
#    res[0][...] = 0.
#    res[1][...] = 0.025 * (1 - cos(4.0 * pi * x)) #* (1.0 - cos(4.0 * pi * z / 3.0))
#    res[2][...] = 0.

# -------------- Cond gaussienne --------------
#    res[0][...] = 0.
#    res[1][...] = 0.01 * sin(2.0 * pi * x / 0.5) * np.exp(-(y-0.)**2/(0.4**2))
    Lx = box.length[0]
    sigm = 0.4
    coef = 0.02
    middle = box.origin + box.length / 2.0
    res[0][...] = -(2.0 * y / sigm ** 2) * coef * (cos(2.0 * 3.0 * pi * x / Lx) / (2.0 * 3.0 * pi / Lx)) \
                   * np.exp(-(y - middle[1]) ** 2 / (sigm ** 2))
#    res[0][...] = -(2.0 * y / sigm ** 2) * coef * cos(2.0 * 3.0 * pi * x / Lx) \
#                   * np.exp(-(y - middle[1]) ** 2 / (sigm ** 2))
    res[1][...] = coef * sin(2.0 * 3.0 * pi * x / Lx) * np.exp(-(y - middle[1]) ** 2/(sigm ** 2))
    res[2][...] = 0.

# -----------Conditions CEA (Jerome Griffond) --------
#    ampli = 0.0027935
#    lamb = 0.05933
#    compr = 0.767
#    width = 0.0025152
#    xd = compr * width / np.sqrt(pi)
#    xk = 2.0 * pi / lamb
#    dadt = 52.29 * xk * ampli
#    v0 = 2.0 * dadt
#    fctG = np.zeros_like(y)
#    fctH = np.zeros_like(y)
#    for i in xrange(np.shape(y)[0]):
#        fctG[0,i,0] = v0 * 0.25 * m.exp((0.5 * xd * xk) ** 2) \
#                      * ((1.0 - m.erf(0.5 * xd * xk - y[0,i,0] / xd)) \
#                      * m.exp(-xk * y[0,i,0]) + (1.0 - m.erf(0.5 * xd * xk + y[0,i,0] / xd)) \
#                      * m.exp(xk * y[0,i,0]))
#        fctH[0,i,0] = v0 * 0.25 * m.exp((0.5 * xd * xk) ** 2) \
#                      * ((1.0 - m.erf(0.5 * xd * xk - y[0,i,0] / xd)) \
#                      * m.exp(-xk * y[0,i,0]) - (1.0 - m.erf(0.5 * xd * xk + y[0,i,0] / xd)) \
#                      * m.exp(xk * x[0,i,0]))
#    res[0][...] = fctH * sin(xk * x)
#    res[1][...] = fctG * cos(xk * x)
#    res[2][...] = 0.

# -----------Conditions Sheu/Yu (Commun. Comput. Phys.) --------
#    res[0][...] = 0.
#    res[1][...] = 2.0 + 0.1 * cos(2.0 * pi * x)
#    res[2][...] = 0.
    return res

def computeVort(res, x, y, z, t):
    res[0][...] = 0.
    res[1][...] = 0.
    res[2][...] = 0.
# -----------Conditions CEA (Jerome Griffond) --------
#    ampli = 0.0027935
#    lamb = 0.05933
#    compr = 0.767
#    width = 0.0025152
#    xd = compr * width / np.sqrt(pi)
#    xk = 2.0 * pi / lamb
#    dadt = 52.29 * xk * ampli
#    v0 = 2.0 * dadt
#    res[0][...] = 0.
#    res[1][...] = v0 * np.exp(-(y / xd) ** 2) / (xd * np.sqrt(pi))
#    res[2][...] = 0.
    return res

def computeDensity(res, x, y, z, t):
#    perturb = 0.2 * np.sin(2.0 * pi * y * z/ 5.9333)
#    d = (3.0 + perturb + 0.5 - x)/(2.0 * 0.5)
#    c = 1.0
#    res[0][...] = densityVal[1]
#    ind0 = np.where(np.logical_and(d > 0., d < 1.0))
#    res[0][ind0] = densityVal[0] + (densityVal[1] - densityVal[0]) * 0.5
#    ind1 = np.where(d >= 1.0)
#    res[0][ind1] = densityVal[0]

# -------------- Cond papier Arepo --------------
#    res[0][...] = densityVal[0]
#    ind = np.where(np.logical_and(np.logical_and(y >= 0.75, x>=0.), z>=0.))
#    res[0][ind] = densityVal[1]

# -----------Conditions CEA (Jerome Griffond) : lissage --------
#    compr = 0.767
#    width = 0.0025152
#    origin = box.origin
#    middle = box.origin + box.length / 2.0
##    xd = compr * width / np.sqrt(pi)
#    xd = 0.02

#    ind_rhoBott = np.where(np.logical_and(np.logical_and(y< middle[1] - xd, x>=origin[0]), z>=origin[2]))
#    ind_rhoTop = np.where(np.logical_and(np.logical_and(y> middle[1] + xd, x>=origin[0]), z>=origin[2]))

#    res[0][ind_rhoBott] = densityVal[0]
#    res[0][ind_rhoTop] = densityVal[1]
#    Mrho = 0.5 * (densityVal[1] + densityVal[0])
#    Drho = 0.5 * (densityVal[1] - densityVal[0])
#    for j in xrange(np.shape(y)[1]):
#        if (y[0,j,0]<= middle[1] + xd) and (y[0,j,0]>= middle[1] - xd):
#            y[0,j,0] = m.erf((y[0,j,0] - middle[1]) / xd)
#            res[0][:,j,:] = Mrho + Drho * y[0,j,0]

# -----------Conditions GHC (log(rho))--------
    origin = box.origin
    middle = box.origin + box.length / 2.0
    ind_rhoTop = np.where(np.logical_and(np.logical_and(y> middle[1], x>=origin[0]), z>=origin[2]))

    res[0][...] = m.log(densityVal[0])
    res[0][ind_rhoTop] = m.log(densityVal[1])
    return res

## Fields
velo = pp.Field(domain=box, formula=computeVel, 
                name='Velocity', isVector=True)
vorti = pp.Field(domain=box, formula=computeVort, 
                 name='Vorticity', isVector=True)
rho = pp.Field(domain=box, formula=computeDensity,
                 name='Density', isVector=False)

## Usual Cartesian topology definition
ghosts = np.ones((box.dimension)) * NBGHOSTS
topo = Cartesian(box, box.dimension, nbElem,
                 ghosts=ghosts)

## Operators
advec = Advection(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  method=ADVECTION_METHOD)

advecScalar = Advection(velo, rho,
                        resolutions={velo: nbElem,
                                     rho: nbElem},
                        method=ADVECTION_METHOD)

stretch = Stretching(velo, vorti,
                     resolutions={velo: nbElem,
                                  vorti: nbElem},
                     topo=topo)

baroclinic = Baroclinic(velo, vorti, rho,
                        viscosity=VISCOSITY,
                        resolutions={velo: nbElem,
                                     vorti: nbElem,
                                     rho: nbElem},
                        topo=topo,
                        method=BAROCLINIC_METHOD,
                        formula=computeVel)

poisson = Poisson(velo, vorti,
                  resolutions={velo: nbElem,
                               vorti: nbElem},
                  projection=PROJ)

#-----------------------------
# Topology without ghost points
poisson.discretize()
topofft = poisson.discreteFields[poisson.vorticity].topology
#-----------------------------

diffusion = Diffusion(vorti,
                      resolution=nbElem,
                      topo=topofft,
                      viscosity=VISCOSITY)

densityColor = DensityColor(rho,
                            resolutions={rho: nbElem},
                            topo=topofft,
                            densityVal=densityVal)

curl = Curl(velo, vorti,
            resolutions={velo: nbElem,
                         vorti: nbElem},
            method=CURL_METHOD,
            topo=topofft)

#bc = PlaneBoundaries(box, 1, thickness=0.2)

#penal = Penalization(velo, [bc],
#                     coeff=[1e8],
#                     topo=topofft,
#                     resolutions={velo: nbElem})

## Bridges between the different topologies in order to
## redistribute data.
# 0 - Poisson to Curl_FD and Curl_FD to advection
distrCurlAdv = Redistribute([vorti, velo], curl, advec)
# 1 -Advection to stretching
distrPoiStr_velo = Redistribute([velo], poisson, stretch)
distrAdvStr_vorti = Redistribute([vorti], advec, stretch)
# 3 - Stretching to Diffusion
distrStrDiff = Redistribute([vorti], stretch, diffusion)
# 4 - Advection_Scalar to Baroclinic and Diff to Baroclinic
distrAdvBaro = Redistribute([rho], advecScalar, baroclinic)
distrDiffBaro = Redistribute([vorti], diffusion, baroclinic)
# 5 - Baroclinic to Poisson
distrBaroPoiss = Redistribute([vorti], baroclinic, poisson)

## Simulation
simu = Simulation(tinit=0.0,
                  tend=250., timeStep=0.005,
                  iterMax=1000000)

#  Define the problem to solve
pb = NSProblem(operators=[distrPoiStr_velo,
                          advecScalar,  #
                          advec,        #
                          distrAdvStr_vorti, 
                          stretch,      #
                          distrStrDiff,
                          diffusion,    #
                          distrAdvBaro,
                          distrDiffBaro,
                          baroclinic,   #
                          distrBaroPoiss,
#                          densityColor, #
                          poisson],     #
               simulation=simu, dumpFreq=-1)


## Setting solver to Problem (only operators for computational tasks)
pb.pre_setUp()

printer = Printer(variables=[velo, vorti, rho],
                  topo=topofft,
                  frequency=50,
                  prefix='./res/NS_RMI',
                  formattype=VTK)

energy = Energy_enstrophy(velo, vorti,
                          topo=topofft,
                          viscosity=VISCOSITY,
                          isNormalized=False,
                          frequency=OUTPUT_FREQ,
                          filename=FILENAME_ENER,
                          safeOutput=True)

## Add monitors and setting solver to Problem
vorti.setTopoInit(topofft)
velo.setTopoInit(topofft)
rho.setTopoInit(topofft)
pb.addMonitors([energy, printer])
pb.setUp()

# setUp for temporary operators
curl.discretize()
curl.setUp()
#poisson.setUp()
#distrCurlAdv.setUp()

# compute vorticity from initial velocity field
#poisson.apply(simu)
curl.apply(simu)
#distrCurlAdv.apply(simu)

def run():
## Solve problem
    pb.solve()

## end of time loop ##

from parmepy.mpi import MPI
print "Start computation ..."
time = MPI.Wtime()
run()
print 'total time (rank):', MPI.Wtime() - time, '(', topo.rank, ')'

## Clean memory buffers
fftw2py.clean_fftw_solver(box.dimension)
