from parmepy.methods_keys import Scales, TimeIntegrator, Interpolation,\
    Remesh, Support, Splitting, dtCrit, SpaceDiscretisation, GhostUpdate
from parmepy.numerics.integrators.runge_kutta2 import RK2 as RK2
from parmepy.numerics.integrators.runge_kutta3 import RK3 as RK3
from parmepy.numerics.integrators.runge_kutta4 import RK4 as RK4
from parmepy.numerics.finite_differences import FD_C_4, FD_C_2
from parmepy.numerics.interpolation import Linear
from parmepy.numerics.remeshing import L6_4 as rmsh
from parmepy.f2py import fftw2py
import numpy as np

# problem dimension
dim = 3
# resolution
nb = 33
# number of ghosts in usual cartesian topo
NBGHOSTS = 2
# Adaptative timestep method
# For the "dtCrit" key, the possible choices are the following:
# 'vort' : def of dt_advection based on infinite norm of vorticity, 
# 'gradU' : def of dt_advection based on infinite norm of velocity gradient,
# 'deform' : def of dt_advection based on infinite norm of deformation tensor
TIMESTEP_METHOD = {TimeIntegrator: RK3, SpaceDiscretisation: FD_C_4, 
                   dtCrit: 'deform'}
# Lagrangian CFL for adaptative timestep
LCFL = 0.125
# CFL (if CFL is None, no CFL condition is taken into account 
# in the computation of adaptative timesteps)
CFL = 0.5
# Advection method
ADVECTION_METHOD = {Scales: 'p_M6', Splitting: 'classic'}
#ADVECTION_METHOD = {TimeIntegrator: RK2,
#                    Interpolation: Linear,
#                    Remesh: rmsh,
#                    Support: '',
#                    Splitting: 'o2_FullHalf'}
# Curl method
CURL_METHOD = {SpaceDiscretisation: FD_C_4, GhostUpdate: True}
#CURL_METHOD = {SpaceDiscretisation: 'fftw'}
# Viscosity of the fluid
VISCOSITY = 1. / 300.
# Vorticity projection (list of 2 elements): 
# - the 1st element determines if the reprojection is needed
# - the 2nd element determines the reprojection frequency 
# is terms of number of iterations IF the 1st element is True
# ex : PROJ = [True, 10] means that the reprojection 
# of vorticty will be applied every 10 iterations
PROJ = [False, 10]

## pi constant
pi = np.pi
# Obstacle description
OBST_Ox = 0.
OBST_Oy = 0.
OBST_Oz = 0.
RADIUS = 0.5

# post treatment ...
OUTPUT_FREQ = 1
FILENAME = './res/energy_NS_sphere_FFTcurl_8cpu.dat'
