#!/usr/bin/env python
import parmepy
#parmepy.__VERBOSE__ = True
from parmepy.domain.box import Box
from parmepy.gpu import PARMES_REAL_GPU, PARMES_DOUBLE_GPU
#parmepy.gpu.CL_PROFILE = True

from parmepy.fields.continuous import Field
from parmepy.operator.advection import Advection
from parmepy.problem.transport import TransportProblem
from parmepy.operator.analytic import Analytic
from parmepy.gpu.QtRendering import QtOpenGLRendering
from parmepy.problem.simulation import Simulation
import math
import numpy as np

norm2 = lambda x, y: x * x + y * y
norm1 = lambda x, y: abs(x) + abs(y)
norminf = lambda x, y: max(abs(x), abs(y))


def initScalar(x, y):
    return math.exp(-(norm2(x - 0.5, y - 0.75) / 0.0225) ** 6) \
        + 0.75 * math.exp(-(norm2(x - 0.75, y - 0.25) / 0.0225) ** 6) \
        + 0.5 * math.exp(-(norm1(x - 0.4, y - 0.4) / 0.1) ** 6) \
        + 0.25 * math.exp(-(norminf(x - 0.6, y - 0.5) / 0.08) ** 6)
    # if norm2(x - 0.5, y - 0.75) < 0.0225:
    #     return  0.25
    # elif norm2(x - 0.75, y - 0.25) < 0.0225:
    #     return 0.5
    # elif norm1(x - 0.4, y - 0.4) < 0.1:
    #     return 0.75
    # elif norminf(x - 0.6, y - 0.5) < 0.08:
    #     return 1.
    # else:
    #     return 0.


def vitesse(x, y, t=0):
    vx = -math.sin(x * math.pi) ** 2 * math.sin(y * math.pi * 2.) * \
        math.cos(t * math.pi / 3.)
    vy = math.sin(y * math.pi) ** 2 * math.sin(x * math.pi * 2.) * \
        math.cos(t * math.pi / 3.)
    return vx, vy


dim = 2
boxLength = [1., 1.]
boxMin = [0., 0.]
nbElem = [513, 513]

timeStep = 0.075
finalTime = 3.0 + timeStep
simu = Simulation(tinit=0.0, tend=finalTime, timeStep=timeStep)

## Domain
box = Box(dim, length=boxLength, origin=boxMin)

## Fields
scal = Field(domain=box, name='Scalar', formula=initScalar)
velo = Field(domain=box, name='Velocity', isVector=True)

## Operators
advec = Advection(velo, scal,
                  resolutions={velo: nbElem,
                               scal: nbElem},
                  method='gpu_1k_m6prime',
                  #src=['./demo_2D.cl'],
                  precision=PARMES_REAL_GPU,
                  splittingConfig='o2'
                  )
velocity = Analytic(velo,  # formula=vitesse,
                    resolutions={velo: nbElem},
                    method='gpu',
                    src=['./demo_2D.cl'],
                    precision=PARMES_REAL_GPU,
                    )
render = QtOpenGLRendering(scal)

# Problem
pb = TransportProblem([velocity, advec], simu,
                      monitors=[render])

# Setting solver to Problem
pb.setUp()

# We copy the first discretisation of scalar
scalar_initial = np.copy(scal.discreteFields.values()[0].data)

## Solve problem
pb.solve()

scal_disc = scal.discreteFields.values()[0]
scal_disc.toHost()
print 'Erreur : ', np.max(np.abs(scalar_initial -
                                 scal_disc.data)) / np.max(scalar_initial)
print np.linalg.norm(scalar_initial - scal_disc.data, ord=2) / \
    np.linalg.norm(scalar_initial, ord=2)

pb.finalize()
