"""
Task parallelism example.

One process is dedicated to file outputs and other are computing the data.
"""
import parmepy as pp
from parmepy.constants import np, VTK, HDF5
from parmepy.mpi.main_var import main_size, main_rank, main_comm
from parmepy.mpi.topology import Cartesian
from parmepy.fields.continuous import Field
from parmepy.operator.analytic import Analytic
from parmepy.operator.redistribute_intercomm import RedistributeIntercomm
from parmepy.operator.monitors.printer import Printer
from parmepy.problem.problem_tasks import ProblemTasks
from parmepy.problem.simulation import Simulation
from parmepy.tools.problem2dot import toDot
pi = np.pi
cos = np.cos
sin = np.sin


def func(res, x, y, z, t=0.):
    res[0][...] = 2. * sin(pi*x)**2*sin(2.*pi*y)*sin(2.*pi*z)*cos(t*pi/3.)
    res[1][...] = -sin(2.*pi*x)*sin(pi*y)**2*sin(2.*pi*z)*cos(t*pi/3.)
    res[2][...] = -sin(2.*pi*x)*sin(2.*pi*y)*sin(pi*z)**2*cos(t*pi/3.)
    return res

assert main_size > 1

nb_elem = [65, ] * 3
b = pp.Box()
proc_tasks = [0,] * main_size
proc_tasks[0] = 1
comm_s = main_comm.Split(color=proc_tasks[main_rank], key=main_rank)


if proc_tasks[main_rank] == 1:
    topo = Cartesian(b, dim=1, globalMeshResolution=nb_elem,
                     cutdir=[True, False, False],
                     comm=comm_s
                     )
elif proc_tasks[main_rank] == 0:
    topo = Cartesian(b, dim=2, globalMeshResolution=nb_elem,
                     cutdir=[False, True, True],
                     comm=comm_s
                     )

field = Field(domain=b, name="Test_Vec",
              isVector=True, formula=func)

op = Analytic([field], {field: nb_elem}, topo=topo, task_id=0)
p = Printer(variables=[field],
            topo=topo,
            frequency=1,
            prefix='./demo_task/',
            formattype=HDF5, task_id=1)

red = RedistributeIntercomm(field, topo, op, p, proc_tasks, main_comm,
                                component=None)

simu = Simulation(tinit=0.0, tend=3., timeStep=0.15, iterMax=200)


pb = ProblemTasks([op, red, p], simu, proc_tasks)
pb.pre_setUp()

# As 1 process is only a printer, one need explicit discretize and
# set the initialization topology for the field.
field.discretize(topo)
field.setTopoInit(topo)
pb.setUp()
pb.solve()
pb.finalize()
