# -*- coding: utf-8 -*-
#
# This file gives some examples on how parmepy integrators must
# be used.
# 
#
# \todo : complete it with 2D and 3D cases.
# 

from parmepy.methods import Euler, RK2, RK3, RK4
from parmepy.constants import WITH_GUESS
from parmepy.tools import numpywrappers as npw
import math
import numpy as np
pi = math.pi
sin = np.sin
cos = np.cos
import matplotlib.pyplot as plt
from parmepy.constants import MPI
from parmepy.mpi.topology import Cartesian

# Grid resolution for tests
nb = 5
# Initial time
tinit = 0.
# Final time
tend = 0.5
# Time step
#dt = 1e-3
# time sequence
#time = npu.seq(tinit, tend, dt)
#nbSteps = time.size
import math as m
import parmepy as pp
pi = m.pi

# A set of tests and reference functions

def func1D(t, y, sol):
    sol[0][...] = -y[0]
    return sol


def func2D(t, y, sol):
    sol[0][...] = y[1]
    sol[1][...] = -y[0]
    return sol

### odespy utilities, for comparisons ... ###
try:
    import odespy as ode

except ImportError, e:
    msg = 'This tutorial needs odespy package.'
    msg += 'Please check https://github.com/hplgit/odespy for details.'
    print msg


def funcODESPY(y, t):
    return -y


def func2DODESPY(y, t):
    return [y[1], -y[0]]


def rhs(y, w):
    w[0][...] = y[0]
    w[1][...] = 2 * y[1]
    return w

shapey = (500, 500, 500)


# No work, no guess, no result.
def solve0(y):
    nb = 2
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    #print "solve ...", id(y[0]), " ", id(work[0])
    work[:nb] = rhs(y, work[:nb])
    #print "post rhs  ...", id(y[0]), " ", id(work[0])
    [np.add(y[i], work[i] * 0.1, work[i])
     for i in xrange(nb)]
    #print "post op  ...", id(y[0]), " ", id(work[0])
    return work[:nb]

nb = 2


# work, no guess, no result
def solve(y, work):
    work[:nb] = rhs(y, work[:nb])
    [np.add(y[i], work[i] * 0.1, work[i])
     for i in xrange(nb)]
    return work[:nb]


def solve2(y, work):
    [np.add(y[i], work[i] * 0.1, work[i])
     for i in xrange(nb)]
    return work[:nb]


def solve3(y, work):
    [np.add(y[i], work[i] * 0.1, work[i + 2])
     for i in xrange(nb)]
    return work[nb:nb * 2]


def solve4(y, work, res):
    [np.add(y[i], work[i] * 0.1, res[i])
     for i in xrange(nb)]
    #res[0][...] = work[0][...]
    #res[1][...] = work[1][...]
    return res


def solve6(y, work, res):
    [np.add(y[i], work[i] * 0.1, res[i])
     for i in xrange(nb)]
    #res[0][...] = work[0][...]
    #res[1][...] = work[1][...]
    #    return res

## Deepcopy for res : bad perf, change res address.
## def solve5(y, work, res):
##     nb = 2
##     #print "solve2 ...", id(y[0]), " ", id(work[0])
##     [np.add(y[i], work[i] * 0.1, work[i])
##      for i in xrange(nb)]
##     #print "post op  ...", id(y[0]), " ", id(work[0])
##     res = copy.deepcopy(work[:nb])
##     return res


### No inplace computation ###
# OK
def cas_basic():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    start = MPI.Wtime()
    sol = solve0(y)
    print "basic time : ",  MPI.Wtime() - start
    print sol[0].mean() + sol[1].mean()
    print id0, id(y[0])


# OK
def cas_work():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    start = MPI.Wtime()
    sol = solve(y, work)
    print "work time", MPI.Wtime() - start
    print sol[0].mean() + sol[1].mean()
    print id0, id(y[0])


# Ok, little better than cas_work
def cas_work2():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    print id(work[0])
    start = MPI.Wtime()
    work = solve(y, work)
    print "work 2 time", MPI.Wtime() - start
    print id(work[0])
    print work[0].mean() + work[1].mean()
    print id0, id(y[0])


# Ok
def cas_guess():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    sol = solve2(y, work)
    print "guess time", MPI.Wtime() - start
    print sol[0].mean() + sol[1].mean()
    print id0, id(y[0])


#  Ok, same as cas_guess
def cas_guess2():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    print id(work[0])
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    work = solve2(y, work)
    print "guess2 time", MPI.Wtime() - start
    print id(work[0])
    print work[0].mean() + work[1].mean()
    print id0, id(y[0])


# Ok, same as  guess and guess2
def cas_guess3():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    #res = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    work = [npw.zeros(shapey), npw.zeros(shapey),
            npw.zeros(shapey), npw.zeros(shapey)]
    #  res[0], res[1]]
    res = work[2:4]
    print id(work[2])
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    res = solve3(y, work)
    print "guess3 time", MPI.Wtime() - start
    print id(work[2])
    print res[0].mean() + res[1].mean()
    print id0, id(y[0])


# Ok, poor perf.
def cas_guess7():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    res = [npw.zeros(shapey), npw.zeros(shapey)]
    print id(res[0])
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    res = solve4(y, work, res)
    print "guess7 time", MPI.Wtime() - start
    print id(res[0])
    print res[0].mean() + res[1].mean()
    print id0, id(y[0])

# Ok but poor performances.
## def cas_guess8():
##     y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
##     id0 = id(y[0])
##     work = [npw.zeros(shapey), npw.zeros(shapey)]
##     work[:2] = rhs(y, work[:2])
##     res = [npw.zeros(shapey), npw.zeros(shapey)]
##     #print "main ...", id(y[0]), " ", id(work[0])
##     print id(res[0])
##     start = MPI.Wtime()
##     res = solve5(y, work, res)
##     print "guess8 time", MPI.Wtime() - start
##     #print "post ...", id(y[0]), " ", id(work[0])
##     print id(res[0])
##     print res[0].mean() + res[1].mean()
##     print id0, id(y[0])


### Inplace computation ###
# Ok ... but y looses its id
def cas_basic_inplace():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    #print "main ...", id(y[0])
    start = MPI.Wtime()
    y = solve0(y)
    print "basic/inplace time : ",  MPI.Wtime() - start
    #print "post ...", id(y[0])
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])


# ok
def cas_basic3():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    start = MPI.Wtime()
    #    print "main ...", id(y[0])
    y[0][...], y[1][...] = solve0(y)
    print "basic3 time : ",  MPI.Wtime() - start
    print "basic3 ...", id(y[0])
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])


# Ok ... but y looses its id
def cas_work_inplace():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    start = MPI.Wtime()
    #print "main ...", id(y[0]), " ", id(work[0])
    y = solve(y, work)
    print "work/inplace time", MPI.Wtime() - start
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])


# Ok. Best perf.
def cas_guess_inplace():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey), y[0], y[1]]
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    y = solve3(y, work)
    print "guess/inplace time", MPI.Wtime() - start
    print y[0].mean() + y[1].mean()
    print id0, id(y[0]), id(work[0])


# Ok. Poor perf.
def cas_guess6():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    sol = solve2(y, work)
    y[0][...] = sol[0][...]
    y[1][...] = sol[1][...]
    print "guess6 time", MPI.Wtime() - start
    print id(sol[0])
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])


# Ok, seems to be the more efficient for inplace computation.
def cas_guess10():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey), y[0], y[1]]
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    y = solve3(y, work)
    print "guess10 time", MPI.Wtime() - start
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])

# No : change y address
## def cas_guess9():
##     y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
##     id0 = id(y[0])
##     work = [npw.zeros(shapey), npw.zeros(shapey)]
##     work[:2] = rhs(y, work[:2])
##     #res = [npw.zeros(shapey), npw.zeros(shapey)]
##     #print "main ...", id(y[0]), " ", id(work[0])
##     start = MPI.Wtime()
##     y = solve5(y, work, y)
##     print "guess9 time", MPI.Wtime() - start
##     #print "post ...", id(y[0]), " ", id(work[0])
##     print y[0].mean() + y[1].mean()
##     print id0, id(y[0])


# Ok. Same as 11.
def cas_guess11():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    y = solve4(y, work, y)
    print "guess11 time", MPI.Wtime() - start
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])


# Ok. Same as 11.
def cas_guess12():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    solve4(y, work, y)
    print "guess11 time", MPI.Wtime() - start
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])


# Ok. Same as 11.
def cas_guess13():
    y = [npw.ones(shapey), npw.ones(shapey) * 1.2]
    id0 = id(y[0])
    work = [npw.zeros(shapey), npw.zeros(shapey)]
    start = MPI.Wtime()
    work[:2] = rhs(y, work[:2])
    solve6(y, work, y)
    print "guess13 time", MPI.Wtime() - start
    print y[0].mean() + y[1].mean()
    print id0, id(y[0])
## cas_basic()
## print '------------'
## cas_work()
## print '------------'
## cas_work2()
## print '------------'
## cas_guess()
## print '------------'
## cas_guess2()
## print '------------'
## cas_guess3()
## print '------------'
## cas_guess7()
## print '------------'


## cas_basic_inplace()
## print '------------'
## cas_basic3()
## print '------------'
## cas_work_inplace()
## print '------------'
## cas_guess_inplace()
## print '------------'
## cas_guess6()
## print '------------'
## cas_guess10()
## print '------------'
## cas_guess11()
## print '------------'
## cas_guess12()
## print '------------'
## cas_guess13()
## print '------------'


def odespy1D(solver, nbSteps):
    """
    Integration with odespy
    """
    time_points = np.linspace(tinit, tend, nbSteps)
    dtt = time_points[1] - time_points[0]
    u, t = solver.solve(time_points)
    err = 0
    err = (np.abs(u[:] - np.exp(-time_points))).max()
    return dtt, err


def odespy2D(solver, nbSteps):
    """
    Integration with odespy for solver.neq > 1
    """
    time_points = np.linspace(tinit, tend, nbSteps)
    dtt = time_points[1] - time_points[0]
    u, t = solver.solve(time_points)
    neq = solver.neq
    result = [0] * neq
    result[0] = (np.abs(u[:, 0] - np.cos(time_points))).max()
    result[1] = (np.abs(u[:, 1] + np.sin(time_points))).max()
    result.insert(0, dtt)
    return result


# -- 1D cases --
def integrate(integ, nbSteps):
    """
    Integration with parmepy
    """
    t = tinit
    time_points = np.linspace(tinit, tend, nbSteps)
    dtt = time_points[1] - time_points[0]
    y = [npw.ones(nb) * math.exp(-tinit)]
    #res = [npw.zeros(nb)]
    # work = None
    i = 1
    ref = npw.zeros((nbSteps, nb))
    ref[0, :] = y[0][:]
    while i < nbSteps:
        y = integ(t, y, dtt)
        #y[0][...] = res[0]
        ref[i, :] = y[0][:]
        t += dtt
        i += 1
    err = 0.0
    for i in xrange(nb):
        err = max(err, (np.abs(ref[:, i] - np.exp(-time_points))).max())
    return dtt, err


def test_1D_0():

    nbdt = 15
    nbSteps = 100
    errE = npw.ones(nbdt)
    dtE = npw.ones(nbdt)
    err2 = npw.ones(nbdt)
    dt2 = npw.ones(nbdt)
    err3 = npw.ones(nbdt)
    dt3 = npw.ones(nbdt)
    err4 = npw.ones(nbdt)
    dt4 = npw.ones(nbdt)
    errodespy = npw.ones(nbdt)
    dtodespy = npw.ones(nbdt)
    err2odespy = npw.ones(nbdt)
    dt2odespy = npw.ones(nbdt)
    err3odespy = npw.ones(nbdt)
    dt3odespy = npw.ones(nbdt)
    err4odespy = npw.ones(nbdt)
    dt4odespy = npw.ones(nbdt)
    for i in xrange(nbdt):
        # Euler
        dtE[i], errE[i] = integrate(Euler(1, func1D), nbSteps)
        solver = ode.Euler(funcODESPY)
        solver.set_initial_condition([math.exp(-tinit)])
        dtodespy[i], errodespy[i] = odespy1D(solver, nbSteps)
        # RK2
        dt2[i], err2[i] = integrate(RK2(1, func1D), nbSteps)
        solver = ode.RK2(funcODESPY)
        solver.set_initial_condition([math.exp(-tinit)])
        dt2odespy[i], err2odespy[i] = odespy1D(solver, nbSteps)
        # RK3
        dt3[i], err3[i] = integrate(RK3(1, func1D), nbSteps)
        solver = ode.RK3(funcODESPY)
        solver.set_initial_condition([math.exp(-tinit)])
        dt3odespy[i], err3odespy[i] = odespy1D(solver, nbSteps)
        # RK4
        dt4[i], err4[i] = integrate(RK4(1, func1D), nbSteps)
        solver = ode.RK4(funcODESPY)
        solver.set_initial_condition([math.exp(-tinit)])
        dt4odespy[i], err4odespy[i] = odespy1D(solver, nbSteps)
        nbSteps += 10

    assert (dtE == dtodespy).all()
    assert (dtE == dt2odespy).all()
    assert (dtE == dt3odespy).all()
    assert (dtE == dt4odespy).all()
    assert (dtE == dt2).all()
    assert (dtE == dt3).all()
    assert (dtE == dt4).all()

    x = np.log10(1. / dtE)
    plt.figure(1)
    plt.plot(x, np.log10(errE), x, np.log10(err2),
             x, np.log10(err3), x, np.log10(err4),
             x, np.log10(errodespy))

    print " ==== EULER ===="
    print "Erreur Euler :", errE
    print "Erreur Euler (odespy) :", errodespy
    print "dt : ", dtE
    print "Pente euler:",\
        (np.log10(errE)[-1] - np.log10(errE)[0]) / (x[-1] - x[0])
    print "Pente odespy: ",\
        (np.log10(errodespy)[-1] - np.log10(errodespy)[0]) / (x[-1] - x[0])
    print " ==== RK2 ===="
    print "Erreur RK2 :", err2
    print "Erreur RK2 (odespy) :", err2odespy
    print "dt : ", dt2
    print "Pente RK2:",\
        (np.log10(err2)[-1] - np.log10(err2)[0]) / (x[-1] - x[0])
    print "Pente odespy (RK2): ",\
        (np.log10(err2odespy)[-1] - np.log10(err2odespy)[0]) / (x[-1] - x[0])
    print " ==== RK3 ===="
    print "Erreur RK3 :", err3
    print "Erreur RK3 (odespy) :", err3odespy
    print "dt : ", dt3
    print "Pente RK3:",\
        (np.log10(err3)[-1] - np.log10(err3)[0]) / (x[-1] - x[0])
    print "Pente odespy (RK3): ",\
        (np.log10(err3odespy)[-1] - np.log10(err3odespy)[0]) / (x[-1] - x[0])
    print " ==== RK4 ===="
    print "Erreur RK4 :", err4
    print "Erreur RK4 (odespy) :", err4odespy
    print "dt : ", dt4
    print "Pente RK4:",\
        (np.log10(err4)[-1] - np.log10(err4)[0]) / (x[-1] - x[0])
    print "Pente odespy (RK4): ",\
        (np.log10(err4odespy)[-1] - np.log10(err4odespy)[0]) / (x[-1] - x[0])


# -- 2D cases --
def integrate2D(integ, nbSteps):
    """
    Integration with parmepy
    """
    t = tinit
    time_points = np.linspace(tinit, tend, nbSteps)
    dtt = time_points[1] - time_points[0]
    y = [npw.ones((nb, nb)), npw.zeros((nb, nb))]
#    res = [npw.zeros((nb, nb)), npw.zeros((nb, nb))]
    i = 1
    refX = npw.zeros((nbSteps, nb, nb))
    refY = npw.zeros((nbSteps, nb, nb))
    refX[0, :, :] = y[0][...]
    refY[0, :, :] = y[1][...]
    work = integ.work
    #  [npw.ones((nb, nb)), npw.zeros((nb, nb)), npw.zeros((nb, nb)),
    #  npw.ones((nb, nb)), npw.zeros((nb, nb)), npw.zeros((nb, nb))]

    while i < nbSteps:
        work[:2] = func2D(t, y, work[:2])
        y = integ(t, y, dtt, result=y)
        refX[i, :, :] = y[0][...]
        refY[i, :, :] = y[1][...]
        t += dtt
        i += 1
    errX = 0.0
    errY = 0.0
    for i in xrange(nb):
        for j in xrange(nb):
            errX = max(errX, (np.abs(refX[:, i, j]
                                     - np.cos(time_points))).max())
            errY = max(errY, (np.abs(refY[:, i, j]
                                     + np.sin(time_points))).max())

    return dtt, errX, errY


def test_2D_0(method, odemethod, optim):
    nbdt = 15
    ghosts = [0, 0]
    box2 = pp.Box(2, length=[2.0 * pi, 2 * pi], origin=[0., 0.])
    topo2 = Cartesian(box2, 1, [nb + 1, nb + 1], ghosts=ghosts)
    nbSteps = 100
    errEx = npw.ones(nbdt)
    errEy = npw.ones(nbdt)
    dtE = npw.ones(nbdt)
    errxodespy = npw.ones(nbdt)
    erryodespy = npw.ones(nbdt)
    dtodespy = npw.ones(nbdt)
    lwork = method.getWorkLengths(2)
    work = [npw.zeros((nb, nb)) for i in xrange(lwork)]
    for i in xrange(nbdt):
        # Euler
        dtE[i], errEx[i], errEy[i] = integrate2D(method(2, work, topo2, func2D,
                                                        optim=optim),
                                                 nbSteps)
        solver = odemethod(func2DODESPY)
        solver.set_initial_condition([1, 0])
        dtodespy[i], errxodespy[i], erryodespy[i] = odespy2D(solver, nbSteps)
        nbSteps += 10

    assert (dtE == dtodespy).all()

    x = np.log10(1. / dtE)
    plt.figure(2)
    plt.plot(x, np.log10(errEx), x, np.log10(errEy),
             x, np.log10(errxodespy), x, np.log10(errxodespy))
    name = str(method).rpartition('.')[-1][0:-2]
    print " ==== ", name, " ===="
    print "Erreur :", errEx, errEy
    print "Erreur (odespy) :", errxodespy, erryodespy
    print "dt : ", dtE
    print "Pente :",\
        (np.log10(errEx)[-1] - np.log10(errEx)[0]) / (x[-1] - x[0])
    print "Pente :",\
        (np.log10(errEy)[-1] - np.log10(errEy)[0]) / (x[-1] - x[0])
    print "Pente odespy: ",\
        (np.log10(errxodespy)[-1] - np.log10(errxodespy)[0]) / (x[-1] - x[0])
    print "Pente odespy: ",\
        (np.log10(erryodespy)[-1] - np.log10(erryodespy)[0]) / (x[-1] - x[0])

print "\n====================== 1D ======================\n"
#test_1D_0()
print "\n====================== 2D ======================\n"
optim = WITH_GUESS
test_2D_0(RK4, ode.RK4, optim)
