__kernel void initScalar(__global float* scalar,
			 float4 minPos,
			 float4 size,
			 float t)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  uint i;
  float4 pos, center=(float4)(0.35, 0.35, 0.35, 0.0);
  for(i=gidX; i<NB_X; i+=WI_NB)
    {
      pos = (float4)(i*size.x + minPos.x, gidY*size.y + minPos.y, gidZ*size.z + minPos.z, 0.0);
      scalar[i+gidY*NB_X+gidZ*NB_X*NB_Y] = ((distance(pos, center)<0.15) ? 1.0 : 0.0);
    }
}

#define PI (float) acos(-1.0)
__kernel void initVelocity(__global float* veloX,
			   __global float* veloY,
			   __global float* veloZ,
			   float4 minPos,
			   float4 size,
			   float t)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  uint i;
  float pix,piy,piz;
  /** Time dependant field */
  float time_term = cos(t*PI/3.0);
  /** Constant field
      float time_term = cos(t*PI/3.0); */
  piy = (minPos.y + gidY*size.y) * PI;
  piz = (minPos.z + gidZ*size.z) * PI;

  for(i=gidX; i<V_NB_X; i+=V_WI_NB)
    {
      pix = (minPos.x + i*size.x) * PI;

      veloX[i + gidY*V_NB_X + gidZ*V_NB_X*V_NB_Y] = 2.0 * sin(pix)*sin(pix)*sin(2.0*piy)*sin(2.0*piz)*time_term;
      veloY[gidY + i*V_NB_Y + gidZ*V_NB_Y*V_NB_X] = -sin(2.0*pix)*sin(piy)*sin(piy)*sin(2.0*piz)*time_term;
      veloZ[gidZ + i*V_NB_Z + gidY*V_NB_Z*V_NB_X] = -sin(2.0*pix)*sin(2.0*piy)*sin(piz)*sin(piz)*time_term;
    }
}

