#!/usr/bin/python

import parmepy as pp
import parmepy.f2py
import numpy as np
import mpi4py.MPI as MPI
import math
import numpy.fft as fft

pi = math.pi
ppfft = parmepy.f2py.fftw2py

rank = MPI.COMM_WORLD.Get_rank()
print "Mpi process number ", rank

# ----------- A 2d problem -----------
print " ========= Start test for 2D poisson problem ========="

# Physical Domain description
Lx = Ly = 2 * pi
myDomain2d = pp.Box(dimension=2, length=[Lx, Ly], origin=[0., 0.])
resolution2d = np.asarray((129, 65))
ncells = resolution2d - 1
hx = Lx / ncells[0]
hy = Ly / ncells[1]
x = np.arange(ncells[0], dtype='float64') * hx
y = np.arange(ncells[1], dtype='float64') * hy
Y, X = np.meshgrid(y, x)

############ REF ##############
omegaRef = np.zeros((ncells), dtype='complex128', order='Fortran')
omegaRef[:, :] = 4 * pi ** 2 * np.cos(2 * pi / Lx * X) * \
    np.sin(2. * pi / Ly * Y) * (1. / Lx ** 2 + 1. / Ly ** 2)
refx1 = 2. * pi / Ly * np.cos(2 * pi / Lx * X) * np.cos(2 * pi / Ly * Y)
refy1 = 2. * pi / Lx * np.sin(2 * pi / Lx * X) * np.sin(2 * pi / Ly * Y)
kx = np.arange(ncells[0], dtype='complex128')
ky = np.arange(ncells[1], dtype='complex128')
midIndX = math.ceil(ncells[0] / 2.)
midIndY = math.ceil(ncells[1] / 2.)
kx[-midIndX + 1:] = -kx[midIndX - 1:0:-1]
ky[-midIndY + 1:] = -ky[midIndY - 1:0:-1]
kx = 2 * pi * kx / Lx
ky = 2 * pi * ky / Ly

kxX, kyY = np.meshgrid(ky, kx)
coeff_tmp = kxX ** 2 + kyY ** 2
coeff = coeff_tmp.copy()
coeff[1:, :] = 1j / coeff_tmp[1:, :]
coeff[0, 1:] = 1j / coeff_tmp[0, 1:]
coeff[0, 0] = 0.0
res = fft.fft2(omegaRef)

tvx = (coeff.copy() * ky) * res
tvy = -(coeff.copy().T * kx).T * res

velx = fft.ifft2(tvx)
vely = fft.ifft2(tvy)

print "test 0", np.allclose(velx, refx1)
print "test 0", np.allclose(vely, refy1)

##########################

localres, localoffset = ppfft.init_fftw_solver(resolution2d,
                                               myDomain2d.length)

print "topo locale :", localres, localoffset


x = np.arange(localoffset[0], localres[0] + localoffset[0],
              dtype='float64') * hx
y = np.arange(localoffset[1], localres[1] + localoffset[1],
              dtype='float64') * hy
Y, X = np.meshgrid(y, x)

omega = np.zeros((localres), order='Fortran', dtype='float64')
vx = np.zeros((localres), order='Fortran', dtype='float64')
vy = np.zeros((localres), order='Fortran', dtype='float64')
omega[:, :] = 4 * pi ** 2 * np.cos(2 * pi / Lx * X) * \
    np.sin(2. * pi / Ly * Y) * (1. / Lx ** 2 + 1. / Ly ** 2)
refx = 2. * pi / Ly * np.cos(2 * pi / Lx * X) * np.cos(2 * pi / Ly * Y)
refy = 2. * pi / Lx * np.sin(2 * pi / Lx * X) * np.sin(2 * pi / Ly * Y)

vx, vy = ppfft.solve_poisson_2d(omega, vx, vy)

print np.allclose(refx, vx)
print np.allclose(refy, vy)

ppfft.clean_fftw_solver(myDomain2d.dimension)
