#!/usr/bin/python
import parmepy as pp
import parmepy.f2py
import numpy as np
import mpi4py.MPI as MPI
import math as m
from parmepy.mpi.topology import Cartesian

#from numpy import linalg as LA

pi = m.pi
ppfft = parmepy.f2py.fftw2py

rank = MPI.COMM_WORLD.Get_rank()
print "Mpi process number ", rank

# ----------- A 3d problem -----------
print " ========= Start test for 3D poisson problem ========="

# Physical Domain description
Lx = Ly = Lz = 2 * pi
myDomain3d = pp.Box(dimension=3, length=[Lx, Ly, Lz], origin=[0., 0., 0.])
resolution3d = np.asarray((65, 65, 65))
ncells = resolution3d - 1
hx = Lx / ncells[0]
hy = Ly / ncells[1]
hz = Lz / ncells[2]
print "INIT ...."
localres, localoffset = ppfft.init_fftw_solver(resolution3d,
                                               myDomain3d.length)

from parmepy.mpi import main_size
topodim = np.ones((3))
topodim[-1] = main_size
print topodim
print "make topo ..."
topoPoisson = Cartesian.withPrecomputedResolution(myDomain3d, topodim, resolution3d,
                                                  localres=localres, offset=localoffset)



for topo in myDomain3d.topologies.values():
    print topo

#print "topo locale :", localres, localoffset

#assert np.equal(localres, topoPoisson.localGridResolution).all()
#assert np.equal(localoffset,topoPoisson.G_start).all()

exit

x = np.arange(localoffset[0], localres[0] + localoffset[0],
              dtype='float64') * hx
y = np.arange(localoffset[1], localres[1] + localoffset[1],
              dtype='float64') * hy
z = np.arange(localoffset[2], localres[2] + localoffset[2],
              dtype='float64') * hz

############ REF ##############
omega_x = np.zeros((localres), dtype='float64', order='Fortran')
omega_y = np.zeros((localres), dtype='float64', order='Fortran')
omega_z = np.zeros((localres), dtype='float64', order='Fortran')
vx = np.zeros((localres), dtype='float64', order='Fortran')
vy = np.zeros((localres), dtype='float64', order='Fortran')
vz = np.zeros((localres), dtype='float64', order='Fortran')
ref_x = np.zeros((localres), dtype='float64', order='Fortran')
ref_y = np.zeros((localres), dtype='float64', order='Fortran')
ref_z = np.zeros((localres), dtype='float64', order='Fortran')

cden = 4 * pi ** 2 * (Ly ** 2 * Lz ** 2 + Lx ** 2 * Lz ** 2 +
                      Lx ** 2 * Ly ** 2) / (Lx ** 2 * Ly ** 2 * Lz ** 2)
cx = 2 * pi / Lx
cy = 2 * pi / Ly
cz = 2 * pi / Lz

# Initialize vorticity (This init should be improved or done if C or Fortran)
for k in range(localres[2]):
    for j in range(localres[1]):
        for i in range(localres[0]):
            omega_x[i, j, k] = cden * (m.sin(cx * x[i]) *
                                       m.sin(cy * y[j]) * m.cos(cz * z[k]))
            omega_y[i, j, k] = cden * (m.cos(cx * x[i]) *
                                       m.sin(cy * y[j]) * m.sin(cz * z[k]))
            omega_z[i, j, k] = cden * (m.cos(cx * x[i]) *
                                       m.cos(cy * y[j]) * m.sin(cz * z[k]))
            ref_x[i, j, k] = -cy * (m.cos(cx * x[i]) * m.sin(cy * y[j])
                                    * m.sin(cz * z[k])) - \
                cz * (m.cos(cx * x[i]) * m.sin(cy * y[j]) * m.cos(cz * z[k]))

            ref_y[i, j, k] = -cz * (m.sin(cx * x[i]) * m.sin(cy * y[j])
                                    * m.sin(cz * z[k])) + \
                cx * (m.sin(cx * x[i]) * m.cos(cy * y[j]) * m.sin(cz * z[k]))

            ref_z[i, j, k] = -cx * (m.sin(cx * x[i]) * m.sin(cy * y[j])
                                    * m.sin(cz * z[k])) - \
                cy * (m.sin(cx * x[i]) * m.cos(cy * y[j]) * m.cos(cz * z[k]))

##########################

print 'start fftw solver ...'
start = MPI.Wtime()
vx, vy, vz = ppfft.solve_poisson_3d(omega_x, omega_y, omega_z, vx, vy, vz)
print "elasped time =", MPI.Wtime() - start
print np.allclose(ref_x, vx)
print np.allclose(ref_y, vy)
print np.allclose(ref_z, vz)

ppfft.clean_fftw_solver(myDomain3d.dimension)
