#!/usr/bin/python
import parmepy as pp
from parmepy.operator.diffusion import Diffusion
from parmepy.operator.poisson import Poisson
from parmepy.operator.monitors.energy_enstrophy import Energy_enstrophy
from parmepy.problem.simulation import Simulation
import numpy as np
import math
pi = math.pi
sin = np.sin
cos = np.cos

## Physical Domain description
dim = 3
LL = 2 * pi * np.ones((dim))
dom = pp.Box(dimension=dim, length=LL)
resol = [65, 65, 65]


## Fields
def computeVel(x, y, z):
#    # --------Taylor Green------------
    vx = np.sin(x) * np.cos(y) * np.cos(z)
    vy = - np.cos(x) * np.sin(y) * np.cos(z)
    vz = 0.
    return vx, vy, vz

velocity = pp.Field(domain=dom, formula=computeVel,
                    isVector=True, name='Velocity')

# formula to compute initial vorticity field
coeff = 4 * pi ** 2 * (LL[1] ** 2 * LL[2] ** 2 + LL[0] ** 2 * LL[2] ** 2 +
                       LL[0] ** 2 * LL[1] ** 2) / (LL[0] ** 2 * LL[1] ** 2
                                                   * LL[2] ** 2)

cc = 2 * pi / LL


def computeVort(x, y, z):
    # --------Taylor Green------------
    wx = - np.cos(x) * np.sin(y) * np.sin(z)
    wy = - np.sin(x) * np.cos(y) * np.sin(z)
    wz = 2. * np.sin(x) * np.sin(y) * np.cos(z)
    return wx, wy, wz

vorticity = pp.Field(domain=dom, formula=computeVort,
                     name='Vorticity', isVector=True)


## Definition of the Diffusion and Poisson operators
output = 'energy.dat'
nu = 1e-3
finalTime = 1.0
dt = 0.01
restartTime = 0.5
filenameVel = 'dump_velo_T=_'
filenameVel += str(restartTime)

filenameVort = 'dump_vort_T=_'
filenameVort += str(restartTime)

diffusion = Diffusion(vorticity, resolution=resol, viscosity=nu)

poisson = Poisson(velocity, vorticity, resolutions={velocity: resol,
                                                    vorticity: resol})

## Definition of monitor
monitor = Energy_enstrophy(velocity, vorticity,
                           resolutions={velocity: resol,
                                        vorticity: resol},
                           viscosity=nu,
                           frequency=1,
                           prefix=output)

## Definition of simulation parameters
tini = restartTime
niter = (finalTime - tini) / dt
simu = Simulation(tinit=tini, tend=finalTime, nbiter=niter)

diffusion.setUp()
poisson.setUp()
monitor.setUp()

velocity.initialize()
vorticity.initialize()
print "pre ", vorticity.norm()
velocity.load('dump_velo_T=_0.5')
vorticity.load('dump_vort_T=_0.5')

while simu.time <= finalTime:
    print 'time, niter     : ', simu.time, simu.currentIteration
#    if (simu.currentIteration  == 50):
#        print 'dump fields ...'
#        velocity.dump(filenameVel)
#        vorticity.dump(filenameVort)
#        print "norm restart ", vorticity.norm()
    diffusion.apply(simu)
    poisson.apply(simu)
    monitor.apply(simu)
    simu.time += simu.timeStep
    simu.currentIteration += 1


print "post ", vorticity.norm()
