import numpy as np
from parmepy.domain.obstacle.controlBox import ControlBox
import parmepy as pp
import parmepy.mpi as mpi
from parmepy.domain.obstacle.planes import SubSpace, SubPlane
from parmepy.domain.obstacle.sphere import Sphere

from parmepy.operator.monitors.printer import Printer
from parmepy.problem.simulation import Simulation
from parmepy.constants import HDF5
from parmepy.operator.monitors.compute_forces import DragAndLift
pi = np.pi
cos = np.cos
sin = np.sin


nb = 33

Lx = Ly = Lz = 2
dom = pp.Box(dimension=3, length=[Lx, 2*Ly, 2*Lz], origin=[-1., 0., 3.])
dom2 = pp.Box(dimension=2, length=[Lx, Ly], origin=[-1., -1.])
resol3D = [17, 33, 45]
resol2D = [nb, nb]


## Function to compute TG velocity
def computeVel(res, x, y, z, t):
    res[0][...] = x # sin(x) * cos(y) * cos(z)
    res[1][...] = cos(y) #)- cos(x) * sin(y) * cos(z)
    res[2][...] = 0.
    return res


## Function to compute reference vorticity
def computeVort(res, x, y, z, t):
    res[0][...] = cos(x) * sin(y) * sin(z)
    res[1][...] = sin(x) * cos(y) * sin(z)
    res[2][...] = 2. * sin(x) * sin(y) * cos(z)
    return res

# 2D Field
scal2 = pp.Field(domain=dom2)

# 3D Field
scal3 = pp.Field(domain=dom, name='s1')
sc3 = pp.Field(domain=dom, name='s2')
velo = pp.Field(domain=dom, formula=computeVel, isVector=True, name='v1')
vorti = pp.Field(domain=dom, formula=computeVort, isVector=True, name='w1')

ng = 2
simulation = Simulation()

from parmepy.operator.poisson import Poisson
poisson = Poisson(velo, vorti, resolutions={velo: resol3D,
                                            vorti: resol3D})

poisson.discretize()

topofft = poisson.discreteFields[poisson.vorticity].topology

topo3 = mpi.topology.Cartesian(dom, 3, resol3D, ghosts=[ng, ng, ng])

pref3 = './res3_' + str(topo3.size) + '/v'

# init fields
scal3.discretize(topo3)
sc3.discretize(topo3)
velo.discretize(topo3)
vorti.discretize(topo3)
vorti.initialize(topo=topo3)
sd3 = scal3.discreteFields[topo3].data
wd = vorti.discreteFields[topo3].data

printer3D = Printer([velo], topo3, prefix=pref3, frequency=1)
printer3D.setUp()
printerHDF5 = Printer([velo], topo3, frequency=1, prefix=pref3,
                      formattype=HDF5)
printerHDF5.setUp()
preffft = './res3_fft' + str(topofft.size) + '/v'
printerFFT = Printer([velo], topofft, prefix=preffft, frequency=1)
printerFFT.setUp()

velo.initialize(topo=topofft)
velo.initialize(topo=topo3)

printerFFT.apply(simulation)
printer3D.apply(simulation)
printerHDF5.apply(simulation)
vd = velo.discreteFields[topo3].data
vdfft = velo.discreteFields[topofft].data


#for i in xrange(3):
#    print np.allclose(vd[i], vdfft[i])
