## Tests of the creation and initialisation of parmepy operators.
## This is to be include later into some 'non-regression' tests.
import parmepy as pp
import math
from parmepy.fields.continuous import Field
from parmepy.operator.analytic import Analytic
from parmepy.operator.stretching import Stretching
from parmepy.operator.advection import Advection
from parmepy.operator.poisson import Poisson
from parmepy.operator.redistribute import Redistribute
pi = math.pi


def vitesse(x, y, z, t, dt, ite):
    vx = x
    vy = y
    vz = z
    return vx, vy, vz


###
# Physical Domain description
Lx = Ly = Lz = 2 * pi
dom = pp.Box(dimension=3, length=[Lx, Ly, Lz])
resol1 = [9, 9, 9]
resol2 = [5, 5, 5]

## Fields
scal = Field(domain=dom, name='Scalar')
velo = Field(domain=dom, name='Velocity', isVector=True)
vort = Field(domain=dom, name='Vorticity', isVector=True)



## Analytic operator
vel = Analytic(velo, formula=vitesse,
               resolutions={velo: resol1})
vel.setUp()

## Advection operator
advec = Advection(velo, scal, resolutions={velo: resol1, scal: resol2},
                  method='scales')
advec.setUp()

## Poisson operator
poisson = Poisson(velo, vort, resolutions={velo: resol2, vort: resol2})
poisson.setUp()

## Stretching operator
stretch = Stretching(velo, vort, resolutions={velo: resol2, vort: resol2})
stretch.setUp()

## Data redistribution for velocity between Stretching and Advection
redistr = Redistribute(velo, stretch, advec)
redistr.setUp()



print scal
print velo

print velo.discreteFields.values()[0]
print velo.discreteFields.keys()[0]
