#!/usr/bin/python
import parmepy as pp
from parmepy.operator.poisson import Poisson
from parmepy.operator.analytic import Analytic
from parmepy.problem.problem import Problem
from parmepy.problem.navier_stokes import NSProblem
from parmepy.problem.simulation import Simulation
from parmepy.operator.monitors.printer import Printer

import numpy as np
import math
pi = math.pi
sin = np.sin
cos = np.cos

## Physical Domain description
dim = 3
LL = 2 * pi * np.ones((dim))
dom = pp.Box(dimension=dim, length=LL)
resol = [65, 65, 65]

## Fields
velocity = pp.Field(domain=dom, isVector=True, name='Velocity')

# formula to compute initial vorticity field
coeff = 4 * pi ** 2 * (LL[1] ** 2 * LL[2] ** 2 + LL[0] ** 2 * LL[2] ** 2 +
                       LL[0] ** 2 * LL[1] ** 2) / (LL[0] ** 2 * LL[1] ** 2
                                                   * LL[2] ** 2)

cc = 2 * pi / LL


def computeVort(x, y, z):
    wx = coeff * sin(x * cc[0]) * sin(y * cc[1]) * cos(z * cc[2])
    wy = coeff * cos(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])
    wz = coeff * cos(x * cc[0]) * cos(y * cc[1]) * sin(z * cc[2])
    return wx, wy, wz

vorticity = pp.Field(domain=dom, formula=computeVort,
                     name='Vorticity', isVector=True)


# ref. field
def computeRef(x, y, z, t):
    refx = -2. * pi / LL[1] * \
        (cos(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        - 2. * pi / LL[2] * (cos(x * cc[0]) * sin(y * cc[1]) * cos(z * cc[2]))

    refy = -2. * pi / LL[2] * \
        (sin(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        + 2. * pi / LL[0] * (sin(x * cc[0]) * cos(y * cc[1]) * sin(z * cc[2]))

    refz = -2. * pi / LL[0] * \
        (sin(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        - 2. * pi / LL[1] * (sin(x * cc[0]) * cos(y * cc[1]) * cos(z * cc[2]))

    return refx, refy, refz


## Definition of the Poisson operator
poisson = Poisson(velocity, vorticity,
                  resolutions={velocity: resol, vorticity: resol})

## Analytic operator to compute the reference field
ref = pp.Field(domain=dom, name='reference', isVector=True)
refOp = Analytic(ref, formula=computeRef,
                 resolutions={ref: resol})

timeStep = 0.25
simu = Simulation()
#simu.iterMax = simu.iterMax - 2
refOp.discretize()
poisson.discretize()
refOp.setUp()
poisson.setUp()
refOp.apply(simu)

#pb = NSProblem([refOp, poisson], simu,
#               monitors=[Printer(fields=[velocity],
#                                 frequency=1)],
#               dumpFreq=2, name='toto')

#pb.setUp()

#pb.solve()

## refOp.apply(0, 0, 0)

vorticity.initialize()
## print vorticity.norm()
poisson.apply(simu)
## print velocity.norm()
## print ref.norm()


refD = ref.discreteFields.values()[0]
vd = velocity.discreteFields.values()[0]

assert np.allclose(ref.norm(), velocity.norm())
for i in range(dom.dimension):
    assert np.allclose(vd[i], refD[i])

print velocity.norm()

poisson.finalize()

#print poisson
