#===========================================================
# Macro to install parmes package
#
# F. Pérignon, LJK-CNRS, april 2011
#
#===========================================================

macro(install_package)
  
  if(ARGV2)
    set(_HEADERS ${ARGV2})
  endif()

  # Offer the user the choice of overriding the installation directories
  set(INSTALL_LIB_DIR lib CACHE PATH "Installation directory for libraries")
  set(INSTALL_BIN_DIR bin CACHE PATH "Installation directory for executables")
  set(INSTALL_INCLUDE_DIR include CACHE PATH "Installation directory for header files")
  set(INSTALL_DATA_DIR share CACHE PATH "Installation directory for data files")

  # Make relative paths absolute (needed later on)
  foreach(p LIB BIN INCLUDE DATA)
    set(var INSTALL_${p}_DIR)
    if(NOT IS_ABSOLUTE "${${var}}")
      set(${var} "${CMAKE_INSTALL_PREFIX}/${${var}}")
    endif()
  endforeach()
  
  # The library
  install(TARGETS ${ARGV1} 
    EXPORT ${ARGV0}LibraryDepends
    ARCHIVE DESTINATION "${INSTALL_LIB_DIR}"  # static libs
    LIBRARY DESTINATION "${INSTALL_LIB_DIR}"  COMPONENT shlib # shared libs
    PUBLIC_HEADER DESTINATION "${INSTALL_INCLUDE_DIR}" COMPONENT dev
    )
  
  # The headers and modules
  if(_HEADERS)
    install(FILES ${_HEADERS} DESTINATION "${INSTALL_INCLUDE_DIR}")
  endif() 
  install(DIRECTORY ${CMAKE_BINARY_DIR}/Modules DESTINATION "${INSTALL_INCLUDE_DIR}")
  
  export(TARGETS ${ARGV1} FILE "${PROJECT_BINARY_DIR}/InstallFiles/${ARGV0}LibraryDepends.cmake")
  
  # Install the export set for use with the install-tree
  install(EXPORT ${ARGV0}LibraryDepends DESTINATION
    "${INSTALL_DATA_DIR}/CMake" COMPONENT dev)
  
  set(${ARGV0}_INCLUDE_DIRS "${INSTALL_INCLUDE_DIR}")
  set(${ARGV0}_LIB_DIR "${INSTALL_LIB_DIR}")
  set(${ARGV0}_CMAKE_DIR "${INSTALL_DATA_DIR}/CMake")

  display(${ARGV0}_CMAKE_DIR)
  configure_file(${CMAKE_SOURCE_DIR}/${ARGV0}Config.cmake.in
    "${PROJECT_BINARY_DIR}/InstallFiles/${ARGV0}Config.cmake")
  configure_file(${CMAKE_SOURCE_DIR}/${ARGV0}ConfigVersion.cmake.in
    "${PROJECT_BINARY_DIR}/InstallFiles/${ARGV0}ConfigVersion.cmake" @ONLY)
  install(FILES
    "${PROJECT_BINARY_DIR}/InstallFiles/${ARGV0}Config.cmake"
    "${PROJECT_BINARY_DIR}/InstallFiles/${ARGV0}ConfigVersion.cmake"
    DESTINATION "${${ARGV0}_CMAKE_DIR}" COMPONENT dev)
  
endmacro()
