"""
@file default_methods.py
Default parameter values for methods in operators.
"""
from parmepy.methods_keys import TimeIntegrator, Interpolation, GhostUpdate,\
    Remesh, Support, Splitting, MultiScale, Formulation, SpaceDiscretisation, \
    dtCrit
from parmepy.numerics.integrators.runge_kutta2 import RK2
from parmepy.numerics.integrators.runge_kutta3 import RK3
from parmepy.numerics.interpolation import Linear
from parmepy.numerics.remeshing import L2_1
#from parmepy.operator.discrete.stretching import Conservative


ADVECTION = {TimeIntegrator: RK2, Interpolation: Linear,
             Remesh: L2_1, Support: '', Splitting: 'o2', MultiScale: L2_1}

from parmepy.numerics.finite_differences import FD_C_4

DIFFERENTIAL = {SpaceDiscretisation: FD_C_4, GhostUpdate: True}

ADAPT_TIME_STEP = {TimeIntegrator: RK3, SpaceDiscretisation: FD_C_4,
                   dtCrit: 'vort'}

BAROCLINIC = {SpaceDiscretisation: FD_C_4}

DIFFUSION = {SpaceDiscretisation: 'fftw', GhostUpdate: True}

POISSON = {SpaceDiscretisation: 'fftw', GhostUpdate: True}

STRETCHING = {TimeIntegrator: RK3, Formulation: "Conservative",
              SpaceDiscretisation: FD_C_4}
