void fill_velocity_cache(__global const float* gvelo,
			 uint gidX, uint gidY, uint gidZ,
			 float4 dx, float4 v_dx,
			 __local float* gvelo_loc);


inline float alpha_l2_1(float y){
  return ((y * (y * (-y + 2.0) - 1.0)) / 2.0);}
inline float beta_l2_1(float y){
  return ((y * y * (3.0 * y - 5.0) + 2.0) / 2.0);}
inline float gamma_l2_1(float y){
  return ((y * (y * (-3.0 * y + 4.0) + 1.0)) / 2.0);}
inline float delta_l2_1(float y){
  return ((y * y * (y - 1.0)) / 2.0);}


inline float alpha_l4_2(float y){
  return ((y * (y * (y * (y * (-5.0 * y + 13.0) - 9.0) - 1.0) + 2.0)) / 24.0);}
inline float beta_l4_2(float y){
  return ((y * (y * (y * (y * (25.0 * y - 64.0) + 39.0) + 16.0) - 16.0)) / 24.0);}
inline float gamma_l4_2(float y){
  return ((y * y * (y * (y * (-50.0 * y + 126.0) - 70.0) - 30.0) + 24.0) / 24.0);}
inline float delta_l4_2(float y){
  return ((y * (y * (y * (y * (50.0 * y - 124.0) + 66.0) + 16.0) + 16.0)) / 24.0);}
inline float eta_l4_2(float y){
  return ((y * (y * (y * (y * (-25.0 * y + 61.0) - 33.0) - 1.0) - 2.0)) / 24.0);}
inline float zeta_l4_2(float y){
  return ((y * y * y * (y * (5.0 * y - 12.0) + 7.0)) / 24.0);}


inline float alpha_l4_4(float y){
  return ((y * (y * (y * (y * (y * (y * (y * (y * (-46.0 * y + 207.0) - 354.0) + 273.0) - 80.0) + 1.0) - 2.0) - 1.0) + 2.0)) / 24.0);}
inline float beta_l4_4(float y){
  return ((y * (y * (y * (y * (y * (y * (y * (y * (230.0 * y - 1035.0) + 1770.0) - 1365.0) + 400.0) - 4.0) + 4.0) + 16.0) - 16.0)) / 24.0);}
inline float gamma_l4_4(float y){
  return ((y * y * (y * y * (y * (y * (y * (y * (-460.0 * y + 2070.0) - 3540.0) + 2730.0) - 800.0) + 6.0) - 30.0) + 24.0) / 24.0);}
inline float delta_l4_4(float y){
  return ((y * (y * (y * (y * (y * (y * (y * (y * (460.0 * y - 2070.0) + 3540.0) - 2730.0) + 800.0) - 4.0) - 4.0) + 16.0) + 16.0)) / 24.0);}
inline float eta_l4_4(float y){
  return ((y * (y * (y * (y * (y * (y * (y * (y * (-230.0 * y + 1035.0) - 1770.0) + 1365.0) - 400.0) + 1.0) + 2.0) - 1.0) - 2.0)) / 24.0);}
inline float zeta_l4_4(float y){
  return ((y * y * y * y * y * (y * (y * (y * (46.0 * y - 207.0) + 354.0) - 273.0) + 80.0)) / 24.0);}

/*** TODO: correct this file to work properly with vector enable remehsing weights ***/

void fill_velocity_cache(__global const float* gvelo,
			 uint gidX, uint gidY, uint gidZ,
			 float4 dx, float4 v_dx,
			 __local float* velocity_cache)
{
  uint i;
  float__N__ v;
#if V_NB_I == NB_I
  // Single scale : Velocity and scalar grids are identical : cache is just read from global
  uint line_index = gidY*NB_I + gidZ*NB_I*NB_II; /* Current 1D problem index */
  for(i=gidX*__N__; i<NB_I; i+=(WI_NB*__N__))
    {
      /* Read velocity */
      v = vload__N__((i+line_index)/__N__, gvelo);
      /* Fill the cache */
      velocity_cache[noBC_id(i+__NN__)] = v.s__NN__;
    }
#else
  // Multi-scale: Velocity cache is interpolated from global

#if NB_III == 1
  // 2D case


  float line_posY, hY;
  int indY;
#if MS_FORMULA == MS_LINEAR
  int2 v_line_index;
  float2 wY;
#elif MS_FORMULA == MS_L2_1
  int4 v_line_index;
  float4 wY;
#elif MS_FORMULA == MS_L4_2 ||  MS_FORMULA == MS_L4_4
  // Only the 6 first elements will be used
  int8 v_line_index;
  float8 wY;
#endif

  line_posY = (gidY * dx.y) / v_dx.y;
  indY = convert_int_rtn(line_posY);
  hY = line_posY - convert_float(indY);

#if MS_FORMULA == MS_LINEAR
  wY.s1 = hY;
  wY.s0 = 1.0 - wY.s1;
#elif MS_FORMULA == MS_L2_1
  wY.s0 = alpha_l2_1(hY);
  wY.s1 = beta_l2_1(hY);
  wY.s2 = gamma_l2_1(hY);
  wY.s3 = 1.0 - wY.s0 - wY.s1 - wY.s2;
#elif MS_FORMULA == MS_L4_2
  wY.s0 = alpha_l4_2(hY);
  wY.s1 = beta_l4_2(hY);
  wY.s2 = gamma_l4_2(hY);
  wY.s3 = delta_l4_2(hY);
  wY.s4 = eta_l4_2(hY);
  wY.s5 = 1.0 - wY.s0 - wY.s1 - wY.s2 - wY.s3 - wY.s4;
#elif MS_FORMULA == MS_l4_4
  wY.s0 = alpha_l4_4(hY);
  wY.s1 = beta_l4_4(hY);
  wY.s2 = gamma_l4_4(hY);
  wY.s3 = delta_l4_4(hY);
  wY.s4 = eta_l4_4(hY);
  wY.s5 = 1.0 - wY.s0 - wY.s1 - wY.s2 - wY.s3 - wY.s4;
#endif

  indY = indY + V_GHOSTS_NB;

#if MS_FORMULA == MS_LINEAR
  v_line_index.s0 = indY * V_NB_I;
  v_line_index.s1 = (indY + 1) * V_NB_I;
#elif MS_FORMULA == MS_L2_1
  v_line_index.s0 = (indY - 1) * V_NB_I;
  v_line_index.s1 = (indY) * V_NB_I;
  v_line_index.s2 = (indY + 1) * V_NB_I;
  v_line_index.s3 = (indY + 2) * V_NB_I;
#elif MS_FORMULA == MS_L4_2 || MS_FORMULA == MS_L4_4
  v_line_index.s0 = (indY - 2) * V_NB_I;
  v_line_index.s1 = (indY - 1) * V_NB_I;
  v_line_index.s2 = (indY) * V_NB_I;
  v_line_index.s3 = (indY + 1) * V_NB_I;
  v_line_index.s4 = (indY + 2) * V_NB_I;
  v_line_index.s5 = (indY + 3) * V_NB_I;
#endif


  for(i=gidX*__N__; i<V_NB_I; i+=(WI_NB*__N__))
    {
#if MS_FORMULA == MS_LINEAR
      v = vload__N__((i+v_line_index.s0)/__N__, gvelo) * wY.s0;
      velocity_cache[noBC_id(i+__NN__)] = v.s__NN__;
      v = vload__N__((i+v_line_index.s1)/__N__, gvelo) * wY.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
#elif MS_FORMULA == MS_L2_1
      v = vload__N__((i+v_line_index.s0)/__N__, gvelo) * wY.s0;
      velocity_cache[noBC_id(i+__NN__)] = v.s__NN__;
      v = vload__N__((i+v_line_index.s1)/__N__, gvelo) * wY.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i+v_line_index.s2)/__N__, gvelo) * wY.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i+v_line_index.s3)/__N__, gvelo) * wY.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
#elif MS_FORMULA == MS_L4_2 || MS_FORMULA == MS_L4_4
      v = vload__N__((i+v_line_index.s0)/__N__, gvelo) * wY.s0;
      velocity_cache[noBC_id(i+__NN__)] = v.s__NN__;
      v = vload__N__((i+v_line_index.s1)/__N__, gvelo) * wY.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i+v_line_index.s2)/__N__, gvelo) * wY.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i+v_line_index.s3)/__N__, gvelo) * wY.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i+v_line_index.s4)/__N__, gvelo) * wY.s4;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i+v_line_index.s5)/__N__, gvelo) * wY.s5;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
#endif
    }

#else
  // 3D case


  float line_posY, hY;
  float line_posZ, hZ;
  int indY, indZ;
#if MS_FORMULA == MS_LINEAR
  int2 v_line_indexY, v_line_indexZ;
  float2 wY, wZ;
#elif MS_FORMULA == MS_L2_1
  int4 v_line_indexY, v_line_indexZ;
  float4 wY, wZ;
#elif MS_FORMULA == MS_L4_2 || MS_FORMULA == MS_L4_4
  int8 v_line_indexY, v_line_indexZ;
  float8 wY, wZ;
#endif

  line_posY = (gidY * dx.y) / v_dx.y;
  line_posZ = (gidZ * dx.z) / v_dx.z;
  indY = convert_int_rtn(line_posY);
  indZ = convert_int_rtn(line_posZ);
  hY = line_posY - convert_float(indY);
  hZ = line_posZ - convert_float(indZ);

#if MS_FORMULA == MS_LINEAR
  wY.s1 = hY;
  wY.s0 = 1.0 - wY.s1;
  wZ.s1 = hZ;
  wZ.s0 = 1.0 - wZ.s1;
#elif MS_FORMULA == MS_L2_1
  wY.s0 = alpha_l2_1(hY);
  wY.s1 = beta_l2_1(hY);
  wY.s2 = gamma_l2_1(hY);
  wY.s3 = 1.0 - wY.s0 - wY.s1 - wY.s2;
  wZ.s0 = alpha_l2_1(hZ);
  wZ.s1 = beta_l2_1(hZ);
  wZ.s2 = gamma_l2_1(hZ);
  wZ.s3 = 1.0 - wZ.s0 - wZ.s1 - wZ.s2;
#elif MS_FORMULA == MS_L4_2
  wY.s0 = alpha_l4_2(hY);
  wY.s1 = beta_l4_2(hY);
  wY.s2 = gamma_l4_2(hY);
  wY.s3 = delta_l4_2(hY);
  wY.s4 = eta_l4_2(hY);
  wY.s5 = 1.0 - wY.s0 - wY.s1 - wY.s2 - wY.s3 - wY.s4;
  wZ.s0 = alpha_l4_2(hZ);
  wZ.s1 = beta_l4_2(hZ);
  wZ.s2 = gamma_l4_2(hZ);
  wZ.s3 = delta_l4_2(hZ);
  wZ.s4 = eta_l4_2(hZ);
  wZ.s5 = 1.0 - wZ.s0 - wZ.s1 - wZ.s2 - wZ.s3 - wZ.s4;
#elif MS_FORMULA == MS_L4_4
  wY.s0 = alpha_l4_4(hY);
  wY.s1 = beta_l4_4(hY);
  wY.s2 = gamma_l4_4(hY);
  wY.s3 = delta_l4_4(hY);
  wY.s4 = eta_l4_4(hY);
  wY.s5 = 1.0 - wY.s0 - wY.s1 - wY.s2 - wY.s3 - wY.s4;
  wZ.s0 = alpha_l4_4(hZ);
  wZ.s1 = beta_l4_4(hZ);
  wZ.s2 = gamma_l4_4(hZ);
  wZ.s3 = delta_l4_4(hZ);
  wZ.s4 = eta_l4_4(hZ);
  wZ.s5 = 1.0 - wZ.s0 - wZ.s1 - wZ.s2 - wZ.s3 - wZ.s4;
#endif

  indY = indY + V_GHOSTS_NB;
  indZ = indZ + V_GHOSTS_NB;

#if MS_FORMULA == MS_LINEAR
  v_line_indexY.s0 = indY * V_NB_I;
  v_line_indexY.s1 = (indY + 1) * V_NB_I;
  v_line_indexZ.s0 = indZ * V_NB_I * V_NB_II;
  v_line_indexZ.s1 = (indZ + 1) * V_NB_I * V_NB_II;
#elif MS_FORMULA == MS_L2_1
  v_line_indexY.s0 = (indY - 1) * V_NB_I;
  v_line_indexY.s1 = (indY) * V_NB_I;
  v_line_indexY.s2 = (indY + 1) * V_NB_I;
  v_line_indexY.s3 = (indY + 2) * V_NB_I;
  v_line_indexZ.s0 = (indZ - 1) * V_NB_I * V_NB_II;
  v_line_indexZ.s1 = (indZ) * V_NB_I * V_NB_II;
  v_line_indexZ.s2 = (indZ + 1) * V_NB_I * V_NB_II;
  v_line_indexZ.s3 = (indZ + 2) * V_NB_I * V_NB_II;
#elif MS_FORMULA == MS_L4_2 || MS_FORMULA == MS_L4_4
  v_line_indexY.s0 = (indY - 2) * V_NB_I;
  v_line_indexY.s1 = (indY - 1) * V_NB_I;
  v_line_indexY.s2 = (indY) * V_NB_I;
  v_line_indexY.s3 = (indY + 1) * V_NB_I;
  v_line_indexY.s4 = (indY + 2) * V_NB_I;
  v_line_indexY.s5 = (indY + 3) * V_NB_I;
  v_line_indexZ.s0 = (indZ - 2) * V_NB_I * V_NB_II;
  v_line_indexZ.s1 = (indZ - 1) * V_NB_I * V_NB_II;
  v_line_indexZ.s2 = (indZ) * V_NB_I * V_NB_II;
  v_line_indexZ.s3 = (indZ + 1) * V_NB_I * V_NB_II;
  v_line_indexZ.s4 = (indZ + 2) * V_NB_I * V_NB_II;
  v_line_indexZ.s5 = (indZ + 3) * V_NB_I * V_NB_II;
#endif


  for(i=gidX*__N__; i<V_NB_I; i+=(WI_NB*__N__))
    {
#if MS_FORMULA == MS_LINEAR
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s0)/__N__, gvelo) * wY.s0 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] = v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s1)/__N__, gvelo) * wY.s0 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s0)/__N__, gvelo) * wY.s1 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s1)/__N__, gvelo) * wY.s1 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

#elif MS_FORMULA == MS_L2_1
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s0)/__N__, gvelo) * wY.s0 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] = v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s1)/__N__, gvelo) * wY.s0 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s2)/__N__, gvelo) * wY.s0 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s3)/__N__, gvelo) * wY.s0 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s0)/__N__, gvelo) * wY.s1 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s1)/__N__, gvelo) * wY.s1 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s2)/__N__, gvelo) * wY.s1 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s3)/__N__, gvelo) * wY.s1 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s0)/__N__, gvelo) * wY.s2 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s1)/__N__, gvelo) * wY.s2 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s2)/__N__, gvelo) * wY.s2 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s3)/__N__, gvelo) * wY.s2 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s0)/__N__, gvelo) * wY.s3 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s1)/__N__, gvelo) * wY.s3 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s2)/__N__, gvelo) * wY.s3 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s3)/__N__, gvelo) * wY.s3 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

#elif MS_FORMULA == MS_L4_2 || MS_FORMULA == MS_L4_4
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s0)/__N__, gvelo) * wY.s0 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] = v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s1)/__N__, gvelo) * wY.s0 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s2)/__N__, gvelo) * wY.s0 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s3)/__N__, gvelo) * wY.s0 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s4)/__N__, gvelo) * wY.s0 * wZ.s4;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s0 + v_line_indexZ.s5)/__N__, gvelo) * wY.s0 * wZ.s5;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s0)/__N__, gvelo) * wY.s1 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s1)/__N__, gvelo) * wY.s1 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s2)/__N__, gvelo) * wY.s1 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s3)/__N__, gvelo) * wY.s1 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s4)/__N__, gvelo) * wY.s1 * wZ.s4;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s1 + v_line_indexZ.s5)/__N__, gvelo) * wY.s1 * wZ.s5;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s0)/__N__, gvelo) * wY.s2 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s1)/__N__, gvelo) * wY.s2 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s2)/__N__, gvelo) * wY.s2 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s3)/__N__, gvelo) * wY.s2 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s4)/__N__, gvelo) * wY.s2 * wZ.s4;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s2 + v_line_indexZ.s5)/__N__, gvelo) * wY.s2 * wZ.s5;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s0)/__N__, gvelo) * wY.s3 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s1)/__N__, gvelo) * wY.s3 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s2)/__N__, gvelo) * wY.s3 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s3)/__N__, gvelo) * wY.s3 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s4)/__N__, gvelo) * wY.s3 * wZ.s4;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s3 + v_line_indexZ.s5)/__N__, gvelo) * wY.s3 * wZ.s5;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s4 + v_line_indexZ.s0)/__N__, gvelo) * wY.s4 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s4 + v_line_indexZ.s1)/__N__, gvelo) * wY.s4 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s4 + v_line_indexZ.s2)/__N__, gvelo) * wY.s4 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s4 + v_line_indexZ.s3)/__N__, gvelo) * wY.s4 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s4 + v_line_indexZ.s4)/__N__, gvelo) * wY.s4 * wZ.s4;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s4 + v_line_indexZ.s5)/__N__, gvelo) * wY.s4 * wZ.s5;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;

      v = vload__N__((i + v_line_indexY.s5 + v_line_indexZ.s0)/__N__, gvelo) * wY.s5 * wZ.s0;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s5 + v_line_indexZ.s1)/__N__, gvelo) * wY.s5 * wZ.s1;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s5 + v_line_indexZ.s2)/__N__, gvelo) * wY.s5 * wZ.s2;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s5 + v_line_indexZ.s3)/__N__, gvelo) * wY.s5 * wZ.s3;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s5 + v_line_indexZ.s4)/__N__, gvelo) * wY.s5 * wZ.s4;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
      v = vload__N__((i + v_line_indexY.s5 + v_line_indexZ.s5)/__N__, gvelo) * wY.s5 * wZ.s5;
      velocity_cache[noBC_id(i+__NN__)] += v.s__NN__;
#endif
    }
#endif
#endif
}
