"""
@package parmepy.mpi
Parmes interface to the mpi implementation.

It contains :
- mpi basic variables (main communicator, rank, size ...)
- topology.Cartesian : mpi process distribution + local mesh
- mesh.SubMesh : local mesh


This package is used to hide the underlying mpi interface
in order to make any change of this interface, if required, easiest.
At the time we use mpi4py : http://mpi4py.scipy.org


"""

## Everything concerning the chosen mpi implementation is hidden in main_var
# Why? --> to avoid that things like mpi4py. ... spread everywhere in the
# soft so to ease a change of this implementation (if needed).
import main_var

## A list of mpi variables that can be "seen" by user
##  MPI underlying implementation
MPI = main_var.MPI

## Main communicator (copy of comm_world)
main_comm = main_var.main_comm

## Rank of the current process in the main communicator
main_rank = main_var.main_rank

## Size of the main communicator
main_size = main_var.main_size
