"""
@file parmepy.mpi.tests.test_mesh
Testing mesh.
"""
import numpy as np
from parmepy.domain.box import Box
from parmepy.mpi.topology import Cartesian


def test_mesh3D():
    """Periodic mesh"""
    dom = Box()
    resolTopo = [33, 33, 17]
    topo = Cartesian(dom, 3, resolTopo)
    topo.setUp()
    dx = [1. / (n - 1) for n in resolTopo]
    assert (topo.mesh.origin == [0. for n in resolTopo]).all()
    assert (topo.mesh.end == [1. - dxx for dxx in dx]).all()
    assert (topo.mesh.global_start == [0 for n in resolTopo]).all()
    assert (topo.mesh.global_end == [n - 2 for n in resolTopo]).all()
    assert (topo.mesh.local_start == [0 for n in resolTopo]).all()
    assert (topo.mesh.local_end == [n - 2 for n in resolTopo]).all()
    assert len(topo.mesh.coords) == 3
    assert topo.mesh.coords[0].shape == (resolTopo[0] - 1, 1, 1)
    assert topo.mesh.coords[1].shape == (1, resolTopo[1] - 1, 1)
    assert topo.mesh.coords[2].shape == (1, 1, resolTopo[2] - 1)
    assert topo.mesh.coords[0][1, 0, 0] == dx[0]
    assert topo.mesh.coords[1][0, 1, 0] == dx[1]
    assert topo.mesh.coords[2][0, 0, 1] == dx[2]


def test_mesh3D_ghost():
    """Periodic mesh"""
    dom = Box()
    resolTopo = [33, 33, 17]
    dx = [1. / (n - 1) for n in resolTopo]
    ghost = np.array([2, 2, 1])
    topo = Cartesian(dom, 3, resolTopo, ghosts=ghost)
    topo.setUp()
    assert (topo.mesh.origin ==
            [0. - g * dxx for dxx, g in zip(dx, ghost)]).all()
    assert (topo.mesh.end ==
            [1. - dxx + g * dxx for dxx, g in zip(dx, ghost)]).all()
    assert (topo.mesh.global_start ==
            [0 for n in resolTopo]).all()
    assert (topo.mesh.global_end == [n - 2 for n in resolTopo]).all()
    assert (topo.mesh.local_start == [g for g in ghost]).all()
    assert (topo.mesh.local_end ==
            [n - 2 + g for n, g in zip(resolTopo, ghost)]).all()
    assert len(topo.mesh.coords) == 3
    assert topo.mesh.coords[0].shape == (resolTopo[0] - 1 + 2 * ghost[0], 1, 1)
    assert topo.mesh.coords[1].shape == (1, resolTopo[1] - 1 + 2 * ghost[1], 1)
    assert topo.mesh.coords[2].shape == (1, 1, resolTopo[2] - 1 + 2 * ghost[2])
    assert topo.mesh.coords[0][1, 0, 0] == (-ghost[0] + 1) * dx[0]
    assert topo.mesh.coords[1][0, 1, 0] == (-ghost[1] + 1) * dx[1]
    assert topo.mesh.coords[2][0, 0, 1] == (-ghost[2] + 1) * dx[2]

# Todo : update tests for multi-proc mpi runs.
#if __name__ == '__main__':
#    test_mesh3D()
#    test_mesh3D_ghost()
