import parmepy as pp
from parmepy.domain.box import Box
from parmepy.mpi.topology import Cartesian
from parmepy.constants import PARMES_INDEX


def test_create_topology1D():
    dom = Box()
    resolTopo = [33, 33, 17]
    topo = Cartesian(dom, 1, resolTopo)
    topo.setUp()

    assert topo.domain == dom
    assert topo.size == pp.mpi.main_size


def test_create_topology2D():
    dom = Box()
    resolTopo = [33, 33, 17]
    topo = Cartesian(dom, 2, resolTopo)
    topo.setUp()

    assert topo.domain == dom
    assert topo.size == pp.mpi.main_size


def test_create_topology3D():
    dom = Box()
    resolTopo = [33, 33, 17]
    topo = Cartesian(dom, 3, resolTopo)
    topo.setUp()

    assert topo.domain == dom
    assert topo.size == pp.mpi.main_size


def test_create_topology_with_dims():
    dom = Box()
    resol = [33, 33, 17]
    import numpy as np
    topoDims = np.asarray([pp.mpi.main_size, 1, 1])
    topo = Cartesian.withResolution(dom, shape=topoDims,
                                    globalMeshResolution=resol)
    topo.setUp()
    assert topo.domain == dom
    assert topo.dim == 1
    assert topo.size == pp.mpi.main_size
    testList = topo.shape[topo.shape != 1] == topoDims[topoDims != 1]
    assert testList.all()


def test_operator_equal():
    dom = Box()
    resolTopo = [33, 33, 17]
    topoDims = [pp.mpi.main_size, 1, 1]
    topo = Cartesian.withResolution(dom, topoDims, resolTopo)
    topo2 = Cartesian(Box(), 2, resolTopo)
    assert not topo == topo2
    topo2 = Cartesian(dom, 2, [11, 33, 17])
    assert not topo == topo2

    topo2 = Cartesian(dom, 2, shape=topoDims[-1::-1],
                      globalMeshResolution=resolTopo)
    assert topo == topo2


def test_operator_notequal():
    dom = Box()
    resol1 = [33, 33, 17]
    resol2 = [33, 33, 12]
    topo = Cartesian(dom, 3, resol1)
    topo2 = Cartesian(dom, 3, resol2)
    assert topo != topo2


def test_bridge2D():
    import math
    Lx = Ly = 2 * math.pi
    from parmepy.mpi.topology import Bridge
    dom = Box(dimension=2, length=[Lx, Ly], origin=[0., 0.])
    resolution = [65, 65]
    topo2 = Cartesian(dom, 2, resolution)
    topo2.setUp()
    topo1 = Cartesian(dom, 1, resolution)
    topo1.setUp()
    bridge = Bridge(topo1, topo2)
    print (bridge)
    # We cannot really check something interesting,
    # so we just create a bridge.


def test_bridge3D():
    import numpy as np
    from parmepy.mpi.topology import Bridge
    dom = Box()
    resolution3d = np.asarray((65, 65, 65), dtype=PARMES_INDEX)
    topo1 = Cartesian(dom, 3, resolution3d)
    topo1.setUp()
    topo2 = Cartesian(dom, 1, resolution3d)
    topo2.setUp()
    bridge = Bridge(topo1, topo2)
    print (bridge)
    # We cannot really check something interesting,
    # so we just create a bridge.


if __name__ == "__main__":
    test_create_topology1D()
    test_create_topology2D()
    test_create_topology3D()
    test_create_topology_with_dims()
    test_operator_equal()
    test_operator_notequal()
    test_bridge2D()
    test_bridge3D()
    #test_create_topology1D()
    #pp.mpi.main_comm.barrier()

    ## test_create_topology2D()
    ## pp.mpi.main_comm.barrier()
