# -*- coding: utf-8 -*-
"""
@file operator/density.py

"""
from parmepy.operator.continuous import Operator
from parmepy.operator.discrete.density import DensityVisco_d
from parmepy.constants import debug


class DensityVisco(Operator):
    """
    Density and Viscosity reconstruction : operator representation
    """

    @debug
    def __init__(self, density, viscosity,
                 resolutions=None, method=None,
                 task_id=None, comm=None,
                 **other_config):
        """
        Constructor.
        Reconstruct the Density and the Viscosity scalars.

        @param velocity ContinuousVectorField : velocity variable.
        """
        Operator.__init__(self, [density, viscosity], method,
                          name="DensityVisco", task_id=task_id, comm=comm)

        self.density = density
        self.viscosity = viscosity
        self.resolutions = resolutions
        self.config = other_config
        self.input = [self.density, self.viscosity]
        self.output = [self.density, self.viscosity]

    def discretize(self):
        # Variables discretization
        for v in self.variables:
            # the topology for v ...
            topo = self.domain.getOrCreateTopology(self.domain.dimension,
                                                   self.resolutions[v],
                                                   comm=self._comm)
            # ... and the corresponding discrete field
            self.discreteFields[v] = v.discretize(topo)

    @debug
    def setUp(self):
        """
        Density and Viscosity reconstruction operator discretization method.
        Create a discrete operator from given specifications.
        """

        self.discreteOperator = \
            DensityVisco_d(self.discreteFields[self.density],
                           self.discreteFields[self.viscosity],
                           method=self.method, **self.config)

        self.discreteOperator.setUp()
        self._isUpToDate = True
