"""
@file discrete/analytic.py

Apply user-defined formula to a set of discrete variables.
"""
from parmepy.constants import np, debug
from discrete import DiscreteOperator
from parmepy.tools.timers import timed_function


class Analytic_D(DiscreteOperator):
    """
    Compute discrete fields values using a given analytic formula.
    """

    @debug
    def __init__(self, variables, formula, method=None):
        """
        @param[in,out] variables : a list of discrete variables on which
        formula must be applied.
        @param[in] formula : user defined formula to be applied.
        @param method : Method used
        """
        DiscreteOperator.__init__(self, variables, method)
        self.output = variables
        self.vformula = np.vectorize(formula)

    @debug
    @timed_function
    def apply(self, simulation):
        """
        Initialize is always called with coords + current time.
        Any extra parameters must be set using field.setExtraParameters.
        """
        if simulation is None:
            raise ValueError("Missing simulation value for computation.")

        for df in self.variables:
            df.initialize(self.vformula, simulation.time)
