# -*- coding: utf-8 -*-
"""
@file operator/discrete/density.py
Discrete MultiPhase Rot Grad P
"""
from discrete import DiscreteOperator
from parmepy.constants import np, debug
from parmepy.tools.timers import timed_function


class DensityVisco_d(DiscreteOperator):
    """

    """
    @debug
    def __init__(self, density, viscosity,
                 method=None, densityVal=None, viscoVal=None):
        """
        Constructor.
        Reconstruct the Density and the Viscosity scalars.
        @param operator.
        """
        DiscreteOperator.__init__(self, [density, viscosity],
                                  method, name="DensityVisco_d")

        self.density = density
        self.viscosity = viscosity
        self.densityVal = densityVal
        self.viscoVal = viscoVal
        self.input = [self.density, self.viscosity]
        self.output = [self.density, self.viscosity]

    @debug
    @timed_function
    def apply(self, simulation=None):
        if simulation is None:
            raise ValueError("Missing simulation value for computation.")
        # Calling for requirements completion
        DiscreteOperator.apply(self, simulation)

        dt = simulation.timeStep
        iCompute = self.density.topology.mesh.iCompute

        # Density reconstruction
        if self.density[iCompute].all() <= np.absolute(self.densityVal[1] -
                                                       self.densityVal[0]) / 2.0 :
            self.density[iCompute] = self.densityVal[1]
        else :
            self.density[iCompute] = self.densityVal[0]

        # Viscosity reconstruction :
        # nu = nu1 + (nu2 - nu1) * (density - rho1)/(rho2 - rho1)
        self.viscosity.data = self.viscoVal[0] + \
                              (self.viscoVal[1] - self.viscoVal[0]) * \
                              ((self.density.data - self.densityVal[0]) / \
                              (self.densityVal[1] - self.densityVal[0]))

