# -*- coding: utf-8 -*-
import parmepy as pp
from parmepy.domain.obstacle.disk import HalfDisk, Disk
from parmepy.operator.penalization import Penalization
from parmepy.fields.continuous import Field
from parmepy.problem.simulation import Simulation
from parmepy.domain.obstacle.planes import PlaneBoundaries


def computeVel(x, y, ):
    return 1, 1


def computeScal(x, y, t):
    return 1


def testPenalScal2D():
    """
    Penalization in 2D, obstacles = semi-cylinder (disk indeed ...)
    and a plate, field=scalar.
    """
    nb = 33
    Lx = Ly = 2
    dom = pp.Box(dimension=2, length=[Lx, Ly], origin=[-1., -1.])
    resol = [nb, nb]
    scal = Field(domain=dom, name='Scalar')
    hcyl = HalfDisk(dom, position=[0., 0.], radius=0.5,
                    porousLayers=[0.13])
    plates = PlaneBoundaries(dom, normal_dir=1, thickness=0.1)
    penal = Penalization(scal, [hcyl, plates], coeff=[1e6, 10],
                         resolutions={scal: resol})
    penal.discretize()
    penal.setUp()
    topo = scal.discreteFields.keys()[0]
    scd = scal.discreteFields[topo]
    scd[0] = 128
    penal.apply(Simulation(nbIter=100))
    scalRef = Field(domain=dom, name='ScalarRef')
    scalRef.discretize(topo)
    scalRef.load('ref_scal2D_PenalHspherePlane', fieldname='ScalarRef')
    assert scalRef.norm() == scal.norm()


def testPenalScal3D():
    """
    Penalization in 3D, obstacles = hemi-sphere and plates.
    and a plate.
    """
    from parmepy.domain.obstacle.sphere import HemiSphere

    nb = 33
    Lx = Ly = Lz = 2
    dom = pp.Box(dimension=3, length=[Lx, Ly, Lz], origin=[-1., -1., -1.])
    resol = [nb, nb, nb]
    scal = Field(domain=dom, name='Scalar')
    hsphere = HemiSphere(dom, position=[0., 0., 0.],
                         radius=0.5, porousLayers=[0.13])

    plates = PlaneBoundaries(dom, normal_dir=0, thickness=0.1)
    penal = Penalization(scal, [hsphere, plates], coeff=[1e6, 10],
                         resolutions={scal: resol})
    penal.discretize()
    penal.setUp()
    topo = scal.discreteFields.keys()[0]
    scd = scal.discreteFields[topo]
    scd[0] = 128
    penal.apply(Simulation(nbIter=100))
    scalRef = Field(domain=dom, name='ScalarRef')
    scd = scalRef.discretize(topo)
    scalRef.load('ref_scal3D_PenalHspherePlane', fieldname='ScalarRef')
    assert scalRef.norm() == scal.norm()


def testPenalScal2D_2():
    """
    Penalization in 2D, obstacles = cylinder and plate.
    Field=scalar.
    """
    nb = 33
    Lx = Ly = 2
    dom = pp.Box(dimension=2, length=[Lx, Ly], origin=[-1., -1.])
    resol = [nb, nb]
    scal = Field(domain=dom, name='Scalar')
    cyl = Disk(dom, position=[0., 0.], radius=0.5,
               porousLayers=[0.13])
    penal = Penalization(scal, [cyl], coeff=[1e6, 10],
                         resolutions={scal: resol})
    penal.discretize()
    penal.setUp()
    topo = scal.discreteFields.keys()[0]
    scd = scal.discreteFields[topo]
    scd[0] = 128
    penal.apply(Simulation(nbIter=100))
    scalRef = Field(domain=dom, name='ScalarRef')
    scalRef.discretize(topo)
    scalRef.load('ref_scal2D_PenalSphere', fieldname='ScalarRef')
    assert scalRef.norm() == scal.norm()


def testPenalScal3D_2():
    """
    Penalization in 3D, obstacles = Sphere.
    """
    from parmepy.domain.obstacle.sphere import Sphere

    nb = 33
    Lx = Ly = Lz = 2
    dom = pp.Box(dimension=3, length=[Lx, Ly, Lz], origin=[-1., -1., -1.])
    resol = [nb, nb, nb]
    scal = Field(domain=dom, name='Scalar')
    sphere = Sphere(dom, position=[0., 0., 0.],
                    radius=0.5, porousLayers=[0.13])

    penal = Penalization(scal, [sphere], coeff=[1e6, 10],
                         resolutions={scal: resol})
    penal.discretize()
    penal.setUp()
    topo = scal.discreteFields.keys()[0]
    scd = scal.discreteFields[topo]
    scd[0] = 128
    penal.apply(Simulation(nbIter=100))
    scalRef = Field(domain=dom, name='ScalarRef')
    scd = scalRef.discretize(topo)
    scalRef.load('ref_scal3D_PenalSphere', fieldname='ScalarRef')
    assert scalRef.norm() == scal.norm()


def testPenalVec2D():

    nb = 33
    Lx = Ly = 2
    dom = pp.Box(dimension=2, length=[Lx, Ly], origin=[-1., -1.])
    resol = [nb, nb]
    velo = Field(domain=dom, name='Velo', isVector=True)
    hcyl = HalfDisk(dom, position=[0., 0.], radius=0.5,
                    porousLayers=[0.13])
    plates = PlaneBoundaries(dom, normal_dir=1, thickness=0.1)
    penal = Penalization(velo, [hcyl, plates], coeff=[1e6, 10],
                         resolutions={velo: resol})
    penal.discretize()
    penal.setUp()
    topo = velo.discreteFields.keys()[0]
    velod = velo.discreteFields[topo]
    velod[0] = 128
    velod[1] = 12
    penal.apply(Simulation(nbIter=100))
    veloRef = Field(domain=dom, name='VeloRef', isVector=True)
    veloRef.discretize(topo)
    veloRef.load('ref_velo2D_PenalHspherePlane', fieldname='VeloRef')
    assert (veloRef.norm() == velo.norm()).all()


def testPenalVec3D():
    from parmepy.domain.obstacle.sphere import HemiSphere

    nb = 33
    Lx = Ly = Lz = 2
    dom = pp.Box(dimension=3, length=[Lx, Ly, Lz], origin=[-1., -1., -1.])
    resol = [nb, nb, nb]
    velo = Field(domain=dom, name='Velo', isVector=True)
    hsphere = HemiSphere(dom, position=[0., 0., 0.],
                         radius=0.5, porousLayers=[0.13])

    plates = PlaneBoundaries(dom, normal_dir=0, thickness=0.1)
    penal = Penalization(velo, [hsphere, plates], coeff=[1e6, 10],
                         resolutions={velo: resol})
    penal.discretize()
    penal.setUp()
    topo = velo.discreteFields.keys()[0]
    velod = velo.discreteFields[topo]
    velod[0] = 128
    velod[1] = 12
    velod[2] = 4.3
    penal.apply(Simulation(nbIter=100))
    veloRef = Field(domain=dom, name='VeloRef', isVector=True)
    veloRef.discretize(topo)
    veloRef.load('ref_velo3D_PenalHspherePlane', fieldname='VeloRef')
    assert (veloRef.norm() == velo.norm()).all()

if __name__ == "__main__":
    testPenalScal2D()
    ## testPenalScal3D()
    ## testPenalScal2D_2()
    ## testPenalScal3D_2()
    ## testPenalVec2D()
    ## testPenalVec3D()
