# -*- coding: utf-8 -*-

import parmepy as pp
from parmepy.operator.poisson import Poisson
from parmepy.operator.analytic import Analytic
from parmepy.problem.simulation import Simulation
import numpy as np
import math
pi = math.pi
sin = np.sin
cos = np.cos

## Physical Domain description
dim = 3
LL = 2 * pi * np.ones((dim))
# formula to compute initial vorticity field
coeff = 4 * pi ** 2 * (LL[1] ** 2 * LL[2] ** 2 + LL[0] ** 2 * LL[2] ** 2 +
                       LL[0] ** 2 * LL[1] ** 2) / (LL[0] ** 2 * LL[1] ** 2
                                                   * LL[2] ** 2)
cc = 2 * pi / LL


def computeVort(res, x, y, z, t):
    res[0][...] = coeff * sin(x * cc[0]) * sin(y * cc[1]) * cos(z * cc[2])
    res[1][...] = coeff * cos(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])
    res[2][...] = coeff * cos(x * cc[0]) * cos(y * cc[1]) * sin(z * cc[2])
    return res


# ref. field
def computeRef(res, x, y, z, t):
    res[0][...] = -2. * pi / LL[1] * \
        (cos(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        - 2. * pi / LL[2] * (cos(x * cc[0]) * sin(y * cc[1]) * cos(z * cc[2]))

    res[1][...] = -2. * pi / LL[2] * \
        (sin(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        + 2. * pi / LL[0] * (sin(x * cc[0]) * cos(y * cc[1]) * sin(z * cc[2]))

    res[2][...] = -2. * pi / LL[0] * \
        (sin(x * cc[0]) * sin(y * cc[1]) * sin(z * cc[2])) \
        - 2. * pi / LL[1] * (sin(x * cc[0]) * cos(y * cc[1]) * cos(z * cc[2]))

    return res


def computeVort2D(res, x, y, t):
    # todo ...
    res[0][...] = coeff * sin(x * cc[0]) * sin(y * cc[1])
    res[1][...] = coeff * cos(x * cc[0]) * sin(y * cc[1])
    return res


# ref. field
def computeRef2D(res, x, y, t):
    # todo : find a proper ref
    res[0][...] = 1.
    res[1][...] = 1.
    return res


def test_Poisson3D():
    dom = pp.Box(dimension=dim, length=LL)

    # Fields
    velocity = pp.Field(domain=dom, isVector=True, name='Velocity')
    vorticity = pp.Field(domain=dom, formula=computeVort,
                         name='Vorticity', isVector=True)

    resol = [33, 33, 33]
    # Definition of the Poisson operator
    poisson = Poisson(velocity, vorticity,
                      resolutions={velocity: resol, vorticity: resol})

    # Analytic operator to compute the reference field
    ref = pp.Field(domain=dom, name='reference', isVector=True)
    refOp = Analytic(ref, formula=computeRef,
                     resolutions={ref: resol})
    simu = Simulation(nbIter=10)
    refOp.discretize()
    refOp.setUp()
    poisson.discretize()
    poisson.setUp()
    refOp.apply(simu)
    topo = poisson.discreteFields[vorticity].topology
    ## from parmepy.operator.redistribute import Redistribute
    ## distr = Redistribute([ref], refOp, poisson)
    ## distr.discretize()
    ## distr.setUp()
    ## distr.apply(simu)
    vorticity.setTopoInit(topo)
    vorticity.initialize()
    poisson.apply(simu)
    assert np.allclose(ref.norm(), velocity.norm())
    poisson.finalize()
    refD = ref.discretization(topo)
    vd = velocity.discreteFields.values()[0]

    assert np.allclose(ref.norm(), velocity.norm())
    for i in range(dom.dimension):
        assert np.allclose(vd[i], refD[i])


## def test_Poisson2D():
##     dom = pp.Box(dimension=2, length=[2. * pi, 2. * pi], origin=[0., 0.])

##     # Fields
##     velocity = pp.Field(domain=dom, isVector=True, name='Velocity')
##     vorticity = pp.Field(domain=dom, formula=computeVort2D,
##                          name='Vorticity', isVector=True)

##     resol = [33, 33]
##     # Definition of the Poisson operator
##     poisson = Poisson(velocity, vorticity,
##                       resolutions={velocity: resol, vorticity: resol})

##     # Analytic operator to compute the reference field
##     ref = pp.Field(domain=dom, name='reference', isVector=True)
##     refOp = Analytic(ref, formula=computeRef2D,
##                      resolutions={ref: resol})

##     simu = Simulation(nbIter=10)
##     refOp.setUp()
##     poisson.setUp()
##     refOp.apply(simu)
##     vorticity.initialize()
##     poisson.apply(simu)
## #    assert np.allclose(ref.norm(), velocity.norm())
##     poisson.finalize()
##  #   refD = ref.discreteFields.values()[0]
##  #   vd = velocity.discreteFields.values()[0]

## #    assert np.allclose(ref.norm(), velocity.norm())
## #    for i in range(dom.dimension):
## #        assert np.allclose(vd[i], refD[i])

# This may be useful to run mpi tests
if __name__ == "__main__":
    test_Poisson3D()
