# -*- coding: utf-8 -*-
import unittest
import time
import parmepy as pp
import numpy as np
import numpy.testing as npt
from parmepy.constants import *
from math import *



def run():
    # Parameters
    nb = 65
    timeStep = 0.02
    finalTime = 1.
#    outputFilePrefix = './parmepy/test/test_obstacle/Domain_'
    outputFilePrefix = './res/Domain_'
    outputModulo = 1

    t0 = time.time()

    ## Domain
    box = pp.Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])

    ## Obstacle
    sphere = pp.Obstacle(box, name='sphere', zlayer=0.1, radius=0.2, center=[0.5,0.5,0.5], orientation='West', porousLayer=0.1)

    ## ChiDomain
    chiDomain = pp.ContinuousField(domain=box, name='ChiDomain', vector=False)

    ## Solver creation (discretisation of objects is done in solver initialisation)
    topo3D = pp.CartesianTopology(domain=box, resolution=[nb, nb, nb], dim=3, periods = [False, False, False])

    ## Obstacle discretization
    chiDomain.discretize(topo3D)
    chiDomainD = chiDomain.discreteField[0]
    sphere.discretize(topo3D)
    sphereD = sphere.discreteObstacle[0]
    sphereD.chiFunctions()
    for x in sphereD.chiBoundary[:] :
        chiDomainD[x[0], x[1], x[2]]=1.
    for x in sphereD.chiSolid[:] :
        chiDomainD[x[0], x[1], x[2]]=1.
    for x in sphereD.chiPorous[:] :
        chiDomainD[x[0], x[1], x[2]]=0.5
#    for k in xrange (topo3D.mesh.resolution[2]):
#        for j in xrange (topo3D.mesh.resolution[2]):
#            for i in xrange (topo3D.mesh.resolution[2]):
#                if ([i,j,k] in sphereD.chiBoundary) :
#                    chiDomainD[i,j,k]=1.
#                if ([i,j,k] in sphereD.chiSolid) :
#                    chiDomainD[i,j,k]=1.
#                if ([i,j,k] in sphereD.chiPorous) :
#                    chiDomainD[i,j,k]=0.5
    io=pp.Printer(fields=[chiDomain], frequency=outputModulo, outputPrefix=outputFilePrefix)
    io.step()

    t1 = time.time()

    tf = time.time()

    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    run()
