# -*- coding: utf-8 -*-
import time
from parmepy.operator.differentialOperator_d import DifferentialOperator_d
from parmepy.particular_solvers.integrator.euler import Euler
#from parmepy.particular_solvers.integrator.runge_kutta2 import RK2
#from parmepy.particular_solvers.integrator.runge_kutta3 import RK3
#from parmepy.particular_solvers.integrator.runge_kutta4 import RK4
import parmepy as pp
from parmepy.constants import *
import numpy as np
from math import *
import unittest
#import sys
import struct
import array


class test_CondStability(unittest.TestCase):
    """
    Condition Stability test class
    """

    def vitesse(self,x, y, z):
        vx = 1.
        vy = 1.
        vz = 1.
        return vx, vy, vz

    def vorticite(self,x, y, z):
        wx = 1.
        wy = 1.
        wz = 1.
        return wx, wy, wz

    def scalaire(self,x, y, z):
        if x < 0.5 and y < 0.5 and z < 0.5:
            return 1.
        else:
            return 0.


    def testCondStab(self):
        # Parameters
        nb = 128
        timeStep = 0.09
        finalTime = 0.09
        self.t = 0.
        t0 = time.time()

        ## Domain
        box = pp.Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])

        ## Fields
        velo = pp.AnalyticalField(domain=box, formula=self.vitesse, name='Velocity', vector=True)
        vorti = pp.AnalyticalField(domain=box, formula=self.vorticite, name='Vorticity', vector=True)

        inputJBField = [np.zeros((nb,nb,nb), dtype=PARMES_REAL, order=ORDER) for d in xrange(3)]

        f1 = open('./parmepy/test/data/Fields_sav0_U.data','rb')
        f2 = open('./parmepy/test/data/Fields_sav0_V.data','rb')
        f3 = open('./parmepy/test/data/Fields_sav0_W.data','rb')

        nbx = np.asarray(struct.unpack("i",f1.read(4)))
        nby = np.asarray(struct.unpack("i",f1.read(4)))
        nbz = np.asarray(struct.unpack("i",f1.read(4)))

        binvalues = array.array('d')
        binvalues.read(f1, nbx*nby*nbz)

        data = np.array(binvalues, dtype=PARMES_REAL)
        inputJBField[0] = np.reshape(data, (nbx,nby,nbz))
        f1.close()

        nbx = np.asarray(struct.unpack("i",f2.read(4)))
        nby = np.asarray(struct.unpack("i",f2.read(4)))
        nbz = np.asarray(struct.unpack("i",f2.read(4)))

        binvalues = array.array('d')
        binvalues.read(f2, nbx*nby*nbz)

        data = np.array(binvalues, dtype=PARMES_REAL)
        inputJBField[1] = np.reshape(data, (nbx,nby,nbz))
        f2.close()

        nbx = np.asarray(struct.unpack("i",f3.read(4)))
        nby = np.asarray(struct.unpack("i",f3.read(4)))
        nbz = np.asarray(struct.unpack("i",f3.read(4)))

        binvalues = array.array('d')
        binvalues.read(f3, nbx*nby*nbz)

        data = np.array(binvalues, dtype=PARMES_REAL)
        inputJBField[2] = np.reshape(data, (nbx,nby,nbz))
        f3.close()

        ## Operators
        stretch = pp.Stretching(velo,vorti) 

        ## Solver creation (discretisation of objects is done in solver initialisation)
        topo3D = pp.CartesianTopology(domain=box, resolution=[nbx[0]+1, nby[0]+1, nbz[0]+1], dim=3, ghosts=[2,2,2])

        ##Problem
        pb = pp.Problem(topo3D, [stretch])

        ## Setting solver to Problem
        pb.setSolver(finalTime, timeStep, solver_type='basic')
        pb.solver.ODESolver=  Euler#RK4# RK3# RK2# 
        pb.initSolver()


#        self.result = [np.ones((nbx, nby, nbz), dtype=PARMES_REAL, order=ORDER) for d in xrange(3)]
#        vortidata= [np.zeros((128,128,128), dtype=PARMES_REAL, order=ORDER) for d in xrange(3)]
        ## Input of JB velocity Fields
#        print 'shape', stretch.velocity.discreteField[0].data[1][2:nb+2, 2:nb+2,2:nb+2].shape ,inputJBField[1].shape
        stretch.velocity.discreteField[0].data[0][2:nb+2,2:nb+2,2:nb+2] = inputJBField[0]
        stretch.velocity.discreteField[0].data[1][2:nb+2,2:nb+2,2:nb+2] = inputJBField[1]
        stretch.velocity.discreteField[0].data[2][2:nb+2,2:nb+2,2:nb+2] = inputJBField[2]
#        print 'shape', np.asarray(stretch.velocity.discreteField[0].data).shape

        ## Calculation of vorticity Fields from velocity input data (JB)
        self.curl = DifferentialOperator_d(stretch.velocity.discreteField[0].data, stretch.velocity.discreteField[0].data, choice='curl', topology=topo3D)
        stretch.vorticity.discreteField[0].data = self.curl.discreteOperator.apply()
        t1 = time.time()

        ## Solve problem to deduce the LCFL
        pb.solve()

        tf = time.time()

        print "\n"
        print "Total time : ", tf - t0, "sec (CPU)"
        print "Init time : ", t1 - t0, "sec (CPU)"
        print "Solving time : ", tf - t1, "sec (CPU)"


    def runTest(self):
        self.testCondStab()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_CondStability))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
