# -*- coding: utf-8 -*-
import time
from parmepy.physics.compute_forces import Compute_forces
import parmepy as pp
import numpy as np
from math import *
import unittest



class test_Forces(unittest.TestCase):
    """
    DiscreteVariable test class
    """

    def vitesse(self,x, y, z):
#        vx = 1. + x
#        vy = - x * y
#        vz = x * y * z + 10.
        amodul = np.cos(np.pi*self.t/3)
        pix = np.pi*x
        piy = np.pi*y
        piz = np.pi*z
        pi2x = 2.*pix
        pi2y = 2.*piy
        pi2z = 2.*piz
        vx = 2.*np.sin(pix)*np.sin(pix)*np.sin(pi2y)*np.sin(pi2z)*amodul
        vy = -np.sin(pi2x)*np.sin(piy)*np.sin(piy)*np.sin(pi2z)*amodul
        vz = -np.sin(pi2x)*np.sin(piz)*np.sin(piz)*np.sin(pi2y)*amodul
        return vx, vy, vz

    def vorticite(self,x, y, z):
#        wx = x * y
#        wy = y * z
#        wz = - y
        amodul = np.cos(np.pi*self.t/3)
        pix = np.pi*x
        piy = np.pi*y
        piz = np.pi*z
        pi2x = 2.*pix
        pi2y = 2.*piy
        pi2z = 2.*piz
        wx = 2.* np.pi * np.sin(pi2x) * amodul*( - np.cos(pi2y)*np.sin(piz)*np.sin(piz)+ np.sin(piy)*np.sin(piy)*np.cos(pi2z) )
        wy = 2.* np.pi * np.sin(pi2y) * amodul*( 2.*np.cos(pi2z)*np.sin(pix)*np.sin(pix)+ np.sin(piz)*np.sin(piz)*np.cos(pi2x) )
        wz = -2.* np.pi * np.sin(pi2z) * amodul*( np.cos(pi2x)*np.sin(piy)*np.sin(piy)+ np.sin(pix)*np.sin(pix)*np.cos(pi2y) )
        return wx, wy, wz

    def scalaire(self,x, y, z):
        if x < 0.5 and y < 0.5 and z < 0.5:
            return 1.
        else:
            return 0.


    def testComputeForces(self):
        # Parameters
        nb = 11
        timeStep = 0.09
        finalTime = 0.36
        self.t = 0.
        t0 = time.time()

        ## Domain
        box = pp.Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])

        ## Obstacle
        sphere = pp.Obstacle(box, zlayer=0.1, radius=0.1,
                             center=[0.5, 0.5, 0.5], name='sphere',
                             orientation='West', porousLayer=0.05)

        ## Fields
        velo = pp.AnalyticalField(domain=box, formula=self.vitesse, name='Velocity', vector=True)
        vorti = pp.AnalyticalField(domain=box, formula=self.vorticite, name='Vorticity', vector=True)

        ## Solver creation (discretisation of objects is done in solver initialisation)
        topo3D = pp.CartesianTopology(domain=box, resolution=[nb, nb, nb], dim=3, ghosts=[2,2,2])

        ## Fields discretization
        vorti.discretize(topo3D)
        velo.discretize(topo3D)
        vorti.initialize()
        velo.initialize()

        # Forces computation
        Re = 200.
        noca = Compute_forces(topo3D, sphere, boxMin= [0.2, 0.2, 0.2], boxMax=[0.8, 0.8, 0.8])
        if (topo3D.rank == 0):
            f = open('./parmepy/test/test_operator/NocaForces.dat', 'w')

        while (self.t <= finalTime):
            nocares = noca.apply(self.t, timeStep, velo.discreteField[velo._fieldId], vorti.discreteField[vorti._fieldId], Re)
            if (topo3D.rank == 0):
                # print time and forces values in the following order : time, cX, cY, cZ
                f.write("%s   %s   %s   %s\n" % (self.t, nocares[0], nocares[1], nocares[2]))


            self.t = self.t + timeStep

        if (topo3D.rank == 0):
            f.close()

        t1 = time.time()
        tf = time.time()

        print "\n"
        print "Total time : ", tf - t0, "sec (CPU)"
        print "Init time : ", t1 - t0, "sec (CPU)"
        print "Solving time : ", tf - t1, "sec (CPU)"


    def runTest(self):
        self.testComputeForces()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_Forces))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
