# -*- coding: utf-8 -*-
import unittest
import time
import parmepy as pp
import numpy as np
from parmepy.particular_solvers.integrator.runge_kutta4 import RK4
import numpy.testing as npt
from parmepy.constants import *
from math import *

def vitesse(x, y, z):
    vx = 2.
    vy = 2.
    vz = 2.
    return vx, vy, vz

def vorticite(x, y, z):
    wx = 3.
    wy = 3.
    wz = 3.
    return wx, wy, wz


def run():
    # Parameters
    nb = 129
    timeStep = 0.09
    finalTime = 0.09
    outputFilePrefix = './parmepy/test/test_operator/Penalization_'
    outputModulo = 1

    t0 = time.time()

    ## Domain
    box = pp.Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])

    ## Obstacle
    lambd=np.array([0, 10, 10**8],dtype = PARMES_REAL, order=ORDER)
    sphere = pp.Obstacle(box, zlayer=0.1, radius=0.2, center=[0.5, 0.5, 0.5],
                         name='sphere', orientation='West', porousLayer=0.1)

    ## ChiDomain
    chiDomain = pp.ContinuousField(domain=box, name='ChiDomain', vector=False)

    ## Fields
    velo = pp.AnalyticalField(domain=box, formula=vitesse, name='Velocity', vector=True)
    vorti = pp.AnalyticalField(domain=box, formula=vorticite, name='Vorticity', vector=True)

    ## Operators
    penal = pp.Penalization(velo, vorti, sphere, lambd)

    ## Solver creation (discretisation of objects is done in solver initialisation)
    topo3D = pp.CartesianTopology(domain=box, resolution=[nb, nb, nb], dim=3, ghosts=[2,2,2])

    pb = pp.Problem(topo3D, [penal])

    ## Setting solver to Problem
    pb.setSolver(finalTime, timeStep, solver_type='basic', io=pp.Printer(fields=[velo, vorti], frequency=outputModulo, outputPrefix=outputFilePrefix))
    pb.solver.ODESolver= RK4# RK3# RK2# Euler#
    pb.initSolver()
    t1 = time.time()

    ## Solve problem
    pb.solve()

    tf = time.time()

    
    print "\n"
    print "Total time : ", tf - t0, "sec (CPU)"
    print "Init time : ", t1 - t0, "sec (CPU)"
    print "Solving time : ", tf - t1, "sec (CPU)"


if __name__ == "__main__":
    run()
