# -*- coding: utf-8 -*-
import time
from parmepy.particular_solvers.integrator.euler import Euler
from parmepy.particular_solvers.integrator.runge_kutta2 import RK2
from parmepy.particular_solvers.integrator.runge_kutta3 import RK3
from parmepy.particular_solvers.integrator.runge_kutta4 import RK4
import parmepy as pp
import numpy as np
from math import *
import unittest



class test_Stretching(unittest.TestCase):
    """
    DiscreteVariable test class
    """

    def vitesse(self,x, y, z):
#        vx = 1. + x
#        vy = - x * y
#        vz = x * y * z + 10.
        amodul = np.cos(np.pi*self.t/3)
        pix = np.pi*x
        piy = np.pi*y
        piz = np.pi*z
        pi2x = 2.*pix
        pi2y = 2.*piy
        pi2z = 2.*piz
        vx = 2.*np.sin(pix)*np.sin(pix)*np.sin(pi2y)*np.sin(pi2z)*amodul
        vy = -np.sin(pi2x)*np.sin(piy)*np.sin(piy)*np.sin(pi2z)*amodul
        vz = -np.sin(pi2x)*np.sin(piz)*np.sin(piz)*np.sin(pi2y)*amodul
        return vx, vy, vz

    def vorticite(self,x, y, z):
#        wx = x * y
#        wy = y * z
#        wz = - y
        amodul = np.cos(np.pi*self.t/3)
        pix = np.pi*x
        piy = np.pi*y
        piz = np.pi*z
        pi2x = 2.*pix
        pi2y = 2.*piy
        pi2z = 2.*piz
        wx = 2.* np.pi * np.sin(pi2x) * amodul*( - np.cos(pi2y)*np.sin(piz)*np.sin(piz)+ np.sin(piy)*np.sin(piy)*np.cos(pi2z) )
        wy = 2.* np.pi * np.sin(pi2y) * amodul*( 2.*np.cos(pi2z)*np.sin(pix)*np.sin(pix)+ np.sin(piz)*np.sin(piz)*np.cos(pi2x) )
        wz = -2.* np.pi * np.sin(pi2z) * amodul*( np.cos(pi2x)*np.sin(piy)*np.sin(piy)+ np.sin(pix)*np.sin(pix)*np.cos(pi2y) )
        return wx, wy, wz

    def scalaire(self,x, y, z):
        if x < 0.5 and y < 0.5 and z < 0.5:
            return 1.
        else:
            return 0.


    def testOperatorStretching(self):
        # Parameters
        nb = 16
        timeStep = 0.09
        finalTime = 0.09
        self.t = 0.
        t0 = time.time()

        ## Domain
        box = pp.Box(3, length=[1., 1., 1.], origin=[0., 0., 0.])

        ## Fields
        velo = pp.AnalyticalField(domain=box, formula=self.vitesse, name='Velocity', vector=True)
        vorti = pp.AnalyticalField(domain=box, formula=self.vorticite, name='Vorticity', vector=True)

        ## Operators
        stretch = pp.Stretching(velo,vorti)

        ## Solver creation (discretisation of objects is done in solver initialisation)
        topo3D = pp.CartesianTopology(domain=box, resolution=[nb, nb, nb], dim=3, ghosts=[2.,2.,2.])

        ##Problem
        pb = pp.Problem(topo3D, [stretch])

        ## Setting solver to Problem
        pb.setSolver(finalTime, timeStep, solver_type='basic')
        pb.solver.ODESolver=  RK3#RK2#Euler#RK4# 
        pb.initSolver()

        t1 = time.time()

        ## Solve problem
        pb.solve()

        tf = time.time()

        print "\n"
        print "Total time : ", tf - t0, "sec (CPU)"
        print "Init time : ", t1 - t0, "sec (CPU)"
        print "Solving time : ", tf - t1, "sec (CPU)"


    def runTest(self):
        self.testOperatorStretching()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_Stretching))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
