# -*- coding: utf-8 -*-
import time
import parmepy as pp
from parmepy.particular_solvers.integrator.euler import Euler
from parmepy.particular_solvers.integrator.runge_kutta2 import RK2
from parmepy.particular_solvers.integrator.runge_kutta3 import RK3
from parmepy.particular_solvers.integrator.runge_kutta4 import RK4
from math import *
import unittest
import numpy as np
import numpy.testing as npt
import copy
import matplotlib.pyplot as plt
from parmepy.constants import *


class test_EDO(unittest.TestCase):
    """
    DiscreteVariable test class
    """

    def analyticalSolution(self,t, x, y, z):
        sx = (t*np.exp(t) + 1.) * np.exp(-t)
        sy = (t*np.exp(t) + 1.) * np.exp(-t)
        sz = (t*np.exp(t) + 1.) * np.exp(-t)
        return sx, sy, sz

    def f (self, t, u):
        fx = -u[0][:,:,:] + t + 1.
        fy = -u[1][:,:,:] + t + 1.
        fz = -u[2][:,:,:] + t + 1.
        return fx , fy ,fz

    def testIntegratorEDO(self):
        # Parameters
        nb = 32
        timeStep = 0.1
        finalTime = 1.0
        multi = 2.
        maxerror=0.

        t0 = time.time()
        self.t = 0.
        ## Domain
        box = pp.Box(3, length=[4.*np.pi, 4.*np.pi, 4.*np.pi], 
                     origin=[- 2.*np.pi,- 2.*np.pi,- 2.*np.pi])


#####################################################################################

        ## Solver creation (discretisation of objects is done in solver initialisation)
        topo3D = pp.CartesianTopology(domain=box, resolution=[nb, nb, nb], dim=3, periods=[False, False, False])

        Result=np.asarray([np.zeros([nb, nb, nb], 
                          dtype=PARMES_REAL, order=ORDER) for d in xrange(3)])
        Analvorty=np.asarray([np.zeros([nb, nb, nb], 
                             dtype=PARMES_REAL, order=ORDER) for d in xrange(3)])

        compteur = 0
        taille = int(finalTime / timeStep) +1
        compteur = 0
        dt = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
        errEuler = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
        errRK2 = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
        errRK3 = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
        errRK4 = np.zeros([taille], dtype=PARMES_REAL, order=ORDER)
        UEuler = np.asarray([np.zeros([nb, nb, nb], dtype=PARMES_REAL, 
                            order=ORDER) for d in xrange(3)])
        URK2 = np.asarray([np.zeros([nb, nb, nb], dtype=PARMES_REAL, 
                          order=ORDER) for d in xrange(3)])
        URK3 = np.asarray([np.zeros([nb, nb, nb], dtype=PARMES_REAL, 
                          order=ORDER) for d in xrange(3)])
        URK4 = np.asarray([np.zeros([nb, nb, nb], dtype=PARMES_REAL, 
                          order=ORDER) for d in xrange(3)])

        UEuler[:,:,:,:] = 1.
        URK2[:,:,:,:] = 1.
        URK3[:,:,:,:] = 1.
        URK4[:,:,:,:] = 1.

        Analvorty[:,:,:,:]=np.vectorize(self.analyticalSolution)(timeStep,
                                          topo3D.mesh.coords[0], \
                                          topo3D.mesh.coords[1], \
                                          topo3D.mesh.coords[2])[:]

        while self.t < finalTime :
            print "t", self.t

        # Euler
            self.method= Euler
            methodInt=self.method(self.f)
            Result = methodInt.integrate(self.f, self.t , timeStep , UEuler)
            UEuler = Result
            errEuler[compteur]= np.max (abs(Analvorty[:,:,:,:] - Result[:,:,:,:]))

        #  RK2
            self.method= RK2
            methodInt=self.method(self.f)
            Result = methodInt.integrate(self.f, self.t , timeStep , URK2)
            URK2 = Result
            errRK2[compteur]= np.max (abs(Analvorty[:,:,:,:] - Result[:,:,:,:]))

        #  RK3
            self.method= RK3
            methodInt=self.method(self.f)
            Result = methodInt.integrate(self.f, self.t , timeStep , URK3)
            URK3 = Result
            errRK3[compteur]= np.max (abs(Analvorty[:,:,:,:] - Result[:,:,:,:]))

        #  RK4
            self.method= RK4
            methodInt=self.method(self.f)
            Result = methodInt.integrate(self.f, self.t , timeStep , URK4)
            URK4 = Result
            errRK4[compteur]= np.max (abs(Analvorty[:,:,:,:] - Result[:,:,:,:]))
            dt[compteur] = self.t

            self.t = self.t + timeStep
            Analvorty[:,:,:,:]=np.vectorize(self.analyticalSolution)(self.t + timeStep, 
                                          topo3D.mesh.coords[0], \
                                          topo3D.mesh.coords[1], \
                                          topo3D.mesh.coords[2])[:]
            compteur = compteur + 1

        # Check the convergence order of each time integration scheme :
        npt.assert_array_less(errEuler,timeStep)
        npt.assert_array_less(errRK2,timeStep**2)
        npt.assert_array_less(errRK3,timeStep**3)
        npt.assert_array_less(errRK4,timeStep**4)

        ## Table of error values
#        print "erreur Euler", errEuler
#        print "erreur RK2", errRK2
#        print "erreur RK3", errRK3
#        print "erreur RK4", errRK4

        ## Plot error_scheme vs time
#        plt.figure(1)
#        plt.subplot(211)
#        plt.xlabel('dt')
#        plt.xscale('log')
#        plt.ylabel('Erreur')
#        plt.yscale('log')
#        plt.plot(dt, errEuler, '+-',dt ,errRK2, '+-' ,dt, errRK3, '+-',dt,errRK4, '+-')
#        plt.legend([u"Euler", u"RK2", u"RK3", u"RK4"], loc=4)
#        plt.subplot(212)
#        plt.xlabel('dt')
#        plt.ylabel('Erreur')
#        plt.plot(dt, errEuler, '+-',dt ,errRK2, '+-' ,dt, errRK3, '+-',dt,errRK4, '+-')
#        plt.legend([u"Euler", u"RK2", u"RK3", u"RK4"], loc=4)

        plt.show()

        t1 = time.time()

        print "\n"
#        print "Total time : ", tf - t0, "sec (CPU)"
#        print "Init time : ", t1 - t0, "sec (CPU)"
#        print "Solving time : ", tf - t1, "sec (CPU)"


    def runTest(self):
        self.testIntegratorEDO()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_EDO))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
