# -*- coding: utf-8 -*-
import time
import parmepy as pp
from parmepy.particular_solvers.integrator.euler import Euler
from parmepy.particular_solvers.integrator.runge_kutta2 import RK2
from parmepy.particular_solvers.integrator.runge_kutta3 import RK3
from parmepy.particular_solvers.integrator.runge_kutta4 import RK4
from parmepy.particular_solvers.integrator.runge_kutta2stretching import RK2Stretch
from parmepy.particular_solvers.integrator.runge_kutta3stretching import RK3Stretch
from parmepy.particular_solvers.integrator.runge_kutta4stretching import RK4Stretch
from math import *
import unittest
import numpy as np
import numpy.testing as npt
import copy
import matplotlib.pyplot as plt
from parmepy.constants import *



class test_RK(unittest.TestCase):
    """
    DiscreteVariable test class
    """

    def vitesse(self, x, y, z):
        vx = np.sin(x)
        vy = np.sin(y)
        vz = np.sin(z)
        return vx, vy, vz

    def vorticite(self, x, y, z):
        wx = self.t**2+1.
        wy = self.t**2+1.
        wz = self.t**2+1.
        return wx, wy, wz

    def analyticalSolution(self,t, x, y, z):
        sx = t**2 - (t + t**3 /3.) * np.cos(x)
        sy = t**2 - (t + t**3 /3.) * np.cos(y)
        sz = t**2 - (t + t**3 /3.) * np.cos(z)
        return sx, sy, sz

    def f (self, t, u):
        fx = 2.*t -u[0][:,:,:]
        fy = 2.*t -u[1][:,:,:]
        fz = 2.*t -u[2][:,:,:]
        return fx , fy ,fz


    def wgradu (self, t, x, y, z) :
        wx =( 1 + t**2 )* np.cos(x)
        wy =( 1 + t**2 )* np.cos(y)
        wz =( 1 + t**2 )* np.cos(z)
        return wx, wy, wz


    def testIntegratorRK(self):
        # Parameters
        nb = 32
        timeStep = 0.01
        finalTime = 0.01
        maxerror=0.

        t0 = time.time()
        self.t = 0.
        ## Domain
        box = pp.Box(3, length=[4.*np.pi, 4.*np.pi, 4.*np.pi], origin=[- 2.*np.pi,- 2.*np.pi,- 2.*np.pi])


#####################################################################################

        ## Fields
#        velo = pp.AnalyticalField(domain=box, formula=self.vitesse, name='Velocity', vector=True)
#        vorti = pp.AnalyticalField(domain=box, formula=self.vorticite, name='Vorticity', vector=True)

        ## Operators
#        stretch = pp.Stretching(velo,vorti)

        ## Solver creation (discretisation of objects is done in solver initialisation)
        topo3D = pp.CartesianTopology(domain=box, resolution=[nb, nb, nb], dim=3)

        ##Problem
#        pb = pp.Problem(topo3D, [stretch])

        ## Setting solver to Problem
#        pb.setSolver(finalTime, timeStep, solver_type='basic')

        Result=np.asarray([np.zeros([nb, nb, nb], dtype=PARMES_REAL, order=ORDER) for d in xrange(3)])
        Analvorty=np.asarray([np.zeros([nb, nb, nb], dtype=PARMES_REAL, order=ORDER) for d in xrange(3)])
        Analvorty[:,:,:,:]= np.vectorize(self.wgradu)(self.t,topo3D.mesh.coords[0], \
                                          topo3D.mesh.coords[1], \
                                          topo3D.mesh.coords[2])[:]

        self.method= RK3 # RK3Stretch #RK2Stretch #RK4Stretch #Euler #RK4 #

        print "\nODE SOLVER :: ", self.method

        methodInt=self.method(self.f)
        Result = methodInt.integrate(self.f, self.t , timeStep , self.wgradu, topo3D)

        if Euler == self.method :
            maxerror = abs((timeStep**2 - 1./3. * timeStep**3)-.0) + timeStep**3
        if RK2 == self.method :
            maxerror = abs((-1./3. * timeStep**3) - (0.5* timeStep**2)) + timeStep**8
        if RK3 == self.method :
            maxerror = abs(0. - (0.5* timeStep**2 - 1./6. * timeStep**3)) + timeStep**3
        if RK4 == self.method :
            maxerror = abs((timeStep**2 - 1./3. * timeStep**3) - (7./6.* timeStep**2 - 1./2. * timeStep**3 + 1./8. * timeStep**4)) 


#        pb.initSolver()
        t1 = time.time()

        ## Solve problem
#        pb.solve()
        self.t = self.t + timeStep
        Analvorty[:,:,:,:]=Analvorty[:,:,:,:]+np.vectorize(self.analyticalSolution)(self.t, topo3D.mesh.coords[0], \
                                            topo3D.mesh.coords[1], \
                                            topo3D.mesh.coords[2])[:]

## Visual comparison between the numerical resolution and the analytical resolution
        for i in range(1):
            plt.figure(1)
            plt.subplot(211)
            plt.axis([-2*np.pi,2*np.pi,np.min(Analvorty[i][:,0,0]),np.max(Analvorty[i][:,0,0])])
            plt.plot(topo3D.mesh.coords[0][:,0,0], Analvorty[i][:,0,0], '-' ,topo3D.mesh.coords[0][:,0,0],Result[i][:,0,0], '-' )
            plt.legend([u"Solution Analytique", u"Solution Numérique"])
            plt.subplot(212)
            plt.axis([-2*np.pi,2*np.pi,0,max(abs(Analvorty[i][:,0,0] - Result[i][:,0,0]))])
            plt.plot(topo3D.mesh.coords[0][:,0,0], abs(Analvorty[i][:,0,0] - Result[i][:,0,0]), '-' )
            plt.legend([u"Erreur"])
            plt.ax = plt.gca()
            plt.ax.ticklabel_format(style='sci', axis='y') 
            plt.show()
        npt.assert_array_less(abs(Analvorty[:,:,:,:] - Result[:,:,:,:]),maxerror)
        tf = time.time()

        print "\n"
        print "Total time : ", tf - t0, "sec (CPU)"
        print "Init time : ", t1 - t0, "sec (CPU)"
        print "Solving time : ", tf - t1, "sec (CPU)"


    def runTest(self):
        self.testIntegratorRK()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_RK))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
