# -*- coding: utf-8 -*-
"""
@file numpywrappers.py

Tools to build numpy arrays based on parmepy setup (float type ...)
"""
from parmepy.constants import PARMES_REAL, ORDER, PARMES_INTEGER, PARMES_INDEX
import numpy as np
import scitools.filetable as ft


def zeros(shape, dtype=PARMES_REAL):
    """
    @param shape : tuple of int
    @return np.zeros(...)
    Creates a numpy array of shape 'shape', handling
    data of type parmepy.constants.PARMES_REAL using parmepy.constants.ORDER.
    """
    return np.zeros(shape, dtype=dtype, order=ORDER)


def ones(shape, dtype=PARMES_REAL):
    """
    @param shape : tuple of int
    @return np.ones(...)
    Creates a numpy array of shape 'shape', handling
    data of type parmepy.constants.PARMES_REAL using parmepy.constants.ORDER.
    """
    return np.ones(shape, dtype=dtype, order=ORDER)


def zeros_like(tab):
    """
    @param tab : a numpy array
    @return np.zeros_like(...) an array of the same shape as tab, filled with
    zeros.
    Creates a numpy array of shape tab.shape, handling
    data of type parmepy.constants.PARMES_REAL using parmepy.constants.ORDER.
    """
    return np.zeros_like(tab, dtype=tab.dtype, order=ORDER)


def copy(tab):
    """
    @param tab : a numpy array
    @return a copy of tab with the same ordering type (fortran or C) as tab.
    """
    return tab.copy(order='A')


def asarray(tab):
    """
    @param tab : a numpy array
    @return a numpy array in the Parmes
    predefined ordering type (fortran or C).
    """
    return np.asarray(tab, order=ORDER, dtype=tab.dtype)


def realarray(tab):
    """
    @param tab : a numpy array
    @return a numpy array in the Parmes of real,
    precision set by PARMES_REAL,
    predefined ordering type (fortran or C)
    """
    return np.asarray(tab, order=ORDER, dtype=PARMES_REAL)


def indexarray(tab):
    """
    return an array of int, int type define by PARMES_INDEX
    """
    return np.asarray(tab, order=ORDER, dtype=PARMES_INDEX)


def integerarray(tab):
    """
    return an array of int, int type define by PARMES_INDEX
    """
    return np.asarray(tab, order=ORDER, dtype=PARMES_INTEGER)


def abs(tab):
    """
    @param tab : a numpy array
    @return a numpy array in the Parmes
    predefined ordering type (fortran or C), equal to abs(tab)
    """
    return np.abs(tab, order=ORDER, dtype=tab.dtype)


def sum(tab, dtype=PARMES_REAL):
    """
    """
    return np.sum(tab, dtype=dtype)


def prod(tab, dtype=PARMES_REAL):
    """
    """
    return np.prod(tab, dtype=dtype)


def writeToFile(fname, data, mode='a'):
    """
    write data (numpy array) to file fname
    """
    fout = open(fname, mode)
    ft.write(fout, data)
    fout.close()
