!> Physical domain definition and its discretisation.
!! Note FP : fortran structures might be better but I avoid them on purpose, for f2py better compat. 
module Domain

  use client_data
  
  implicit none

  private
  public init_geometry, init_grid
  public physDomainLowerPoint,physDomainUpperPoint,grid_resolution,grid_step,domain_ghostsize,domainLength,domain_bc,gridCells

  !> Physical domain upper limit
  real(mk), dimension(:), pointer :: physDomainUpperPoint=>NULL()
  !> Physical domain lower limit
  real(mk), dimension(:), pointer :: physDomainLowerPoint =>NULL()
  !> Sizes of the domain
  real(mk), dimension(:), pointer :: domainLength =>NULL()
  !> Boundary conditions for the domain
  integer,  dimension(:), pointer :: domain_bc=>NULL()
  !> Number of ghost points in each direction
  integer,  dimension(:), pointer :: domain_ghostsize=>NULL()
  !> Grid resolution (number of points)
  integer,  dimension(:), pointer :: grid_resolution =>NULL()
  !> Number of cells (in each dir) on the grid
  integer,  dimension(dim3) :: gridCells
  !> Grid space step sizes
  real(mk),  dimension(:), pointer :: grid_step =>NULL()
  

  integer, private :: istat

contains
  
  !> Set continuous domain geometry
  !> \param boundary conditions type (ppm way)
  subroutine init_geometry(bc)
    
    !> Boundary conditions type (for ppm)
    integer, intent(in) :: bc
    
    ! Domain size and min/max coords
    allocate(physDomainLowerPoint(dim3), physDomainUpperPoint(dim3), domainLength(dim3),stat = istat)
    if(istat.ne.0) stop 'Geometry, allocation failed'
    physDomainUpperPoint = 0.0
    physDomainLowerPoint = 0.0
    physDomainUpperPoint(c_X) = 6.0!2.*pi!!1.0
    physDomainUpperPoint(c_Y) = 4.0!2.*pi!1.0
    !physDomainLowerPoint(c_X) = 0.0
    !physDomainLowerPoint(c_Y) = 0.0
    ! Boundary conditions and ghosts
    allocate(domain_bc(2*dime), domain_ghostsize(dime), stat = istat)
    if(istat.ne.0) stop 'BC, allocation failed'
    domain_bc = bc
    domain_ghostsize = 0 ! Warning : It seems that ghost>0 is required for ppm remesh 
    domainLength = physDomainUpperPoint - physDomainLowerPoint

  end subroutine init_geometry

  !> Set discretisation parameters for the continuous domain.
  !! Resolution corresponds to the number of points in one direction.
  !! For periodic boundaries, point with index 1 is the same as point with last index
  !! but must be present (ppm required)
  subroutine init_grid()
    
    allocate(grid_resolution(dim3),grid_step(dim3),stat=istat)
    if(istat.ne.0) stop 'grid_resolution, allocation failed'
    grid_resolution = 1
    grid_resolution(c_X) = 129!2001
    grid_resolution(c_Y) = 65!1601
!!$    grid_step(c_X) = (physDomainUpperPoint(c_X) - physDomainLowerPoint(c_X))/(real(grid_resolution(c_X),mk)-1.)
!!$    grid_step(c_Y) = (physDomainUpperPoint(c_Y) - physDomainLowerPoint(c_Y))/(real(grid_resolution(c_Y),mk)-1.)
!!$    grid_step(c_Z) = 1.
!!$    grid_resolution(c_X) = 2001
!!    grid_resolution = 256
!!$    grid_resolution(c_Z) = 1
    gridCells = max(grid_resolution-1,1)
!!$    gridCells(c_Z) = 1
    grid_step = (physDomainUpperPoint - physDomainLowerPoint)/gridCells
    
  end subroutine init_grid

end module Domain
