!> Penalization stuff (init chi, penalize vorticity)
module penalisation

  use client_data
  use mpi
  implicit none

  private
  
  public :: penalise_velocity

  ! Penalisation parameter 
  real(mk),parameter,private :: lambda = 6e10
  
  ! Drag 
  ! real(mk),dimension(dime) :: localDiagnostics

contains
  
  !> Apply penalization functions to velocity (on boundaries to enforce Dirichlet cond. and on the sphere)
  !! We also compute drag and lift (current mpi process local values) during this call
  subroutine penalise_velocity(vel,dt,chi_sphere,chi_boundary)
    ! Velocity, intent(inout)
    real(mk), dimension(:,:,:,:,:), pointer :: vel
    ! Indicators of the sets where velocity must be penalized
    integer,dimension(:,:),pointer::chi_sphere
    ! Indicators of the sets where velocity must be penalized
    integer,dimension(:,:),pointer::chi_boundary
    ! Time step
    real(mk), intent(in) :: dt
    ! Local mesh, number of points in each dir (1st index) for each sub (2nd index)
    !integer, dimension(dime), intent(in)  :: resolution
    integer :: k
    real(mk) :: coef
    !localDiagnostics=0.0

    coef=1./(1.+dt*lambda)
    
    !do k=1,size(chi_boundary,2)
    forall(k=1:size(chi_boundary,2))
       vel(:,chi_boundary(1,k),chi_boundary(2,k),chi_boundary(3,k),:)=&
            coef*vel(:,chi_boundary(1,k),chi_boundary(2,k),chi_boundary(3,k),:)
    end forall!do
    !forall(k=1:size(chi_sphere,2))
       do k=1,size(chi_sphere,2)
       vel(:,chi_sphere(1,k),chi_sphere(2,k),chi_sphere(3,k),:)=&
            coef*vel(:,chi_sphere(1,k),chi_sphere(2,k),chi_sphere(3,k),:)
       ! if( all(chi_sphere(:,k) < resolution(:))) then 
       !   localDiagnostics=localDiagnostics + vel(:,chi_sphere(1,k),chi_sphere(2,k),chi_sphere(3,k),1)
       ! end if
    end do
    !end forall

  end subroutine penalise_velocity

  !> Collect and reduce drag/lift values over all mpi proc
!!$  subroutine get_dragAndLift(diagnostics,dvol,radius)
!!$    !> diagnostics
!!$    real(mk),dimension(:) :: diagnostics
!!$    !> element of volume
!!$    real(mk),intent(in)::dvol
!!$    !> sphere radius
!!$    real(mk),intent(in)::radius
!!$    
!!$    integer :: info
!!$    real(mk) :: coef,uinf
!!$
!!$    uinf=1.0
!!$    coef=2./(uinf**2*2.*radius)*dvol*lambda
!!$    !localDiagnostics=coef*localDiagnostics
!!$    ! Reduce drag and lift values over all procs
!!$    !call MPI_Reduce(localDiagnostics,diagnostics,dime,MPI_DOUBLE_PRECISION,MPI_SUM,0,MPI_COMM_WORLD,info)
!!$    
!!$  end subroutine get_dragAndLift

end module Penalisation
