!> \mainpage CodeScalar Documentation
!!
!! \section intro_sec Introduction
!!
!! This is Code_scalar documentation !
!!
!! \section install_sec Installation
!!
!! \subsection tools_subsec Tools required:
!! - cmake
!! - fftw
!! - mpi
!!
!! \subsection compile Compilation
!! - Obtain the code from the svn repositories.
!! - Go the the code location (the folder must contains at least "src" and "CMake" folders and a "CMakeList" file)
!! - Create a build folder : \verbatim $ mkdir build \endverbatim
!! - Run cmake : \verbatim $ cmake .. \endverbatim
!! - Compile : \verbatim $ make \endverbatim
!!
!! \subsection add_comp Advanced compilation options
!! This section explain how to perform advanced task. All these task can
!! only be performed after having running at least one time cmake.
!!
!! - Update the fortran file generated with the .fortran file : <tt>$ make fortran_file</tt>
!! - Generate (or update) doxygen documentation : <tt> $ make doc</tt>
!! - Obtain the list of possible target for a make : <tt>$ make help</tt>
!! - A <tt>$ make VERBOSE=1</tt>  shows all the command running by the make command.
!!
!! The CMakeLists contains some flag wich can be changed
!! - <tt> WITH_TESTS</tt>: to compile tests.
!! - <tt>WITH_EXAMPLE</tt>: to compile benchmark
!! - <tt>GENERATE_SRC</tt>: to (re-) generate source file from *.fortran file
!!
!! \section running Running the program
!! Run ScaleExe
!!
!! \section doc Additionnal documentation
!!This manual is divided in the following sections:
!!- \subpage part_doc : some information about particle method implementation
!!- \subpage output_doc : some information about posprocessing and input/output
!!

!-----------------------------------------------------------
!
!> \page  part_doc About particle method implementation
!!  This page described the implementation of the advection solver based
!!  on particle method.
!!  
!! \section part_intro Introduction
!!
!! \subsection notation Some notation
!! All the file using some common notation
!!  -   local domain of each processus are indiced from 1 to N_proc.
!!  -   d_sc = (scalar) space step
!!  -   ind = mesh (or particles) indice. As we use fortran, it goes
!!      from 1 to N_proc
!!  -   pos = particles position. In accordance to ind, if there is one
!!      particle at each mesh point, it goes from d_sci to (N_proc)*d_sc. 
!!      A particle "i" belong to the local subdomain if and only if pos(i)
!!      belongs to [d_sc;d_sc*(N_proc+1)[.
!!  -   pos_adim = adimensionned particles position. In accordance to ind, if there is one
!!      particle at each mesh point, it goes from 1 to N_proc
!!
!! \subsection Todo
!! \todo
!!  -   travailler par bloc de lignes plutôt que par ligne 1D
!!      - petits groupes => moins d'usage mémoire en local, plus de
!!        messages qui sont chacun plus petits.
!!      - grands groupes => mutualisation des communications (moins
!!        nombreux mais message plus grands) et nécessite plus de mémoire
!!        en local. (à éviter si la ram est légère.)
!!  -   add new remeshing schemes:
!!      -    corrected lambda 4
!!      -    M'6
!!  -   Au sein d'un group de ligne, on peut utiliser un autre niveau de
!!      parallélisme type open_mp.
!
!-----------------------------------------------------------


!-----------------------------------------------------------
!
!> \page  output_doc About input and output
!!  This page described the implementation of the IO
!!
!! \section io_intro Introduction
!! Two output format are implemented : avs files and parallel vtk xml
!! files.
!!  The first one is provided by the "avs" module (see avs.F90) and the
!! second one by "parallel_io" module. The both can be used for output
!! during a simulation on a parrallel computer.
!!
!! \section Main differences between the two output format
!! -    avs file can also be used as input files (input at format "vtk xml"
!!      will be implement later)
!! -    number of files created and parallel context:
!!      -   the "vtk xml" standart allows to write output in more than one
!!          files. The module provided in scale will perform write in a file
!!          per processus. More precisly, output will not need any communication
!!          neither mpi_io. But it also means it will create a lot of file
!!          if the number of processes is high.
!!      -   on the contrary, the avs module produces only one ouput file
!!          containing all the data.
!!      -   Note that in "small" cluster, the write on hard disk are
!!          sequential. In future update, the number of output for "vtk
!!          xml" file will be defined by the user.
!! - both output are done at binary format.
!!
!! \subsection Todo
!! \todo
!! -    implement hdf5 output:
!!      -    in first time, by creating one output file by mpi processus
!!      -    then allow the user to gather some processus in one output file in
!!           oder to limit the number of output and initialize this to a reasonnable
!!           default value.
!! -    implement hdf5 input in order to restart computation to a given state.
!!          -    define the specification: what is conserved from the previous
!!               computations ?
!!               -    obviously input parameters, last field values, current time ...
!!               -    ... but it is also possible to re-initialize memomry to its last
!!                    state
!!          -    implement the corresponding output (more than the standart ouput)
!!               and the corresponding input.
!!          -    to deal with unexpected bug, add minimal information to standart
!!               output and implement matching input routines in order to also be able
!!               to restart a bugged simulation.
!! -    implement xml input 
!   
!-----------------------------------------------------------

!> @defgroup part Particular method
!! Details about particle method for scalar advection

!> @defgroup output Input/Ouput procedures
!! This group gather all the IO tools, including AVS IO and vtk xml output. 

!> @defgroup cart_structure Cartesian mpi topology and mesh
!! This group gather module devoted to mpi topology and some of the
!! associated mesh informations.
