!------------------------------------------------------------------------------
!
! PROGRAM : io_main
!
! DESCRIPTION: 
!> This files contains binary test on io procedures.
!!
!! @details
!! All these test are unit test : they return a logical value to check if 
!! the code version pass it or not. 
!!
!! That is all these test are logical function, they return true if the result
!! is the right one and false otherwise. 
!! This first version test only the output at vtk xml format in parallel context.
!
!> @author
!! Jean-Baptiste Lagaert, LEGI
!
!------------------------------------------------------------------------------

module io_aux

    implicit none

    public      :: test_io_output 


contains

function test_io_output() result(success)
    
    ! External Library
    use mpi
    ! Scales code
    use cart_topology
    use cart_mesh
    use parallel_io_bin
    ! Test procedures
    use test_common

    implicit none

    logical                                             :: success          ! logical result
    real(WP), dimension(N_proc(1),N_proc(2),N_proc(3))  :: field            ! constant field used in this test
    integer                                             :: tag_cst          ! tag for constant field used in this test
    integer                                             :: tagX,tagY,tagZ   ! tag for unconstant fields used in this test
    integer                                             :: i                ! some boucle indice
    
    
    success = .true.

    ! Initialize output context
    call parallel_io_init_all(4, nb_proc_dim, length, myrank, coord, './io_res/')
    call test_substatus('general context intialization', success, myrank)

    call parallel_io_init_field('cst', tag_cst)
    call parallel_io_init_field('X', tagX)
    call parallel_io_init_field('Y', tagY)
    call parallel_io_init_field('Z', tagZ)
    call test_substatus('fields context intialization', success, myrank)

    ! Make some output
    field = 1.0
    call parallel_write(tag_cst, field, 'cst')
    field = 2.0
    call parallel_write(tag_cst, field, 'cst')
    call test_substatus('constant output', success, myrank)

    do i = 1, N_proc(1)
        field(i,:,:) = i+coord(1)*N_proc(1)
    end do
    call parallel_write(tag_cst, field, 'X')
    field = 2*field
    call parallel_write(tag_cst, field, 'X')
    call test_substatus('X output', success, myrank)

    do i = 1, N_proc(2)
        field(:,i,:) = i+coord(2)*N_proc(2)
    end do
    call parallel_write(tag_cst, field, 'Y')
    call test_substatus('Y output', success, myrank)

    do i = 1, N_proc(3)
        field(:,:,i) = i+coord(3)*N_proc(3)
    end do
    call parallel_write(tag_cst, field, 'Z')
    call test_substatus('Z output', success, myrank)

    
    ! Free memory
    call parallel_io_finish()

    ! The main programm want receive a signal corresponding to error rather than success
    success = .not. success
   
end function test_io_output

end module io_aux
