!------------------------------------------------------------------------------
!
! PROGRAM : io_main
!
! DESCRIPTION: 
!> This program use the function implemented in the module io_aux to
!! test the io routines.
!!
!! @details
!! All these test are unit test : they return a logical value to check if 
!! the code version pass it or not. 
!!
!! That is all these test are logical function, they return true if the result
!! is the right one and false otherwise. 
!! This first version test only the output at vtk xml format in parallel context.
!
!> @author
!! Jean-Baptiste Lagaert, LEGI
!
!------------------------------------------------------------------------------

program io_main

    ! External Library
    use mpi
    ! Scales code
    use cart_topology
    use parallel_io_bin
    ! Test procedures
    use test_common
    use io_aux

    implicit none

    logical     :: error = .false.  ! logical error
    integer     :: ierr             ! mpi error code
    integer     :: rank_world       ! processus rank on "MPI_COMM_WORLD"
    integer     :: nb_proc,nb_procZ ! number of processus
    integer     :: i                ! some boucle indice
   
    ! ===== Initialisation =====

    ! Set the verbosity
    verbose_test = .true.
    verbose_more = .true.
    ! Initialise mpi
    call mpi_init(ierr)    
    call mpi_comm_rank(MPI_COMM_WORLD, rank_world, ierr)
    call mpi_comm_size(MPI_COMM_WORLD, nb_proc, ierr)

    ! Cut the domain along Y and initialize the toppology
    nb_procZ = 1
    if ((mod(nb_proc,5)==0).and.(mod(100, nb_proc/5)==0)) then
        nb_procZ = 5
        nb_proc = nb_proc/5
    else if ((mod(nb_proc,2)==0).and.(mod(100, nb_proc/2)==0)) then
        nb_procZ = 2
        nb_proc = nb_proc/2
    else
        if (mod(100, nb_proc)/=0) stop 'wrong number of processes : it have to divide 100'
    end if
    call cart_create((/ nb_proc, nb_procZ /), ierr)
    call discretisation_default()
    call mpi_barrier(MPI_COMM_WORLD, ierr)


    ! ===== Test about io procedures =====
    call mpi_barrier(MPI_COMM_WORLD, ierr)
    call test_title('io - parallel vtk xml', rank_world)
    error = test_io_output()
    call test_status(error, 'write output', rank_world)
    !error = test_io_input()
    !call test_status(error, 'write input', rank_world)

    call mpi_finalize(ierr)

end program io_main

