!------------------------------------------------------------------------------
!
! MODULE: topo_aux_interface
! DESCRIPTION: 
!> This module provides different tests to validate the interface with the different 
!! data structures.
!!
!! @details
!! Different automatic test are developped in order to check the interface between the 
!! two data structures (the one used for the particular method and the one from the 
!! spectral part). All these test are unit test : they return a logical value to check
!! if the library pass it or not. 
!!
!! That is all these test are logical function, they return true if the result
!! is the right one and false otherwise. 
!!
!! The following test are included :
!!      1 -> Test the interface between the data structure in the advection solver 
!!              based on particular method and the one used in the
!!              pseudo-spectral method.
!!      X -> TODO check the output wich could be done directly from this
!!          format.
!           
!
!> @author
!! Jean-Baptiste Lagaert, LEGI
!
!------------------------------------------------------------------------------

module topo_aux_interface

    use cart_topology
    use precision

    implicit none

    ! ===== Test the topology =====
    ! Public function
    public              :: test_topo_coupling

contains

!> Test the communication between the pseudo-spectral and particles method.
!!    @return success   = logical success (= false if the ierr pass the test)
!! @details
!!        Some advection-diffusion law on scalar field are solved by mixed numerical
!!    method. In such a case, the advection part are solved with a particular
!!    solver (with time order 2 and space-order of 2 or 4) and the diffusion part
!!    are compute with a pseudo-spectral solver. The operator splitting is a
!!    Strang splitting. All these method are provided in a parrallel
!!    implemention. The data distribution used in the pseudo-spectral solver is
!!    described in the module "datalayout" and the one used in the parrticle
!!    solver is described in cart_topology. Of course, to avoid useless
!!    communication and ensure efficient implementation, these data distribution
!!    are supposed to be the same between the storage of field in the real space
!!    described in "datalayout" and the one described in "cart_topology". This
!!    function are provided to test it and check if the procedure provided in the
!!    different solver are compatible.
function test_topo_coupling() result(success)

    use cart_topology

    logical     :: success

    success = .true.


    success = .not. success

end function test_topo_coupling

end module topo_aux_interface
