!------------------------------------------------------------------------------
!
! PROGRAM : topo_main
!
! DESCRIPTION: 
!> Test the cartesian topology and all the associated variable. 
!! test the advection solver.
!! This program perform all the test include in "topo_aux". This module provide
!! unit test, ie logical function wich return a logical error.
!! There is a verbosity parameter to decide to print on screen the status of
!! result of each test (and sub-test) or not.
!!
!! See topo_aux for a list of available test.
!!
!
!> @author
!! Jean-Baptiste Lagaert, LEGI
!
!------------------------------------------------------------------------------

program topo_main

    use mpi
    use topo_aux
    use topo_aux_interface
    use test_common

    implicit none

    logical     :: error = .true.   ! logical error
    integer     :: ierr             ! mpi error code
    integer     :: rank_world       ! processus rank on "MPI_COMM_WORLD"
    integer     :: nb_proc          ! number of processus
    
    ! Set the verbosity
    verbose_test = .true.
    verbose_more = .true.

    ! Initialise mpi
    call mpi_init(ierr)    
    call mpi_comm_size(MPI_COMM_WORLD, nb_proc, ierr)
    call mpi_comm_rank(MPI_COMM_WORLD, rank_world, ierr)
    call mpi_test_substatus(ierr, error, 'mpi initialization', rank_world)

    ! Initialize the topology and test is
    error=test_topo_init()
    call test_status(error, '(mpi) topology initialisation', rank_world)

    ! Initialize the topology and test is
    error=test_topo_perio()
    call test_status(error, 'periodicity', rank_world)

    ! Initialize the topology and test is
    error=test_topo_submesh()
    call test_status(error, 'subdomain size', rank_world)

    call mpi_finalize(ierr)

end program topo_main

