!> Temp modules for ppm_client
module ppmExample

  use ppm_module_init
  use ppm_module_data, only : ppm_kind_double
  use ppm_module_finalize
    
  !  use client_io
  use client_data, only: mk, dime
  ! Physical domain and grid
  use Domain
  ! Fields on the grid
  use Fields, only: init_fields, velocity, vorticity
  ! Topology
  use client_topology, only: init_topo,  topo
  ! Multigrid solver
  !use Solver, only : init_multigrid, solve_poisson
  
  use mpi
  use WrapFort

  implicit none

  include "ppm_numerics.h"

  integer, private :: info 

contains

  subroutine init_client()

    integer :: prec,tol
    ! MPI comm
    integer :: comm
    ! debug mode
    integer :: debug
    ! error status
    integer :: info
    !======================
    ! Init ppm 
    !======================
    prec = ppm_kind_double ! Defined in ppm_param.h
    comm = MPI_COMM_WORLD  ! 
    debug = 2
    tol = -10
    info = -1
    call ppm_init(dime,prec,tol,comm,debug,info)
    
    !======================
    ! Read and broadcast 
    ! some parameters 
    !======================
    ! call read_data()

    !======================
    ! Geometry and grid
    !======================
    call init_geometry()
    call init_grid()
    
    !======================
    ! Creates the topology
    !======================
    call init_topo(domain_minCoords, domain_maxCoords, domain_bc, domain_ghostsize, grid_resolution)
    
    !======================
    ! Fields allocation
    !======================
    call init_fields(domain_ghostsize, topo)
    
    !======================
    ! Init solver
    !======================
!    call init_multigrid(topo%ID, mesh%ID, domain_ghostsize, domain_bc)

    !======================
    ! Init Physics
    !======================
    velocity = 0.0_mk
    vorticity = 0.0_mk

    !======================
    ! Init Particles
    !======================
     
    print *, "end of parmes:ppm:init_client"

  end subroutine init_client

  subroutine main_client() bind(c,name='plouhmans')

    ! Multigrid parameters ...
    print *, 'run ppm simulation ...'
    ! init ppm ...
    call init_client()

!    call solve_poisson(topo%ID, stream_function, vorticity)
    
    call ppm_finalize(info)
    print *, 'end ppm simulation'
  end subroutine main_client

  subroutine read_data()

    ! Set precision
!    mpi_prec = MPI_DOUBLE_PRECISION

!!$    ! Read input parameters on proc 0
!!$    if(rank == 0) call readparams()
!!$    
!!$        
!!$    call MPI_BCast(runtag,256,MPI_CHARACTER,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(iruntag,1,MPI_INTEGER,0,MPI_COMM_WORLD,info)
!!$    !----------------------------------------------------------------------------!
!!$    ! MPI Broadcasts ...
!!$    !----------------------------------------------------------------------------!
!!$    call MPI_BCast(nx,dime,MPI_INTEGER,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(min_physg,dime,mpi_prec,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(max_physg,dime,mpi_prec,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(dt,1,mpi_prec,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(dt_max,1,mpi_prec,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(tend,1,mpi_prec,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(itend,1,mpi_prec,0,MPI_COMM_WORLD,info)
!!$    call MPI_BCast(nu,1,mpi_prec,0,MPI_COMM_WORLD,info)
!!$    call mpi_bcast(verbose,1,mpi_logical,0,mpi_comm_world,info)
!!$    call mpi_bcast(maxlev,1,mpi_integer,0,mpi_comm_world,info)

    

  end subroutine read_data

end module ppmexample
