/** \file ppm_wrapper.hpp Interfaces to ppm (fortran) routines
 */
#ifndef PPMWRAPPER_HPP
#define PPMWRAPPER_HPP 
#include"F2CMangle.hpp"
#include<mpi.h>
#include <cstring>

/** Namespace for ppm functions and subroutines */

namespace PPM 
{
  
  extern "C" {

    //F2C_GLOBAL(pass,PASS);
    // Init and close ppm
    void F2C_MODULE(ppm_module_init,ppm_init, PPM_MODULE_INIT,PPM_INIT)(int*, int*,int*,int*,int*,int*,int*,int*,int*);
    void F2C_MODULE(ppm_module_finalize, ppm_finalize,PPM_MODULE_FINALIZE,PPM_FINALIZE)(int&);
    // Display functions
    void F2C_MODULE(charfunctions, start, CHARFUNCTIONS, START)(double*,int*,char*,int );
    void F2C_MODULE(charfunctions, stop, CHARFUNCTIONS, stop)(double*,int*,char*,int );

    void F2C_MODULE(testppm,mult,TESTPPM,MULT)(double*,double*,int*);

    // Topologies
    //    void F2C_MODULE(ppm_module_mktopo, ppm_mktopo, PPM_MODULE_MKTOPO,PPM_MKTOPO)(int*, double*, int*,
    
    void F2C_MODULE(modtest, cas1, MODTEST,CAS1)(double*);
    void F2C_MODULE(modtest, cas2, MODTEST,CAS2)(double*);
    void F2C_MODULE(modtest, cas3, MODTEST,CAS3)(double*);
    void F2C_MODULE(modtest, cas4, MODTEST,CAS4)(double*);
    void F2C_MODULE(modtest, cas5, MODTEST,CAS5)(double*);
    void F2C_MODULE(modtest, cas6, MODTEST,CAS6)(double*);
    void F2C_MODULE(modtest, application3, MODTEST,APPLICATION3)(); 
    //    int F2C_MODULE(ppm_module_data,ppm_debug, PPM_MODULE_DATA,PPM_DEBUG);
  }


  /** A static class to call wrapped ppm functions */
  class wrapper 
  {
    
    static int _info; 

  public : 
    
    /** PPM initialization 
	\param problem dimension
	\param precision for real numbers
	\param exp tolerance
	\param MPI comm 
	\param debug mode value
	\param error status
	\param log unit value for io
	\param stderr unit value
	\param stdout unit value
    */
    static void init(int ndim, int MK,int tolexp, MPI::Intracomm& comm, int debug, int* info, int ppm_log_unit, int err, int out)
    {
      MPI_Fint Fcomm = MPI_Comm_c2f(comm);
      F2C_MODULE(ppm_module_init,ppm_init, PPM_MODULE_INIT,PPM_INIT)(&ndim,&MK,&tolexp,&Fcomm,&debug,info,&ppm_log_unit,&err,&out);
    }

    /** Terminates ppm library 
	\param[in,out] error status
    */ 
    static void finalize(int& info)
    {
      F2C_MODULE(ppm_module_finalize, ppm_finalize,PPM_MODULE_FINALIZE,PPM_FINALIZE)(info);
    }
    
    /** Wrapper to ppm substart function
	@param[in]  caller name
	@param[in,out] cpu time when this function is called
	@param[in] error status
    */
    static void substart(std::string& msg, double* t0, int* info)
    {
      size_t size = msg.size()+1;
      char * msgF = new char[size];
      strncpy(msgF, msg.c_str(),size);
      F2C_MODULE(charfunctions, start, CHARFUNCTIONS, START)(t0,info,msgF,strlen(msgF));
    }
    
    /** Wrapper to ppm substopt function
	@param[in]  caller name
	@param[in] cpu time when substart for this function has been called
	@param[in] error status
    */
    static void substop(std::string& msg, double* t0, int* info)
    {
      size_t size = msg.size()+1;
      char * msgF = new char[size];
      strncpy(msgF, msg.c_str(),size);      
      F2C_MODULE(charfunctions, stop, CHARFUNCTIONS, STOP)(t0,info,msgF,strlen(msgF));
    }
    
  };

}


#endif
