!> Tools for vtk output
!! \todo print a real and proper vtk output ...
module io_vtk

  use client_data
  implicit none

contains
  
  !> print velocity and vorticity to vtk file 
  !> 
  subroutine printToVTK(filename,iter,field1,field2,resolution,spacing,coordMin)

    ! File name prefix for velocity and vorticity outputs
    character(len=*), intent(in) :: filename
    ! Current iteration number
    integer, intent(in) :: iter
    real(mk), dimension(:,:,:), pointer :: field1, field2
    !> Number of points in each dir (1st index) for each sub (2nd index)
    integer, dimension(3),intent(in)  :: resolution
    real(mk),dimension(3),intent(in)::coordMin
    real(mk), dimension(3),intent(in) :: spacing

    character(len=60)           :: buffer
    integer :: nbpoints
    
    ! local resolution
    integer :: nx,ny,nz

    !local filename (depends on subproc number)
    character(len=30) :: localname,buffer2,localname2

    ! output = nb of iterations
    write(buffer,*) iter
    buffer = adjustl(buffer)
    ! output =  velocity
!    write(name_vit,'(A)') trim(filename)//"_velocity_"//trim(buffer)//".vtk"
 !   write(name_omg,'(A)') trim(filename)//"_vorticity_"//trim(buffer)//".vtk"

    !call res_vtk_vit(trim(name_vit))
    !call res_vtk_omega(trim(name_omg))

    nx = resolution(1)
    ny = resolution(2)
    nz = resolution(3)
    nbpoints = nx*ny*nz
    write(buffer2, *) rank
    buffer2 = adjustl(buffer2)
    localname = trim(filename)//"_velocity_it"//trim(buffer)//"_"//trim(buffer2)//".vti"
    localname2 = trim(filename)//"_vorticity_it"//trim(buffer)//"_"//trim(buffer2)//".vti"
    open(unit=11,file=localname,form="formatted")
    open(unit=12,file=localname2,form="formatted")
    write(11,'(A26)')"# vtk DataFile Version 3.0"
    write(11,'(A7)')"vecteur"
    write(11,'(A5)')"ASCII"
    write(11,'(A25)')"DATASET STRUCTURED_POINTS"
    write(11,'(A10,3(i6,1x))')"DIMENSIONS",nx,ny,nz
    write(11,'(a6,3(f10.5))')"ORIGIN", coordMin
    write(11,'(A7,3(f10.5))')"SPACING",spacing
    write(11,'(A10,i10)') "POINT_DATA",nbpoints
    write(11,'(A21)') "VECTORS velo FLOAT"
    write(12,'(A26)')"# vtk DataFile Version 3.0"
    write(12,'(A7)')"vecteur"
    write(12,'(A5)')"ASCII"
    write(12,'(A25)')"DATASET STRUCTURED_POINTS"
    write(12,'(A10,3(i6,1x))')"DIMENSIONS",nx,ny,nz
    write(12,'(a6,3(f10.5))')"ORIGIN", coordMin
    write(12,'(A7,3(f10.5))')"SPACING",spacing
    write(12,'(A10,i10)') "POINT_DATA",nbpoints
    write(12,'(A21)') "VECTORS vort FLOAT"
    write(11,'(3(f20.9))') field1(1:nx,1:ny,1:nz)
    write(12,'(3(f20.9))') field2(1:nx,1:ny,1:nz)
    close(11)
    close(12)
    
  end subroutine printToVTK
  

  !> print test function (scalar) values to a vtk file 
  !> Warning : only for 3D field
  subroutine printChi3dToVTK(filename,testFunc,resolution,spacing,coordMin)
    character(len=*), intent(in) :: filename
    real(mk), dimension(:,:,:), pointer :: testFunc
    real(mk), dimension(3),intent(in) :: spacing
    !> Number of points in each dir (1st index) for each sub (2nd index)
    integer, dimension(3),intent(in)  :: resolution
    real(mk),dimension(3),intent(in)::coordMin
 
    integer :: nbpoints
    ! sub proc number
    ! local resolution
    integer :: nx,ny,nz
    
    !local filename (depends on subproc number)
    character(len=30) :: localname,buffer

    ! output = nb of iterations
    nx = resolution(1)
    ny = resolution(2)
    nz = resolution(3)
    nbpoints = nx*ny*nz
    write(buffer, *) rank
    buffer = adjustl(buffer)
    localname = trim(filename)//"_chi_"//trim(buffer)//".vti"
    open(unit=11,file=localname,form="formatted")
    write(11,'(A26)')"# vtk DataFile Version 3.0"
    write(11,'(A7)')"scalaire"
    write(11,'(A5)')"ASCII"
    write(11,'(A25)')"DATASET STRUCTURED_POINTS"
    write(11,'(A10,3(i8,1x))')"DIMENSIONS",nx,ny,nz
    write(11,'(a6,3(f10.5))')"ORIGIN", coordMin
    write(11,'(A7,3(f10.5))')"SPACING",spacing
    write(11,'(A10,i10)') "POINT_DATA",nbpoints
    write(11,'(A21)') "SCALARS chi FLOAT"
    write(11,'(A21)') "LOOKUP_TABLE default" 
    write(11,'(f20.9)') testFunc(1:nx,1:ny,1:nz)

  end subroutine printChi3dToVTK
  
  subroutine printScalar3dToVTK(filename,scalar,resolution,spacing,coordMin)
    character(len=*), intent(in) :: filename
    real(mk), dimension(:,:,:), pointer :: scalar
    real(mk), dimension(3),intent(in) :: spacing
    !> Number of points in each dir (1st index) for each sub (2nd index)
    integer, dimension(3),intent(in)  :: resolution
    real(mk),dimension(3),intent(in)::coordMin
 
    integer :: nbpoints
    ! sub proc number
    ! local resolution
    integer :: nx,ny,nz
    
    !local filename (depends on subproc number)
    character(len=30) :: localname,buffer

    ! output = nb of iterations
    nx = resolution(1)
    ny = resolution(2)
    nz = resolution(3)
    nbpoints = nx*ny*nz
    write(buffer, *) rank
    buffer = adjustl(buffer)
    localname = trim(filename)//"_chi_"//trim(buffer)//".vti"
    open(unit=11,file=localname,form="formatted")
    write(11,'(A26)')"# vtk DataFile Version 3.0"
    write(11,'(A7)')"scalaire"
    write(11,'(A5)')"ASCII"
    write(11,'(A25)')"DATASET STRUCTURED_POINTS"
    write(11,'(A10,3(i4,1x))')"DIMENSIONS",nx,ny,nz
    write(11,'(a6,3(f10.5))')"ORIGIN", coordMin
    write(11,'(A7,3(f10.5))')"SPACING",spacing
    write(11,'(A10,i10)') "POINT_DATA",nbpoints
    write(11,'(A21)') "SCALARS scal FLOAT"
    write(11,'(A21)') "LOOKUP_TABLE default" 
    write(11,'(3(f20.9))') scalar(1:nx,1:ny,1:nz)

  end subroutine printScalar3dToVTK

  subroutine printPvtkFile(filename,spacing,coordMin,coordMax)
    
    ! File name prefix for the output
    character(len=*), intent(in) :: filename
    real(mk), dimension(3), intent(in) :: spacing
    real(mk),dimension(3),intent(in) ::coordMin,coordMax
    character(len=30)::outputname,subfilename
    real(mk)::x1,x2,y1,y2,z1,z2
    
    x1=coordMin(1)
    x2=coordMax(1)
    y1=coordMin(2)
    y2=coordMax(2)
    z1=coordMin(3)
    z2=coordMax(3)
    subfilename="boundaries_chi_0.vti"
    outputname = trim(filename)//".pvti"
    open(unit=11,file=outputname,form="formatted")
    write(11,'(A70)') "<VTKFile type=""PImageData"" version=""0.1"" byte_order=""LittleEndian"">"
    write(11,'(A25,6(f10.5),A30,3(f10.5),A15,3(f10.5),A3)') "<PImageData WholeExtent=""",x1,x2,y1,y2,z1,z2,&
         " "" GhostLevel=""2"" Origin=""",coordMin,""" Spacing=""",spacing,""""
    write(11,'(A30)') "<PPointData Scalars=”Chi”>"
    write(11,'(A70)') "<PDataArray type=""Float32"" Name=""Chi"" /> "
    write(11,'(A15)') "</PPointData>"
    write(11,'(A70)') "<Piece Extent=""x1 x2 y1 y2 z1 z2"" Source=""", subfilename,"""/>"

    write(11,'(A15)') "</PImageData>"

    write(11,'(A11)') "</VTKFile>"
    
  end subroutine printPvtkFile

end module io_vtk
 
