!> Some specific (i.e. only dedicated to a particular simulation ...) routines
!! => pertubation on velocity, offset introduction on velocity ...
module FieldsComputation
  
  use client_data, only: mk,pi
  use client_topology, only: ppm_t_topo
  use mpi, only:MPI_COMM_WORLD,MPI_DOUBLE_PRECISION,MPI_SUM
  implicit none

  private

  public :: fit_velocity, perturb

contains
 
  !> offset velocity to fit with desired flowrate (average)
  subroutine fit_velocity(vel,lengths,lowerPoint,upperPoint)
    ! Velocity, intent(inout)
    real(mk), dimension(:,:,:,:,:), pointer :: vel
    ! grid step, intent(in)
    !real(mk), dimension(:), pointer :: stepSize
    ! dims of the domain, intent(in)
    real(mk), dimension(:), pointer :: lengths
    real(mk), dimension(:), pointer :: lowerPoint,upperPoint
        
    !real(mk) :: theoretical_flowrate, computed_flowrate,surf
    real(mk) :: constant

    ! The flow is supposed to be along direction 1
    ! physical domain surface
    !surf = (upperPoint(3)-lowerPoint(3))*lengths(2)
    ! Value of the flow rate we want to fix
    !theoretical_flowrate = 1.
    !call computeFlowRate(vel, stepSize, topo,computed_flowrate)
    ! Correction is added to velocity ...
    constant = (upperPoint(3)-lowerPoint(3))/lengths(3) !(theoretical_flowrate - computed_flowrate)/surf
    !    print *, 'const', constant
    !  print *, computed_flowrate, '//',surf
    
    vel(1,:,:,:,1) = vel(1,:,:,:,1) + constant

  end subroutine fit_velocity
  
  !> Introduce a small perturbation to velocity
  subroutine perturb(vel, time)
    ! Velocity, intent(inout)
    real(mk), dimension(:,:,:,:,:), pointer :: vel
    ! current time
    real(mk) :: time

    vel(2,:,:,:,1) = sin(pi*(time-3.0))
    
  end subroutine perturb

  subroutine computeFlowRate(vel, stepSize, topo, flowRate)
    
    real(mk), dimension(:,:,:,:,:), pointer :: vel
    ! grid step, intent(in)
    real(mk), dimension(:), pointer :: stepSize
    type(ppm_t_topo), pointer :: topo
    real(mk), intent(out) :: flowRate
    
    integer, dimension(:,:), pointer :: nnodes => NULL()
    integer :: isub, isubl,info
    
    nnodes => topo%mesh(1)%nnodes
    
    do isub=1,topo%nsublist
       isubl=topo%isublist(isub)
       flowRate = vel(1,1,1,1,isub) + vel(1,1,1,nnodes(3,isubl),isub) +&
         vel(1,1,nnodes(2,isubl),1,isub) + vel(1,1,nnodes(2,isubl),nnodes(3,isubl),isub)
       flowRate = 0.25*flowRate + 0.5*(sum(vel(1,1,2:nnodes(2,isubl)-1,1,isub)) + &
            sum(vel(1,1,2:nnodes(2,isubl)-1,nnodes(3,isubl),isub)) &
            + sum(vel(1,1,1,2:nnodes(3,isubl)-1,isub)) + sum(vel(1,1,nnodes(2,isubl),2:nnodes(3,isubl)-1,isub))) &
            + sum(vel(1,1,2:nnodes(2,isubl)-1,2:nnodes(3,isubl)-1,isub))
       flowRate = stepSize(2)*stepSize(3)*flowRate
    enddo
    
    call MPI_AllReduce(flowRate, flowRate, 1, MPI_DOUBLE_PRECISION, MPI_SUM, MPI_COMM_WORLD,info)
    
  end subroutine computeFlowRate

end module FieldsComputation
