!> Tools to create and init the topology
!! Based on ppm
module client_topology
  
  use ppm_module_typedef, only : ppm_t_topo
  use ppm_module_mktopo
  use ppm_module_topo_get
  use ppm_module_mesh_define
  use ppm_module_data, only : ppm_param_assign_internal,ppm_param_decomp_cartesian,ppm_topo,&
       ppm_param_decomp_xpencil,ppm_param_decomp_ypencil,ppm_param_decomp_zpencil

  use client_data, only : mk,dime,dim3
  use parmesTools

  implicit none

  !> A pointer to a ppm topology
  type :: topoPtr
     !> pointer to ppm topo
     type(ppm_t_topo), pointer :: ptr
     !> id of the local mesh on the current topo (Warning : we consider only 1 mesh/topo, at the present time)
     integer :: meshid
  end type topoPtr

  !> A pointer to ppm topology object 
  type(ppm_t_topo), pointer :: topo => null()
  !> Array of 1D topologies for FFTW solvers
  type(topoPtr), dimension(dime) :: topo1D
  !> 1D topology for FFTW solvers
  type(ppm_t_topo), pointer :: topoY => NULL()
  !> number of sub-domains on the current (mpi)proc. This is for ppm and should be 1 in our case. 
  integer :: nsublist
  !> number of the current mpi proc according to ppm. May be different from rank and is required.
  integer :: isubl
  !> local mesh number
  integer :: meshNum

  private

  public PPMinitTopo, topo, meshNum, ppm_t_topo, getPPMLocalResolution,nsublist,isubl,createTopologyY,topoY

contains

  !> Create the topology - Based on ppm_topo_mkfield routine (interface = ppm_mktopo)
  subroutine PPMinitTopo(minPos,maxPos,bc,ghostsize,resolution)

    real(mk), dimension(:),pointer :: minpos,maxpos
    integer,  dimension(:),pointer :: bc
    integer,  dimension(:),pointer :: ghostsize
    integer,  dimension(:),pointer :: resolution
    
    ! Unused parameter that must be present as an input arg ...
    real(mk), dimension(:,:),pointer :: false_xp => NULL()
    real(mk), dimension(:),pointer :: subcost => NULL()
    integer :: false_np
    integer assigning, decomposition
    integer :: info, meshid, topoid

    info = -1
    topoid=0
    meshid =-1
    decomposition = ppm_param_decomp_cartesian  
    assigning     = ppm_param_assign_internal
    false_np = -1 ! Must be negative to have purely mesh-based decomposition
    ! topology
    ! Note FP: ghostsize are never used in mktopo for decomp_cartesian.
    ! Note2 FP: struct topo as input results in failure. 

    call ppm_mktopo(topoid,meshid,false_xp,false_np,decomposition,assigning,minPos(1:dime),maxPos(1:dime),bc, & 
         ghostsize,subcost,resolution(1:dime),info)
    !decomposition = ppm_param_decomp_xy_slab
    !ghostsize =0.0
    !CALL ppm_mktopo(topoid,meshid,false_xp,0,decomposition,assigning,minPos,maxPos,bc,&
    !     ghostsize,subcost,resolution,info)
    
    !meshid2=-1
    !nmxyc=resolution
    !nmxyc(1)=(resolution(1)-1)/2 +1
    !CALL ppm_mesh_define(topoid,meshid2,nmxyc,istartxyc,ndataxyc,info)

    !print *, 'iaiaaoaioaioa', ppm_topo(topoid)%t%mesh(meshid2)%nnodes
    if(info.ne.0) stop 'Topology:init initialisation failed'

    ! Get the created topology ...
    topo=>ppm_topo(topoid)%t
    
    ! We init ppm numbers for sub domains
    nsublist=topo%nsublist
    call parmesAssert(nsublist,1,'there are several sub-domains on the current process and that is not allowed.')
   !> Number of the current process for ppm. May be different from rank.
    isubl = topo%isublist(nsublist)
    !! If we need nsublist > 1, every place where isubl is used should look like :
    ! do isub=1,nsublist
    !   isubl=topo%isublist(isub)
    !   call something(...,isubl,...)
    ! enddo
 
    ! we suppose that there is only on mesh created on each subdomain 
    meshNum = topo%max_meshid
    call parmesAssert(meshNum,1,&
         'it seems that several meshes are defined on the current sub-domain, which may result in simulation failure.')

    print *, rank, "AAAAAAAAAAAAAA", shape(topo%mesh(meshNum)%nnodes), "nbnbnb", topo%mesh(meshNum)%nnodes
    
!!$    call ppm_topo_get_meshinfo(topoid,meshid,nm,istart,ndata,maxndata,isublist,nsublist,info)
!!$    
!!$    print *, 'nm', nm
!!$    print *, 'istart', istart
!!$    print *, 'ndata', ndata
!!$    print *, 'maxndata',maxndata
!!$    print *, 'isublist', isublist
!!$    print *, 'nsublist',nsublist
    ! Notes Franck:
    ! nsublist is the number of subdomains for one processus (mpi proc not physical proc ...)
    ! isublist[i] = global number of the subdomain i (i being the local number)
    ! Warning: it seems that isublist is of size mpi-number-of proc but only the 1:nsublist elements are relevent
    ! sub2proc[i] = global number of the processus on which subdomain i is attached 
    ! 
!!$    
!!    print *, '[', rank,'], subs :  ', topo%nsublist, '//', shape(topo%isublist),'//', topo%isublist(1)
!!$    print *, '======================= [', rank,'] ', shape(topo%sub2proc)
!!$    print *, '======================= [', rank,'] ', topo%sub2proc
!!$    
!!$    
!!$    print *, '======================= [', rank,'] ', shape(topo%mesh(1)%nnodes)
!!$    print *, '======================= [', rank,'] ', topo%mesh(1)%nnodes
!!$    print *, '======================= [', rank,'] ', shape(topo%mesh(1)%istart)
!!$    print *, '======================= [', rank,'] ', topo%mesh(1)%Nm

  end subroutine PPMinitTopo

  !> Return the local (mpi) subdomain resolution
  function getPPMLocalResolution(localTopo,meshid)
    
    !> topology of interest
    type(ppm_t_topo), pointer :: localTopo
    !> id of the mesh from which we need to get the resolution
    integer, intent(in) :: meshid
    integer,dimension(dim3) :: getPPMLocalResolution

    getPPMLocalResolution = 1
    getPPMLocalResolution(1:dime) = maxval(localTopo%mesh(meshid)%nnodes(:,localTopo%isublist(1:localTopo%nsublist)),2) 

  end function getPPMLocalResolution

  !> 1D topologies creation, for fftw solvers
  !! 
  subroutine create1DTopologies(minPos,maxPos,bc,resolution)
    
    real(mk), dimension(:),pointer :: minpos,maxpos
    integer,  dimension(:),pointer :: bc
    integer,  dimension(:),pointer :: resolution
    
    ! Unused parameter that must be present as an input arg ...
    real(mk), dimension(:,:),pointer :: false_xp => NULL()
    real(mk), dimension(:),pointer :: subcost => NULL()
    integer :: false_np
    integer :: assigning
    integer, dimension(dim3) :: decomposition
    integer :: info, meshid, topoid,i
    integer,dimension(dim3), parameter :: zeros = (/0,0,0/)
    
    info = 0
    decomposition(c_X) = ppm_param_decomp_xpencil
    decomposition(c_Y) = ppm_param_decomp_ypencil
    decomposition(c_Z) = ppm_param_decomp_zpencil
    assigning     = ppm_param_assign_internal
    false_np = 0 ! Purely mesh-based decomposition
    
    ! No ghosts for these topologies

    ! loop over dimensions
    do i=1,dime
       topoid = 0
       meshid = -1
       call ppm_mktopo(topoid,meshid,false_xp,0,decomposition(i),assigning,minpos(1:dime),maxpos(1:dime),bc,&
            zeros(1:dime),subcost,resolution(1:dime),info)
       topo1D(i)%ptr=>ppm_topo(topoid)%t
       topo1D(i)%meshid = meshid
    end do

    print *,"iijiji", topo1D(c_X)%ptr%mesh(meshid)%nnodes
    
  end subroutine create1DTopologies

  !> 1D topologies creation, for fftw solvers
  !! 
  subroutine createTopologyY(topoid,meshid,minPos,maxPos,bc,resolution)
    
    real(mk), dimension(:),pointer :: minpos,maxpos
    integer,  dimension(:),pointer :: bc
    integer,  dimension(:),pointer :: resolution
    integer, intent(inout) :: topoid
    integer, intent(inout) :: meshid

    ! Unused parameter that must be present as an input arg ...
    real(mk), dimension(:,:),pointer :: false_xp => NULL()
    real(mk), dimension(:),pointer :: subcost => NULL()
    integer :: false_np
    integer :: assigning
    integer  :: decomposition
    integer :: info
    integer,dimension(dime), parameter :: zeros = (/0,0/)
    
    info = 0

    decomposition = ppm_param_decomp_xpencil
    assigning     = ppm_param_assign_internal
    false_np = 0 ! Purely mesh-based decomposition
    
    ! No ghosts for these topologies
    ! loop over dimensions
    topoid = 0
    meshid = -1
    call ppm_mktopo(topoid,meshid,false_xp,0,decomposition,assigning,minpos(1:dime),maxpos(1:dime),bc,&
         zeros(1:dime),subcost,resolution(1:dime),info)
    topoY=>ppm_topo(topoid)%t
    
    print *,rank,"iijiji", topoY%mesh(meshid)%nnodes
    print *,rank,"iooooooooijiji", getPPMLocalResolution(topoY,meshid)
    
  end subroutine createTopologyY


end module client_topology
