/** \file testAllocatedPtr.cxx 
    Test F2003 C interoperability 
    Send a vector (i.e allocated pointer) to Fortran. Must get it back still properly allocated with a different content.

 */
#include<iostream>
#include <string>
#include "WrapC.hpp"
#include "ParmesDef.hpp"
#include<vector>
#include <math.h>

using namespace std ;

using Parmes::Def::real_t;

// Declare the Fortran subroutine
extern "C"  void wrapC2F_allocatedPtr(double*, int*, real_t*);


int main(int argc, char* argv[])
{

  int length = 12; 
  std::vector<real_t> myVector(length);
  for(int i = 0; i<length; ++i)
    myVector[i] = 2.3;
  
  real_t expectedContent = 1.1; 
  
  // Send myVector to the Fortran Wrapper.
  // myVector is supposed to be modified such that myVector[i] = expectedContent *i
  wrapC2F_allocatedPtr(&myVector[0], &length, &expectedContent);
  
  for(int i = 0; i<length ; ++i)
    cout << myVector[i] << endl;

  if(myVector.size()!= length)
    {
      cout << "ERROR" << endl;
      return 1;
    }
  
  
  for(int i = 0; i< length ; ++i)
    {
      real_t check = myVector[i] - expectedContent*(i+1);
      real_t tol = 1e-10;
      if(fabs(check)>tol)
	{
	  cout << "ERROR" << endl;
	  return 1;
	}
    }
   
}

