module testWrap

  ! The fortran wrapper from Parmes
  use WrapFort
  ! Some subroutines for tests
  use userMod

  implicit none
  
contains

    !> Send an already allocated C pointer to a Fortran subroutine.
  !! @param type(c_Ptr) a C pointer (void*)
  !! @param type(c_int) size of C pointer
  subroutine wrapC2F_allocatedPtr(cptr, sizeCptr, expectedContent) bind(C, name='wrapC2F_allocatedPtr')

    type(c_Ptr),intent(in),VALUE :: cptr
    integer (kind=c_int), intent(IN) :: sizeCptr
    real(kind = c_double), intent(IN) :: expectedContent

    real(kind=c_double), pointer, dimension(:) :: xp => NULL()

    if(NDEBUG) print *, '=== wrapC2F_allocatedPtr ===' 
    ! Associate cptr and xp. 
    call c_f_pointer (cptr, xp, (/sizeCPtr/))

    if(.not.associated(xp) ) then
       print *, 'Error, association failed'
    end if

    ! Do some stuff on xp ...
    xp(1) = -3.9
    call modifyX(xp,expectedContent)
    print *, '=== End wrapC2F_allocatedPtr === '

  end subroutine WrapC2F_allocatedPtr
  
  !> Send a NULL C pointer to a Fortran subroutine and get it back properly allocated 
  !! @param[inout] type(c_Ptr) a C pointer (void*)
  !! @param[out] type(c_int) size of C pointer
  subroutine wrapC2F_NULLPtr(vector) bind(C, name='wrapC2F_NULLPtr')

    type(C2FPtr) :: vector

    real(kind=c_double), pointer, dimension(:) :: xp => NULL()

    if(NDEBUG) print *, '=== wrapC2F_NULLPtr ===' 

    call cas5(xp)

    if(.not.associated(xp) ) then
       print *, 'Error, association failed'
    end if

    vector%length = size(xp)
    vector%elements = c_loc(xp(1))

    print *, '=== End of wrapC2F_NULLPtr === '
  end subroutine wrapC2F_NULLPtr

    !> Send a NULL C pointer to a Fortran subroutine and get it back properly allocated 
  !! @param[inout] type(c_Ptr) a C pointer (void*)
  !! @param[out] type(c_int) size of C pointer
  subroutine wrapC2F_NULLPtrBis(vector, length) bind(C, name='wrapC2F_NULLPtrBis')

    type(c_ptr),intent(inout) :: vector
    integer(c_int), intent(out) :: length

    real(kind=c_double), pointer, dimension(:) :: xp => NULL()

    if(NDEBUG) print *, '=== wrapC2F_NULLPtr ===' 

    call cas5(xp)

!!$    if(.not.associated(xp) ) then
!!$       print *, 'Error, association failed'
!!$    end if
!!$    length = size(xp)
!!$    vector= c_loc(xp(1))

    call aliasF2C(vector, xp, length)
    
    print *, '=== End of wrapC2F_NULLPtr === '
  end subroutine wrapC2F_NULLPtrBis

end module testWrap
