# -*- coding: utf-8 -*-
"""
@package Domain
Physical domain representation.
"""
from ..Param import *
from DiscreteDomain import DiscreteDomain


class ParticleField(DiscreteDomain):
    """
    Particle field.
    DiscreteDomain.DiscreteDomain specialization.
    """

    def __init__(self, grid):
        """
        Constructor.
        Create a list of Particle from each node of a given grid.

        @param grid Grid.Grid : grid to create Particles from
        """
        DiscreteDomain.__init__(self, grid.dimension)
        self.elementNumber = np.copy(grid.elementNumber)

    def setPositions(self, ppos):
        """
        Set positions as variable.
        Change exploring method.

        @param ppos : position variable.
        """
        self.positions = ppos.values

    def __str__(self):
        """
        ToString method.
        """
        s = "ParticleField of " + str(np.prod(self.elementNumber)) + " Particles"
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : ParticleField"
    print ParticleField.__doc__
