# -*- coding: utf-8 -*-
"""
@package Operator
Operator representation
"""
from ..Param import *
from ContinuousOperator import ContinuousOperator
from VelocityDOp import VelocityDOp


class VelocityOp(ContinuousOperator):
    """
    Velocity operator representation
    """

    def __init__(self, velocity, formula=None, period=10.):
        """
        Constructor.
        Create an Velocity operator from given variables velocity and scalar.

        @param velocity ContinuousVariable.ContinuousVariable : velocity variable.
        @param scalar ContinuousVariable.ContinuousVariable : scalar variable.
        """
        ContinuousOperator.__init__(self)
        ## advection velocity variable
        self.velocity = velocity
        ## advected scalar variable
        self.formula = formula
        self.period = period
        self.addVariable([self.velocity])

    def discretize(self, spec=None):
        """
        Velocity operator discretization method.
        Create an VelocityDOp.VelocityDOp from given specifications.

        @param spec : discretization specifications, not used.
        """
        if self.discreteOperator is None:
            self.discreteOperator = VelocityDOp(self)

    def __str__(self):
        """ToString method"""
        s = " Velocity operator (ContinuousOperator)"
        return s + "\n"

if __name__ == "__main__":
    print __doc__
    print "- Provided class : VelocityOp"
    print VelocityOp.__doc__
