"""
@package parmepy.problem.continuous

Abstract problem representation.
"""
from abc import ABCMeta, abstractmethod


class ContinuousProblem:
    """
    Abstract description of continuous problem.
    """

    __metaclass__ = ABCMeta

    @abstractmethod
    def __init__(self):
        """
        Create an empty problem.
        """
        print "CONTINUOUS PROBLEM"
        ## Operators of the problem.
        self.operators = []
        ## Discretization of the problem.
        self.discreteProblem = None

    @abstractmethod
    def discretize(self, spec=None):
        """
        Abstract method.
        Must be implemented by sub-class.

        @param spec : discretization specifications
        """
        raise NotImplementedError("Need to override method in a subclass")

    @abstractmethod
    def setSolver(self, solver):
        """
        Set solver for the discrete problem.
        It must have a discrete Problem.

        @param solver : solver to use.
        """
        if self.discreteProblem == None:
            raise NotImplementedError("Cannot set solver on a non discretized problem ")
        self.discreteProblem.setSolver(solver)

    @abstractmethod
    def setPrinter(self, printer):
        """
        Set result printer.
        It must have a discrete Problem.

        @param printer : printer to use.
        """
        if self.discreteProblem == None:
            raise NotImplementedError("Cannot set printer on a non discretized problem ")
        self.discreteProblem.setPrinter(printer)

    @abstractmethod
    def solve(self, T, dt):
        """
        Solving the problem.
        It must have a discrete Problem to solve.

        @param T : Simulation final time
        @param dt : Simulation time step
        """
        if self.discreteProblem == None:
            raise NotImplementedError("Cannot solve a non discretized problem ")
        self.discreteProblem.solve(T, dt)

if __name__ == "__main__":
    print __doc__
    print "- Provided class : ContinuousProblem"
    print ContinuousProblem.__doc__
