# -*- codingind utf-8 -*-
"""
@package operator
Discrete stretching representation
"""
from parmepy.constants import np, PARMES_REAL, ORDER
from parmepy.operator.discrete import DiscreteOperator
from parmepy.tools.cpu_data_transfer import Synchronize
import sys
import time
import io


class DifferentialOperator_d(DiscreteOperator):
    """
    Operator representation.
    DiscreteOperator.DiscreteOperator specialization.
    """

    def __init__(self, diffop, method=''):
        """
        Constructor.
        Create a Stretching operator on a given continuous domain.
        Work on a given field2 and field1 to compute the stretching term.

        @param stretch ind Stretching operator.
        """
        DiscreteOperator.__init__(self)
        self.field1 = diffop.field1
        self.field2 = diffop.field2
        self.choice = diffop.choice
        self.compute_time = 0.

    def setUp(self):
        #La topologie est recuperee des variables discretes
        self.topology = self.field1.topology
        self.topology = self.field2.topology
        self.meshSize = self.topology.mesh.space_step_size

    def apply(self):
        """
        Apply operator.
        """
        ind0a = self.topology.mesh.local_start[0]
        ind0b = self.topology.mesh.local_end[0] + 1
        ind1a = self.topology.mesh.local_start[1]
        ind1b = self.topology.mesh.local_end[1] + 1
        ind2a = self.topology.mesh.local_start[2]
        ind2b = self.topology.mesh.local_end[2] + 1
        ind0 = np.arange(ind0a,ind0b)
        ind1 = np.arange(ind1a,ind1b)
        ind2 = np.arange(ind2a,ind2b)

        if self.choice == 'divWU':
##           second order scheme
##           X components of temp and result
#            temp1 = ( self.field1[0][ind+1,ind,ind] * self.field2[0][ind+1,ind,ind] -\
#                     self.field1[0][ind-1,ind,ind] * self.field2[0][ind-1,ind,ind]) / (2. * self.meshSize[0])

#            temp2 = (self.field1[1][ind,ind+1,ind] * self.field2[0][ind,ind+1,ind] -\
#                     self.field1[1][ind,ind-1,ind] * self.field2[0][ind,ind-1,ind] ) / (2. * self.meshSize[1])
#
#            temp3 = (self.field1[2][ind,ind,ind+1] * self.field2[0][ind,ind,ind+1] -\
#                     self.field1[2][ind,ind,ind-1] * self.field2[0][ind,ind,ind-1] ) / (2. * self.meshSize[2])

#            self.result[0][ind,ind]=  temp1 + temp2 + temp3

##           Y components of temp and result
#            temp1 = (self.field1[0][ind+1,ind,ind] * self.field2[1][ind+1,ind,ind] -\
#                     self.field1[0][ind-1,ind,ind] * self.field2[1][ind-1,ind,ind] ) / (2. * self.meshSize[0])

#            temp2 = (self.field1[1][ind,ind+1,ind] * self.field2[1][ind,ind+1,ind] -\
#                     self.field1[1][ind,ind-1,ind] * self.field2[1][ind,ind-1,ind] ) / (2. * self.meshSize[1])
#
#            temp3 = (self.field1[2][ind,ind,ind+1] * self.field2[1][ind,ind,ind+1] -\
#                     self.field1[2][ind,ind,ind-1] * self.field2[1][ind,ind,ind-1] ) / (2. * self.meshSize[2])

#            self.result[1][ind,ind]= temp1 + temp2 +temp3

##           Z components of temp and result
#            temp1 = (self.field1[0][ind+1,ind,ind] * self.field2[2][ind+1,ind,ind] -\
#                     self.field1[0][ind-1,ind,ind] * self.field2[2][ind-1,ind,ind] ) / (2. * self.meshSize[0])

#            temp2 = (self.field1[1][ind,ind+1,ind] * self.field2[2][ind,ind+1,ind] -\
#                     self.field1[1][ind,ind-1,ind] * self.field2[2][ind,ind-1,ind] ) / (2. * self.meshSize[1])
#
#            temp3 = (self.field1[2][ind,ind,ind+1] * self.field2[2][ind,ind,ind+1] -\
#                     self.field1[2][ind,ind,ind-1] * self.field2[2][ind,ind,ind-1]) / (2. * self.meshSize[2])

            # Ghosts synchronization
            linenb = 0
            if (self.topology.rank == 0):
                time.sleep(2)
                print 'Bligne 1', field1[:,linenb,linenb]
                time.sleep(2)
                print 'Bligne 2', field1[linenb,:,linenb]
                time.sleep(2)
                print 'Bligne 3', field1[linenb,linenb,:]
                time.sleep(2)

            OpSynchronize = Synchronize(self.topology)
            OpSynchronize.apply(self.field2, self.field1)

            if (self.topology.rank == 0):
                time.sleep(2)
                print 'Aligne 1', field1[:,linenb,linenb]
                time.sleep(2)
                print 'Aligne 2', field1[linenb,:,linenb]
                time.sleep(2)
                print 'Aligne 3', field1[linenb,linenb,:]
                time.sleep(2)

#            OpSynchronize.apply(self.field2)
#            OpSynchronize.apply(self.field1)
#           X components of temp and result
            temp1 = (1.0 * self.field1[0][ind0-2,ind1a:ind1b,ind2a:ind2b] * self.field2[0][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field1[0][ind0-1,ind1a:ind1b,ind2a:ind2b] * self.field2[0][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field1[0][ind0+1,ind1a:ind1b,ind2a:ind2b] * self.field2[0][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field1[0][ind0+2,ind1a:ind1b,ind2a:ind2b] * self.field2[0][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (12. * self.meshSize[0])

            temp2 = (1.0 * self.field1[1][ind0a:ind0b,ind1-2,ind2a:ind2b] * self.field2[0][ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field1[1][ind0a:ind0b,ind1-1,ind2a:ind2b] * self.field2[0][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field1[1][ind0a:ind0b,ind1+1,ind2a:ind2b] * self.field2[0][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field1[1][ind0a:ind0b,ind1+2,ind2a:ind2b] * self.field2[0][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (12. * self.meshSize[1])

            temp3 = (1.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2-2] * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2-1] * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2+1] * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2+2] * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (12. * self.meshSize[2])

            tmp1 = np.array([temp1 + temp2 + temp3])

#           Y components of temp and result
            temp1 = (1.0 * self.field1[0][ind0-2,ind1a:ind1b,ind2a:ind2b] * self.field2[1][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field1[0][ind0-1,ind1a:ind1b,ind2a:ind2b] * self.field2[1][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field1[0][ind0+1,ind1a:ind1b,ind2a:ind2b] * self.field2[1][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field1[0][ind0+2,ind1a:ind1b,ind2a:ind2b] * self.field2[1][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (12. * self.meshSize[0])

            temp2 = (1.0 * self.field1[1][ind0a:ind0b,ind1-2,ind2a:ind2b] * self.field2[1][ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field1[1][ind0a:ind0b,ind1-1,ind2a:ind2b] * self.field2[1][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field1[1][ind0a:ind0b,ind1+1,ind2a:ind2b] * self.field2[1][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field1[1][ind0a:ind0b,ind1+2,ind2a:ind2b] * self.field2[1][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (12. * self.meshSize[1])

            temp3 = (1.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2-2] * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2-1] * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2+1] * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2+2] * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (12. * self.meshSize[2])

            tmp2 = np.array([temp1 + temp2 + temp3])

#           Z components of temp and result
            temp1 = (1.0 * self.field1[0][ind0-2,ind1a:ind1b,ind2a:ind2b] * self.field2[2][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field1[0][ind0-1,ind1a:ind1b,ind2a:ind2b] * self.field2[2][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field1[0][ind0+1,ind1a:ind1b,ind2a:ind2b] * self.field2[2][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field1[0][ind0+2,ind1a:ind1b,ind2a:ind2b] * self.field2[2][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (12. * self.meshSize[0])

            temp2 = (1.0 * self.field1[1][ind0a:ind0b,ind1-2,ind2a:ind2b] * self.field2[2][ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field1[1][ind0a:ind0b,ind1-1,ind2a:ind2b] * self.field2[2][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field1[1][ind0a:ind0b,ind1+1,ind2a:ind2b] * self.field2[2][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field1[1][ind0a:ind0b,ind1+2,ind2a:ind2b] * self.field2[2][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (12. * self.meshSize[1])

            temp3 = (1.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2-2] * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2-1] * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2+1] * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field1[2][ind0a:ind0b,ind1a:ind1b,ind2+2] * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (12. * self.meshSize[2])

            tmp3 = np.array([temp1 + temp2 + temp3])
            result = np.concatenate((np.array(tmp1), np.array(tmp2), np.array(tmp3)))
            return result

        elif self.choice == 'gradV':

            # Ghosts synchronization
            OpSynchronize = Synchronize(self.topology)
            OpSynchronize.apply(self.field2)


            maxgersh = np.zeros(2, dtype=PARMES_REAL, order=ORDER)

##           Fourth order scheme
#           X components of temp and result
#            temp1 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp1 = self.field2[0][...] * 0.
            temp1[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[0][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field2[0][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field2[0][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field2[0][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (12. * self.meshSize[0])

#            temp2 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp2 = self.field2[0][...] * 0.
            temp2[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = (1.0 * self.field2[0][ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field2[0][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field2[0][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field2[0][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (12. * self.meshSize[1])

#            temp3 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp3 = self.field2[0][...] * 0.
            temp3[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = (1.0 * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (12. * self.meshSize[2])

            maxstr1= np.max(abs(temp1)+abs(temp2)+abs(temp3))
            maxadv1= np.max(abs(temp1))

#           Y components of temp and result
#            temp4 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp4 = self.field2[1][...] * 0.
            temp4[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[1][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field2[1][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field2[1][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field2[1][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (12. * self.meshSize[0])

#            temp5 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp5 = self.field2[1][...] * 0.
            temp5[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[1][ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field2[1][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field2[1][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field2[1][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (12. * self.meshSize[1])

#            temp6 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp6 = self.field2[1][...] * 0.
            temp6[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (12. * self.meshSize[2])

            maxstr2= np.max(abs(temp4)+abs(temp5)+abs(temp6))
            maxadv2= np.max(abs(temp5))

#           Z components of temp and result
#            temp7 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp7 = self.field2[2][...] * 0.
            temp7[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[2][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field2[2][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field2[2][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field2[2][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (12. * self.meshSize[0])

#            temp8 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp8 = self.field2[2][...] * 0.
            temp8[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[2][ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field2[2][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field2[2][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field2[2][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (12. * self.meshSize[1])

#            temp9 = np.zeros((self.resolution), dtype=PARMES_REAL, order=ORDER)
            temp9 = self.field2[2][...] * 0.
            temp9[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (12. * self.meshSize[2])

            maxstr3= np.max(abs(temp7)+abs(temp8)+abs(temp9))
            maxadv3= np.max(abs(temp9))
            maxgersh[0] = max(maxstr1,maxstr2,maxstr3)
            maxgersh[1] = max(maxadv1,maxadv2,maxadv3)
            result = np.concatenate((np.array([temp1, temp2, temp3]),np.array([temp4, temp5, temp6]),np.array([temp7, temp8, temp9])))

            return result, maxgersh


        elif self.choice == 'gradS':

            # Ghosts synchronization
            OpSynchronize = SynchronizeS(self.topology)
            OpSynchronize.apply(self.field2)

            # Fourth order scheme
            temp1 = self.field2[...] * 0.
            temp1[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field2[ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field2[ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field2[ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field2[ind0+2,ind1a:ind1b,ind2a:ind2b]) / (12. * self.meshSize[0])

            temp2 = self.field2[...] * 0.
            temp2[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = (1.0 * self.field2[ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field2[ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field2[ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field2[ind0a:ind0b,ind1+2,ind2a:ind2b]) / (12. * self.meshSize[1])

            temp3 = self.field2[...] * 0.
            temp3[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = (1.0 * self.field2[ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field2[ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field2[ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field2[ind0a:ind0b,ind1a:ind1b,ind2+2]) / (12. * self.meshSize[2])

            result = np.array([temp1, temp2, temp3])

            return result

        elif self.choice == 'laplacianV':

            # Ghosts synchronization
            OpSynchronize = Synchronize(self.topology)
            OpSynchronize.apply(self.field2)


##           Fourth order scheme for the laplacian operator
#           X components of temp and result
            temp1 = self.field2[0][...] * 0.
            temp1[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = \
                    (-1./12. * self.field2[0][ind0-2,ind1a:ind1b,ind2a:ind2b] +
                     4./3. * self.field2[0][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     - 5./2. * self.field2[0][ind0+0,ind1a:ind1b,ind2a:ind2b] +\
                     4./3. * self.field2[0][ind0+1,ind1a:ind1b,ind2a:ind2b] +\
                     -1./12. * self.field2[0][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (self.meshSize[0]**2)

            temp2 = self.field2[0][...] * 0.
            temp2[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = \
                    ( -1./12.* self.field2[0][ind0a:ind0b,ind1-2,ind2a:ind2b] +\
                      4./3.* self.field2[0][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                      -5./2.* self.field2[0][ind0a:ind0b,ind1+0,ind2a:ind2b] +\
                      4./3.* self.field2[0][ind0a:ind0b,ind1+1,ind2a:ind2b] +\
                      -1./12.* self.field2[0][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (self.meshSize[1]**2)

            temp3 = self.field2[0][...] * 0.
            temp3[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = \
                    ( -1./12.* self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2-2] +\
                      4./3.* self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                      -5./2.* self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2+0] +\
                      4./3.* self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2+1] +\
                      -1./12.* self.field2[0][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (self.meshSize[2]**2)
            temp1 = temp1 + temp2 + temp3

#           Y components of temp and result
            temp4 = self.field2[1][...] * 0.
            temp4[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = \
                    (-1./12. * self.field2[1][ind0-2,ind1a:ind1b,ind2a:ind2b] +
                     4./3. * self.field2[1][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     - 5./2. * self.field2[1][ind0+0,ind1a:ind1b,ind2a:ind2b] +\
                     4./3. * self.field2[1][ind0+1,ind1a:ind1b,ind2a:ind2b] +\
                     -1./12. * self.field2[1][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (self.meshSize[0]**2)

            temp5 = self.field2[1][...] * 0.
            temp5[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = \
                    ( -1./12.* self.field2[1][ind0a:ind0b,ind1-2,ind2a:ind2b] +\
                      4./3.* self.field2[1][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                      -5./2.* self.field2[1][ind0a:ind0b,ind1+0,ind2a:ind2b] +\
                      4./3.* self.field2[1][ind0a:ind0b,ind1+1,ind2a:ind2b] +\
                      -1./12.* self.field2[1][ind0a:ind0b,ind1+2,ind2a:ind2b]) / (self.meshSize[1]**2)

            temp6 = self.field2[1][...] * 0.
            temp6[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = \
                    ( -1./12.* self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2-2] +\
                      4./3.* self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                      -5./2.* self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2+0] +\
                      4./3.* self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2+1] +\
                      -1./12.* self.field2[1][ind0a:ind0b,ind1a:ind1b,ind2+2]) / (self.meshSize[2]**2)
            temp4 = temp4 + temp5 + temp6

#           Z components of temp and result
            temp7 = self.field2[2][...] * 0.
            temp7[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = \
                    (-1./12. * self.field2[2][ind0-2,ind1a:ind1b,ind2a:ind2b] +
                     4./3. * self.field2[2][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     - 5./2. * self.field2[2][ind0+0,ind1a:ind1b,ind2a:ind2b] +\
                     4./3. * self.field2[2][ind0+1,ind1a:ind1b,ind2a:ind2b] +\
                     -1./12. * self.field2[2][ind0+2,ind1a:ind1b,ind2a:ind2b]) / (self.meshSize[0]**2)

            temp8 = self.field2[2][...] * 0.
            temp8[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = \
                    ( -1./12.* self.field2[2][ind0a:ind0b,ind1-2,ind2a:ind2b] +\
                      4./3.* self.field2[2][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                      -5./2.* self.field2[2][ind0a:ind0b,ind1+0,ind2a:ind2b] +\
                      4./3.* self.field2[2][ind0a:ind0b,ind1+1,ind2a:ind2b] +\
                      -1./12.* self.field2[2][ind0a:ind0b,ind1+2,ind2a:ind2b]) / ( self.meshSize[1]**2)

            temp9 = self.field2[2][...] * 0.
            temp9[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b]  = \
                    ( -1./12.* self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2-2] +\
                      4./3.* self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                      -5./2.* self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2+0] +\
                      4./3.* self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2+1] +\
                      -1./12.* self.field2[2][ind0a:ind0b,ind1a:ind1b,ind2+2]) / ( self.meshSize[2]**2)
            temp7 = temp7 + temp8 + temp9

            result = np.concatenate((np.array([temp1]),np.array([temp4]),np.array([temp7])))

            return result


        elif self.choice == 'curl':

            OpSynchronize = Synchronize(self.topology)
            OpSynchronize.apply(self.field1)

            temp1 = self.field1[0][...] * 0.
            temp2 = self.field1[0][...] * 0.

            temp1[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field1[2][ind0a:ind0b,ind1-2,ind2a:ind2b] -
                     8.0 * self.field1[2][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field1[2][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field1[2][ind0a:ind0b,ind1+2,ind2a:ind2b] ) / (12. * self.meshSize[1])

            temp2[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field1[1][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field1[1][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field1[1][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field1[1][ind0a:ind0b,ind1a:ind1b,ind2+2] ) / (12. * self.meshSize[2])
            res1 = temp1 - temp2

            temp1[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field1[0][ind0a:ind0b,ind1a:ind1b,ind2-2] -\
                     8.0 * self.field1[0][ind0a:ind0b,ind1a:ind1b,ind2-1] +\
                     8.0 * self.field1[0][ind0a:ind0b,ind1a:ind1b,ind2+1] -\
                     1.0 * self.field1[0][ind0a:ind0b,ind1a:ind1b,ind2+2] ) / (12. * self.meshSize[2])

            temp2[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field1[2][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field1[2][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field1[2][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field1[2][ind0+2,ind1a:ind1b,ind2a:ind2b] ) / (12. * self.meshSize[0])
            res2 = temp1 - temp2

            temp1[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field1[1][ind0-2,ind1a:ind1b,ind2a:ind2b] -
                     8.0 * self.field1[1][ind0-1,ind1a:ind1b,ind2a:ind2b] +\
                     8.0 * self.field1[1][ind0+1,ind1a:ind1b,ind2a:ind2b] -\
                     1.0 * self.field1[1][ind0+2,ind1a:ind1b,ind2a:ind2b] ) / (12. * self.meshSize[0])

            temp2[ind0a:ind0b,ind1a:ind1b,ind2a:ind2b] = (1.0 * self.field1[0][ind0a:ind0b,ind1-2,ind2a:ind2b] -\
                     8.0 * self.field1[0][ind0a:ind0b,ind1-1,ind2a:ind2b] +\
                     8.0 * self.field1[0][ind0a:ind0b,ind1+1,ind2a:ind2b] -\
                     1.0 * self.field1[0][ind0a:ind0b,ind1+2,ind2a:ind2b] ) / (12. * self.meshSize[1])
            res3 = temp1 - temp2
            result = np.concatenate((np.array([res1]), np.array([res2]), np.array([res3])))
            return result

        else:
            raise ValueError("Unknown differiential operatorind " + str(self.choice))

    def printComputeTime(self):
        self.timings_info[0] = "\"Differential Operator total\""
        self.timings_info[1] = str(self.total_time)
        self.timings_info[1] += str(self.compute_time[0]) + "  " + str(self.compute_time[1]) + "  " + str(self.compute_time[2]) + "  "
        print "Differential Operator total time ind ", self.total_time
        print "\t Differential Operator ind", self.compute_time


    def __str__(self):
        s = "DifferentialOperator_d (DiscreteOperator). " + DiscreteOperator.__str__(self)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class ind DifferentialOperator_d"
    print DifferentialOperator_d.__doc__
