"""
@package parmepy.problem.discrete

Problem representation.
"""
from ..Param import *


class DiscreteProblem:
    """
    Abstract description of discrete problem.
    """

    def __init__(self):
        """
        Create an empty discerte problem.
        """
        print "DISCRETE PROBLEM"
        ## Domains of the problem.
        self.domains = []
        ## Variables of the problem.
        self.variables = []
        ## Operators of the problem.
        self.operators = []
        ## Problem solver
        self.solver = None
        ## Results printer
        self.printer = None

    def addDomain(self, cDomain):
        """
        Add a DiscreteDomain.DiscreteDomain to the problem.

        @param cDomain DiscreteDomain.DiscreteDomain : list of domains to add.
        """
        print "Add domain"
        for d in cDomain:
            if self.domains.count(d) == 0:
                self.domains.append(d)

    def addVariable(self, cVariable):
        """
        Add a DiscreteVariable.DiscreteVariable to the problem.
        Also add variables' domains to the problem.

        @param cVariable DiscreteVariable.DiscreteVariable : list of variables to add.
        """
        print "add variable"
        for v in cVariable:
            if self.variables.count(v) == 0:
                self.variables.append(v)
            if self.domains.count(v.domain) == 0:
                self.domains.append(v.domain)

    def addOperator(self, cOperator, index=None):
        """
        Add a DiscreteOperator.DiscreteOperator to the problem.
        Also add operators' variables and variables' domains to the problem.

        @param cOperator DiscreteOperator.DiscreteOperator : list od operators to add.
        """
        print "add operator"
        if isinstance(cOperator, list):
            for i,o in enumerate(cOperator):
                if self.operators.count(o) == 0:
                    try:
                        self.operators.insert(index[i],o)
                    except TypeError:
                        self.operators.append(o)
                for v in o.variables:
                    if self.variables.count(v) == 0:
                        self.variables.append(v)
                    if self.domains.count(v.domain) == 0:
                        self.domains.append(v.domain)
        else:
            if self.operators.count(cOperator) == 0:
                print index
                try:
                    self.operators.insert(index, cOperator)
                except TypeError:
                    self.operators.append(cOperator)
            for v in cOperator.variables:
                if self.variables.count(v) == 0:
                    self.variables.append(v)
                if self.domains.count(v.domain) == 0:
                    self.domains.append(v.domain)

    def setSolver(self, solver):
        """
        Set solver for the discrete problem.

        @param solver : Solver.Solver to use.
        """
        print "set solver"
        self.solver = solver
        self.solver.initialize()

    def setPrinter(self, printer):
        """
        Set results printer.

        @param printer : Printer.Printer to use.
        """
        print "set printer"
        self.printer = printer

    def solve(self, T, dt):
        """
        Abstract method.
        Must be implemented by sub-class.

        @param T : Simulation final time.
        @param dt : Simulation time step.
        """
        raise NotImplementedError("Need to override method in a subclass of " + providedClass)


if __name__ == "__main__":
    print __doc__
    print "- Provided class : DiscreteProblem"
    print DiscreteProblem.__doc__
