# -*- coding: utf-8 -*-
"""
@package Utils
Forward Euler method.
"""
from ODESolver import ODESolver


providedClass = "ForwardEuler"


class ForwardEuler(ODESolver):
    """
    ODESolver implementation for solving an equation system with forward Euler method.
    y'(t) = f(t,y)

    y(t_{n+1}) = y(t_n) + dt*f(y(t_n))

    """
    def __init__(self, f=None, dim=1, conditions=lambda x: x):
        """
        Constructor.

        @param f function f(t,y) : Right hand side of the equation to solve.
        @param dim : dimensions
        @param conditions : function to apply boundary conditions.
        """
        ODESolver.__init__(self, f)
        # Boundary conditions function.
        self.boundaryConditions = conditions
        # Integration methods for each direction.
        self.integrate = [self._integrate for i in xrange(dim)]

    def _integrate(self, y, t, dt):
        """
        Integration step for forward Euler method.
         y(t_{n+1}) = y(t_n) + dt*f(y(t_n))

        @param y : position at time t.
        @param t : current time.
        @param dt : time step.
        @return y at t+dt.
        """
        return [y[i] + dt * self.f(t, y, i)[i] for i in xrange(len(y))]

    def __str__(self):
        """ToString method"""
        return "Forward Euler Method"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : " + providedClass
    print eval(providedClass).__doc__
