"""
Obstacles representation.
"""
from parmepy.constants import np, PARMES_REAL
from obstacle_d import Obstacle_d


class Obstacle(object):
    """
    Obstacles representation.
    """

    def __init__(self, domain, name='sphere', zlayer=0.001, radius=0.05,
                 center=[0.5, 0.5, 0.5], orientation='West',
                 porousLayer=0.001):
        """
        Constructor.
        Create an obsctacle and z-boundaries from given parameters :

        @param domain : obstacle application domain.
        @param name : name of the obstacle ("sphere" or "hemisphere")
        @param zlayer : width of z-boundaries layer
        @param radius : radius of the obstacle
        @param center : position of the obstacle center
        """
        ## Application domain of the obstacle
        self.domain = domain
        ## Obstacle dimension.
        self.dimension = self.domain.dimension
        ## Obstacle discretisation
        self.discreteObstacle = None
        ## Number of different discretizations
        self._obstacleId = -1
        ## List of the various discretizations of the obstacle
        self.discreteObstacle = []
        ## Obstacle name
        self.name = name
        ## Width of z-boundaries layer
        self.zlayer = zlayer
        ## Obstacle radius
        self.radius = radius
        ## Position of the obstacle center
        self.center = np.asarray(center, dtype=PARMES_REAL)
        ## Obstacle orientation
        self.orientation = orientation
        ## Thickness of the porous layer at the obstacle surface
        self.porousLayer = porousLayer

    def discretize(self, topology=None):
        """
        Obstacle discretization method.
        """
        self._obstacleId += 1
        self.discreteObstacle.append(Obstacle_d(self, topology,
                                                  name=self.name,
                                                  idFromParent=self._obstacleId))

    def __str__(self):
        """ Obstacle info display """
        s = self.name + ' , ' + str(self.dimension) + 'D obstacle '
        if len(self.Obstacle_d) != 0:
            s += "with the following discretizations:\n"
            for i in range(len(self.Obstacle_d)):
                s += self.Obstacle_d[i].__str__()
        else:
            s += ". Not discretised\n"
        return s

if __name__ == "__main__" :
    print __doc__
    print "- Provided class : Obstacle"
    print Obstacle.__doc__
