inline float alpha(float y, float s){
  return (y * (y * (y * (y * (13.0f - 5.0f * y) - 9.0f) - 1.0f) + 2.0f) / 24.0f) * s;}
inline float beta(float y, float s){
  return (y * (y * (y * (y * (25.0f * y - 64.0f) + 39.0f) + 16.0f) - 16.0f) / 24.0f)*s;}
inline float gamma(float y, float s){
  return (y * y * (y * (y * (126.0f - 50.0f * y) - 70.0f) - 30.0f) / 24.0f + 1.0f)*s;}
inline float delta(float y, float s){
  return (y * (y * (y * (y * (50.0f * y - 124.0f) + 66.0f) + 16.0f) + 16.0f) / 24.0f)*s;}
inline float eta(float y, float s){
  return (y * (y * (y * (y * (61.0f - 25.0f * y) - 33.0f) - 1.0f) - 2.0f) / 24.0f)*s;}
inline float zeta(float y, float s){
  return (y * y * y * (y * (5.0f * y - 12.0f) + 7.0f) / 24.0f)*s;}

#if DIM==3
__kernel void advection_3D_basic(__global const float* gvelo,
				 __global float* ppos,
				 float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  int ind, i_ind, i_ind_p;
  float invdx = 1.0f/dx;
  uint i;
  float v, y, p;
  for(i=gidX; i<WIDTH; i+=NB_WI_ADVECTION)
    {
      v = gvelo[i+gidY*WIDTH+gidZ*WIDTH*WIDTH];
      p = i*dx + 0.5f*dt*v;
      ind = convert_int_rtn(p * invdx);
      y = (p - convert_float(ind) * dx) * invdx;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((ind + 1) % WIDTH);
      p=(gvelo[i_ind+gidY*WIDTH+gidZ*WIDTH*WIDTH]*(1.0f - y) + gvelo[i_ind_p+gidY*WIDTH+gidZ*WIDTH*WIDTH]*y)*dt + i*dx + min_position;
      ppos[i+gidY*WIDTH+gidZ*WIDTH*WIDTH] = p;
    }
}
__kernel void advection_3D_float4(__global const float* gvelo,
				__global float* ppos,
				float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  int4 ind, i_ind, i_ind_p;
  float invdx = 1.0f/dx;
  uint i;
  float4 v, vp, y, p, coord;
  __local float velocity_cache[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=NB_WI_ADVECTION*4)
    {
      v = vload4((i+gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, gvelo);
      velocity_cache[i] = v.x;
      velocity_cache[i+1] = v.y;
      velocity_cache[i+2] = v.z;
      velocity_cache[i+3] = v.w;
    }

  barrier(CLK_LOCAL_MEM_FENCE);

  for(i=gidX*4; i<WIDTH; i+=NB_WI_ADVECTION*4)
    {
      v = vload4(i/4, velocity_cache);
      coord = (float4)(i*dx, (i+1)*dx, (i+2)*dx, (i+3)*dx);
      p = coord + 0.5f*dt*v;
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((i_ind + 1) % WIDTH);
      v = (float4)(velocity_cache[i_ind.x], velocity_cache[i_ind.y], velocity_cache[i_ind.z], velocity_cache[i_ind.w]);
      vp = (float4)(velocity_cache[i_ind_p.x], velocity_cache[i_ind_p.y], velocity_cache[i_ind_p.z], velocity_cache[i_ind_p.w]);
      p = (v*(1.0f - y) + vp*y)*dt + coord + (float4)(min_position);
      vstore4(p, (i+gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, ppos);
    }
}
__kernel void remeshing_3D_float4(__global const float* ppos,
		     __global const float* pscal,
		     __global float* gscal, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  float invdx = 1.0f/dx;
  int4 ind;
  uint i, nb_part = WIDTH/NB_WI_REMESHING;
  float4 p, s, y;
  uint4 index4;

  __local float gscal_loc[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=(NB_WI_REMESHING*4))
{
    gscal_loc[i] = 0.0f;
    gscal_loc[i+1] = 0.0f;
    gscal_loc[i+2] = 0.0f;
    gscal_loc[i+3] = 0.0f;
}
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*nb_part; i<(gidX + 1)*nb_part; i+=4)
    {
      p = vload4((i + gidY*WIDTH + gidZ*WIDTH*WIDTH)/4, ppos) - (float4)(min_position);
      s = vload4((i + gidY*WIDTH + gidZ*WIDTH*WIDTH)/4, pscal);
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      index4 = convert_uint4((ind - 2 + WIDTH) % WIDTH);
      gscal_loc[index4.x] += (alpha(y.x,s.x));
      gscal_loc[index4.y] += (alpha(y.y,s.y));
      gscal_loc[index4.z] += (alpha(y.z,s.z));
      gscal_loc[index4.w] += (alpha(y.w,s.w));
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gscal_loc[index4.x] += (beta(y.x,s.x));
      gscal_loc[index4.y] += (beta(y.y,s.y));
      gscal_loc[index4.z] += (beta(y.z,s.z));
      gscal_loc[index4.w] += (beta(y.w,s.w));
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gscal_loc[index4.x] += (gamma(y.x, s.x));
      gscal_loc[index4.y] += (gamma(y.y, s.y));
      gscal_loc[index4.z] += (gamma(y.z, s.z));
      gscal_loc[index4.w] += (gamma(y.w, s.w));
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gscal_loc[index4.x] += (delta(y.x, s.x));
      gscal_loc[index4.y] += (delta(y.y, s.y));
      gscal_loc[index4.z] += (delta(y.z, s.z));
      gscal_loc[index4.w] += (delta(y.w, s.w));
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gscal_loc[index4.x] += (eta(y.x, s.x));
      gscal_loc[index4.y] += (eta(y.y, s.y));
      gscal_loc[index4.z] += (eta(y.z, s.z));
      gscal_loc[index4.w] += (eta(y.w, s.w));
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gscal_loc[index4.x] += (zeta(y.x, s.x));
      gscal_loc[index4.y] += (zeta(y.y, s.y));
      gscal_loc[index4.z] += (zeta(y.z, s.z));
      gscal_loc[index4.w] += (zeta(y.w, s.w));
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*4; i<WIDTH; i+=(NB_WI_REMESHING*4))
    vstore4((float4)(gscal_loc[i],gscal_loc[i+1], gscal_loc[i+2], gscal_loc[i+3]), (i + gidY*WIDTH + gidZ*WIDTH*WIDTH)/4, gscal);
}

__kernel void copyLocMem_3D(__global const float* in,
			    __global float* out)
{
 uint xIndex = get_group_id(0) * TILE_DIM_COPY_3D + get_local_id(0);
 uint yIndex = get_group_id(1) * TILE_DIM_COPY_3D + get_local_id(1);
 uint zIndex = get_global_id(2);
 uint index = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

  __local float tile[TILE_DIM_COPY_3D][TILE_DIM_COPY_3D];

 for(uint i=0; i<TILE_DIM_COPY_3D; i+=BLOCK_ROWS_COPY_3D)
  {
    tile[get_local_id(1)+i][get_local_id(0)] = in[index + i*WIDTH];
  }
barrier(CLK_LOCAL_MEM_FENCE);
 for(uint i=0; i<TILE_DIM_COPY_3D; i+=BLOCK_ROWS_COPY_3D)
  {
    out[index + i*WIDTH] = tile[get_local_id(1)+i][get_local_id(0)];
  }
}

__kernel void transpose_3D_xy_coalesced_locPad_Diag(__global const float* in,
						    __global float* out)
{
 uint group_id_x = get_group_id(0);
 uint group_id_y = (get_group_id(0) + get_group_id(1)) % get_num_groups(0);

 uint xIndex = group_id_x * TILE_DIM_TRANSPOSE_3D_XY + get_local_id(0);
 uint yIndex = group_id_y * TILE_DIM_TRANSPOSE_3D_XY + get_local_id(1);
 uint zIndex = get_global_id(2);
 uint index_in = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

 xIndex = group_id_y * TILE_DIM_TRANSPOSE_3D_XY + get_local_id(0);
 yIndex = group_id_x * TILE_DIM_TRANSPOSE_3D_XY + get_local_id(1);
 uint index_out = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

  __local float tile[TILE_DIM_TRANSPOSE_3D_XY][TILE_DIM_TRANSPOSE_3D_XY+1];

 for(uint i=0; i<TILE_DIM_TRANSPOSE_3D_XY; i+=BLOCK_ROWS_TRANSPOSE_3D_XY)
  {
    tile[get_local_id(1) + i][get_local_id(0)] = in[index_in + i*WIDTH];
  }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(uint i=0; i<TILE_DIM_TRANSPOSE_3D_XY; i+=BLOCK_ROWS_TRANSPOSE_3D_XY)
  {
    out[index_out + i*WIDTH] = tile[get_local_id(0)][get_local_id(1) + i];
  }
}
__kernel void transpose_3D_xz_coalesced(__global const float* in,
					__global float* out)
{
 uint xIndex = get_group_id(0) * TILE_DIM_TRANSPOSE_3D_XZ + get_local_id(0);
 uint yIndex = get_group_id(1) * TILE_DIM_TRANSPOSE_3D_XZ + get_local_id(1);
 uint zIndex = get_group_id(2) * TILE_DIM_TRANSPOSE_3D_XZ + get_local_id(2);
 uint index_in = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

 xIndex = get_group_id(2) * TILE_DIM_TRANSPOSE_3D_XZ + get_local_id(0);
 zIndex = get_group_id(0) * TILE_DIM_TRANSPOSE_3D_XZ + get_local_id(2);
 uint index_out = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

  __local float tile[TILE_DIM_TRANSPOSE_3D_XZ][TILE_DIM_TRANSPOSE_3D_XZ][TILE_DIM_TRANSPOSE_3D_XZ];

for(uint j=0; j<TILE_DIM_TRANSPOSE_3D_XZ; j+=BLOCK_ROWS_TRANSPOSE_3D_XZ)
{
 for(uint i=0; i<TILE_DIM_TRANSPOSE_3D_XZ; i+=BLOCK_ROWS_TRANSPOSE_3D_XZ)
  {
    tile[get_local_id(2) + j][get_local_id(1) + i][get_local_id(0)] = in[index_in + i*WIDTH + j*WIDTH*WIDTH];
  }
}
  barrier(CLK_LOCAL_MEM_FENCE);
for(uint j=0; j<TILE_DIM_TRANSPOSE_3D_XZ; j+=BLOCK_ROWS_TRANSPOSE_3D_XZ)
{
  for(uint i=0; i<TILE_DIM_TRANSPOSE_3D_XZ; i+=BLOCK_ROWS_TRANSPOSE_3D_XZ)
  {
    out[index_out + i*WIDTH + j*WIDTH*WIDTH] = tile[get_local_id(0)][get_local_id(1)+i][get_local_id(2) + j];
  }
}
}
#else
__kernel void advection_2D_float4(__global const float* gvelo,
				  __global float* ppos,
				  float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  int4 ind, i_ind, i_ind_p;
  float invdx = 1.0f/dx;
  uint i;
  float4 v, vp, y, p, coord;
  __local float velocity_cache[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=NB_WI_ADVECTION*4)
    {
      v = vload4((i+gidY*WIDTH)/4, gvelo);
      velocity_cache[i] = v.x;
      velocity_cache[i+1] = v.y;
      velocity_cache[i+2] = v.z;
      velocity_cache[i+3] = v.w;
    }

  barrier(CLK_LOCAL_MEM_FENCE);

  for(i=gidX*4; i<WIDTH; i+=NB_WI_ADVECTION*4)
    {
      v = vload4(i/4, velocity_cache);
      coord = (float4)(i*dx, (i+1)*dx, (i+2)*dx, (i+3)*dx);
      p = coord + 0.5f*dt*v;
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((i_ind + 1) % WIDTH);
      v = (float4)(velocity_cache[i_ind.x], velocity_cache[i_ind.y], velocity_cache[i_ind.z], velocity_cache[i_ind.w]);
      vp = (float4)(velocity_cache[i_ind_p.x], velocity_cache[i_ind_p.y], velocity_cache[i_ind_p.z], velocity_cache[i_ind_p.w]);
      p = (v*(1.0f - y) + vp*y)*dt + coord + (float4)(min_position);
      vstore4(p, (i+gidY*WIDTH)/4, ppos);
    }
}

__kernel void advection_2D_basic(__global const float* gvelo,
				 __global float* ppos,
				 float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  int ind, i_ind, i_ind_p;
  float invdx = 1.0f/dx;
  uint i;
  float v, y, p;
  for(i=gidX; i<WIDTH; i+=NB_WI_ADVECTION)
    {
      v = gvelo[i+gidY*WIDTH];
      p = i*dx + 0.5f*dt*v;
      ind = convert_int_rtn(p * invdx);
      y = (p - convert_float(ind) * dx) * invdx;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((ind + 1) % WIDTH);
      p=(gvelo[i_ind+gidY*WIDTH]*(1.0f - y) + gvelo[i_ind_p+gidY*WIDTH]*y)*dt + i*dx + min_position;
      ppos[i+gidY*WIDTH] = p;
    }
}


__kernel void remeshing_2D_float4(__global const float* ppos,
				  __global const float* pscal,
				  __global float* gscal, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  float invdx = 1.0f/dx;
  int4 ind;
  uint i, nb_part = WIDTH/NB_WI_REMESHING;
  float4 p, s, y;
  float gsx, gsy, gsz, gsw;
  uint4 index4;

  __local float gscal_loc[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=(NB_WI_REMESHING*4))
{
    gscal_loc[i] = 0.0f;
    gscal_loc[i+1] = 0.0f;
    gscal_loc[i+2] = 0.0f;
    gscal_loc[i+3] = 0.0f;
}
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*nb_part; i<(gidX + 1)*nb_part; i+=4)
    {
      p = vload4((i + gidY*WIDTH)/4, ppos) - (float4)(min_position);
      s = vload4((i + gidY*WIDTH)/4, pscal);
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      index4 = convert_uint4((ind - 2 + WIDTH) % WIDTH);
      gsx = gscal_loc[index4.x];
      gsy = gscal_loc[index4.y];
      gsz = gscal_loc[index4.z];
      gsw = gscal_loc[index4.w];
      gsx = gsx + (alpha(y.x, s.x));
      gsy = gsy + (alpha(y.y, s.y));
      gsz = gsz + (alpha(y.z, s.z));
      gsw = gsw + (alpha(y.w, s.w));
      gscal_loc[index4.x] = gsx;
      gscal_loc[index4.y] = gsy;
      gscal_loc[index4.z] = gsz;
      gscal_loc[index4.w] = gsw;
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gsx = gscal_loc[index4.x];
      gsy = gscal_loc[index4.y];
      gsz = gscal_loc[index4.z];
      gsw = gscal_loc[index4.w];
      gsx = gsx + (beta(y.x, s.x));
      gsy = gsy + (beta(y.y, s.y));
      gsz = gsz + (beta(y.z, s.z));
      gsw = gsw + (beta(y.w, s.w));
      gscal_loc[index4.x] = gsx;
      gscal_loc[index4.y] = gsy;
      gscal_loc[index4.z] = gsz;
      gscal_loc[index4.w] = gsw;
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gsx = gscal_loc[index4.x];
      gsy = gscal_loc[index4.y];
      gsz = gscal_loc[index4.z];
      gsw = gscal_loc[index4.w];
      gsx = gsx + (gamma(y.x, s.x));
      gsy = gsy + (gamma(y.y, s.y));
      gsz = gsz + (gamma(y.z, s.z));
      gsw = gsw + (gamma(y.w, s.w));
      gscal_loc[index4.x] = gsx;
      gscal_loc[index4.y] = gsy;
      gscal_loc[index4.z] = gsz;
      gscal_loc[index4.w] = gsw;
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gsx = gscal_loc[index4.x];
      gsy = gscal_loc[index4.y];
      gsz = gscal_loc[index4.z];
      gsw = gscal_loc[index4.w];
      gsx = gsx + (delta(y.x, s.x));
      gsy = gsy + (delta(y.y, s.y));
      gsz = gsz + (delta(y.z, s.z));
      gsw = gsw + (delta(y.w, s.w));
      gscal_loc[index4.x] = gsx;
      gscal_loc[index4.y] = gsy;
      gscal_loc[index4.z] = gsz;
      gscal_loc[index4.w] = gsw;
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gsx = gscal_loc[index4.x];
      gsy = gscal_loc[index4.y];
      gsz = gscal_loc[index4.z];
      gsw = gscal_loc[index4.w];
      gsx = gsx + (eta(y.x, s.x));
      gsy = gsy + (eta(y.y, s.y));
      gsz = gsz + (eta(y.z, s.z));
      gsw = gsw + (eta(y.w, s.w));
      gscal_loc[index4.x] = gsx;
      gscal_loc[index4.y] = gsy;
      gscal_loc[index4.z] = gsz;
      gscal_loc[index4.w] = gsw;
barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gsx = gscal_loc[index4.x];
      gsy = gscal_loc[index4.y];
      gsz = gscal_loc[index4.z];
      gsw = gscal_loc[index4.w];
      gsx = gsx + (zeta(y.x, s.x));
      gsy = gsy + (zeta(y.y, s.y));
      gsz = gsz + (zeta(y.z, s.z));
      gsw = gsw + (zeta(y.w, s.w));
      gscal_loc[index4.x] = gsx;
      gscal_loc[index4.y] = gsy;
      gscal_loc[index4.z] = gsz;
      gscal_loc[index4.w] = gsw;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*4; i<WIDTH; i+=(NB_WI_REMESHING*4))
    vstore4((float4)(gscal_loc[i],gscal_loc[i+1], gscal_loc[i+2], gscal_loc[i+3]), (i + gidY*WIDTH)/4, gscal);
}

__kernel void copyLocMem_2D(__global const float* in,
			    __global float* out)
{
 uint xIndex = get_group_id(0) * TILE_DIM_COPY_2D + get_local_id(0);
 uint yIndex = get_group_id(1) * TILE_DIM_COPY_2D + get_local_id(1);
 uint index = xIndex + yIndex * WIDTH;

  __local float tile[TILE_DIM_COPY_2D][TILE_DIM_COPY_2D];

 for(uint i=0; i<TILE_DIM_COPY_2D; i+=BLOCK_ROWS_COPY_2D)
  {
    tile[get_local_id(1)+i][get_local_id(0)] = in[index + i*WIDTH];
  }
barrier(CLK_LOCAL_MEM_FENCE);
 for(uint i=0; i<TILE_DIM_COPY_2D; i+=BLOCK_ROWS_COPY_2D)
  {
    out[index + i*WIDTH] = tile[get_local_id(1)+i][get_local_id(0)];
  }
}

__kernel void transpose_2D_coalesced_locPad_Diag(__global const float* in,
						 __global float* out)
{
 uint group_id_x = get_group_id(0);
 uint group_id_y = (get_group_id(0) + get_group_id(1)) % get_num_groups(0);

 uint xIndex = group_id_x * TILE_DIM_TRANSPOSE_2D + get_local_id(0);
 uint yIndex = group_id_y * TILE_DIM_TRANSPOSE_2D + get_local_id(1);
 uint index_in = xIndex + yIndex * WIDTH;

 xIndex = group_id_y * TILE_DIM_TRANSPOSE_2D + get_local_id(0);
 yIndex = group_id_x * TILE_DIM_TRANSPOSE_2D + get_local_id(1);
 uint index_out = xIndex + yIndex * WIDTH;

  __local float tile[TILE_DIM_TRANSPOSE_2D][TILE_DIM_TRANSPOSE_2D+1];

 for(uint i=0; i<TILE_DIM_TRANSPOSE_2D; i+=BLOCK_ROWS_TRANSPOSE_2D)
  {
    tile[get_local_id(1) + i][get_local_id(0)] = in[index_in + i*WIDTH];
  }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(uint i=0; i<TILE_DIM_TRANSPOSE_2D; i+=BLOCK_ROWS_TRANSPOSE_2D)
  {
    out[index_out + i*WIDTH] = tile[get_local_id(0)][get_local_id(1) + i];
  }
}


#endif












/*

// volume computing
__kernel void volume(__global const float* pscal,
			__global float* volume,
			__private const float level,
			__private const float particle_volume,
			__private const int dir,
			__private const uint line_nb_pts,
			__private const float4 min_v,
			__private const float4 max_v,
			__private const uint4 nb,
			__private const float4 size
			)
{
  __private uint p;  // Particle line loop index
  __private uint stride_along_dir, stride_along_dir_scal;   // Buffer stride between two particle of the same line
  __private uint ind_line, ind_line_scal;  // Line begin buffer index
  __private float dx_dir;  // Space step along line
  __private float pscal_c;  // Particle scalar along line
  // Compute strides and index. Depend on array order (C order here)
  dx_dir = size[dir];

  if(DIM == 3){
    if (dir == 0)
      {
	kernel_init_3_0(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
      }
    else if (dir == 1)
      {
	kernel_init_3_1(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
      }
    else
      {
	kernel_init_3_2(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
      }}
  else{
  if (dir == 0)
    {
      kernel_init_2_0(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
    }
  else
    {
      kernel_init_2_1(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
    }
  }

  volume[ind_line_scal]= 0.0f;
  for(p=0;p<line_nb_pts;p++)
    {
      // read particle scalar
      pscal_c = pscal[ind_line_scal + p*stride_along_dir_scal]; // Read : 1float
      if (pscal_c > level)
	volume[ind_line_scal] += particle_volume;
    }
}




// Colorize scalar
__kernel void colorize(__global const float* pscal,
		       __global float* pcolor,
		       __global float* points,
		       __private const uint dir,
		       __private const uint line_nb_pts,
		       __private const float4 min,
		       __private const uint4 nb,
		       __private const float4 size
		       )
{
  __private uint p;
  __private float color, dx_dir;
  __private size_t stride_vec;
  __private uint stride_along_dir, stride_along_dir_scal;
  __private uint ind_line, ind_line_scal;
  __private float line_c[DIM];
  stride_vec = 1; // Les composantes des vecteurs sont à la suite dans les buffers
  dx_dir = size[dir];
  if(DIM == 3)
    {
      if (dir == 0)
	{
	  kernel_init_3_0(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
	  advection_init_3_0(size, line_c);
	}
      else if (dir == 1)
	{
	  kernel_init_3_1(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
	  advection_init_3_1(size, line_c);
	}
      else
	{
	  kernel_init_3_2(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
	  advection_init_3_2(size, line_c);
	}
    }
  else
    {
      if (dir == 0)
	{
	  kernel_init_2_0(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
	  advection_init_2_0(size, line_c);
	}
      else
	{
	  kernel_init_2_1(nb, &stride_along_dir, &stride_along_dir_scal, &ind_line, &ind_line_scal);
	  advection_init_2_1(size, line_c);
	}
    }

  for(p=0;p<line_nb_pts;p++)
    {
      // Write result in buffer
      line_c[dir] = p*dx_dir;
      points[ind_line + p*stride_along_dir] = min.s0 + line_c[0];
      points[ind_line + stride_vec + p*stride_along_dir] = min.s1 + line_c[1];
#if DIM == 3
      points[ind_line + 2*stride_vec + p*stride_along_dir] = min.s2 + line_c[2];
#else
      points[ind_line + 2*stride_vec + p*stride_along_dir] = 0.0f;
#endif
      color = pscal[ind_line_scal + p*stride_along_dir_scal];
      if (color > 0.5){
	pcolor[4*ind_line_scal + 4*p*stride_along_dir_scal] = 1.0f; //Red
	pcolor[4*ind_line_scal + 4*p*stride_along_dir_scal + 3] = 1.0f; //Alpha
      }
      else{
	pcolor[4*ind_line_scal + 4*p*stride_along_dir_scal] = 1.0f; //Red
	pcolor[4*ind_line_scal + 4*p*stride_along_dir_scal + 3] = 0.0f; //Alpha
      }
      pcolor[4*ind_line_scal + 4*p*stride_along_dir_scal + 1] = 0.0f; //Green
      pcolor[4*ind_line_scal + 4*p*stride_along_dir_scal + 2] = 0.0f; //Blue
    }

}
*/
