/**
 * 2D advection kernel.
 *
 * Computations are done using builtin functions on vector types.
 * A local array is used un local memory as a cache for velocity.
 *
 * @param gvelo Velocity.
 * @param ppos Particle position.
 * @param dt Time step.
 * @param min_position Domain lower point.
 * @param dx Space step.
 */
__kernel void advection(__global const float* gvelo,
			__global float* ppos,
			float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  int ind, i_ind, i_ind_p;
  float invdx = 1.0/dx;
  uint i;
  float v, y, p;
  for(i=gidX; i<WIDTH; i+=WGN)
    {
      v = gvelo[i+gidY*WIDTH];
      p = fma((float)(0.5*dt),v,i*dx);
      ind = convert_int_rtn(p * invdx);
      y = (fma(- convert_float(ind),dx,p))* invdx ;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((ind + 1) % WIDTH);
      p=fma(mix(gvelo[i_ind+gidY*WIDTH],gvelo[i_ind_p+gidY*WIDTH],y),dt,i*dx + min_position);
      ppos[i+gidY*WIDTH] = p;
    }
}
