
__kernel void advection_and_remeshing(__global const float* gvelo,
				      __global const float* pscal,
				      __global float* gscal,
				      float dt, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  float invdx = 1.0/dx;
  int4 ind, i_ind, i_ind_p;
  uint i, nb_part = WIDTH/WGN;
  float4 p, s, y, gs, v, vp, coord;
  uint4 index4;

  __local float gscal_loc[WIDTH];
  __local float gvelo_loc[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=(WGN*4))
    {
      v = vload4((i+gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, gvelo);
      gvelo_loc[i] = v.x;
      gvelo_loc[i+1] = v.y;
      gvelo_loc[i+2] = v.z;
      gvelo_loc[i+3] = v.w;
      gscal_loc[i] = 0.0;
      gscal_loc[i+1] = 0.0;
      gscal_loc[i+2] = 0.0;
      gscal_loc[i+3] = 0.0;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*nb_part; i<(gidX + 1)*nb_part; i+=4)
    {
      v = vload4(i/4, gvelo_loc);
      coord = (float4)(i*dx, (i+1)*dx, (i+2)*dx, (i+3)*dx);
      p = fma((float4)(0.5*dt),v,coord);
      ind = convert_int4_rtn(p * invdx);
      y = (fma(- convert_float4(ind),dx,p))* invdx ;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((i_ind + 1) % WIDTH);
      v = (float4)(gvelo_loc[i_ind.x], gvelo_loc[i_ind.y],
                   gvelo_loc[i_ind.z], gvelo_loc[i_ind.w]);
      vp = (float4)(gvelo_loc[i_ind_p.x], gvelo_loc[i_ind_p.y],
                    gvelo_loc[i_ind_p.z], gvelo_loc[i_ind_p.w]);
      p=fma(mix(v,vp,y),dt,coord);

      s = vload4((i + gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, pscal);
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      index4 = convert_uint4((ind - 2 + WIDTH) % WIDTH);
      barrier(CLK_LOCAL_MEM_FENCE);
      gs = (alpha(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      //barrier(CLK_LOCAL_MEM_FENCE);
      gs = (beta(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      //barrier(CLK_LOCAL_MEM_FENCE);
      gs =  (gamma(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      //barrier(CLK_LOCAL_MEM_FENCE);
      gs =  (delta(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      //barrier(CLK_LOCAL_MEM_FENCE);
      gs = (eta(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      //barrier(CLK_LOCAL_MEM_FENCE);
      gs =  (zeta(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*4; i<WIDTH; i+=(WGN*4))
    vstore4((float4)(gscal_loc[i],gscal_loc[i+1], gscal_loc[i+2], gscal_loc[i+3]), (i + gidY*WIDTH+gidZ*WIDTH*WIDTH)/4, gscal);
}
