
inline uint noBC_id(int id, int nb_part){
  return (id%nb_part)*WGN+(id/nb_part);
}
__kernel void advection_and_remeshing(__global const float* gvelo,
				      __global const float* pscal,
				      __global float* gscal,
				      float dt,float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  float invdx = 1.0/dx;
  int8 ind, i_ind, i_ind_p;
  uint i, nb_part = WIDTH/WGN;
  float8 p, s, y, gs, v, vp, coord;
  uint8 index8;

  __local float gscal_loc[WIDTH];
  __local float gvelo_loc[WIDTH];

  for(i=gidX*8; i<WIDTH; i+=(WGN*8))
    {
      v = vload8((i+gidY*WIDTH)/8, gvelo);
      gvelo_loc[noBC_id(i,nb_part)] = v.s0;
      gvelo_loc[noBC_id(i+1,nb_part)] = v.s1;
      gvelo_loc[noBC_id(i+2,nb_part)] = v.s2;
      gvelo_loc[noBC_id(i+3,nb_part)] = v.s3;
      gvelo_loc[noBC_id(i+4,nb_part)] = v.s4;
      gvelo_loc[noBC_id(i+5,nb_part)] = v.s5;
      gvelo_loc[noBC_id(i+6,nb_part)] = v.s6;
      gvelo_loc[noBC_id(i+7,nb_part)] = v.s7;
      gscal_loc[i] = 0.0;
      gscal_loc[i+1] = 0.0;
      gscal_loc[i+2] = 0.0;
      gscal_loc[i+3] = 0.0;
      gscal_loc[i+4] = 0.0;
      gscal_loc[i+5] = 0.0;
      gscal_loc[i+6] = 0.0;
      gscal_loc[i+7] = 0.0;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*nb_part; i<(gidX + 1)*nb_part; i+=8)
    {
      v = (float8)(gvelo_loc[noBC_id(i,nb_part)],
                   gvelo_loc[noBC_id(i+1,nb_part)],
                   gvelo_loc[noBC_id(i+2,nb_part)],
                   gvelo_loc[noBC_id(i+3,nb_part)],
                   gvelo_loc[noBC_id(i+4,nb_part)],
                   gvelo_loc[noBC_id(i+5,nb_part)],
                   gvelo_loc[noBC_id(i+6,nb_part)],
                   gvelo_loc[noBC_id(i+7,nb_part)]);
      coord = (float8)(i*dx,
		       (i+1)*dx,
		       (i+2)*dx,
		       (i+3)*dx,
		       (i+4)*dx,
		       (i+5)*dx,
		       (i+6)*dx,
		       (i+7)*dx);
      p = fma((float8)(0.5*dt),v,coord);
      ind = convert_int8_rtn(p * invdx);
      y = (fma(- convert_float8(ind),dx,p))* invdx ;
      i_ind = ((ind + WIDTH) % WIDTH);
      i_ind_p = ((i_ind + 1) % WIDTH);
      v = (float8)(gvelo_loc[noBC_id(i_ind.s0,nb_part)],
                   gvelo_loc[noBC_id(i_ind.s1,nb_part)],
                   gvelo_loc[noBC_id(i_ind.s2,nb_part)],
                   gvelo_loc[noBC_id(i_ind.s3,nb_part)],
                   gvelo_loc[noBC_id(i_ind.s4,nb_part)],
                   gvelo_loc[noBC_id(i_ind.s5,nb_part)],
                   gvelo_loc[noBC_id(i_ind.s6,nb_part)],
                   gvelo_loc[noBC_id(i_ind.s7,nb_part)]);
      vp = (float8)(gvelo_loc[noBC_id(i_ind_p.s0,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.s1,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.s2,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.s3,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.s4,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.s5,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.s6,nb_part)],
                    gvelo_loc[noBC_id(i_ind_p.s7,nb_part)]);
      p = fma(mix(v,vp,y),dt,coord);

      s = vload8((i + gidY*WIDTH)/8, pscal);
      ind = convert_int8_rtn(p * invdx);
      y = (p - convert_float8(ind) * dx) * invdx;
      index8 = convert_uint8((ind - 3 + WIDTH) % WIDTH);
      gscal_loc[noBC_id(index8.s0,nb_part)] += (alpha(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (alpha(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (alpha(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (alpha(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (alpha(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (alpha(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (alpha(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (alpha(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
      index8 = (index8 + 1) % WIDTH;
      gscal_loc[noBC_id(index8.s0,nb_part)] += (beta(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (beta(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (beta(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (beta(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (beta(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (beta(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (beta(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (beta(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
      index8 = (index8 + 1) % WIDTH;
      gscal_loc[noBC_id(index8.s0,nb_part)] += (gamma(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (gamma(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (gamma(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (gamma(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (gamma(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (gamma(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (gamma(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (gamma(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
      index8 = (index8 + 1) % WIDTH;
      gscal_loc[noBC_id(index8.s0,nb_part)] += (delta(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (delta(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (delta(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (delta(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (delta(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (delta(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (delta(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (delta(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
      index8 = (index8 + 1) % WIDTH;
      gscal_loc[noBC_id(index8.s0,nb_part)] += (eta(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (eta(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (eta(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (eta(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (eta(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (eta(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (eta(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (eta(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
      index8 = (index8 + 1) % WIDTH;
      gscal_loc[noBC_id(index8.s0,nb_part)] += (zeta(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (zeta(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (zeta(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (zeta(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (zeta(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (zeta(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (zeta(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (zeta(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
      index8 = (index8 + 1) % WIDTH;
      gscal_loc[noBC_id(index8.s0,nb_part)] += (theta(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (theta(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (theta(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (theta(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (theta(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (theta(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (theta(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (theta(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
      index8 = (index8 + 1) % WIDTH;
      gscal_loc[noBC_id(index8.s0,nb_part)] += (iota(y.s0,s.s0));
      gscal_loc[noBC_id(index8.s1,nb_part)] += (iota(y.s1,s.s1));
      gscal_loc[noBC_id(index8.s2,nb_part)] += (iota(y.s2,s.s2));
      gscal_loc[noBC_id(index8.s3,nb_part)] += (iota(y.s3,s.s3));
      gscal_loc[noBC_id(index8.s4,nb_part)] += (iota(y.s4,s.s4));
      gscal_loc[noBC_id(index8.s5,nb_part)] += (iota(y.s5,s.s5));
      gscal_loc[noBC_id(index8.s6,nb_part)] += (iota(y.s6,s.s6));
      gscal_loc[noBC_id(index8.s7,nb_part)] += (iota(y.s7,s.s7));
      barrier(CLK_LOCAL_MEM_FENCE);
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*8; i<WIDTH; i+=(WGN*8))
    vstore8((float8)(gscal_loc[noBC_id(i,nb_part)],
                     gscal_loc[noBC_id(i+1,nb_part)],
                     gscal_loc[noBC_id(i+2,nb_part)],
                     gscal_loc[noBC_id(i+3,nb_part)],
                     gscal_loc[noBC_id(i+4,nb_part)],
                     gscal_loc[noBC_id(i+5,nb_part)],
                     gscal_loc[noBC_id(i+6,nb_part)],
                     gscal_loc[noBC_id(i+7,nb_part)]), (i + gidY*WIDTH)/8, gscal);
}
