
inline uint noBC_id(int id, int nb_part){
  return (id%nb_part)*WGN+(id/nb_part);
}
__kernel void remeshing(__global const float* ppos,
			__global const float* pscal,
			__global float* gscal, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  float invdx = 1.0/dx;
  int4 ind;
  uint i, nb_part = WIDTH/WGN;
  float4 p, s, y, gs;
  uint4 index4;

  __local float gscal_loc[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=(WGN*4))
    {
      gscal_loc[i] = 0.0;
      gscal_loc[i+1] = 0.0;
      gscal_loc[i+2] = 0.0;
      gscal_loc[i+3] = 0.0;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*nb_part; i<(gidX + 1)*nb_part; i+=4)
    {
      p = vload4((i + gidY*WIDTH)/4, ppos) - (float4)(min_position);
      s = vload4((i + gidY*WIDTH)/4, pscal);
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      index4 = convert_uint4((ind - 2 + WIDTH) % WIDTH);
      gs =  (alpha(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (beta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs = (gamma(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs = (delta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (eta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs = (zeta(y, s));
      gscal_loc[noBC_id(index4.x, nb_part)] += gs.x;
      gscal_loc[noBC_id(index4.y, nb_part)] += gs.y;
      gscal_loc[noBC_id(index4.z, nb_part)] += gs.z;
      gscal_loc[noBC_id(index4.w, nb_part)] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*4; i<WIDTH; i+=(WGN*4))
    vstore4((float4)(gscal_loc[noBC_id(i, nb_part)],gscal_loc[noBC_id(i+1, nb_part)], gscal_loc[noBC_id(i+2, nb_part)], gscal_loc[noBC_id(i+3, nb_part)]), (i + gidY*WIDTH)/4, gscal);
}
