
__kernel void remeshing(__global const float* ppos,
			__global const float* pscal,
			__global float* gscal, float min_position, float dx)
{
  uint gidX = get_global_id(0);
  uint gidY = get_global_id(1);
  uint gidZ = get_global_id(2);
  float invdx = 1.0/dx;
  int4 ind;
  uint i, nb_part = WIDTH/WGN;
  float4 p, s, y, gs;
  uint4 index4;

  __local float gscal_loc[WIDTH];

  for(i=gidX*4; i<WIDTH; i+=(WGN*4))
    {
      gscal_loc[i] = 0.0;
      gscal_loc[i+1] = 0.0;
      gscal_loc[i+2] = 0.0;
      gscal_loc[i+3] = 0.0;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*nb_part; i<(gidX + 1)*nb_part; i+=4)
    {
      p = vload4((i + gidY*WIDTH + gidZ*WIDTH*WIDTH)/4, ppos) - (float4)(min_position);
      s = vload4((i + gidY*WIDTH + gidZ*WIDTH*WIDTH)/4, pscal);
      ind = convert_int4_rtn(p * invdx);
      y = (p - convert_float4(ind) * dx) * invdx;
      index4 = convert_uint4((ind - 2 + WIDTH) % WIDTH);
      gs =  (alpha(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (beta(y, s));
      gscal_loc[index4.x]+= gs.x;
      gscal_loc[index4.y]+= gs.y;
      gscal_loc[index4.z]+= gs.z;
      gscal_loc[index4.w]+= gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (gamma(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (delta(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (eta(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
      index4 = (index4 + 1) % WIDTH;
      gs =  (zeta(y, s));
      gscal_loc[index4.x] += gs.x;
      gscal_loc[index4.y] += gs.y;
      gscal_loc[index4.z] += gs.z;
      gscal_loc[index4.w] += gs.w;
      barrier(CLK_LOCAL_MEM_FENCE);
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(i=gidX*4; i<WIDTH; i+=(WGN*4))
    vstore4((float4)(gscal_loc[i],
                     gscal_loc[i+1],
                     gscal_loc[i+2],
                     gscal_loc[i+3]),
            (i + gidY*WIDTH + gidZ*WIDTH*WIDTH)/4, gscal);
}
