
__kernel void transpose_xy(__global const float* in,
			   __global float* out)
{
  float4 temp;
  uint group_id_x = get_group_id(0);
  uint group_id_y = (get_group_id(0) + get_group_id(1)) % get_num_groups(0);

  uint xIndex = group_id_x * TILE_DIM_XY + get_local_id(0)*4;
  uint yIndex = group_id_y * TILE_DIM_XY + get_local_id(1);
  uint zIndex = get_global_id(2);
  uint index_in = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

  xIndex = group_id_y * TILE_DIM_XY + get_local_id(0)*4;
  yIndex = group_id_x * TILE_DIM_XY + get_local_id(1);
  uint index_out = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

  __local float tile[TILE_DIM_XY][TILE_DIM_XY+1];

  for(uint i=0; i<TILE_DIM_XY; i+=BLOCK_ROWS_XY)
    {
      temp = vload4((index_in + i*WIDTH)/4, in);
      tile[get_local_id(1) + i][get_local_id(0)*4] = temp.x;
      tile[get_local_id(1) + i][get_local_id(0)*4+1] = temp.y;
      tile[get_local_id(1) + i][get_local_id(0)*4+2] = temp.z;
      tile[get_local_id(1) + i][get_local_id(0)*4+3] = temp.w;
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(uint i=0; i<TILE_DIM_XY; i+=BLOCK_ROWS_XY)
    {
      temp = (float4)(tile[get_local_id(0)*4][get_local_id(1) + i],
		      tile[get_local_id(0)*4+1][get_local_id(1) + i],
		      tile[get_local_id(0)*4+2][get_local_id(1) + i],
		      tile[get_local_id(0)*4+3][get_local_id(1) + i]);
      vstore4(temp, (index_out + i*WIDTH)/4, out);
    }
}
