
__kernel void transpose_xz(__global const float* in,
			   __global float* out)
{
  uint group_id_x = get_group_id(0);
  uint group_id_z = (get_group_id(0) + get_group_id(2)) % get_num_groups(0);

  uint xIndex = group_id_x * TILE_DIM_XZ + get_local_id(0);
  uint yIndex = get_group_id(1) * TILE_DIM_XZ + get_local_id(1);
  uint zIndex = group_id_z * TILE_DIM_XZ + get_local_id(2);
  uint index_in = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

  xIndex = group_id_z * TILE_DIM_XZ + get_local_id(0);
  zIndex = group_id_x * TILE_DIM_XZ + get_local_id(2);
  uint index_out = xIndex + yIndex * WIDTH + zIndex * WIDTH * WIDTH;

  __local float tile[TILE_DIM_XZ][TILE_DIM_XZ][TILE_DIM_XZ];

  for(uint j=0; j<TILE_DIM_XZ; j+=BLOCK_ROWS_XZ)
    {
      for(uint i=0; i<TILE_DIM_XZ; i+=BLOCK_ROWS_XZ)
	{
	  tile[get_local_id(2) + j][get_local_id(1) + i][get_local_id(0)] = in[index_in + i*WIDTH + j*WIDTH*WIDTH];
	}
    }
  barrier(CLK_LOCAL_MEM_FENCE);
  for(uint j=0; j<TILE_DIM_XZ; j+=BLOCK_ROWS_XZ)
    {
      for(uint i=0; i<TILE_DIM_XZ; i+=BLOCK_ROWS_XZ)
	{
	  out[index_out + i*WIDTH + j*WIDTH*WIDTH] = tile[get_local_id(0)][get_local_id(1)+i][get_local_id(2) + j];
	}
    }
}
