# -*- coding: utf-8 -*-
"""
@package Utils
RemeshingMethod interface.
"""
from RemeshingMethod import RemeshingMethod


providedClass = "Lambda1"


class Lambda1(RemeshingMethod):
    """
    Lambda1 remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        raise Exception("Obsolete methods : Lambda1 (not implemented with tag)")
        RemeshingMethod.__init__(self)
        self.grid = grid
        self.gvalues = gvalues

    def remesh(self, ppos, pscal):
        """
        Abstract method, apply operaton on a variable.
        Must be implemented by sub-class.

        @param ppos : particle position.
        @param pscal : particle scalar.
        @param gvalues : grid scalar field.
        """
        ii = xrange(len(ppos))
        # Calcul des indices sur la grille
        indGrid = [int((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i]) for i in ii]
        iX = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        iXp = [iX[i] for i in ii]
        iXp[0] = (iXp[0] + 1) % self.grid.elementNumber[0]
        # Calcul de la distance
        y = [(ppos[i] - self.grid[iX][i]) / self.grid.elementSize[i] for i in ii]
        # Remaillage
        self.gvalues.values[tuple(iX)] += (1. - y[0]) * pscal
        self.gvalues.values[tuple(iXp)] += y[0] * pscal

    def __str__(self):
        """ToString method"""
        return "Lambda1 kernel"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : " + providedClass
    print eval(providedClass).__doc__


class Lambda12D(RemeshingMethod):
    """
    Lambda1 remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        self.grid = grid
        self.gvalues = gvalues

    def remesh(self, ppos, pscal):
        """
        \Lambda_1 2D tensorial remeshing kernel implementation.
        Remesh the given particle on the grid according to \Lambda_1 remeshing kernel.

        @param part Particle.Particle : particle to remesh
        """
        ii = xrange(len(ppos))
        # Calcul des l'indices sur la grille
        indGrid = [int((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i]) for i in ii]
        iXY = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        iXpY = [iXY[i] for i in ii]
        iXpY[0] = (iXpY[0] + 1) % self.grid.elementNumber[0]
        iXYp = [iXY[i] for i in ii]
        iXYp[1] = (iXYp[1] + 1) % self.grid.elementNumber[1]
        iXpYp = [iXY[i] for i in ii]
        iXpYp[0] = (iXpYp[0] + 1) % self.grid.elementNumber[0]
        iXpYp[1] = (iXpYp[1] + 1) % self.grid.elementNumber[1]
        # Calcul de la distance
        y = [(ppos[i] - self.grid[iXY][i]) / self.grid.elementSize[i] for i in ii]
        # Remaillage
        self.gvalues.values[tuple(iXY)] += (1. - y[0]) * (1. - y[1]) * pscal
        self.gvalues.values[tuple(iXpY)] += (y[0]) * (1. - y[1]) * pscal
        self.gvalues.values[tuple(iXYp)] += (1. - y[0]) * (y[1]) * pscal
        self.gvalues.values[tuple(iXpYp)] += (y[0]) * (y[1]) * pscal

    def __str__(self):
        """ToString method"""
        return "Lambda1 kernel"


class Lambda13D(RemeshingMethod):
    """
    Lambda1 remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        self.grid = grid
        self.gvalues = gvalues

    def remesh(self, ppos, pscal):
        """
        \Lambda_1 3D tensorial remeshing kernel implementation.
        Remesh the given particle on the grid according to \Lambda_1 remeshing kernel.

        @param part Particle.Particle : particle to remesh
        """
        ii = xrange(len(ppos))
        # Calcul des l'indices sur la grille
        indGrid = [int((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i]) for i in ii]
        iXYZ = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        iXpYZ = [iXYZ[i] for i in ii]
        iXpYZ[0] = (iXpYZ[0] + 1) % self.grid.elementNumber[0]
        iXYpZ = [iXYZ[i] for i in ii]
        iXYpZ[1] = (iXYpZ[1] + 1) % self.grid.elementNumber[1]
        iXYZp = [iXYZ[i] for i in ii]
        iXYZp[2] = (iXYZp[2] + 1) % self.grid.elementNumber[2]
        iXpYpZ = [iXYZ[i] for i in ii]
        iXpYpZ[0] = (iXpYpZ[0] + 1) % self.grid.elementNumber[0]
        iXpYpZ[1] = (iXpYpZ[1] + 1) % self.grid.elementNumber[1]
        iXpYZp = [iXYZ[i] for i in ii]
        iXpYZp[0] = (iXpYZp[0] + 1) % self.grid.elementNumber[0]
        iXpYZp[2] = (iXpYZp[2] + 1) % self.grid.elementNumber[2]
        iXYpZp = [iXYZ[i] for i in ii]
        iXYpZp[1] = (iXYpZp[1] + 1) % self.grid.elementNumber[1]
        iXYpZp[2] = (iXYpZp[2] + 1) % self.grid.elementNumber[2]
        iXpYpZp = [iXYZ[i] for i in ii]
        iXpYpZp[0] = (iXpYpZp[0] + 1) % self.grid.elementNumber[0]
        iXpYpZp[1] = (iXpYpZp[1] + 1) % self.grid.elementNumber[1]
        iXpYpZp[2] = (iXpYpZp[2] + 1) % self.grid.elementNumber[2]
        # Calcul des distances
        y = [(ppos[i] - self.grid[iXYZ][i]) / self.grid.elementSize[i] for i in ii]
        # Remaillage
        self.gvalues.values[tuple(iXYZ)] += (1. - y[0]) * (1. - y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXpYZ)] += (y[0]) * (1. - y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXYpZ)] += (1. - y[0]) * (y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXYZp)] += (1. - y[0]) * (1. - y[1]) * (y[2]) * pscal
        self.gvalues.values[tuple(iXpYpZ)] += (y[0]) * (y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXpYZp)] += (y[0]) * (1. - y[1]) * (y[2]) * pscal
        self.gvalues.values[tuple(iXYpZp)] += (1. - y[0]) * (y[1]) * (y[2]) * pscal
        self.gvalues.values[tuple(iXpYpZp)] += (y[0]) * (y[1]) * (y[2]) * pscal

    def __str__(self):
        """ToString method"""
        return "Lambda1 kernel"
