# -*- coding: utf-8 -*-
"""
@package Utils
RemeshingMethod interface.
"""
from RemeshingMethod import RemeshingMethod
import math

providedClass = "Lambda2"


class Lambda2(RemeshingMethod):
    """
    Lambda2 remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        RemeshingMethod.__init__(self, grid, gvalues)
        # Epsilon value to avoid rounding errors.
        self.epsilon = self.grid.elementSize[0] * 0.000000001

    def nint(self, x):
        """
        Nearest interger function.

        @param x : real number.
        @return : x nearest integer.
        """
        if x > -0.5:
            return int(x + 0.5)
        else:
            return int(x + 0.5) - 1

    def remesh_Tag_Center(self, ppos, pscal, pposP, pscalP, splittingDirection):
        """
        Remeshing method for Tagged Center particles.

        @param ppos : particle position.
        @param pscal : particle scalar.
        @param pposP : following particle position.
        @param pscalP : following particle scalar.
        @param splittingDirection : direction to remesh.
        """
        print "Remaillage Tag Centré"
        ii = xrange(len(ppos))
        d = splittingDirection
        # Calcul des indices sur la grille
        indGrid = [self.nint((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i] + self.epsilon) for i in ii]
        indGrid_bis = [int(math.floor((pposP[i] - self.grid.min[i]) / self.grid.elementSize[i] + self.epsilon)) for i in ii]
        i0 = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        i0_bis = [indGrid_bis[i] % self.grid.elementNumber[i] for i in ii]
        i0m = [(i0[i] - 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        i0p = [(i0[i] + 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        # Calcul de la distance
        x0 = (ppos[d] - i0[d] * self.grid.elementSize[d] - self.grid.min[d]) / self.grid.elementSize[d]
        x0_bis = (pposP[d] - i0_bis[d] * self.grid.elementSize[d] - self.grid.min[d]) / self.grid.elementSize[d]
        if x0 <= 0.5:  # Article Table 2, case (d')
            # Calcul des poids
            a0m = 0.5 * x0 * (x0 - 1.)
            a0 = 1. - a0m
            b0p = 0.5 * x0_bis * (x0_bis + 1.)
            b0 = 1. - b0p
        else:  # Article Table 2, case (c')
            # Calcul des poids
            a0m = 0.5 * (x0 - 1.) * (x0 - 2.)
            a0 = 1. - a0m
            b0p = 0.5 * x0_bis * (x0_bis + 1.)
            b0 = 1. - b0p
        # Remaillage
        self.gvalues.values[tuple(i0m)] += pscal * a0m
        self.gvalues.values[tuple(i0)] += pscal * a0 + pscalP * b0
        self.gvalues.values[tuple(i0p)] += pscalP * b0p

    def remesh_Tag_Left(self, ppos, pscal, pposP, pscalP, splittingDirection):
        """
        Remeshing method for Tagged Left particles.

        @param ppos : particle position.
        @param pscal : particle scalar.
        @param pposP : following particle position.
        @param pscalP : following particle scalar.
        @param splittingDirection : direction to remesh.
        """
        print "Remaillage Tag Left"
        ii = xrange(len(ppos))
        d = splittingDirection
        # Calcul des indices sur la grille
        indGrid = [int(math.floor((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i] + self.epsilon)) for i in ii]
        indGrid_bis = [self.nint((pposP[i] - self.grid.min[i]) / self.grid.elementSize[i] + self.epsilon) for i in ii]
        i0 = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        i0_bis = [indGrid_bis[i] % self.grid.elementNumber[i] for i in ii]
        i0m = [(i0[i] - 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        i0p = [(i0[i] + 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        i0p2 = [(i0[i] + 2) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        i0p3 = [(i0[i] + 3) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        # Calcul de la distance
        x0 = (ppos[d] - i0[d] * self.grid.elementSize[d] - self.grid.min[d]) / self.grid.elementSize[d]
        x0_bis = (pposP[d] - i0_bis[d] * self.grid.elementSize[d] - self.grid.min[d]) / self.grid.elementSize[d]
        if x0_bis <= 0.5:  # Article Table 1, case (d)
            # Calcul des poids
            am = 0.5 * x0 * (x0 - 1.)
            a0 = 1. - x0 * x0
            ap = x0
            ap2 = 0.5 * x0 * (x0 - 1.)
            b0 = 0.5 * (x0_bis + 1.) * x0_bis
            bp = -x0_bis
            bp2 = 1. - x0_bis * x0_bis
            bp3 = 0.5 * x0_bis * (x0_bis + 1.)
        else:  # Article Table 1, case (c)
            # Calcul des poids
            am = 0.5 * x0 * (x0 - 1.)
            a0 = 1. - x0 * x0
            ap = x0
            ap2 = 0.5 * x0 * (x0 - 1.)
            b0 = 0.5 * x0_bis * (x0_bis - 1.)
            bp = 1. - x0_bis
            bp2 = x0_bis * (2. - x0_bis)
            bp3 = 0.5 * x0_bis * (x0_bis - 1.)
        # Remaillage
        self.gvalues.values[tuple(i0m)] += pscal * am
        self.gvalues.values[tuple(i0)] += pscal * a0 + pscalP * b0
        self.gvalues.values[tuple(i0p)] += pscal * ap + pscalP * bp
        self.gvalues.values[tuple(i0p2)] += pscal * ap2 + pscalP * bp2
        self.gvalues.values[tuple(i0p3)] += pscalP * bp3

    def remesh_noTag_Center(self, ppos, pscal, splittingDirection):
        """
        Remeshong method for non-Tagged Center particles.

        @param ppos : particle position.
        @param pscal : particle scalar.
        @param splittingDirection : direction to remesh.
        """
        #print "Remaillage noTag Centre"
        ii = xrange(len(ppos))
        d = splittingDirection
        # Calcul des indices sur la grille
        indGrid = [self.nint((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i] + self.epsilon) for i in ii]
        i0 = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        i0p = [(i0[i] + 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        i0p2 = [(i0[i] + 2) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        i0m = [(i0[i] - 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        # Calcul de la distance
        x0 = (ppos[d] - indGrid[d] * self.grid.elementSize[d] - self.grid.min[d]) / self.grid.elementSize[d]
        if x0 <= 0.5:
            # Calcul des poids
            am = 0.5 * x0 * (x0 - 1.)
            a0 = 1. - x0 * x0
            ap = 0.5 * x0 * (x0 + 1.)
            # Remaillage
            self.gvalues.values[tuple(i0m)] += pscal * am
            self.gvalues.values[tuple(i0)] += pscal * a0
            self.gvalues.values[tuple(i0p)] += pscal * ap
        else:
            # Calcul des poids
            am = 0.5 * (x0 - 1.) * (x0 - 2.)
            a0 = x0 * (2. - x0)
            ap = 0.5 * x0 * (x0 - 1.)
            # Remaillage
            self.gvalues.values[tuple(i0)] += pscal * am
            self.gvalues.values[tuple(i0p)] += pscal * a0
            self.gvalues.values[tuple(i0p2)] += pscal * ap

    def remesh_noTag_Left(self, ppos, pscal, splittingDirection):
        """
        Remeshing method for non-Tagged Left particles.

        @param ppos : particle position.
        @param pscal : particle scalar.
        @param splittingDirection : direction to remesh.
        """
        #print "Remaillage no Tag Left"
        ii = xrange(len(ppos))
        d = splittingDirection
        # Calcul des indices sur la grille
        indGrid = [int(math.floor((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i] + self.epsilon)) for i in ii]
        i0 = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        i0p = [(i0[i] + 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        i0m = [(i0[i] - 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        # Calcul de la distance
        x0 = (ppos[d] - indGrid[d] * self.grid.elementSize[d] - self.grid.min[d]) / self.grid.elementSize[d]
        # Calcul des poids
        am = 0.5 * x0 * (x0 - 1.)
        a0 = 1. - x0 * x0
        ap = 0.5 * x0 * (x0 + 1.)
        # Remaillage
        self.gvalues.values[tuple(i0m)] += pscal * am
        self.gvalues.values[tuple(i0)] += pscal * a0
        self.gvalues.values[tuple(i0p)] += pscal * ap

    def __str__(self):
        """ToString method"""
        return "Lambda2 kernel"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : " + providedClass
    print eval(providedClass).__doc__


class Lambda22D(RemeshingMethod):
    """
    Lambda2 remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        raise Exception("Obsolete methods : Must use 1D version with splitting.")
        self.grid = grid
        self.gvalues = gvalues

    def remesh(self, ppos, pscal):
        """
        \Lambda_1 2D tensorial remeshing kernel implementation.
        Remesh the given particle on the grid according to \Lambda_1 remeshing kernel.

        @param part Particle.Particle : particle to remesh
        """
        ii = xrange(len(ppos))
        # Calcul des l'indices sur la grille
        indGrid = [int((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i]) for i in ii]
        iXY = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        iXpY = [iXY[i] for i in ii]
        iXpY[0] = (iXpY[0] + 1) % self.grid.elementNumber[0]
        iXYp = [iXY[i] for i in ii]
        iXYp[1] = (iXYp[1] + 1) % self.grid.elementNumber[1]
        iXpYp = [iXY[i] for i in ii]
        iXpYp[0] = (iXpYp[0] + 1) % self.grid.elementNumber[0]
        iXpYp[1] = (iXpYp[1] + 1) % self.grid.elementNumber[1]
        # Calcul de la distance
        y = [(ppos[i] - self.grid[iXY][i]) / self.grid.elementSize[i] for i in ii]
        # Remaillage
        self.gvalues.values[tuple(iXY)] += (1. - y[0]) * (1. - y[1]) * pscal
        self.gvalues.values[tuple(iXpY)] += (y[0]) * (1. - y[1]) * pscal
        self.gvalues.values[tuple(iXYp)] += (1. - y[0]) * (y[1]) * pscal
        self.gvalues.values[tuple(iXpYp)] += (y[0]) * (y[1]) * pscal

    def __str__(self):
        """ToString method"""
        return "Lambda2 kernel"


class Lambda23D(RemeshingMethod):
    """
    Lambda2 remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        raise Exception("Obsolete methods : Must use 1D version with splitting.")
        self.grid = grid
        self.gvalues = gvalues

    def remesh(self, ppos, pscal):
        """
        \Lambda_1 3D tensorial remeshing kernel implementation.
        Remesh the given particle on the grid according to \Lambda_1 remeshing kernel.

        @param part Particle.Particle : particle to remesh
        """
        ii = xrange(len(ppos))
        # Calcul des l'indices sur la grille
        indGrid = [int((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i]) for i in ii]
        iXYZ = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        iXpYZ = [iXYZ[i] for i in ii]
        iXpYZ[0] = (iXpYZ[0] + 1) % self.grid.elementNumber[0]
        iXYpZ = [iXYZ[i] for i in ii]
        iXYpZ[1] = (iXYpZ[1] + 1) % self.grid.elementNumber[1]
        iXYZp = [iXYZ[i] for i in ii]
        iXYZp[2] = (iXYZp[2] + 1) % self.grid.elementNumber[2]
        iXpYpZ = [iXYZ[i] for i in ii]
        iXpYpZ[0] = (iXpYpZ[0] + 1) % self.grid.elementNumber[0]
        iXpYpZ[1] = (iXpYpZ[1] + 1) % self.grid.elementNumber[1]
        iXpYZp = [iXYZ[i] for i in ii]
        iXpYZp[0] = (iXpYZp[0] + 1) % self.grid.elementNumber[0]
        iXpYZp[2] = (iXpYZp[2] + 1) % self.grid.elementNumber[2]
        iXYpZp = [iXYZ[i] for i in ii]
        iXYpZp[1] = (iXYpZp[1] + 1) % self.grid.elementNumber[1]
        iXYpZp[2] = (iXYpZp[2] + 1) % self.grid.elementNumber[2]
        iXpYpZp = [iXYZ[i] for i in ii]
        iXpYpZp[0] = (iXpYpZp[0] + 1) % self.grid.elementNumber[0]
        iXpYpZp[1] = (iXpYpZp[1] + 1) % self.grid.elementNumber[1]
        iXpYpZp[2] = (iXpYpZp[2] + 1) % self.grid.elementNumber[2]
        # Calcul des distances
        y = [(ppos[i] - self.grid[iXYZ][i]) / self.grid.elementSize[i] for i in ii]
        # Remaillage
        self.gvalues.values[tuple(iXYZ)] += (1. - y[0]) * (1. - y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXpYZ)] += (y[0]) * (1. - y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXYpZ)] += (1. - y[0]) * (y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXYZp)] += (1. - y[0]) * (1. - y[1]) * (y[2]) * pscal
        self.gvalues.values[tuple(iXpYpZ)] += (y[0]) * (y[1]) * (1. - y[2]) * pscal
        self.gvalues.values[tuple(iXpYZp)] += (y[0]) * (1. - y[1]) * (y[2]) * pscal
        self.gvalues.values[tuple(iXYpZp)] += (1. - y[0]) * (y[1]) * (y[2]) * pscal
        self.gvalues.values[tuple(iXpYpZp)] += (y[0]) * (y[1]) * (y[2]) * pscal

    def __str__(self):
        """ToString method"""
        return "Lambda2 kernel"
