# -*- coding: utf-8 -*-
"""
@package Utils
RemeshingMethod interface.
"""
from RemeshingMethod import RemeshingMethod
import numpy as np
import math

providedClass = "M4Prime"


class M4Prime(RemeshingMethod):
    """
    M4Prime remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        RemeshingMethod.__init__(self, grid, gvalues)
        # Epsilon value to avoid rounding errors.
        self.epsilon = self.grid.elementSize[0] * 0.000000001
        self.dim_range = xrange(self.grid.dimension)

    def remesh(self, ppos, pscal, dir):
        """
        Remeshing method for non-Tagged Left particles.

        @param ppos : particle position.
        @param pscal : particle scalar.
        @param dir : direction to remesh.
        """
        #print "Remaillage no Tag Left"
        # ii = xrange(len(ppos))
        # d = splittingDirection
        # # Calcul des indices sur la grille
        # indGrid = [int(math.floor((ppos[i] - self.grid.min[i]) / self.grid.elementSize[i] + self.epsilon)) for i in ii]
        # i0 = [indGrid[i] % self.grid.elementNumber[i] for i in ii]
        # i0p = [(i0[i] + 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        # i0p2 = [(i0[i] + 2) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        # i0m = [(i0[i] - 1) % self.grid.elementNumber[i] if i == d else i0[i] for i in ii]
        # # Calcul de la distance
        # x0 = (ppos[d] - indGrid[d] * self.grid.elementSize[d] - self.grid.min[d]) / self.grid.elementSize[d]
        # # Calcul des poids
        # am = -0.5 * x0 * (x0 - 1.) * (x0 - 1.)
        # a0 = 0.5 * (x0 - 1.) * (3. * x0 * x0 - 2 * x0 - 2.)
        # ap = 0.5 * x0 * (1. + 4. * x0 - 3. * x0 * x0)
        # ap2 = 0.5 * x0 * x0 * (x0 - 1.)
        # # Remaillage
        # self.gvalues.values[tuple(i0m)] += pscal * am
        # self.gvalues.values[tuple(i0)] += pscal * a0
        # self.gvalues.values[tuple(i0p)] += pscal * ap
        # self.gvalues.values[tuple(i0p2)] += pscal * ap2
        ## ------------------- NumPy Optimisation
        ind = ((ppos - self.grid.min) / self.grid.elementSize + self.epsilon).astype(np.int)
        i0 = ind % self.grid.elementNumber
        ip = np.copy(i0)
        ip[..., dir] = (i0[..., dir] + 1) % self.grid.elementNumber[dir]
        ip2 = np.copy(i0)
        ip2[..., dir] = (i0[..., dir] + 2) % self.grid.elementNumber[dir]
        im = np.copy(i0)
        im[..., dir] = (i0[..., dir] - 1) % self.grid.elementNumber[dir]
        i0 = tuple([i0[..., i] for i in self.dim_range])
        ip = tuple([ip[..., i] for i in self.dim_range])
        ip2 = tuple([ip2[..., i] for i in self.dim_range])
        im = tuple([im[..., i] for i in self.dim_range])
        x0 = (ppos[..., dir] - ind[..., dir] * self.grid.elementSize[dir] - self.grid.min[dir]) / self.grid.elementSize[dir]
        am = -0.5 * x0 * (x0 - 1.) * (x0 - 1.)
        a0 = 0.5 * (x0 - 1.) * (3. * x0 * x0 - 2 * x0 - 2.)
        ap = 0.5 * x0 * (1. + 4. * x0 - 3. * x0 * x0)
        ap2 = 0.5 * x0 * x0 * (x0 - 1.)
        self.gvalues.values[i0] += pscal * a0
        self.gvalues.values[im] += pscal * am
        self.gvalues.values[ip] += pscal * ap
        self.gvalues.values[ip2] += pscal * ap2
        ## -------------------

    def __str__(self):
        """ToString method"""
        return "M4Prime kernel"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : " + providedClass
    print eval(providedClass).__doc__
