# -*- coding: utf-8 -*-
"""
@package Utils
RemeshingMethod interface.
"""
from ..Param import *
from RemeshingMethod import RemeshingMethod


class M6Prime(RemeshingMethod):
    """
    M6Prime remeshing method implementation.
    """
    def __init__(self, grid, gvalues):
        """
        Constructor.
        """
        RemeshingMethod.__init__(self, grid, gvalues)
        self.dim_range = xrange(self.grid.dimension)
        self.nb_flop = 106

    def remesh(self, ppos, pscal, dir):
        """
        Remeshing method for non-Tagged Left particles.

        @param ppos : particle position.
        @param pscal : particle scalar.
        @param dir : direction to remesh.
        """
        ind = np.empty(self.grid.shape, dtype=dtype_integer)
        a0 = ((ppos - self.grid.min) / self.grid.elementSize)
        ind[...] = (np.round(np.abs(a0))) * np.sign(a0)
        ind[..., dir] = np.floor(a0[..., dir])
        x0 = (ppos[..., dir] - ind[..., dir] * self.grid.elementSize[dir] - self.grid.min[dir]) / self.grid.elementSize[dir]
        am2 = x0 * (x0 * (x0 * (x0 * (-5. * x0 + 13.) - 9.) - 1.) + 2.) / 24.
        am = x0 * (x0 * (x0 * (x0 * (25. * x0 - 64.) + 39.) + 16.) - 16.) / 24.
        a0 = x0 * x0 * (x0 * (x0 * (-50. * x0 + 126.) - 70.) - 30.) / 24. + 1.
        ap = x0 * (x0 * (x0 * (x0 * (50. * x0 - 124.) + 66.) + 16.) + 16.) / 24.
        ap2 = x0 * (x0 * (x0 * (x0 * (-25. * x0 + 61.) - 33.) - 1.) - 2.) / 24.
        ap3 = x0 * x0 * x0 * (x0 * (5. * x0 - 12.) + 7.) / 24.

        ind[..., dir] = (ind[..., dir] - 2 + self.grid.elementNumber[dir]) % self.grid.elementNumber[dir]

        for i_index, ps, w in zip(ind.reshape((-1,self.grid.dimension)), pscal.reshape((-1)), am2.reshape((-1))):
            self.gvalues.values[tuple(i_index)] += ps * w

        ind[..., dir] = (ind[..., dir] + 1) % self.grid.elementNumber[dir]
        for i_index, ps, w in zip(ind.reshape((-1,self.grid.dimension)), pscal.reshape((-1)), am.reshape((-1))):
            self.gvalues.values[tuple(i_index)] += ps * w

        ind[..., dir] = (ind[..., dir] + 1) % self.grid.elementNumber[dir]
        for i_index, ps, w in zip(ind.reshape((-1,self.grid.dimension)), pscal.reshape((-1)), a0.reshape((-1))):
            self.gvalues.values[tuple(i_index)] += ps * w

        ind[..., dir] = (ind[..., dir] + 1) % self.grid.elementNumber[dir]
        for i_index, ps, w in zip(ind.reshape((-1,self.grid.dimension)), pscal.reshape((-1)), ap.reshape((-1))):
            self.gvalues.values[tuple(i_index)] += ps * w

        ind[..., dir] = (ind[..., dir] + 1) % self.grid.elementNumber[dir]
        for i_index, ps, w in zip(ind.reshape((-1,self.grid.dimension)), pscal.reshape((-1)), ap2.reshape((-1))):
            self.gvalues.values[tuple(i_index)] += ps * w

        ind[..., dir] = (ind[..., dir] + 1) % self.grid.elementNumber[dir]
        for i_index, ps, w in zip(ind.reshape((-1,self.grid.dimension)), pscal.reshape((-1)), ap3.reshape((-1))):
            self.gvalues.values[tuple(i_index)] += ps * w

    def __str__(self):
        """ToString method"""
        return "M6Prime kernel"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : M6Prime"
    print M6Prime.__doc__
