# -*- coding: utf-8 -*-
"""
@package Utils
RK2 method interface.
"""
from .integrator import ODESolver
from ...operator.differentialOperator_d import DifferentialOperator_d
from ...operator.differentialOperator import DifferentialOperator
from parmepy.constants import *
import copy
import sys
import numpy as np


class RK2Stretch(ODESolver):
    """
    ODESolver implementation for solving an equation system with RK2 method.
    y'(t) = f(t,y)

    y(t_n+1)= y(t_n) + dt*y'[y(t_n)+dt/2*y'(y(t_n))].

    """
    def __init__(self, f=None, dim=3):
        """
        Constructor.

        @param f function f(t,y) : Right hand side of the equation to solve.
        @param dim : dimensions
        @param conditions : function to apply boundary conditions.
        """
        ODESolver.__init__(self, f)


    def integrate(self, f, fd ,field1 , field2, res_vorticity, t, dt, dimension):
#    def integrate(self, f,field1 , field2, res_vorticity, t, dt, dimension):
        """
        Integration step for RK4 Method
         yp= y + dt*y'[y+dt/2*y'(y)].

        @param y : position at time t.
        @param f : operator to calcul y'(y) value at time t.
        @param fd : result position at time t + dt
        @param field1 : vorticity field discretized
        @param field2 : velocity field discretized
        @param t : current time.
        @param dt : time step.
        """
        maxgersh = np.zeros(1, dtype=PARMES_REAL, order=ORDER)
        resultTmp = np.asarray([np.zeros((field2.topology.resolution), dtype=PARMES_REAL, order=ORDER) for d in xrange(field2.topology.dim)])
        K1=np.asarray(fd)
        K1[:,:,:,:]= K1[:,:,:,:] *0.5*dt + field1[:,:,:] 
        if False in [(field1[i][...].shape == field2[i][...].shape) for i in xrange(dimension)] :
            print "Error, not the same mesh on field1 and  on field2"
        f.discreteOperator = DifferentialOperator_d(field1=K1, field2=field2, result=resultTmp, choice='divConservation', maxgersh=maxgersh)
        f.discreteOperator.apply()
        res_vorticity[:,:,:] = field1[:,:,:] + resultTmp[:,:,:,:] * dt
        return res_vorticity



    def __str__(self):
        """ToString method"""
        return "RK2 Method"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : RK2"
    print RK2.__doc__
