# -*- coding: utf-8 -*-
"""
@package Utils
RK4 method interface.
"""
from .integrator import ODESolver
from ...operator.differentialOperator_d import DifferentialOperator_d
from ...operator.differentialOperator import DifferentialOperator
from parmepy.constants import *
import numpy as np
import copy

class RK4Stretch(ODESolver):
    """
    ODESolver implementation for solving an equation system with RK4 method.
    y'(t) = f(t,y)

    y(t_n+1)= y(t_n) + dt*y'[y(t_n)+dt/2*y'(y(t_n))].

    """
    def __init__(self, f=None, dim=3):
        """
        Constructor.

        @param f function f(t,y) : Right hand side of the equation to solve.
        @param dim : dimensions
        @param conditions : function to apply boundary conditions.
        """
        ODESolver.__init__(self, f)


    def integrate(self, f, fd ,field1 , field2, res_vorticity, t, dt, dimension):
        """
        Integration step for RK4 Method
         yp= y + dt*y'[y+dt/2*y'(y)].

        @param y : position at time t.
        @param f : operator to calcul y'(y) value at time t.
        @param fd : result position at time t + dt
        @param field1 : vorticity field discretized
        @param field2 : velocity field discretized
        @param t : current time.
        @param dt : time step.
        """
        print "ehoh"
        maxgersh = np.zeros(1, dtype=PARMES_REAL, order=ORDER)
        resultTmp = np.asarray([np.zeros((field2.topology.resolution), dtype=PARMES_REAL, order=ORDER) for d in xrange(field2.topology.dim)])
        K1=np.asarray(fd)
        K2=np.asarray([np.zeros((field2.topology.resolution), dtype=PARMES_REAL, order=ORDER) for d in xrange(field2.topology.dim)])
        K3=np.asarray([np.zeros((field2.topology.resolution), dtype=PARMES_REAL, order=ORDER) for d in xrange(field2.topology.dim)])
        K4=np.asarray([np.zeros((field2.topology.resolution), dtype=PARMES_REAL, order=ORDER) for d in xrange(field2.topology.dim)])
        K1[:,:,:,:] =K1[:,:,:,:]* dt
        resultTmp[:,:,:,:] = K1[:,:,:,:] * 0.5 + field1[:,:,:]
        f.discreteOperator = DifferentialOperator_d(resultTmp , field2, K2, choice='divConservation',maxgersh=maxgersh)
        f.discreteOperator.apply()
        K2[:,:,:,:] = K2[:,:,:,:] * dt
        resultTmp[:,:,:,:] = K2[:,:,:,:] * 0.5 + field1[:,:,:]
        f.discreteOperator = DifferentialOperator_d(resultTmp, field2, K3, choice='divConservation',maxgersh=maxgersh)
        f.discreteOperator.apply()
        K3[:,:,:,:] = K3[:,:,:,:] * dt
        resultTmp[:,:,:,:] = K3[:,:,:,:] + field1[:,:,:]
        f.discreteOperator = DifferentialOperator_d(resultTmp, field2, K4, choice='divConservation',maxgersh=maxgersh)
        f.discreteOperator.apply()
        K4[:,:,:,:] = K4[:,:,:,:] * dt
        res_vorticity[:,:,:] = field1[:,:,:] + 1./6. * (K1[:,:,:,:] + 2.0 * K2[:,:,:,:] + 2.0  * K3[:,:,:,:] + K4[:,:,:,:])
        return res_vorticity


    def __str__(self):
        """ToString method"""
        return "RK4 Method"


if __name__ == "__main__":
    print __doc__
    print "- Provided class : RK4"
    print RK4.__doc__
