"""
@file splitting.py

Splitting operator representation
"""
from parmepy.numerics.method import NumMethod


class Splitting(NumMethod):
    """
    Splitting operator representation.

    Operator of operators applying in a Strang splitting.

    Implements a 2nd order splitting in 3D:
    @li X-dir, half time step
    @li Y-dir, half time step
    @li Z-dir, full time step
    @li Y-dir, half time step
    @li X-dir, half time step
    \n
    Implements a 2nd order splitting in 2D:
    @li X-dir, half time step
    @li Y-dir, full time step
    @li X-dir, half time step
    """

    def __init__(self, function, dim, config='o2_FullHalf'):
        """
        Create a Splitting operator on a given list of operators
        and a dimension.

        @param function : function to split.
        @param dim : problem dimension.
        @param config : Splitting configuration :
          - o2_FullHalf : All steps are performed with time step equals to dt/2
          - o2 : middle step is performed with time step equals to dt
        """
        raise ValueError("Splitting became obsolete")
        NumMethod.__init__(self, name="splitting")
        ## Operators to split
        self.function = function
        ## Splitting at 2nd order.
        self.splitting = []
        if config == 'o2_FullHalf':
            ## Half timestep in all directions
            [self.splitting.append((i, 0.5)) for i in xrange(dim)]
            [self.splitting.append((dim - 1 - i, 0.5)) for i in xrange(dim)]
        elif config == 'o2':
            ## Half timestep in all directions but last
            [self.splitting.append((i, 0.5)) for i in xrange(dim - 1)]
            self.splitting.append((dim - 1, 1.))
            [self.splitting.append((dim - 2 - i, 0.5))
             for i in xrange(dim - 1)]
        elif config == 'o1':
            [self.splitting.append((i, 1.)) for i in xrange(dim)]
        elif config == 'x_only':
            self.splitting.append((0, 1.))
        elif config == 'y_only':
            self.splitting.append((1, 1.))
        elif config == 'z_only':
            self.splitting.append((2, 1.))
        else:
            ## Full timestep in all directions
            [self.splitting.append((i, 1.)) for i in xrange(dim)]

    def __call__(self, t, dt):
        """
        Apply Remeshing operator.

        @param t : current time.
        @param dt : time step.
        """
        for split_id, split in enumerate(self.splitting):
            print "direction : " + str(split[0]) +\
                " dt*" + str(split[1]) + " id:" + str(split_id)
            self.function(t, dt * split[1], split[0], split_id)

    def __str__(self):
        s = "Splitting (DiscreteOperator). Splitting steps : \n"
        for split in self.splitting:
            s += "direction : " + str(split[0]) + " dt=" + str(split[1]) + "\n"
        s += "Concerned operators : \n"
        for op in self.operators:
            s += str(op)
        return s

if __name__ == "__main__":
    print __doc__
    print "- Provided class : Splitting"
    print Splitting.__doc__
