# -*- coding: utf-8 -*-
import time
import parmepy as pp
import numpy as np
from parmepy.tools.cpu_data_transfer import Synchronize
from parmepy.tools.printer import Printer
from math import *
import unittest



class test_Cpu_data_transfert(unittest.TestCase):
    """
    DiscreteVariable test class
    """

    def vitesse(self,x, y, z):
        vx = x
        vy = y
        vz = z
        return vx, vy, vz


    def analyticalDivProduct(self, x, y, z):
        sx = np.cos(x)
        sy = np.cos(y)
        sz = np.cos(z)
        return sx, sy, sz

    def setUp(self):
        nb = 32.
        # Parameters
        self.finalTime = 0.09
        self.t = 0.
        self.e = 0.002  # Accepted error between result and analytical result
        self.dim = 3
        self.boxLength = [1., 1., 1.]
        self.boxMin = [ 0., 0., 0.]
        self.nbPts = [nb, nb, nb]
        self.timeStep = 0.02
        self.comm = main_comm
        self.outputFilePrefix0 = './parmepy/test/test_tools/INIT_'
        self.outputFilePrefix = './parmepy/test/test_tools/Trans_'
        ## Domain
        self.box = pp.Box(dimension=self.dim,
                       length=self.boxLength,
                       origin=self.boxMin)

    def testOperatorSynchronize(self):
        t0 = time.time()

        ## Fields
        velo = pp.AnalyticalField(domain=self.box, formula=self.vitesse, name='Velocity', vector=True)

        ## Solver creation (discretisation of objects is done in solver initialisation)
        topo3D = pp.CartesianTopology(domain=self.box, resolution=self.nbPts, dim=self.dim, periods=[True,True,True] ,ghosts=[2.,2.,2.])


#        self.anal=np.asarray(np.vectorize(self.analyticalDivProduct)(self.topo3D.mesh.coords[0], \
#                                                     self.topo3D.mesh.coords[1], \
#                                                     self.topo3D.mesh.coords[2]))

        OpSynchronize = Synchronize(topo3D)
        ## Discretization
        velo.discretize(topo3D)
        velo.initialize()
        t1 = time.time()
#        print 'rank', topo3D.rank, 'tab', topo3D.tabSort
        io = Printer(fields=[velo], frequency=1, outputPrefix=self.outputFilePrefix0+str(topo3D.rank))
#        io.step()
        OpSynchronize.apply(velo.discreteField[0])
        io = Printer(fields=[velo], frequency=1, outputPrefix=self.outputFilePrefix+str(topo3D.rank))
        io.step()

        tf = time.time()
#        print 'velo :', velo.discreteField[0].data[0][:,0,0],velo.discreteField[0].data[0][:,1,1]

        print "\n"
        print "Total time : ", tf - t0, "sec (CPU)"
        print "Init time : ", t1 - t0, "sec (CPU)"
        print "Solving time : ", tf - t1, "sec (CPU)"


    def runTest(self):
        self.testOperatorSynchronize()

def suite():
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(test_Cpu_data_transfert))
    return suite

if __name__ == "__main__":
    unittest.TextTestRunner(verbosity=2).run(suite())
