/** \file ppm_wrapper.hpp Interfaces to ppm (fortran) routines
 */
#ifndef PPMWRAPPER_HPP
#define PPMWRAPPER_HPP 
#include"FCMangle.h"
#include<mpi.h>
#include <cstring>
#include "Mesh.hpp"

/** Namespace for ppm functions and subroutines */

Mesh* p_obj = 0;
double * globalPtr = 0;

namespace PPM 
{
  
  

  
  extern "C" {
    // Init and close ppm
    void PPM_MODULE(ppm_module_init,ppm_init, PPM_MODULE_INIT,PPM_INIT)(int*, int*,int*,int*,int*,int*,int*,int*,int*);
    void PPM_MODULE(ppm_module_finalize, ppm_finalize,PPM_MODULE_FINALIZE,PPM_FINALIZE)(int&);
    // Display functions
    void PPM_MODULE(charfunctions, start, CHARFUNCTIONS, START)(double*,int*,char*,int );
    void PPM_MODULE(charfunctions, stop, CHARFUNCTIONS, stop)(double*,int*,char*,int );

    void PPM_MODULE(testppm,mult,TESTPPM,MULT)(double*,double*,int*);
    // Topologies
    //    void PPM_MODULE(ppm_module_mktopo, ppm_mktopo, PPM_MODULE_MKTOPO,PPM_MKTOPO)(int*, double*, int*,
    
    void PPM_MODULE(modtest, cas1, MODTEST,CAS1)(double*);
    void PPM_MODULE(modtest, cas2, MODTEST,CAS2)(double*);
    void PPM_MODULE(modtest, cas3, MODTEST,CAS3)(double*);
    void PPM_MODULE(modtest, cas4, MODTEST,CAS4)(double*);
    void PPM_MODULE(modtest, cas5, MODTEST,CAS5)(double*);
    void PPM_MODULE(modtest, cas6, MODTEST,CAS6)(double**);
    void PPM_MODULE(modtest, testglob, MODTEST,TESTGLOB)();
    void PPM_MODULE(modtest, application, MODTEST,APPLICATION)(); 
    void PPM_MODULE(modtest, application2, MODTEST,APPLICATION2)(); 
    void PPM_MODULE(modtest, application3, MODTEST,APPLICATION3)(); 
    void PPM_MODULE(modtest, bridge, MODTEST,BRIDGE)(int*,int*,int*,double*);
    void PPM_MODULE(modtest, bridge2, MODTEST,BRIDGE2)(int*,double*);
    
    //    int PPM_MODULE(ppm_module_data,ppm_debug, PPM_MODULE_DATA,PPM_DEBUG);
    

   
    
  }

  //#define DEBUG PPM_MODULE_(ppm_module_data,ppm_debug, PPM_MODULE_DATA,PPM_DEBUG)

  /** A static class to call wrapped ppm functions */
  class wrapper 
  {
    
    static int _info; 

  public : 
    
    /** PPM initialization
	\param problem dimension
	\param precision for real numbers
	\param exp tolerance
	\param MPI comm 
	\param debug mode value
	\param error status
	\param log unit value for io
	\param stderr unit value
	\param stdout unit value
    */
    static void init(int ndim, int MK,int tolexp, MPI::Intracomm& comm, int debug, int* info, int ppm_log_unit, int err, int out)
    {
      MPI_Fint Fcomm = MPI_Comm_c2f(comm);
      PPM_MODULE(ppm_module_init,ppm_init, PPM_MODULE_INIT,PPM_INIT)(&ndim,&MK,&tolexp,&Fcomm,&debug,info,&ppm_log_unit,&err,&out);
    }

    /** Terminates ppm library 
	\param[in,out] error status
    */ 
    static void finalize(int& info)
    {
      PPM_MODULE(ppm_module_finalize, ppm_finalize,PPM_MODULE_FINALIZE,PPM_FINALIZE)(info);
    }
    
    /** Wrapper to ppm substart function
	@param[in]  caller name
	@param[in,out] cpu time when this function is called
	@param[in] error status
    */
    static void substart(std::string& msg, double* t0, int* info)
    {
      size_t size = msg.size()+1;
      char * msgF = new char[size];
      strncpy(msgF, msg.c_str(),size);
      PPM_MODULE(charfunctions, start, CHARFUNCTIONS, START)(t0,info,msgF,strlen(msgF));
    }
    
    /** Wrapper to ppm substopt function
	@param[in]  caller name
	@param[in] cpu time when substart for this function has been called
	@param[in] error status
    */
    static void substop(std::string& msg, double* t0, int* info)
    {
      size_t size = msg.size()+1;
      char * msgF = new char[size];
      strncpy(msgF, msg.c_str(),size);      
      PPM_MODULE(charfunctions, stop, CHARFUNCTIONS, STOP)(t0,info,msgF,strlen(msgF));
    }
    
  };

}
  extern "C" {
    void cinitmesh_(int *nx, int *ny, int *nz, double *geom) {
      p_obj = new Mesh(nx,ny,nz,geom);
    }
    
    
    void cdeletemesh_() { delete p_obj; }
    
    void cgetgrid_() {
      std::cout << "cgetgrid ..." << std::endl;
    double *p_mem = p_obj->getGrid();
    int nx = p_obj->getSizeX();
    int ny = p_obj->getSizeY();
    int nz = p_obj->getSizeZ();
    std::cout << p_mem[1] << std::endl;
    PPM::PPM_MODULE(modtest, bridge, MODTEST,BRIDGE)(&nx, &ny, &nz, p_mem); 
    p_mem = 0;
    std::cout << "Fin cgetgrid ..." << std::endl;
    }

    void cinit_(double* toto)
    {
      globalPtr = toto;
    }
    
    void cgetptr_() {
      std::cout << "cgetPtr ..." << std::endl;
      double *p_mem = globalPtr;
      int nx = 2;
      PPM::PPM_MODULE(modtest, bridge2, MODTEST,BRIDGE2)(&nx, p_mem); 
      p_mem = 0;
      std::cout << "Fin cgetptr ..." << std::endl;
    }
  }

#endif
