subroutine main_init

  use parallel
  use parser
  use string
  implicit none

  character(len=str_long) :: sim_name
  integer :: itype, ipost, iscal

  call parser_init
  call parser_parsefile('input')

  call mpi_initialize
  call random_init
  call transform_init
  call data_init
  
  call parser_read('Simulation name', sim_name)
  call parser_read('Simulation type', itype)
  call parser_read('Re-init scalar', iscal)

  
  if (itype.eq.0) then !!!! Initialize and compute
     if (trim(sim_name).eq.'Taylor Green') then
        call tg_init
     elseif(trim(sim_name).eq.'Isotropic turbulence') then
        call hit_init
     else
        print *,' Unknown Simulation Name'
        stop
     endif
  elseif (itype.eq.1) then
     call data_read
     if( iscal.eq.1) then
       call hit_init
     endif
  else
     print *,' Unknown simulation type'
     stop
  endif
  
  call solver_init

  call x_advec_init

!  call parser_read('Postprocessing', ipost)
!  if (ipost.eq.1) print*,'not yet implemented'

end subroutine main_init


program main
 
  use parallel 
  use parser
  use string
  implicit none
  integer :: i,n_iter
  character(len=str_long) :: sim_name
  integer :: ifreq, ipost,itest
  
  call main_init

  call parser_read('Postprocessing',ipost)

  call parser_read('Simulation iterations',n_iter)
  call parser_read('Simulation name',sim_name)
  call parser_read('Write frequency',ifreq,n_iter)
  if (sim_name.eq.'Isotropic turbulence'.or.sim_name.eq.'Plane jet') call dns_stats



  if (ipost.eq.1) then
    call postprocess
  else if(ipost.eq.2) then
    call data_filter_sc
    call data_write
    stop
  else if(ipost.eq.5) then
    call postprocess5
    stop
  else if(ipost.eq.6) then
    call postprocess6
    stop
  else if(ipost.eq.7) then
    call postprocess7
    stop
  else
    call data_write
    do i = 1,n_iter
       call solver_step
       if (trim(sim_name).eq.'Taylor Green') then
          call tg_stats
       elseif (trim(sim_name).eq.'Isotropic turbulence'.or.sim_name.eq.'Plane jet') then
          call dns_stats
       endif
       if (mod(i,ifreq).eq.0) call data_write
    enddo
!    call postprocess
  end if

end program main
