subroutine hit_init

  use parallel
  use random
  use transform
  use data
  implicit none
!  include 'fftw3.f'
  
  ! Spectrum computation
  real(WP) :: psr,ps1,ps2, xx, yy, zz
  real(WP) :: ke,kd,ks,dk,ksk0ratio,kc,kcksratio,kk,kx,ky,kz,kk2,kcut
  real(WP) :: alpha,spec_amp,eps,amp_disc,f_phi,sch
  integer  :: kl,jl,il,impose,itype
  real(WP) :: e_total,energy_spec,diss_total,ndense
  real(WP), dimension(:,:), pointer :: spect,sg
  real(WP), dimension(:), pointer :: s1,tabs2
  complex(WP), dimension(:,:,:), pointer :: ak,bk
  real(WP) :: scmean,scmean_global
  ! Other
  integer :: i,j,k,ik,iunit,dim
  real(WP), dimension(:), pointer :: nnodes
  complex(WP) :: ii=(0.0_WP,1.0_WP)
  real(WP) :: rand,pi,spec,kfix

  real(WP) :: keta, Rlambda, L11, tke,L_large, dissipation,eta
  real(WP) :: e_spec,d_spec, e_local, d_local, spec_factor
  ! Fourier coefficients
  real(WP) :: umax,vmax,wmax,cos,sin
  complex(WP), dimension(:,:,:), pointer :: Uk,Vk,Wk,scalar
  complex(WP), dimension(:,:,:), pointer :: Cbuf
  real(WP), dimension(:,:,:), pointer :: Rbuf

  logical :: flg_inplace

  integer :: ierr,gg(3),iforce,iscal
  integer,dimension(:,:,:),pointer :: node_index 
  integer(KIND=8) :: plan_c2r

!!$  real(WP), dimension(:,:,:), pointer :: A
!!$  complex(WP), dimension(:,:,:), pointer :: B

  call parser_read('Kmax eta', keta)
  call parser_read('R Lambda', Rlambda)
  call parser_read('Forcing', iforce)

  if (trim(spectrum).eq.'VKP') then
     call parser_read('Dissipative scale',ld)
     call parser_read('Dissipation',epsilon)
  end if

  ! Create pi
  pi = acos(-1.0_WP)
  
!!$  ! =================
!!$  ! Velocity Spectrum
!!$  
!!$  ! Spectrum computation
!!$  ke = 2.0_WP*pi/le
!!$  if (trim(spectrum).eq.'VKP') kd = 2.0_WP*pi/ld
!!$  dk = 2.0_WP*pi/L
!!$  kc = real(nx/2,WP)*dk
!!$  eps=ke/1000000.0_WP
!!$  if (trim(spectrum).eq.'PP') then
!!$     spec_amp = 16.0_WP*sqrt(2.0_WP/pi)*Ut**2/ke
!!$  else if (trim(spectrum).eq.'VKP') then
!!$     alpha=1.5_WP
!!$     spec_amp = 1.5_WP*Ut**5/epsilon
!!$  end if
!!$  amp_disc = sqrt(dk)**3


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! Modified Input for the Spectral Code
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  
!!!! Need to input Viscosity, kmax X eta, R_Lambda 

!!!!! Assumes high-reynolds limit to determine the largest eddy
!!!!! Equations for Pope's book, chapter 6

 

  eta = (2.0_WP*keta)/real(nx,WP)
  L_large = eta*(3.0_WP/20.0_WP*Rlambda*Rlambda)**(3.0/4.0)

  L11 = L_large*0.43
  
  ke = 1.3_WP/L11
  
  if (ke.lt.1) then
     print *,' Peak wavenumber not in the box, reduce R Lambda'
     call parser_read('Re-init scalar', iscal)
     if(iscal.eq.0) then
       stop
     end if
  endif


  dk = 2.0_WP*pi/L
  kc = pi*real(nx,WP)/L
  eps = kc/10000000_WP


  e_spec = 0.0_WP
  d_spec = 0.0_WP

  ndense = 0.0_WP

  do k = fnzs,fnze
     do j = fnys,fnye
        do i = fnxs,fnxe
           kx=real(i-1,WP)*dk
           ky=real(j-1,WP)*dk
           if (j.gt.nk) ky=-real(nx+1-j,WP)*dk
           kz=real(k-1,WP)*dk
           if (k.gt.nk) kz=-real(nx+1-k,WP)*dk
           kk=sqrt(kx**2+ky**2+kz**2)
           if ((kk.gt.eps).and.(kk.le.kc)) then
              ndense = ndense + 1.0_WP
              energy_spec = (kk/ke)**4.0_WP*exp(-2.0_WP*(kk/ke)**2.0_WP)/(4.0_WP*pi*kk**2.0_WP)
              e_spec = e_spec + dk*energy_spec
              d_spec = d_spec + dk*kk**2.0_WP*energy_spec
           endif
        enddo
     enddo
  enddo

  e_local = e_spec
  d_local = d_spec
  call MPI_ALLREDUCE(e_local,e_spec,1,MPI_REAL_WP,MPI_SUM,&
       &             MPI_COMM_WORLD,ierr)
  call MPI_ALLREDUCE(d_local,d_spec,1,MPI_REAL_WP,MPI_SUM,&
       &             MPI_COMM_WORLD,ierr)


  spec_amp = 3.0_WP/10.0_WP*Rlambda**2.0_WP*mu**2.0_WP*d_spec/e_spec**2.0_WP

!mu = sqrt(e_spec**2.0_WP/d_spec*(10.0_WP/(3.0_WP*Rlambda**2.0_WP))) 
  tke = spec_amp*e_spec

  Ut = sqrt(tke/1.5_WP)   !Rlambda*mu/(sqrt(10.0_WP)*eta**(2.0_WP/3.0_WP)*L_large**(1.0_WP/3.0_WP))
!  tke = 1.5_WP*Ut*Ut
  dissipation = 2.0_WP*mu*spec_amp*d_spec
  amp_disc = sqrt(dk)**3.0_WP


  if (rank.eq.0) then
     print *, '------------------------------------------------------------------'
     print *, '------------------------------------------------------------------'
     print *,' L11 ', L11
     print *,' L_large ', L_large
     print *,' kc ', kc
     print *,' ke ', ke
     print *,' eta ', eta
     print *,' Ut  ', Ut
     print *,' kinetic energy ', tke
     print *,' dissipation ', dissipation
     print *,' spec_amp ', spec_amp
     print *,' amp_disc ', amp_disc
     print *,' viscosity', mu
     print *,' e_spec ', e_spec
     print *,' d_spec ', d_spec
     print *,' Number of modes', ndense
     print *,' R lambda calculated', sqrt(20.0_WP/3.0_WP*tke**2.0/(dissipation*mu))
     print *, '------------------------------------------------------------------'
     print *, '------------------------------------------------------------------'
  endif

  

  ! Output spectrum for comparison
  allocate(spect(2,nx+1))
  e_total=0.0_WP
  diss_total = 0.0_WP
  spect = 0.0_WP


  ! Compute spectrum

  allocate(ak(fnxs:fnxe,fnys:fnye,fnzs:fnze),bk(fnxs:fnxe,fnys:fnye,fnzs:fnze))
  !allocate(nnodes(nx+1))
  !nnodes = 0.0_WP
  ndense = 0.0_WP
  do k=fnzs,fnze
     do j=fnys,fnye
        do i=fnxs,fnxe
           ! Random numbers
           call random_number(rand)
           psr= 2.0_WP*pi*rand                !2.0_WP*pi*(rand-0.5_WP)
           call random_number(rand)
           ps1=2.0_WP*pi*rand                 !*(rand-0.5_WP)
           call random_number(rand)
           ps2=2.0_WP*pi*rand                 !(rand-0.5_WP)
           ! Wavenumbers
           kx=real(i-1,WP)*dk
           ky=real(j-1,WP)*dk
           if (j.gt.nk) ky=-real(nx+1-j,WP)*dk
           kz=real(k-1,WP)*dk
           if (k.gt.nk) kz=-real(nx+1-k,WP)*dk
           kk=sqrt(kx**2.+ky**2.+kz**2.)
           ! Spectrums
           energy_spec=spec_amp*(kk/ke)**4.0_WP*exp(-2.0_WP*(kk/ke)**2.0_WP)
           ! Coeff
           ik = 1+idint(kk/dk + 0.5_WP)
           if ((kk.gt.eps).and.(kk.le.kc)) then
              ndense = ndense + 1.0_WP
              ak(i,j,k)= amp_disc*sqrt(energy_spec/(4.0_WP*pi*kk**2.0_WP))*exp(ii*ps1)*cos(psr)
              bk(i,j,k)= amp_disc*sqrt(energy_spec/(4.0_WP*pi*kk**2.0_WP))*exp(ii*ps2)*sin(psr)
              spect(2,ik) = spect(2,ik) + real(ak(i,j,k)*conjg(ak(i,j,k))) + real(bk(i,j,k)*conjg(bk(i,j,k)))
              e_total = e_total + dk*(real(ak(i,j,k)*conjg(ak(i,j,k))) + real(bk(i,j,k)*conjg(bk(i,j,k))))
              diss_total = diss_total + dk*2.0_WP*mu*kk**2.0_WP*(real(ak(i,j,k)*conjg(ak(i,j,k)))&
                   & + real(bk(i,j,k)*conjg(bk(i,j,k))))
           end if
        end do
     end do
  end do

  e_local = e_total
  d_local = diss_total
  allocate(tabs2(1:nx+1))
  tabs2 = 0.0_WP
  call MPI_ALLREDUCE(spect(2,:),tabs2,nx+1,MPI_REAL_WP,MPI_SUM,MPI_COMM_WORLD,ierr)
  call MPI_ALLREDUCE(e_local,e_total,1,MPI_REAL_WP,MPI_SUM,&
       &             MPI_COMM_WORLD,ierr)
  call MPI_ALLREDUCE(d_local,diss_total,1,MPI_REAL_WP,MPI_SUM,&
       &             MPI_COMM_WORLD,ierr)
  spect(2,:) = tabs2
  deallocate(tabs2)


!!$  if (rank.eq.0) then
!!$     print *,' Total energy ', e_total, tke
!!$     print *,' Total dissipation ', diss_total, dissipation
!!$     print *,' Number of modes ', ndense
!!$  endif

  iunit=iopen()
  open(iunit,file='spectrum.analytic',form='formatted')
  do i=1,nx+1
     spect(1,i) = real(i-1,WP)*dk
     if ((spect(1,i).ne.0.0_WP).and.(spect(2,i).ne.0.0_WP)) then
        write(11,*) spect(1,i),',',spect(2,i)  
     end if
  end do
  close(iclose(iunit))


  
  ! Compute 3D velocity field
  allocate(Uk(fns1,fns2,fns3))
  allocate(Vk(fns1,fns2,fns3))
  allocate(Wk(fns1,fns2,fns3))
 
  Uk=(0.0_WP,0.0_WP)
  Vk=(0.0_WP,0.0_WP)
  Wk=(0.0_WP,0.0_WP)


     
     ! Compute the Fourier coefficients
  do k=fnzs,fnze
     kl = k - fnzs + 1
     do j=fnys,fnye
        jl = j - fnys +1
        do i=fnxs,fnxe
           
           ! Wavenumbers
           kx=real(i-1,WP)*dk
           ky=real(j-1,WP)*dk
           if (j.gt.nk) ky=-real(nx-j+1,WP)*dk
           kz=real(k-1,WP)*dk
           if (k.gt.nk) kz=-real(nx-k+1,WP)*dk
           kk =sqrt(kx**2+ky**2+kz**2)
           kk2=sqrt(kx**2+ky**2)
           
           il = i - fnxs+1
            ! Compute the Fourier coefficients
           if ((kk.gt.eps).and.(kk.le.kc)) then
              if (kk2.lt.eps) then
                 Uk(il,jl,kl)=(ak(i,j,k)+bk(i,j,k))/sqrt(2.0_WP)
              else
                 Uk(il,jl,kl)=(ak(i,j,k)*kk*ky+bk(i,j,k)*kx*kz)/(kk*kk2)
              end if
              if (kk2.lt.eps) then
                 Vk(il,jl,kl)=(bk(i,j,k)-ak(i,j,k))/sqrt(2.0_WP)
              else
                 Vk(il,jl,kl)=(bk(i,j,k)*ky*kz-ak(i,j,k)*kk*kx)/(kk*kk2)
              end if
              Wk(il,jl,kl)=-bk(i,j,k)*kk2/kk
           end if
        end do
     end do
  end do
  
  deallocate(ak)
  deallocate(bk)



!!$!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!$!!!! ODDBALL NOT PROPERLY HANDLED - NEED TO BE FIXED
!!$!!!! FOR NOW - ONLY THE SIMPLE BOUNDARY (1,1,K) HANDLED
!!$!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!$  
!!$  ! Oddball
!!$  do k=2,nx
!!$     do j=nk+1,nx
!!$        Uk(1,j,k)=conjg(Uk(1,ny+2-j,nz+2-k))
!!$        Vk(1,j,k)=conjg(Vk(1,ny+2-j,nz+2-k))
!!$        Wk(1,j,k)=conjg(Wk(1,ny+2-j,nz+2-k))
!!$     end do
!!$  end do

  do k=nk+1,fns3
     Uk(1,1,k)=conjg(Uk(1,1,nz+2-k))
     Vk(1,1,k)=conjg(Vk(1,1,nz+2-k))
     Wk(1,1,k)=conjg(Wk(1,1,nz+2-k))
  end do
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!


  call parser_read('Simulation type',itype)
  if(itype.eq.0) then 

  call btran_wrap(Uk,U,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(Vk,V,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(Wk,W,ns1,ns2,ns3,fns1,fns2,fns3)

!!!!!!!!!
!!  Simple correction to enforce energy spectrum after transform
!!  Convert back to wavenumber space and force spectrum
!!!!!!!!!



  call ftran_wrap(U,Uk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(V,Vk,ns1,ns2,ns3,fns1,fns2,fns3)
  call ftran_wrap(W,Wk,ns1,ns2,ns3,fns1,fns2,fns3)
  end if

  Uk = Uk/real(ns1**3.0_WP,WP)
  Vk = Vk/real(ns1**3.0_WP,WP)
  Wk = Wk/real(ns1**3.0_WP,WP)


  allocate(sg(2,nx+1),s1(nx+1))
  s1 = 0.0_WP
  !nnodes = 0
 
  allocate(node_index(fns1,fns2,fns3))

  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           il = i + fnxs -1
           jl = j + fnys -1
           kl = k + fnzs -1
           kx=real(il-1,WP)*dk
           ky=real(jl-1,WP)*dk
           if (jl.gt.nk) ky=-real(nx-jl+1,WP)*dk
           kz=real(kl-1,WP)*dk
           if (kl.gt.nk) kz=-real(nx-kl+1,WP)*dk
           kk =sqrt(kx**2+ky**2+kz**2)
           ik = 1+idint(kk/dk + 0.5_WP)
           node_index(i,j,k) = ik
           if (kk.gt.eps.and.kk.le.kc) then
              s1(ik) = s1(ik) + real(Uk(i,j,k)*conjg(Uk(i,j,k))) + &
                   & real(Vk(i,j,k)*conjg(Vk(i,j,k))) +&
                   & real(Wk(i,j,k)*conjg(Wk(i,j,k)))
              
           endif
        enddo
     enddo
  enddo


  call MPI_ALLREDUCE(s1,sg(2,:),nx+1,MPI_REAL_WP,MPI_SUM,&
       &MPI_COMM_WORLD,ierr)
  do i = 1,nx+1
     sg(1,i) = real(i-1,WP)*dk
     if (sg(2,i).gt.1e-20_WP) then
        sg(2,i) = spect(2,i)/sg(2,i)
     else
        sg(2,i) = 1.0_WP
     endif
  enddo

  do k = 1,fns3
     do j = 1,fns2
        do i = 1,fns1
           Uk(i,j,k) = Uk(i,j,k)*sqrt(sg(2,node_index(i,j,k)))
           Vk(i,j,k) = Vk(i,j,k)*sqrt(sg(2,node_index(i,j,k)))
           Wk(i,j,k) = Wk(i,j,k)*sqrt(sg(2,node_index(i,j,k)))
        enddo
     enddo
  enddo

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!! End of correction
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  call parallel_max(maxval(abs(Uk)),umax)
  call parallel_max(maxval(abs(Vk)),vmax)
  call parallel_max(maxval(abs(Wk)),wmax)


!!$  if (rank.eq.0) then
!!$     print *,' Maximum U,V,W after transforms',umax,vmax,wmax
!!$  endif
!!$  if (umax.eq.maxval(abs(Uk))) then
!!$     print *,' In processor rank ', rank
!!$     print *,' Location of maximum ', maxloc(abs(Uk))
!!$     gg = maxloc(abs(Uk))
!!$     print *,' Maximum value ', Uk(gg(1),gg(2),gg(3))
!!$  endif
!!$  if (wmax.eq.maxval(abs(Wk))) then
!!$     print *,' In processor rank ', rank
!!$     print *,' Location of maximum ', maxloc(abs(Wk))
!!$     gg = maxloc(abs(Wk))
!!$     print *,' Maximum value ', Wk(gg(1),gg(2),gg(3))
!!$  endif
  
!!$  call data_write_fourier(Uk,Vk,Wk,fns1,fns2,fns3)
  
  if(itype.eq.0) then 

  call btran_wrap(Uk,U,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(Vk,V,ns1,ns2,ns3,fns1,fns2,fns3)
  call btran_wrap(Wk,W,ns1,ns2,ns3,fns1,fns2,fns3)

  end if

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!! End of velocity initialization
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!! Input for scalar field
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

  
  call parser_read('ks/ko', ksk0ratio)
  ks = ksk0ratio*dk
!!! Assume kc/ks to be 2 as in Eswaran and Pope
  kc = 2.0_WP*ks


  allocate(scalar(fns1sc,fns2sc,fns3sc))

  scalar = (0.0_WP,0.0_WP)
  if(rank.eq.0) print*,'test nksc',nksc

  do k = 1,fns3sc
     do j = 1,fns2sc
        do i = 1,fns1sc
           il = i + fnxssc - 1
           jl = j + fnyssc - 1
           kl = k + fnzssc - 1

           kx=real(il-1,WP)*dk
           ky=real(jl-1,WP)*dk
           if (jl.gt.nksc) ky=-real(nxsc-jl+1,WP)*dk
           kz=real(kl-1,WP)*dk
           if (kl.gt.nksc) kz=-real(nxsc-kl+1,WP)*dk
           kk =sqrt(kx**2+ky**2+kz**2)
           kk2=sqrt(kx**2+ky**2)
           if ((ks-dk/2.0_WP.le.kk).and.(kk.le.ks+dk/2.0_WP)) then
              f_phi = 1.0_WP
           else
              f_phi = 0.0_WP
           endif
           call random_number(rand)
           if (kk.lt.1e-10) then
              scalar(i,j,k) = 0.0_WP
           else
              scalar(i,j,k) = sqrt(f_phi/(4.0_WP*pi*kk**2.0_WP))*exp(ii*2.0_WP*pi*rand)
           endif
        enddo
     enddo
  enddo


  call btran_wrap(scalar,SC,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
  SC = SC/real(ns1sc**3.0,WP)

  do k = 1,ns3sc
     do j = 1,ns2sc
        do i = 1,ns1sc
           if (SC(i,j,k).le.0.0_WP) then
              SC(i,j,k) = 0.0_WP
           else
              SC(i,j,k) = 1.0_WP
           endif
        enddo
     enddo
  enddo

  call ftran_wrap(SC,scalar,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)

!  kcut = sqrt(2.0_WP)*real(nxsc,WP)/3.0_WP
  kcut = real(nx,WP)/2.0_WP
!  kcut = real(nxsc,WP)/2.0_WP
  do k = 1,fns3sc
     do j = 1,fns2sc
        do i = 1,fns1sc
           il = i + fnxssc - 1
           jl = j + fnyssc - 1
           kl = k + fnzssc - 1

           kx=real(il-1,WP)*dk
           ky=real(jl-1,WP)*dk
           if (jl.gt.nksc) ky=-real(nxsc-jl+1,WP)*dk
           kz=real(kl-1,WP)*dk
           if (kl.gt.nksc) kz=-real(nxsc-kl+1,WP)*dk
           kk =sqrt(kx**2+ky**2+kz**2)
           kk2=sqrt(kx**2+ky**2)

           if (kk.gt.kcut) scalar(i,j,k) = 0.0_WP

           if (kk.le.kc) then
              scalar(i,j,k) = scalar(i,j,k)
           else
              scalar(i,j,k) = scalar(i,j,k)*(kc/kk)**2.0_WP
           endif
        enddo
     enddo
  enddo

  call btran_wrap(scalar,SC,ns1sc,ns2sc,ns3sc,fns1sc,fns2sc,fns3sc)
  SC = SC/real(ns1sc**3.0,WP)


  call parser_read('Imposing mean gradient',impose,0)
  

!  if (impose.eq.1) then
!     scmean = 0.0_WP
!     do k = 1,ns3
!        do j = 1,ns2
!           do i = 1,ns1
!              scmean = scmean + SC(i,j,k)
!           enddo
!        enddo
!     enddo
  
!     call MPI_ALLREDUCE(scmean,scmean_global,1,MPI_REAL_WP,MPI_SUM,&
!          &MPI_COMM_WORLD,ierr)
     
!     scmean = scmean_global/(real(nx,WP)**3.0_WP)
     
!!!!!!!!!!!! Setting scalar to zero - see if it lies within bounds
!     do k = 1,ns3
!        do j = 1,ns2
!           do i = 1,ns1
!              SC(i,j,k) = 0.0_WP
!           enddo
!        enddo
!     enddo
     
!  endif


  ! Inverse Fourier transform

  ! Clean up
  deallocate(Uk)
  deallocate(Vk)
  deallocate(Wk)
  deallocate(scalar)
  deallocate(node_index)
  deallocate(s1)
  deallocate(sg)
  deallocate(spect)

  spfilename = 'spectrum_init'
  spfilename2 = 'spectrum_init.scalar'
  call get_dns_numbers(1)

!  do i = 1, ns1sc
!  do j = 1, ns2sc
!  do k = 1, ns3sc
!     xx = i*L/ns1sc - L/2
!     yy = j*L/ns2sc - L/2
!     zz = k*L/ns3sc - L/2
!     SC(i,j,k) = sqrt(xx**2.0 + yy**2.0 + zz**2.0)
!  end do
!  end do
!  end do

end subroutine hit_init


