subroutine dump_data(scalar,name)

  use solver
  use parallel
  use data

  implicit none

  real(WP), dimension(ns1,ns2,ns3) :: scalar
  character(len=13) :: name 
  character(len=80) :: buffer
  integer :: iunit,ierr,size,ibuffer, k, kp
  integer :: fileview,datasize,gdatasize
  integer, dimension(MPI_STATUS_SIZE) :: status
  integer(kind=MPI_OFFSET_KIND) :: disp
  integer :: gsizes(3),lsizes(3),start(3)
  real(SP), dimension(:,:,:), pointer :: tab 

  call MPI_FILE_OPEN(MPI_COMM_WORLD,trim(name),MPI_MODE_WRONLY+MPI_MODE_CREATE,MPI_INFO_NULL,iunit,ierr)
 
  if (rank.eq.0) then
     buffer = trim(name)
     size = 80
     call MPI_FILE_WRITE(iunit,buffer,size,MPI_CHARACTER,status,ierr)
     buffer = 'part'
     size = 80
     call MPI_FILE_WRITE(iunit,buffer,size,MPI_CHARACTER,status,ierr)
     ibuffer = 1
     size = 1
     call MPI_FILE_WRITE(iunit,ibuffer,size,MPI_INTEGER,status,ierr)
     buffer = 'block'
     size = 80
     call MPI_FILE_WRITE(iunit,buffer,size,MPI_CHARACTER,status,ierr)
  end if


  ! Create the view
  gsizes(1) = nx
  gsizes(2) = nx
  gsizes(3) = nx
  lsizes(1) = ns1
  lsizes(2) = ns2
  lsizes(3) = ns3
  start(1) = nxs - 1 
  start(2) = nys - 1
  start(3) = nzs - 1
  datasize = lsizes(1)*lsizes(2)*lsizes(3)
  gdatasize= gsizes(1)*gsizes(2)*gsizes(3)

  call MPI_TYPE_CREATE_SUBARRAY(3,gsizes,lsizes,start,MPI_ORDER_FORTRAN,MPI_REAL,fileview,ierr)
  call MPI_TYPE_COMMIT(fileview,ierr)
	
  disp = 3*80+4
  allocate(tab(1:nx,1:nx,nzs:nze))
  do k = 1,ns3
     kp = k + rank*ns3 
     tab(:,:,kp) = scalar(:,:,k)
  end do
  call MPI_FILE_SET_VIEW(iunit,disp,MPI_REAL,fileview,"native",MPI_INFO_NULL,ierr)
  call MPI_FILE_WRITE_ALL(iunit,tab,datasize,MPI_REAL,status,ierr)

  call MPI_FILE_CLOSE(iunit,ierr)

end subroutine dump_data

subroutine dump_geometry

  use solver
  use parallel
  use data

  implicit none

  integer :: ierr,ipart,iunit,i,j,k
  character(len=80) :: buffer
  real(SP), dimension(:,:,:), pointer :: xbuf,ybuf,zbuf
  integer, dimension(:,:,:), pointer :: iblank
  real(SP) :: max_x,max_y,max_z
  real(SP) :: min_x,min_y,min_z

  allocate(xbuf(1:nx+1,1:nx+1,1:nx+1),ybuf(1:nx+1,1:nx+1,1:nx+1),zbuf(1:nx+1,1:nx+1,1:nx+1))
  do i=1,nx+1
     do j=1,ny+1
        do k=1,nz+1
           xbuf(i,j,k)= (real(i)-1.5)*L/real(nx)
           ybuf(i,j,k)= (real(j)-1.5)*L/real(nx)
           zbuf(i,j,k)= (real(k)-1.5)*L/real(nx)
        end do
     end do
  end do
  max_x = maxval(xbuf)
  max_y = maxval(ybuf)
  max_z = maxval(zbuf)
  min_x = minval(xbuf)
  min_y = minval(ybuf)
  min_z = minval(zbuf)

  ! Open the file
  !call BINARY_FILE_OPEN(12345,"geometry","w",ierr)
  OPEN(UNIT=12345, FILE='geometry', FORM='unformatted',access='stream')
  REWIND(12345)

  ! Write the geometry
  buffer = 'C Binary'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'Ensight Gold Geometry File'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'Structured Geometry from ARTS'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'node id off'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'element id off'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'part'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  ipart = 1
  !call BINARY_FILE_WRITE(12345,ipart,1,kind(ipart),ierr)
  write(12345) ipart

  buffer = 'Complete geometry'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'block curvilinear iblanked'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  !call BINARY_FILE_WRITE(12345,nx+1,1,kind(nx),ierr)
  !call BINARY_FILE_WRITE(12345,ny+1,1,kind(ny),ierr)
  !call BINARY_FILE_WRITE(12345,nz+1,1,kind(nz),ierr)
  write(12345) nx+1
  write(12345) ny+1
  write(12345) nz+1
  

  !call BINARY_FILE_WRITE(12345,xbuf,(nx+1)*(ny+1)*(nz+1),kind(xbuf),ierr)
  !call BINARY_FILE_WRITE(12345,ybuf,(nx+1)*(ny+1)*(nz+1),kind(ybuf),ierr)
  !call BINARY_FILE_WRITE(12345,zbuf,(nx+1)*(ny+1)*(nz+1),kind(zbuf),ierr)
  write(12345) xbuf
  write(12345) ybuf
  write(12345) zbuf

  ! Get the iblank in 3D
  allocate(iblank(nx+1,ny+1,nz+1))
  do k=1,nz+1
     iblank(1:nx+1,1:ny+1,k) = 1
  end do
  !call BINARY_FILE_WRITE(12345,iblank,(nx+1)*(ny+1)*(nz+1),kind(iblank),ierr)
  write(12345) iblank

  ! Close the file
  !call BINARY_FILE_CLOSE(12345,ierr)
  CLOSE(12345)

end subroutine dump_geometry

subroutine dump_geometry_scalar

  use solver
  use parallel
  use data

  implicit none

  integer :: ierr,ipart,iunit,i,j,k
  character(len=80) :: buffer
  real(SP), dimension(:,:,:), pointer :: xbuf,ybuf,zbuf
  integer, dimension(:,:,:), pointer :: iblank
  real(SP) :: max_x,max_y,max_z
  real(SP) :: min_x,min_y,min_z

  allocate(xbuf(1:nxsc+1,1:nxsc+1,1:nxsc+1),ybuf(1:nxsc+1,1:nxsc+1,1:nxsc+1),zbuf(1:nxsc+1,1:nxsc+1,1:nxsc+1))
  do i=1,nxsc+1
     do j=1,nysc+1
        do k=1,nzsc+1
           xbuf(i,j,k)= (real(i)-1.5)*L/real(nxsc)
           ybuf(i,j,k)= (real(j)-1.5)*L/real(nxsc)
           zbuf(i,j,k)= (real(k)-1.5)*L/real(nxsc)
        end do
     end do
  end do
  max_x = maxval(xbuf)
  max_y = maxval(ybuf)
  max_z = maxval(zbuf)
  min_x = minval(xbuf)
  min_y = minval(ybuf)
  min_z = minval(zbuf)

  print*,"j'y passe"

  ! Open the file
  !call BINARY_FILE_OPEN(12345,"geomesca","w",ierr)
  OPEN(UNIT=12345, FILE='geomesca', FORM='unformatted',access='stream')
  REWIND(12345)

  ! Write the geometry
  buffer = 'C Binary'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'Ensight Gold Geometry File'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'Structured Geometry from ARTS'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'node id off'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'element id off'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'part'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  ipart = 1
  !call BINARY_FILE_WRITE(12345,ipart,1,kind(ipart),ierr)
  write(12345) ipart

  buffer = 'Complete geometry'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  buffer = 'block curvilinear iblanked'
  !call BINARY_FILE_WRITE(12345,buffer,80,kind(buffer),ierr)
  write(12345) buffer 

  !call BINARY_FILE_WRITE(12345,nxsc+1,1,kind(nxsc),ierr)
  !call BINARY_FILE_WRITE(12345,nysc+1,1,kind(nysc),ierr)
  !call BINARY_FILE_WRITE(12345,nzsc+1,1,kind(nzsc),ierr)
  write(12345) nxsc+1
  write(12345) nysc+1
  write(12345) nzsc+1
  

  !call BINARY_FILE_WRITE(12345,xbuf,2*(nysc+1)*(nzsc+1),kind(xbuf),ierr)
  !call BINARY_FILE_WRITE(12345,ybuf,2*(nysc+1)*(nzsc+1),kind(ybuf),ierr)
  !call BINARY_FILE_WRITE(12345,zbuf,2*(nysc+1)*(nzsc+1),kind(zbuf),ierr)
  write(12345) xbuf
  write(12345) ybuf
  write(12345) zbuf

  ! Get the iblank in 3D
  allocate(iblank(nxsc+1,nysc+1,nzsc+1))
  do k=1,nzsc+1
     iblank(1:nxsc+1,1:nysc+1,k) = 1
  end do
  !call BINARY_FILE_WRITE(12345,iblank,(nxsc+1)*(nysc+1)*(nzsc+1),kind(iblank),ierr)
  write(12345) iblank

  ! Close the file
  !call BINARY_FILE_CLOSE(12345,ierr)
  CLOSE(12345)

end subroutine dump_geometry_scalar


subroutine dump_data_sc(scalar,name)

  use solver
  use parallel
  use data

  implicit none

  real(WP), dimension(ns1sc,ns2sc,ns3sc) :: scalar
  character(len=13) :: name 
  character(len=80) :: buffer
  integer :: iunit,ierr,size,ibuffer, k, kp
  integer :: fileview,datasize,gdatasize
  integer, dimension(MPI_STATUS_SIZE) :: status
  integer(kind=MPI_OFFSET_KIND) :: disp
  integer :: gsizes(3),lsizes(3),start(3)
  real(SP), dimension(:,:,:), pointer :: tab 

  call MPI_FILE_OPEN(MPI_COMM_WORLD,trim(name),MPI_MODE_WRONLY+MPI_MODE_CREATE,MPI_INFO_NULL,iunit,ierr)
 
  if (rank.eq.0) then
     buffer = trim(name)
     size = 80
     call MPI_FILE_WRITE(iunit,buffer,size,MPI_CHARACTER,status,ierr)
     buffer = 'part'
     size = 80
     call MPI_FILE_WRITE(iunit,buffer,size,MPI_CHARACTER,status,ierr)
     ibuffer = 1
     size = 1
     call MPI_FILE_WRITE(iunit,ibuffer,size,MPI_INTEGER,status,ierr)
     buffer = 'block'
     size = 80
     call MPI_FILE_WRITE(iunit,buffer,size,MPI_CHARACTER,status,ierr)
  end if


  ! Create the view
  gsizes(1) = nxsc
  gsizes(2) = nxsc
  gsizes(3) = nxsc
  lsizes(1) = ns1sc
  lsizes(2) = ns2sc
  lsizes(3) = ns3sc
  start(1) = nxssc - 1 
  start(2) = nyssc - 1
  start(3) = nzssc - 1
  datasize = lsizes(1)*lsizes(2)*lsizes(3)
  gdatasize= gsizes(1)*gsizes(2)*gsizes(3)

  call MPI_TYPE_CREATE_SUBARRAY(3,gsizes,lsizes,start,MPI_ORDER_FORTRAN,MPI_REAL,fileview,ierr)
  call MPI_TYPE_COMMIT(fileview,ierr)
	
  disp = 3*80+4
  allocate(tab(1:nxsc,1:nxsc,nzssc:nzesc))
  do k = 1,ns3sc
     kp = k + rank*ns3sc 
     tab(:,:,kp) = scalar(:,:,k)
  end do
  call MPI_FILE_SET_VIEW(iunit,disp,MPI_REAL,fileview,"native",MPI_INFO_NULL,ierr)
  call MPI_FILE_WRITE_ALL(iunit,tab,datasize,MPI_REAL,status,ierr)

  call MPI_FILE_CLOSE(iunit,ierr)

end subroutine dump_data_sc
