module remaillage_mod
  use donnees_mod
  use tab_mod
contains




  subroutine remaill_l3 (donne,posx,posy,remaille)       
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(0:3)                        :: ip,jp
    real(kind=8),dimension(0:3)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2

       jp(1) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(1) - 1        
       jp(2) = jp(1) + 1         
       jp(3) = jp(1) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx   
       yy1 = (posy(i) - real(jp(1),kind=8)*dy-yb)/dy  


       !conditions au bord
       !------------------
       !periodique:         
       do c=0,3               
          ip(c)=mod(ip(c)+nx,nx)                
       end do
       do c=0,3               
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !----------------
       poidx(0)=-1./6.*xx1*(xx1-1.)*(xx1-2.)
       poidx(1)=0.5*(1.-xx1)*(1.+xx1)*(2.-xx1)
       poidx(2)=-0.5*xx1*(xx1+1.)*(xx1-2.)
       poidx(3)=1/6.*xx1*(1.+xx1)*(xx1-1.)

       poidy(0)=-1./6.*yy1*(yy1-1.)*(yy1-2.)
       poidy(1)=0.5*(1.-yy1)*(1.+yy1)*(2.-yy1)
       poidy(2)=-0.5*yy1*(yy1+1.)*(yy1-2.)
       poidy(3)=1/6.*yy1*(1.+yy1)*(yy1-1.)

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,3
          do d=0,3              
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)
          end do
       end do
    end do

  end subroutine remaill_l3

  subroutine remaill_l4 (donne,posx,posy,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(0:5)                        :: ip,jp
    real(kind=8),dimension(0:5)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1,x2,x3,x4,y2,y3,y4

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   !relatif
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy   !relatif



       !conditions au bord
       !------------------
       !periodique:    
       do c=0,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !----------------   
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4

       if (xx1<=0.5) then
          poidx(0)=(2.*xx1-x2-2*x3+x4)/24.
          poidx(1)=(-4.*xx1+4.*x2+x3-x4)/6.
          poidx(2)=1.+(-5.*x2+x4)/4.
          poidx(3)=(4.*xx1+4.*x2-x3-x4)/6.
          poidx(4)=(-2.*xx1-x2+2*x3+x4)/24.
          poidx(5)=0.
       else
          poidx(0)=0.
          poidx(1)=(-6.*xx1+11.*x2-6.*x3+x4)/24.
          poidx(2)=1.+(-5.*xx1-5.*x2+5.*x3-x4)/6.
          poidx(3)=(6.*xx1+x2-4.*x3+x4)/4.
          poidx(4)=(-3.*xx1+x2+3.*x3-x4)/6.
          poidx(5)=(2.*xx1-x2-2.*x3+x4)/24.
       end if

       if (yy1<=0.5) then
          poidy(0)=(2.*yy1-y2-2*y3+y4)/24.
          poidy(1)=(-4.*yy1+4.*y2+y3-y4)/6.
          poidy(2)=1.+(-5.*y2+y4)/4.
          poidy(3)=(4.*yy1+4.*y2-y3-y4)/6.
          poidy(4)=(-2.*yy1-y2+2*y3+y4)/24.
          poidy(5)=0.
       else
          poidy(0)=0.
          poidy(1)=(-6.*yy1+11.*y2-6.*y3+y4)/24.
          poidy(2)=1.+(-5.*yy1-5.*y2+5.*y3-y4)/6.
          poidy(3)=(6.*yy1+y2-4.*y3+y4)/4.
          poidy(4)=(-3.*yy1+y2+3.*y3-y4)/6.
          poidy(5)=(2.*yy1-y2-2.*y3+y4)/24.
       end if

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          do d=0,5              
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)
          end do
       end do
    end do
  end subroutine remaill_l4

 subroutine remaill_mppp6 (donne,posx,posy,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(0:5)                        :: ip,jp
    real(kind=8),dimension(0:5)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1,x2,x3,x4,y2,y3,y4

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   !relatif
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy   !relatif



       !conditions au bord
       !------------------
       !periodique:    
       do c=0,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !----------------   
       x2=xx1**2
       x3=xx1*x2
       x4=xx1*x3  

       poidx(0)=9./88.*xx1-3./16.*x2+27./176.*x4-3./44.*xx1**5
       poidx(1)=-31./44.*xx1+19./16.*x2-1./44.*x3-141./176.*x4+15./44.*xx1**5
       poidx(2)=147./88.*x4-175./88.*x2+1.-15./22.*xx1**5
       poidx(3)=31./44.*xx1+107./88.*x2+3./22.*x3-153./88.*x4+15./22.*xx1**5
       poidx(4)=-9./88.*xx1-49./176.*x2-2./11.*x3+159./176.*x4-15./44.*xx1**5
       poidx(5)=9./176.*x2+3./44.*x3-3./16.*x4+3./44.*xx1**5

       y2=yy1**2
       y3=yy1*y2
       y4=yy1*y3  

       poidy(0)=9./88.*yy1-3./16.*y2+27./176.*y4-3./44.*yy1**5
       poidy(1)=-31./44.*yy1+19./16.*y2-1./44.*y3-141./176.*y4+15./44.*yy1**5
       poidy(2)=147./88.*y4-175./88.*y2+1.-15./22.*yy1**5
       poidy(3)=31./44.*yy1+107./88.*y2+3./22.*y3-153./88.*y4+15./22.*yy1**5
       poidy(4)=-9./88.*yy1-49./176.*y2-2./11.*y3+159./176.*y4-15./44.*yy1**5
       poidy(5)=9./176.*y2+3./44.*y3-3./16.*y4+3./44.*yy1**5

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          do d=0,5              
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)
          end do
       end do
    end do
  end subroutine remaill_mppp6

  subroutine remaill_l5 (donne,posx,posy,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(0:5)                        :: ip,jp
    real(kind=8),dimension(0:5)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1,x2,x3,x4,x5,y1,y2,y3,y4,y5

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(2) - 2        
       jp(1) = jp(2) - 1         
       jp(3) = jp(2) + 1
       jp(4) = jp(2) + 2
       jp(5) = jp(2) + 3



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   !relatif
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy   !relatif



       !conditions au bord
       !------------------
       !periodique:          
       do c=0,5               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,5               
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !----------------  
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4
       y5=yy1**5

       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.

       poidy(0)=yy1/20.-y2/24.-y3/24.+y4/24.-y5/120.
       poidy(1)=-yy1/2.+2.*y2/3.-y3/24.-y4/6.+y5/24.
       poidy(2)=1.-yy1/3.-5*y2/4.+5.*y3/12.+y4/4.-y5/12.
       poidy(3)=yy1+2.*y2/3.-7.*y3/12.-y4/6.+y5/12.
       poidy(4)=-yy1/4.-y2/24.+7.*y3/24.+y4/24.-y5/24.
       poidy(5)=yy1/30.-y3/24.+y5/120.



       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          do d=0,5              
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)
          end do
       end do
    end do

  end subroutine remaill_l5

  subroutine remaill_l6 (donne,posx,posy,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(-3:4)                        :: ip,jp
    real(kind=8),dimension(-3:4)                   :: poidx,poidy
    real(kind=8)                                  :: xx,yy,x2,x3,x4,x5,y1,y2,y3,y4,y5

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4

       jp(0) = floor((posy(i)-yb)/dy)         
       jp(-1) = jp(0) - 1        
       jp(-2) = jp(0) - 2         
       jp(-3) = jp(0) - 3
       jp(1) = jp(0) + 1
       jp(2) = jp(0) + 2
       jp(3) = jp(0) + 3
       jp(4) = jp(0) + 4
      



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif
       yy = (posy(i) - real(jp(0),kind=8)*dy-yb)/dy   !relatif



       !conditions au bord
       !------------------
       !periodique:          
       do c=-3,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=-3,4              
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !---------------- 
       if (xx<=0.5) then
          poidx(-3)=(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/720.
          poidx(-2)=-(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+3.)/120.
          poidx(-1)=(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+2.)*(xx+3.)/48.
          poidx(0)=-(xx-3.)*(xx-2.)*(xx-1.)*(xx+1.)*(xx+2.)*(xx+3.)/36.
          poidx(1)=(xx-3.)*(xx-2.)*xx*(xx+1.)*(xx+2.)*(xx+3.)/48.
          poidx(2)=-(xx-3.)*(xx-1.)*xx*(xx+1.)*(xx+2.)*(xx+3.)/120.
          poidx(3)=(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)*(xx+3.)/720.
          poidx(4)=0.
       else
          poidx(-3)=0.
          poidx(-2)=(xx-4.)*(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)/720.
          poidx(-1)=-(xx-4.)*(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+2.)/120.
          poidx(0)=(xx-4.)*(xx-3.)*(xx-2.)*(xx-1.)*(xx+1.)*(xx+2.)/48.
          poidx(1)=-(xx-4.)*(xx-3.)*(xx-2.)*xx*(xx+1.)*(xx+2.)/36.
          poidx(2)=(xx-4.)*(xx-3.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/48.
          poidx(3)=-(xx-4.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/120.
          poidx(4)=(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/720.
       end if

       if (yy<=0.5) then
          poidy(-3)=(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/720.
          poidy(-2)=-(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+3.)/120.
          poidy(-1)=(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+2.)*(yy+3.)/48.
          poidy(0)=-(yy-3.)*(yy-2.)*(yy-1.)*(yy+1.)*(yy+2.)*(yy+3.)/36.
          poidy(1)=(yy-3.)*(yy-2.)*yy*(yy+1.)*(yy+2.)*(yy+3.)/48.
          poidy(2)=-(yy-3.)*(yy-1.)*yy*(yy+1.)*(yy+2.)*(yy+3.)/120.
          poidy(3)=(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)*(yy+3.)/720.
          poidy(4)=0.
       else
          poidy(-3)=0.
          poidy(-2)=(yy-4.)*(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)/720.
          poidy(-1)=-(yy-4.)*(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+2.)/120.
          poidy(0)=(yy-4.)*(yy-3.)*(yy-2.)*(yy-1.)*(yy+1.)*(yy+2.)/48.
          poidy(1)=-(yy-4.)*(yy-3.)*(yy-2.)*yy*(yy+1.)*(yy+2.)/36.
          poidy(2)=(yy-4.)*(yy-3.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/48.
          poidy(3)=-(yy-4.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/120.
          poidy(4)=(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/720.
       end if

      


       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=-3,4
          do d=-3,4              
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)
          end do
       end do
    end do

  end subroutine remaill_l6

  subroutine remaill_mp4 (donne,posx,posy,remaille)       
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(0:3)                        :: ip,jp
    real(kind=8),dimension(0:3)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1,x2,x3,x4,x5,y2,y3,y4,y5

    remaille=0.      

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2

       jp(1) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(1) - 1        
       jp(2) = jp(1) + 1         
       jp(3) = jp(1) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx   
       yy1 = (posy(i) - real(jp(1),kind=8)*dy-yb)/dy  


       !conditions au bord
       !------------------
       !periodique:         
       do c=0,3               
          ip(c)=mod(ip(c)+nx,nx)                
       end do
       do c=0,3               
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       poidx(0)=-0.5*xx1+x2-0.5*x3
       poidx(1)=1-5./2.*x2+3.*0.5*x3
       poidx(2)=0.5*xx1+2.*x2-3.*0.5*x3
       poidx(3)=-0.5*x2+0.5*x3
       y2=yy1**2
       y3=yy1**3
       y4=yy1**4
       y5=yy1**5
       poidy(0)=-0.5*yy1+y2-0.5*y3
       poidy(1)=1-5./2.*y2+3.*0.5*y3
       poidy(2)=0.5*yy1+2.*y2-3.*0.5*y3
       poidy(3)=-0.5*y2+0.5*y3

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,3
          do d=0,3              
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)
          end do
       end do
    end do

  end subroutine remaill_mp4

  subroutine remaill_l2_g (donne,posx,posy,remaille)       
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(0:3)                        :: ip,jp
    real(kind=8),dimension(0:3)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2

       jp(1) = floor((posy(i)-yb)/dy)         
       jp(0) = jp(1) - 1        
       jp(2) = jp(1) + 1         
       jp(3) = jp(1) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx   !relatif
       yy1 = (posy(i) - real(jp(1),kind=8)*dy-yb)/dy   !relatif

       !conditions au bord
       !------------------
       !periodique:          
       do c=0,3               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,3               
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !----------------
       poidx(0)=-0.5*xx1*(1.-xx1)
       poidx(1)=1.-xx1**2
       poidx(2)=0.5*xx1*(1.+xx1)
       poidx(3)=0.

       poidy(0)=-0.5*yy1*(1.-yy1)
       poidy(1)=1.-yy1**2
       poidy(2)=0.5*yy1*(1.+yy1)
       poidy(3)=0.

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,3
          do d=0,3
             if ((ip(c)>-1).and.(ip(c)<nx).and.(jp(d)>-1).and.(jp(d)<ny)) then
                remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)
             end if
          end do
       end do
    end do

  end subroutine remaill_l2_g

 subroutine remaill_l2 (donne,posx,posy,remaille)       
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(1:4)                        :: ip,jp
    real(kind=8),dimension(1:4)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(1) = ip(2) - 1        
       ip(3) = ip(2) + 1         
       ip(4) = ip(2) + 2

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(1) = jp(2) - 1        
       jp(3) = jp(2) + 1         
       jp(4) = jp(2) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   !relatif
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy   !relatif

       !conditions au bord
       !------------------
       !periodique:          
       do c=1,4             
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=1,4               
          jp(c)=mod(jp(c)+ny,ny)                
       end do
     

       !calcul des poids
       !----------------
       if (xx1<=0.5) then   

          poidx(1)=-0.5*xx1*(1.-xx1)
          poidx(2)=1.-xx1**2
          poidx(3)=0.5*xx1*(1.+xx1)
          poidx(4)=0.
          
       else
          
          poidx(1)=0.
          poidx(2)=0.5*(xx1-1.)*(xx1-2.)
          poidx(3)=xx1*(2.-xx1)
          poidx(4)=0.5*xx1*(xx1-1.)
          
       end if

       if (yy1<=0.5) then
          
          poidy(1)=-0.5*yy1*(1.-yy1)
          poidy(2)=1.-yy1**2
          poidy(3)=0.5*yy1*(1.+yy1)
          poidy(4)=0.
          
       else
          
          poidy(1)=0.
          poidy(2)=0.5*(yy1-1.)*(yy1-2.)
          poidy(3)=yy1*(2.-yy1)
          poidy(4)=0.5*yy1*(yy1-1.)        
          
       end if

      
       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=1,4
          do d=1,4             
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)            
          end do
       end do
    end do

  end subroutine remaill_l2

  subroutine remaill_l1 (donne,posx,posy,remaille)       
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(1:2)                        :: ip,jp
    real(kind=8),dimension(0:3)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)                  
       ip(2) = ip(1) + 1         


       jp(1) = floor((posy(i)-yb)/dy)                      
       jp(2) = jp(1) + 1         


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx  
       yy1 = (posy(i) - real(jp(1),kind=8)*dy-yb)/dy  

       !conditions au bord
       !------------------
       !periodique: 
       do c=0,3               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=0,3               
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids:
       !----------------
       poidx(1)=(1.-xx1)
       poidx(2)=xx1

       poidy(1)=(1.-yy1)
       poidy(2)=yy1

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=1,2
          do d=1,2              
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)              
          end do
       end do

    end do

  end subroutine remaill_l1

  subroutine remaill_tsc (donne,posx,posy,remaille)       
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb
    integer,dimension(1:4)                        :: ip,jp
    real(kind=8),dimension(1:4)                   :: poidx,poidy
    real(kind=8)                                  :: xx1,yy1,alpha

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(1) = ip(2) - 1        
       ip(3) = ip(2) + 1         
       ip(4) = ip(2) + 2

       jp(2) = floor((posy(i)-yb)/dy)         
       jp(1) = jp(2) - 1        
       jp(3) = jp(2) + 1         
       jp(4) = jp(2) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   !relatif
       yy1 = (posy(i) - real(jp(2),kind=8)*dy-yb)/dy   !relatif

       !conditions au bord
       !------------------
       !periodique:          
       do c=1,4             
          ip(c)=mod(ip(c)+nx,nx)                
       end do

       do c=1,4               
          jp(c)=mod(jp(c)+ny,ny)                
       end do

       alpha=1./8.

       !calcul des poids
       !----------------
       if (xx1<=0.5) then
          
          poidx(1)=alpha-0.5*xx1+0.5*xx1**2
          poidx(2)=(1.-2.*alpha)-xx1**2
          poidx(3)=alpha+0.5*xx1+0.5*xx1**2
          poidx(4)=0.
          
       else
          
          poidx(1)=0.
          poidx(2)=1.+alpha-3.*0.5*xx1+0.5*xx1**2
          poidx(3)=-2.*alpha+2.*xx1-xx1**2
          poidx(4)=alpha-0.5*xx1+0.5*xx1**2     
          
       end if

       if (yy1<=0.5) then
          
          poidy(1)=alpha-0.5*yy1+0.5*yy1**2
          poidy(2)=(1.-2.*alpha)-yy1**2
          poidy(3)=alpha+0.5*yy1+0.5*yy1**2
          poidy(4)=0.
          
       else
          
          poidy(1)=0.
          poidy(2)=1.+alpha-3.*0.5*yy1+0.5*yy1**2
          poidy(3)=-2.*alpha+2.*yy1-yy1**2
          poidy(4)=alpha-0.5*yy1+0.5*yy1**2     
          
       end if

      
       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=1,4
          do d=1,4             
             remaille(ip(c)+1,jp(d)+1)=remaille(ip(c)+1,jp(d)+1)+donne(i)*poidx(c)*poidy(d)            
          end do
       end do
    end do

  end subroutine remaill_tsc

























  subroutine remaill_l3_x(donne,posx,posy,remaille)    
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:3)                                       :: ip
    real(kind=8),dimension(0:3)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                           
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx   !relatif

       !conditions au bord
       !------------------         
       !periodique: 
       do c=0,3             
          ip(c)=mod(ip(c)+nx,nx)            
       end do


       !calcul des poids
       !----------------
       poidx(0)=-1./6.*xx1*(xx1-1.)*(xx1-2.)
       poidx(1)=0.5*(1.-xx1)*(1.+xx1)*(2.-xx1)
       poidx(2)=-0.5*xx1*(xx1+1.)*(xx1-2.)
       poidx(3)=1/6.*xx1*(1.+xx1)*(xx1-1.)

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,3           
          remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poidx(c)           
       end do


    end do

  end subroutine remaill_l3_x


  subroutine remaill_l3_y(donne,posx,posy,remaille)   
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:3)                                       :: ip
    real(kind=8),dimension(0:3)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posy(i)-yb)/dy)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                  
       xx1 = (posy(i) - real(ip(1),kind=8)*dy-yb)/dy   !relatif       

       !conditions au bord
       !------------------         
       !periodique: 
       do c=0,3             
          ip(c)=mod(ip(c)+ny,ny)            
       end do

       !calcul des poids
       !----------------
       poidx(0)=-1./6.*xx1*(xx1-1.)*(xx1-2.)
       poidx(1)=0.5*(1.-xx1)*(1.+xx1)*(2.-xx1)
       poidx(2)=-0.5*xx1*(xx1+1.)*(xx1-2.)
       poidx(3)=1/6.*xx1*(1.+xx1)*(xx1-1.)

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,3           
          remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poidx(c)           
       end do


    end do

  end subroutine remaill_l3_y






  subroutine remaill_mp4_x(donne,posx,posy,remaille)    
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:3)                                       :: ip
    real(kind=8),dimension(0:3)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos
    real(kind=8)                                                 :: x2,x3,x4,x5
 

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                           
       xx1 = (posx(i) - real(ip(1),kind=8)*dx-xg)/dx   !relatif

       !conditions au bord
       !------------------         
       !periodique: 
       do c=0,3             
          ip(c)=mod(ip(c)+nx,nx)            
       end do


       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       poidx(0)=-0.5*xx1+x2-0.5*x3
       poidx(1)=1-5./2.*x2+3.*0.5*x3
       poidx(2)=0.5*xx1+2.*x2-3.*0.5*x3
       poidx(3)=-0.5*x2+0.5*x3


       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,3           
          remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poidx(c)           
       end do


    end do

  end subroutine remaill_mp4_x


  subroutine remaill_mp4_y(donne,posx,posy,remaille)   
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:3)                                       :: ip
    real(kind=8),dimension(0:3)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos
    real(kind=8)                                                 :: x2,x3,x4,x5

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(1) = floor((posy(i)-yb)/dy)         
       ip(0) = ip(1) - 1        
       ip(2) = ip(1) + 1         
       ip(3) = ip(1) + 2


       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                  
       xx1 = (posy(i) - real(ip(1),kind=8)*dy-yb)/dy   !relatif       

       !conditions au bord
       !------------------         
       !periodique: 
       do c=0,3             
          ip(c)=mod(ip(c)+ny,ny)            
       end do

       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       poidx(0)=-0.5*xx1+x2-0.5*x3
       poidx(1)=1-5./2.*x2+3.*0.5*x3
       poidx(2)=0.5*xx1+2.*x2-3.*0.5*x3
       poidx(3)=-0.5*x2+0.5*x3

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,3           
          remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poidx(c)           
       end do


    end do

  end subroutine remaill_mp4_y






  subroutine remaill_l4_x(donne,posx,posy,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos,x2,x3,x4

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3
      

       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                           
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   !relatif

       !conditions au bord
       !------------------
      
       !periodique: 
       do c=0,5             
          ip(c)=mod(ip(c)+nx,nx)            
       end do
      

       !calcul des poids
       !---------------- 
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4

       if (xx1<=0.5) then
          poidx(0)=(2.*xx1-x2-2*x3+x4)/24.
          poidx(1)=(-4.*xx1+4.*x2+x3-x4)/6.
          poidx(2)=1.+(-5.*x2+x4)/4.
          poidx(3)=(4.*xx1+4.*x2-x3-x4)/6.
          poidx(4)=(-2.*xx1-x2+2*x3+x4)/24.
          poidx(5)=0.
       else
          poidx(0)=0.
          poidx(1)=(-6.*xx1+11.*x2-6.*x3+x4)/24.
          poidx(2)=1.+(-5.*xx1-5.*x2+5.*x3-x4)/6.
          poidx(3)=(6.*xx1+x2-4.*x3+x4)/4.
          poidx(4)=(-3.*xx1+x2+3.*x3-x4)/6.
          poidx(5)=(2.*xx1-x2-2.*x3+x4)/24.
       end if

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poidx(c)            
       end do


    end do

  end subroutine remaill_l4_x


  subroutine remaill_l4_y(donne,posx,posy,remaille)   
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos,x2,x3,x4

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posy(i)-yb)/dy)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2         
       ip(5) = ip(2) + 3

     

       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                  
       xx1 = (posy(i) - real(ip(2),kind=8)*dy-yb)/dy   !relatif       

       !conditions au bord
       !------------------
      
       !periodique: 
       do c=0,5            
          ip(c)=mod(ip(c)+ny,ny)            
       end do
      

       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       if (xx1<=0.5) then
          poidx(0)=(2.*xx1-x2-2*x3+x4)/24.
          poidx(1)=(-4.*xx1+4.*x2+x3-x4)/6.
          poidx(2)=1.+(-5.*x2+x4)/4.
          poidx(3)=(4.*xx1+4.*x2-x3-x4)/6.
          poidx(4)=(-2.*xx1-x2+2*x3+x4)/24.
          poidx(5)=0.
       else
          poidx(0)=0.
          poidx(1)=(-6.*xx1+11.*x2-6.*x3+x4)/24.
          poidx(2)=1.+(-5.*xx1-5.*x2+5.*x3-x4)/6.
          poidx(3)=(6.*xx1+x2-4.*x3+x4)/4.
          poidx(4)=(-3.*xx1+x2+3.*x3-x4)/6.
          poidx(5)=(2.*xx1-x2-2.*x3+x4)/24.
       end if

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l4_y

 subroutine remaill_l6_x (donne,posx,posy,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb,j
    integer,dimension(-3:4)                        :: ip,jp
    real(kind=8),dimension(-3:4)                   :: poidx,poidy
    real(kind=8)                                  :: xx,yy,x2,x3,x4,x5,y1,y2,y3,y4,y5

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(0) = floor((posx(i)-xg)/dx)         
       ip(-1) = ip(0) - 1        
       ip(-2) = ip(0) - 2         
       ip(-3) = ip(0) - 3
       ip(1) = ip(0) + 1
       ip(2) = ip(0) + 2
       ip(3) = ip(0) + 3
       ip(4) = ip(0) + 4

      



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------         
       xx = (posx(i) - real(ip(0),kind=8)*dx-xg)/dx   !relatif      



       !conditions au bord
       !------------------
       !periodique:          
       do c=-3,4               
          ip(c)=mod(ip(c)+nx,nx)                
       end do

      

       !calcul des poids
       !---------------- 
       if (xx<=0.5) then
          poidx(-3)=(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/720.
          poidx(-2)=-(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+3.)/120.
          poidx(-1)=(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+2.)*(xx+3.)/48.
          poidx(0)=-(xx-3.)*(xx-2.)*(xx-1.)*(xx+1.)*(xx+2.)*(xx+3.)/36.
          poidx(1)=(xx-3.)*(xx-2.)*xx*(xx+1.)*(xx+2.)*(xx+3.)/48.
          poidx(2)=-(xx-3.)*(xx-1.)*xx*(xx+1.)*(xx+2.)*(xx+3.)/120.
          poidx(3)=(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)*(xx+3.)/720.
          poidx(4)=0.
       else
          poidx(-3)=0.
          poidx(-2)=(xx-4.)*(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)/720.
          poidx(-1)=-(xx-4.)*(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+2.)/120.
          poidx(0)=(xx-4.)*(xx-3.)*(xx-2.)*(xx-1.)*(xx+1.)*(xx+2.)/48.
          poidx(1)=-(xx-4.)*(xx-3.)*(xx-2.)*xx*(xx+1.)*(xx+2.)/36.
          poidx(2)=(xx-4.)*(xx-3.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/48.
          poidx(3)=-(xx-4.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/120.
          poidx(4)=(xx-3.)*(xx-2.)*(xx-1.)*xx*(xx+1.)*(xx+2.)/720.
       end if

      

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=-3,4                
          remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poidx(c)         
       end do
    end do

  end subroutine remaill_l6_x

 subroutine remaill_l6_y (donne,posx,posy,remaille)  
    implicit none
    real(kind=8),dimension(1:npart),intent(in)    :: donne
    real(kind=8),dimension(1:npart),intent(in)    :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out) :: remaille
    integer                                       :: i,c,d,ib,jb,ivar
    integer,dimension(-3:4)                        :: ip,jp
    real(kind=8),dimension(-3:4)                   :: poidx,poidy
    real(kind=8)                                  :: xx,yy,x2,x3,x4,x5,y1,y2,y3,y4,y5

    remaille=0.      

    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
      

       jp(0) = floor((posy(i)-yb)/dy)         
       jp(-1) = jp(0) - 1        
       jp(-2) = jp(0) - 2         
       jp(-3) = jp(0) - 3
       jp(1) = jp(0) + 1
       jp(2) = jp(0) + 2
       jp(3) = jp(0) + 3
       jp(4) = jp(0) + 4
      



       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------               
       yy = (posy(i) - real(jp(0),kind=8)*dy-yb)/dy   !relatif



       !conditions au bord
       !------------------
       !periodique:          
      
       do c=-3,4              
          jp(c)=mod(jp(c)+ny,ny)                
       end do


       !calcul des poids
       !---------------- 
     
       if (yy<=0.5) then
          poidy(-3)=(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/720.
          poidy(-2)=-(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+3.)/120.
          poidy(-1)=(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+2.)*(yy+3.)/48.
          poidy(0)=-(yy-3.)*(yy-2.)*(yy-1.)*(yy+1.)*(yy+2.)*(yy+3.)/36.
          poidy(1)=(yy-3.)*(yy-2.)*yy*(yy+1.)*(yy+2.)*(yy+3.)/48.
          poidy(2)=-(yy-3.)*(yy-1.)*yy*(yy+1.)*(yy+2.)*(yy+3.)/120.
          poidy(3)=(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)*(yy+3.)/720.
          poidy(4)=0.
       else
          poidy(-3)=0.
          poidy(-2)=(yy-4.)*(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)/720.
          poidy(-1)=-(yy-4.)*(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+2.)/120.
          poidy(0)=(yy-4.)*(yy-3.)*(yy-2.)*(yy-1.)*(yy+1.)*(yy+2.)/48.
          poidy(1)=-(yy-4.)*(yy-3.)*(yy-2.)*yy*(yy+1.)*(yy+2.)/36.
          poidy(2)=(yy-4.)*(yy-3.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/48.
          poidy(3)=-(yy-4.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/120.
          poidy(4)=(yy-3.)*(yy-2.)*(yy-1.)*yy*(yy+1.)*(yy+2.)/720.
       end if

      


       !remaillage à l' interrieur domaine
       !---------------------------------         
       
          do d=-3,4              
             remaille(ivar,jp(d)+1)=remaille(ivar,jp(d)+1)+donne(i)*poidy(d)
          end do
      
    end do

  end subroutine remaill_l6_y



  subroutine remaill_l2_x(donne,posx,posy,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos,x2,x3,x4

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = int((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3

       if ((posx(i)-xg)<0) then
          do c=0,5
             ip(c)=ip(c)-1
          end do
       end if

       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                           
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   !relatif

       !conditions au bord
       !------------------

       do c=0,5             
          ip(c)=mod(ip(c)+nx,nx)            
       end do


       !calcul des poids
       !---------------- 
       if (xx1<=0.5) then
          poidx(1)=0.5*xx1*(xx1-1.)
          poidx(2)=1.-xx1**2
          poidx(3)=0.5*xx1*(1.+xx1)
          poidx(4)=0.
       else
          poidx(1)=0.
          poidx(2)=0.5*(1.-xx1)*(2.-xx1)
          poidx(3)=2.*xx1-xx1**2
          poidx(4)=0.5*(xx1-1.)*xx1
       end if

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=1,4
          remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poidx(c)            
       end do


    end do

  end subroutine remaill_l2_x



  subroutine remaill_l2_y(donne,posx,posy,remaille)   
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos,x2,x3,x4

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = int((posy(i)-yb)/dy)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2         
       ip(5) = ip(2) + 3

       if ((posy(i)-yb)<0) then
          do c=0,5
             ip(c)=ip(c)-1
          end do
       end if

       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                  
       xx1 = (posy(i) - real(ip(2),kind=8)*dy-yb)/dy   !relatif       

       !conditions au bord
       !------------------

       do c=0,5            
          ip(c)=mod(ip(c)+ny,ny)            
       end do


       !calcul des poids
       !----------------
       if (xx1<=0.5) then
          poidx(1)=0.5*xx1*(xx1-1.)
          poidx(2)=1.-xx1**2
          poidx(3)=0.5*xx1*(1.+xx1)
          poidx(4)=0.
       else
          poidx(1)=0.
          poidx(2)=0.5*(1.-xx1)*(2.-xx1)
          poidx(3)=2.*xx1-xx1**2
          poidx(4)=0.5*(xx1-1.)*xx1
       end if

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=1,4
          remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l2_y


  subroutine remaill_l5_x(donne,posx,posy,remaille)
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos,x2,x3,x4,x5

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       j=nint((posy(i)-yb)/dy)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posx(i)-xg)/dx)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2
       ip(5) = ip(2) + 3
     

       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                           
       xx1 = (posx(i) - real(ip(2),kind=8)*dx-xg)/dx   


       !conditions au bord
       !------------------
      
       !periodique: 
       do c=0,5             
          ip(c)=mod(ip(c)+nx,nx)            
       end do
      

       !calcul des poids
       !---------------- 
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poidx(c)            
       end do


    end do

  end subroutine remaill_l5_x


  subroutine remaill_l5_y(donne,posx,posy,remaille)   
    implicit none
    real(kind=8),dimension(:),intent(in)                         :: donne
    real(kind=8),dimension(:),intent(in)                         :: posx,posy
    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
    integer                                                      :: i,c,d,ib,j,ivar,k
    integer,dimension(0:5)                                       :: ip
    real(kind=8),dimension(0:5)                                  :: poidx
    real(kind=8)                                                 :: xx1,tmp_pos,x2,x3,x4,x5

    remaille=0.      


    !interpole en 2d la vorticite sur la grille en fonction de la position des particules et de leur vorticité
    !---------------------------------------------------------------------------------------------------------

    do i=1,npart

       ivar=nint((posx(i)-xg)/dx)+1

       !numero des points sur le maillage
       !--------------------------------
       ip(2) = floor((posy(i)-yb)/dy)         
       ip(0) = ip(2) - 2        
       ip(1) = ip(2) - 1         
       ip(3) = ip(2) + 1
       ip(4) = ip(2) + 2         
       ip(5) = ip(2) + 3
      

       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
       !----------------------------------------------------                  
       xx1 = (posy(i) - real(ip(2),kind=8)*dy-yb)/dy   !relatif  

       !conditions au bord
       !------------------
       
       !periodique: 
       do c=0,5            
          ip(c)=mod(ip(c)+ny,ny)            
       end do
      

       !calcul des poids
       !----------------
       x2=xx1**2
       x3=xx1**3
       x4=xx1**4
       x5=xx1**5
       poidx(0)=xx1/20.-x2/24.-x3/24.+x4/24.-x5/120.
       poidx(1)=-xx1/2.+2.*x2/3.-x3/24.-x4/6.+x5/24.
       poidx(2)=1.-xx1/3.-5*x2/4.+5.*x3/12.+x4/4.-x5/12.
       poidx(3)=xx1+2.*x2/3.-7.*x3/12.-x4/6.+x5/12.
       poidx(4)=-xx1/4.-x2/24.+7.*x3/24.+x4/24.-x5/24.
       poidx(5)=xx1/30.-x3/24.+x5/120.

       !remaillage à l' interrieur domaine
       !---------------------------------         
       do c=0,5
          remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poidx(c)
       end do


    end do

  end subroutine remaill_l5_y


!!$  subroutine remaill_m6kernel_x (donne,posx,posy,remaille)      
!!$    implicit none
!!$    real(kind=8),dimension(:),intent(in)                         :: donne
!!$    real(kind=8),dimension(:),intent(in)                         :: posx,posy
!!$    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
!!$    integer                                                      :: i,c,d,j,per
!!$    integer,dimension(-3:3)                                      :: ip
!!$    real(kind=8),dimension(-3:3)                                 :: xx
!!$
!!$
!!$    remaille=0.           
!!$
!!$    do i=1,npart
!!$
!!$       j=nint((posy(i)-yb)/dy)+1
!!$
!!$       !numero des points sur le maillage
!!$       !--------------------------------
!!$       ip(0) = floor((posx(i)-xg)/dx)         
!!$       ip(-1) = ip(0) - 1        
!!$       ip(-2) = ip(0) - 2         
!!$       ip(1) = ip(0) + 1
!!$       ip(2) = ip(0) + 2
!!$       ip(3) = ip(0) + 3
!!$
!!$
!!$
!!$       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
!!$       !----------------------------------------------------  
!!$       per=mod(i -partxun(j) +ndimx(j),ndimx(j)) + partxun(j) 
!!$       do c=-2,3
!!$          xx(c) = (-posx(per) + real(ip(c),kind=8)*dx+xg)/dx             
!!$       end do
!!$
!!$       !conditions au bord
!!$       !------------------
!!$       !periodique: 
!!$       do c=-2,3             
!!$          ip(c)=mod(ip(c)+nx,nx)     
!!$       end do
!!$
!!$
!!$       !remaillage à l'interrieur domaine
!!$       !---------------------------------         
!!$       do c=-2,3
!!$          remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(xx(c))
!!$       end do
!!$
!!$    end do
!!$  contains
!!$    function poids(x) result(p)
!!$      implicit none
!!$      real(kind=8)      :: x
!!$      real(kind=8)      :: p
!!$
!!$      !mp4
!!$      !if (abs(x)<=1.) p=1.-5.*x*x/2.+3.*abs(x)**3/2.
!!$      !if ((abs(x)>1.).and.(abs(x)<=2.)) p=(2.-abs(x))**2*(1.-abs(x))/2.
!!$      !if (abs(x)>=2.) p=0.
!!$
!!$      !lambda 3
!!$      !if (abs(x)<=1.) p=(1.-x*x)*(2.-abs(x))/2.
!!$      !if ((abs(x)>1.).and.(abs(x)<=2.)) p=(1.-abs(x))*(2.-abs(x))*(3.-abs(x))/6.
!!$      !if (abs(x)>=2.) p=0.
!!$
!!$      !M6*
!!$      !if (abs(x)<1.) p=-1./12.*(abs(x)-1.)*(24.*abs(x)**4+38.*abs(x)**3-3.*abs(x)**2+12.*abs(x)+12.)
!!$      !if ((abs(x)>=1.).and.(abs(x)<2.)) p=1./24.*(abs(x)-1.)*(abs(x)-2.)*(25.*abs(x)**3-114.*abs(x)**2+153.*abs(x)-48.)
!!$      !if ((abs(x)>=2.).and.(abs(x)<3.)) p=-1./24.*(abs(x)-2.)*(abs(x)-3.)**3*(5.*abs(x)-8.)
!!$      !if (abs(x)>=3.) p=0.
!!$
!!$      !M6'''
!!$      if (abs(x)<1.) p=-1./88.*(abs(x)-1.)*(60.*abs(x)**4-87.*abs(x)**3-87.*abs(x)**2+88.*abs(x)+88.)
!!$      if ((abs(x)>=1.).and.(abs(x)<2.)) p=1./176.*(abs(x)-1.)*(abs(x)-2.)*(60.*abs(x)**3-261.*abs(x)**2+257.*abs(x)+68.)
!!$      if ((abs(x)>=2.).and.(abs(x)<3.)) p=-3./176.*(abs(x)-2.)*(4.*abs(x)**2-17.*abs(x)+12.)*(abs(x)-3.)**2
!!$      if (abs(x)>=3.) p=0.
!!$
!!$    end function poids
!!$
!!$  end subroutine remaill_m6kernel_x
!!$
!!$
!!$
!!$  subroutine remaill_m6kernel_y (donne,posx,posy,remaille)      
!!$    implicit none
!!$    real(kind=8),dimension(:),intent(in)                         :: donne
!!$    real(kind=8),dimension(:),intent(in)                         :: posx,posy
!!$    real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
!!$    integer                                                      :: i,c,d,ivar,per
!!$    integer,dimension(-3:3)                                      :: ip
!!$    real(kind=8),dimension(-3:3)                                 :: xx
!!$
!!$
!!$    remaille=0.           
!!$
!!$    do i=1,npart
!!$
!!$       ivar=nint((posx(i)-xg)/dx)+1
!!$
!!$       !numero des points sur le maillage
!!$       !--------------------------------
!!$       ip(0) = floor((posy(i)-yb)/dy)         
!!$       ip(-1) = ip(0) - 1        
!!$       ip(-2) = ip(0) - 2         
!!$       ip(1) = ip(0) + 1
!!$       ip(2) = ip(0) + 2
!!$       ip(3) = ip(0) + 3
!!$
!!$
!!$
!!$       !distance de la particule à remailler au second point (des quatres utilisés pour le remaillage)
!!$       !----------------------------------------------------   
!!$       per=mod(i -partyun(ivar) +ndimy(ivar),ndimy(ivar)) + partyun(ivar)       
!!$       do c=-2,3
!!$          xx(c) = (-posy(per) + real(ip(c),kind=8)*dy+yb)/dy             
!!$       end do
!!$
!!$       !conditions au bord
!!$       !------------------        
!!$       do c=-2,3             
!!$          ip(c)=mod(ip(c)+ny,ny)     
!!$       end do
!!$
!!$
!!$       !remaillage à l'interrieur domaine
!!$       !---------------------------------         
!!$       do c=-2,3
!!$          remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(xx(c))
!!$       end do
!!$
!!$    end do
!!$  contains
!!$    function poids(x) result(p)
!!$      implicit none
!!$      real(kind=8)      :: x
!!$      real(kind=8)      :: p
!!$
!!$      !mp4
!!$      !if (abs(x)<=1.) p=1.-5.*x*x/2.+3.*abs(x)**3/2.
!!$      !if ((abs(x)>1.).and.(abs(x)<=2.)) p=(2.-abs(x))**2*(1.-abs(x))/2.
!!$      !if (abs(x)>=2.) p=0.
!!$
!!$      !lambda 3
!!$      !if (abs(x)<=1.) p=(1.-x*x)*(2.-abs(x))/2.
!!$      !if ((abs(x)>1.).and.(abs(x)<=2.)) p=(1.-abs(x))*(2.-abs(x))*(3.-abs(x))/6.
!!$      !if (abs(x)>=2.) p=0.
!!$
!!$      !M6*
!!$      !if (abs(x)<1.) p=-1./12.*(abs(x)-1.)*(24.*abs(x)**4+38.*abs(x)**3-3.*abs(x)**2+12.*abs(x)+12.)
!!$      !if ((abs(x)>=1.).and.(abs(x)<2.)) p=1./24.*(abs(x)-1.)*(abs(x)-2.)*(25.*abs(x)**3-114.*abs(x)**2+153.*abs(x)-48.)
!!$      !if ((abs(x)>=2.).and.(abs(x)<3.)) p=-1./24.*(abs(x)-2.)*(abs(x)-3.)**3*(5.*abs(x)-8.)
!!$      !if (abs(x)>=3.) p=0.
!!$
!!$      !M6'''
!!$      if (abs(x)<1.) p=-1./88.*(abs(x)-1.)*(60.*abs(x)**4-87.*abs(x)**3-87.*abs(x)**2+88.*abs(x)+88.)
!!$      if ((abs(x)>=1.).and.(abs(x)<2.)) p=1./176.*(abs(x)-1.)*(abs(x)-2.)*(60.*abs(x)**3-261.*abs(x)**2+257.*abs(x)+68.)
!!$      if ((abs(x)>=2.).and.(abs(x)<3.)) p=-3./176.*(abs(x)-2.)*(4.*abs(x)**2-17.*abs(x)+12.)*(abs(x)-3.)**2
!!$      if (abs(x)>=3.) p=0.
!!$
!!$    end function poids
!!$
!!$  end subroutine remaill_m6kernel_y












  subroutine remaill_l2_bloc_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx


          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)
                poids(0)=1.-xx**2
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx)
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
          case(2)
             poids(0)=1.-0.5*xx*(1.+xx)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
          case(4)
             poids(-2)=0.5*xx*(1.+xx)
             poids(-1)=-xx
             poids(0)=1.-xx**2
          case(5)
             poids(-1)=-0.5*xx+0.5*xx**2
             poids(0)=1.-poids(-1)

          end select


          select case (blocd(i))

          case(0)
             if (xx<=0.5) then
                poids(1)=0.5*xx*(1.+xx)               
             else
                poids(1)=2.*xx-xx**2
                poids(2)=0.5*(xx-1.)*xx
             end if
          case(1)
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1)
          case(3)
             poids(1)=3.*0.5*xx-0.5*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_x


     subroutine remaill_l2_bloc_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

          case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)
                poids(0)=1.-xx**2
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx)
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
          case(2)
             poids(0)=1.-0.5*xx*(1.+xx)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
          case(4)
             poids(-2)=0.5*xx*(1.+xx)
             poids(-1)=-xx
             poids(0)=1.-xx**2
          case(5)
             poids(-1)=-0.5*xx+0.5*xx**2
             poids(0)=1.-poids(-1)

          end select

          select case (blocd(i))

          case(0)
             if (xx<=0.5) then
                poids(1)=0.5*xx*(1.+xx)               
             else
                poids(1)=2.*xx-xx**2
                poids(2)=0.5*(xx-1.)*xx
             end if
          case(1)
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1)
          case(3)
             poids(1)=3.*0.5*xx-0.5*xx**2
          case(4)

          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_y


    subroutine remaill_l4_bloc_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   


       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1 
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx 


          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx-1.) 
                t2=xx**2-4.
                poids(-2)=t1/24.*(xx-2.)*(xx+1.)
                poids(-1)=-t1/6.*t2
                poids(0)=t2/4.*(xx**2-1.)
             else  
                t1=(xx-1.)*(xx-2.)*(xx-3.)
                poids(-1)=t1*xx/24.
                poids(0)=-t1*(xx+1.)/6.
             end if
          case(1)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=t2/4.*(xx**2-1.)

          case(2)
             t1=(xx-1.)*(xx-2.)/12.
             poids(-1)=t1*xx*(xx-3)/2.
             poids(0)=t1*(xx+6.)*(xx+1.)
          case(3)
             t1=0.5*(xx-1.)*(xx-2.)
             poids(-2)=t1*xx/12.*(xx+1.)
             poids(-1)=-t1*xx/3.*(xx+2.)
             poids(0)=t1*(xx+1.)
          case(4)
             t2=(xx**2-1.)
             t1=xx*(xx+2.)*0.5
             poids(-3)=t2*t1/12.
             poids(-2)=-t2*xx/6.*(xx+3.)
             poids(-1)=t1*(xx-1.)
             poids(0)=t2*(xx**2-4.)/4.
          case(5)
             t1=(xx-2.)*(xx-1.)/4.
             poids(-1)=-t1*xx/6.*(3*xx+7.)
             poids(0)=t1*(xx+2.)*(xx+1.)
          case(6)
             t1= (xx-1.)*(xx-2.)/6.
             t2=t1*(xx+1.)
             poids(-2)=t2*xx/4.
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.)
          case(7)
             t1=(xx-1.)*(xx+2.)
             t2=xx*(xx+1)/6.
             poids(-3)=t1*t2/4.
             poids(-2)=-t2*(xx-1.)
             poids(-1)=-xx*t1/6.*(xx-2.)
             poids(0)=t1*(xx-2.)/4.*(xx+1.)
          case(8)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=(xx-6.)*(xx+2.)*(xx**2-1.)/12.
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx+1.)*(xx+2.)
                poids(1)=-t1*(xx-2.)/6.
                poids(2)=t1*(xx-1.)/24.
             else  
                t1=xx*(xx+1.)*(xx-3.)
                poids(1)=t1*(xx-2.)/4.
                poids(2)=-t1*(xx-1.)/6.
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             end if
          case(1)
             t1=xx*(xx+1.)*(xx+2.)
             poids(1)=-t1*(xx-2.)/6.
             poids(2)=t1*(xx-1.)/24.           
          case(2)
             t1=xx*(xx+1.)*0.5
             t2=t1*(xx-1.)
             poids(1)=-t1*(xx-2.)
             poids(2)=-t2/3.*(xx-3.)
             poids(3)=t2*(xx-2.)/12.
          case(3)
             t1=xx*(xx+1.)/12.
             poids(1)=t1*(xx-2.)*(xx-7.)
             poids(2)=t1*(xx+2.)*(xx-1.)/2.
          case(4)
             t1=xx*(xx+1.)
             t2=t1*(xx-1.)/6.
             poids(1)=-t1*(xx+2.)*(xx-2.)/6.
             poids(2)=t2
             poids(3)=t2*(xx-2.)/4.
          case(5)
             poids(1)=-xx/24.*(3.*xx-7.)*(xx+2.)*(xx+1.)
          case(6)
             t1=xx*(xx+1.)/4.
             poids(1)=t1*(xx-3.)*(xx-2.)
             poids(2)=-t1*(3.*xx-10.)*(xx-1.)/6.
          case(7)
             poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_x


     subroutine remaill_l4_bloc_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-3:3)                                      :: ip
       real(kind=8),dimension(-3:3)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy

          !conditions au bord
          !------------------       
          do c=-3,3             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poidss
          !----------------


          select case (blocg(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx-1.) 
                t2=xx**2-4.
                poids(-2)=t1/24.*(xx-2.)*(xx+1.)
                poids(-1)=-t1/6.*t2
                poids(0)=t2/4.*(xx**2-1.)
             else  
                t1=(xx-1.)*(xx-2.)*(xx-3.)
                poids(-1)=t1*xx/24.
                poids(0)=-t1*(xx+1.)/6.
             end if
          case(1)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=t2/4.*(xx**2-1.)

          case(2)
             t1=(xx-1.)*(xx-2.)/12.
             poids(-1)=t1*xx*(xx-3)/2.
             poids(0)=t1*(xx+6.)*(xx+1.)
          case(3)
             t1=0.5*(xx-1.)*(xx-2.)
             poids(-2)=t1*xx/12.*(xx+1.)
             poids(-1)=-t1*xx/3.*(xx+2.)
             poids(0)=t1*(xx+1.)
          case(4)
             t2=(xx**2-1.)
             t1=xx*(xx+2.)*0.5
             poids(-3)=t2*t1/12.
             poids(-2)=-t2*xx/6.*(xx+3.)
             poids(-1)=t1*(xx-1.)
             poids(0)=t2*(xx**2-4.)/4.
          case(5)
             t1=(xx-2.)*(xx-1.)/4.
             poids(-1)=-t1*xx/6.*(3*xx+7.)
             poids(0)=t1*(xx+2.)*(xx+1.)
          case(6)
             t1= (xx-1.)*(xx-2.)/6.
             t2=t1*(xx+1.)
             poids(-2)=t2*xx/4.
             poids(-1)=-t1*xx
             poids(0)=-t2*(xx-3.)
          case(7)
             t1=(xx-1.)*(xx+2.)
             t2=xx*(xx+1)/6.
             poids(-3)=t1*t2/4.
             poids(-2)=-t2*(xx-1.)
             poids(-1)=-xx*t1/6.*(xx-2.)
             poids(0)=t1*(xx-2.)/4.*(xx+1.)
          case(8)
             t1=xx*(xx-1.) 
             t2=xx**2-4.
             poids(-2)=t1/24.*(xx-2.)*(xx+1.)
             poids(-1)=-t1/6.*t2
             poids(0)=(xx-6.)*(xx+2.)*(xx**2-1.)/12.
          end select





          select case (blocd(i))

          case(0)
             if (xx<=0.5) then 
                t1=xx*(xx+1.)*(xx+2.)
                poids(1)=-t1*(xx-2.)/6.
                poids(2)=t1*(xx-1.)/24.
             else  
                t1=xx*(xx+1.)*(xx-3.)
                poids(1)=t1*(xx-2.)/4.
                poids(2)=-t1*(xx-1.)/6.
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             end if
          case(1)
             t1=xx*(xx+1.)*(xx+2.)
             poids(1)=-t1*(xx-2.)/6.
             poids(2)=t1*(xx-1.)/24.           
          case(2)
             t1=xx*(xx+1.)*0.5
             t2=t1*(xx-1.)
             poids(1)=-t1*(xx-2.)
             poids(2)=-t2/3.*(xx-3.)
             poids(3)=t2*(xx-2.)/12.
          case(3)
             t1=xx*(xx+1.)/12.
             poids(1)=t1*(xx-2.)*(xx-7.)
             poids(2)=t1*(xx+2.)*(xx-1.)/2.
          case(4)
             t1=xx*(xx+1.)
             t2=t1*(xx-1.)/6.
             poids(1)=-t1*(xx+2.)*(xx-2.)/6.
             poids(2)=t2
             poids(3)=t2*(xx-2.)/4.
          case(5)
             poids(1)=-xx/24.*(3.*xx-7.)*(xx+2.)*(xx+1.)
          case(6)
             t1=xx*(xx+1.)/4.
             poids(1)=t1*(xx-3.)*(xx-2.)
             poids(2)=-t1*(3.*xx-10.)*(xx-1.)/6.
          case(7)
             poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,3
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_y









  subroutine remaill_l2_bloc_v2_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx


          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------

          select case (blocg(i))

         case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)     !alpha
                poids(0)=1.-xx**2            !beta
                poids(1)=0.5*xx*(1.+xx)      !gamma
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx) !alpha'
                poids(1)=2.*xx-xx**2         !beta'
                poids(2)=0.5*(xx-1.)*xx      !gamma'
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(0)=0.5*(1.-xx)*(2.-xx)
             poids(1)=1.-poids(0)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-poids(-1)
          case(4)
             poids(0)=-0.5*xx*(xx+1.)+1.
             poids(1)=1.-poids(0)
          case(5)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2 
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1.)
          case(6)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
             poids(1)=2.*xx-xx**2 
             poids(2)=0.5*(xx-1.)*xx
          case(7)
             poids(-2)=0.5*xx*(xx+1)
             poids(-1)=-xx
             poids(0)=1.-xx**2 
             poids(1)=0.5*xx*(1.+xx)  
          end select



          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_v2_x

    subroutine remaill_l2_bloc_v2_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poids
          !----------------

         select case (blocg(i))

          case(0)
             if (xx<=0.5) then
                poids(-1)=0.5*xx*(xx-1.)     !alpha
                poids(0)=1.-xx**2            !beta
                poids(1)=0.5*xx*(1.+xx)      !gamma
             else               
                poids(0)=0.5*(1.-xx)*(2.-xx) !alpha'
                poids(1)=2.*xx-xx**2         !beta'
                poids(2)=0.5*(xx-1.)*xx      !gamma'
             end if
          case(1)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2
             poids(1)=0.5*xx*(1.+xx)
          case(2)
             poids(0)=0.5*(1.-xx)*(2.-xx)
             poids(1)=1.-poids(0)
          case(3)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-poids(-1)
          case(4)
             poids(0)=-0.5*xx*(xx+1.)+1.
             poids(1)=1.-poids(0)
          case(5)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx**2 
             poids(1)=xx
             poids(2)=0.5*xx*(xx-1.)
          case(6)
             poids(-1)=0.5*xx*(xx-1.)
             poids(0)=1.-xx
             poids(1)=2.*xx-xx**2 
             poids(2)=0.5*(xx-1.)*xx
          case(7)
             poids(-2)=0.5*xx*(xx+1)
             poids(-1)=-xx
             poids(0)=1.-xx**2 
             poids(1)=0.5*xx*(1.+xx)  
          end select


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l2_bloc_v2_y


  subroutine remaill_l4_bloc_v2_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   


       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1 
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx 


          !conditions au bord
          !------------------       
          do c=-3,4             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------
  select case (blocg(i))
          case(0)
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
                poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
                poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
                poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
                poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
             end if
          case(1)
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          case(2)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx**4/12.-2.*xx**3/3.+5.*xx**2/12.+7.*xx/6.
          case(3)             
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6. 
             poids(0)=1.+xx**4/12.-13.*xx**2/12.-xx**3/3.+xx/3.   
          case(4)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=xx*(xx-1.)*(xx-2.)*(xx-3.)/24.
             poids(0)=xx**4/12.+xx**3/3.-13.*xx**2/12.-xx/3.+1.
          case(5)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=0.
             poids(0)=1.-xx**4/8.+7.*xx**3/12.-3.*xx**2/8.-13.*xx/12.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.
          case(6)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=-xx**4/8.+xx**3/12.+5.*xx**2/8.-7.*xx/12.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.
          case(7)
             poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=0.5*xx**3-xx**2-0.5*xx+1.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.  
          case(8)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=-xx*(xx+3.)*(xx**2-1.)/6.
             poids(-1)=0.5*xx**3+0.5*xx**2-xx
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.  
          case(9)
            poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx**3/6.+0.5*xx**2-xx/3.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4. 
          case(10)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=xx*(1.-xx**2)/6.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.   
          end select

          select case (blocd(i))        
          case(0)
             if (xx<=0.5) then                
                poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
                poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
                poids(3)=0.
                poids(4)=0.
             else                 
                poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
                poids(4)=0.
             end if
          case(1)
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
             poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
             poids(3)=0.
             poids(4)=0.
          case(2)
             poids(2)=-xx**4/8.+5.*xx**3/12.+xx**2/8.-5.*xx/12.
             poids(3)=0.
             poids(4)=0.
          case(3)
              poids(1)=-xx**4/8.-xx**3/12.+5.*xx**2/8.+7.*xx/12.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(4)
             poids(2)=xx*(xx+2.)*(xx**2-1.)/24.
             poids(3)=0.
             poids(4)=0.
          case(5)
            poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(6)            
             poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.
             poids(3)=xx**3/6.-0.5*xx*2+xx/3.
             poids(4)=xx*(xx-1)*(xx-2.)*(xx-3.)/24.
          case(7)            
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.
             poids(2)=xx*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          case(8)
               poids(1)=-0.5*xx**3+0.5*xx**2+xx
             poids(2)=-xx*(xx-3)*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          end select
          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_v2_x




     subroutine remaill_l4_bloc_v2_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3

       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy

          !conditions au bord
          !------------------       
          do c=-3,4            
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poidss
          !----------------
          select case (blocg(i))
          case(0)
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
                poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
                poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.       !a'
                poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.   !b'
                poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.         !c'
             end if
          case(1)
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24. !a
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.      !b
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.        !c
          case(2)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=(xx-1.)*(xx-2.)*(xx-3.)*xx/24.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx**4/12.-2.*xx**3/3.+5.*xx**2/12.+7.*xx/6.
          case(3)             
             poids(-3)=0.
             poids(-2)=xx*(xx-1.)*(xx-2.)*(xx+1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6. 
             poids(0)=1.+xx**4/12.-13.*xx**2/12.-xx**3/3.+xx/3.   
          case(4)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=xx*(xx-1.)*(xx-2.)*(xx-3.)/24.
             poids(0)=xx**4/12.+xx**3/3.-13.*xx**2/12.-xx/3.+1.
          case(5)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=0.
             poids(0)=1.-xx**4/8.+7.*xx**3/12.-3.*xx**2/8.-13.*xx/12.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.
          case(6)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=-xx**4/8.+xx**3/12.+5.*xx**2/8.-7.*xx/12.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.
          case(7)
             poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=0.5*xx**3-xx**2-0.5*xx+1.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4.  
          case(8)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=-xx*(xx+3.)*(xx**2-1.)/6.
             poids(-1)=0.5*xx**3+0.5*xx**2-xx
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.  
          case(9)
            poids(-3)=0.
             poids(-2)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(-1)=-xx**3/6.+0.5*xx**2-xx/3.
             poids(0)=-(xx-1.)*(xx-2.)*(xx-3.)*(xx+1.)/6.
             poids(1)=xx*(xx+1.)*(xx-3.)*(xx-2.)/4. 
          case(10)
             poids(-3)=(xx+2.)*xx*(xx**2-1.)/24.
             poids(-2)=xx*(1.-xx**2)/6.
             poids(-1)=-xx*(xx-1.)*(xx**2-4.)/6.
             poids(0)=(xx**2-4.)*(xx**2-1.)/4.   
          end select

          select case (blocd(i))        
          case(0)
             if (xx<=0.5) then                
                poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
                poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
                poids(3)=0.
                poids(4)=0.
             else                 
                poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.  !d'
                poids(3)=xx*(xx-2.)*(xx**2-1.)/24.       !e'
                poids(4)=0.
             end if
          case(1)
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.  !d
             poids(2)=xx*(xx+1.)*(xx+2.)*(xx-1.)/24.  !e
             poids(3)=0.
             poids(4)=0.
          case(2)
             poids(2)=-xx**4/8.+5.*xx**3/12.+xx**2/8.-5.*xx/12.
             poids(3)=0.
             poids(4)=0.
          case(3)
              poids(1)=-xx**4/8.-xx**3/12.+5.*xx**2/8.+7.*xx/12.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(4)
             poids(2)=xx*(xx+2.)*(xx**2-1.)/24.
             poids(3)=0.
             poids(4)=0.
          case(5)
            poids(1)=xx*(xx+1.)*(xx+2.)*(xx+3.)/24.
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(6)            
             poids(2)=-xx*(xx+1.)*(xx-3.)*(xx-1.)/6.
             poids(3)=xx**3/6.-0.5*xx*2+xx/3.
             poids(4)=xx*(xx-1)*(xx-2.)*(xx-3.)/24.
          case(7)            
             poids(1)=-xx*(xx+1.)*(xx+2.)*(xx-2.)/6.
             poids(2)=xx*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          case(8)
               poids(1)=-0.5*xx**3+0.5*xx**2+xx
             poids(2)=-xx*(xx-3)*(xx**2-1.)/6.
             poids(3)=xx*(xx-2.)*(xx**2-1.)/24.
             poids(4)=0.
          end select
        
          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do

     end subroutine remaill_l4_bloc_v2_y



     subroutine remaill_l2_bloc_v2_limit_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos
       real(kind=8),dimension(0:2)              :: phi_mdemi,phi_pdemi     
       real(kind=8),dimension(0:1)              :: phi

       remaille=0.      

       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx


          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------
  !calcul du limiteur pour cas de bloc centre avec xx>0.5
             !--------------------------------------------------------
             call calcul_phi_tvd(phi,1)         
             phi_mdemi(1)=phi(0)
             phi_pdemi(1)=phi(1)           

             !les autres cas ...
             !---------------------
             call calcul_phi_tvd(phi,0)         
             phi_mdemi(0)=phi(0)
             phi_pdemi(0)=phi(1) 

             phi_mdemi(2)=0.
             phi_pdemi(2)=0.     


          select case (blocg(i))

          case(0)
             if (xx<=0.5) then

                poids(-1)=ca(xx,0)
                if (test (i,1) ) then
                   poids(0)=cb(xx,0,1)
                   poids(1)=cc(xx,1)
                else
                   poids(0)=cb(xx,0,0)
                   poids(1)=cc(xx,0)
                end if

             else     
               
                poids(0)=ca(xx-1.,1)
                if (test (i,1) ) then
                   poids(1)=cb(xx-1.,1,1)
                   poids(2)=cc(xx-1.,1)
                else                  
                   poids(1)=cb(xx-1.,1,0)
                   poids(2)=cc(xx-1.,0)
                end if                        
          
             end if

          case(1)
               
             poids(-1)=ca(xx,0)
             if (test (i,1) ) then
                poids(0)=cb(xx,0,1)
                poids(1)=cc(xx,1)
             else
                poids(0)=cb(xx,0,0)
                poids(1)=cc(xx,0)
             end if
               

          case(2)
                poids(0)=ca(xx-1.,1)
                poids(1)=cb(xx-1.,1,0)+cc(xx-1.,0)
          case(3)
                poids(-1)=ca(xx,0)
                poids(0)=cb(xx,0)+cc(xx,0)
          case(4)
                poids(0)=ca(xx,1)+cb(xx,1,0)
                poids(1)=cc(xx,0)


                     
          case(5)
             poids(-1)=ca(xx,0)
             poids(0)=cb(xx,0)             
             poids(1)=cc(xx,0)-cc(xx-1.,1)
             poids(2)=cc(xx-1.,1)
            
          case(6)
             poids(-1)=ca(xx,0)
             poids(0)=ca(xx-1.,1)-ca(xx,0)
             poids(1)=cb(xx-1.,1)
             poids(2)=cc(xx-1.,1)


          case(7)
             poids(-2)=ca(xx+1.,0)
             poids(-1)=ca(xx,0)-ca(xx+1.,0)
             poids(0)=cb(xx,0)
             poids(1)=cc(xx,0)
          end select

         


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

 contains
   
       function coeff_l2g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)
        case(0)
           c=1.-x**2
        case(1)
           c=0.5*x*(1.+x)
        end select
        
      end function coeff_l2g

      function coeff_tscg (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)  +1./8. ! +1./6.
        case(0)
           c=-x**2+1.    -2./8. !-2./6. 
        case(1)
           c=0.5*x*(1.+x)   +1./8. ! +1./6.
        end select
        
      end function coeff_tscg
 function ca(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: ca
        
        ca=coeff_tscg(-1,x)+phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))
        
      end function ca

      function cb(x,num,num2)
        integer      :: num
        real(kind=8) :: x
        integer,optional :: num2
        real(kind=8) :: cb
       
        if (present (num2)) then
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num2)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        else
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        end if

      end function cb

      function cc(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: cc
      
        cc=coeff_tscg(1,x)+phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))
        

      end function cc
            
      function test (ipart,vois)
        implicit none
        integer       :: ipart,vois
        logical       :: test
        real(kind=8)  :: x
        
        test=.false.       
        x=(posx(mod(ipart-1+vois+npart,npart)+1) - real( floor((posx(mod(ipart-1+vois+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx
        if ( ((blocg(mod(ipart-1+vois+npart,npart)+1)==0).or.(blocg(mod(ipart-1+vois+npart,npart)+1)==2)).and.(x>0.5)) test=.true.
        
      end function test



 subroutine calcul_phi_tvd(phi_out,num) 
        implicit none
        integer                                :: num
        real(kind=8),dimension(0:1),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u
        real(kind=8)                           :: rp,rm,diff1,diff2,diff3,diff0,rmm,y,r 
        integer                                :: ib,jb
        real(kind=8),dimension(-5:5)           :: xx_vois,x,tp1,tp2,tp3

        !particules voisines
        !-------------------       
        do ib=-2,2
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        do ib=-1,2
           x(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
        end do


        !cas bloc centre xx>0.5 (psi)
        !-----------------------
        if (num==1) then
          
           do ib=0,1                          
              x(ib)=x(ib)-1.              
              tp1(ib)=6.-8.*x(ib)*x(ib)             
              tp3(ib)=4.*(x(ib)-0.5)**2          
           end do
           
           do ib=0,1
              r=(u(ib+1)-u(ib))/(u(ib)-u(ib-1))               

              !anti diffusif
              !phi_out(ib)=max(0.,min(tp3(ib),tp1(ib)*r))                            

              !du type Minmod (attention non justifié pour y>=0.8)
              phi_out(ib)=max(0.,min(1.,4.*r))

              !bornes dépendent de la condition cfl 
              !phi_out(ib)=max(0.,min(1.,tp3(ib),tp1(ib)*r))              
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.
           end do
 



        else  
           !sinon (phi)
           !-----------------------       


        do ib=-1,0
           tp1(ib)=6.-8.*x(ib)*x(ib)
           tp2(ib)=4.*(x(ib)+0.5)**2                
        end do


        do ib=0,1
              r=(u(ib-1)-u(ib-2))/(u(ib)-u(ib-1))                                                 
              !phi_out(ib)=max(0.,min(tp2(ib-1),tp1(ib-1)*r))
              phi_out(ib)=max(0.,min(1.,4.*r))
              !phi_out(ib)=max(0.,min(1.,tp2(ib-1),tp1(ib-1)*r))
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.          
        end do

     end if
    

      end subroutine calcul_phi_tvd
                       
                          

     end subroutine remaill_l2_bloc_v2_limit_x

    subroutine remaill_l2_bloc_v2_limit_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos
       real(kind=8),dimension(0:2):: phi_mdemi,phi_pdemi     
       real(kind=8),dimension(0:1):: phi
       

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poids
          !----------------
     !calcul du limiteur pour cas de bloc centre avec xx>0.5
             !--------------------------------------------------------
             call calcul_phi_tvd(phi,1)         
             phi_mdemi(1)=phi(0)
             phi_pdemi(1)=phi(1)           

             !les autres cas ...
             !---------------------
             call calcul_phi_tvd(phi,0)         
             phi_mdemi(0)=phi(0)
             phi_pdemi(0)=phi(1) 

             phi_mdemi(2)=0.
             phi_pdemi(2)=0.     


          select case (blocg(i))

          case(0)
             if (xx<=0.5) then

                poids(-1)=ca(xx,0)
                if (test (i,1) ) then
                   poids(0)=cb(xx,0,1)
                   poids(1)=cc(xx,1)
                else
                   poids(0)=cb(xx,0,0)
                   poids(1)=cc(xx,0)
                end if

             else     
               
                poids(0)=ca(xx-1.,1)
                if (test (i,1) ) then
                   poids(1)=cb(xx-1.,1,1)
                   poids(2)=cc(xx-1.,1)
                else                  
                   poids(1)=cb(xx-1.,1,0)
                   poids(2)=cc(xx-1.,0)
                end if                        
          
             end if

          case(1)
               
             poids(-1)=ca(xx,0)
             if (test (i,1) ) then
                poids(0)=cb(xx,0,1)
                poids(1)=cc(xx,1)
             else
                poids(0)=cb(xx,0,0)
                poids(1)=cc(xx,0)
             end if
               

          case(2)
                poids(0)=ca(xx-1.,1)
                poids(1)=cb(xx-1.,1,0)+cc(xx-1.,0)
          case(3)
                poids(-1)=ca(xx,0)
                poids(0)=cb(xx,0)+cc(xx,0)
          case(4)
                poids(0)=ca(xx,1)+cb(xx,1,0)
                poids(1)=cc(xx,0)


                     
          case(5)
             poids(-1)=ca(xx,0)
             poids(0)=cb(xx,0)             
             poids(1)=cc(xx,0)-cc(xx-1.,1)
             poids(2)=cc(xx-1.,1)
            
          case(6)
             poids(-1)=ca(xx,0)
             poids(0)=ca(xx-1.,1)-ca(xx,0)
             poids(1)=cb(xx-1.,1)
             poids(2)=cc(xx-1.,1)


          case(7)
             poids(-2)=ca(xx+1.,0)
             poids(-1)=ca(xx,0)-ca(xx+1.,0)
             poids(0)=cb(xx,0)
             poids(1)=cc(xx,0)
          end select



          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do
     contains
   
       function coeff_l2g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)
        case(0)
           c=1.-x**2
        case(1)
           c=0.5*x*(1.+x)
        end select
        
      end function coeff_l2g

      function coeff_tscg (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)  +1./8. ! +1./6.
        case(0)
           c=-x**2+1.    -2./8. !-2./6. 
        case(1)
           c=0.5*x*(1.+x)   +1./8. ! +1./6.
        end select
        
      end function coeff_tscg

    function ca(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: ca
        
        ca=coeff_tscg(-1,x)+phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))
        
      end function ca

      function cb(x,num,num2)
        integer      :: num
        real(kind=8) :: x
        integer,optional :: num2
        real(kind=8) :: cb
       
        if (present (num2)) then
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num2)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        else
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        end if

      end function cb

      function cc(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: cc
      
        cc=coeff_tscg(1,x)+phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))
        

      end function cc
            
      function test (ipart,vois)
        implicit none
        integer       :: ipart,vois
        logical       :: test
        real(kind=8)  :: x
        
        test=.false.       
        x=(posy(mod(ipart-1+vois+npart,npart)+1) - real( floor((posy(mod(ipart-1+vois+npart,npart)+1)-yb)/dy) ,kind=8)*dy-yb)/dy
        if ( ((blocg(mod(ipart-1+vois+npart,npart)+1)==0).or.(blocg(mod(ipart-1+vois+npart,npart)+1)==2)).and.(x>0.5)) test=.true.
        
      end function test



 subroutine calcul_phi_tvd(phi_out,num) 
        implicit none
        integer                                :: num
        real(kind=8),dimension(0:1),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u
        real(kind=8)                           :: rp,rm,diff1,diff2,diff3,diff0,rmm,y,r 
        integer                                :: ib,jb
        real(kind=8),dimension(-5:5)           :: xx_vois,x,tp1,tp2,tp3

        !particules voisines
        !-------------------       
        do ib=-2,2
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        do ib=-1,2
           x(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
        end do


        !cas bloc centre xx>0.5 (psi)
        !-----------------------
        if (num==1) then
          
           do ib=0,1                          
              x(ib)=x(ib)-1.              
              tp1(ib)=6.-8.*x(ib)*x(ib)             
              tp3(ib)=4.*(x(ib)-0.5)**2          
           end do
           
           do ib=0,1
              r=(u(ib+1)-u(ib))/(u(ib)-u(ib-1))               

              !anti diffusif
              !phi_out(ib)=max(0.,min(tp3(ib),tp1(ib)*r))                            

              !du type Minmod (attention non justifié pour y>=0.8)
              phi_out(ib)=max(0.,min(1.,4.*r))

              !bornes dépendent de la condition cfl 
              !phi_out(ib)=max(0.,min(1.,tp3(ib),tp1(ib)*r))              
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.
           end do
 



        else  
           !sinon (phi)
           !-----------------------       


        do ib=-1,0         
           tp1(ib)=6.-8.*x(ib)*x(ib)
           tp2(ib)=4.*(x(ib)+0.5)**2                
        end do


        do ib=0,1
              r=(u(ib-1)-u(ib-2))/(u(ib)-u(ib-1))                                                 
              !phi_out(ib)=max(0.,min(tp2(ib-1),tp1(ib-1)*r))
              phi_out(ib)=max(0.,min(1.,4.*r))
              !phi_out(ib)=max(0.,min(1.,tp2(ib-1),tp1(ib-1)*r))
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.          
        end do

     end if
    

      end subroutine calcul_phi_tvd

     end subroutine remaill_l2_bloc_v2_limit_y


  subroutine remaill_l2_limit_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos
       real(kind=8),dimension(0:2)              :: phi_mdemi,phi_pdemi     
       real(kind=8),dimension(0:1)              :: phi

       remaille=0.      

       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx


          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !----------------
  !calcul du limiteur pour cas de bloc centre avec xx>0.5
             !--------------------------------------------------------
             call calcul_phi_tvd(phi,1)         
             phi_mdemi(1)=phi(0)
             phi_pdemi(1)=phi(1)           

             !les autres cas ...
             !---------------------
             call calcul_phi_tvd(phi,0)         
             phi_mdemi(0)=phi(0)
             phi_pdemi(0)=phi(1) 

             phi_mdemi(2)=0.
             phi_pdemi(2)=0.     

             if (xx<=0.5) then

                poids(-1)=ca(xx,0)
                if (test (i,1) ) then
                   poids(0)=cb(xx,0,1)
                   poids(1)=cc(xx,1)
                else
                   poids(0)=cb(xx,0,0)
                   poids(1)=cc(xx,0)
                end if

             else     
               
                poids(0)=ca(xx-1.,1)
                if (test (i,1) ) then
                   poids(1)=cb(xx-1.,1,1)
                   poids(2)=cc(xx-1.,1)
                else                  
                   poids(1)=cb(xx-1.,1,0)
                   poids(2)=cc(xx-1.,0)
                end if                        
          
             end if


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

 contains
   
       function coeff_l2g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)
        case(0)
           c=1.-x**2
        case(1)
           c=0.5*x*(1.+x)
        end select
        
      end function coeff_l2g

      function coeff_tscg (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)  +1./8. ! +1./6.
        case(0)
           c=-x**2+1.    -2./8. !-2./6. 
        case(1)
           c=0.5*x*(1.+x)   +1./8. ! +1./6.
        end select
        
      end function coeff_tscg
 function ca(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: ca
        
        ca=coeff_tscg(-1,x)+phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))
        
      end function ca

      function cb(x,num,num2)
        integer      :: num
        real(kind=8) :: x
        integer,optional :: num2
        real(kind=8) :: cb
       
        if (present (num2)) then
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num2)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        else
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        end if

      end function cb

      function cc(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: cc
      
        cc=coeff_tscg(1,x)+phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))
        

      end function cc
            
      function test (ipart,vois)
        implicit none
        integer       :: ipart,vois
        logical       :: test
        real(kind=8)  :: x
        
        test=.false.       
        x=(posx(mod(ipart-1+vois+npart,npart)+1) - real( floor((posx(mod(ipart-1+vois+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx
        if ( ((blocg(mod(ipart-1+vois+npart,npart)+1)==0).or.(blocg(mod(ipart-1+vois+npart,npart)+1)==2)).and.(x>0.5)) test=.true.
        
      end function test



 subroutine calcul_phi_tvd(phi_out,num) 
        implicit none
        integer                                :: num
        real(kind=8),dimension(0:1),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u
        real(kind=8)                           :: rp,rm,diff1,diff2,diff3,diff0,rmm,y,r 
        integer                                :: ib,jb
        real(kind=8),dimension(-5:5)           :: xx_vois,x,tp1,tp2,tp3

        !particules voisines
        !-------------------       
        do ib=-2,2
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        do ib=-1,2
           x(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
        end do


        !cas bloc centre xx>0.5 (psi)
        !-----------------------
        if (num==1) then
          
           do ib=0,1                          
              x(ib)=x(ib)-1.              
              tp1(ib)=6.-8.*x(ib)*x(ib)             
              tp3(ib)=4.*(x(ib)-0.5)**2          
           end do
           
           do ib=0,1
              r=(u(ib+1)-u(ib))/(u(ib)-u(ib-1))               

              !anti diffusif
              !phi_out(ib)=max(0.,min(tp3(ib),tp1(ib)*r))                            

              !du type Minmod (attention non justifié pour y>=0.8)
              phi_out(ib)=max(0.,min(1.,4.*r))

              !bornes dépendent de la condition cfl 
              !phi_out(ib)=max(0.,min(1.,tp3(ib),tp1(ib)*r))              
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.
           end do
 



        else  
           !sinon (phi)
           !-----------------------       


        do ib=-1,0
           tp1(ib)=6.-8.*x(ib)*x(ib)
           tp2(ib)=4.*(x(ib)+0.5)**2                
        end do


        do ib=0,1
              r=(u(ib-1)-u(ib-2))/(u(ib)-u(ib-1))                                                 
              !phi_out(ib)=max(0.,min(tp2(ib-1),tp1(ib-1)*r))
              phi_out(ib)=max(0.,min(1.,4.*r))
              !phi_out(ib)=max(0.,min(1.,tp2(ib-1),tp1(ib-1)*r))
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.          
        end do

     end if
    

      end subroutine calcul_phi_tvd
                       
                          

     end subroutine remaill_l2_limit_x

    subroutine remaill_l2_limit_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-2:2)                                      :: ip
       real(kind=8),dimension(-2:2)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos
       real(kind=8),dimension(0:2):: phi_mdemi,phi_pdemi     
       real(kind=8),dimension(0:1):: phi
       

       remaille=0.      

       do i=1,npart

          ivar=nint((posx(i)-xg)/dx)+1
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------          
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy       

          !conditions au bord
          !------------------                
          !periodique: 
          do c=-2,2             
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poids
          !----------------
     !calcul du limiteur pour cas de bloc centre avec xx>0.5
             !--------------------------------------------------------
             call calcul_phi_tvd(phi,1)         
             phi_mdemi(1)=phi(0)
             phi_pdemi(1)=phi(1)           

             !les autres cas ...
             !---------------------
             call calcul_phi_tvd(phi,0)         
             phi_mdemi(0)=phi(0)
             phi_pdemi(0)=phi(1) 

             phi_mdemi(2)=0.
             phi_pdemi(2)=0.     


             if (xx<=0.5) then

                poids(-1)=ca(xx,0)
                if (test (i,1) ) then
                   poids(0)=cb(xx,0,1)
                   poids(1)=cc(xx,1)
                else
                   poids(0)=cb(xx,0,0)
                   poids(1)=cc(xx,0)
                end if

             else     
               
                poids(0)=ca(xx-1.,1)
                if (test (i,1) ) then
                   poids(1)=cb(xx-1.,1,1)
                   poids(2)=cc(xx-1.,1)
                else                  
                   poids(1)=cb(xx-1.,1,0)
                   poids(2)=cc(xx-1.,0)
                end if                        
          
             end if
         


          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-2,2
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do
     contains
   
       function coeff_l2g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)
        case(0)
           c=1.-x**2
        case(1)
           c=0.5*x*(1.+x)
        end select
        
      end function coeff_l2g

      function coeff_tscg (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-1)
           c=0.5*x*(x-1.)  +1./8. ! +1./6.
        case(0)
           c=-x**2+1.    -2./8. !-2./6. 
        case(1)
           c=0.5*x*(1.+x)   +1./8. ! +1./6.
        end select
        
      end function coeff_tscg

    function ca(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: ca
        
        ca=coeff_tscg(-1,x)+phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))
        
      end function ca

      function cb(x,num,num2)
        integer      :: num
        real(kind=8) :: x
        integer,optional :: num2
        real(kind=8) :: cb
       
        if (present (num2)) then
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num2)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        else
           cb=1.-(coeff_tscg(-1,x)+coeff_tscg(1,x))-phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))-phi_mdemi(num)*(coeff_l2g(-1,x)-coeff_tscg(-1,x))        
        end if

      end function cb

      function cc(x,num)
        integer      :: num
        real(kind=8) :: x
        real(kind=8) :: cc
      
        cc=coeff_tscg(1,x)+phi_pdemi(num)*(coeff_l2g(1,x)-coeff_tscg(1,x))
        

      end function cc
            
      function test (ipart,vois)
        implicit none
        integer       :: ipart,vois
        logical       :: test
        real(kind=8)  :: x
        
        test=.false.       
        x=(posy(mod(ipart-1+vois+npart,npart)+1) - real( floor((posy(mod(ipart-1+vois+npart,npart)+1)-yb)/dy) ,kind=8)*dy-yb)/dy
        if ( ((blocg(mod(ipart-1+vois+npart,npart)+1)==0).or.(blocg(mod(ipart-1+vois+npart,npart)+1)==2)).and.(x>0.5)) test=.true.
        
      end function test



 subroutine calcul_phi_tvd(phi_out,num) 
        implicit none
        integer                                :: num
        real(kind=8),dimension(0:1),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u
        real(kind=8)                           :: rp,rm,diff1,diff2,diff3,diff0,rmm,y,r 
        integer                                :: ib,jb
        real(kind=8),dimension(-5:5)           :: xx_vois,x,tp1,tp2,tp3

        !particules voisines
        !-------------------       
        do ib=-2,2
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        do ib=-1,2
           x(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
        end do


        !cas bloc centre xx>0.5 (psi)
        !-----------------------
        if (num==1) then
          
           do ib=0,1                          
              x(ib)=x(ib)-1.              
              tp1(ib)=6.-8.*x(ib)*x(ib)             
              tp3(ib)=4.*(x(ib)-0.5)**2          
           end do
           
           do ib=0,1
              r=(u(ib+1)-u(ib))/(u(ib)-u(ib-1))               

              !anti diffusif
              !phi_out(ib)=max(0.,min(tp3(ib),tp1(ib)*r))                            

              !du type Minmod (attention non justifié pour y>=0.8)
              phi_out(ib)=max(0.,min(1.,4.*r))

              !bornes dépendent de la condition cfl 
              !phi_out(ib)=max(0.,min(1.,tp3(ib),tp1(ib)*r))              
              
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.
           end do
 



        else  
           !sinon (phi)
           !-----------------------       


        do ib=-1,0         
           tp1(ib)=6.-8.*x(ib)*x(ib)
           tp2(ib)=4.*(x(ib)+0.5)**2                
        end do


        do ib=0,1
              r=(u(ib-1)-u(ib-2))/(u(ib)-u(ib-1))                                                 
              !phi_out(ib)=max(0.,min(tp2(ib-1),tp1(ib-1)*r))
              phi_out(ib)=max(0.,min(1.,4.*r))
              !phi_out(ib)=max(0.,min(1.,tp2(ib-1),tp1(ib-1)*r))
              if(abs(u(ib)-u(ib-1))<0.0000001) phi_out(ib)=0.          
        end do

     end if
    

      end subroutine calcul_phi_tvd

     end subroutine remaill_l2_limit_y





     subroutine remaill_l4_bloc_v2_limit_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3
       real(kind=8)                                                 :: phi_m3demi,phi_mdemi,phi_pdemi,phi_p3demi
       real(kind=8),dimension(-1:2)                                 :: phi


       remaille=0.   


       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1 
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx 


          !conditions au bord
          !------------------       
          do c=-3,4             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !---------------- 

             call calcul_phi_minmax(phi)                           
             phi_m3demi=phi(-1)
             phi_mdemi=phi(0)
             phi_pdemi=phi(1)
             phi_p3demi=phi(2)
         
  select case (blocg(i))

          case(0)
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=ca(xx)    
                poids(-1)=cb(xx)    
                poids(0)=cc(xx)     
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=ca(xx-1.)
                poids(0)=cb(xx-1.)
                poids(1)=cc(xx-1)
             end if
          case(1)
             poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=cb(xx)
             poids(0)=cc(xx)
          case(2)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=ca(xx-1.)
             poids(0)=cb(xx-1.)
             poids(1)=cc(xx-1.)+cd(xx-1.)+ce(xx-1.)-ce(xx)
          case(3)
             poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=cb(xx)
             poids(0)=cc(xx)+cd(xx)+ce(xx)-ce(xx+1.)
          case(4)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=ca(xx-1.)
             poids(0)=ca(xx)-ca(xx-1.)+cb(xx)+cc(xx)
          case(5)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=0.
             poids(0)=ca(xx-1.)+cb(xx-1.)
             poids(1)=cc(xx-1.)
          case(6)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=ca(xx)+cb(xx)
             poids(0)=cc(xx)
          case(7)
             poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=cb(xx)
             poids(0)=ca(xx-1.)-ca(xx)+cb(xx-1.)-cb(xx)
             poids(1)=cc(xx-1)
          case(8)
             poids(-3)=ca(xx+1.)
             poids(-2)=cb(xx+1.)
             poids(-1)=ca(xx)-ca(xx+1.)+cb(xx)-cb(xx+1.)
             poids(0)=cc(xx)
          case(9)
            poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=ca(xx-1.)-ca(xx)
             poids(0)=cb(xx-1.)
             poids(1)=cc(xx-1)
          case(10)
             poids(-3)=ca(xx+1.)
             poids(-2)=ca(xx)-ca(xx+1.)
             poids(-1)=cb(xx)
             poids(0)=cc(xx)
        
          end select

          select case (blocd(i))        
          case(0)
             if (xx<=0.5) then                
                poids(1)=cd(xx)
                poids(2)=ce(xx)
                poids(3)=0.
                poids(4)=0.
             else                 
                poids(2)=cd(xx-1.)
                poids(3)=ce(xx-1.)
                poids(4)=0.
             end if
          case(1)
             poids(1)=cd(xx)
             poids(2)=ce(xx)
             poids(3)=0.
             poids(4)=0.
          case(2)
             poids(2)=cd(xx-1.)+ce(xx-1.)
             poids(3)=0.
             poids(4)=0.
          case(3)
            poids(1)=cd(xx)+ce(xx)
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(4)
             poids(2)=ce(xx)
             poids(3)=0.
             poids(4)=0.
          case(5)
             poids(1)=ce(xx+1.)
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(6)            
             poids(2)=cd(xx-1.)
             poids(3)=ce(xx-1.)-ce(xx-2.)
             poids(4)=ce(xx-2.)
          case(7)            
             poids(1)=cd(xx)
             poids(2)=ce(xx)-ce(xx-1.)
             poids(3)=ce(xx-1.)
             poids(4)=0.
          case(8)
             poids(1)=cd(xx)-cd(xx-1.)+ce(xx)-ce(xx-1.)
             poids(2)=cd(xx-1.)
             poids(3)=ce(xx-1.)
             poids(4)=0.
        
            
          end select
       

          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

 contains

       function ca(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ca
         !ca=x*(x-1.)*(x-2.)*(x+1.)/24.
         ca=coeff_M5g(-2,x)+phi_m3demi*coeff_l4mM5g(-2,x)
       end function ca
       function cb(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cb
         !cb=-x*(x-1.)*(x**2-4.)/6.
         cb=coeff_M5g(-1,x)-phi_m3demi*coeff_l4mM5g(-2,x)+phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))
       end function cb
       function cc(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cc
         !cc=(x**2-4.)*(x**2-1.)/4.
         cc=coeff_M5g(0,x)-phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))-phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))  
       end function cc
       function cd(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cd
         !cd=-x*(x+1.)*(x+2.)*(x-2.)/6.   
         cd=coeff_M5g(1,x)+phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))-phi_p3demi*coeff_l4mM5g(2,x)         
       end function cd
       function ce(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ce
         !ce=x*(x+1.)*(x+2.)*(x-1.)/24.
         ce=coeff_M5g(2,x)+phi_p3demi*coeff_l4mM5g(2,x)    
       end function ce


       function coeff_l4g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=x*(x-1)*(x-2)*(x+1)/24.
        case(-1)
           c=-x*(x-1)*(x-2)*(x+2)/6.
        case(0)
           c=(x-2)*(x-1)*(x+1)*(x+2)/4.
        case(1)
           c=-x*(x-2)*(x+1)*(x+2)/6.
        case(2)
           c=x*(x-1)*(x+1)*(x+2)/24.
        end select
        
      end function coeff_l4g

      function coeff_M5g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=(2.*x-1.)**4/384.
        case(-1)
           c=19./96.-11/24.*x+x**2/4.+x**3/6.-x**4/6.
        case(0)
           c=115./192.-5./8.*x**2+x**4/4.
        case(1)
           c=19./96.+11/24.*x-x**3/6.+x**2/4.-x**4/6.
        case(2)
           c=(2.*x+1.)**4/384.
        end select
        
      end function coeff_M5g

      function coeff_l4mM5g (num,x) result(c) 
        !lambda4 - M5  gauche
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=5.*x/48.*(1.-x)-1./384.
        case(-1)
           c=-5.*x/24.*(1.-2.*x)-19/96.
        case(0)
           c=-5.*x**2/8.+77./192.
        case(1)
           c=5.*x/24.*(1.+2.*x)-19./96.
        case(2)
           c=-5.*x/48.*(1.+x)-1./384.
        end select
        
      end function coeff_l4mM5g

    


      subroutine calcul_phi_minmax(phi_out) 
        implicit none
        real(kind=8),dimension(-1:2),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u,bmin,bmax,x,xx_vois
        real(kind=8),dimension(-5:5,-5:5)      :: ej,fj
        real(kind=8)                           :: tp1,tp2,phi1,phi2
        integer                                :: ib,jb


        !particules voisines
        !-------------------       
        do ib=-4,5
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        !coeff decentre droite = coeff decentre gauche -1
        !-------------------------------------------------           
        do ib=-4,3
           xx_vois(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
           
           if ((xx_vois(ib)>0.5).and.((blocg(mod(i-1+ib+npart,npart)+1)==0).or.(blocd(mod(i-1+ib+npart,npart)+1)==0))) then
              x(ib)=xx_vois(ib)-1.
           else
              x(ib)=xx_vois(ib)
           end if
        end do


        do ib=-4,3         

           bmin(ib)=min(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           bmax(ib)=max(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           
          
           do jb=-4,3
              ej(ib,jb)=coeff_M5g(-2,x(jb))*u(ib+2)+coeff_M5g(-1,x(jb))*u(ib+1)+coeff_M5g(0,x(jb))*u(ib)+coeff_M5g(1,x(jb))*u(ib-1)+coeff_M5g(2,x(jb))*u(ib-2)
              fj(ib,jb)=coeff_l4mM5g(-2,x(jb))*u(ib+1)+(coeff_l4mM5g(-2,x(jb))+coeff_l4mM5g(-1,x(jb)))*u(ib)-(coeff_l4mM5g(1,x(jb))+coeff_l4mM5g(2,x(jb)))*u(ib-1)-coeff_l4mM5g(2,x(jb))*u(ib-2)
           end do

        end do


        do ib=-1,2

           tp1=(bmin(ib-1)-ej(ib-1,ib)+max(0.,fj(ib-1,ib)))/fj(ib,ib)
           tp2=(bmax(ib-1)-ej(ib-1,ib)+min(0.,fj(ib-1,ib)))/fj(ib,ib)
           !phi1=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi1=max(0.,min(1.,max(tp1,tp2)))

           tp1=(bmin(ib)-ej(ib,ib-1)+max(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           tp2=(bmax(ib)-ej(ib,ib-1)+min(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           !phi2=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi2=max(0.,min(1.,max(tp1,tp2)))

           phi_out(ib)=min(phi1,phi2)

           if(abs(fj(ib,ib))<0.0000001) phi_out(ib)=0.
           if(abs(fj(ib,ib-1))<0.0000001) phi_out(ib)=0.

        end do              


      end subroutine calcul_phi_minmax


     end subroutine remaill_l4_bloc_v2_limit_x




     subroutine remaill_l4_bloc_v2_limit_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3
       real(kind=8)                                                 :: phi_m3demi,phi_mdemi,phi_pdemi,phi_p3demi
       real(kind=8),dimension(-1:2)                                 :: phi


       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy

          !conditions au bord
          !------------------       
          do c=-3,4            
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poidss
          !----------------         
             call calcul_phi_minmax(phi)                           
             phi_m3demi=phi(-1)
             phi_mdemi=phi(0)
             phi_pdemi=phi(1)
             phi_p3demi=phi(2)       
  select case (blocg(i))

          case(0)
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=ca(xx)    
                poids(-1)=cb(xx)    
                poids(0)=cc(xx)     
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=ca(xx-1.)
                poids(0)=cb(xx-1.)
                poids(1)=cc(xx-1)
             end if
          case(1)
             poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=cb(xx)
             poids(0)=cc(xx)
          case(2)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=ca(xx-1.)
             poids(0)=cb(xx-1.)
             poids(1)=cc(xx-1.)+cd(xx-1.)+ce(xx-1.)-ce(xx)
          case(3)
             poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=cb(xx)
             poids(0)=cc(xx)+cd(xx)+ce(xx)-ce(xx+1.)
          case(4)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=ca(xx-1.)
             poids(0)=ca(xx)-ca(xx-1.)+cb(xx)+cc(xx)
          case(5)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=0.
             poids(0)=ca(xx-1.)+cb(xx-1.)
             poids(1)=cc(xx-1.)
          case(6)
             poids(-3)=0.
             poids(-2)=0.
             poids(-1)=ca(xx)+cb(xx)
             poids(0)=cc(xx)
          case(7)
             poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=cb(xx)
             poids(0)=ca(xx-1.)-ca(xx)+cb(xx-1.)-cb(xx)
             poids(1)=cc(xx-1)
          case(8)
             poids(-3)=ca(xx+1.)
             poids(-2)=cb(xx+1.)
             poids(-1)=ca(xx)-ca(xx+1.)+cb(xx)-cb(xx+1.)
             poids(0)=cc(xx)
          case(9)
            poids(-3)=0.
             poids(-2)=ca(xx)
             poids(-1)=ca(xx-1.)-ca(xx)
             poids(0)=cb(xx-1.)
             poids(1)=cc(xx-1)
          case(10)
             poids(-3)=ca(xx+1.)
             poids(-2)=ca(xx)-ca(xx+1.)
             poids(-1)=cb(xx)
             poids(0)=cc(xx)
        
          end select

          select case (blocd(i))        
          case(0)
             if (xx<=0.5) then                
                poids(1)=cd(xx)
                poids(2)=ce(xx)
                poids(3)=0.
                poids(4)=0.
             else                 
                poids(2)=cd(xx-1.)
                poids(3)=ce(xx-1.)
                poids(4)=0.
             end if
          case(1)
             poids(1)=cd(xx)
             poids(2)=ce(xx)
             poids(3)=0.
             poids(4)=0.
          case(2)
             poids(2)=cd(xx-1.)+ce(xx-1.)
             poids(3)=0.
             poids(4)=0.
          case(3)
            poids(1)=cd(xx)+ce(xx)
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(4)
             poids(2)=ce(xx)
             poids(3)=0.
             poids(4)=0.
          case(5)
             poids(1)=ce(xx+1.)
             poids(2)=0.
             poids(3)=0.
             poids(4)=0.
          case(6)            
             poids(2)=cd(xx-1.)
             poids(3)=ce(xx-1.)-ce(xx-2.)
             poids(4)=ce(xx-2.)
          case(7)            
             poids(1)=cd(xx)
             poids(2)=ce(xx)-ce(xx-1.)
             poids(3)=ce(xx-1.)
             poids(4)=0.
          case(8)
             poids(1)=cd(xx)-cd(xx-1.)+ce(xx)-ce(xx-1.)
             poids(2)=cd(xx-1.)
             poids(3)=ce(xx-1.)
             poids(4)=0.
        
            
          end select
          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do
 contains

       function ca(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ca
         !ca=x*(x-1.)*(x-2.)*(x+1.)/24.
         ca=coeff_M5g(-2,x)+phi_m3demi*coeff_l4mM5g(-2,x)
       end function ca
       function cb(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cb
         !cb=-x*(x-1.)*(x**2-4.)/6.
         cb=coeff_M5g(-1,x)-phi_m3demi*coeff_l4mM5g(-2,x)+phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))
       end function cb
       function cc(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cc
         !cc=(x**2-4.)*(x**2-1.)/4.
         cc=coeff_M5g(0,x)-phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))-phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))  
       end function cc
       function cd(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cd
         !cd=-x*(x+1.)*(x+2.)*(x-2.)/6.   
         cd=coeff_M5g(1,x)+phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))-phi_p3demi*coeff_l4mM5g(2,x)         
       end function cd
       function ce(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ce
         !ce=x*(x+1.)*(x+2.)*(x-1.)/24.
         ce=coeff_M5g(2,x)+phi_p3demi*coeff_l4mM5g(2,x)    
       end function ce


       function coeff_l4g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=x*(x-1)*(x-2)*(x+1)/24.
        case(-1)
           c=-x*(x-1)*(x-2)*(x+2)/6.
        case(0)
           c=(x-2)*(x-1)*(x+1)*(x+2)/4.
        case(1)
           c=-x*(x-2)*(x+1)*(x+2)/6.
        case(2)
           c=x*(x-1)*(x+1)*(x+2)/24.
        end select
        
      end function coeff_l4g

      function coeff_M5g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=(2.*x-1.)**4/384.
        case(-1)
           c=19./96.-11/24.*x+x**2/4.+x**3/6.-x**4/6.
        case(0)
           c=115./192.-5./8.*x**2+x**4/4.
        case(1)
           c=19./96.+11/24.*x-x**3/6.+x**2/4.-x**4/6.
        case(2)
           c=(2.*x+1.)**4/384.
        end select
        
      end function coeff_M5g

      function coeff_l4mM5g (num,x) result(c) 
        !lambda4 - M5  gauche
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=5.*x/48.*(1.-x)-1./384.
        case(-1)
           c=-5.*x/24.*(1.-2.*x)-19/96.
        case(0)
           c=-5.*x**2/8.+77./192.
        case(1)
           c=5.*x/24.*(1.+2.*x)-19./96.
        case(2)
           c=-5.*x/48.*(1.+x)-1./384.
        end select
        
      end function coeff_l4mM5g

    


      subroutine calcul_phi_minmax(phi_out) 
        implicit none
        real(kind=8),dimension(-1:2),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u,bmin,bmax,x,xx_vois
        real(kind=8),dimension(-5:5,-5:5)      :: ej,fj
        real(kind=8)                           :: tp1,tp2,phi1,phi2
        integer                                :: ib,jb


        !particules voisines
        !-------------------       
        do ib=-4,5
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        !coeff decentre droite = coeff decentre gauche -1
        !-------------------------------------------------           
        do ib=-4,3
           xx_vois(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
           
           if ((xx_vois(ib)>0.5).and.((blocg(mod(i-1+ib+npart,npart)+1)==0).or.(blocd(mod(i-1+ib+npart,npart)+1)==0))) then
              x(ib)=xx_vois(ib)-1.
           else
              x(ib)=xx_vois(ib)
           end if
        end do


        do ib=-4,3         

           bmin(ib)=min(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           bmax(ib)=max(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           
          
           do jb=-4,3
              ej(ib,jb)=coeff_M5g(-2,x(jb))*u(ib+2)+coeff_M5g(-1,x(jb))*u(ib+1)+coeff_M5g(0,x(jb))*u(ib)+coeff_M5g(1,x(jb))*u(ib-1)+coeff_M5g(2,x(jb))*u(ib-2)
              fj(ib,jb)=coeff_l4mM5g(-2,x(jb))*u(ib+1)+(coeff_l4mM5g(-2,x(jb))+coeff_l4mM5g(-1,x(jb)))*u(ib)-(coeff_l4mM5g(1,x(jb))+coeff_l4mM5g(2,x(jb)))*u(ib-1)-coeff_l4mM5g(2,x(jb))*u(ib-2)
           end do

        end do


        do ib=-1,2

           tp1=(bmin(ib-1)-ej(ib-1,ib)+max(0.,fj(ib-1,ib)))/fj(ib,ib)
           tp2=(bmax(ib-1)-ej(ib-1,ib)+min(0.,fj(ib-1,ib)))/fj(ib,ib)
           !phi1=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi1=max(0.,min(1.,max(tp1,tp2)))

           tp1=(bmin(ib)-ej(ib,ib-1)+max(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           tp2=(bmax(ib)-ej(ib,ib-1)+min(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           !phi2=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi2=max(0.,min(1.,max(tp1,tp2)))

           phi_out(ib)=min(phi1,phi2)

           if(abs(fj(ib,ib))<0.0000001) phi_out(ib)=0.
           if(abs(fj(ib,ib-1))<0.0000001) phi_out(ib)=0.

        end do              


      end subroutine calcul_phi_minmax

     end subroutine remaill_l4_bloc_v2_limit_y




 subroutine remaill_l4_limit_x (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,j,per
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3
       real(kind=8)                                                 :: phi_m3demi,phi_mdemi,phi_pdemi,phi_p3demi
       real(kind=8),dimension(-1:2)                                 :: phi


       remaille=0.   


       do i=1,npart

          j=nint((posy(i)-yb)/dy)+1 
          poids=0.

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posx(i)-xg)/dx)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posx(i) - real(ip(0) ,kind=8)*dx-xg)/dx 


          !conditions au bord
          !------------------       
          do c=-3,4             
             ip(c)=mod(ip(c)+nx,nx)     
          end do

          !calcul des poids
          !---------------- 

             call calcul_phi_minmax(phi)                           
             phi_m3demi=phi(-1)
             phi_mdemi=phi(0)
             phi_pdemi=phi(1)
             phi_p3demi=phi(2)
         
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=ca(xx)    
                poids(-1)=cb(xx)    
                poids(0)=cc(xx)
                poids(1)=cd(xx)
                poids(2)=ce(xx)
                poids(3)=0.
                poids(4)=0.     
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=ca(xx-1.)
                poids(0)=cb(xx-1.)
                poids(1)=cc(xx-1)
                poids(2)=cd(xx-1.)
                poids(3)=ce(xx-1.)
                poids(4)=0.
             end if
        
         

          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ip(c)+1,j)=remaille(ip(c)+1,j)+donne(i)*poids(c)
          end do


       end do

 contains

       function ca(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ca
         !ca=x*(x-1.)*(x-2.)*(x+1.)/24.
         ca=coeff_M5g(-2,x)+phi_m3demi*coeff_l4mM5g(-2,x)
       end function ca
       function cb(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cb
         !cb=-x*(x-1.)*(x**2-4.)/6.
         cb=coeff_M5g(-1,x)-phi_m3demi*coeff_l4mM5g(-2,x)+phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))
       end function cb
       function cc(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cc
         !cc=(x**2-4.)*(x**2-1.)/4.
         cc=coeff_M5g(0,x)-phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))-phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))  
       end function cc
       function cd(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cd
         !cd=-x*(x+1.)*(x+2.)*(x-2.)/6.   
         cd=coeff_M5g(1,x)+phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))-phi_p3demi*coeff_l4mM5g(2,x)         
       end function cd
       function ce(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ce
         !ce=x*(x+1.)*(x+2.)*(x-1.)/24.
         ce=coeff_M5g(2,x)+phi_p3demi*coeff_l4mM5g(2,x)    
       end function ce


       function coeff_l4g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=x*(x-1)*(x-2)*(x+1)/24.
        case(-1)
           c=-x*(x-1)*(x-2)*(x+2)/6.
        case(0)
           c=(x-2)*(x-1)*(x+1)*(x+2)/4.
        case(1)
           c=-x*(x-2)*(x+1)*(x+2)/6.
        case(2)
           c=x*(x-1)*(x+1)*(x+2)/24.
        end select
        
      end function coeff_l4g

      function coeff_M5g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=(2.*x-1.)**4/384.
        case(-1)
           c=19./96.-11/24.*x+x**2/4.+x**3/6.-x**4/6.
        case(0)
           c=115./192.-5./8.*x**2+x**4/4.
        case(1)
           c=19./96.+11/24.*x-x**3/6.+x**2/4.-x**4/6.
        case(2)
           c=(2.*x+1.)**4/384.
        end select
        
      end function coeff_M5g

      function coeff_l4mM5g (num,x) result(c) 
        !lambda4 - M5  gauche
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=5.*x/48.*(1.-x)-1./384.
        case(-1)
           c=-5.*x/24.*(1.-2.*x)-19/96.
        case(0)
           c=-5.*x**2/8.+77./192.
        case(1)
           c=5.*x/24.*(1.+2.*x)-19./96.
        case(2)
           c=-5.*x/48.*(1.+x)-1./384.
        end select
        
      end function coeff_l4mM5g

    


      subroutine calcul_phi_minmax(phi_out) 
        implicit none
        real(kind=8),dimension(-1:2),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u,bmin,bmax,x,xx_vois
        real(kind=8),dimension(-5:5,-5:5)      :: ej,fj
        real(kind=8)                           :: tp1,tp2,phi1,phi2
        integer                                :: ib,jb


        !particules voisines
        !-------------------       
        do ib=-4,5
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        !coeff decentre droite = coeff decentre gauche -1
        !-------------------------------------------------           
        do ib=-4,3
           xx_vois(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
           
           if ((xx_vois(ib)>0.5).and.((blocg(mod(i-1+ib+npart,npart)+1)==0).or.(blocd(mod(i-1+ib+npart,npart)+1)==0))) then
              x(ib)=xx_vois(ib)-1.
           else
              x(ib)=xx_vois(ib)
           end if
        end do


        do ib=-4,3         

           bmin(ib)=min(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           bmax(ib)=max(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           
          
           do jb=-4,3
              ej(ib,jb)=coeff_M5g(-2,x(jb))*u(ib+2)+coeff_M5g(-1,x(jb))*u(ib+1)+coeff_M5g(0,x(jb))*u(ib)+coeff_M5g(1,x(jb))*u(ib-1)+coeff_M5g(2,x(jb))*u(ib-2)
              fj(ib,jb)=coeff_l4mM5g(-2,x(jb))*u(ib+1)+(coeff_l4mM5g(-2,x(jb))+coeff_l4mM5g(-1,x(jb)))*u(ib)-(coeff_l4mM5g(1,x(jb))+coeff_l4mM5g(2,x(jb)))*u(ib-1)-coeff_l4mM5g(2,x(jb))*u(ib-2)
           end do

        end do


        do ib=-1,2

           tp1=(bmin(ib-1)-ej(ib-1,ib)+max(0.,fj(ib-1,ib)))/fj(ib,ib)
           tp2=(bmax(ib-1)-ej(ib-1,ib)+min(0.,fj(ib-1,ib)))/fj(ib,ib)
           !phi1=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi1=max(0.,min(1.,max(tp1,tp2)))

           tp1=(bmin(ib)-ej(ib,ib-1)+max(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           tp2=(bmax(ib)-ej(ib,ib-1)+min(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           !phi2=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi2=max(0.,min(1.,max(tp1,tp2)))

           phi_out(ib)=min(phi1,phi2)

           if(abs(fj(ib,ib))<0.0000001) phi_out(ib)=0.
           if(abs(fj(ib,ib-1))<0.0000001) phi_out(ib)=0.

        end do              


      end subroutine calcul_phi_minmax


     end subroutine remaill_l4_limit_x




     subroutine remaill_l4_limit_y (donne,posx,posy,remaille)  
       implicit none
       real(kind=8),dimension(:),intent(in)                         :: donne
       real(kind=8),dimension(:),intent(in)                         :: posx,posy
       real(kind=8),dimension(1:nx,1:ny),intent(out)                :: remaille
       integer                                                      :: i,c,d,ib,ivar,per
       integer,dimension(-3:4)                                      :: ip
       real(kind=8),dimension(-3:4)                                 :: poids
       real(kind=8)                                                 :: xx,tmp_pos,t1,t2,t3
       real(kind=8)                                                 :: phi_m3demi,phi_mdemi,phi_pdemi,phi_p3demi
       real(kind=8),dimension(-1:2)                                 :: phi


       remaille=0.   

       do i=1,npart

          poids=0.
          ivar=nint((posx(i)-xg)/dx)+1

          !numero des points sur le maillage
          !--------------------------------
          ip(0) = floor((posy(i)-yb)/dy)         
          ip(-3) = ip(0) - 3      
          ip(-2) = ip(0) - 2        
          ip(-1) = ip(0) - 1         
          ip(1) = ip(0) + 1
          ip(2) = ip(0) + 2
          ip(3) = ip(0) + 3
          ip(4) = ip(0) + 4



          !distance de la particule à remailler au point de grille de gauche
          !-----------------------------------------------------------------
          xx = (posy(i) - real(ip(0) ,kind=8)*dy-yb)/dy

          !conditions au bord
          !------------------       
          do c=-3,4            
             ip(c)=mod(ip(c)+ny,ny)     
          end do

          !calcul des poidss
          !----------------         
             call calcul_phi_minmax(phi)                           
             phi_m3demi=phi(-1)
             phi_mdemi=phi(0)
             phi_pdemi=phi(1)
             phi_p3demi=phi(2)       
 
             if (xx<=0.5) then 
                poids(-3)=0.
                poids(-2)=ca(xx)    
                poids(-1)=cb(xx)    
                poids(0)=cc(xx) 
                poids(1)=cd(xx)
                poids(2)=ce(xx)
                poids(3)=0.
                poids(4)=0.    
             else 
                poids(-3)=0.
                poids(-2)=0. 
                poids(-1)=ca(xx-1.)
                poids(0)=cb(xx-1.)
                poids(1)=cc(xx-1)
                poids(2)=cd(xx-1.)
                poids(3)=ce(xx-1.)
                poids(4)=0.
             end if
                  

          !remaillage à l' interrieur domaine
          !---------------------------------         
          do c=-3,4
             remaille(ivar,ip(c)+1)=remaille(ivar,ip(c)+1)+donne(i)*poids(c)
          end do


       end do
 contains

       function ca(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ca
         !ca=x*(x-1.)*(x-2.)*(x+1.)/24.
         ca=coeff_M5g(-2,x)+phi_m3demi*coeff_l4mM5g(-2,x)
       end function ca
       function cb(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cb
         !cb=-x*(x-1.)*(x**2-4.)/6.
         cb=coeff_M5g(-1,x)-phi_m3demi*coeff_l4mM5g(-2,x)+phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))
       end function cb
       function cc(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cc
         !cc=(x**2-4.)*(x**2-1.)/4.
         cc=coeff_M5g(0,x)-phi_mdemi*(coeff_l4mM5g(-2,x)+coeff_l4mM5g(-1,x))-phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))  
       end function cc
       function cd(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: cd
         !cd=-x*(x+1.)*(x+2.)*(x-2.)/6.   
         cd=coeff_M5g(1,x)+phi_pdemi*(coeff_l4mM5g(1,x)+coeff_l4mM5g(2,x))-phi_p3demi*coeff_l4mM5g(2,x)         
       end function cd
       function ce(x)
         implicit none
         real(kind=8),intent(in)  :: x
         real(kind=8)             :: ce
         !ce=x*(x+1.)*(x+2.)*(x-1.)/24.
         ce=coeff_M5g(2,x)+phi_p3demi*coeff_l4mM5g(2,x)    
       end function ce


       function coeff_l4g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=x*(x-1)*(x-2)*(x+1)/24.
        case(-1)
           c=-x*(x-1)*(x-2)*(x+2)/6.
        case(0)
           c=(x-2)*(x-1)*(x+1)*(x+2)/4.
        case(1)
           c=-x*(x-2)*(x+1)*(x+2)/6.
        case(2)
           c=x*(x-1)*(x+1)*(x+2)/24.
        end select
        
      end function coeff_l4g

      function coeff_M5g (num,x) result(c) 
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=(2.*x-1.)**4/384.
        case(-1)
           c=19./96.-11/24.*x+x**2/4.+x**3/6.-x**4/6.
        case(0)
           c=115./192.-5./8.*x**2+x**4/4.
        case(1)
           c=19./96.+11/24.*x-x**3/6.+x**2/4.-x**4/6.
        case(2)
           c=(2.*x+1.)**4/384.
        end select
        
      end function coeff_M5g

      function coeff_l4mM5g (num,x) result(c) 
        !lambda4 - M5  gauche
        implicit none
        integer,intent(in)           :: num
        real(kind=8),intent(in)      :: x
        real(kind=8)                 :: c
        
        select case(num)
        case(-2)
           c=5.*x/48.*(1.-x)-1./384.
        case(-1)
           c=-5.*x/24.*(1.-2.*x)-19/96.
        case(0)
           c=-5.*x**2/8.+77./192.
        case(1)
           c=5.*x/24.*(1.+2.*x)-19./96.
        case(2)
           c=-5.*x/48.*(1.+x)-1./384.
        end select
        
      end function coeff_l4mM5g

    


      subroutine calcul_phi_minmax(phi_out) 
        implicit none
        real(kind=8),dimension(-1:2),intent(out):: phi_out
        real(kind=8),dimension(-5:5)           :: u,bmin,bmax,x,xx_vois
        real(kind=8),dimension(-5:5,-5:5)      :: ej,fj
        real(kind=8)                           :: tp1,tp2,phi1,phi2
        integer                                :: ib,jb


        !particules voisines
        !-------------------       
        do ib=-4,5
           u(ib)=donne(mod(i-1+ib+npart,npart)+1)  
        end do

        !coeff decentre droite = coeff decentre gauche -1
        !-------------------------------------------------           
        do ib=-4,3
           xx_vois(ib)=(posx(mod(i-1+ib+npart,npart)+1) - real( floor((posx(mod(i-1+ib+npart,npart)+1)-xg)/dx) ,kind=8)*dx-xg)/dx 
           
           if ((xx_vois(ib)>0.5).and.((blocg(mod(i-1+ib+npart,npart)+1)==0).or.(blocd(mod(i-1+ib+npart,npart)+1)==0))) then
              x(ib)=xx_vois(ib)-1.
           else
              x(ib)=xx_vois(ib)
           end if
        end do


        do ib=-4,3         

           bmin(ib)=min(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           bmax(ib)=max(u(ib-2),u(ib-1),u(ib),u(ib+1),u(ib+2)) 
           
          
           do jb=-4,3
              ej(ib,jb)=coeff_M5g(-2,x(jb))*u(ib+2)+coeff_M5g(-1,x(jb))*u(ib+1)+coeff_M5g(0,x(jb))*u(ib)+coeff_M5g(1,x(jb))*u(ib-1)+coeff_M5g(2,x(jb))*u(ib-2)
              fj(ib,jb)=coeff_l4mM5g(-2,x(jb))*u(ib+1)+(coeff_l4mM5g(-2,x(jb))+coeff_l4mM5g(-1,x(jb)))*u(ib)-(coeff_l4mM5g(1,x(jb))+coeff_l4mM5g(2,x(jb)))*u(ib-1)-coeff_l4mM5g(2,x(jb))*u(ib-2)
           end do

        end do


        do ib=-1,2

           tp1=(bmin(ib-1)-ej(ib-1,ib)+max(0.,fj(ib-1,ib)))/fj(ib,ib)
           tp2=(bmax(ib-1)-ej(ib-1,ib)+min(0.,fj(ib-1,ib)))/fj(ib,ib)
           !phi1=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi1=max(0.,min(1.,max(tp1,tp2)))

           tp1=(bmin(ib)-ej(ib,ib-1)+max(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           tp2=(bmax(ib)-ej(ib,ib-1)+min(0.,-fj(ib+1,ib-1)))/(-fj(ib,ib-1))
           !phi2=max(0.,min(tp1,tp2),min(1.,max(tp1,tp2)))
           phi2=max(0.,min(1.,max(tp1,tp2)))

           phi_out(ib)=min(phi1,phi2)

           if(abs(fj(ib,ib))<0.0000001) phi_out(ib)=0.
           if(abs(fj(ib,ib-1))<0.0000001) phi_out(ib)=0.

        end do              


      end subroutine calcul_phi_minmax

     end subroutine remaill_l4_limit_y




end module remaillage_mod


